/*###############################################################

	BasicTypesAE.c
	
	MacTelnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// Mac includes
#ifndef __AEDATAMODEL__
#	include <AEDataModel.h>
#endif
#ifndef __APPLEEVENTS__
#	include <AppleEvents.h>
#endif
#ifndef __ASREGISTRY__
#	include <ASRegistry.h>
#endif
#ifndef __ERRORS__
#	include <Errors.h>
#endif

// MacTelnet includes
#include "BasicTypesAE.h"
#include "Terminology.h"



//
// public methods
//

/*!
Creates an Apple Event descriptor wrapper
for a "true" or "false" value.

(3.0)
*/
OSErr
BasicTypesAE_CreateBooleanDesc		(Boolean const		inBooleanValue,
									 AEDesc*			outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(typeBoolean, &inBooleanValue, sizeof(inBooleanValue), outDescPtr);
	return result;
}// CreateBooleanDesc


/*!
Creates an Apple Event descriptor wrapper
for an enumerated type.

(3.0)
*/
OSErr
BasicTypesAE_CreateEnumerationDesc		(FourCharCode const		inEnumeration,
										 AEDesc*				outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(typeEnumerated, &inEnumeration, sizeof(inEnumeration), outDescPtr);
	return result;
}// CreateEnumerationDesc


/*!
Creates an Apple Event descriptor wrapper
for a file system specification record.

(3.0)
*/
OSErr
BasicTypesAE_CreateFileOrFolderDesc		(FSSpec const*		inFSSpecPtr,
										 AEDesc*			outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(typeFSS, inFSSpecPtr, sizeof(FSSpec), outDescPtr);
	return result;
}// CreateFileOrFolderDesc


/*!
Creates an Apple Event descriptor wrapper
for a list containing two unsigned, long
integers.

(3.0)
*/
OSErr
BasicTypesAE_CreatePairDesc		(AEDesc const*		inDesc1Ptr,
								 AEDesc const*		inDesc2Ptr,
								 AEDescList*		outDescListPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateList(null/* factoring */, 0/* size */, false/* is a record */, outDescListPtr);
	AEPutDesc(outDescListPtr, 1/* index */, inDesc1Ptr);
	AEPutDesc(outDescListPtr, 2/* index */, inDesc2Ptr);
	
	return result;
}// CreatePairDesc


/*!
Creates an Apple Event descriptor wrapper
for a QuickDraw Point.

(3.0)
*/
OSErr
BasicTypesAE_CreatePointDesc		(Point const	inPoint,
									 AEDesc*		outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(cQDPoint, &inPoint, sizeof(inPoint), outDescPtr);
	return result;
}// CreatePointDesc


/*!
Creates an Apple Event descriptor wrapper
for a property ID.

(3.0)
*/
OSErr
BasicTypesAE_CreatePropertyIDDesc		(FourCharCode const		inPropertyID,
										 AEDesc*				outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(typeType, &inPropertyID, sizeof(inPropertyID), outDescPtr);
	return result;
}// CreatePropertyIDDesc


/*!
Creates an Apple Event descriptor wrapper
for text, using Pascal string data.

(3.0)
*/
OSErr
BasicTypesAE_CreatePStringDesc		(Str255			inPascalStringData,
									 AEDesc*		outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(cString, inPascalStringData + 1,
							StrLength(inPascalStringData) * sizeof(unsigned char), outDescPtr);
	return result;
}// CreatePStringDesc


/*!
Creates an Apple Event descriptor wrapper
for a QuickDraw rectangle.

(3.0)
*/
OSErr
BasicTypesAE_CreateRectDesc		(Rect const*	inRectPtr,
								 AEDesc*		outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(cQDRectangle, inRectPtr, sizeof(Rect), outDescPtr);
	return result;
}// CreateRectDesc


/*!
Creates an Apple Event descriptor wrapper
for a QuickDraw RGB color.

(3.0)
*/
OSErr
BasicTypesAE_CreateRGBColorDesc		(RGBColor const*	inColor,
									 AEDesc*			outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(cRGBColor, inColor, sizeof(RGBColor), outDescPtr);
	return result;
}// CreateRGBColorDesc


/*!
Creates an Apple Event descriptor wrapper
for a signed, long integer.

(3.0)
*/
OSErr
BasicTypesAE_CreateSInt32Desc		(SInt32 const		inInteger,
									 AEDesc*			outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(typeSInt32, &inInteger, sizeof(inInteger), outDescPtr);
	return result;
}// CreateSInt32Desc


/*!
Creates an Apple Event descriptor wrapper
for an unsigned, long integer.

(3.0)
*/
OSErr
BasicTypesAE_CreateUInt32Desc		(UInt32 const		inInteger,
									 AEDesc*			outDescPtr)
{
	OSErr		result = noErr;
	
	
	result = AECreateDesc(typeUInt32, &inInteger, sizeof(inInteger), outDescPtr);
	return result;
}// CreateUInt32Desc


/*!
Creates an Apple Event descriptor wrapper
for a list containing two unsigned, long
integers.

(3.0)
*/
OSErr
BasicTypesAE_CreateUInt32PairDesc	(UInt32 const		inInteger1,
									 UInt32 const		inInteger2,
									 AEDesc*			outDescListPtr)
{
	OSErr		result = noErr;
	AEDesc		integerDesc = { typeNull, null };
	
	
	result = AECreateList(null/* factoring */, 0/* size */, false/* is a record */, outDescListPtr);
	result = AECreateDesc(typeUInt32, &inInteger1, sizeof(inInteger1), &integerDesc);
	AEPutDesc(outDescListPtr, 1/* index */, &integerDesc);
	AEDisposeDesc(&integerDesc);
	result = AECreateDesc(typeUInt32, &inInteger2, sizeof(inInteger2), &integerDesc);
	AEPutDesc(outDescListPtr, 2/* index */, &integerDesc);
	AEDisposeDesc(&integerDesc);
	
	return result;
}// CreateUInt32PairDesc