/*###############################################################

	BasicTypesAE.h
	
	Lists convenient functions for creating Apple Event
	descriptor wrappers of commonly-used types; indispensable
	for recordable applications.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

#ifndef __BASICTYPESAE__
#define __BASICTYPESAE__

// Mac includes
#include <AppleEvents.h>
#include <Files.h>
#include <MacTypes.h>



//
// public methods
//

OSErr
	BasicTypesAE_CreateBooleanDesc			(Boolean const		inBooleanValue,
											 AEDesc*			outDescPtr);

OSErr
	BasicTypesAE_CreateEnumerationDesc		(FourCharCode const	inEnumeration,
											 AEDesc*			outDescPtr);

OSErr
	BasicTypesAE_CreateFileOrFolderDesc		(FSSpec const*		inFSSpecPtr,
											 AEDesc*			outDescPtr);

OSErr
	BasicTypesAE_CreatePairDesc				(AEDesc const*		inDesc1Ptr,
											 AEDesc const*		inDesc2Ptr,
											 AEDescList*		outDescListPtr);

OSErr
	BasicTypesAE_CreatePointDesc			(Point const		inPoint,
											 AEDesc*			outDescPtr);

OSErr
	BasicTypesAE_CreatePropertyIDDesc		(FourCharCode const	inPropertyID,
											 AEDesc*			outDescPtr);

OSErr
	BasicTypesAE_CreatePStringDesc			(Str255				inPascalStringData,
											 AEDesc*			outDescPtr);

OSErr
	BasicTypesAE_CreateRectDesc				(Rect const*		inRectPtr,
											 AEDesc*			outDescPtr);

OSErr
	BasicTypesAE_CreateRGBColorDesc			(RGBColor const*	inColor,
											 AEDesc*			outDescPtr);

OSErr
	BasicTypesAE_CreateSInt32Desc			(SInt32 const		inInteger,
											 AEDesc*			outDescPtr);

// ALTHOUGH YOU COULD USE BasicTypesAE_CreateUInt32Desc() FOR GET DATA SIZE, PLEASE USE THIS INSTEAD
#ifdef __cplusplus
inline OSErr BasicTypesAE_CreateSizeDesc	(Size const			inSize,
											 AEDesc*			outDescPtr)
{ return BasicTypesAE_CreateUInt32Desc(inSize, outDescPtr); }
#else
#	define BasicTypesAE_CreateSizeDesc(inSize, outDescPtr) BasicTypesAE_CreateUInt32Desc(inSize, outDescPtr)
#endif

OSErr
	BasicTypesAE_CreateUInt32Desc			(UInt32 const		inInteger,
											 AEDesc*			outDescPtr);

OSErr
	BasicTypesAE_CreateUInt32PairDesc		(UInt32 const		inInteger1,
											 UInt32 const		inInteger2,
											 AEDescList*		outDescListPtr);

#endif