/*###############################################################

	CoercionsAE.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// standard-C includes
#include <ctype.h>
#include <string.h>

// Mac includes
#include <AEDataModel.h>
#include <AEObjects.h>
#include <AppleEvents.h>
#include <ASRegistry.h>
#include <MacErrors.h>
#include <PLStringFuncs.h>
#include <Processes.h>

// resource includes
#include "StringResources.h"

// MacTelnet includes
#include "AppleEventUtilities.h"
#include "BasicTypesAE.h"
#include "CoercionsAE.h"
#include "Console.h"
#include "GenesisAE.h"
#include "InstallAE.h"
#include "ObjectClassesAE.h"
#include "Parse.h"
#include "RequiredAE.h"
#include "Terminology.h"



//
// public methods
//

/*!
Handles coercions between Apple Event descriptor data types.
For example, a string data type can be coerced into a class
if there is some kind of list of names for classes of that
type (like terminal and session names), and a window can be
coerced into any class associated with that window.

(3.0)
*/
pascal OSErr
CoercionsAE_CoerceOneDataTypeToAnother	(DescType				inCurrentDataType,
										 const void*			inDataPtr,
										 Size					inDataSize,
										 DescType				inNewDataType,
										 SInt32					UNUSED_ARGUMENT(inRefCon),
										 AEDesc*				outResultDescPtr)
{
	OSErr		result = errAECoercionFail;
	
	
	switch (inCurrentDataType)
	{
		default:
			// ???
			break;
	}
	
	return result;
}// CoerceOneDataTypeToAnother


/*!
Handles coercions between objects and other types
using standard means.  This allows objects to be
transformed into any MacTelnet-defined class or
property.

(3.0)
*/
pascal OSErr
CoercionsAE_CoerceObjectToAnotherType	(const AEDesc*	inOriginalDescPtr,
										 DescType		inDesiredType,
										 SInt32			inRefCon,
										 AEDesc*		outResultDescPtr)
{
	OSErr		result = noErr;
	
	
	Console_BeginFunction();
	Console_WriteLine("coercing object to token");
	if (inOriginalDescPtr->descriptorType != typeObjectSpecifier) result = errAEWrongDataType;
	else
	{
		AEDesc		token = { typeNull, null };
		
		
		result = AEResolve(inOriginalDescPtr, kAEIDoMinimum, &token); // *might* do the trick...
		if (result == noErr) result = AECoerceDesc(&token, inDesiredType, outResultDescPtr);
		if (token.dataHandle != null) AEDisposeToken(&token);
	}
	Console_WriteValue("result", result);
	Console_EndFunction();
	return result;
}// CoerceObjectToAnotherType