/*###############################################################

	CoreSuiteAE.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// Mac includes
#include <AppleEvents.h>
#include <Folders.h>
#include <MacErrors.h>
#include <OSA.h>
#include <Processes.h>
#include <Resources.h>

// MacTelnet includes
#include "AppleEventUtilities.h"
#include "Connections.h"
#include "Console.h"
#include "CoreSuiteAE.h"
#include "ObjectClassesAE.h"
#include "TelnetMacros.h"
#include "Terminology.h"



// internal methods
static OSErr		countElements		(AEDesc const*		inFromWhichObject,
										 DescType			inGetDataOfThisType,
										 AEDesc*			outCount);



//
// public methods
//

/*!
Counts elements in the given container.

(3.0)
*/
pascal OSErr
CoreSuiteAE_CountElements		(AppleEvent const*		inAppleEventPtr,
								 AppleEvent*			outReplyAppleEventPtr, 
								 unsigned long			UNUSED_ARGUMENT(inData))
{
	AEDesc        	directObject = { typeNull, null },
					replyDesc = { typeNull, null };
	DescType		classType = typeNull;
	Size			actualSize = 0L;
	OSErr         	result = noErr;
	
	
	Console_BeginFunction();
	Console_WriteLine("AppleScript: count event");
	
	result = AEGetParamDesc(inAppleEventPtr, keyDirectObject, typeWildCard, &directObject);
	
	// set up class parameter: this is a required parameter
	if (result == noErr)
	{
		DescType		returnedType = typeWildCard;
		
		
		result = AEGetParamPtr(inAppleEventPtr, kTelnetPartialCoreParameterKeyCountElementsEach,
								kTelnetPartialCoreParmTypeCountElementsEach,
								&returnedType, &classType, sizeof(classType), &actualSize);
		if (result == noErr) result = AppleEventUtilities_RequiredParametersError(inAppleEventPtr);
		if (result == noErr) result = countElements(&directObject, classType, &replyDesc);
	}
	
	result = AppleEventUtilities_AddResultToReply(&replyDesc, outReplyAppleEventPtr, result);
	
	if (directObject.dataHandle) AEDisposeDesc(&directObject);
	
	Console_WriteValue("result", result);
	Console_EndFunction();
	return result;
}// CountElements


/*!
This callback routine, of OSLCountUPP form,
counts the number of items in the specified
container object.

Incomplete.

(3.0)
*/
pascal OSErr
CoreSuiteAE_CountElementsOfContainer	(DescType			inTypeOfCountedElements,
										 DescType			inContainerClass,
										 AEDesc const*		inContainerTokenDescriptor,
										 long*				outCount)
{
	OSErr		result = noErr;
	AEDesc		containerDesc = { typeNull, null };
	
	
	// ensure the data is a token
	result = AppleEventUtilities_DuplicateOrResolveObject(inContainerTokenDescriptor, &containerDesc);
	if (result == noErr)
	{
		// initialize count to illegal value to catch errors
		*outCount = -1L;
		
		switch (inTypeOfCountedElements)
		{
			case kTelnetClassWindow:
				// count all open windows
				*outCount = 0L;
				if ((inContainerClass == typeNull) || (inContainerClass == cApplication))
				{
					WindowRef		window = FrontWindow();
					
					
					while (window != null)
					{
						(*outCount)++;
						window = GetNextWindow(window);
					}
				}
				break;
			
			case kTelnetClassSession:
				// count all open sessions
				*outCount = Connections_GetCount();
				break;
			
			case kTelnetClassMacroSet:
				// count all macro sets
				*outCount = 0L;
				if (inContainerClass == typeNull)
				{
					// this value is currently a constant
					*outCount = MACRO_SET_COUNT;
				}
				break;
			
			case kTelnetClassMacro:
				// count all macros in a set
				*outCount = 0L;
				if (inContainerClass == kTelnetClassMacroSet)
				{
					// this value is currently a constant
					*outCount = MACRO_COUNT;
				}
				break;
			
			case kTelnetClassApplicationPreferences:
			case kTelnetClassClipboardWindow:
			case kTelnetClassApplication:
				// return 1 for objects that exist in exactly once place (who knows
				// why theyd ever be counted, but hey, its gotta be correct...)
				*outCount = 1L;
				break;
			
			default:
				result = errAECantHandleClass;
				break;
		}
	}
	return result;
}// CountElementsOfContainer


//
// internal methods
//

/*!
Returns a count of the elements in the specified
container (or a sum of the counts in a list of
containers).

(3.0)
*/
static OSErr
countElements		(AEDesc const*		inFromWhichObject,
					 DescType			inGetDataOfThisType,
					 AEDesc*			outCount)
{
	AEDesc		tokenDescriptor = { typeNull, null };
	OSErr		result = noErr;
	
	
	Console_WriteValueFourChars("desired type", inGetDataOfThisType);
	
	result = AppleEventUtilities_DuplicateOrResolveObject(inFromWhichObject, &tokenDescriptor);
	if (result == noErr)
	{
		Console_WriteLine("successfully resolved");
		Console_WriteValueFourChars("to type", tokenDescriptor.descriptorType);
		switch (tokenDescriptor.descriptorType)
		{
			case typeAEList:
				result = AECreateList(null, 0, false/* is a record */, outCount);
				if (result == noErr)
				{
					AEDesc		listItemDescriptor = { typeNull, null },
								resultDescriptor = { typeNull, null };
					DescType	returnedType;
					long		index = 0L;
					
					
					result = AECountItems(&tokenDescriptor, &index);
					for (; index > 0; index--)
					{
						result = AEGetNthDesc(&tokenDescriptor, index, typeWildCard,
												&returnedType, &listItemDescriptor);
						
						// handle the request by adding the resultant data to the end of the list
						if (result == noErr) result = countElements(&listItemDescriptor,
																	inGetDataOfThisType, &resultDescriptor);
						if (result == noErr) result = AEPutDesc(&tokenDescriptor/* list */, index,
																&resultDescriptor/* item to add */);
						
						if (listItemDescriptor.dataHandle != null) AEDisposeDesc(&listItemDescriptor);
						if (resultDescriptor.dataHandle != null) AEDisposeDesc(&resultDescriptor);
					}
					result = AEDuplicateDesc(&tokenDescriptor, outCount);
				}
				break;
			
			default:
				{
					SInt32		count = 0L;
					
					
					result = CoreSuiteAE_CountElementsOfContainer(inGetDataOfThisType,
																	tokenDescriptor.descriptorType,
																	&tokenDescriptor, &count);
					if (result == noErr) result = AECreateDesc(kTelnetPartialCoreReturnTypeCountElements,
																&count, sizeof(count), outCount);
				}
				break;
		}
	}
	
	if (tokenDescriptor.dataHandle != null) AEDisposeToken(&tokenDescriptor);
	
	return result;
}// countElements