/*###############################################################

	HiddenAE.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// Mac includes
#include <AEDataModel.h>
#include <AppleEvents.h>
#include <ASRegistry.h>

// library includes
#include "InterfaceLib.h"

// resource includes
#include "MenuResources.h"

// MacTelnet includes
#include "AppleEventUtilities.h"
#include "Commands.h"
#include "Console.h"
#include "HiddenAE.h"
#include "NewSessionsDialog.h"
#include "PreferencesDialog.h"
#include "TelnetDescriptors.h"
#include "Terminology.h"



//
// public methods
//

/*!
Handles the following event types:

kTelnetEventClass
-----------------
	kTelnetHiddenEventIDConsoleWriteLine

(3.0)
*/
pascal OSErr
HiddenAE_HandleConsoleWriteLine		(const AppleEvent*	inAppleEventPtr,
									 AppleEvent*		outReplyAppleEventPtr,
									 UInt32				inData)
{
	OSErr		result = noErr;
	DescType	returnedType = '????';
	Size		actualSize = 0L;
	SInt32		dialogDescriptor = kInvalidWindowInfoDescriptor;
	char		buffer[256];
	
	
	result = AEGetParamPtr(inAppleEventPtr, keyDirectObject, cStringClass, &returnedType, buffer + 1,
							sizeof(buffer) - 1 * sizeof(char), &actualSize);
	buffer[0] = actualSize / sizeof(unsigned char);
	StringUtilities_PToCInPlace((StringPtr)buffer);
	if (result == noErr)
	{
		// check for missing parameters
		result = AppleEventUtilities_RequiredParametersError(inAppleEventPtr);
		if (result == noErr)
		{
			Console_WriteLine(buffer);
		}
	}
	
	if (result != noErr) BEEP;
	return result;
}// HandleConsoleWriteLine


/*!
Handles the following event types:

kTelnetEventClass
-----------------
	kTelnetHiddenEventIDDisplayDialog

(3.0)
*/
pascal OSErr
HiddenAE_HandleDisplayDialog		(const AppleEvent*	inAppleEventPtr,
									 AppleEvent*		outReplyAppleEventPtr,
									 UInt32				inData)
{
	OSErr			result = noErr;
	DescType		returnedType = '????';
	Size			actualSize = 0L;
	FourCharCode	dialogCode = '????';
	
	
	result = AEGetParamPtr(inAppleEventPtr, keyDirectObject, typeEnumerated, &returnedType,
								&dialogCode, sizeof(dialogCode), &actualSize);
	if (result == noErr)
	{
		// check for missing parameters
		result = AppleEventUtilities_RequiredParametersError(inAppleEventPtr);
		if (result == noErr) switch (dialogCode)
		{
			case 'news':
				unless (NewSessionsDialog_IsDisplayed())
				{
					(Boolean)Commands_ExecuteByID(kCommandOpenConnection, 0/* modifiers */);
				}
				else result = errAEEventNotHandled;
				break;
			
			case 'pref':
				PreferencesDialog_Display();
				break;
			
			case 'bugs':
				#ifndef NDEBUG
				Console_SetVisible(true);
				#else
				BEEP;
				#endif
				break;
			
			default:
				result = errAEEventNotHandled;
				break;
		}
	}
	
	if (result != noErr) BEEP;
	return result;
}// HandleDisplayDialog