/*###############################################################

	Terminology.h
	
	This file contains Apple Events and corresponding classes
	that MacTelnet recognizes (for scriptability).  All of this
	information is very important!  Grouped four-character codes
	must be unique within the group - the group is determined by
	a common prefix and/or suffix in the name.
	
	IMPORTANT:	Although most important information is external
				from the terminology extension resource file
				(" TelnetAE.r"), it may be necessary to update
				the terminology extension resource definition if
				you change certain things in this file (for
				example, if you change a parameter type from a
				class to an enumeration).
	
	MacTelnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __TERMINOLOGY__
#define __TERMINOLOGY__

// resource includes
#include "GeneralResources.h"



// types
#define kASTypeDefinitionsSuite										'tpdf'
#define kAESpecialClassPropertiesParentType							'c@#^'

/*###########################################
	Standard Suite Extensions
###########################################*/

#define kTelnetPartialAECoreSuite									typeWildCard

// constants for existing parameters of Standard Suite events that MacTelnet allows
#define			kTelnetPartialCoreReturnTypeCountElements				typeSInt32
#define			kTelnetPartialCoreParmTypeCountElementsDirect			cReference
#define		kTelnetPartialCoreParameterKeyCountElementsEach			keyAEObjectClass
#define			kTelnetPartialCoreParmTypeCountElementsEach				typeType

#define			kTelnetPartialCoreParmTypeCloseDirect					cReference
#define		kTelnetPartialCoreParameterKeyCloseSavingInFile			'KlSF'
#define			kTelnetPartialCoreParmTypeCloseSavingInFile				typeAlias

#define			kTelnetPartialCoreReturnTypeCreateElement				cReference
#define		kTelnetPartialCoreParameterKeyCreateElementNew			keyAEObjectClass
#define			kTelnetPartialCoreParmTypeCreateElementNew				typeType
#define		kTelnetPartialCoreParameterKeyCreateElementProperties	keyAEProperties
#define			kTelnetPartialCoreParmTypeCreateElementProperties		cRecord

#define			kTelnetPartialCoreReturnTypeGetDataSize					typeSInt32
#define			kTelnetPartialCoreParmTypeGetDataSizeDirect				cReference
#define		kTelnetPartialCoreParameterKeyGetDataSizeAs				keyAERequestedType
#define			kTelnetPartialCoreParmTypeGetDataSizeAs					typeType

#define			kTelnetPartialCoreParmTypeDeleteDirect					cReference

#define			kTelnetPartialCoreReturnTypeDuplicate					cReference
#define			kTelnetPartialCoreParmTypeDuplicateDirect				cReference

#define			kTelnetPartialCoreReturnTypeExists						typeBoolean
#define			kTelnetPartialCoreParmTypeExistsDirect					cReference

#define			kTelnetPartialCoreReturnTypeGetData						typeWildCard
#define			kTelnetPartialCoreParmTypeGetDataDirect					cReference
#define		kTelnetPartialCoreParameterKeyGetDataAs					keyAERequestedType
#define			kTelnetPartialCoreParmTypeGetDataAs						typeType

#define			kTelnetPartialCoreParmTypePrintDirect					cReference
#define		kTelnetPartialCoreParameterKeyPrintCopies				'PCop'
#define			kTelnetPartialCoreParmTypePrintCopies					typeSInt16

#define			kTelnetPartialCoreParmTypeSetDataDirect					cReference
#define		kTelnetPartialCoreParameterKeySetDataTo					keyAEData
#define			kTelnetPartialCoreParmTypeSetDataTo						typeWildCard

// classes, with properties
#define kTelnetClassApplication												cApplication
#define		kTelnetPropertyActiveMacroSet										'RAAM'
#define			kTelnetPTypActiveMacroSet											kTelnetClassMacroSet
#define		kTelnetPropertyClipboard											'RACl'
#define			kTelnetPTypClipboard												kTelnetClassClipboardWindow
#define		kTelnetPropertyFolderScriptsMenuItems								'RAĺ'
#define			kTelnetPTypFolderScriptsMenuItems									typeFSS
#define		kTelnetPropertyFolderStartupItems									'RAS'
#define			kTelnetPTypFolderStartupItems										typeFSS
#define		kTelnetPropertyIPAddress											'RAIP'
#define			kTelnetPTypIPAddress												cStringClass
#define		kTelnetPropertyIsFrontmost											pIsFrontProcess
#define			kTelnetPTypIsFrontmost												typeBoolean
#define		kTelnetPropertyPreferences											'RAPf'
#define			kTelnetPTypPreferences												kTelnetClassApplicationPreferences
#define		kTelnetPropertyProgramName											pName
#define			kTelnetPTypProgramName												cIntlText
#define		kTelnetPropertyProgramVersion										pVersion
#define			kTelnetPTypProgramVersion											cVersion
#define kTelnetClassWindow													cWindow
#define		kTelnetPropertyWindowBounds											pBounds
#define			kTelnetPTypWindowBounds												cQDRectangle
#define		kTelnetPropertyWindowHasCloseBox									pHasCloseBox
#define			kTelnetPTypWindowHasCloseBox										typeBoolean
#define		kTelnetPropertyWindowHasCollapseBox									'RW=='
#define			kTelnetPTypWindowHasCollapseBox										typeBoolean
#define		kTelnetPropertyWindowHasTitleBar									pHasTitleBar
#define			kTelnetPTypWindowHasTitleBar										typeBoolean
#define		kTelnetPropertyWindowHidden											'RWOb'
#define			kTelnetPTypWindowHidden												typeBoolean
#define		kTelnetPropertyWindowIndexInWindowList								pIndex
#define			kTelnetPTypWindowIndexInWindowList									typeSInt16
#define		kTelnetPropertyWindowIsCollapsed									'RWSh'
#define			kTelnetPTypWindowIsCollapsed										typeBoolean
#define		kTelnetPropertyWindowIsFloating										pIsFloating
#define			kTelnetPTypWindowIsFloating											typeBoolean
#define		kTelnetPropertyWindowIsModal										pIsModal
#define			kTelnetPTypWindowIsModal											typeBoolean
#define		kTelnetPropertyWindowIsResizeable									pIsResizable
#define			kTelnetPTypWindowIsResizeable										typeBoolean
#define		kTelnetPropertyWindowIsVisible										pVisible
#define			kTelnetPTypWindowIsVisible											typeBoolean
#define		kTelnetPropertyWindowIsZoomable										pIsZoomable
#define			kTelnetPTypWindowIsZoomable											typeBoolean
#define		kTelnetPropertyWindowIsZoomed										pIsZoomed
#define			kTelnetPTypWindowIsZoomed											typeBoolean
#define		kTelnetPropertyWindowName											pName
#define			kTelnetPTypWindowName												cIntlText
#define		kTelnetPropertyWindowPosition										'RWxy'
#define			kTelnetPTypWindowPosition											cQDPoint

/*###########################################
	URL Suite
###########################################*/

#define kStandardURLEventClass										'GURL'

// implemented / extended events
#define kStandardURLEventIDGetURL									'GURL'
#define		kStandardURLReturnTypeGetURL								typeSInt16
#define		kStandardURLParmTypeGetURLDirect							cReference

/*###########################################
	MacTelnet Suite
###########################################*/

// the MacTelnet suite ID matches its application signature, to avoid collisions with other application suites
#define kTelnetEventClass											kApplicationCreatorSignature

#define kTelnetEventIDAlert											'TDAl'
#define			kTelnetReturnTypeAlert									kTelnetClassDialogReply
#define			kTelnetParmTypeAlertDirect								cIntlText
#define		kTelnetParameterKeyAlertBeeps							'ASnd'
#define			kTelnetParmTypeAlertBeeps								typeBoolean
#define		kTelnetParameterKeyAlertButtons							'ABtL'
#define			kTelnetParmTypeAlertButtons								cList
#define		kTelnetParameterKeyAlertButtonKeys						'AKyL'
#define			kTelnetParmTypeAlertButtonKeys							cList
#define		kTelnetParameterKeyAlertDisruptsNetwork					'ADsN'
#define			kTelnetParmTypeAlertDisruptsNetwork						typeBoolean
#define		kTelnetParameterKeyAlertGivingUpAfter					'AGUA'
#define			kTelnetParmTypeAlertGivingUpAfter						typeSInt16
#define		kTelnetParameterKeyAlertHasHelpButton					'AHs?'
#define			kTelnetParmTypeAlertHasHelpButton						typeBoolean
#define		kTelnetParameterKeyAlertHasTitleBar						'AHsT'
#define			kTelnetParmTypeAlertHasTitleBar							enumBooleanValues
#define		kTelnetParameterKeyAlertHelpText						'A?Tx'
#define			kTelnetParmTypeAlertHelpText							cIntlText
#define		kTelnetParameterKeyAlertIcon							'AIcn'
#define			kTelnetParmTypeAlertIcon								kTelnetEnumeratedClassAlertIconType
#define		kTelnetParameterKeyAlertSpeech							'ASpk'
#define			kTelnetParmTypeAlertSpeech								typeBoolean
#define		kTelnetParameterKeyAlertTitle							'ATtl'
#define			kTelnetParmTypeAlertTitle								cIntlText

#define kTelnetEventIDConnectionOpen								'TCnO'
#define			kTelnetReturnTypeConnectionOpen							kTelnetClassConnection
#define		kTelnetParameterKeyConnectionOpenHost					'OHst'
#define			kTelnetParmTypeConnectionOpenHost						cStringClass
#define		kTelnetParameterKeyConnectionOpenPortNumber				'OPtN'
#define			kTelnetParmTypeConnectionOpenPortNumber					typeSInt16
#define		kTelnetParameterKeyConnectionOpenProtocol				'OPro'
#define			kTelnetParmTypeConnectionOpenProtocol					kTelnetEnumeratedClassConnectionMethod
#define		kTelnetParameterKeyConnectionOpenProxyServer			'OPrx'
#define			kTelnetParmTypeConnectionOpenProxyServer				kTelnetClassProxyServerConfiguration
#define		kTelnetParameterKeyConnectionOpenTimeoutInSeconds		'OWai'
#define			kTelnetParmTypeConnectionOpenTimeoutInSeconds			typeUInt32
#define		kTelnetParameterKeyConnectionOpenWindowTitle			'OTtl'
#define			kTelnetParmTypeConnectionOpenWindowTitle				cStringClass

#define kTelnetEventIDCopyToClipboard								'TCpy'
#define		kTelnetParameterKeyCopyToClipboardTable					'CTbl'
#define			kTelnetParmTypeCopyToClipboardTable						enumBooleanValues

#define kTelnetEventIDDataSend										'TDat'
#define			kTelnetParmTypeDataSendDirect							typeWildCard
#define			kTelnetParmTypeDataSendDirect2							kTelnetEnumeratedClassSpecialCharacter
#define		kTelnetParameterKeyDataSendToConnection					'DCon'
#define			kTelnetParmTypeDataSendToConnection						kTelnetClassConnection
#define		kTelnetParameterKeyDataSendTimeoutInSeconds				'DLim'
#define			kTelnetParmTypeDataSendTimeoutInSeconds					typeUInt32
#define		kTelnetParameterKeyDataSendWithLocalEcho				'DEco'
#define			kTelnetParmTypeDataSendWithLocalEcho					enumBooleanValues
#define		kTelnetParameterKeyDataSendWithNewLine					'DRet'
#define			kTelnetParmTypeDataSendWithNewLine						enumBooleanValues

#define kTelnetEventIDDataWait										'TWai'
#define		kTelnetParameterKeyDataWaitForWhat						'WFor'
#define			kTelnetParmTypeDataWaitForWhat							typeWildCard
#define		kTelnetParameterKeyDataWaitFromConnection				'WCon'
#define			kTelnetParmTypeDataWaitFromConnection					kTelnetClassConnection
#define		kTelnetParameterKeyDataWaitTimeoutInSeconds				'WLim'
#define			kTelnetParmTypeDataWaitTimeoutInSeconds					typeUInt32

#define kTelnetEventIDLaunchFind									'TFnd'

// HIDDEN events; these are only used by MacTelnet Help scripts (to do things for the user),
// and by MacTelnets libraries, in order to communicate with the debugging console.
#define kTelnetHiddenEventIDConsoleWriteLine						'?WrL'
#define kTelnetHiddenEventIDDisplayDialog							'?Dlg'

// classes, with properties
#define kTelnetClassApplicationPreferences							'TPrf'
#define		kTelnetPropertyApplicationPreferencesCursorShape			'RPCs'
#define			kTelnetPTypApplicationPreferencesCursorShape			kTelnetEnumeratedClassCursorShape
#define		kTelnetPropertyApplicationPreferencesSimpleTelnet			'RPST'
#define			kTelnetPTypApplicationPreferencesSimpleTelnet			typeBoolean
#define		kTelnetPropertyApplicationPreferencesVisualBell				'RPVB'
#define			kTelnetPTypApplicationPreferencesVisualBell				typeBoolean
#define		kTelnetPropertyApplicationPreferencesWindowStackingOrigin	'RPSO'
#define			kTelnetPTypApplicationPreferencesWindowStackingOrigin	cQDPoint
#define kTelnetClassClipboardWindow									'TClp'
#define		kTelnetPropertyClipboardWindowContents						'RClC'
#define kTelnetClassConnection										'TCxn'
#define		kTelnetPropertyConnectionLocationNameOrIPAddress			'RCNI'
#define		kTelnetPropertyConnectionLocationPortNumber					'RCPN'
#define		kTelnetPropertyConnectionProxy								'RCPx'
#define		kTelnetPropertyConnectionStatus								'RCSt'
#define kTelnetClassDialogReply										'TDgR'
#define		kTelnetPropertyDialogReplyButtonReturned					'RDBR'
#define		kTelnetPropertyDialogReplyGaveUp							'RDGU'
#define kTelnetClassFileCapture										'TFlC'
#define		kTelnetPropertyFileCaptureConnection						'RFCn'
#define			kTelnetPTypFileCaptureConnection						kTelnetClassConnection
#define		kTelnetPropertyFileCaptureCreator							'RFFC'
#define			kTelnetPTypFileCaptureCreator							typeApplSignature
#define		kTelnetPropertyFileCaptureFile								'RFFl'
#define			kTelnetPTypFileCaptureFile								typeAlias
#define		kTelnetPropertyFileCaptureStatus							'RFSt'
#define			kTelnetPTypFileCaptureStatus							kTelnetEnumeratedClassFileCaptureStatus
#define		kTelnetPropertyFileCaptureText								'RFTx'
#define			kTelnetPTypFileCaptureText								typeChar
#define kTelnetClassMacro											'TMcr'
#define		kTelnetPropertyMacroKeyEquivalent							'RmKE'
#define		kTelnetPropertyMacroTextString								'RmTS'
#define kTelnetClassMacroSet										'TMcS'
#define		kTelnetPropertyMacroSetKeyEquivalents						'RMKE'
#define kTelnetClassProxyServerConfiguration						'TPrx'
#define		kTelnetPropertyProxyServerLocationNameOrIPAddress			'RpNI'
#define		kTelnetPropertyProxyServerLocationPortNumber				'RpPN'
#define		kTelnetPropertyProxyServerMethod							'RpMt'
#define		kTelnetPropertyProxyServerAuthenticationType				'RpAT'
#define		kTelnetPropertyProxyServerUserName							'RpUN'
#define kTelnetClassSession											'TSss'
#define		kTelnetPropertySessionConnection							'RSCn'
#define		kTelnetPropertySessionWindow								'RSWn'
#define kTelnetClassTerminalWindow									'TTWn'
#define		kTelnetPropertyTerminalWindowColumns						'RTvc'
#define			kTelnetPTypTerminalWindowColumns						typeSInt16
#define		kTelnetPropertyTerminalWindowRows							'RTvr'
#define			kTelnetPTypTerminalWindowRows							typeSInt16
#define		kTelnetPropertyTerminalWindowDimensions						'RTwh'
#define			kTelnetPTypTerminalWindowDimensions						typeSInt16
#define		kTelnetPropertyTerminalWindowScrollBack						'RTir'
#define			kTelnetPTypTerminalWindowScrollBack						typeSInt16
#define		kTelnetPropertyTerminalWindowFontFamilyName					'RTAa'
#define			kTelnetPTypTerminalWindowFontFamilyName					cIntlText
#define		kTelnetPropertyTerminalWindowFontFamilyNameBold				'RT'
#define			kTelnetPTypTerminalWindowFontFamilyNameBold				cIntlText
#define		kTelnetPropertyTerminalWindowFontSize						'RTN'
#define			kTelnetPTypTerminalWindowFontSize						typeSInt16
#define		kTelnetPropertyTerminalWindowFontSizeBold					'RTB'
#define		kTelnetPropertyTerminalWindowFontColorText					'RTCT'
#define		kTelnetPropertyTerminalWindowFontColorBackground			'RTCB'
#define		kTelnetPropertyTerminalWindowFontColorBoldText				'RTCt'
#define		kTelnetPropertyTerminalWindowFontColorBoldBackground		'RTCb'
#define		kTelnetPropertyTerminalWindowFontColorBlinkingText			'RTC'
#define		kTelnetPropertyTerminalWindowFontColorBlinkingBackground	'RTC'
#define		kTelnetPropertyTerminalWindowContents						'RTtx'
#define			kTelnetPTypTerminalWindowContents						kTelnetClassText
#define		kTelnetPropertyTerminalWindowScreenContents					'RTsc'
#define			kTelnetPTypTerminalWindowScreenContents					kTelnetClassText
#define		kTelnetPropertyTerminalWindowScrollbackContents				'RTsb'
#define			kTelnetPTypTerminalWindowScrollbackContents				kTelnetClassText
#define kTelnetClassText											'TTxt'
#define		kTelnetPropertyTextLength									'Rtsz'
#define			kTelnetPTypTextLength									typeUInt32
#define		kTelnetPropertyTextOffset									'Rtof'
#define			kTelnetPTypTextOffset									typeUInt32
#define		kTelnetPropertyTextWordCount								'Rtwc'
#define			kTelnetPTypTextWordCount								typeUInt32
#define kTelnetClassWord											'TWrd'
#define		kTelnetPropertyWordLength									'Rwsz'
#define			kTelnetPTypWordLength									typeUInt32
#define		kTelnetPropertyWordOffset									'Rwof'
#define			kTelnetPTypWordOffset									typeUInt32

// HIDDEN classes; these are used internally to process Apple Events and resolve objects
#define kTelnetHiddenClassTelnetToken								'TTok'

// enumerations
#define kTelnetEnumeratedClassAlertIconType							'EAIT'
#define		kTelnetEnumeratedClassAlertIconTypeNote						'AIT*'
#define		kTelnetEnumeratedClassAlertIconTypeStop						'AITX'
#define		kTelnetEnumeratedClassAlertIconTypeCaution					'AIT!'
#define kTelnetEnumeratedClassConnectionMethod						'ECMt'
#define		kTelnetEnumeratedClassConnectionMethodTelnet				'CMt1'
#define		kTelnetEnumeratedClassConnectionMethodFTP					'CMt2'
#define kTelnetEnumeratedClassConnectionStatus						'ECSt'
#define		kTelnetEnumeratedClassConnectionStatusWaitingForDNR			'CStD'
#define		kTelnetEnumeratedClassConnectionStatusOpening				'CStO'
#define		kTelnetEnumeratedClassConnectionStatusActive				'CStA'
#define		kTelnetEnumeratedClassConnectionStatusDead					'CStX'
#define		kTelnetEnumeratedClassConnectionStatusNotInUse				'CSt-'
#define kTelnetEnumeratedClassCursorShape							'ECur'
#define		kTelnetEnumeratedClassCursorShapeBlock						'CurB'
#define		kTelnetEnumeratedClassCursorShapeVerticalBar				'CurV'
#define		kTelnetEnumeratedClassCursorShapeVerticalBarBold			'Cur'
#define		kTelnetEnumeratedClassCursorShapeUnderscore					'CurU'
#define		kTelnetEnumeratedClassCursorShapeUnderscoreBold				'Cur'
#define kTelnetEnumeratedClassFileCaptureStatus						'EFSt'
#define		kTelnetEnumeratedClassFileCaptureStatusNotBegun				'FStU'
#define		kTelnetEnumeratedClassFileCaptureStatusInProgress			'FStC'
#define		kTelnetEnumeratedClassFileCaptureStatusComplete				'FStD'
#define kTelnetEnumeratedClassMacroSetKeyEquivalents				'EMKE'
#define		kTelnetEnumeratedClassMacroSetKeyEquivalentsCommandDigit	'MKE'
#define		kTelnetEnumeratedClassMacroSetKeyEquivalentsFunctionKeys	'MKEF'
#define kTelnetEnumeratedClassProxyServerAuthenticationType			'EPAu'
#define		kTelnetEnumeratedClassProxyServerAuthenticationTypePassword	'PAUP'
#define kTelnetEnumeratedClassProxyServerMethod						'EPrx'
#define		kTelnetEnumeratedClassProxyServerMethodSOCKS4				'PMS4'
#define		kTelnetEnumeratedClassProxyServerMethodSOCKS4A				'PM4A'
#define		kTelnetEnumeratedClassProxyServerMethodSOCKS5				'PMS5'
#define kTelnetEnumeratedClassSpecialCharacter						'ESpC'
#define		kTelnetEnumeratedClassSpecialCharacterAreYouThere			'SpCY'
#define		kTelnetEnumeratedClassSpecialCharacterAbortOutput			'SpC.'
#define		kTelnetEnumeratedClassSpecialCharacterBreak					'SpC-'
#define		kTelnetEnumeratedClassSpecialCharacterCarriageReturn		'SpCR'
#define		kTelnetEnumeratedClassSpecialCharacterInterruptProcess		'SpCI'
#define		kTelnetEnumeratedClassSpecialCharacterEraseCharacter		'SpCE'
#define		kTelnetEnumeratedClassSpecialCharacterEraseLine				'SpCX'
#define		kTelnetEnumeratedClassSpecialCharacterEndOfFile				'SpCD'
#define		kTelnetEnumeratedClassSpecialCharacterLineFeed				'SpCL'
#define		kTelnetEnumeratedClassSpecialCharacterSync					'SpC='

#endif