/*###############################################################

	FTPBin.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// standard-C includes
#ifdef __cplusplus
#	include <cstdio>
#	include <cstdlib>
#	include <cstring>
#else
#	include <stdio.h>
#	include <stdlib.h>
#	include <string.h>
#endif
#include <ctype.h>

// Mac includes
#include <MacTypes.h>
#include <PLStringFuncs.h>
#include <TextUtils.h>

// resource includes
#include "StringResources.h"

// MacTelnet includes
#include "ConnectionData.h"
#include "Connections.h"
#include "DialogUtilities.h"
#include "FileUtilities.h"
#include "FTPInterpreter.h"
#include "FTPBin.h"
#include "InternalEvents.h"
#include "MacBinary.h"
#include "MacLook.h"
#include "NetBackground.h"
#include "NetEvents.h"
#include "network.proto.h"
#include "NewSessionsDialog.h"
#include "Parse.h"
#include "Preferences.h"
#include "RealScreen.h"
#include "ScreenManager.h"
#include "TelnetDescriptors.h"
#include "TelnetHeader.h"
#include "TelnetMenus.h"
#include "vsdata.h"
#include "VSkeys.h"

//#define	DEBUG_FTPCLIENT
#ifdef	DEBUG_FTPCLIENT
#define	ftpclient_debug_print(x) putln(x)
#else
#define ftpclient_debug_print(x)
#endif


// 3.0 - localization stuff
#define YES_CHAR	'y'										/* what is typed to answer "yes" to questions */


#define FASCII 0
#define FIMAGE 1										   /* This is different than the definition
														      in bkgr.c */
#define FAMODE 0
#define FIMODE 1
//#define FMMODE 2										   /* Mac Binary, when ready */

#define SUCCESS	   2
#define ERROR	  -1
#define NONE	  -2
#define ABORT	  -3
#define INCOMPLETE -4
#define AMBIGUOUS -5
#define HAVEDATA   4

#define BUFFERS 8000									   /* size of buffer */
#define PATHLEN 256
#define READSIZE 128									   /* how much to read */

#define HELPRESID 23229



long
        filelen = 0,									   /* file transfer length */
        start = 0L,										   /* timing var */
        towrite = 0,									   /* file transfer pointer */
        translen = 0,									   /* length of file transfered */
        transtime;

short
        xp = 0,											   /* general pointer */
        ftpfh,											   /* file handle for ftp */
        fcnt = 0,										   /* counter for ftpd */
        ftpfilemode = 0,								   /* file open mode for transfer */
        hash = 0,										   /* hash mark printing */
        sendport = 1,									   /* to send ports or not */
        verbose = 1,									   /* informative messages */
        bell = 0,										   /* sound bell */
        autologin = 1,									   /* login on connect */
        prompt = 1,										   /* check on multiple commands */
        glob = 1,										   /* expand wildcards */
        slashflip = 1,									   /* change \ to / */
        fromtty = 1;									   /* default input from tty */

char	Template[256];
CInfoPBRec	ClientDirectoryState;

static long gFTPClientDirID = 2;
static short gFTPClientVRefNum = -1;
static short ftpdata;
static char bellchar = {7};
static char backspace = {8};
static char poundsign = {'#'};

FILE*	fromfp = null;						/* file pointer for input */
MBFile*	ftp_mbfp;

unsigned char
       *captlist = 0L, *newfile = 0L, *transfile = 0L,	   /* name of file being sent/received */
       *xs = 0L;										   /* response string */
unsigned short curftpprt = 0;								   /* port to use */

#define RCPSEGSIZE 512
#define O_RAW O_RDONLY
#define EOLCHAR 13
#define captlistsize 2001L



// internal method prototypes
static void		putFTPClientMessage		(ConnectionDataPtr		inScreen,
										 const short			inStringIndex);
static void		putFTPClientPrompt		(ConnectionDataPtr		inScreen,
										 const short			inStringIndex);
static void		putstring				(ConnectionDataPtr		inScreen,
										 char*					inString);



/**
To get a pointer to the first non blank
(space or tab) character in a string,
use this method.

(2.6)
*/
char*
stpblkp		(char*		mystring)
{
	short		count = 0;
	char*		result = mystring;
	
	
	for (; (((*result == ' ') || (*result == '\t')) && (count < 256)); result++, count++) {}
	return result;
}// stpblkp


/**
To tokenize and get a pointer to the first
non blank (space or tab) character in a
string, use this method.

(2.6)
*/
char*
stptok		(char*			mystring,
			 char*			myword,
			 short			mylen,
			 char*			myterms)
{
	/* Return pointer to first non blank in string */
	short		done = 0,
				i = 0;
	char*		p = null;
	char*		q = null;
	
	
	p = stpblkp(mystring); // skip leading blanks
	i = 0;
	done = 0;
	while (*(p + i) && (i <= mylen) && (done == 0))
	{
		q = myterms;
		while ((*q) && (*q != *(p + i))) q++;
		if (!(*q))
		{
			*(myword + i) = *(p + i);
			i++;
		}
		else done = 1;
	}
	*(myword + i) = '\0';
	q = p + i;
	
	return q;
}// stptok


/**
To open an FTP client file, call this method.

(2.6)
*/
short
ftpopen		(Str255		name)
{
	short	result = 0;
	
	
	(OSErr)HOpen(gFTPClientVRefNum, gFTPClientDirID, name, fsRdWrPerm, &result);
	return result;
}// ftpopen


/*	ftpgets - read a line from the keyboard 
	returns ABORT if aborted, non-zero on success */

char	ftpgets (char *s, short lim, short echo)
//char   *s;									/* where to put the line */
//short   lim, echo;							/* max chars to read, echo? */
{
	int					count = 0;
	register int		i = 0;
	unsigned char		c = '\0';
	char*				save = null;
	Boolean				gotOne = false;										   /* Did we get an event */
	EventRecord			myEvent;								   /* Event Record for this loop */
	ConnectionDataPtr	connectionDataPtr = null;
	UInt32				ticksWaitNextEvent = 0L;
	Size				actualSize = 0L;
	
	
	if (ScreenManager_IsWindowAScreen(GetRealFrontWindow()))
	{
		connectionDataPtr = ScreenManager_GetWindowConnectionData(GetRealFrontWindow());
	}
	
	count = 0;											   /* none read */
	save = s; /* beginning of line */
	
	// get the users process service preference, if possible
	unless (Preferences_GetData(kTelnetPreferenceClassGeneral,
								kTelnetPreferenceTag_TicksWaitNextEvent,
								sizeof(ticksWaitNextEvent), &ticksWaitNextEvent,
								&actualSize) == kTelnetPrefsResultCodeSuccess)
	{
		ticksWaitNextEvent = 4; // default ticks if preference cant be found
	}

	while (1) {
		gotOne = WaitNextEvent (everyEvent, &myEvent, ticksWaitNextEvent, 0L);

		if ((myEvent.what == keyDown) || (myEvent.what == autoKey))
			c = myEvent.message & charCodeMask;
		else {
			NetEvents_Service();								   /* process event queue */
			c = 0;
		}

		switch (c) {									   /* allow certain editing chars */
		case 8:										   /* backspace */
			if (count) {
				if (echo) {
					Parse_ShowOrInterpretAsCommand(connectionDataPtr, &c, 1);
					Parse_ShowOrInterpretAsCommand(connectionDataPtr, (unsigned char *) " ", 1);
					Parse_ShowOrInterpretAsCommand(connectionDataPtr, &c, 1);
				}
				count--;								   /* one less character */
				s--;									   /* move pointer backward */
			}
			break;
		case 13:										   /* carriage return, = ok */
			if (echo)
				putstring (connectionDataPtr, "\015\012");			   /* newline */
			*s = '\0';									   /* terminate the string */
			return (c);									   /* return ok */
			break;
		case 21:										   /* kill line */
			for (i = 0; i < s - save; i++) {			   /* length of line */
				if (echo) {								   /* erase */
					Parse_ShowOrInterpretAsCommand(connectionDataPtr, (unsigned char *) &backspace, 1);
					Parse_ShowOrInterpretAsCommand(connectionDataPtr, (unsigned char *) " ", 1);
					Parse_ShowOrInterpretAsCommand(connectionDataPtr, (unsigned char *) &backspace, 1);
				}
			}
			s = save;									   /* reset line */
			break;
		case 0:										   /* do nothing */
			break;
		default:										   /* not special char */
			if (c > 31 && c < 127) {					   /* printable */
				if (echo)
					Parse_ShowOrInterpretAsCommand(connectionDataPtr, &c, 1);		   /* display */
				*s++ = c;								   /* add to string */
				count++;								   /* length of string */
			}
			else										   /* acts as eol */
				return (c);								   /* value of special char */

			if (count == lim) {							   /* to length limit */
				*s = '\0';								   /* terminate */
				return (c);
			}
			break;
		}

	}
}

/************************************************************************/
/* captcon capture everything from a connection into the capture list return -1 on closed
   connection, else 0, 1 if paused 
\*********************************************************************** */

short captcon(short cnum, short mywindow)
{
	short				capturesize = 0,
						cnt = 0;
	unsigned			char s[80];								   /* temporary string */
	ConnectionDataPtr	connectionDataPtr = ScreenManager_GetConnectionDataOf(mywindow);
	
	
	// if (fromtty && n_scrlck()) return(true);	// if paused, nothing to do
	capturesize = connectionDataPtr->capturesize;
	do {
		cnt = Network_GetData(cnum, &s[0], 64);				   /* get some from queue */
		if ((cnt + capturesize) < captlistsize) {
			BlockMove (&s[0], &captlist[capturesize], (long) cnt);
			capturesize += cnt;
		}

		/* demux all packets */
	} while ((cnt > 0) && ((cnt + capturesize) < captlistsize));

	captlist[capturesize] = 0;
	connectionDataPtr->capturesize = capturesize;

	return (cnt);										   /* 0 normally, -1 if connection closed */
}

/************************************************************************/
/* telnet filter telnet options on incoming data
\*********************************************************************** */
short
telnet		(int				port,
			 int				cnt,
			 unsigned char*		s)
{
	short				result = 1;
	register int		i = 0,
						mylen = 0;
	char    			printline[256];		   // line to display
	ConnectionDataPtr	connectionDataPtr = null;
	
	
	connectionDataPtr = ScreenManager_GetConnectionDataFromPort(port);
	if (connectionDataPtr != null)
	{
		connectionDataPtr = ScreenManager_GetConnectionDataFromPort(port);
		for (i = 0; i < cnt; i++)
		{							   /* put on screen */
			if (*(s + i) & 128)
			{							   /* if over ASCII 128 */
				sprintf(printline, " %d ", *(s + i));		   /* show as number */
				mylen = strlen(printline);
				Parse_ShowOrInterpretAsCommand(connectionDataPtr, (unsigned char*)&printline[0], mylen);
			}
			else Parse_ShowOrInterpretAsCommand(connectionDataPtr, (s + i), 1);
		}
	}
	
	return result;
}// telnet


/************************************************************************/
/* 	dumpcon take everything from a connection and send it to the screen
	return -1 on closed connection, else 0, 1 if paused
\*********************************************************************** */

int dumpcon (int cnum, int mywindow)
{
	short   i1, i2, cnt = 0, result = 0;
	unsigned char s1[64], s2[128];						   /* temporary strings */

	// if (fromtty && n_scrlck()) return(true);		// if paused, nothing to do

	do {
		cnt = Network_GetData(cnum, &s1[0], 64);				   /* get some from queue */

		/* Search the incoming data for status codes and newlines */
		i1 = 0;
		i2 = 0;
		while (i1 < cnt) {

			/* Check for 3 digit codes beginning with "5" at the beginning of each line. */
			if (s1[i1] == '\012' || s1[i1] == '\015') {
				ScreenManager_SetResponseCountOf(mywindow, 0);
			}
			else if (ScreenManager_GetResponseCountOf(mywindow) < 4) {
				int				oldCount = ScreenManager_GetResponseCountOf(mywindow);
				static char		response_data[20][4];
				
				
				response_data[mywindow][oldCount] = s1[i1];
				ScreenManager_SetResponseCountOf(mywindow, oldCount + 1);
				if (ScreenManager_GetResponseCountOf(mywindow) == 4) {
					if (response_data[mywindow][0] == '5' &&
						response_data[mywindow][1] >= '0' &&
						response_data[mywindow][1] <= '9' &&
						response_data[mywindow][2] == '0' &&
						response_data[mywindow][2] <= '9' &&
						response_data[mywindow][3] == ' ')
						result = -1;					   /* abort indicated by other host */
				}
			}

			/* If a "newline" is found then force a "carriage return" immediately. */
			if ('\012' == (s2[i2++] = s1[i1++]))
				s2[i2++] = '\015';
		}

		telnet (cnum, i2, &s2[0]);						   /* display on screen, etc. */

		/* demux all packets */
	} while (cnt > 0);

	return (result);									   /* 0 normally, -1 if connection closed */
}

/************************************************************************/
/* 	getword: remove a word from a string.  Things within quotes are assumed
	to be one word. return true on success, false on end of string
\*********************************************************************** */

short getword (char *string, char *word)
{
	char   *p, *q;
	int     i = 0;

	p = stpblkp (string);								   /* skip leading blanks */
	if (!(*p)) {										   /* no words in string */
		word[0] = '\0';
		return (false);
	}
	if (*p == '!') {									   /* ! is a word */
		word[0] = *p;
		word[1] = '\0';
		strcpy (string, ++p);
		return (true);
	}
	if (*p == '\"') {									   /* word delimited by quotes */
		while (p[++i] && p[i] != '\"')
			word[i - 1] = p[i];
		word[i - 1] = '\0';
		if (!p[i])
			ftpclient_debug_print ("Missing \". Assumed at end of string.");
		else
			i++;
		q = p + i;
	}
	else
		q = stptok (p, word, 50, " \t\015\012");		   /* get word, max len 50 */
	p = stpblkp (q);									   /* remove trailing blanks */
	strcpy (string, p);									   /* remove extracted stuff */
	return (true);
}

/************************************************************************/
/* lowercase: convert a string to lowercase
\*********************************************************************** */

short lowercase(char *word)
{
	short     i;

	for (i = 0; (word[i] = tolower (word[i])); i++) {}
	return (true);
}

/************************************************************************/
/*	finduniq: find name that is a unique prefix of one of the entries in a list.
	Return position of the entry, NONE if none, AMBIGUOUS if more than one.
\*********************************************************************** */

int     finduniq (unsigned char *name, int listsize)
{
	int     i, j = NONE, len;
	unsigned char *p, cmdstring[256];

	len = strlen ((char *) name);
	for (i = 0; i < listsize; i++) {
		GetIndString (cmdstring, rStringsFTPCommands, i + 1);
		p = (unsigned char *) StringUtilities_PToCInPlace(cmdstring);
		if (!strncmp ((char *) name, (char *) cmdstring, len)) {	/* prefix */
			if (len == strlen ((char *) cmdstring))
				return (i + 1);							   /* exact match */
			if (j != NONE)
				j = AMBIGUOUS;							   /* more than one match */
			else
				j = i + 1;								   /* note prefix found */
		}
	}
	return (j);											   /* prefix */
}

/************************************************************************/
/* 	checkoredir: check for output redirection.  If the command contains a >,
	assume a filename follows and extract it.  Remove the redirection from
	the original command. Also change \ to / return true if redirection
	specified, false otherwise
\*********************************************************************** */
Boolean checkoredir(char *command, char *filename, short slashflip)
{
	short	i;

	filename[0] = '\0';
	for (i = 0; (command[i] != '>'); i++) {				   /* process command part */
		if (slashflip && command[i] == '\\')
			command[i] = '/';
		if (!command[i])
			return (false);								   /* no redirection */
	}
	getword (&command[i + 1], filename);				   /* get redirected filename */
	command[i] = '\0';
	return (true);
}

/************************************************************************/
/* getnname: get next name from captured list names delimited by newlines - <CR> or <LF>
\*********************************************************************** */
Boolean getnname(char *string, char *word)
{
	char   *s;

	s = string;
	while ((*string == '\012') || (*string == '\015'))
		string++;										   /* skip initial newlines */
	if (!(*string))
		return (false);									   /* end of captlist */
	while ((*string != '\012') && (*string != '\015') && (*string))
		*(word++) = *(string++);
	while ((*string == '\012') || (*string == '\015'))
		string++;										   /* skip trailing newline */
	*word = '\0';
	strcpy (s, string);
	return (true);
}

/********************************************************************/
/* 	FTP PI Protocol interpreter for user interface commands Will permit any command to
	be abbreviated uniquely. Recognizes commands, translates them to the protocol
	commands to be sent to the other server, and uses userftpd, the daemon, to do data
	transfers.
\*********************************************************************** */
short ftppi (char *command)
{
	Boolean				needanswer = false;
	int     			cmdno = 0,
						i = 0,
						cmd_status = 0;
	char    			cmdname[20],
						word[PATHLEN],
						line[MAXFTP],
						answer[20],
						ofilename[PATHLEN];
	char*				p = null;
	unsigned char		destname[50];							   /* who to connect to */
	char    			printline[256];								   /* line to display */
	ConnectionDataPtr	connectionDataPtr = null;
	
	
	if (ScreenManager_IsWindowAScreen(GetRealFrontWindow()))
	{
		connectionDataPtr = ScreenManager_GetWindowConnectionData(GetRealFrontWindow());
	}
	
	if (captlist == 0) {
		captlist = (unsigned char *) NewPtrClear ((long) captlistsize);
		// newfile = (unsigned char *) NewPtrClear( (long) PATHLEN);		// Pointer, no allocation necessary
		transfile = (unsigned char *) NewPtrClear ((long) PATHLEN);
		xs = (unsigned char *) NewPtrClear ((long) (BUFFERS + 10));
	}

	// Should check for not enough memory here!
	
	/* Indicates waiting for response */
	ScreenManager_SetResponseCountOf(connectionDataPtr->vs, 0);

	/* get command number */
	if (!getword (command, cmdname))
		return (false);									   /* get command name */

	/* removes first word from command */
	lowercase (cmdname);
	cmdno = finduniq ((unsigned char *) cmdname, rStringCountFTPCommands);   /* search cmdlist for prefix */
	if (cmdno == AMBIGUOUS) {							   /* not unique abbreviation */
		putFTPClientMessage(connectionDataPtr, siFTPCommandAmbiguous);
		return (false);
	}
	if (cmdno == NONE) {								   /* not a prefix of any command */
		putFTPClientMessage(connectionDataPtr, siFTPCommandInvalid);
		return (false);
	}

	/* change \ to / and check if command output redirected */
	if (cmdno != BANG) {								   /* don't alter shell escape */
		if (cmdno != LLS)								   /* do not flip slashes for LLS */
			checkoredir(command, ofilename, slashflip);   /* check redirection, flip \ */
		else
			checkoredir(command, ofilename, false);	   /* check redirection */
	}

	/* process commands */
	switch (cmdno) {
	case QMARK:
	case HELP:
		if (!command[0]) {								   /* no argument */
			putFTPClientMessage(connectionDataPtr, siFTPCommandsMayBeAbbreviated);

			/* display command list */
			printline[0] = '\0';
			for (i = 2; i < rStringCountFTPCommands; i++) {
				GetIndString ((StringPtr)word, rStringsFTPCommands, i + 1);
				p = StringUtilities_PToCInPlace((StringPtr)word);
				sprintf (word, "%-16s", word);			   /* get word from list */
				strcat (printline, word);				   /* add to line */
				if ((i - 2) % 5 == 4) {					   /* display line */
					printline[79] = '\0';
					putstring (connectionDataPtr, printline);
					putstring (connectionDataPtr, "\015\012");
					printline[0] = '\0';
				}
			}
			if (i % 5 != 4) {
				putstring (connectionDataPtr, printline);			   /* last line */
				putstring (connectionDataPtr, "\015\012");
			}
			return (true);
		}

		/* help for specific commands */
		else {
			while (getword (command, word)) {			   /* loop for all args */
				i = finduniq ((unsigned char *) word, rStringCountFTPCommands);	/* which command? */
				if (i == AMBIGUOUS)						   /* non-unique command name */
					sprintf (printline, "?Ambiguous help command %s", word);
				else if (i == NONE)						   /* no such command */
					sprintf (printline, "?Invalid help command %s", word);
				else {									   /* display help string */
					GetIndString ((StringPtr)printline, HELPRESID, i);
					p = StringUtilities_PToCInPlace((StringPtr)printline);
				}
				putstring (connectionDataPtr, printline);
				putstring (connectionDataPtr, "\015\012");
			}
			return (true);
		}
		break;
	case BANG:											   /* shell escape */
		return (true);
	case BELL:
		if (getword (command, word)) {					   /* scan arg */
			lowercase (word);
			if (!strcmp (word, "off"))
				bell = false;
			else if (!strcmp (word, "on"))
				bell = true;
			else
				bell = !bell;
		}
		else
			bell = !bell;
		if (bell)
			ftpclient_debug_print ("Bell mode on.");
		else
			ftpclient_debug_print ("Bell mode off.");
		return (true);
	case BYE:
	case QUIT:
		cmd_status = ftpdo (connectionDataPtr, "QUIT", ofilename);
		Connections_Disconnect(connectionDataPtr);
		return (true);
	case DEBUG:										   /* turn on/off debugging, optional level */
		return (true);
	case GLOB:											   /* wildcard expansion */
		if (getword (command, word)) {
			lowercase (word);
			if (!strcmp (word, "off"))
				glob = false;
			else if (!strcmp (word, "on"))
				glob = true;
			else
				glob = !glob;
		}
		else
			glob = !glob;
		if (glob)
			ftpclient_debug_print ("Globbing on.");
		else
			ftpclient_debug_print ("Globbing off.");
		return (true);
	case HASH:											   /* hash mark printing */
		if (getword (command, word)) {
			lowercase (word);
			if (!strcmp (word, "off"))
				hash = false;
			else if (!strcmp (word, "on"))
				hash = true;
			else
				hash = !hash;
		}
		else
			hash = !hash;
		if (hash)
			ftpclient_debug_print ("Hash mark printing on (1024 bytes/hash mark).");
		else
			ftpclient_debug_print ("Hash printing off.");
		return (true);
	case INTERACTIVE:									   /* prompting on multiple transfers */
		prompt = true;
		putFTPClientMessage(connectionDataPtr, siFTPInteractiveModeOn);
		return (true);
	case LCD:											   /* change local directory */
		if (*(stpblkp (command))) {
			getword (command, word);
			if (FileUtilities_ChangeDirectory(&(gFTPClientDirID), &(gFTPClientVRefNum), word))
				putFTPClientMessage(connectionDataPtr, siFTPUnableToChangeDirectory);
		}
		FileUtilities_GetPathnameFromDirectoryID(gFTPClientDirID, gFTPClientVRefNum, (StringPtr) p);
		p = StringUtilities_PToCInPlace((StringPtr)p);
		sprintf (printline, "Local directory now %s\015\012", p);
		putstring (connectionDataPtr, printline);
		return (true);
	case LDIR:											   /* local DIR */
	case LLS:
		if (!(*(stpblkp (command)))) {					   /* null response ok */
			FileUtilities_GetPathnameFromDirectoryID(gFTPClientDirID, gFTPClientVRefNum, (StringPtr) p);
			p = StringUtilities_PToCInPlace((StringPtr)p);
			sprintf (line, "*");
		}
		else {
			p = command;
			sprintf (line, "%s", command);
		}
		putstring (connectionDataPtr, "Directory ");
		putstring (connectionDataPtr, p);
		putstring (connectionDataPtr, "\015\012");
		strcpy(Template, &line[0]);
		if (null == (newfile = (unsigned char *) firstname (Template, gFTPClientDirID, gFTPClientVRefNum, &ClientDirectoryState))) {
			putFTPClientMessage(connectionDataPtr, siFTPNoFilesFound);
		}
		else {
			putstring (connectionDataPtr, (char *) newfile);
			putstring (connectionDataPtr, "\015\012");
			while (null != (newfile = (unsigned char *) nextname (Template, &ClientDirectoryState))) {
				putstring (connectionDataPtr, (char *) newfile);
				putstring (connectionDataPtr, "\015\012");
			}
		}
		return (true);
	case NONINTERACTIVE:								   /* turn off interactive prompting */
		prompt = false;
		ftpclient_debug_print ("Interactive mode off.");
		return (true);
	case OPEN:											   /* open connection to host */
		NewSessionsDialog_Init();
		NewSessionsDialog_Display();
		NewSessionsDialog_Done();
		return (true);
	case PROMPT:										   /* interactive prompting */
		if (getword (command, word)) {
			lowercase (word);
			if (!strcmp (word, "off"))
				prompt = false;
			else if (!strcmp (word, "on"))
				prompt = true;
			else
				prompt = !prompt;
		}
		else
			prompt = !prompt;
		if (prompt)
			ftpclient_debug_print ("Interactive mode on.");
		else
			ftpclient_debug_print ("Interactive mode off.");
		return (true);
	case SENDPORT:										   /* send PORT commands for each transfer */
		if (getword (command, word)) {
			lowercase (word);
			if (!strcmp (word, "off"))
				sendport = false;
			else if (!strcmp (word, "on"))
				sendport = true;
			else
				sendport = !sendport;
		}
		else
			sendport = !sendport;
		if (sendport)
			ftpclient_debug_print ("Use of PORT cmds on.");
		else
			ftpclient_debug_print ("Use of PORT cmds off.");
		return (true);
	case SLASHFLIP:									   /* change \ to / */
		if (getword (command, word)) {
			lowercase (word);
			if (!strcmp (word, "off"))
				slashflip = false;
			else if (!strcmp (word, "on"))
				slashflip = true;
			else
				slashflip = !slashflip;
		}
		else
			slashflip = !slashflip;
		if (slashflip)
			ftpclient_debug_print ("Slash translation on.");
		else
			ftpclient_debug_print ("Slash translation off.");
		return (true);
	case STATUS:										   /* display status info */
		sprintf (printline, "Connected to %s", destname);
		ftpclient_debug_print (printline);
		if (ftpfilemode == FASCII)
			ftpclient_debug_print ("Transfer mode is ascii.");
		else
			ftpclient_debug_print ("Transfer mode is binary.");
		if (bell)
			ftpclient_debug_print ("Bell on.");
		else
			ftpclient_debug_print ("Bell off.");
		//if (TelInfo->debug) {
		//	ftpclient_debug_print ("Debug on");
		//}
		//else
		//	ftpclient_debug_print ("Debug off");
		if (glob)
			ftpclient_debug_print ("Filename globbing on.");
		else
			ftpclient_debug_print ("Filename globbing off.");
		if (hash)
			ftpclient_debug_print ("Hash-mark printing on.");
		else
			ftpclient_debug_print ("Hash-mark printing off.");
		if (prompt)
			ftpclient_debug_print ("Interactive prompting on.");
		else
			ftpclient_debug_print ("Interactive prompting off.");
		if (sendport)
			ftpclient_debug_print ("Sending of port commands on.");
		else
			ftpclient_debug_print ("Sending of PORT cmds off.");
		if (slashflip)
			ftpclient_debug_print ("Flipping \\ to / on.");
		else
			ftpclient_debug_print ("Flipping \\ to / off.");
		if (verbose)
			ftpclient_debug_print ("Verbose mode on.");
		else
			ftpclient_debug_print ("Verbose mode off.");

		ftpclient_debug_print ("\012Remote status:");
		cmd_status = ftpdo (connectionDataPtr, "STAT", ofilename);

		return (true);
	case VERBOSE:										   /* display informative messages */
		if (getword (command, word)) {
			lowercase (word);
			if (!strcmp (word, "off"))
				verbose = false;
			else if (!strcmp (word, "on"))
				verbose = true;
			else
				verbose = !verbose;
		}
		else
			verbose = !verbose;
		if (verbose)
			ftpclient_debug_print ("Verbose mode on.");
		else
			ftpclient_debug_print ("Verbose mode off.");
		return (true);
	default:											   /* The other commands valid only if
														      connected (we always are here) */
		switch (cmdno) {
		case ACCT:										   /* get file in binary mode */
			while (!(*(stpblkp (command)))) {
				putFTPClientPrompt(connectionDataPtr, siFTPEnterAccount);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);
			}
			sprintf (line, "ACCT %s", command);
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);	   /* get file */
			return (true);
		case ASCII:									   /* transfer mode */
			cmd_status = ftpdo (connectionDataPtr, "TYPE A", ofilename);
			return (true);
		case MACB:										   /* transfer mode */
			if (FlagManager_Test(kFlagMacBinaryIIEnabled)) {
				FlagManager_Set(kFlagMacBinaryIIEnabled, false);
				cmd_status = ftpdo (connectionDataPtr, "MACB D", ofilename);
			}
			else {
				FlagManager_Set(kFlagMacBinaryIIEnabled, true);
				cmd_status = ftpdo (connectionDataPtr, "MACB E", ofilename);
			}
			return (true);
		case BGET:										   /* get file in binary mode */
			i = ftpfilemode;							   /* save current mode */
			if (i == FASCII)
				cmd_status = ftpdo (connectionDataPtr, "TYPE I", ofilename);
			while (!(*(stpblkp (command)))) {
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNameGetBinaryFile);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);
			}
			sprintf (line, "RETR %s", command);
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);	   /* get file */
			if (i == FASCII)
				cmd_status = ftpdo (connectionDataPtr, "TYPE A", ofilename);	/* restore mode */
			return (true);
		case BINARY:									   /* binary mode */
			cmd_status = ftpdo (connectionDataPtr, "TYPE I", ofilename);
			return (true);
		case BPUT:										   /* put file in binary mode */
			i = ftpfilemode;
			if (i == FASCII)
				cmd_status = ftpdo (connectionDataPtr, "TYPE I", ofilename);
			while (!(*(stpblkp (command)))) {			   /* if no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNamePutBinaryFile); /* get from user */
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);
			}
			sprintf (line, "STOR %s", command);
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			if (i == FASCII)
				cmd_status = ftpdo (connectionDataPtr, "TYPE A", ofilename);
			return (true);
		case CD:										   /* change remote directory */
			while (!(*(stpblkp (command)))) {			   /* if no arg, get from user */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterDirectoryNameToChangeTo);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);						   /* abort */
			}
			getword (command, word);
			if (!strcmp (word, "..")) {					   /* special case */
				cmd_status = ftpdo (connectionDataPtr, "CDUP", ofilename);
				if (cmd_status != ERROR)
					return (true);						   /* if CDUP understood */
#if 0
				ftpclient_debug_print ("Trying again...");
				cmd_status = ftpdo (connectionDataPtr, "XCUP", ofilename);	/* try alternative */
				if (cmd_status != ERROR)
					return (true);
				ftpclient_debug_print ("Trying again...");				   /* else try usual CD */
#endif
			}
			sprintf (line, "CWD %s", word);				   /* try CWD */
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			if (cmd_status != ERROR)
				return (true);
#if 0
			ftpclient_debug_print ("Trying again...");
			sprintf (line, "XCWD %s", word);			   /* try XCWD */
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
#endif
			return (true);
		case CLOSE:									   /* drop connection */
			cmd_status = ftpdo (connectionDataPtr, "QUIT", ofilename);
			Connections_Disconnect(connectionDataPtr);
			return (true);
		case DEL:
		case RM:
			getword (command, word);
			while (!word[0]) {							   /* get arg from user */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNameToDelete);
				if (ftpgets (word, 100, 1) == ABORT)
					return (false);						   /* abort */
			}
			if (prompt) {								   /* check interactively */
				sprintf (printline, "Delete %s? ", word);
				putstring (connectionDataPtr, printline);
				ftpgets (answer, 20, 1);
				if (tolower (*(stpblkp (answer))) != YES_CHAR)
					return (true);
			}
			sprintf (line, "DELE %s", word);
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			return (true);
		case DIR:										   /* get list of remote files */
			if (ftpfilemode == FIMAGE) {
				cmd_status = ftpdo (connectionDataPtr, "TYPE A", ofilename);
				if (getword (command, word)) {
					sprintf (line, "LIST %s", word);
				}
				else
					sprintf (line, "LIST");
				strcpy (connectionDataPtr->ftpbuf, line);
				connectionDataPtr->clientflags = TYPE_I_ON_CLOSE;
				connectionDataPtr->ftpState = 3;				   /* Do "LIST" when "TYPE" is finished */
				connectionDataPtr->ftpnext = 1;
			}
			else {
				if (getword (command, word)) {			   /* Do DIR */
					sprintf (line, "LIST %s", word);
					cmd_status = ftpdo (connectionDataPtr, line, ofilename);
				}
				else
					cmd_status = ftpdo (connectionDataPtr, "LIST", ofilename);
			}
			return (true);
		case GET:
		case RECV:										   /* get remote file */
			while (!(*(stpblkp (command)))) {			   /* if no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNameGetFile);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);						   /* abort */
			}
			sprintf (line, "RETR %s", command);
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			return (true);
		case LS:										   /* get remote file list - short */
			if (ftpfilemode == FIMAGE) {
				cmd_status = ftpdo (connectionDataPtr, "TYPE A", ofilename);
				if (getword (command, word)) {
					sprintf (line, "NLST %s", word);
				}
				else
					sprintf (line, "NLST");
				strcpy (connectionDataPtr->ftpbuf, line);
				connectionDataPtr->clientflags = TYPE_I_ON_CLOSE;
				connectionDataPtr->ftpState = 3;				   /* Do "NLST" when "TYPE" is finished */
				connectionDataPtr->ftpnext = 1;
			}
			else {
				if (getword (command, word)) {
					sprintf (line, "NLST %s", word);
					cmd_status = ftpdo (connectionDataPtr, line, ofilename);
				}
				else
					cmd_status = ftpdo (connectionDataPtr, "NLST", ofilename);
			}
			return (true);
		case MDELETE:
			while (!(*(stpblkp (command)))) {			   /* no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNamesToDelete);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);						   /* abort */
			}
			while (getword (command, word)) {			   /* for each arg */
				if (glob) {								   /* wildcard expansion */
					sprintf (line, "NLST %s", word);
					connectionDataPtr->clientflags |= CAPTURE_DATA;
					cmd_status = ftpdo (connectionDataPtr, line, ofilename);	/* put exapnsion in captlist */
				}
				else
					strcpy ((char *) captlist, word);	   /* captlist has name(s) now */
				while (getnname ((char *) captlist, word)) {	/* for each name */
					if (prompt) {						   /* check */
						sprintf (printline, "mdelete %s? ", word);
						putstring (connectionDataPtr, printline);
						if (ftpgets (answer, 20, 1) == ABORT) {	/* abort */
							command[0] = '\0';			   /* no more processing */
							break;						   /* quit immediately */
						}
						if (tolower (*(stpblkp (answer))) != YES_CHAR)
							continue;
					}
					sprintf (line, "DELE %s", word);	   /* delete */
					cmd_status = ftpdo (connectionDataPtr, line, ofilename);
				}
			}
			return (true);
		case MDIR:										   /* remote multiple DIR */
			i = ftpfilemode;							   /* save mode */
			if (i == FIMAGE)
				cmd_status = ftpdo (connectionDataPtr, "TYPE A", ofilename);
			while (!(*(stpblkp (command)))) {			   /* no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterDirectoryNamesToList);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);						   /* abort */
			}
			while (getword (command, word)) {			   /* for each arg */
				if (glob) {								   /* expand wildcards */
					sprintf (line, "NLST %s", word);
					connectionDataPtr->clientflags |= CAPTURE_DATA;
					cmd_status = ftpdo (connectionDataPtr, line, ofilename);
				}
				else
					strcpy ((char *) captlist, word);
				while (getnname ((char *) captlist, word)) {	/* for each name */
					if (prompt) {						   /* check */
						sprintf (printline, "mdir %s? ", word);
						putstring (connectionDataPtr, printline);
						if (ftpgets (answer, 20, 1) == ABORT) {	/* abort */
							command[0] = '\0';			   /* no more processing */
							break;						   /* quit immediately */
						}
						if (tolower (*(stpblkp (answer))) != YES_CHAR)
							continue;
					}
					sprintf (line, "LIST %s", word);	   /* DIR */
					cmd_status = ftpdo (connectionDataPtr, line, ofilename);
				}
			}
			if (i == FIMAGE)
				cmd_status = ftpdo (connectionDataPtr, "TYPE I", ofilename);
			return (true);
		case MGET:										   /* get multiple files */
			getword (command, line);
			while (!line[0]) {							   /* no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNamesGet);
				if (ftpgets (line, 100, 1) == ABORT)
					return (false);						   /* abort */
			}
			if (getword (line, word)) {
				sprintf (command, "NLST %s", word);
				connectionDataPtr->clientflags |= CAPTURE_DATA;
				cmd_status = ftpdo (connectionDataPtr, command, ofilename);

				connectionDataPtr->clientflags |= MGET_STATE;
			}
			return (true);
		case MKDIR:									   /* create directory */
			while (!(*(stpblkp (command)))) {			   /* no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterDirectoryNameForNewDirectory);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);						   /* abort */
			}
			sprintf (line, "XMKD %s", command);			   /* try XMKD */
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			if (cmd_status != ERROR)
				return (true);
			ftpclient_debug_print ("Trying again...");
			sprintf (line, "MKD %s", command);			   /* else try MKD */
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			return (true);
		case MLS:
			i = ftpfilemode;
			if (i == FIMAGE)
				cmd_status = ftpdo (connectionDataPtr, "TYPE A", ofilename);
			while (!(*(stpblkp (command)))) {			   /* no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterDirectoryNamesToList);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);						   /* abort */
			}
			while (getword (command, word)) {			   /* for each arg */
				if (glob) {								   /* exapnd wildcards */
					sprintf (line, "NLST %s", word);
					connectionDataPtr->clientflags |= CAPTURE_DATA;
					cmd_status = ftpdo (connectionDataPtr, line, ofilename);
				}
				else
					strcpy ((char *) captlist, word);
				while (getnname ((char *) captlist, word)) {	/* for each name */
					if (prompt) {						   /* check */
						sprintf (printline, "mls %s? ", word);
						putstring (connectionDataPtr, printline);
						if (ftpgets (answer, 20, 1) == ABORT) {	/* abort */
							command[0] = '\0';			   /* no more processing */
							break;						   /* quit immediately */
						}
						if (tolower (*(stpblkp (answer))) != YES_CHAR)
							continue;
					}
					sprintf (line, "NLST %s", word);	   /* DIR */
					cmd_status = ftpdo (connectionDataPtr, line, ofilename);
				}
			}
			if (i == FIMAGE)
				cmd_status = ftpdo (connectionDataPtr, "TYPE I", ofilename);
			return (true);
		case MODE:										   /* set stream mode */
			getword (command, word);
			lowercase (word);
			if (strncmp (word, "stream", strlen (word)))
				ftpclient_debug_print ("We only support stream mode, sorry.");
			else
				ftpclient_debug_print ("Mode is stream.");
			return (true);
		case MPUT:										   /* put multiple files */
			getword (command, (char *) captlist);
			while (!captlist[0]) {						   /* no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNamesPut);
				if (ftpgets ((char *) captlist, 100, 1) == ABORT)
					return (false);						   /* abort */
			}
			if (getword ((char *) captlist, (char *) transfile)) {	/* for each arg */
				newfile = null;
				needanswer = true;
				while (needanswer) {
					needanswer = false;
					if (glob) {							   /* local wildcard expansion */
						if (newfile == null) {			   /* if no expansions yet */
							strcpy(Template, (char *)transfile);
							newfile = (unsigned char *) firstname (Template, gFTPClientDirID, gFTPClientVRefNum, &ClientDirectoryState);	/* get first name */
							if (newfile == null) {		   /* if no expansions */
								sprintf (printline, "No match for %s", transfile);
								ftpclient_debug_print (printline);
								if (getword ((char *) captlist, (char *) transfile)) {
									needanswer = true;
								}
							}
						}
						else {							   /* not first name */
							newfile = (unsigned char *) nextname (Template, &ClientDirectoryState);	/* get next name */
							if (newfile == null) {		   /* if no names, next arg */
								if (getword ((char *) captlist, (char *) transfile)) {
									needanswer = true;
								}
							}
						}
					}
					else
						newfile = transfile;			   /* no expansion */
					if (newfile != null) {
#if 1													   /* When prompting is fixed switch to the
														      code below */
						sprintf (command, "STOR \"%s\"", newfile);	/* name may have spl chars */
						cmd_status = ftpdo (connectionDataPtr, command, ofilename);
						connectionDataPtr->clientflags |= MPUT_STATE;
#else
						if (prompt) {					   /* check */
							sprintf (printline, "mput %s? ", newfile);
							putstring (connectionDataPtr, printline);
							if (ftpgets (answer, 20, 1) == ABORT) {	/* abort */
								command[0] = '\0';		   /* no more processing */
								break;
							}
							else if (tolower (*(stpblkp (answer))) == YES_CHAR) {
								sprintf (command, "STOR \"%s\"", newfile);	/* name may have spl
																			   chars */
								cmd_status = ftpdo (connectionDataPtr, command, ofilename);
								connectionDataPtr->clientflags |= MPUT_STATE;
							}
						}
						else {
							sprintf (command, "STOR \"%s\"", newfile);	/* name may have spl chars */
							cmd_status = ftpdo (connectionDataPtr, command, ofilename);
							connectionDataPtr->clientflags |= MPUT_STATE;
						}
#endif
					}
				}
			}
			return (true);
		case PUT:
		case SEND:										   /* put file */
			while (!(*(stpblkp (command)))) {			   /* no args */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNamePutFile);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);
			}
			sprintf (line, "STOR %s", command);			   /* put file */
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			return (true);
		case PWD:
#if 0
			cmd_status = ftpdo (connectionDataPtr, "XPWD", ofilename);  /* try XPWD */
			if (i != ERROR)
				return (true);
			ftpclient_debug_print ("Trying again...");
#endif
			cmd_status = ftpdo (connectionDataPtr, "PWD", ofilename);   /* else try PWD */
			return (true);
		case QUOTE:
			while (!(*(stpblkp (command)))) {			   /* no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterCommandNameForQuote);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);
			}
			cmd_status = ftpdo (connectionDataPtr, command, ofilename); /* send command */
			return (true);
		case REMOTEHELP:								   /* get help */
			if (*(stpblkp (command))) {					   /* for specific command */
				sprintf (line, "HELP %s", command);
				cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			}
			else
				cmd_status = ftpdo (connectionDataPtr, "HELP", ofilename);	/* generic help */
			return (true);
		case RENAME:									   /* rename remote file */
			while (!(*(stpblkp (command)))) {			   /* no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNameOldNameRename);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);
			}
			getword (command, word);
			sprintf (line, "RNFR %s", word);
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);	   /* send rename from name */
			while (!(*(stpblkp (command)))) {			   /* no second arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterFileNameNewNameRename);
				if (ftpgets (command, 100, 1) == ABORT) {
					cmd_status = ftpdo (connectionDataPtr, "ABOR", ofilename);
					return (false);
				}
			}
			sprintf (line, "RNTO %s", command);			   /* send rename to name */
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			return (true);
		case RMDIR:									   /* remove remote dir */
			while (!(*(stpblkp (command)))) {			   /* no arg */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterDirectoryNameToDelete);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);
			}
			sprintf (line, "XRMD %s", command);			   /* try XRMD */
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			if (cmd_status != ERROR)
				return (true);
			ftpclient_debug_print ("Trying again...");
			sprintf (line, "RMD %s", command);			   /* try RMD */
			cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			return (true);
		case STRUCT:									   /* set structure type - only file */
			getword (command, word);
			lowercase (word);
			if (strncmp (word, "file", strlen (word)))
				ftpclient_debug_print ("We only support file structure, sorry.");
			else
				ftpclient_debug_print ("Structure is file.");
			return (true);
		case TYPE:										   /* set transfer type */
			if (!getword (command, word)) {				   /* no arg, just show */
				if (ftpfilemode == FASCII)
					ftpclient_debug_print ("Transfer type is ascii.");
				else
					ftpclient_debug_print ("Transfer type is binary.");
			}
			lowercase (word);
			if (!strncmp (word, "ascii", strlen (word)))
				cmd_status = ftpdo (connectionDataPtr, "TYPE A", ofilename);
			else if (!strncmp (word, "binary", strlen (word)) ||
					 !strncmp (word, "image", strlen (word)))
				cmd_status = ftpdo (connectionDataPtr, "TYPE I", ofilename);
			else {
				sprintf (printline, "Unrecognized type: %s", word);
				ftpclient_debug_print (printline);
			}
			return (true);
		case USER:										   /* login to remote machine */
			if (!(*(stpblkp (command)))) {				   /* null response to prompt ok */
				putFTPClientPrompt(connectionDataPtr, siFTPEnterUserName);
				if (ftpgets (command, 100, 1) == ABORT)
					return (false);
			}
			sprintf (line, "USER %s", command);			   /* username */
			if ((cmd_status = ftpdo (connectionDataPtr, line, ofilename)) == true)
				return (true);
#if 0
			putFTPClientPrompt(connectionDataPtr, siFTPEnterPassword);
#endif
			if (ftpgets (word, PATHLEN, 0) == ABORT)
				return (false);							   /* no echoing */
			sprintf (line, "PASS %s", word);			   /* password */
			if ((cmd_status = ftpdo (connectionDataPtr, line, ofilename)) == INCOMPLETE) {	/* if account needed */
				do {
					putFTPClientPrompt(connectionDataPtr, siFTPEnterAccount);
					if (ftpgets (command, 100, 1) == ABORT)
						return (false);
				} while (!(*(stpblkp (command))));
				sprintf (line, "ACCT %s", command);
				cmd_status = ftpdo (connectionDataPtr, line, ofilename);
			}
			return (true);
		default:										   /* unknown command */
			putFTPClientMessage(connectionDataPtr, siFTPCommandInvalid);
#if 0
			sprintf (printline, "***Unknown command no: %d", cmdno);
			ftpclient_debug_print (printline);
#endif
			break;
		}
	}

	return false;
}

/************************************************************************/
/* 	ftpport return a new port number so that we don't try to re-use ports before the
	mandatory TCP timeout period.  (lifetime of a packet) use a time-based initial port
	selection scheme.
\*********************************************************************** */

short ftpport(ConnectionDataPtr	theFTPScreen)
{
	unsigned int i;
	unsigned char hostnum[5];
	char    sendline[60];							/* for port command */

	if (!sendport)									/* default port */
		return (HFTP - 1);
		
	if (curftpprt < 40000) {						/* restart cycle */
		i = (unsigned int) TickCount ();
		curftpprt = 40000 + (i & 0x3fff);
	}

	i = curftpprt--;								/* get port, update for next time */

	Network_GetFourCharIPAddress(hostnum);								/* get my ip number */

	ftpdata = Network_ListenToTCPPortAndGetItsPortNumber(i);
	theFTPScreen->ftpport = ftpdata;
	Network_SetPortType(ftpdata, kNetworkPortTypeFTPClientSession);

	sprintf (sendline, "PORT %d,%d,%d,%d,%d,%d\015\012", hostnum[0], hostnum[1], hostnum[2],
			 hostnum[3], i / 256, i & 255);				   /* full port number */

	Network_SendFlushData(theFTPScreen->port, sendline, strlen (sendline));	/* send PORT command */

	return (i);										/* port number */
}

/************************************************************************/
/* 	ftpdo Do whatever command is sent from the user interface using userftpd,
	the background file handler
\*********************************************************************** */
int     ftpdo (ConnectionDataPtr theFTPScreen, char *s, char *ofile)
{
	int     			i = 0;
	char    			name[MAXFTP],
						name2[MAXFTP];
	char    			commandbuf[MAXFTP];
	HFileParam			iop;
	
	
	strcpy (commandbuf, s);
	for (i = 0; i < 4; i++) {
		commandbuf[i] = toupper (commandbuf[i]);		   /* command to upper case */
	}

	if (!strncmp (commandbuf, "STOR", 4)) {				   /* put file */
		getword (&commandbuf[5], name);					   /* first arg - local file */
		if (!commandbuf[5])
			strcpy (&commandbuf[5], name);				   /* if only one argument */
		else {
			getword (&commandbuf[5], name2);			   /* second arg - removes quotes etc. */
			strcpy (&commandbuf[5], name2);				   /* copy back into command */
		}

		ftp_mbfp = MBopen (&name[0], gFTPClientVRefNum, gFTPClientDirID, 
				MB_READ +
				( ((!FlagManager_Test(kFlagMacBinaryIIEnabled)) || (ftpfilemode == FAMODE)) ? MB_DISABLE : 0) +
				((ftpfilemode == FAMODE) ? MB_ISASCII : 0)
				);
		if (ftp_mbfp == 0L) {
			putFTPClientMessage(theFTPScreen, siFTPCouldNotOpenFile);
			return (-1);
		}
		ftpfh = 12;

		StringUtilities_CToPInPlace(&name[0]);
		GetFileInfo (gFTPClientVRefNum, gFTPClientDirID, (StringPtr)&name[0], &iop);
		filelen = iop.ioFlLgLen;
		if (FlagManager_Test(kFlagMacBinaryIIEnabled) && (ftpfilemode == FIMODE))
			filelen += iop.ioFlRLgLen;

		ftpport (theFTPScreen);								   /* open data connection */
		theFTPScreen->ftpState = 2;
		theFTPScreen->ftpnext = 20;
		strcpy (theFTPScreen->ftpbuf, commandbuf);
		strcat (theFTPScreen->ftpbuf, "\015\012");
		return (1);

	}
	else if (!strncmp (commandbuf, "RETR", 4)) {		   /* get file */
		getword (&commandbuf[5], name);					   /* remote file */
		if (commandbuf[5]) {							   /* two args present */
			getword (&commandbuf[5], name2);			   /* local file */

			if ((ftp_mbfp = MBopen (&name2[0], gFTPClientVRefNum, gFTPClientDirID, MB_WRITE +
							  (((!FlagManager_Test(kFlagMacBinaryIIEnabled)) || (ftpfilemode == FAMODE)) ? MB_DISABLE : 0) +
							  ((ftpfilemode == FAMODE) ? MB_ISASCII : 0)
						)) == 0L) {
				putFTPClientMessage(theFTPScreen, siFTPCannotOpenFileToReceive);
				return (-1);
			}
			else
				ftpfh = 12;
		}
		else {
			if ((ftp_mbfp = MBopen (&name[0], gFTPClientVRefNum, gFTPClientDirID, MB_WRITE +
							  (((!FlagManager_Test(kFlagMacBinaryIIEnabled)) || (ftpfilemode == FAMODE)) ? MB_DISABLE : 0)
							  + ((ftpfilemode == FAMODE) ? MB_ISASCII : 0)
					)) == 0L) {
				putFTPClientMessage(theFTPScreen, siFTPCannotOpenFileToReceive);
				return (-1);
			}
			else
				ftpfh = 12;
		}

		strcpy (&commandbuf[5], name);					   /* Put remote name back into command */

		ftpport (theFTPScreen);								   /* open data connection */
		theFTPScreen->ftpState = 2;
		theFTPScreen->ftpnext = 30;
		strcpy (theFTPScreen->ftpbuf, commandbuf);
		strcat (theFTPScreen->ftpbuf, "\015\012");
		return (1);
	}
	else if (!strncmp (commandbuf, "LIST", 4) || !strncmp (commandbuf, "NLST", 4)) {

		if ((theFTPScreen->clientflags & CAPTURE_DATA) != 0)
			theFTPScreen->capturesize = 0;			   /* Size of captured data */

		ftpport (theFTPScreen);								   /* data connection */
		theFTPScreen->ftpState = 2;
		theFTPScreen->ftpnext = 40;
		strcpy (theFTPScreen->ftpbuf, commandbuf);
		strcat (theFTPScreen->ftpbuf, "\015\012");
		return (1);

	}
	else if (!strncmp (commandbuf, "TYPE", 4)) {
		if (toupper (commandbuf[5]) == 'I')
			ftpfilemode = FIMAGE;						   /* Remember mode */
		else if (toupper (commandbuf[5]) == 'A')
			ftpfilemode = FASCII;

	}

	strcat (commandbuf, "\015\012");
	Network_SendFlushData(theFTPScreen->port, commandbuf, strlen (commandbuf));	/* send command */
	StringUtilities_CToPInPlace(ofile);
	
	if (((theFTPScreen->clientflags & CAPTURE_DATA) == 0) && ofile[0]) {	/* command redirected */
		if ((theFTPScreen->ftpState != 20) && (theFTPScreen->ftpState != 30)) {	/* not get or put */
			if (0 > (ftpfh = ftpopen ((StringPtr)ofile)))
				ftpclient_debug_print (" Cannot open output file.");
			else if (ftpdata > -1) {
				theFTPScreen->ftpState = 30;			   /* act as get, since data goes into file */
			}
			else {
				// Close(ftpfh);
				ftpfh = 0;
			}
		}
	}

	return (false);
}

/************************************************************************/
/* userftpd FTP receive and send file functions
\*********************************************************************** */

short userftpd (int code, int myport)
{
	Boolean 			needanswer = false;
	long    			mytime = 0L;
	int     			cmd_status = 0,
						connection_status = 0,
						ftpState = 0,
						i = 0,
						mycode = 0,
						myftpdata = 0,
						mytelport = 0,
						mywindow = 0;
	char    			command[MAXFTP],
						printline[MAXFTP],
						word[MAXFTP];
	ConnectionDataPtr	connectionDataPtr = null;
	
	
	mycode = code;
	connection_status = 0;								   /* assume good connection status */
	ftpState = 1;
	
	connectionDataPtr = ScreenManager_GetConnectionDataFromPort(myport);
	if (connectionDataPtr != null) {
		mywindow = connectionDataPtr->vs;
		if (myport == connectionDataPtr->port) {
			ftpState = connectionDataPtr->ftpState;
			myftpdata = -1;
		}
		else {
			ftpState = connectionDataPtr->ftpState;
			myftpdata = connectionDataPtr->ftpport;
		}
	}
	else return (0);

	mytelport = connectionDataPtr->port;

	switch (ftpState) {
	default:											   /* unknown */
		break;

	case 1:
		if ((mycode == kConnectionEventData) && (myport == mytelport))
			connection_status = dumpcon (myport, mywindow);
		break;

		/* Wait for response from "PORT" command, then send ftp command */
	case 2:
		if (mycode == kConnectionEventData) {
			connection_status = dumpcon (myport, mywindow);
			Network_SendFlushData(connectionDataPtr->port, connectionDataPtr->ftpbuf, strlen (connectionDataPtr->ftpbuf));	/* send command */
			connectionDataPtr->ftpState = connectionDataPtr->ftpnext;
		}
		break;

		/* Wait for response from "TYPE" command, then send ftp command */
	case 3:
		if (mycode == kConnectionEventData) {
			connection_status = dumpcon (myport, mywindow);
			cmd_status = ftpdo (connectionDataPtr, connectionDataPtr->ftpbuf, "");
		}
		break;

	case 5:
		/* Get the next file of an MGET. */
		if ((connectionDataPtr->clientflags & CAPTURE_DATA) != 0) {
			connectionDataPtr->clientflags &= ~CAPTURE_DATA;
		}

		/* After capturing the data, initiate the file transfer(s) */
		if ((connectionDataPtr->clientflags & MGET_STATE) != 0) {
			if (connectionDataPtr->capturesize > 0) {
				connectionDataPtr->ftpState = 6;
				NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection, kConnectionEventData, mytelport,0);  /* Stay alive to transfer file */
			}
			else {
				connectionDataPtr->clientflags &= ~MGET_STATE;
				connectionDataPtr->ftpState = 1;
			}
		}
		break;

		/* Receive one MGET file. */
	case 6:
		connection_status = dumpcon (myport, mywindow);
		if (getnname ((char *) captlist, (char *) word)) { /* for each name */
#if 0
			if (prompt) {								   /* check */
				sprintf (printline, "mget %s? ", word);
				putstring (mywindow, printline);
				if (ftpgets (answer, 20, 1) == ABORT) {	   /* abort */
					command[0] = '\0';					   /* no more processing */
					break;								   /* quit immediately */
				}
				if (tolower (*(stpblkp (answer))) != YES_CHAR)
					continue;
			}
			else {
#endif
				sprintf (printline, "Receiving %s\015\013", word);
				putstring (connectionDataPtr, printline);
#if 0
			}
#endif
			sprintf (command, "RETR \"%s\"", word);
			if ((cmd_status = ftpdo (connectionDataPtr, command, "")) < 0) {
				connectionDataPtr->clientflags &= ~MGET_STATE;
				connectionDataPtr->ftpState = 1;
			}
		}
		else {
			connectionDataPtr->clientflags &= ~MGET_STATE;
			connectionDataPtr->ftpState = 1;
		}
		break;

		/* Send a file to the remote connection */
	case 20:

		if (mycode == kConnectionEventFailure)
			mycode = kConnectionEventClose;							   /* something went wrong */

		if (mycode == kConnectionEventOpen) {
			connectionDataPtr->ftpState = 21;
			transtime = TickCount ();
			translen = 0;
			towrite = 0;
			xp = 0;
			NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection, kConnectionEventData, myport,0);
		}
		else if ((mycode == kConnectionEventData) && (myport == mytelport)) {
			connection_status = dumpcon (myport, mywindow);
		}
		break;

	case 21:
		/* transfer file(s) to the other host via ftp request file is already open = ftpfh */
		if (mycode == kConnectionEventData) {
			if (myport == mytelport) {
				connection_status = dumpcon (myport, mywindow);
			}
			else if (myport == myftpdata) {

				NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection, kConnectionEventData, myport,0);	   /* Stay alive */

				if (towrite <= xp) {

					i = BUFFERS;
					towrite = MBread (ftp_mbfp, xs, i);
					xp = 0;
				}

				if ((towrite <= 0) || Network_IsSessionOnPortBroken(myftpdata)) {/* we are done */
					if (ftp_mbfp->fd != 0)
						MBclose (ftp_mbfp);				   /* close input file */
					connectionDataPtr->ftpState = 22;
					break;
				}

				if (ftpfilemode == FAMODE)
					i = Sfwrite (myftpdata, &xs[xp], (int) towrite - xp);
				else
					i = Network_SendData(myftpdata, &xs[xp], (int) towrite - xp);

				if (i > 0) {
					xp += i;
					translen += i;
				}

				mytime = TickCount ();
				if ((transtime + 800) < mytime) {
					transtime = mytime;
					sprintf (printline, "%ld/%ld bytes sent.\015\013", translen, filelen);
					i = strlen (printline);
					Parse_ShowOrInterpretAsCommand(connectionDataPtr, (unsigned char *) &printline[0], i);
				}
			}
		}

		break;

	case 22:											   /* wait for data to be accepted */
		NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection, kConnectionEventData, myport,0);			   /* Stay alive */

		fcnt = Network_SendFlush(myftpdata);						   /* will go negative on err */
		if (!fcnt || Network_IsSessionOnPortBroken(myftpdata)) {
			mycode = kConnectionEventClose;
			sprintf (printline, "%ld/%ld bytes sent.\015\013", translen, filelen);
			Parse_ShowOrInterpretAsCommand(connectionDataPtr, (unsigned char *) &printline[0], strlen (printline));
		}
		if ((mycode == kConnectionEventData) && (myport == mytelport))
			connection_status = dumpcon (myport, mywindow);
		break;

		/* Get a file from the remote connection */
	case 30:
		if (mycode == kConnectionEventFailure)
			mycode = kConnectionEventClose;							   /* something went wrong */

		if (mycode == kConnectionEventOpen) {
			connectionDataPtr->ftpState = 31;
			filelen = xp = 0;
			transtime = TickCount ();
			translen = 0L;
		}
		else if ((mycode == kConnectionEventData) && (myport == mytelport)) {
			connection_status = dumpcon (myport, mywindow);
		}
		break;

		/* file has already been opened, take everything from the connection and place into the
		   open file: ftpfh */
	case 31:
		if (mycode == kConnectionEventData) {
			if (myport == mytelport) {
				connection_status = dumpcon (myport, mywindow);
			}
			else if (myport == myftpdata) {

				/* wait until xs is full before writing to disk */
				if (filelen <= 2000) {

					if (xp) {
						if (0 > MBwrite (ftp_mbfp, xs, xp))
							mycode = kConnectionEventClose;
						xp = 0;
					}
					filelen = BUFFERS;					   /* expected or desired len to go */
				}

				if (mycode == kConnectionEventData) {
					if (ftpfilemode == FAMODE)
						fcnt = Sfread (myftpdata, (char *)&xs[xp], (long) filelen);
					else
						fcnt = Network_GetData(myftpdata, &xs[xp], (int) filelen);

					if (fcnt >= 0) {
						filelen -= fcnt;
						xp += fcnt;
						translen += fcnt;
					}

					mytime = TickCount ();
					if ((transtime + 800) < mytime) {
						transtime = mytime;
						if ((connectionDataPtr->clientflags & MGET_STATE) == 0) {
							sprintf (printline, "%ld bytes received.\015\013", translen);
							i = strlen (printline);
							Parse_ShowOrInterpretAsCommand(connectionDataPtr, (unsigned char *) &printline[0], i);
						}
					}

					/* printf(" %d %d %d \012",filelen,xp,fcnt); n_row();  */
					if (fcnt < 0) {
						if (0 > MBwrite (ftp_mbfp, xs, xp)) {
							break;
						}
						if (ftp_mbfp->fd != 0) {
							MBclose (ftp_mbfp);
						}
						ftpfh = 0;
					}
					NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection, kConnectionEventData, myport,0);
				}
			}
		}
		/* Ouput any remaining data before closing */
		if (mycode == kConnectionEventClose) {
			if (xp) {
				if (0 > MBwrite (ftp_mbfp, xs, xp))
					Network_CloseConnection (myftpdata);
			}
			if ((connectionDataPtr->clientflags & MGET_STATE) == 0) {
				sprintf (printline, "%ld bytes received.\015\013", translen);
				i = strlen (printline);
				Parse_ShowOrInterpretAsCommand(connectionDataPtr, (unsigned char *) &printline[0], i);
			}
		}
		break;

		/* Waiting for "ftpdata" connection to open, display any communication from the telnet
		   port. */
	case 40:
		if (mycode == kConnectionEventOpen)
			connectionDataPtr->ftpState = 41;

		if ((myport == myftpdata) &&
			((connectionDataPtr->clientflags & CAPTURE_DATA) != 0)) {
			captcon (myport, mywindow);
		}
		else {
			connection_status = dumpcon (myport, mywindow);
		}
		break;

		/* If the "ftpdata" port is open, then postpone reading and displaying any data from other
		   ports once data has been received.  Otherwise, read and display data from anywhere. */
	case 41:
		if (mycode == kConnectionEventData) {
			if ((myport == myftpdata) &&
				((connectionDataPtr->clientflags & CAPTURE_DATA) != 0)) {
				captcon (myport, mywindow);
			}
			else {
				connection_status = dumpcon (myport, mywindow);
			}
			if (myport == myftpdata)
				connectionDataPtr->ftpState = 42;
			NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection, kConnectionEventData, myport,0);
		}
		else if (mycode == kConnectionEventClose) {
			connectionDataPtr->ftpState = 40;
		}
		break;

		/* Only data from "ftpdata" */
	case 42:
		if (mycode == kConnectionEventData) {
			if (myport == myftpdata) {
				if ((connectionDataPtr->clientflags & CAPTURE_DATA) != 0) {
					captcon (myport, mywindow);
				}
				else {
					connection_status = dumpcon (myport, mywindow);
				}
			}
			NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection, kConnectionEventData, myport,0);		   /* Keep port monitoring alive */
		}
		else if (mycode == kConnectionEventClose) {
			connectionDataPtr->ftpState = 40;
		}
		break;

	}													   /* end of switch */

	/* After reading from connection, if the connection is closed, reset up shop. */
	if ((mycode == kConnectionEventClose) || (connection_status < 0)) {
		if (connection_status < 0)
			connectionDataPtr->clientflags &= ~MPUT_STATE;
		if (ftpfh > 0) {
			// Close(ftpfh);
			ftpfh = 0;
		}
		if (ftp_mbfp->fd != 0)
			MBclose (ftp_mbfp);
		connectionDataPtr->ftpState = 1;
		fcnt = 0;
		if (myftpdata == myport) {
			Network_CloseConnection (myftpdata);
			Network_SetPortType(myftpdata, kInvalidNetworkPortType);
			connectionDataPtr->ftpport = -1;
		}
		if ((connectionDataPtr->clientflags & TYPE_I_ON_CLOSE) != 0) {
			connectionDataPtr->clientflags = 0;
			cmd_status = ftpdo (connectionDataPtr, "TYPE I", "");
		}

		/* Send the next file of an MPUT */
		if ((connectionDataPtr->clientflags & MPUT_STATE) != 0) {
			needanswer = true;
			while (needanswer) {
				needanswer = false;
				if (glob) {								   /* local wildcard expansion */
					newfile = (unsigned char *) nextname (Template, &ClientDirectoryState);	/* get next name */
					if (newfile == null) {				   /* if no expansions yet */
						if (getword ((char *) captlist, (char *) transfile)) {
							strcpy(Template, (char *) transfile);
							newfile = (unsigned char *) firstname (Template, gFTPClientDirID, gFTPClientVRefNum, &ClientDirectoryState);	/* get first name */
							if (newfile == null) {		   /* if no expansions */
								sprintf (printline, "No match for %s", transfile);
								ftpclient_debug_print (printline);
							}
						}
					}
				}
				else if (getword ((char *) captlist, (char *) transfile)) {
					newfile = transfile;
				}
				else
					newfile = null;

				if (newfile == null) {
					connectionDataPtr->clientflags &= ~MPUT_STATE;
					connectionDataPtr->ftpState = 1;
				}
				else {
#if 1													   /* When prompting is fixed switch to the
														      code below */
					sprintf (connectionDataPtr->ftpbuf, "STOR \"%s\"", newfile);
					cmd_status = ftpdo (connectionDataPtr, connectionDataPtr->ftpbuf, "");
#else
					if (prompt) {						   /* check */
						sprintf (printline, "mput %s? ", newfile);
						putstring (mywindow, printline);
						if (ftpgets (answer, 20, 1) == ABORT) {	/* abort */
							connectionDataPtr->clientflags &= ~MPUT_STATE;
							connectionDataPtr->ftpState = 1;
						}
						else if (tolower (*(stpblkp (answer))) != YES_CHAR) {
							needanswer = true;
						}
						else {
							sprintf (connectionDataPtr->ftpbuf, "STOR \"%s\"", newfile);
							cmd_status = ftpdo (connectionDataPtr, connectionDataPtr->ftpbuf, "");
						}
					}
					else {
						/* The Mac file may have special characters, so quote the whole thing */
						sprintf (connectionDataPtr->ftpbuf, "STOR \"%s\"", newfile);
						cmd_status = ftpdo (connectionDataPtr, connectionDataPtr->ftpbuf, "");
					}
#endif
				}
			}
		}

		if ((connectionDataPtr->clientflags & MGET_STATE) != 0) {
			connectionDataPtr->ftpState = 5;
			NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection, kConnectionEventData, mytelport,0);	   /* stay alive to transfer file */
		}
	}
	return (true);
}


//
// internal methods
//

/**
To display a string from the FTP Client
string list on the given screen, followed
by a carriage return and line feed, call
this method.

(3.0)
*/
static void
putFTPClientMessage	(ConnectionDataPtr	inScreen,
					 const short		inStringIndex)
{
	Str255		string;
	char*		cString = null;
	
	
	GetIndString(string, rStringsFTPClientMessages, inStringIndex);
	PLstrcat(string, "\p\015\012");
	StringUtilities_PToC(string, cString);
	putstring(inScreen, cString);
}// putFTPClientMessage


/**
To display a string from the FTP Client
string list on the given screen with no
carriage return or line feed after it,
but a space instead, call this method.

(3.0)
*/
static void
putFTPClientPrompt	(ConnectionDataPtr	inScreen,
					 const short		inStringIndex)
{
	Str255		string;
	char*		cString = null;
	
	
	GetIndString(string, rStringsFTPClientMessages, inStringIndex);
	PLstrcat(string, "\p ");
	StringUtilities_PToC(string, cString);
	putstring(inScreen, cString);
}// putFTPClientPrompt


/**
To display a string using VT100 emulation
routines, use this method.

(2.6)
*/
static void
putstring		(ConnectionDataPtr	inScreen,
				 char*				inString)
{
	for (; *inString; inString++) Parse_ShowOrInterpretAsCommand(inScreen, (unsigned char *)inString, 1);
}// putstring
