/*###############################################################

	FTPLog.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// standard-C includes
#ifdef __cplusplus
#	include <cstring>
#else
#	include <string.h>
#endif

// resource includes
#include "StringResources.h"

// MacTelnet includes
#include "ConnectionData.h"
#include "FTPLog.h"
#include "Preferences.h"
#include "RealScreen.h"
#include "ResourceFileManager.h"
#include "ScreenManager.h"
#include "TelnetDescriptors.h"
#include "Terminal.h"



// variables
static WindowRef			gFTPLogWindow = null;
static TelnetScreenID		gFTPLogID = 0;



//
// public methods
//

/*!
This method sets up the initial configurations for
the file transfer protocol activity log window.
Call this method before calling any other FTP log
methods from this file.

(3.0)
*/
void
FTPLog_Init	(void)
{
	// create the FTP log
	gFTPLogID = Terminal_NewWindow("\pMonaco"/* font */, 9/* font size */, kRealScreenFlagTextWraps,
									100/* scrollback size */, 30/* rows */, 70/* columns */, false/* force save */);
	if (gFTPLogID >= 0)
	{
		Str255		windowName;
		
		
		GetIndString(windowName, rStringsMiscellaneous, siFTPLogName);
		RealScreen_SetWTitle(gFTPLogID, windowName);
		
		gFTPLogWindow = ScreenManager_GetWindowOf(gFTPLogID);
		SetWindowKind(gFTPLogWindow, WIN_LOG);
		MoveWindow(gFTPLogWindow, 400, 120, false/* activate */);
		if (gFTPServerPrefs != null) if (gFTPServerPrefs->showLog) ShowWindow(FTPLog_GetWindow());
		
		// initialize more of the logs connection settings, many of which dont apply to the FTP log anyway
		VSwrite(gFTPLogID, "\033[24;0H", 7); // move to the bottom of the display
	}
	else BEEP;
}// Init


/*!
This method cleans up the file transfer protocol
activity log window by destroying data structures
allocated by FTPLog_Init().

(3.0)
*/
void
FTPLog_Done	(void)
{
}// Done


/*!
To obtain the Mac OS window pointer for the
FTP Log window, call this method.

(3.0)
*/
WindowRef
FTPLog_GetWindow	(void)
{
	WindowRef		result = null;
	
	
	result = gFTPLogWindow;
	return result;
}// GetWindow


/*!
To determine if the log is showing, call this
method.

(3.0)
*/
Boolean
FTPLog_IsVisible	(void)
{
	return IsWindowVisible(FTPLog_GetWindow());
}// IsVisible


/*!
To show or hide the FTP Log, call this
method.

(3.0)
*/
void
FTPLog_SetVisible	(Boolean	inIsVisible)
{
	if (inIsVisible)
	{
		ShowWindow(FTPLog_GetWindow());
	}
	else
	{
		HideWindow(FTPLog_GetWindow());
	}
}// SetVisible


/*!
To write a standard horizontal line (made of
dashes) to the FTP log, followed by a carriage
return and line feed, call this method.

(3.0)
*/
void
FTPLog_WriteHorizontalRule	(void)
{
	FTPLog_WriteLine("-----------------------------------------------------------------------------");
}// WriteHorizontalRule


/*!
To write data to the FTP Log, followed by a
carriage return and line feed, call this method.
Pass an empty string to write just a new-line
character.

(3.0)
*/
void
FTPLog_WriteLine	(char*		inString)
{
	FTPLog_WriteString(inString);
	FTPLog_WriteString("\015\012");
}// WriteLine


/*!
To write data to the FTP Log, call this
method.

(3.0)
*/
void
FTPLog_WriteString	(char*		inString)
{
	// safety check - do nothing unless FTPLog_Init() was called
	if ((gFTPLogID >= 0) && (FTPLog_GetWindow() != null))
	{
		short		size = 0;
		
		
		size = strlen(inString);
		if ((size >= 1) && (size <= 80)) VSwrite(gFTPLogID, inString, size);
	}
}// WriteString


