/*###############################################################

	NetBackground.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// standard-C includes
#ifdef __cplusplus
#	include <cstdio>
#	include <cstring>
#else
#	include <stdio.h>
#	include <string.h>
#endif
#include <ctype.h>

// Mac includes
#include <ControlDefinitions.h>
#include <PLStringFuncs.h>

// resource includes
#include "DialogResources.h"
#include "StringResources.h"

// MacTelnet includes
#include "ConnectionData.h"
#include "Console.h"
#include "DialogItemIndices.h"
#include "DialogUtilities.h"
#include "FileUtilities.h"
#include "FTPLog.h"
#include "InternalEvents.h"
#include "MacBinary.h"
#include "MacLook.h"
#include "NetBackground.h"
#include "NetEvents.h"
#include "network.proto.h"
#include "Preferences.h"
#include "ProgressDialog.h"
#include "ResourceFileManager.h"
#include "TelnetErrors.h"
#include "TelnetMenus.h"
#include "TextTranslation.h"
#include "vsdata.h"



// constants
#define MACBINARY
#define O_RAW O_RDONLY
enum
{
	PRCP = 2,
	HTELNET = 23,
	HRSHD = 514,
	BUFFERS = 8000,
	PATHLEN = 256,
	RCPSEGSIZE = 512,
	EOLCHAR = 13
};

#define FIMAGE O_RAW
enum
{
	FASCII = 0,
	FAMODE = 0,
	FIMODE = 1,
	FMMODE = 2			/* Mac Binary, when ready */
};

MBFile
	*mbfp=null;

// variables
char *nextfile;

static Handle mungbuf = 0;

	
static long gDefaultDirDirID = 2;
static short gDefaultDirVRefNum = -1;

static short fcnt=0;
static  short
	ftpenable=0,				/* is file transfer enabled? */
	rcpenable=0,				/* is rcp enabled? */
	rsnum=-1,					/* port number for incoming rshell */
	rserr=-1;					/* port number for rshd() stderr */

static  unsigned char *xs=0L,	/* buffer space for file transfer */
		*pathname=0L,			/* space to keep path names */
		*newfile=0L,			/* current file being received */
		myuser[17],					/* user name on my machine */
		waitchar;					/* character waiting for from net */

static short 
	curstate = -1,			/* state machine for background processes */
	retstate = 200,			/* to emulate a subroutine call */
	ftpState = 0,			/* state of the ftp data transfer */
	isdir=0,				/* flag for rcp target pathname */
	waitpos=0,				/* marker for gathering strings from net */
	cnt=0,					/* number of characters from last Network_GetData() */
	fh=0,					/* file handle when transfer file is open */
	ftpfh=0,				/* file handle for ftp data */
	rc=0,					/* telnet flag */
	xp=0,					/* general pointer */
	towrite=0,				/* file transfer pointer */
	len=0;					/* file transfer length */

static long
	filelen=0L;				/* length of current file for transfer */

short mainport[kNetworkMaximumNumberOfPorts];		/* map FTP data port to main port */
short dataport[kNetworkMaximumNumberOfPorts];		/* map FTP main port to data port */
short mainstate[kNetworkMaximumNumberOfPorts];
short datastate[kNetworkMaximumNumberOfPorts];
short retnstate[kNetworkMaximumNumberOfPorts];

ip_port	fileport[kNetworkMaximumNumberOfPorts];

static char crfound=0;

static short rfstate,
	ftptmode=FAMODE;			/* how to transfer the file on net */

static	ip_addr	portaddr[kNetworkMaximumNumberOfPorts],
				hishostaddr;
static	ip_port	portremoteport[kNetworkMaximumNumberOfPorts],
				portlocalport[kNetworkMaximumNumberOfPorts];
				
static unsigned char messs[100];

static char	ListingTemplate[256];
static CInfoPBRec	theDirectoryState;

// internal methods
static Boolean				canUserChangeDirectories	(char*			inUserName);
static void					CRESP						(short			fnum,
														 short			msg_number);
static ProgressDialogRef	displayFTPStatusDialog		(const short	inStringIndex);
static short				ftpgo						(short			fnum);
static void					sendLogInMessage			(short			fnum);
static void					setftp						(const Boolean	inShowProgress);
static void					unsetftp					(void);



//
// public methods
//

/*!
To start or stop the FTP server, or to
change it to a different runtime mode
(i.e. anonymous versus registered users
only), use this method.

If you do not know what mode the server
was in before, just use the off mode,
"kFTPServerModeOff".

(3.0)
*/
void
NewFTPServerMode	(FTPServerMode		inOldMode,
					 FTPServerMode		inNewMode)
{
	if (inOldMode != inNewMode)
	{
		Boolean		showProgress = ((inOldMode == kFTPServerModeOff) || (inNewMode == kFTPServerModeOff));
		
		
		Console_BeginFunction();
		Console_WriteLine("NewFTPServerMode");
		if (inNewMode != kFTPServerModeOff) setftp(showProgress);
		else unsetftp();
		Cursors_UseArrow();
		Console_EndFunction();
	}
}// NewFTPServerMode


void rftpd
  (
	short code,
	short fnum
  )
{
	ip_port	fdport;
	short	ftpdata, i, append = 0;
	char	tempp[256], theDir[256];
	char updir[4];						/* JMB 2.6 */
	
	if (!ftpenable)
		return;

	Network_SendFlush(fnum);

	rfstate = mainstate[fnum];
	ftpState = datastate[fnum];
	retstate = retnstate[fnum];
	fdport = fileport[fnum];

	switch (rfstate) {
		case 0:
			Console_WriteValue("rfstate", rfstate);
			if (code != kConnectionEventOpen) 
				break;
			ftptmode = FAMODE;
			NetEvents_Post(kTelnetNetworkEventClassUser,kUserEventFTPConnectionOpen,-1,0);
			if (!xs) xs=(unsigned char *)Memory_NewPtr(BUFFERS+10);
			if (!pathname) pathname=(unsigned char *)Memory_NewPtr(PATHLEN);
			if (!newfile) newfile=(unsigned char *)Memory_NewPtr(PATHLEN);
			if (!xs || !pathname || !newfile)	/* no memory for ftp names */
			{
				// this method does not return
				Errors_DisplayStopQuitMessage(rStringsMemoryErrors, siMemErrorFatalOutOfMemory, -1);
			}	
		
			rfstate = 1;				/* drop through */
		case 1:
			Console_WriteValue("rfstate", rfstate);
			sendLogInMessage(fnum);
			Console_WriteLine("done writing log-in message");
			Network_MakeFTPQueriesOnPort(fnum, &portaddr[fnum], &portlocalport[fnum], &portremoteport[fnum]);		/* get default ftp information */
			Console_WriteLine("done making queries");
			hishostaddr = portaddr[fnum];
			fdport = portremoteport[fnum];

			waitpos = 0; waitchar = '\012';
			rfstate = 50;       		/* note skips over */
			Console_WriteLine("about to set state");
			if ((gFTPServerPrefs->serverMode == kFTPServerModeOnPasswordProtection))
				retstate = 3;				/* check pass */
			else
				retstate = 5;				/* who needs one ? */
			Console_WriteValue("finished with rfstate", rfstate);
			break;
		case 3:				/* check for passwords */
		case 4:
			Console_WriteValue("rfstate", rfstate);
			waitpos = 0;  waitchar = '\012';
			rfstate = 50;  
			if (!strncmp("USER",(char *) xs,4)) {
				if (strlen((char *) xs) < 6)		/* make sure blank name stays blank */
					xs[5] = myuser[0] = 0;
				strncpy((char *) myuser,(char *) &xs[5],16);		/* keep user name */
				FTPLogUserLoggingIn();
				CRESP(fnum,23);
				retstate = 4;		/* wait for password */
				break;
			}
			if (!strncmp("PASS",(char *) xs,4)) {
				if (Scheckpass((char *) myuser,(char *) &xs[5])) {
					FTPLog_WriteLine("FTP password OK - logged in");
					CRESP(fnum,10);
					retstate = 5;
				}
				else {
					FTPLog_WriteLine("FTP password failed - access denied");
					CRESP(fnum,24);
					retstate = 3;
				}
				break;
			}
			if (!strncmp("QUIT",(char *) xs,4)) {
				CRESP(fnum,2);
				cnt = -1;
			}
			else {
				CRESP(fnum,22);	
			}
			retstate = 3;			/* must have password first */
			break;				
				
/* interpret commands that are received from the other side */
			
		case 5:
			Console_WriteValue("rfstate", rfstate);
/* set to a safe state to handle recursion, wait for another command line from client */
			rfstate = 50;
			retstate = 5;
			waitchar = '\012';
			waitpos = 0;

			if (!strncmp((char *) xs,"LIST",4) || !strncmp((char *) xs,"NLST",4)) {
				if ((strlen((char *) xs) < 6) || xs[5] == '.')
					strcpy((char *) xs,"LIST *");
				strcpy(ListingTemplate, (char *) &xs[5]);

				theDirectoryState.hFileInfo.ioFDirIndex=1;
				theDirectoryState.hFileInfo.ioVRefNum = gDefaultDirVRefNum;
				theDirectoryState.hFileInfo.ioDirID = gDefaultDirDirID;

				nextfile = FileUtilities_GetNameUsingWildcard(ListingTemplate, &theDirectoryState);	/* find first name */
				if (nextfile == null) {
					CRESP(fnum,16);
				}
				else {
					Str255	tempstr;
					char	tempstr2[256];
					ftpdata = ftpgo(fnum);				/* open the connection */
					fdport = portremoteport[fnum]; /* reset to def */

					if (ftpdata >= 0) Network_SetPortType(ftpdata, kNetworkPortTypeFTPServerData);
					ftpState = 40;		/* ready to transmit */
					CRESP(fnum,4);							
					FileUtilities_GetPathnameFromDirectoryID(gDefaultDirDirID, gDefaultDirVRefNum, tempstr);
					StringUtilities_PToCInPlace(tempstr);
					sprintf(tempstr2, "FTP Sending Directory %s\015\012", (char *)tempstr);
					FTPLog_WriteString(tempstr2);
				}
			}
			else if (!strncmp((char*)xs, "CWD", 3))
			{	
				Boolean		allowed = canUserChangeDirectories((char*)myuser);
				
				
				allowed = true; // users may tunnel down, but not up; to restrict CWD as well, comment this out
				if (allowed)
				{
					if (FileUtilities_ChangeDirectory(&(gDefaultDirDirID), &(gDefaultDirVRefNum), (char*)&xs[4]))
					{
						// failed!
						CRESP(fnum, 12);						
					}
					else
					{
						// success!
						CRESP(fnum, 13);			
					}
				}
				else CRESP(fnum, 12); // permission denied - the "12" message has been modified to include this
			}
			else if (!strncmp((char*)xs, "CDUP", 4))
			{
				Boolean		allowed = canUserChangeDirectories((char*)myuser);
				
				
				strcpy(updir, "..");
				if (allowed)
				{
					if (FileUtilities_ChangeDirectory(&(gDefaultDirDirID), &(gDefaultDirVRefNum), updir))
					{
						// failed!
						CRESP(fnum, 12);
					}
					else
					{
						// success!
						CRESP(fnum, 13);
					}
				}
				else CRESP(fnum, 12); // permission denied - the "12" message has been modified to include this
			}
			else if (!strncmp((char *) xs,"STOR",4) || !strncmp((char *) xs,"APPE", 4))
			  {
				short	MBflag;

				if (!strncmp((char *) xs,"APPE", 4)) {
					if (ftptmode != FASCII) {
						CRESP(fnum, 35);
						break;
						}
					else
						append = 1;
					}
					
				/* had to simplify this, else MPW C 3.0/3.1 generates bad code! */
				if ((!FlagManager_Test(kFlagMacBinaryIIEnabled)) || (ftptmode == FAMODE))
					MBflag = MB_DISABLE;
				else
					MBflag = 0;
					
				if (ftptmode == FAMODE)	MBflag |= MB_ISASCII;
				if (append) MBflag |= MB_APPEND;
				append = 0;
				
				mbfp = MBopen((char *)&xs[5], gDefaultDirVRefNum, gDefaultDirDirID, (short)(MB_WRITE | MBflag));
				if (mbfp == 0L)
				  {
					CRESP(fnum,21);			
					break;
				  }
				else
					ftpfh = 12;

				ftpState = 0;

				strncpy((char *) newfile,(char *) &xs[5],PATHLEN-1);	
				

				ftpdata = ftpgo(fnum);				/* open connection */
				fdport = portremoteport[fnum]; /* reset to def */

				if (ftpdata >= 0) Network_SetPortType(ftpdata, kNetworkPortTypeFTPServerData);

				CRESP(fnum,4);		

				ftpState = 30;		/* ready for data */
			  }

			else if (!strncmp((char *) xs,"RETR",4))	
			  {
			  /* had to simplify this, else MPW C 3.0/3.1 generates bad code! */
				short
					MBflag;
				if ((!FlagManager_Test(kFlagMacBinaryIIEnabled)) || (ftptmode == FAMODE))
					MBflag = MB_DISABLE;
				else
					MBflag = 0;
					
				if (ftptmode == FAMODE)	MBflag |= MB_ISASCII;
				
				mbfp = MBopen((char *) &xs[5], gDefaultDirVRefNum, gDefaultDirDirID, MB_READ | MBflag);
				
				if (mbfp == 0L) {
					CRESP(fnum,11);						
					break;
				}
				ftpfh = 12;

				strncpy((char *) newfile,(char *) &xs[5],PATHLEN-1);	

				FileUtilities_GetPathnameFromDirectoryID(gDefaultDirDirID, gDefaultDirVRefNum, (StringPtr) theDir);
				StringUtilities_PToCInPlace((StringPtr)theDir);
				sprintf(tempp,"remote <-- %s/%s\015\012",theDir,newfile);
				FTPLog_WriteString(tempp);

				ftpdata = ftpgo(fnum);				/* open connection */
				fdport = portremoteport[fnum];

				ftpState = 20;		/* ready for data */
				if (ftpdata >= 0) Network_SetPortType(ftpdata, kNetworkPortTypeFTPServerData);

				CRESP(fnum,4);							
			  }
			else if (!strncmp((char *) xs,"TYPE",4)) {	
				if (toupper(xs[5]) == 'I') {
					ftptmode = FIMODE;
					if (FlagManager_Test(kFlagMacBinaryIIEnabled)) {
						CRESP(fnum,33);					/* Binary on, MACB ON */
						}
					else {
						CRESP(fnum,34);					/* Binary on, MACB off */
						}
				}
				else if (toupper(xs[5]) == 'A') {
					ftptmode = FAMODE;
					CRESP(fnum,6);					
					}
				else {
					CRESP(fnum,17);					
				}

			}
			else if (!strncmp((char *) xs,"MACB",4)) {	
				if (toupper(xs[5]) == 'E') {
					FlagManager_Set(kFlagMacBinaryIIEnabled, true);
					CRESP(fnum,25);					
				}
				else {
					FlagManager_Set(kFlagMacBinaryIIEnabled, false);
					CRESP(fnum,26);					
				}
			}
			else if (!strncmp((char *) xs,"PORT",4)) {	
/*	get the requested port number from the command given */
				ip_addrbytes	tempaddr;
				unsigned char 	*stringPtr, *tempPtr, storage[10];
				long			numbers[6];
				short index;

				for (tempPtr = stringPtr = &xs[5], index = 0; (*stringPtr != '\0') && (index < 6);index++) {
    				while ((*tempPtr != '\0') && (*tempPtr != ',')) { tempPtr++; };
     				BlockMoveData(stringPtr, &(storage[1]), tempPtr-stringPtr);
     				storage[0] = tempPtr-stringPtr;
     				StringToNum(storage, &numbers[index]);
     				tempPtr++;
     				stringPtr = tempPtr;
     			}
				
				tempaddr.a.byte[0] = (char) numbers[0];
				tempaddr.a.byte[1] = (char) numbers[1];
				tempaddr.a.byte[2] = (char) numbers[2];
				tempaddr.a.byte[3] = (char) numbers[3];
				portaddr[fnum] = tempaddr.a.addr;
				portlocalport[fnum] = numbers[4]*256 + numbers[5];
				fdport = portlocalport[fnum];
				CRESP(fnum,3);							
			}
			else if (!strncmp((char *) xs,"QUIT",4)) {	
				CRESP(fnum,2);							
				rfstate = 60;
				NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection,kConnectionEventData,fnum,0);	/* post back to me */
			}
			else if (!strncmp((char *) xs,"XPWD",4) || !strncmp((char *) xs,"PWD",3)) {	
				CRESP(fnum,14);						/* start reply */
				FileUtilities_GetPathnameFromDirectoryID(gDefaultDirDirID, gDefaultDirVRefNum, (StringPtr) xs);
				StringUtilities_PToCInPlace((StringPtr) xs);
				Network_SendData(fnum,(char *) xs,strlen((char *) xs));	/* write dir name */
				CRESP(fnum,15);						/* finish reply */
			}
			else if (!strncmp((char *) xs,"USER",4)) {	
				if (strlen((char *) xs) < 6)			/* make sure blank name stays blank */
					xs[5] = myuser[0] = 0;
				strncpy((char *) myuser,(char *) &xs[5],16);	/* keep user name */
				FTPLogUserLoggingIn();
				/* confirm log in without password */
				CRESP(fnum,10);							
			}
			else if (!strncmp((char *) xs,"STRU",4)) {	/* only one stru allowed */
				if (xs[5] == 'F') {
					CRESP(fnum,18); }					
				else {
					CRESP(fnum,17); }					
			}
			else if (!strncmp((char *) xs,"MODE",4)) {	/* only one mode allowed */
				if (xs[5] == 'S') {
					CRESP(fnum,19); }					
				else {
					CRESP(fnum,17); }					
			}
			else if (!strncmp((char *) xs,"ALLO",4) || !strncmp((char *) xs,"ACCT",4)) {	
				CRESP(fnum,20); }						
			else if (!strncmp((char *) xs,"HELP",4)) {	
				for (i=28; i<33; i++) {
					CRESP(fnum,i); }					
			}
			else if (!strncmp((char *) xs,"NOOP",4)) {		
				CRESP(fnum,9); }							
			else if (!strncmp((char *) xs,"SYST",4)) {	// JMB 2.6
				  Network_SendFlushData(fnum, "225 MACOS System Version:\015\012", 27);	// JMB 2.6
					}	// JMB 2.6
			else {			/* command not understood */
				CRESP(fnum,8); 								
			}

			break;

/*	subroutine to wait for a particular character */
		case 50:
			Console_WriteValue("rfstate", rfstate);
			while (0 < (cnt = Network_GetData(fnum,&xs[waitpos],1))) {
				if (xs[waitpos] == waitchar) {
					rfstate = retstate;

					while (xs[waitpos] < 33)		/* find end of string */
						waitpos--;
					xs[++waitpos] = '\0';			/* put in terminator */

					for (i=0; i<4; i++)				/* want upper case */
						xs[i] = toupper(xs[i]);

					break;
				}
				else
					waitpos += cnt;

			}
			break;

		case 60:					/* wait for message to get through */
									/* or connection is broken */
			Console_WriteValue("rfstate", rfstate);
			if (!Network_SendFlush(fnum) || Network_IsSessionOnPortBroken(fnum))
				cnt = -1;
			else
				NetEvents_PostProvisionally(kTelnetNetworkEventClassConnection,kConnectionEventData,fnum,0);	/* post back to me */
			break;

		default:
			break;

	}

	if (cnt < 0) {
		if (mbfp) {
			MBclose( mbfp );
			mbfp = null;
		}
		ftpdata = dataport[fnum];					
		if (ftpdata > 0) {
			Network_CloseConnection(ftpdata);
			NetEvents_Post(kTelnetNetworkEventClassUser,kUserEventFTPTransferEnd,-1,0);
			dataport[fnum] = -1;					
			mainport[ftpdata] = -1;					
		}
		rfstate = 100;
		ftpState = 0;
		cnt = 0;
		Network_CloseConnection(fnum);
		Network_SetPortType(fnum, kInvalidNetworkPortType);
		NetEvents_Post(kTelnetNetworkEventClassUser,kUserEventFTPConnectionClose,-1,0);
#if 0												
		fnum = -1;
		ftpdata = -1;
#endif												
		setftp(false/* show progress */);				/* reset it */
	}

	mainstate[fnum] = rfstate;						
	datastate[fnum] = ftpState;						
	retnstate[fnum] = retstate;						
	fileport[fnum] = fdport;						

}

/*	important note:  for Sfread, nwant must be 256 bytes LARGER than the amount
	which will probably be read from the connection.
	Sfread will stop anywhere from 0 to 256 bytes short of filling nwant
	number of bytes. */
	
long Sfread( short pnum, char *buf, long nwant)
{
	long	i, ndone, lim;
	char	*p, *q;

	if (nwant < 1024)
		return(-1);

	ndone = 0;

	HLock(mungbuf);
	while (ndone < nwant - 1024) {

		if (0 >= (lim = Network_GetData(pnum,*mungbuf,1024))) {
			if (ndone || !lim)			/* if this read is valid, but no data */
				return(ndone);
			else
				return(-1);				/* if connection is closed for good */
		}

		p = *mungbuf;
		q = (char *) buf + ndone;

/*		printf("\012 lim=%d done=%d want=%d",lim,ndone,nwant);
		n_row();
*/
		for (i=0; i < lim; i++) {

			if (crfound) {
				if (*p == 10)
					*q++ = EOLCHAR;
				else if (*p == 0)
					*q++ = 13;			/* CR-NUL means CR */
				crfound = 0;
			}
			else if (*p == 13)
				crfound = 1;
			else
				if (gFTPServerPrefs->useISOTranslation)
				{
					*q++ = (char)TextTranslation_ConvertISOCharacterToMac((unsigned char*)p); // transform to Mac
				}
				else
					*q++ = *p;				/* copy the char */

			p++;
		}

		ndone = q-buf;					/* count chars ready */
	}
	
	HUnlock(mungbuf);
	
	return(ndone);
}

/***************************************************************************/
/*  Sfwrite
*   Write an EOL translated buffer into Network_SendData.
*   Returns the number of bytes which were processed from the incoming
*   buffer.  Uses its own 1024 byte buffer for the translation (with Sfread).
*/

long Sfwrite( short pnum, void *buf, long nsrc)
{
	long	i,ndone,nout,lim;
	char	*p,*q;

	ndone = 0;
	HLock (mungbuf);
	while (ndone < nsrc) {

		if (0 > ( i = Network_GetNumberOfBytesOfRoomOnPort(pnum)))
			return(-1);

		if (i < 1024)					/* not enough room to work with */
			return(ndone);

/*	process up to 512 source bytes for output (could produce 1K bytes out) */
		if (nsrc - ndone > 512)
			lim = 512;
		else
			lim = nsrc-ndone;

		p = (char *) buf + ndone;				/* where to start this block */
		q = *mungbuf;					/* where munged stuff goes */
		for (i=0; i < lim; i++) {
			if (*p == EOLCHAR) {
				*q++ = 13;
				*q++ = 10;
				p++;
			}
			else
				if (gFTPServerPrefs->useISOTranslation)
				{
					*q++ = (char)TextTranslation_ConvertMacCharacterToISO((unsigned char*)p); // transform to ISO
					p++;
				}
				else
					*q++ = *p++;
		}
		ndone += lim;					/* # of chars processed */
		nout = q-*mungbuf;				/* # of chars new */

		
		if (nout != Network_SendData(pnum, *mungbuf, nout))
		{
			// 3.0 - some error, but who cares?
			BEEP;
		}
	}
	HUnlock (mungbuf);
	return(ndone);
}


/*	FTP receive and send file functions */

void ftpd
  (
	short code,
	short ftpdata							
  )
{
	short fnum = mainport[ftpdata];		
	short i,a;
	char tempp[256];
	char theDir[256];


	ftpState = datastate[fnum];			

	switch (ftpState) {
		default:
			break;

		case 40:				/* list file names in current dir */
			Console_WriteValue("ftpState", ftpState);
			if (code == kConnectionEventFailure)	/* something went wrong */
				fcnt = -1;
			if (code != kConnectionEventOpen) 	/* waiting for connection to open */
				break;
			
			ftpState = 41;

/*
*  send the "nextfile" string and then see if there is another file
*  name to send
*/
		case 41:
			Console_WriteValue("ftpState", ftpState);
			NetEvents_PostProvisionally(kTelnetNetworkEventClassSession,kSessionEventFTPTransferActive,ftpdata,0);
			Network_SendFlush(ftpdata);
			strcpy(tempp,nextfile);
			strcat(tempp,"\015\012");
			i = strlen(tempp);
			if (gFTPServerPrefs->useISOTranslation)
				TextTranslation_ConvertMacTextToISO((unsigned char*)tempp, i); // transform to ISO
			if (i != Network_SendData(ftpdata,tempp,i)) {
				CRESP(fnum, 1);
				fcnt = -1;
				break;
			}
			theDirectoryState.hFileInfo.ioFDirIndex++;
			if (null == (nextfile = FileUtilities_GetNameUsingWildcard(ListingTemplate, &theDirectoryState))) {	/* normal end */
				ftpState = 22;   			/* push data through */
			}
			break;
			
		case 30:
			Console_WriteValue("ftpState", ftpState);
			if (code == kConnectionEventFailure)	/* something went wrong */
				fcnt = -1;
			if (code != kConnectionEventOpen)	/* waiting for connection to open */
				break;
			ftpState = 31;
			crfound = 0;
			len = xp = 0;
			filelen = 0L;
			FileUtilities_GetPathnameFromDirectoryID(gDefaultDirDirID, gDefaultDirVRefNum, (StringPtr) theDir);
			StringUtilities_PToCInPlace((StringPtr) theDir);
			sprintf(tempp,"remote --> %s/%s\015\012",theDir,newfile);
			FTPLog_WriteString(tempp);
			NetEvents_Post(kTelnetNetworkEventClassUser,kUserEventFTPTransferBegin,-2,0);
			
		case 31:
/*
* file has already been opened, take everything from the connection
* and place into the open file: ftpfh
*/
			Console_WriteValue("ftpState", ftpState);
			do {
			/* wait until xs is full before writing to disk */
				if (len <= 2000) {

					if (xp) {
						if (0 > MBwrite(mbfp, xs, xp)) {
							Errors_DisplayStopMessage(rStringsDiskErrors, siResErrorDiskIsFull, -1);
							MBclose( mbfp);			/* Close on Disk Full Error */
							ftpState=22;
							CRESP(fnum,27);				
							FTPLog_WriteString((char *) messs);	
							if( (a = Network_CloseConnection(ftpdata)) !=0 )
							{
								// 3.0 - error closing the network, but who cares?
							}
							fcnt = -1;
							mbfp=null;
							break;
						}
						xp = 0;
					}
					len = BUFFERS;		/* expected or desired len to go */
				}

				if (ftptmode == FAMODE)
					fcnt = Sfread(ftpdata,(char *) &xs[xp],len);	
				else
					fcnt = Network_GetData(ftpdata,&xs[xp],len);

				if (fcnt >= 0) {
					len -= fcnt;
					xp += fcnt;
					filelen += fcnt;
				}

				if (fcnt < 0) {
					if (0 > MBwrite( mbfp, xs, xp)) {
						CRESP(fnum,27);			
						MBclose( mbfp);			/* Close file on error */
						break;
					}
					MBclose( mbfp );
					ftpfh = 0;
					CRESP(fnum,5);					
				}

			} while (fcnt > 0);
			break;

		case 20:
			Console_WriteValue("ftpState", ftpState);
			if (code == kConnectionEventFailure)	/* something went wrong */
				fcnt = -1;
			if (code != kConnectionEventOpen)	/* waiting for connection to open */
				break;
			ftpState = 21;
			filelen = MBsize( mbfp );
			towrite = 0;
			xp = 0;
			NetEvents_Post(kTelnetNetworkEventClassUser,kUserEventFTPTransferBegin,-1,0);

		case 21:
/*
*  transfer file(s) to the other host via ftp request
*  file is already open = ftpfh
*/
			Console_WriteValue("ftpState", ftpState);
			NetEvents_PostProvisionally(kTelnetNetworkEventClassSession,kSessionEventFTPTransferActive,ftpdata,0);
		
			if (towrite <= xp) {

				i = BUFFERS;
				towrite = MBread( mbfp, xs, i);
				xp = 0;
			}

			if (towrite <= 0 || Network_IsSessionOnPortBroken(ftpdata)) {		/* we are done */
				ftpState = 22;
				break;
			}

			if (ftptmode == FAMODE)
				i = Sfwrite(ftpdata,&xs[xp],towrite-xp);
			else
				i = Network_SendData(ftpdata,&xs[xp],towrite-xp);



			if (i > 0) {
				xp += i;
				filelen -= i;
				if (filelen < 0L)
					filelen = 0L;
			}

			break;

		case 22:		/* wait for data to be accepted */
			Console_WriteValue("ftpState", ftpState);
			NetEvents_PostProvisionally(kTelnetNetworkEventClassSession,kSessionEventFTPTransferActive,ftpdata,0);

			fcnt = Network_SendFlush(ftpdata);		/* will go negative on err */
			if (!fcnt || Network_IsSessionOnPortBroken(ftpdata))
				fcnt = -1;
			if (fcnt < 0) {
				CRESP(fnum,5);				
			}
			break;

		case 0:
			Console_WriteValue("ftpState", ftpState);
			break;

	}  /* end of switch */

/*	after reading from connection, if the connection is closed, reset up shop. */
	if (fcnt < 0) {
		if (mbfp) {
			MBclose( mbfp );
			mbfp = null;
		}
		ftpState = 0;
		fcnt = 0;
		if (ftpdata >= 0) {
			Network_CloseConnection(ftpdata);
			NetEvents_Post(kTelnetNetworkEventClassUser,kUserEventFTPTransferEnd,-1,0);
			mainport[ftpdata] = -1;				
			dataport[fnum] = -1;				
		}
	}

	datastate[fnum] = ftpState;			

}


/* Sftpname and Sftpuser and Sftphost
*  record the name of the file being transferred, to use in the status
*  line updates
*/

void Sftpname(char *s)
{
	strcpy(s, (char *) newfile);	
}

UInt32 Sftphost(void)
{
	return(hishostaddr);
}

void Sftpstat(long *byt)
{
	*byt = filelen;
}

void FTPLogUserLoggingIn(void)
{
	FTPLog_WriteString("FTP user ");
	FTPLog_WriteString((char *) myuser);
	FTPLog_WriteString(" logging in\015\012");
}

/****************************************************************************/
/* Sencompass - Compute the encrypted password */
void Sencompass(char *ps, char *en)
{
	short	i,ck;
	char	*p,c;

	ck = 0;
	p = ps;
	while (*p)				/* checksum the string */
		ck += *p++;

	c = ck;

	for (i=0; i<10; i++) {
		*en =  (((*ps ^ c) | 32) & 127); 	/* XOR with checksum */
		if (*ps)
			ps++;
		else
			c++;		/* to hide length */
		en++;
	}

	*en = 0;
} /* Sencompass */

/****************************************************************************/
/* Scompass - compute and check the encrypted password */
Boolean Scompass(char *ps, char *en)
{
	short	ck;
	char	*p,c;

	ck = 0;
	p = ps;
	while (*p)				/* checksum the string */
		ck += *p++;

	c = ck;

	while (*en) {
		if ((((*ps ^ c) | 32) & 127) != *en)	/* XOR with checksum */
			return(false);
		if (*ps)
			ps++;
		else
			c++;			/* increment checksum to hide length */
		en++;
	}

	return(true);
}


/*!
To check a users name and password against a known
registered user, use this method.  If the user name
given was found and the corresponding password was
correct, "true" is returned; otherwise, "false" is
returned.

(2.6)
*/
Boolean
Scheckpass		(char*		us,
				 char*		ps)
{
	FTPUser**		userDataResourceHandle = null;
	char			string[255];
	Boolean			result = false;
	
	
	// the resources name is the same as the user name - look for the resource
	strcpy(string, us);
	StringUtilities_CToPInPlace(string);
	ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
	userDataResourceHandle = (FTPUser**)Get1NamedResource(FTPUSERPREFS_RESTYPE, (StringPtr)string);
	
	// resource loaded OK?
	if (ResError() == noErr)
	{
		HLock((Handle)userDataResourceHandle);
		StringUtilities_PToC((**userDataResourceHandle).password, string);
		
		if (Scompass(ps, string))
		{
			TelnetPrefsAliasID		aliasID = kInvalidTelnetPrefsAliasID;
			FSSpec					defaultDirectory;
			
			
			// find the alias record in the preferences file
			aliasID = Preferences_NewSavedAlias((**userDataResourceHandle).defaultDirAliasID);
			(Boolean)Preferences_ParseAlias(aliasID, &defaultDirectory);
			
			// password matches
			gDefaultDirDirID = FileUtilities_GetDirectoryIDFromFSSpec(&defaultDirectory);
			gDefaultDirVRefNum = defaultDirectory.vRefNum;
			Preferences_DisposeAlias(aliasID);
			result = true;
		}
		
		ReleaseResource((Handle)userDataResourceHandle), userDataResourceHandle = null;
	}
	
	return result;
}// Scheckpass


//
// internal methods
//

/*!
This method determines if the user with the
specified name can change the current working
directory (based on the server preferences for
that user).

If the user cannot be found, or if the user
cannot change directories, "false" is returned;
otherwise, "true" is returned.

(3.0)
*/
static Boolean
canUserChangeDirectories	(char*	inUserName)
{
	FTPUser**		prefsHandle = null;
	Str255			string;
	SInt16			oldResourceFile = CurResFile();
	Boolean			result = false;
	
	
	StringUtilities_CToP(inUserName, string);
	
	ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
	prefsHandle = (FTPUser**)Get1NamedResource(FTPUSERPREFS_RESTYPE, string);
	if (ResError() == noErr)
	{
		HLock((Handle)prefsHandle);
		result = (**prefsHandle).canChangeDirectories;
		ReleaseResource((Handle)prefsHandle);
	}
	
	UseResFile(oldResourceFile); // restore original resource file
	
	return result;
}// canUserChangeDirectories


/*!
Sends a response to a user logged in to the
FTP server.  The given message index is zero-
based, but refers to a string in the string
resource of ID "rStringsFTPMessages".

(2.6)
*/
static void
CRESP		(short		fnum,
			 short		msg_number)
{
	GetIndString(messs, rStringsFTPMessages, msg_number + 1);
	Network_SendFlushData(fnum, &messs[1], (short)messs[0]);
}// CRESP


/*!
To display a status dialog box for FTP
server operation, use this method.  Pass
a string index for a string in the
"rStatusStrings" string list resource
to indicate the message text.

Call IdleControls() repeatedly for the
returned dialog's window to animate the
indeterminate progress bar.  Call
DisposeDialog() when you are done with
the dialog.

(3.0)
*/
static ProgressDialogRef
displayFTPStatusDialog	(const short		inStringIndex)
{
	ProgressDialogRef 		result = null;
	
	
	Console_WriteLine("status dialog setup");
	Cursors_UseWatch();
	
	{
		Str255			text;
		
		
		GetIndString(text, rStringsStatus, inStringIndex);
		result = ProgressDialog_New(text, true/* modal */);
	}
	
	// set up to receive a telnet connection for FTP commands
	rfstate = 0;
	ftpState = 0;
	
	// 3.0
	Console_WriteLine("displaying");
	ProgressDialog_Display(result);
	ProgressDialog_SetProgressIndicator(result, kTelnetProgressIndicatorIndeterminate);
	{
		short		i = 0;
		
		
		// generate a delay sufficient for the user to see what the message says
		for (i = 0; i < 2; i++)
		{
			unsigned long	dummy = 0L;
			
			
			Console_WriteLine("idling");
			IdleControls(ProgressDialog_GetWindow(result));
			Delay(8, &dummy);
		}
		ProgressDialog_SetProgressIndicator(result, kTelnetProgressIndicatorThermometer);
	}
	Console_WriteLine("done status dialog setup");
	
	return result;
}// displayFTPStatusDialog



/* open the FTP data connection to the remote host */
static short
ftpgo	(short		fnum)
{
	short	ftpdata;
	
	
	ftpdata = Network_SocketOpen(portaddr[fnum],fileport[fnum], 20/* FTP data port */,
									15/* seconds of timeout */);
	mainport[ftpdata] = fnum;
	dataport[fnum] = ftpdata;
	return(ftpdata);
}// ftpgo


/*!
To send the FTP server log-in message back
to a remote computer, use this method.

(3.0)
*/
static void
sendLogInMessage	(short			fnum)
{
	Handle					messageData = null;
	Size					actualSize = 0L;
	TelnetPrefsResultCode	error = kTelnetPrefsResultCodeSuccess;
	
	
	// now find any log-in message from the users preferences
	error = Preferences_GetData(kTelnetPreferenceClassFTP, kTelnetPreferenceTag_ServerLogInMessage,
								sizeof(messageData), &messageData, &actualSize);
	if (error == kTelnetPrefsResultCodeSuccess)
	{
		Str255				line;
		unsigned char*		dataPtr = null;
		register SInt32		charactersLeft = GetHandleSize(messageData) / sizeof(unsigned char);
		
		
		HLock(messageData);
		dataPtr = *((unsigned char**)messageData);
		while (charactersLeft > 0)
		{
			// read a line of text from the buffer (or the first 255 characters, whichever comes first)
			StringUtilities_PExtractFirstLine(dataPtr, charactersLeft, line);
			
			// offset to next line, skipping the newline character
			charactersLeft -= (1 + StrLength(line));
			dataPtr += (1 + StrLength(line));
			
			// modify the string prior to sending it
			StringUtilities_PInsert(line, "\p220-"); // standard prefix...
			PLstrcat(line, "\p\015\012"); // CR & LF are required to terminate any line
			
			// send the line (since its a Pascal string, skip the length byte)
			Network_SendFlushData(fnum, line + 1, StrLength(line));
		}
		HUnlock(messageData);
		Memory_DisposeHandle(&messageData);
	}
	
	// send ready message
	CRESP(fnum, 0);
}// sendLogInMessage


static void
setftp	(const Boolean	inShowProgress)
{
	ProgressDialogRef		dialog = null;
	
	
	Console_BeginFunction();
	Console_WriteLine("setftp");
	if (inShowProgress) dialog = displayFTPStatusDialog(siStatusFTPServerStarting);
	
	// 3.0 - force the network connection in this case, because we're a server!
	Console_WriteLine("Network_Connect");
	if (Network_Connect(false/* display warning to user */))
	{
		Console_WriteLine("connected");
		
		// listen for FTP if not listening already
		if (Network_GetFirstPortOfType(kNetworkPortTypeFTPServerSession) == kInvalidNetworkPortRef)
		{
			short		fnum = 0;
			
			
			// non-SLIP listen
			Console_WriteLine("listening");
			fnum = Network_ListenToTCPPortAndGetItsPortNumber(HFTP); // MacTCP listen
			Console_WriteLine("done");
			ftpenable = 1;
	
			if (fnum >= 0)
			{
				// signal that events should be caught
				Console_WriteLine("signaling");
				Network_SetPortType(fnum, kNetworkPortTypeFTPServerSession);
				dataport[fnum] = -1; // -1 = no data needed yet
				mainstate[fnum] = 0;
				datastate[fnum] = 0;
				retnstate[fnum] = 200;
			}
		}
	
		strcpy((char*)myuser, "unknown");
		
		Console_WriteLine("allocating");
		if (mungbuf == null) mungbuf = Memory_NewHandle(1024);
		Console_WriteLine("done");
	}
	
	Console_WriteLine("dialog dispose");
	if (dialog != null) ProgressDialog_Dispose(&dialog);
	Console_EndFunction();
}// setftp


static void
unsetftp	(void)
{
	NetworkPortRef		port = kInvalidNetworkPortRef;
	ProgressDialogRef	dialog = null;
	
	
	Console_BeginFunction();
	Console_WriteLine("unsetftp");
	rfstate = 0;
	ftpState = 0;
	
	port = Network_GetFirstPortOfType(kNetworkPortTypeFTPServerSession);
	if (port != kInvalidNetworkPortRef)
	{
		// close this FTP server connection
		if (dialog == null) dialog = displayFTPStatusDialog(siStatusFTPServerShuttingDown);
		Console_WriteLine("closing FTP server connection");
		Network_CloseConnection(port);
		Network_SetPortType(port, kInvalidNetworkPortType);
		mainstate[port] = 0;
		datastate[port] = 0;
	}
	
	ftpenable = 0;
	Console_WriteLine("freeing");
	if (mungbuf != null) Memory_DisposeHandle(&mungbuf);
	
	Console_WriteLine("dialog dispose");
	if (dialog != null) ProgressDialog_Dispose(&dialog);
	Console_EndFunction();
}// unsetftp