/*###############################################################

	AppleGuideUtilities.c
	
	Contexts Library 1.0
	 1998-2001 by Kevin Grant
	
	This library is free software; you can redistribute it or
	modify it under the terms of the GNU Lesser Public License
	as published by the Free Software Foundation; either version
	2.1 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU Lesser Public License for details.

    You should have received a copy of the GNU Lesser Public
	License along with this library; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// Mac includes
#include <AppleGuide.h>
#if TARGET_API_MAC_CARBON
#	include <AppleHelp.h>
#	include <CFString.h>
#endif
#include <Errors.h>
#include <MacTypes.h>

// library includes
#include "ContextsLib.h"



//
// public methods
//

/*!
To open the first available main
Apple Guide help file in no particular
context, call this method.

If no files can be found, this method
may return an error code of fidNotFound;
otherwise, it returns an AGErr result
(from AGOpen()) coerced to an OSStatus.

For the Carbon version of the Contexts
Library, Apple Guide is not supported and
this routine returns "cfragNoSymbolErr".

(1.0)
*/
OSStatus
AppleGuideUtilities_OpenFirstMainGuide	(void)
{
	OSStatus		result = 0L;
#if TARGET_API_MAC_OS8
	UInt32			availableFiles = 0L;
	
	
	// look at databases: there must be at least one main database!
	availableFiles = AGGetAvailableDBTypes();
	if (availableFiles & kAGDBTypeBitHelp)
	{
		AGRefNum		refNum = 0;
		
		
		result = (OSStatus)AGOpen(null, 0L, null, &refNum);
		if (result != noErr) BEEP;
	}
	else
	{
		result = fidNotFound;
		BEEP;
	}
#else
	result = cfragNoSymbolErr;
#endif
	
	return result;
}// OpenFirstMainGuide


/*!
To open the first available main
Apple Guide help file and immediately
do a Look For search, use this method.

If no files can be found, this method
may return an error code of fidNotFound;
otherwise, it returns an AGErr result
(from AGOpen()) coerced to an OSStatus.

For the Carbon version of the Contexts
Library, Apple Guide is not supported and
this routine returns "cfragNoSymbolErr".

(1.0)
*/
OSStatus
AppleGuideUtilities_OpenFirstMainGuideInSearch	(ConstStr255Param		inSearchString)
{
	OSStatus		result = 0L;
#if TARGET_API_MAC_OS8
	UInt32			availableFiles = 0L;
	
	
	// look at databases: there must be at least one main database!
	availableFiles = AGGetAvailableDBTypes();
	if (availableFiles & kAGDBTypeBitHelp)
	{
		AGRefNum		refNum = 0;
		
		
		result = (OSStatus)AGOpenWithSearch(null, 0L, null, inSearchString, &refNum);
		if (result != noErr) BEEP;
	}
	else
	{
		result = fidNotFound;
		BEEP;
	}
#else
	// in Carbon, this can be equivocated by opening the Help Viewer in a search
	if (API_AVAILABLE(AHSearch))
	{
		CFStringRef		queryString = CFStringCreateWithPascalString(kCFAllocatorDefault, inSearchString, CFStringGetSystemEncoding());
		
		
		result = AHSearch(null/* search all help books */, queryString);
	}
	else result = cfragNoSymbolErr;
#endif
	
	return result;
}// OpenFirstMainGuideInSearch


/*!
To open the first available main
Apple Guide help file and display a
specific view (Index, Look For, etc.),
call this method.

Valid view numbers are "kAGView..."
constants from Apple Guide.

If no files can be found, this method
may return an error code of fidNotFound;
otherwise, it returns an AGErr result
(from AGOpen()) coerced to an OSStatus.

For the Carbon version of the Contexts
Library, Apple Guide is not supported and
this routine returns "cfragNoSymbolErr".

(1.0)
*/
OSStatus
AppleGuideUtilities_OpenFirstMainGuideInView	(const short		inViewType)
{
	OSStatus		result = 0L;
#if TARGET_API_MAC_OS8
	UInt32			availableFiles = 0L;
	
	
	// look at databases: there must be at least one main database!
	availableFiles = AGGetAvailableDBTypes();
	if (availableFiles & kAGDBTypeBitHelp)
	{
		AGRefNum		refNum = 0;
		
		
		result = (OSStatus)AGOpenWithView(null, 0L, null, inViewType, &refNum);
		if (result != noErr) BEEP;
	}
	else
	{
		result = fidNotFound;
		BEEP;
	}
#else
	result = cfragNoSymbolErr;
#endif
	
	return result;
}// OpenFirstMainGuideInView


/*!
To open the first available main
Apple Guide help file and display a
specific sequence instead of the main
topics window, call this method.

Valid sequence numbers are assigned
by Guide Maker when the guide file is
generated.  Run a Names-to-IDs Report
to get a list of sequence IDs for
each sequence in a guide file.

If no files can be found, this method
may return an error code of fidNotFound;
otherwise, it returns an AGErr result
(from AGOpen()) coerced to an OSStatus.

For the Carbon version of the Contexts
Library, Apple Guide is not supported and
this routine returns "cfragNoSymbolErr".

(1.0)
*/
OSStatus
AppleGuideUtilities_OpenFirstMainGuideShowSequence	(const short		inSequenceID)
{
	OSStatus		result = 0L;
#if TARGET_API_MAC_OS8
	UInt32			availableFiles = 0L;
	
	
	// look at databases: there must be at least one main database!
	availableFiles = AGGetAvailableDBTypes();
	if (availableFiles & kAGDBTypeBitHelp)
	{
		AGRefNum		refNum = 0;
		
		
		result = (OSStatus)AGOpenWithSequence(null, 0L, null, inSequenceID, &refNum);
		if (result != noErr) BEEP;
	}
	else
	{
		result = fidNotFound;
		BEEP;
	}
#else
	result = cfragNoSymbolErr;
#endif
	
	return result;
}// OpenFirstMainGuideShowSequence