/*###############################################################

	Clipboard.h
	
	The clipboard management system in Telnet 3.0.
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __CLIPBOARD__
#define __CLIPBOARD__

// Mac includes
#ifndef __MACTYPES__
#	include <MacTypes.h>
#endif
#ifndef __MACWINDOWS__
#	include <MacWindows.h>
#endif

// library includes
#ifndef __INTERFACELIB__
#	include "InterfaceLib.h"
#endif

// Telnet includes
#ifndef __SCREENMANAGER__
#	include "ScreenManager.h"
#endif



// constants
typedef UInt16 TelnetCopyMethod; // do not redefine
enum
{
	kTelnetCopyMethodStandard = 0,		// use this value, or...
	kTelnetCopyMethodTable = 1 << 0,	// ...a bitwise OR of one or more of these mask values
	kTelnetCopyMethodInline = 1 << 1
};

typedef UInt16 TelnetPasteMethod; // do not redefine
enum
{
	kTelnetPasteMethodStandard = 0,
	kTelnetPasteMethodBlock = 1
};



//
// public methods
//

void
	Clipboard_Init						(void);

void
	Clipboard_Done						(void);

OSErr
	Clipboard_AEDescToScrap				(AEDesc const*			inDescPtr);

OSErr
	Clipboard_CreateContentsAEDesc		(AEDesc*				outDescPtr);

ResType
	Clipboard_GetContentType			(void);

UInt32
	Clipboard_GetFlavorCount			(void);

void
	Clipboard_GetViewRectangle			(Rect*					outViewRectPtr);

WindowRef
	Clipboard_GetWindow					(void);

void
	Clipboard_GraphicsToScrap			(short					inDrawingNumber);

void
	Clipboard_Idle						(void);

Boolean
	Clipboard_IsVisible					(void);

void
	Clipboard_SetSuspended				(Boolean				inIsSuspended);

void
	Clipboard_SetVisible				(Boolean				inIsVisible);

void
	Clipboard_TextFromScrap				(void);

void
	Clipboard_TextToScrap				(TelnetScreenID			inScreen,
										 TelnetCopyMethod		inHowToCopy);

void
	Clipboard_TextType					(void);

void
	Clipboard_ToggleVisible				(void);

OSErr
	Clipboard_Update					(GrafPtr				inDrawingPort);	

void
	Clipboard_Zoom						(WindowPartCode			inWindowPart,
										 EventModifiers			inModifiers);

#endif