/*###############################################################

	ColorBox.h
	
	A powerful, Appearance-compliant control that allows the
	user to select a color.  The control looks like a bevel
	button, and has a standard size.  If the button is clicked
	quickly, a standard Color Picker appears immediately and
	the user can select a new color or cancel.  If the user
	clicks and holds on the button, a menu of color choices
	appears.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __COLORBOX__
#define __COLORBOX__

// Mac includes
#include <Controls.h>
#include <Dialogs.h>



/*!
Color Box Color Change Notification Method

This type of routine is invoked whenever a color boxs
content color is changed.  Since it is possible for the
user to modify the content color beyond your control,
this routine ensures you are notified whenever the color
changes (useful for modeless windows that might need to
respond right away to changes).
*/
typedef void	 (*ColorBoxChangeNotifyProcPtr)		(ControlRef			inColorBoxThatWasChanged,
													 RGBColor const*	inNewColor,
													 void const*		inContext);
#define InvokeColorBoxChangeNotifyProc(userRoutine, inColorBoxThatWasChanged, inNewColor, inContext) \
	(*userRoutine)((inColorBoxThatWasChanged), (inNewColor))



/*###############################################################
	CREATING AND DESTROYING COLOR BOXES
###############################################################*/

// WHEN FINISHED WITH YOUR COLOR BOX, CALL THE APPROPRIATE DISPOSAL ROUTINE BELOW
void
	ColorBox_AttachToUserPaneControl		(ControlRef						inoutUserPaneControl,
											 RGBColor const*				inInitialColorOrNull);

void
	ColorBox_AttachToUserPaneDialogItem		(DialogRef						inDialog,
											 DialogItemIndex				inItemIndex,
											 RGBColor const*				inInitialColorOrNull);

ControlRef
	ColorBox_GetNewControl					(WindowRef						inOwningWindow,
											 RGBColor const*				inInitialColorOrNull);

void
	ColorBox_DetachFromUserPaneControl		(ControlRef						inControl);

void
	ColorBox_DetachFromUserPaneDialogItem	(DialogRef						inDialog,
											 DialogItemIndex				inItemIndex);

/*###############################################################
	CHANGING OR OBTAINING COLOR BOX VALUES
###############################################################*/

void
	ColorBox_GetColor						(ControlRef						inControl,
											 RGBColor*						outColorPtr);

void
	ColorBox_SetColor						(ControlRef						inControl,
											 RGBColor const*				inColorPtr);

// CALL IN RESPONSE TO A CONTROL CLICK - DISPLAYS A COLOR PICKER FOR THE USER, AUTOMATICALLY UPDATING THE BUTTONS COLOR
Boolean
	ColorBox_UserSetColor					(ControlRef						inControl);

/*###############################################################
	RECEIVING CHANGE NOTIFICATION
###############################################################*/

void
	ColorBox_SetColorChangeNotifyProc		(ControlRef						inControl,
											 ColorBoxChangeNotifyProcPtr	inNotifyProcPtr,
											 void const*					inContextPtr);

#endif