/*###############################################################

	Configure.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
	This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.
	
	You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// Mac includes
#include <ControlDefinitions.h>
#include <Dialogs.h>
#include <Lists.h>
#include <Menus.h>
#include <Palettes.h>
#include <PLStringFuncs.h>
#include <Resources.h>
#include <StandardFile.h>
#include <TextUtils.h>

// library includes
#include "DataAccessLib.h"
#include "InterfaceLib.h"

// resource includes
#include "ControlResources.h"
#include "DialogResources.h"
#include "GeneralResources.h"
#include "MenuResources.h"
#include "StringResources.h"
#include "TelnetVersion.h"

// MacTelnet includes
#include "Clipboard.h"
#include "ColorBox.h"
#include "configure.proto.h"
#include "Console.h"
#include "DialogItemIndices.h"
#include "DialogUtilities.h"
#include "EventLoop.h"
#include "FileUtilities.h"
#include "MacLook.h"
#include "NetBackground.h"
#include "NetEvents.h"
#include "Preferences.h"
#include "RealScreen.h"
#include "ResourceFileManager.h"
#include "TelnetDescriptors.h"
#include "TelnetMenus.h"
#include "VirtualScreenEmulation.h"



//
// constants
//

enum
{
	// dialog item indices
	iNameNewSessionFavoriteCreateButton = 1,
	iNameNewSessionFavoriteCancelButton = 2,
	iNameNewSessionFavoriteAppleGuideButton = 3,
	iNameNewSessionFavoriteFieldLabelText = 4,
	iNameNewSessionFavoriteNameField = 5,
	TermOKButton			= 1,
	TermCancelButton		= 2,
	TermNameStatText		= 3,	// terminal configuration name label
	TermName				= 4,	// terminal configuration name field
	TermANSI				= 5,	// ANSI escape sequences checkbox
	TermXterm				= 6,	// Xterm escape sequences checkbox
	Termvtwrap				= 7,	// text wrap mode checkbox
	Termarrow				= 8,	// EMACS arrow keys checkbox
	Termeightbit			= 9,	// 8-bit connection checkbox
	Termclearsave			= 10,	// clear screen saves lines checkbox
	TermRemapKeypad			= 11,	// VT220 keypad checkbox
	TermMAT					= 12,	// map page-up, page-down, etc. checkbox
	TermMetaIsCmdCntrol		= 14,	// use control-cmd as meta
	TermMetaIsOption		= 15,	// use option as meta
	TermMetaIsOff			= 16,	// no meta
	TermVT100				= 18,	// VT100 emulation radio button
	TermVT220				= 19,	// VT220 emulation radio button
	TermDumb				= 20,	// dumb terminal radio button
	TermAnswerback			= 22,	// perceived emulator field
	TermFontPopup			= 26,	// font pop-up menu
	TermFontSize			= 27,	// font size field
	TermWidth				= 29,	// terminal width field
	TermHeight				= 31,	// terminal height field
	TermScrollback			= 33,	// scrollback field
	TermNFcolor				= 36,	// normal text color box
	TermBoldFcolor			= 37,	// bold text color box
	TermBFcolor				= 38,	// blinking text color box
	TermNBcolor				= 40,	// normal background color box
	TermBoldBcolor			= 41,	// bold background color box
	TermBBcolor				= 42,	// blinking background color box
	TermHelpButton			= 43	// Apple Guide button
};

//
// variables
//

// temporarily external (see SessionEditorDialog.c)
/*static */MenuHandle		gListItemsDummyMenu = null;

//
// internal methods
//

static pascal Boolean	TerminalModalProc( DialogRef dptr, EventRecord *evt, DialogItemIndex *item);

static pascal Boolean	adjustNameNewSessionFavoriteDialog				(WindowRef				inWindow,
																		 SInt32					inDeltaSizeX,
																		 SInt32					inDeltaSizeY,
																		 SInt32					inData);

static pascal Boolean	nameNewSessionFavoriteDialogEventFilter			(DialogRef				inDialog,
																		 EventRecord*			inoutEventPtr,
																		 DialogItemIndex*		outItemIndex);



//
// public methods
//

/*!
Displays a dialog box with a text field asking the user
to name a new configuration.  If the user cancels, "false"
is returned; otherwise, "true" is returned.

You can optionally specify the title of the window; if
you dont specify a title (by passing "null" for the
"inWindowTitle" parameter), the windows title is blank.

You must specify the initial text for the name field; if
you dont want any default text, pass an empty string
(if you pass "null", this routine does nothing!).  On
output, the name field string is set to an empty string
if the user cancels, or the users chosen name text if
the user does not cancel.

(3.0)
*/
Boolean
Configure_NameNewConfigurationDialogDisplay		(StringPtr		inWindowTitle,
												 Str255			inoutNamePtr)
{
	Boolean		result = false;
	
	
	if (inoutNamePtr != null)
	{
		WindowInfoRef		windowInfo = null;
		DialogRef			dialogPtr = null;
		WindowRef			windowPtr = null;
		DialogItemIndex		itemHit = 0;
		
		
		GetNewDialogWithWindowInfo(kDialogIDNewSessionFavoriteName, &dialogPtr, &windowInfo);
		windowPtr = GetDialogWindow(dialogPtr);
		
		// set the window title properly
		if (inWindowTitle != null) SetWTitle(windowPtr, inWindowTitle);
		else SetWTitle(windowPtr, EMPTY_PSTRING);
		
		// determine if the Help button should be disabled...
		SetUpHelpButtonDialogItem(dialogPtr, iNameNewSessionFavoriteAppleGuideButton);
		
		SetDialogDefaultItem(dialogPtr, iNameNewSessionFavoriteCreateButton);
		SetDialogCancelItem(dialogPtr, iNameNewSessionFavoriteCancelButton);
		
		SetDialogItemControlText(dialogPtr, iNameNewSessionFavoriteNameField, inoutNamePtr);
		
		itemHit = 0;
		
		// adjust the Create and Cancel buttons so they are big enough for their titles
		(UInt16)Localization_AdjustDialogButtons(dialogPtr, true/* OK */, true/* Cancel */);
		Localization_AdjustHelpButtonItem(dialogPtr, iNameNewSessionFavoriteAppleGuideButton);
		Localization_HorizontallyPlaceItems(dialogPtr, iNameNewSessionFavoriteFieldLabelText, iNameNewSessionFavoriteNameField);
		
		TransitionDialogDisplay(dialogPtr);
		
		Cursors_UseArrow();
		
		// set up the Window Info information
		{
			WindowResizeResponderProcPtr	resizeProc = adjustNameNewSessionFavoriteDialog;
			Rect							rect;
			
			
			MacLook_GetWindowRegionBounds(GetDialogWindow(dialogPtr), kWindowContentRgn, &rect);
			
			WindowInfo_SetWindowDescriptor(windowInfo, kTelnetDialogDescriptorNameNewFavorite);
			WindowInfo_SetWindowResizeLimits(windowInfo,
													rect.bottom - rect.top + 1/* minimum height */,
													rect.right - rect.left + 1/* minimum width */,
													rect.bottom - rect.top + 1/* maximum height */,
													rect.right - rect.left + 140/* maximum width - arbitrary */);
			WindowInfo_SetWindowResizeResponderUPP(windowInfo, resizeProc, 0L);
		}
		WindowInfo_SetForDialog(dialogPtr, windowInfo);
		
		{
			ControlRef		fieldControl = null;
			
			
			(OSErr)GetDialogItemAsControl(dialogPtr, iNameNewSessionFavoriteNameField, &fieldControl);
			(OSErr)SetKeyboardFocus(GetDialogWindow(dialogPtr), fieldControl, kControlFocusNextPart);
		}
		
		Menus_EnableEditMenu(); // allows edit commands to show up as enabled
		
		{
			ModalFilterUPP		filterProc = NewModalFilterUPP(nameNewSessionFavoriteDialogEventFilter);
			
			
			do
			{
				ModalDialog(filterProc, &itemHit);
				if (itemHit == iNameNewSessionFavoriteAppleGuideButton)
				{
					Str255		searchString;
					
					
					GetIndString(searchString, rStringsAppleGuideLookup, siAGIndexTermFavorites);
					AppleGuideUtilities_OpenFirstMainGuideInSearch(searchString);
				}
			} while ((itemHit != iNameNewSessionFavoriteCreateButton) && (itemHit != iNameNewSessionFavoriteCancelButton));
		}
		
		if (itemHit == iNameNewSessionFavoriteCreateButton)
		{
			// return the title
			GetDialogItemControlText(dialogPtr, iNameNewSessionFavoriteNameField, inoutNamePtr);
			if (StrLength(inoutNamePtr) <= 0) PLstrcpy(inoutNamePtr, EMPTY_PSTRING);
			TransitionDialogClose(dialogPtr);
			result = true;
		}
		else
		{
			// cancelled
			PLstrcpy(inoutNamePtr, EMPTY_PSTRING);
			TransitionDialogCloseCancel(dialogPtr);
			result = false;
		}
		WindowInfo_Dispose(windowInfo);
		DisposeDialog(dialogPtr), dialogPtr = null;
	}
	
	return result;
}// NameNewConfigurationDialogDisplay


/*!
Use this routine if you need to ask the user
where an application is.  The given prompt and
title only apply if Navigation Services is
available.

(3.0)
*/
Boolean
GetApplicationSpec	(ConstStr255Param		inPrompt,
					 ConstStr255Param		inTitle,
					 FSSpec*				outSpec)
{
	Boolean		good = false;
	
	
	// updated for Navigation Services support
	if (FlagManager_Test(kFlagNavigationServices))
	{
		FileInfo	fi;
		OSType		fileType;
		OSType		types[] = {'APPL'};
		OSStatus	error = noErr;
		short		hit = 0;
		
		
		Alert_ReportOSStatus(error = Navigation_GetFile(inPrompt, inTitle, kApplicationCreatorSignature,
														kTelnetNavPrefKeyChooseTextEditor,
														kNavNoTypePopup |
														kNavDontAutoTranslate |
														kNavSupportPackages |
														kNavDontAddTranslateItems, 1, types,
														EventLoop_HandleNavigationUpdate, outSpec, &fileType, &fi));
		good = (error == noErr);
	}
#if TARGET_API_MAC_OS8
	else
	{
		SFTypeList			types;
		StandardFileReply	sfr;
		
		
		types[0] = 'APPL';
		
		DeactivateFrontmostWindow();
		ThemeSavvyStandardGetFile(inTitle, inPrompt, null, 1, types, &sfr);
		RestoreFrontmostWindow();
		if (sfr.sfGood) Alert_ReportOSStatus(FSMakeFSSpec(sfr.sfFile.vRefNum, sfr.sfFile.parID, sfr.sfFile.name, outSpec));
		good = sfr.sfGood;
	}
#endif
	
	return good;
}// GetApplicationSpec


/*!
Use this routine whenever you need to ask the user to
choose a text editing application.  Now uses
Navigation Services in version 3.0.

(2.6)
*/
Boolean
GetApplicationType	(OSType*	type)
{
	Boolean		good = false;
	FileInfo	fi;
	Str255		prompt,
				title;
	
	
	// updated for Navigation Services support
	GetIndString(prompt, rStringsNavigationServices, siNavPromptSelectTextFile);
	GetIndString(title, rStringsNavigationServices, siNavDialogTitleExampleTextApplication);
	if (FlagManager_Test(kFlagNavigationServices))
	{
		FSSpec		file;
		OSType		fileType;
		OSType		types[] = { 'APPL' };
		OSStatus	error = noErr;
		short		hit = 0;
		
		
		Alert_ReportOSStatus(error = Navigation_GetFile(prompt, title, kApplicationCreatorSignature,
														kTelnetNavPrefKeyChooseTextEditor,
														kNavNoTypePopup |
														kNavDontAutoTranslate |
														kNavSupportPackages |
														kNavDontAddTranslateItems, 1, types,
														EventLoop_HandleNavigationUpdate, &file, &fileType, &fi));
		good = (error == noErr);
	}
#if TARGET_API_MAC_OS8
	else
	{
		SFTypeList			types;
		StandardFileReply	sfr;
		
		
		types[0] = 'APPL';
		
		DeactivateFrontmostWindow();
		ThemeSavvyStandardGetFile(title, prompt, (FileFilterUPP)null, 1, types, &sfr);
		RestoreFrontmostWindow();
		good = sfr.sfGood;
		if (good) HGetFInfo(sfr.sfFile.vRefNum, sfr.sfFile.parID, sfr.sfFile.name, (FInfo*)&fi);
	}
#endif
	
	if (good) BlockMoveData(&fi.fileCreator, type, sizeof(OSType)); // copy the application creator type
		
	return(good);
}// GetApplicationType


pascal Boolean TerminalModalProc	(DialogRef			inDialog,
									 EventRecord*		inoutEventPtr,
									 DialogItemIndex*	outItemIndex)
{
	Boolean		result = pascal_false,
				isButtonAndShouldFlash = false; // simplifies handling of multiple key-to-button maps
	
	
	// first, make sure help balloons show up properly for the cursor location, if Balloon Help is on
	if (IsPopUpHelpEnabled()) DisplayBalloonForDialogItemUnderMouse(inDialog, inoutEventPtr->where, kDialogBalloonsIDTerminalEditor);
	
	if (!result) result = (StandardDialogEventFilter(inDialog, inoutEventPtr, outItemIndex));
	else if (isButtonAndShouldFlash) FlashButton(inDialog, *outItemIndex);
	
	return result;
}


Boolean EditTerminal(StringPtr inoutPreferencesResourceName)
{
	DialogRef		dptr;
	ModalFilterUPP	filterUPP = null;
	short			ditem, resourceID;
	long			scratchlong;
	ResType			scratchResType;
	Boolean			isNewDataHandle, wasInAliasText;
	TerminalPrefs**	TermPrefsHdl;
	TerminalPrefs*	TermPrefsPtr;
	Str255			scratchPstring;
	
	
	dptr = GetNewDialog(kDialogIDTerminalEditor, null, kInFront);
	SetDialogDefaultItem(dptr, DLOGOk);
	SetDialogCancelItem(dptr, DLOGCancel);
	SetUpHelpButtonDialogItem(dptr, TermHelpButton);
	SetDialogTracksCursor(dptr, true);
	
	ditem = 3;
	
	if (inoutPreferencesResourceName[0] != 0) {
		isNewDataHandle = false;
		ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
		TermPrefsHdl = (TerminalPrefs**)Get1NamedResource(TERMINALPREFS_RESTYPE, inoutPreferencesResourceName);
		// TEMPORARY - MUST CHECK ResError() HERE!!!!!!
		{
			Str255		defaultName;
			
			
			StringUtilities_CToP(TERMINALPREFS_DEFAULTNAME, defaultName);
			if (EqualString(inoutPreferencesResourceName, defaultName, false/* case */, false/* diacritics */))
			{
				HideDialogItem(dptr, TermName);
				SetDialogItemControlText(dptr, TermNameStatText, defaultName);
			}
		}
	}
	else { //make sure we have a unique name
		TermPrefsHdl = Preferences_GetDefaultTerminal();
		isNewDataHandle = true;
		GetIndString(inoutPreferencesResourceName, rStringsMiscellaneous, siNewTerminalDefaultName);
		Menus_GetUniqueItemText(gListItemsDummyMenu, inoutPreferencesResourceName);
	}
	HLock((Handle) TermPrefsHdl);
	TermPrefsPtr = *TermPrefsHdl;
	SetDialogItemControlText(dptr, TermName, inoutPreferencesResourceName);
	SetDialogItemValue(dptr, TermANSI, TermPrefsPtr->usesANSIColors);
	SetDialogItemValue(dptr, TermXterm, TermPrefsPtr->usesXTermSequences);
	SetDialogItemValue(dptr, Termvtwrap, TermPrefsPtr->usesVTWrap);
	SetDialogItemValue(dptr, TermMetaIsCmdCntrol, (TermPrefsPtr->metaKey == kEMACSMetaKeyControlCommand));
	SetDialogItemValue(dptr, TermMetaIsOption, (TermPrefsPtr->metaKey == kEMACSMetaKeyOption));
	DeactivateDialogItem(dptr, TermMetaIsOption); // 3.0 - this hack is not supported
	SetDialogItemValue(dptr, TermMetaIsOff, (TermPrefsPtr->metaKey == kEMACSMetaKeyOff));
	SetDialogItemValue(dptr, Termarrow, TermPrefsPtr->usesEMACSArrows);
	SetDialogItemValue(dptr, TermMAT, TermPrefsPtr->mapsPageJumpKeys);
	SetDialogItemValue(dptr, Termeightbit, TermPrefsPtr->usesEightBits);
	SetDialogItemValue(dptr, Termclearsave, TermPrefsPtr->savesOnClear);
	SetDialogItemValue(dptr, TermVT100, (TermPrefsPtr->emulation == kTerminalEmulationTypeVT100));
	SetDialogItemValue(dptr, TermVT220, (TermPrefsPtr->emulation == kTerminalEmulationTypeVT220));
	HideDialogItem(dptr, TermDumb); // NOT AVAILABLE IN CLASSIC
	SetDialogItemValue(dptr, TermRemapKeypad, TermPrefsPtr->remapKeypad);
	scratchlong = (long)(TermPrefsPtr->columnCount);
	NumToString(scratchlong, scratchPstring);
	SetDialogItemControlText(dptr, TermWidth, scratchPstring);
	scratchlong = (short)(TermPrefsPtr->rowCount);
	NumToString(scratchlong, scratchPstring);
	SetDialogItemControlText(dptr, TermHeight, scratchPstring);
	scratchlong = TermPrefsPtr->fontSize;
	NumToString(scratchlong, scratchPstring);
	SetDialogItemControlText(dptr, TermFontSize, scratchPstring);
	scratchlong = TermPrefsPtr->scrollbackBufferSize;
	NumToString(scratchlong, scratchPstring);
	SetDialogItemControlText(dptr, TermScrollback, scratchPstring);
	SetDialogItemControlText(dptr, TermAnswerback, TermPrefsPtr->answerBackMessage);
	
	// pre-select the appropriate font in the menu
	{
		MenuHandle			fontMenu = null;
		ControlRef			fontMenuButton = null;
		Size				actualSize = 0L;
		register SInt16		i = 0;
		
		
		GetDialogItemAsControl(dptr, TermFontPopup, &fontMenuButton);
		(OSStatus)GetControlData(fontMenuButton, kControlMenuPart, kControlPopupButtonMenuHandleTag, sizeof(fontMenu), (Ptr)&fontMenu, &actualSize);
		for (i = CountMenuItems(fontMenu); i > 0; i--)
		{
			GetMenuItemText(fontMenu, i, scratchPstring);
			if (EqualString(scratchPstring, (TermPrefsPtr->normalFont), true/* case sensitive */, false/* diacritics sensitive */))
			{
				SetControlValue(fontMenuButton, i);
			}
		}
	}
	
	// set up certain text fields to only allow numerical input
	{
		ControlRef				field = null;
		ControlKeyFilterUPP		upp = NewControlKeyFilterUPP(NumericalLimiter);
		
		
		GetDialogItemAsControl(dptr, TermFontSize, &field);
		(OSStatus)SetControlData(field, kControlNoPart, kControlKeyFilterTag, sizeof(ControlKeyFilterUPP), (Ptr)&upp);
		GetDialogItemAsControl(dptr, TermWidth, &field);
		(OSStatus)SetControlData(field, kControlNoPart, kControlKeyFilterTag, sizeof(ControlKeyFilterUPP), (Ptr)&upp);
		GetDialogItemAsControl(dptr, TermHeight, &field);
		(OSStatus)SetControlData(field, kControlNoPart, kControlKeyFilterTag, sizeof(ControlKeyFilterUPP), (Ptr)&upp);
		GetDialogItemAsControl(dptr, TermScrollback, &field);
		(OSStatus)SetControlData(field, kControlNoPart, kControlKeyFilterTag, sizeof(ControlKeyFilterUPP), (Ptr)&upp);
	}
	
	if (TermPrefsPtr->emulation != kTerminalEmulationTypeVT220)
	{
		HideDialogItem(dptr,TermRemapKeypad);
		HideDialogItem(dptr,TermMAT);
	}
	
	ColorBox_AttachToUserPaneDialogItem(dptr, TermNFcolor, &TermPrefsPtr->foregroundNormalColor);
	ColorBox_AttachToUserPaneDialogItem(dptr, TermNBcolor, &TermPrefsPtr->backgroundNormalColor);
	ColorBox_AttachToUserPaneDialogItem(dptr, TermBoldFcolor, &TermPrefsPtr->foregroundBoldColor);
	ColorBox_AttachToUserPaneDialogItem(dptr, TermBoldBcolor, &TermPrefsPtr->backgroundBoldColor);
	ColorBox_AttachToUserPaneDialogItem(dptr, TermBFcolor, &TermPrefsPtr->foregroundBlinkingColor);
	ColorBox_AttachToUserPaneDialogItem(dptr, TermBBcolor, &TermPrefsPtr->backgroundBlinkingColor);
	
	SelectDialogItemText(dptr, TermName, 0, 32767);
	
	// adjust the OK and Cancel buttons so they are big enough for their titles
	(UInt16)Localization_AdjustDialogButtons(dptr, true/* OK */, true/* Cancel */);
	Localization_AdjustHelpButtonItem(dptr, TermHelpButton);
	
	wasInAliasText = false;
	
	TransitionDialogDisplay(dptr);
	
	filterUPP = NewModalFilterUPP(TerminalModalProc);
	do
	{
		ModalDialog(filterUPP, &ditem);
		if ((wasInAliasText)&&(ditem != TermName))
		{
			wasInAliasText = false;
			GetTEText(dptr, TermName,scratchPstring);
			if (EqualString(scratchPstring,inoutPreferencesResourceName,true,false))
				goto okayTermName;
			if (!Menus_IsItemUnique(gListItemsDummyMenu, scratchPstring)) // we already have that name
			{
				InterfaceLibAlertRef	box = null;
				Str255					dialogText,
										helpText,
										configurationText;
				
				
				GetIndString(configurationText, rStringsConfigurationData, siConfigurationDataATerminal);
				GetIndString(dialogText, rStringsGeneralMessages, siDuplicateConfiguration);
				StringUtilities_PMetaSetDefaults();
				StringUtilities_PBuild(dialogText, configurationText, EMPTY_PSTRING, EMPTY_PSTRING, EMPTY_PSTRING);
				GetIndString(configurationText, rStringsConfigurationData, siConfigurationDataTerminals);
				GetIndString(helpText, rStringsGeneralMessages, siDuplicateConfigurationHelpText);
				StringUtilities_PMetaSetDefaults();
				StringUtilities_PBuild(helpText, configurationText, EMPTY_PSTRING, EMPTY_PSTRING, EMPTY_PSTRING);
				
				box = Alert_New();
				Alert_SetHelpButton(box, false);
				Alert_SetParamsFor(box, kAlert_StyleOK);
				Alert_SetText(box, dialogText, helpText);
				Alert_SetType(box, kAlertStopAlert);
				Alert_Display(box);
				Alert_Dispose(&box);
				
				SetPortWindowPort(GetDialogWindow(dptr)); // back to old dialog
				SetDialogItemControlText(dptr,TermName,inoutPreferencesResourceName);// set the string back to the original
				SelectDialogItemText(dptr,TermName,0,32767);
				ditem = TermName; // so that if they hit ok or cancel, it doesn't kill the window
			}
			else
				goto okayTermName;
		}
		else
		{
				okayTermName:
			switch (ditem) {
				case	TermHelpButton:
					{
						Str255		searchString;
						
						
						GetIndString(searchString, rStringsAppleGuideLookup, siAGIndexTermFavorites);
						AppleGuideUtilities_OpenFirstMainGuideInSearch(searchString);
					}
					break;
				
				case	TermANSI:
				case	TermXterm:
				case	Termvtwrap:
				case	Termarrow:
				case	TermMAT:
				case	Termeightbit:
				case	Termclearsave:
				case 	TermRemapKeypad:
					FlipCheckBox(dptr, ditem);
					break;
				
				case	TermVT100:
					SetDialogItemValue(dptr, TermVT100, 1);
					SetDialogItemValue(dptr, TermVT220, 0);
					SetDialogItemValue(dptr, TermDumb, 0);
					SetDialogItemControlText(dptr, TermAnswerback, "\pvt100");
					DrawOneDialogItem(dptr, TermAnswerback);
					HideDialogItem(dptr,TermRemapKeypad);
					HideDialogItem(dptr,TermMAT);
					break;
	
				case	TermVT220:
					SetDialogItemValue(dptr, TermVT100, 0);
					SetDialogItemValue(dptr, TermVT220, 1);
					SetDialogItemValue(dptr, TermDumb, 0);
					SetDialogItemControlText(dptr, TermAnswerback, "\pvt220");
					DrawOneDialogItem(dptr, TermAnswerback);
					ShowDialogItem(dptr,TermRemapKeypad);
					ShowDialogItem(dptr,TermMAT);
					break;
	
				case	TermDumb:
					SetDialogItemValue(dptr, TermVT100, 0);
					SetDialogItemValue(dptr, TermVT220, 0);
					SetDialogItemValue(dptr, TermDumb, 1);
					SetDialogItemControlText(dptr, TermAnswerback, "\p");
					DrawOneDialogItem(dptr, TermAnswerback);
					ShowDialogItem(dptr,TermRemapKeypad);
					HideDialogItem(dptr,TermMAT);
					break;
				
				case	TermMetaIsCmdCntrol:
					SetDialogItemValue(dptr, TermMetaIsOption, 0);
					SetDialogItemValue(dptr, TermMetaIsOff, 0);
					SetDialogItemValue(dptr, TermMetaIsCmdCntrol, 1);
					break;
				case	TermMetaIsOption:
					SetDialogItemValue(dptr, TermMetaIsOff, 0);
					SetDialogItemValue(dptr, TermMetaIsCmdCntrol, 0);
					SetDialogItemValue(dptr, TermMetaIsOption, 1);
					break;
				case	TermMetaIsOff:
					SetDialogItemValue(dptr, TermMetaIsCmdCntrol, 0);
					SetDialogItemValue(dptr, TermMetaIsOption, 0);
					SetDialogItemValue(dptr, TermMetaIsOff, 1);
					break;
				
				case TermNFcolor:
				case TermNBcolor:
				case TermBoldFcolor:
				case TermBoldBcolor:
				case TermBFcolor:
				case TermBBcolor:
					{
						ControlRef		control = null;
						
						
						(OSStatus)GetDialogItemAsControl(dptr, ditem, &control);
						ColorBox_UserSetColor(control);
					}
					break;
				
				case 	TermName:
					wasInAliasText = true;
				default:
					break;
				
				} // switch
			}//else not wasInAliasText
	} while ((ditem != DLOGOk) && (ditem != DLOGCancel));
	DisposeModalFilterUPP(filterUPP), filterUPP = null;
	
	// get font info
	{
		MenuHandle		fontMenu = null;
		ControlRef		fontMenuButton = null;
		Size			actualSize = 0L;
		
		
		GetDialogItemAsControl(dptr, TermFontPopup, &fontMenuButton);
		(OSStatus)GetControlData(fontMenuButton, kControlMenuPart, kControlPopupButtonMenuHandleTag,
									sizeof(fontMenu), (Ptr)&fontMenu, &actualSize);
		GetMenuItemText(fontMenu, GetControlValue(fontMenuButton), scratchPstring);
	}
	
	if (ditem == 2) {
		if (isNewDataHandle) Memory_DisposeHandle((Handle*)&TermPrefsHdl);
		else ReleaseResource((Handle)TermPrefsHdl);
		
		TransitionDialogCloseCancel(dptr);
		DisposeDialog(dptr), dptr = null;
		return(false);			// No changes should be made.
		}
	
	
	// update color preferences
	{
		ControlRef		control = null;
		
		
		(OSStatus)GetDialogItemAsControl(dptr, TermNFcolor, &control);
		ColorBox_GetColor(control, &TermPrefsPtr->foregroundNormalColor);
		(OSStatus)GetDialogItemAsControl(dptr, TermNBcolor, &control);
		ColorBox_GetColor(control, &TermPrefsPtr->backgroundNormalColor);
		(OSStatus)GetDialogItemAsControl(dptr, TermBoldFcolor, &control);
		ColorBox_GetColor(control, &TermPrefsPtr->foregroundBoldColor);
		(OSStatus)GetDialogItemAsControl(dptr, TermBoldBcolor, &control);
		ColorBox_GetColor(control, &TermPrefsPtr->backgroundBoldColor);
		(OSStatus)GetDialogItemAsControl(dptr, TermBFcolor, &control);
		ColorBox_GetColor(control, &TermPrefsPtr->foregroundBlinkingColor);
		(OSStatus)GetDialogItemAsControl(dptr, TermBBcolor, &control);
		ColorBox_GetColor(control, &TermPrefsPtr->backgroundBlinkingColor);
	}
		
	if (PLstrlen(scratchPstring) > 63) scratchPstring[0] = 63;
	BlockMoveData(scratchPstring, (TermPrefsPtr->normalFont), scratchPstring[0]+1);
	
	TermPrefsPtr->usesANSIColors = GetDialogItemValue(dptr, TermANSI);
	TermPrefsPtr->usesXTermSequences = GetDialogItemValue(dptr, TermXterm);
	TermPrefsPtr->usesVTWrap = GetDialogItemValue(dptr, Termvtwrap);

	if (GetDialogItemValue(dptr, TermMetaIsCmdCntrol)) TermPrefsPtr->metaKey = kEMACSMetaKeyControlCommand;
	else if (GetDialogItemValue(dptr, TermMetaIsOption)) TermPrefsPtr->metaKey = kEMACSMetaKeyOption;
	else TermPrefsPtr->metaKey = kEMACSMetaKeyOff;
	
	TermPrefsPtr->usesEMACSArrows = GetDialogItemValue(dptr, Termarrow);
	TermPrefsPtr->mapsPageJumpKeys = GetDialogItemValue(dptr, TermMAT);
	TermPrefsPtr->usesEightBits = GetDialogItemValue(dptr, Termeightbit);
	TermPrefsPtr->savesOnClear = GetDialogItemValue(dptr, Termclearsave);
	TermPrefsPtr->remapKeypad = GetDialogItemValue(dptr, TermRemapKeypad);
	if (GetDialogItemValue(dptr, TermVT220) != kControlRadioButtonUncheckedValue) TermPrefsPtr->emulation = kTerminalEmulationTypeVT220;
	else TermPrefsPtr->emulation = kTerminalEmulationTypeVT100;
	
	GetTEText(dptr, TermWidth, scratchPstring);
	StringToNum(scratchPstring, &scratchlong);
	if (scratchlong > 133) scratchlong = 133;
	if (scratchlong < 10) scratchlong = 10;
	TermPrefsPtr->columnCount = (short) scratchlong;
	
	GetTEText(dptr, TermHeight, scratchPstring);
	StringToNum(scratchPstring, &scratchlong);
	if (scratchlong > 80) scratchlong = 80;
	if (scratchlong < 10) scratchlong = 10;
	TermPrefsPtr->rowCount = (short) scratchlong;

	GetTEText(dptr, TermFontSize, scratchPstring);
	StringToNum(scratchPstring, &scratchlong);
	if (scratchlong > 24) scratchlong = 24;
	if (scratchlong < 4) scratchlong = 4;
	TermPrefsPtr->fontSize = (short) scratchlong;

	GetTEText(dptr, TermScrollback, scratchPstring);
	StringToNum(scratchPstring, &scratchlong);
	if (scratchlong > 50000) scratchlong = 50000;
	if (scratchlong < 24) scratchlong = 24;
	TermPrefsPtr->scrollbackBufferSize = (short) scratchlong;

	GetTEText(dptr, TermAnswerback, scratchPstring);
	if (PLstrlen(scratchPstring) > 63) scratchPstring[0] = 63;
	BlockMoveData(scratchPstring, TermPrefsPtr->answerBackMessage, scratchPstring[0]+1);
	
	GetTEText(dptr, TermName, inoutPreferencesResourceName);
	
	if (isNewDataHandle) {
		ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
		resourceID = Unique1ID(TERMINALPREFS_RESTYPE);
		AddResource((Handle)TermPrefsHdl, TERMINALPREFS_RESTYPE, resourceID, inoutPreferencesResourceName);
		ResourceFileManager_UpdateResFile(kTelnetResourceFilePreferences);
		ReleaseResource((Handle)TermPrefsHdl);
		}
	else {
		ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
		GetResInfo((Handle)TermPrefsHdl, &resourceID, &scratchResType, (StringPtr)&scratchPstring);
		SetResInfo((Handle)TermPrefsHdl, resourceID, inoutPreferencesResourceName);
		ChangedResource((Handle)TermPrefsHdl);
		if (ResError() == noErr) WriteResource((Handle)TermPrefsHdl);
		ReleaseResource((Handle)TermPrefsHdl);
		}
	
	ColorBox_DetachFromUserPaneDialogItem(dptr, TermNFcolor);
	ColorBox_DetachFromUserPaneDialogItem(dptr, TermNBcolor);
	ColorBox_DetachFromUserPaneDialogItem(dptr, TermBoldFcolor);
	ColorBox_DetachFromUserPaneDialogItem(dptr, TermBoldBcolor);
	ColorBox_DetachFromUserPaneDialogItem(dptr, TermBFcolor);
	ColorBox_DetachFromUserPaneDialogItem(dptr, TermBBcolor);
	
	TransitionDialogClose(dptr);
	DisposeDialog(dptr), dptr = null;
	return(true);			// A resource has changed or been added.	
}// EditTerminal


/*!
Displays a dialog box for editing the information
in the specified FTP registered user preferences
record.  If the settings are okayed by the user,
"true" is returned; otherwise, "false" is returned.

(2.6)
*/
Boolean
EditFTPUser		(StringPtr		inoutPreferencesResourceName)
{
	DialogRef			editFTPUserDialog = null;
	ModalFilterUPP		upp = null;
	register SInt16		i = 0;
	TelnetPrefsAliasID	aliasID = kInvalidTelnetPrefsAliasID;
	DialogItemIndex		itemIndex = 0;
	FSSpec				defaultDirectory;
	FTPUser**			userDataHandle = null;
	FTPUser*			userDataPtr = null;
	Str255				string;
	Boolean				isNewDataHandle = false;
	Boolean				result = false;
	
	
	// create the dialog box
	editFTPUserDialog = GetNewDialog(FTPUserDLOG, null, kInFront);
	SetDialogDefaultItem(editFTPUserDialog, DLOGOk);
	SetDialogCancelItem(editFTPUserDialog, DLOGCancel);
	SetDialogTracksCursor(editFTPUserDialog, true);
	
	// the existence of a name indicates whether the resource exists - create if necessary, otherwise, read it
	if (inoutPreferencesResourceName[0] != 0)
	{
		// read the existing data structure
		isNewDataHandle = false;
		ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
		userDataHandle = (FTPUser**)Get1NamedResource(FTPUSERPREFS_RESTYPE, inoutPreferencesResourceName);
		HLock((Handle)userDataHandle);
		userDataPtr = *userDataHandle;
		SetDialogItemValue(editFTPUserDialog, FTPUcanchangeCWD, userDataPtr->canChangeDirectories);
		SetDialogItemControlText(editFTPUserDialog, FTPUusername, inoutPreferencesResourceName);
		for (i = 8, string[0] = 8; i > 0; i--) string[i] = '';
 		SetDialogItemControlText(editFTPUserDialog, FTPUpassword, string);
	}
	else
	{
		// resource doesnt exist - create and initialize a new data structure
		isNewDataHandle = true;
		userDataHandle = (FTPUser**)Memory_NewHandle(sizeof(FTPUser));
		HLock((Handle)userDataHandle);
		userDataPtr = *userDataHandle;
		SetDialogItemValue(editFTPUserDialog, FTPUcanchangeCWD, 0);
	}
	
	// the users default directory alias record is stored as a separate resource - find it or create it
	{
		OSErr		error = noErr;
		
		
		// initialize this structure, in case FSMakeFSSpec() fails
		PLstrcpy(defaultDirectory.name, EMPTY_PSTRING);
		defaultDirectory.vRefNum = -1;
		defaultDirectory.parID = fsRtDirID;
		
		// 3.0 - read an alias record for the default directory
		if (Preferences_IsAliasStored(userDataPtr->defaultDirAliasID))
		{
			// a directory preference exists - read it and resolve it (re-saving changes if necessary)
			aliasID = Preferences_NewSavedAlias(userDataPtr->defaultDirAliasID);
			(Boolean)Preferences_ParseAlias(aliasID, &defaultDirectory);
		}
		else
		{
			// no directory preference exists - create a new one with defaults
			error = FSMakeFSSpec(-1, fsRtDirID, null, &defaultDirectory);
			if (error == noErr)
			{
				// create an 'alis' resource in the preferences file, and remember its ID
				aliasID = Preferences_NewAlias(&defaultDirectory);
			}
			else
			{
				aliasID = kInvalidTelnetPrefsAliasID;
				Console_WriteValue("FTP user alias creation error", error);
			}
		}
		if (aliasID == kInvalidTelnetPrefsAliasID)
		{
			BEEP;
			Console_WriteLine("FTP user alias could not be created");
		}
		FileUtilities_GetPathnameFromFSSpec(&defaultDirectory, string, true/* is directory */);
	}
	
	// update the text area to show the default pathname
	SetDialogItemCenterTruncatedText(editFTPUserDialog, FTPUDfltDirDsply, string, kThemeSmallSystemFont);
	
	// select the log-in name text field
	SelectDialogItemText(editFTPUserDialog, FTPUusername, 0, 32767);
	
	// put the buttons where theyre supposed to be, with the correct sizes
	(UInt16)Localization_AdjustDialogButtons(editFTPUserDialog, true/* OK */, true/* Cancel */);
	
	// display the dialog and handle events
	upp = NewModalFilterUPP(StandardDialogEventFilter);
	TransitionDialogDisplay(editFTPUserDialog);
	do
	{
		ModalDialog(upp, &itemIndex);
		switch (itemIndex)
		{
			case FTPUcanchangeCWD:
				FlipCheckBox(editFTPUserDialog, itemIndex);
				break;
			
			case FTPUDfltDirButton:
				{
					SInt16		vRefNum = 0;
					SInt32		dirID = 0L;
					
					
					if (SelectDirectory(&vRefNum, &dirID))
					{
						// SelectDirectory() returns true if the user actually picked something
						if (FSMakeFSSpec(vRefNum, dirID, EMPTY_PSTRING, &defaultDirectory) == noErr)
						{
							FileUtilities_GetPathnameFromFSSpec(&defaultDirectory, string, true/* is directory */);
							SetDialogItemCenterTruncatedText(editFTPUserDialog, FTPUDfltDirDsply, string, kThemeSmallSystemFont);
						}
						else BEEP;
					}
				}
				break;
				
			default:
				break;
			
		}
	}
	while ((itemIndex != DLOGOk) && (itemIndex != DLOGCancel));
	DisposeModalFilterUPP(upp), upp = null;
	
	if (itemIndex == DLOGCancel)
	{
		if (isNewDataHandle) Memory_DisposeHandle(&((Handle)userDataHandle));
		else ReleaseResource((Handle)userDataHandle);
		
		TransitionDialogCloseCancel(editFTPUserDialog);
		result = false;
	}
	else
	{
		// read options
		userDataPtr->canChangeDirectories = GetDialogItemValue(editFTPUserDialog, FTPUcanchangeCWD);
		
		// read the user name, and the default directory, and use the
		// user name as the resource name of the default directory alias
		GetTEText(editFTPUserDialog, FTPUusername, inoutPreferencesResourceName);
		//FileUtilities_GetDirectoryName(vRefNum, fsRtDirID, userDataPtr->defaultDirVolName);
		// 3.0 - create an alias record for the default directory
		{
			OSErr		error = noErr;
			
			
			if (error == noErr)
			{
				Preferences_ChangeAlias(aliasID, &defaultDirectory);
				Preferences_SaveAlias(aliasID, inoutPreferencesResourceName);
				userDataPtr->defaultDirAliasID = aliasID;
			}
			Alert_ReportOSErr(error);
		}
		
		// read the password
		GetTEText(editFTPUserDialog, FTPUpassword, string);
		if (string[0] != '')
		{
			Str255		encryptedPassword;
			
			
			// encrypt the new (or possibly modified) password
			StringUtilities_PToCInPlace(string);
			Sencompass(STATIC_CAST(string, char*), STATIC_CAST(encryptedPassword, char*));
			StringUtilities_CToPInPlace(STATIC_CAST(encryptedPassword, char*));
			BlockMove(encryptedPassword, userDataPtr->password, StrLength(encryptedPassword) + 1);
		}
		
		// update or create the required resources
		{
			SInt16		resourceID = 0;
			
			
			ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
			if (isNewDataHandle)
			{
				resourceID = Unique1ID(FTPUSERPREFS_RESTYPE);
				AddResource((Handle)userDataHandle, FTPUSERPREFS_RESTYPE, resourceID, inoutPreferencesResourceName);
				ResourceFileManager_UpdateResFile(kTelnetResourceFilePreferences);
			}
			else
			{
				ResType		unusedResType = '????';
				
				
				GetResInfo((Handle)userDataHandle, &resourceID, &unusedResType, (StringPtr)&string);
				SetResInfo((Handle)userDataHandle, resourceID, inoutPreferencesResourceName);
				ChangedResource((Handle)userDataHandle);
				if (ResError() == noErr) WriteResource((Handle)userDataHandle);
			}
			ReleaseResource((Handle)userDataHandle);
		}
		
		TransitionDialogClose(editFTPUserDialog);
		result = true;
	}
	Preferences_DisposeAlias(aliasID);
	DisposeDialog(editFTPUserDialog), editFTPUserDialog = null;
	
	return result;
}// EditFTPUser


//
// internal methods
//
#pragma mark -

/*!
This method moves the buttons and resizes the text field
in response to a resize of the Name New Session Favorite
movable modal dialog box.

(3.0)
*/
static pascal Boolean
adjustNameNewSessionFavoriteDialog	(WindowRef	inWindow,
									 SInt32		inDeltaSizeX,
									 SInt32		inDeltaSizeY,
									 SInt32		UNUSED_ARGUMENT(inData))
{
	Boolean		result = pascal_true;
	
	
	DialogAdjust_BeginDialogItemAdjustment(GetDialogFromWindow(inWindow));
	{
		if (inDeltaSizeX)
		{
			// controls which are moved horizontally
			if (Localization_IsLeftToRight())
			{
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iNameNewSessionFavoriteCreateButton, inDeltaSizeX);
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iNameNewSessionFavoriteCancelButton, inDeltaSizeX);
			}
			else
			{
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iNameNewSessionFavoriteAppleGuideButton, inDeltaSizeX);
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iNameNewSessionFavoriteFieldLabelText, inDeltaSizeX);
			}
			
			// controls which are resized horizontally
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iNameNewSessionFavoriteNameField, inDeltaSizeX);
		}
	}
	DialogAdjust_EndAdjustment(inDeltaSizeX, inDeltaSizeY); // moves and resizes controls properly
	
	return result;
}// adjustNameNewSessionFavoriteDialog


/*!
Handles display of help balloons in the
Name New Session Favorite dialog box.

(3.0)
*/
static pascal Boolean
nameNewSessionFavoriteDialogEventFilter		(DialogRef			inDialog,
											 EventRecord*		inoutEventPtr,
											 DialogItemIndex*	outItemIndex)
{
	Boolean		result = pascal_false;
	
	
	// first, make sure help balloons show up properly for the cursor location, if Balloon Help is on
	if (IsPopUpHelpEnabled()) DisplayBalloonForDialogItemUnderMouse(inDialog, inoutEventPtr->where, kDialogBalloonsIDNewSessionFavoriteName);
	
	if (!result) result = (StandardDialogEventFilter(inDialog, inoutEventPtr, outItemIndex));
	
	return result;
}// nameNewSessionFavoriteDialogEventFilter