/*###############################################################

	Connections.h
	
	Utility library for use with the network kernel.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __CONNECTIONS__
#define __CONNECTIONS__

// Mac includes
#include <MacTCP.h>
#include <MacTypes.h>
#include <Menus.h>
#include <OpenTransportProviders.h>

// resource includes
#include "MenuResources.h"

// MacTelnet includes
#include "ConnectionData.h"
#include "PreferencesData.h"
#include "ScreenManager.h"



// constants
#define TYPE_I_ON_CLOSE		(1 << 0)
#define CAPTURE_DATA		(1 << 1)
#define MGET_STATE			(1 << 2)
#define MPUT_STATE			(1 << 3)

// ASCII character set defines
enum
{
	ESC = 0x1B,	// Esc
	BS = 0x08,	// Backspace
	DEL = 0x7F,	// Delete
	CR = 0x0D,	// Carriage return
	LF = 0x0A	// Line feed
};

// important virtual key codes (see page 2-43 of "Inside Macintosh: Macintosh Toolbox Essentials")
enum
{
	BScode = 0x33,		// virtual (i.e. the same on any keyboard) key code of the delete key
	KPlowest = 0x41		// lowest virtual key code of any keypad key
};

// emulation type: what does a window contain?
typedef SInt16 TelnetEmulationType;
enum
{
	VTEKTYPE = 1,	// VT screen (standard terminal)
	TEKTYPE = 2,	// TEK screen
	DUMBTYPE = 3,	// dumb terminal (FTP connections use this)
	RASTYPE = 4		// ICR screen
};

// miscellaneous constants
enum
{
	kConnectionTimeMaximumForNoWarningClose = 15	// in seconds; connections this brief can be disconnected
													//    without having to OK the (annoying) warning message
};



// types
struct TelnetConnectionParameterBlock
{
	struct
	{
		StringPtr			host;			// domain name, IP address, or Session Favorite name
		TerminalPrefs**		terminal;		// terminal preferences to use
		UInt32				timeout;		// number of seconds to wait before giving up on opening
		SInt16				port;			// port number (default is 23 for telnet, 21 for FTP)
		FourCharCode		method;			// the appropriate Apple Event enumeration (see "Terminology.h").
	} connection;
	
	struct
	{
		Str255				title;		// initial window title
	} window;
};
#ifndef __cplusplus
typedef struct TelnetConnectionParameterBlock	TelnetConnectionParameterBlock;
#endif
typedef struct TelnetConnectionParameterBlock*	TelnetConnectionParameterBlockPtr;



struct ConnectionSetup
{
	TerminalPrefs**		terminal;
	SessionPrefs**		session;
	Str255				windowName;
	UInt16				ftpState;
	UInt32				openTimeout;
	ProxyServerRef		proxy;
};
#ifndef __cplusplus
typedef struct ConnectionSetup		ConnectionSetup;
#endif
typedef ConnectionSetup*			ConnectionSetupPtr;
typedef ConnectionSetupPtr*			ConnectionSetupHandle;



//
// public methods
//

void
	Connections_CompleteOpening				(TelnetScreenID						inWhichScreen,
											 UInt32								inIPAddress,
											 OSStatus							inDNRError,
											 char*								inConnectionName);

Boolean
	Connections_CreateOne					(ConnectionSetup**					inConnectionInitParams);

void
	Connections_CreateOneParameterized		(TelnetConnectionParameterBlockPtr	inParameters);

void
	Connections_Disconnect					(ConnectionDataPtr					inConnectionDataPtr);

UInt16
	Connections_GetActiveCount				(void);

UInt16
	Connections_GetCount					(void);

ConnectionSetup**
	Connections_GetDefault					(void);

UInt16
	Connections_GetLiveCount				(void);

void
	Connections_MakeActive					(ConnectionDataPtr					inConnectionDataPtr);

void
	Connections_OpenFromMenuItem			(MenuHandle							inMenuHandle,
											 MenuItemIndex const				inItemIndex);

void
	Connections_OpenFromURL					(char*								inHost,
											 char*								inPortString,
											 char*								inUserName,
											 char*								inUserPassword);

OSErr
	Connections_OpenFromURLString			(char const*						inURLStringData,
											 Size								inSize);

Boolean
	Connections_ProcessHostNameString		(StringPtr							inoutString,
											 SInt16*							outPort);

void
	Connections_SetCount					(UInt16 const						inNewCount);

#endif