/*###############################################################

	Console.h
	
	One of the benefits of having a built-in terminal emulation
	system in an application like MacTelnet is the fact that
	its really easy to create a simple UNIX-like console output
	window that displays retainable debugging messages.  Use it!
	
	The great thing about this module is that its completely
	self-contained and protected by a pre-processor directive.
	A single directive in "Init.c" determines whether this
	module gets initialized, and if its not initialized, EVERY
	call to any write-string method in this module has no effect!
	This means that the source code does not have to be filled
	with pre-processor directives for the purpose of excluding
	debugging printouts, because none of those printouts will do
	anything if this module is turned off.
	
	The console also supports Apple Events (although, theyre
	hidden).  If you send an Apple Event, from MacTelnets event
	class, of type "kTelnetHiddenEventIDConsoleWriteLine", with
	a string as a direct object, MacTelnet automatically writes
	the string to the console by calling the Console_WriteLine()
	method.  This allows external modules (such as shared
	libraries) to be debugged as well, since they can also
	write to the console by sending events to MacTelnet!
	
	WARNING:	There are certain times when you CANNOT use the
				debugging console.  For example, you cant send
				debug statements to the console within any of
				the application initialization code that appears
				prior to the Console_Init() call.  You also may
				not send debug statements within certain parts
				of the various terminal screen modules - any
				code that might get executed in order to show
				your debugging statement in the console would
				cause a recursive loop.  Similarly, some of the
				high-level, generic drawing code may indirectly
				apply to the debugging consoles window, so you
				may cause recursion by sending debugging lines
				to the console within update-event handlers.
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __DEBUGGINGCONSOLE__
#define __DEBUGGINGCONSOLE__

// Mac includes
#ifndef __MACTYPES__
#	include <MacTypes.h>
#endif
#ifndef __MACWINDOWS__
#	include <MacWindows.h>
#endif



/*###############################################################
	INITIALIZING AND FINISHING WITH THE CONSOLE
###############################################################*/

// CALL THIS ROUTINE ONCE, BEFORE ANY OTHER CONSOLE ROUTINE, AFTER TERMINAL SCREEN MODULES ARE INITIALIZED
void
	Console_Init					(void);

// CALL THIS ROUTINE AFTER YOU ARE PERMANENTLY DONE WITH THE CONSOLE
void
	Console_Done					(void);

/*###############################################################
	MANIPULATING THE CONSOLE WINDOW
###############################################################*/

WindowRef
	Console_GetWindow				(void);

Boolean
	Console_IsVisible				(void);

void
	Console_SetVisible				(Boolean		inIsVisible);

/*###############################################################
	WRITING MESSAGES TO THE CONSOLE
###############################################################*/

void
	Console_WriteHorizontalRule		(void);

void
	Console_WriteLine				(char*			inString);

void
	Console_WriteTimeStamp			(char*			inLabel);

void
	Console_WriteValue				(char*			inLabel,
									 SInt32			inValue);

void
	Console_WriteValueFile			(char*			inLabel,
									 FSSpec const*	inFSSpecPtr);

void
	Console_WriteValueFourChars		(char*			inLabel,
									 FourCharCode	inValue);

void
	Console_WriteValuePString		(char*			inLabel,
									 StringPtr		inValue);

/*###############################################################
	HANDLING INDENTING OF OUTPUT
###############################################################*/

void
	Console_BeginFunction			(void);

void
	Console_EndFunction				(void);

#endif