/*
**************************************************************************
	ControlResources.h
	
	 1998-2000 by Kevin Grant
	This file contains constants used to describe 'CNTL' resources.
	
	IMPORTANT:	The constants in this file are generally cumulative
				within a section, as defined by the major bases at the
				top of the file.
				
				The base for the next dialog box is always set as one
				greater than the last control ID for the previous one
				listed in its section.  (Upon careful examination of
				this file, the pattern should be obvious.)  If you add
				any controls to a group, make sure you not only add one
				to the offset of the previous control in the group, but
				that you also change the offset for the base of the
				following group.
**************************************************************************
*/

// defines resource ID range for 'CNTL' resources
#define	DIALOG_CONTROL_BASE_ID			200										/* controls used in dialog windows */
#define	WINDOW_CONTROL_BASE_ID			(DIALOG_CONTROL_BASE_ID + 300)			/* controls used in non-dialog windows */
#define	REZ_DITL_CONTROL_BASE_ID		(WINDOW_CONTROL_BASE_ID + 100)			/* avoid changing this base if at all possible! */
#define	REUSABLE_CONTROL_BASE_ID		(REZ_DITL_CONTROL_BASE_ID + 300)		/* controls used in dozens of places */



/*####################### REUSABLE CONTROLS #######################*/

/*
controls from " TelnetReusableControls.r"
*/
#define kIDAnchorUserPane					REUSABLE_CONTROL_BASE_ID
#define kIDAsynchronousArrows				(REUSABLE_CONTROL_BASE_ID + 1)
#define kIDCancelButton						(REUSABLE_CONTROL_BASE_ID + 2)
#define kIDCheckBox							(REUSABLE_CONTROL_BASE_ID + 3)
#define kIDColorBox							(REUSABLE_CONTROL_BASE_ID + 4)
// 5 is unused
#define kIDDisclosureTriangle				(REUSABLE_CONTROL_BASE_ID + 6)
#define kIDDisclosureTriangleDown			(REUSABLE_CONTROL_BASE_ID + 7)
#define kIDDisclosureTriangleDownReversed	(REUSABLE_CONTROL_BASE_ID + 8)
#define kIDDisclosureTriangleReversed		(REUSABLE_CONTROL_BASE_ID + 9)
#define kIDEditableText						(REUSABLE_CONTROL_BASE_ID + 10)
#define kIDHelpTextGroupBox					(REUSABLE_CONTROL_BASE_ID + 11)
#define kIDLittleArrows						(REUSABLE_CONTROL_BASE_ID + 12)
#define kIDOKButton							(REUSABLE_CONTROL_BASE_ID + 13)
#define kIDPanelUserPane					(REUSABLE_CONTROL_BASE_ID + 14)
#define kIDRadioButton						(REUSABLE_CONTROL_BASE_ID + 15)
#define kIDSeparator						(REUSABLE_CONTROL_BASE_ID + 16)
#define kIDStaticText						(REUSABLE_CONTROL_BASE_ID + 17)
#define kIDStatusIcon						(REUSABLE_CONTROL_BASE_ID + 18)
#define kIDTabUserPane						(REUSABLE_CONTROL_BASE_ID + 19)



/*####################### REZ 'DITL' DEPENDENT CONTROLS #######################*/

// These definitions are temporary.  Some of the old Telnet 2.6 dialogs are
// not defined by Rez, but rather by Derezed-ResEdit data.  As a result, any
// reference to modern Appearance controls requires the data to be changed in
// ResEdit (because Im too lazy to rewrite the soon-to-die older dialogs in
// Rez input format).  A separate control ID base has been created for these
// special Appearance controls, because control IDs of other windows tend
// to fluctuate easily (as more controls are added to one dialog, the IDs of
// all dialogs that follow it in this file are offset), which would require
// me to open ResEdit and alter references to 'CNTL' IDs to match.  Since this
// dependency exists, do NOT change the REZ_DITL_CONTROL_BASE_ID unless you
// ALSO change the appropriate ResEdit 'DITL's, Derez them and replace the
// code in the appropriate '.r' file for Telnet; otherwise, Telnet will crash
// or display the wrong control when it opens a dialog that uses a Rez-'DITL'
// control.  Got that?  No?  In a nutshell, dont touch anything, and this mess
// will be gone in the next few releases or so.

/*
*******************************************
	CONFIGURATION LIST DIALOG BOX
*******************************************
*/

#define CONFIGURATIONS_CONTROL_BASE_ID							REZ_DITL_CONTROL_BASE_ID

#define kIDConfigurationsList									CONFIGURATIONS_CONTROL_BASE_ID

/*
*******************************************
	COLOR BOX CONTROLS IN OLDER DIALOGS
*******************************************
*/

#define OLDCOLORBOX_CONTROL_BASE_ID								(CONFIGURATIONS_CONTROL_BASE_ID + 1)

#define kIDRezDITLColorBox										OLDCOLORBOX_CONTROL_BASE_ID

/*
*******************************************
	SESSION FAVORITE EDITOR DIALOG BOX
*******************************************
*/

#define SESSIONEDITOR_CONTROL_BASE_ID							(OLDCOLORBOX_CONTROL_BASE_ID + 1)

#define	kIDSessionEditorTerminalsPopUpMenu						SESSIONEDITOR_CONTROL_BASE_ID
#define	kIDSessionEditorTranslationTablePopUpMenu				(SESSIONEDITOR_CONTROL_BASE_ID + 1)

/*
*******************************************
	TERMINAL EDITOR DIALOG BOX
*******************************************
*/

#define TERMINALEDITOR_CONTROL_BASE_ID							(SESSIONEDITOR_CONTROL_BASE_ID + 2)

#define	kIDTerminalEditorFontPopUpMenu							TERMINALEDITOR_CONTROL_BASE_ID



/*####################### WINDOW CONTROLS #######################*/

/*
*******************************************
	TERMINAL WINDOW HEADER CONTROLS
*******************************************
*/

#define TERMINALWINDOWHEADER_CONTROL_BASE_ID					WINDOW_CONTROL_BASE_ID

#define kIDTerminalWindowHeaderPlacard							TERMINALWINDOWHEADER_CONTROL_BASE_ID
#define kIDTerminalWindowHeaderDisplayGroupBox					(TERMINALWINDOWHEADER_CONTROL_BASE_ID + 2)

/*
*******************************************
	TERMINAL WINDOW CONTENT CONTROLS
*******************************************
*/

#define TERMINALWINDOW_CONTROL_BASE_ID							(TERMINALWINDOWHEADER_CONTROL_BASE_ID + 3)

#define kIDTerminalWindowBackgroundPicture						TERMINALWINDOW_CONTROL_BASE_ID
#define kIDTerminalWindowScreenPane								(TERMINALWINDOW_CONTROL_BASE_ID + 1)

/*
*******************************************
	CLIPBOARD CONTROLS
*******************************************
*/

#define CLIPBOARDWINDOW_CONTROL_BASE_ID							(TERMINALWINDOW_CONTROL_BASE_ID + 2)

#define kIDClipboardWindowContentUserPane						CLIPBOARDWINDOW_CONTROL_BASE_ID

/*
*******************************************
	INSPECTOR WINDOW
*******************************************
*/

#define INSPECTOR_CONTROL_BASE_ID								(CLIPBOARDWINDOW_CONTROL_BASE_ID + 1)

#define	kIDInspectorPlacard										INSPECTOR_CONTROL_BASE_ID

/*
*******************************************
	SETUP ASSISTANT CONTROLS
*******************************************
*/

#define SETUPASSISTANT_CONTROL_BASE_ID							(INSPECTOR_CONTROL_BASE_ID + 1)

#define kIDSetupAssistantBackBevelButton						SETUPASSISTANT_CONTROL_BASE_ID
#define kIDSetupAssistantPageBevelButton						(SETUPASSISTANT_CONTROL_BASE_ID + 1)
#define kIDSetupAssistantForwardBevelButton						(SETUPASSISTANT_CONTROL_BASE_ID + 2)
#define kIDSetupAssistantHeaderContentUserPane					(SETUPASSISTANT_CONTROL_BASE_ID + 3)
#define kIDSetupAssistantFooterPlacard							(SETUPASSISTANT_CONTROL_BASE_ID + 4)

/*
*******************************************
	KEYPAD CONTROLS
*******************************************
*/

#define KEYPAD_CONTROL_BASE_ID									(SETUPASSISTANT_CONTROL_BASE_ID + 5)

#define kIDKeypadContentUserPane								KEYPAD_CONTROL_BASE_ID



/*####################### DIALOG BOX CONTROLS #######################*/

/*
*******************************************
	PREFERENCES DIALOG BOX
*******************************************
*/

#define PREFS_CONTROL_BASE_ID									DIALOG_CONTROL_BASE_ID

#define	kIDPrefBevelButtonPlacard								PREFS_CONTROL_BASE_ID
#define	kIDPrefPanelPlacard										(PREFS_CONTROL_BASE_ID + 1)
#define	kIDPrefGenericBevelButton								(PREFS_CONTROL_BASE_ID + 2)
#define	kIDPrefFileTypeField									(PREFS_CONTROL_BASE_ID + 3)
// General
#define	kIDPrefGeneralTabs										(PREFS_CONTROL_BASE_ID + 4)
#define	kIDPrefGeneralCursorGroupBox							(PREFS_CONTROL_BASE_ID + 5)
#define	kIDPrefGeneralCursorShapeBlockBevelButton				(PREFS_CONTROL_BASE_ID + 6)
#define	kIDPrefGeneralCursorShapeVerticalBarBevelButton			(PREFS_CONTROL_BASE_ID + 7)
#define	kIDPrefGeneralCursorShapeUnderlineBevelButton			(PREFS_CONTROL_BASE_ID + 8)
#define	kIDPrefGeneralCursorShapeThickVerticalBarBevelButton	(PREFS_CONTROL_BASE_ID + 9)
#define	kIDPrefGeneralCursorShapeThickUnderlineBevelButton		(PREFS_CONTROL_BASE_ID + 10)
#define	kIDPrefGeneralTimeoutsGroupBox							(PREFS_CONTROL_BASE_ID + 11)
#define	kIDPrefGeneralCopyTableGroupBox							(PREFS_CONTROL_BASE_ID + 12)
#define	kIDPrefGeneralCaptureFileGroupBox						(PREFS_CONTROL_BASE_ID + 13)
#define	kIDPrefGeneralAutoCalcANSIColorsButton					(PREFS_CONTROL_BASE_ID + 14)
#define	kIDPrefGeneralBackgroundNotificationGroupBox			(PREFS_CONTROL_BASE_ID + 15)
#define	kIDPrefGeneralBackgroundNotificationBeepsCheckBox		(PREFS_CONTROL_BASE_ID + 16)
#define	kIDPrefGeneralNotificationVisualBellCheckBox			(PREFS_CONTROL_BASE_ID + 17)
// Configurations
#define kIDPrefConfigurationsTabs								(PREFS_CONTROL_BASE_ID + 18)
#define kIDPrefConfigurationsListBox							(PREFS_CONTROL_BASE_ID + 19)
#define kIDPrefConfigurationsNewButton							(PREFS_CONTROL_BASE_ID + 20)
#define kIDPrefConfigurationsNewGroupButton						(PREFS_CONTROL_BASE_ID + 21)
#define kIDPrefConfigurationsRemoveButton						(PREFS_CONTROL_BASE_ID + 22)
#define kIDPrefConfigurationsDuplicateButton					(PREFS_CONTROL_BASE_ID + 23)
#define kIDPrefConfigurationsEditButton							(PREFS_CONTROL_BASE_ID + 24)
// Macros
#define kIDPrefMacrosTabs										(PREFS_CONTROL_BASE_ID + 25)
#define kIDPrefUseSetAboveButton								(PREFS_CONTROL_BASE_ID + 26)
// File Transfers
#define	kIDPrefFileTransfersTabs								(PREFS_CONTROL_BASE_ID + 27)
#define	kIDPrefFileTransfersTextFilesGroupBox					(PREFS_CONTROL_BASE_ID + 28)
#define	kIDPrefFileTransfersTextFilesFromAppButton				(PREFS_CONTROL_BASE_ID + 29)
#define	kIDPrefFileTransfersBinaryFilesGroupBox					(PREFS_CONTROL_BASE_ID + 30)
#define	kIDPrefFileTransfersBinaryFilesExampleButton			(PREFS_CONTROL_BASE_ID + 31)
#define	kIDPrefFileTransfersServerUserSetupButton				(PREFS_CONTROL_BASE_ID + 32)
#define	kIDPrefFileTransfersServerEditLogInMessageButton		(PREFS_CONTROL_BASE_ID + 33)
// overall
#define kIDPrefCategoryListBox									(PREFS_CONTROL_BASE_ID + 34)

/*
*******************************************
	SMALL, GENERIC PROGRESS DIALOG BOX
*******************************************
*/

#define GENERICPROGRESS_CONTROL_BASE_ID							(PREFS_CONTROL_BASE_ID + 35)

#define kIDGenericProgressBar									GENERICPROGRESS_CONTROL_BASE_ID
#define kIDGenericProgressText									(GENERICPROGRESS_CONTROL_BASE_ID + 1)

/*
*******************************************
	NEW SESSIONS DIALOG BOX
*******************************************
*/

#define NEWSESSIONS_CONTROL_BASE_ID								(GENERICPROGRESS_CONTROL_BASE_ID + 2)

#define	kIDNewSessionsConnectButton								NEWSESSIONS_CONTROL_BASE_ID
#define	kIDNewSessionsAddButton									(NEWSESSIONS_CONTROL_BASE_ID + 1)
#define kIDNewSessionsFavoritesPopUpMenuBevelButton				(NEWSESSIONS_CONTROL_BASE_ID + 2)
#define kIDNewSessionsRecentPopUpMenuBevelButton				(NEWSESSIONS_CONTROL_BASE_ID + 3)
#define	kIDNewSessionsNewConnectionPrimaryGroup					(NEWSESSIONS_CONTROL_BASE_ID + 4)
#define kIDNewSessionsHostNameField								(NEWSESSIONS_CONTROL_BASE_ID + 5)
#define	kIDNewSessionsPortNumberField							(NEWSESSIONS_CONTROL_BASE_ID + 6)
#define	kIDNewSessionsSessionInfoDisclosurePane					(NEWSESSIONS_CONTROL_BASE_ID + 7)
#define	kIDNewSessionsSessionInfoSeparator						(NEWSESSIONS_CONTROL_BASE_ID + 8)
#define kIDNewSessionsWindowTitleField							(NEWSESSIONS_CONTROL_BASE_ID + 9)
#define kIDNewSessionsTerminalsPopUpMenu						(NEWSESSIONS_CONTROL_BASE_ID + 10)
#define kIDNewSessionsProxiesPopUpMenu							(NEWSESSIONS_CONTROL_BASE_ID + 11)
#define kIDNewSessionsConnectionsToOpenList						(NEWSESSIONS_CONTROL_BASE_ID + 12)
#define	kIDNewSessionsRemoveButton								(NEWSESSIONS_CONTROL_BASE_ID + 13)
#define	kIDNewSessionsBottomSeparator							(NEWSESSIONS_CONTROL_BASE_ID + 14)

/*
*******************************************
	SET WINDOW TITLE DIALOG BOX
*******************************************
*/

#define SETWINDOWTITLE_CONTROL_BASE_ID							(NEWSESSIONS_CONTROL_BASE_ID + 15)

#define	kIDSetWindowTitleSetTitleButton							SETWINDOWTITLE_CONTROL_BASE_ID
#define kIDSetWindowTitleWindowTitleField						(SETWINDOWTITLE_CONTROL_BASE_ID + 1)

/*
*******************************************
	MACRO SETUP DIALOG BOX
*******************************************
*/

#define MACROSETUP_CONTROL_BASE_ID								(SETWINDOWTITLE_CONTROL_BASE_ID + 2)

#define	kIDMacroSetupCommandZeroButton							MACROSETUP_CONTROL_BASE_ID
#define	kIDMacroSetupCommandOneButton							(MACROSETUP_CONTROL_BASE_ID + 1)
#define	kIDMacroSetupCommandTwoButton							(MACROSETUP_CONTROL_BASE_ID + 2)
#define	kIDMacroSetupCommandThreeButton							(MACROSETUP_CONTROL_BASE_ID + 3)
#define	kIDMacroSetupCommandFourButton							(MACROSETUP_CONTROL_BASE_ID + 4)
#define	kIDMacroSetupCommandFiveButton							(MACROSETUP_CONTROL_BASE_ID + 5)
#define	kIDMacroSetupCommandSixButton							(MACROSETUP_CONTROL_BASE_ID + 6)
#define	kIDMacroSetupCommandSevenButton							(MACROSETUP_CONTROL_BASE_ID + 7)
#define	kIDMacroSetupCommandEightButton							(MACROSETUP_CONTROL_BASE_ID + 8)
#define	kIDMacroSetupCommandNineButton							(MACROSETUP_CONTROL_BASE_ID + 9)
#define	kIDMacroSetupCommandXField								(MACROSETUP_CONTROL_BASE_ID + 10)
#define	kIDMacroSetupImportButton								(MACROSETUP_CONTROL_BASE_ID + 11)
#define	kIDMacroSetupExportButton								(MACROSETUP_CONTROL_BASE_ID + 12)
#define	kIDMacroSetupMacroTextEquivalentsPrimaryGroup			(MACROSETUP_CONTROL_BASE_ID + 13)
#define	kIDMacroSetupExportHelpText								(MACROSETUP_CONTROL_BASE_ID + 14)
#define	kIDMacroSetupMacroTextEquivalentsDivider				(MACROSETUP_CONTROL_BASE_ID + 15)
#define	kIDMacroSetupActiveSetPrimaryGroup						(MACROSETUP_CONTROL_BASE_ID + 16)
#define	kIDMacroSetupActiveSetCurrentButton						(MACROSETUP_CONTROL_BASE_ID + 17)
#define	kIDMacroSetupActiveSet1BevelButton						(MACROSETUP_CONTROL_BASE_ID + 18)
#define	kIDMacroSetupActiveSet2BevelButton						(MACROSETUP_CONTROL_BASE_ID + 19)
#define	kIDMacroSetupActiveSet3BevelButton						(MACROSETUP_CONTROL_BASE_ID + 20)
#define	kIDMacroSetupActiveSet4BevelButton						(MACROSETUP_CONTROL_BASE_ID + 21)
#define	kIDMacroSetupActiveSet5BevelButton						(MACROSETUP_CONTROL_BASE_ID + 22)
#define	kIDMacroSetupMacroKeysPrimaryGroup						(MACROSETUP_CONTROL_BASE_ID + 23)

/*
*******************************************
	ABOUT BOX
*******************************************
*/

#define ABOUT_BOX_CONTROL_BASE_ID								(MACROSETUP_CONTROL_BASE_ID + 24)

#define kIDAboutPicture											ABOUT_BOX_CONTROL_BASE_ID
#define kIDAboutScrollingTextUserPane							(ABOUT_BOX_CONTROL_BASE_ID + 1)
#define kIDAboutNCSATextUserPane								(ABOUT_BOX_CONTROL_BASE_ID + 2)

/*
*******************************************
	FORMAT DIALOG BOX
*******************************************
*/

#define FORMAT_CONTROL_BASE_ID									(ABOUT_BOX_CONTROL_BASE_ID + 3)

#define kIDFormatEditMenuPrimaryGroup							FORMAT_CONTROL_BASE_ID
#define kIDFormatEditMenuFontItemsPane							(FORMAT_CONTROL_BASE_ID + 1)
#define kIDFormatEditFontPopUpMenu								(FORMAT_CONTROL_BASE_ID + 2)
#define kIDFormatEditMenuFontSizeItemsPane						(FORMAT_CONTROL_BASE_ID + 3)
#define kIDFormatEditSizeLabel									(FORMAT_CONTROL_BASE_ID + 4)
#define kIDFormatEditSizeField									(FORMAT_CONTROL_BASE_ID + 5)
#define kIDFormatEditSizePopUpMenu								(FORMAT_CONTROL_BASE_ID + 6)
#define kIDFormatEditMenuForeColorItemsPane						(FORMAT_CONTROL_BASE_ID + 7)
#define kIDFormatEditForegroundColorLabel						(FORMAT_CONTROL_BASE_ID + 8)
#define kIDFormatEditMenuBackColorItemsPane						(FORMAT_CONTROL_BASE_ID + 9)
#define kIDFormatEditBackgroundColorLabel						(FORMAT_CONTROL_BASE_ID + 10)

/*
*******************************************
	TERMINAL SCREEN SIZE DIALOG BOX
*******************************************
*/

#define SCREENSIZE_CONTROL_BASE_ID								(FORMAT_CONTROL_BASE_ID + 11)

#define kIDScreenSizeEditDimensionField							SCREENSIZE_CONTROL_BASE_ID

/*
*******************************************
	NEW SESSION FAVORITE DIALOG BOX
*******************************************
*/

#define NEWSESSIONFAVORITE_CONTROL_BASE_ID						(SCREENSIZE_CONTROL_BASE_ID + 1)

#define	kIDNewSessionFavoriteCreateButton						NEWSESSIONFAVORITE_CONTROL_BASE_ID
#define kIDNewSessionFavoriteNameField							(NEWSESSIONFAVORITE_CONTROL_BASE_ID + 1)

/*
*******************************************
	PROXY SERVER EDITOR DIALOG BOX
*******************************************
*/

#define PROXYSERVER_CONTROL_BASE_ID								(NEWSESSIONFAVORITE_CONTROL_BASE_ID + 2)

#define	kIDProxyServerNameField									PROXYSERVER_CONTROL_BASE_ID
#define kIDProxyServerTabs										(PROXYSERVER_CONTROL_BASE_ID + 1)
#define kIDProxyServerInfoTabPane								(PROXYSERVER_CONTROL_BASE_ID + 2)
#define kIDProxyServerInfoHostField								(PROXYSERVER_CONTROL_BASE_ID + 3)
#define kIDProxyServerInfoPortField								(PROXYSERVER_CONTROL_BASE_ID + 4)
#define kIDProxyServerInfoProtocolMenu							(PROXYSERVER_CONTROL_BASE_ID + 5)
#define kIDProxyServerInfoTipSecondaryGroupBox					(PROXYSERVER_CONTROL_BASE_ID + 6)
#define kIDProxyServerInfoTipNoteIcon							(PROXYSERVER_CONTROL_BASE_ID + 7)
#define kIDProxyServerAuthenticationTabPane						(PROXYSERVER_CONTROL_BASE_ID + 8)
#define kIDProxyServerAuthenticationNameField					(PROXYSERVER_CONTROL_BASE_ID + 9)
#define kIDProxyServerAuthenticationSecurityIcon				(PROXYSERVER_CONTROL_BASE_ID + 10)
#define kIDProxyServerAuthenticationPasswordSetButton			(PROXYSERVER_CONTROL_BASE_ID + 11)
#define kIDProxyServerDirectDomainsTabPane						(PROXYSERVER_CONTROL_BASE_ID + 12)
#define kIDProxyServerDirectDomainsList							(PROXYSERVER_CONTROL_BASE_ID + 13)
#define kIDProxyServerDirectDomainsHostField					(PROXYSERVER_CONTROL_BASE_ID + 14)
#define kIDProxyServerDirectDomainsNewDomainButton				(PROXYSERVER_CONTROL_BASE_ID + 15)
#define kIDProxyServerDirectDomainsRemoveButton					(PROXYSERVER_CONTROL_BASE_ID + 16)