/*###############################################################

	CoolStuff.h
	
	Lists utility routines and Easter eggs.  Almost all of
	these routines require Apple Game Sprockets to be installed,
	but every routine verifies that the appropriate Sprocket is
	installed before using it.
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __MACROS__
#define __MACROS__

// Mac includes
#ifndef __DRAWSPROCKET
#	include <DrawSprocket.h>
#endif
#ifndef __FILES__
#	include <Files.h>
#endif



/*###############################################################
	INITIALIZING AND FINISHING WITH COOL STUFF
###############################################################*/

// CALL THIS ROUTINE ONCE, BEFORE ANY OTHER COOL STUFF ROUTINE
OSStatus
	CoolStuff_Init								(void);

// CALL THIS ROUTINE AFTER YOU ARE PERMANENTLY DONE WITH COOL STUFF
void
	CoolStuff_Done								(void);

/*###############################################################
	APPLE GAME SPROCKETS INTERFACES
###############################################################*/

DSpContextReference
	CoolStuff_GetDefaultDrawSprocketContext		(void);

/*###############################################################
	GRAPHICS DEVICE GAMMA UTILITIES
###############################################################*/

void
	CoolStuff_GammaBlack						(DSpContextReference	inContext);

void
	CoolStuff_GammaControlledFade				(DSpContextReference	inContext,
												 RGBColor*				inCurrentColorPtr,
												 const UInt8			inStartingPercentage,
												 const UInt8			inEndingPercentage,
												 const SInt8			inGammaChangePercentage,
												 const UInt8			inMinimumTicksBetweenGammaChanges,
												 const Boolean			inYieldToThreads);

void
	CoolStuff_GammaControlledFadeFromBlack		(DSpContextReference	inContext,
												 const UInt8			inGammaIncreasePercentage,
												 const UInt8			inMinimumTicksBetweenGammaChanges,
												 const Boolean			inYieldToThreads);

void
	CoolStuff_GammaControlledFadeToBlack		(DSpContextReference	inContext,
												 const UInt8			inGammaDecreasePercentage,
												 const UInt8			inMinimumTicksBetweenGammaChanges,
												 const Boolean			inYieldToThreads);

void
	CoolStuff_GammaControlledFadeFromWhite		(DSpContextReference	inContext,
												 const UInt8			inGammaIncreasePercentage,
												 const UInt8			inMinimumTicksBetweenGammaChanges,
												 const Boolean			inYieldToThreads);

void
	CoolStuff_GammaControlledFadeToWhite		(DSpContextReference	inContext,
												 const UInt8			inGammaDecreasePercentage,
												 const UInt8			inMinimumTicksBetweenGammaChanges,
												 const Boolean			inYieldToThreads);

void
	CoolStuff_GammaNormal						(DSpContextReference	inContext);

void
	CoolStuff_GammaSet							(DSpContextReference	inContext,
												 RGBColor*				inCurrentColorPtr,
												 const UInt8			inGammaPercentage);

/*###############################################################
	MOVING STARS
###############################################################*/

void
	CoolStuff_MovingStarsInit					(ControlRef				inUserPaneControl);

void
	CoolStuff_MovingStarsDone					(ControlRef				inUserPaneControl);

void
	CoolStuff_MovingStarsGetCenter				(ControlRef				inUserPaneControl,
												 Point*					inoutCenterPtr);

SInt16
	CoolStuff_MovingStarsGetSpeed				(ControlRef				inUserPaneControl);

void
	CoolStuff_MovingStarsSetCenter				(ControlRef				inUserPaneControl,
												 Point					inCenter);

void
	CoolStuff_MovingStarsSetSpeed				(ControlRef				inUserPaneControl,
												 SInt16					inAverageZPlaneDelta);

/*###############################################################
	PULSING BUTTONS
###############################################################*/

void
	CoolStuff_PulseDefaultButton				(WindowRef				inWindow);

/*###############################################################
	QUAKE QUAD DAMAGE EGG
###############################################################*/

void
	CoolStuff_QuadDamageActivity				(void);

void
	CoolStuff_QuadDamageBegin					(void);

void
	CoolStuff_QuadDamageEnd						(void);

void
	CoolStuff_QuadDamageGetPulseColor			(RGBColor*				outColorPtr);

void
	CoolStuff_QuadDamageToggle					(void);

#endif