/*###############################################################

	DNR.h
	
	Domain name resolver utility library for use with the
	network kernel; completely rewritten in 3.0 to be native to
	Open Transport instead of MacTCP.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __TELNETDNR__
#define __TELNETDNR__

// Mac includes
#include <MacTypes.h>
#include <OpenTransportProviders.h>

// MacTelnet includes
#include "ProxyServer.h"
#include "ScreenManager.h"



// universal procedure pointers
/*!
DNR Done Notification Method

This routine is invoked when the finished domain name
lookup signal is received in "NetEvents.c".  The
interrupt-level notification is handled internally, so
your DNR notification routine does not run at
interrupt time.

You *must* call DNR_Dispose() on the provided data
pointer, which will render it invalid and free up
the memory allocated by the DNR module.
*/
typedef void (*DNRDoneNotificationProcPtr)	(SInt32 const					inData,
											 struct DomainNameLookupData*	inResolverPtr);
#define InvokeDNRDoneNotificationProc(userRoutine, inData, inResolverPtr) \
			(*(userRoutine))((inData), (inResolverPtr))



// types
struct DomainNameLookupData
{
	SInt32							data;					// additional information for the completion method
	OSStatus						error;					// on completion, any error that occurred
	InetDomainName					hostName;				// the domain name that is being resolved
	InetHostInfo*					hostInfoPtr;			// on completion, IP addresses for the domain name
	InetSvcRef						servicesRef;			// reference to Open Transport Internet Provider
	DNRDoneNotificationProcPtr		dnrCompletionMethod;	// the routine to call when the resolving completes
	OTNotifyUPP						uppToDispose;			// points to data that is destroyed after DNR lookup
};
#ifndef __cplusplus
typedef struct DomainNameLookupData		DomainNameLookupData;
#endif
typedef DomainNameLookupData*			DomainNameLookupDataPtr;



//
// public methods
//

void
	DNR_Dispose							(DomainNameLookupDataPtr*		inoutDataPtr);

OSStatus
	DNR_LookUpDomainName				(Ptr							inHostNameBuffer,
										 Size const						inHostNameSize,
										 DNRDoneNotificationProcPtr		inCompletionMethod,
										 SInt32 const					inCompletionMethodData);

OSStatus
	DNR_LookUpNewConnection				(StringPtr						inHostName,
										 TelnetScreenID const			inScreenID);

OSStatus
	DNR_LookUpNewConnectionViaProxy		(StringPtr						inHostName,
										 ProxyServerRef					inProxyServer,
										 TelnetScreenID const			inScreenID);

#endif