/*###############################################################

	DialogUtilities.h
	
	Lists methods that simplify window management, dialog box
	management, and window content control management.  Actually,
	this module contains a ton of routines that really should be
	part of the Mac OS, but arent.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __DIALOGUTILITIES__
#define __DIALOGUTILITIES__

// Mac includes
#include <Appearance.h>
#include <Dialogs.h>
#if TARGET_API_MAC_OS8
#include <StandardFile.h>
#endif

// library includes
#include "InterfaceLib.h"

// MacTelnet includes
#include "TelnetHeader.h"



//
// public methods
//

void
	ActivateDialogItem						(DialogRef				inDialog,
											 DialogItemIndex const	inItemIndex);

pascal Boolean
	AnyKeyDefaultDialogEventFilter			(DialogRef				inDialog,
											 EventRecord*			inoutEventPtr,
											 DialogItemIndex*		outItemIndex);

#define	BooleanToCheckBoxValue(a)			((a) ? (kControlCheckBoxCheckedValue) : (kControlCheckBoxUncheckedValue))

#define	BooleanToRadioButtonValue(a)		((a) ? (kControlRadioButtonCheckedValue) : (kControlRadioButtonUncheckedValue))

void
	DeactivateDialogItem					(DialogRef				inDialog,
											 DialogItemIndex const	inItemIndex);

void
	DeactivateFrontmostWindow				(void);

void
	DebugFlashDialogItem					(DialogRef const		inDialog,
											 DialogItemIndex const	inItemIndex);

void
	DebugFlashDialogItemSubControls			(DialogRef const		inDialog,
											 DialogItemIndex const	inItemIndex);

void
	DebugOutlineDialogItem					(DialogRef const		inDialog,
											 DialogItemIndex const	inItemIndex,
											 UInt32 const			inDelay);

void
	DebugSelectControlHierarchyDumpFile		(WindowRef				inForWindow);

void
	DeviceLoopClipAndDraw					(RgnHandle				inNewClipRegion,
											 DeviceLoopDrawingUPP	inProc,
											 long					inDeviceLoopUserData,
											 DeviceLoopFlags		inDeviceLoopFlags);

void
	DeviceLoopClipAndDrawControl			(ControlRef				inControl,
											 DeviceLoopDrawingUPP	inProc);

void
	DisplayBalloonForDialogItemUnderMouse	(DialogRef const		inDialog,
											 Point const			inGlobalMouse,
											 SInt16 const			inResourceID_hdlg);

void
	DisplayDialogItemBalloon				(DialogRef const		inDialog,
											 DialogItemIndex const	inItemIndex,
											 SInt16 const			inResourceID_hdlg);

void
	DrawOneDialogItem						(DialogRef const		inDialog,
											 DialogItemIndex const	inItemIndex);

DialogItemIndex
	FindControlDialogItemIndex				(DialogRef const		inDialog,
											 ControlRef				inControl);

Boolean
	FlashButton								(DialogRef				theDialog,
											 short					item);

Boolean
	FlashButtonControl						(ControlRef				inControl,
											 Boolean				inIsDefaultButton,
											 Boolean				inIsCancelButton);

void
	FlashDialogItem							(DialogRef const		inDialog,
											 DialogItemIndex const	inItemIndex,
											 unsigned long const	inDelay,
											 short const			inCount);

void
	FlipCheckBox							(DialogRef				dptr,
											 short					item);

void
	FlipRadioButton							(DialogRef				dptr,
											 short					item);

pascal void
	FrameUserPaneArea						(ControlRef				inControl,
											 SInt16					inPart);

void
	GetControlNumericalText					(ControlRef				inControl,
											 SInt32*				outNumberPtr);

void
	GetControlOSTypeText					(ControlRef				inControl,
											 OSType*				outTypePtr);

void
	GetControlText							(ControlRef				inControl,
											 Str255					outText);

Handle
	GetControlTextAsNewHandle				(ControlRef				inControl);

#if TARGET_API_MAC_OS8
DialogRef
	GetCurrentThemeSavvyStandardFileModal	(void);
#endif

void
	GetDialogItemControlText				(DialogRef const		inDialog,
											 DialogItemIndex const	inItemIndex,
											 Str255					outText);

SInt16
	GetDialogItemValue						(DialogRef				inDialog,
											 DialogItemIndex		inItemIndex);

void
	GetInactiveColors						(RGBColor*				outInactiveForegroundColor,
											 RGBColor*				outInactiveBackgroundColor);

void
	GetLighterColors						(RGBColor*				outLighterForegroundColorOrNull,
											 RGBColor*				outLighterBackgroundColorOrNull);

void
	GetNewDialogWithWindowInfo				(SInt16 const			inDialogResourceID,
											 DialogRef*				outDialog,
											 WindowInfoRef*			outWindowInfoRefPtr);

WindowRef
	GetRealFrontNonDialogWindow				(void);

WindowRef
	GetRealFrontWindow						(void);

#if TARGET_API_MAC_OS8
pascal OSStatus
	GetThemeMetric							(ThemeMetric 			inMetric,
											 SInt32*				outMetric);
#endif

//! since the Carbon implementation of IsControlHilited() isnt smart enough to consider the inactive case, this API exists...
#define IsControlActiveAndHilited(inControlHandle)			((GetControlHilite(inControlHandle) > 0) && (GetControlHilite(inControlHandle) < 255))

Boolean
	IsPopUpHelpEnabled						(void);

void
	LocalToGlobalRegion						(RgnHandle				inoutRegion);

pascal ControlKeyFilterResult
	NumericalLimiter						(ControlRef				inControl,
											 SInt16*				inKeyCode,
											 SInt16*				inCharCode,
											 EventModifiers*		inModifiers);

pascal ControlKeyFilterResult
	OSTypeLengthLimiter						(ControlRef				inControl,
											 SInt16*				inKeyCode,
											 SInt16*				inCharCode,
											 EventModifiers*		inModifiers);

void
	OutlineRegion							(RgnHandle				inoutRegion);

void
	RestoreFrontmostWindow					(void);

void
	SelectBehindDialogWindows				(WindowRef				inWindowToSelect);

Boolean
	SelectDirectory							(short*					inoutVolumeReferenceNumber,
											 long*					inoutDirectoryID);

void
	SelectOverRealFrontWindow				(WindowRef				inWindowToSelect);

void
	SetControlNumericalText					(ControlRef				inControl,
											 SInt32					inNumber);

void
	SetControlOSTypeText					(ControlRef				inControl,
											 OSType const			inType);

void
	SetControlText							(ControlRef				inControl,
											 Str255					inText);

void
	SetControlTextHandle					(ControlRef				inControl,
											 Handle					inText);

SInt16
	SetCursorForControlUnderMouse			(WindowRef const		inWindow);

void
	SetDialogItemCenterTruncatedText		(DialogRef				inDialog,
											 DialogItemIndex		inItemIndex,
											 StringPtr				inString,
											 ThemeFontID			inThemeFontToUse);

void
	SetDialogItemControlText				(DialogRef const		inDialog,
											 DialogItemIndex const	inItemIndex,
											 StringPtr const		inText);

// WARNING: THIS ROUTINE DOES NOT WORK PROPERLY YET
void
	SetDialogItemControlTextSelection		(DialogRef const		inDialog,
											 DialogItemIndex const	inItemIndex,
											 SInt16 const			inStart,
											 SInt16 const			inEnd);

void
	SetDialogItemValue						(DialogRef				inDialog,
											 DialogItemIndex		inItemIndex,
											 SInt16					inValue);

void
	SetUpHelpButtonControl					(ControlRef const		inControl);

void
	SetUpHelpButtonDialogItem				(DialogRef const		inDialog,
											 DialogItemIndex const	inHelpButtonIndex);

pascal Boolean
	StandardAlertEventFilter				(DialogRef				inDialog,
											 EventRecord*			inoutEventPtr,
											 DialogItemIndex*		outItemIndex);

pascal Boolean
	StandardDialogEventFilter				(DialogRef				inDialog,
											 EventRecord*			inoutEventPtr,
											 DialogItemIndex*		outItemIndex);

Boolean
	SwitchDialogDefaultItem					(DialogRef const		inDialog,
											 EventRecord const*		inEventPtr,
											 DialogItemIndex const	inDefaultableItemIndices[],
											 short const			inDefaultableItemIndicesArraySize);

void
	TextFontByName							(StringPtr				inFontName);

// NO-OP UNDER CARBON
void
	ThemeSavvyStandardPutFile				(ConstStr255Param 		inTitle,
											 ConstStr255Param 		inPrompt,
											 ConstStr255Param 		inOriginalName,
											 StandardFileReply*		outReplyRecordPtr);

// NO-OP UNDER CARBON
void
	ThemeSavvyStandardGetFile				(ConstStr255Param 		inTitle,
											 ConstStr255Param 		inPrompt,
											 FileFilterUPP 			inFileFilterUPP,
											 short 					inTypeListElementCount,
											 ConstSFTypeListPtr 	inTypeListPtr,
											 StandardFileReply*		outReplyRecordPtr);

// NO-OP UNDER CARBON
void
	ThemeSavvyStandardGetDirectory			(ConstStr255Param 		inTitle,
											 ConstStr255Param 		inPrompt,
											 StandardFileReply*		outReplyRecordPtr);

void
	TransitionDialogClose					(DialogRef const		inDialog);

#define	TransitionDialogCloseCancel(d)		TransitionDialogCloseLowerRight(d)

void
	TransitionDialogCloseLowerRight			(DialogRef const		inDialog);

void
	TransitionDialogCloseToControl			(DialogRef const		inDialog,
											 ControlRef const		inToWhat);

void
	TransitionDialogCloseToWindowRegion		(DialogRef const		inDialog,
											 WindowRef const		inParentWindow,
											 WindowRegionCode const	inToWhere);

void
	TransitionDialogCloseUpperLeft			(DialogRef const		inDialog);

void
	TransitionDialogDisplay					(DialogRef const		inDialog);

void
	TransitionDialogDisplayFromControl		(DialogRef const		inDialog,
											 ControlRef const		inFromWhat);

void
	TransitionDialogDisplayFromWindowRegion	(DialogRef const		inDialog,
											 WindowRef const		inParentWindow,
											 WindowRegionCode const	inFromWhere);

void
	UseInactiveColors						(void);

void
	UseLighterColors						(void);


// DEPRECATED METHODS

void GetTEText(DialogRef dptr, short item, Str255 str);

void Hilite(DialogRef dptr, short item, short value);
short findPopupMenuItem(MenuHandle theHandle, Str255 name);
short GetLeading(short fontID, short fontSize);
short GetAscent(short fontID, short fontSize);
short GetMaxDepth(void);
void CenterRectIn(Rect* inner, Rect* outer);

pascal void VersionNumber(DialogRef dptr, short itm);

#endif