/*###############################################################

	EventLoop.h
	
	This module handles events, including the main event loop
	that gets invoked in "MacLook.c".  Some new update event
	handlers in particular have been added in Telnet 3.0, for
	special things like the Color Picker and Navigation Services
	dialogs.
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __EVENTLOOP__
#define __EVENTLOOP__

// Mac includes
#ifndef __EVENTS__
#	include <Events.h>
#endif
#ifndef __MENUS__
#	include <Menus.h>
#endif
#ifndef __NAVIGATION__
#	include <Navigation.h>
#endif

// Telnet includes
#ifndef __SCREENMANAGER__
#	include "ScreenManager.h"
#endif
#ifndef __CONTEXTSLIB__
#	include "ContextsLib.h"
#endif



// useful macros
#define IS_SHOW_CONTEXTUAL_MENU_EVENT(eventPtr) (FlagManager_Test(kFlagContextualMenus) && IsShowContextualMenuClick(eventPtr))



//
// public methods
//

void
	EventLoop_Init					(void);

void
	EventLoop_Done					(void);

void
	CloseAWindow					(WindowRef						theWindow);

EventModifiers
	EventLoop_CurrentModifiers		(void);

pascal Boolean
	EventLoop_HandleColorPickerUpdate		(EventRecord*					inoutEventPtr);

Boolean
	EventLoop_HandleEvent			(EventRecord*					inoutEventPtr);

pascal void
	EventLoop_HandleNavigationUpdate		(NavEventCallbackMessage		callBackSelector, 
											 NavCBRecPtr					callBackParms, 
											 NavCallBackUserData			callBackUD);

Boolean
	EventLoop_IsCommandKeyDown		(void);

Boolean
	EventLoop_IsNextCancel			(void);

Boolean
	EventLoop_IsOptionKeyDown		(void);

Boolean
	EventLoop_IsShiftKeyDown		(void);

void
	EventLoop_Service				(void);

void
	EventLoop_SetFloatersHilited	(Boolean const					inAreHilited);

void
	EventLoop_SetFloatersSuspended	(Boolean const					inAreInvisible);

void
	EventLoop_SetScriptFor			(ConnectionDataPtr				inConnectionDataPtr);

void
	NotifyUser						(void);

void
	NoWindow						(void);

void
	SetDefaultKCHR					(void);

#endif