/*###############################################################

	FileUtilities.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// standard-C includes
#ifdef __MWERKS__
#	include <eof.h>
#else
#	include <stdio.h>
#endif
#include <string.h>

// Mac includes
#include <PLStringFuncs.h>

// library includes
#include "DataAccessLib.h"

// MacTelnet includes
#include "AppleEventUtilities.h"
#include "DialogUtilities.h"
#include "FileUtilities.h"
#include "MacLook.h"
#include "InstallAE.h"
#include "telneterrors.h"



// constants
#define STRING_PATHNAME_DELIMITER "\p:"

// variables
static char					FileName[256]; // needed by nextname() and firstname()

// internal methods
static OSErr		getFilesInDirectory			(const FSSpec*			inDirectorySpecPtr,
												 const SInt16			inSearchBits,
												 const OSType			inDesiredTypeSignature,
												 const OSType			inDesiredCreatorSignature,
												 FSSpec*				inFileArrayStoragePtr,
												 UInt32*				inoutFileMaxCountFileActualCountPtr);
static OSErr		threadedGetFilesInDirectory	(const FSSpec*			inDirectorySpecPtr,
												 const SInt16			inSearchBits,	// bitwise OR of "fsSB..." constants
												 const OSType			inDesiredTypeSignature,		// used with "fsSBFlFndrInfo"
												 const OSType			inDesiredCreatorSignature,	// used with "fsSBFlFndrInfo"
												 FSSpec*				inFileArrayStoragePtr,
												 UInt32*				inoutFileMaxCountFileActualCountPtr,
												 const SInt32			inMaximumDelayTime);
static short		wccheck						(char*					inFileName,
												 char*					inTemplate);



//
// public methods
//

/*!
Changes the directory relative to the provided
directory and volume.  Handles preceding ".."
as well as absolute paths.  Returns 0 on
success, 1 on failure.

(2.6)
*/
short
FileUtilities_ChangeDirectory		(long*		inoutDirID,
									 short*		inoutVRefNum,
									 char*		inoutPathname)
{
	CInfoPBRec		theDirInfo;
	char			tempst[256];
	char*			nSlash = null;
	char*			start = null;
	short 			NumberOfLevelsToGoUp = 0,
					temp = 0,
					localvRefNum = 0;
	long			localdirID = 0L;
	OSErr			err = noErr;
	
	
	start = inoutPathname;
	localvRefNum = *inoutVRefNum;
	localdirID = *inoutDirID;
	
	while ( (nSlash = strchr(inoutPathname,'/') ) !=0L) {
		if (( (nSlash - inoutPathname) == 2) && (*inoutPathname=='.') && (*(inoutPathname+1) =='.') ) {
			inoutPathname += 3L;
			start = inoutPathname;
			NumberOfLevelsToGoUp++;
			}
		else {
			*nSlash=':';
			inoutPathname=nSlash + 1L;
			}
		}
	
	if ( strcmp("..",inoutPathname)==0) {
		start[0]=0;
		NumberOfLevelsToGoUp++;
		}

	// Now start points to the beginning of a relative pathname devoid of ".."s, and colons
	//		for seperators rather than slashes.
	
	if ( *start==':') {							// This is a fully qualified pathname
		if (NumberOfLevelsToGoUp) return(1);	// Someone did a "../..//blah"...
		nSlash = strchr(start+1L, ':');			// Seperate out volume name
		if (strlen(start) == 1)					// Someone did a "cd /"
			localdirID = fsRtDirID;						//		Use top level directory
		else {
			long	junk1 = 0L,
					junk2 = 0L;
			
			
			if (nSlash != null) *nSlash = 0;
			strcpy(tempst, start+1);			// Remove leading colon
			strcat(tempst, ":");				// Add trailing colon
			StringUtilities_CToPInPlace(tempst);
			HSetVol((StringPtr)tempst, 0, fsRtDirID);
			HGetVol(null, &localvRefNum, &junk1);
			localdirID = fsRtDirID;

			if (nSlash != null) {				// Theres more than a volume name
				*nSlash = ':';					// Restore the leading slash
				strcpy( tempst, nSlash);
				StringUtilities_CToPInPlace(tempst);				// tempst now contains relative pathname
				
				Memory_WriteZero((Ptr)&theDirInfo, (long)sizeof(CInfoPBRec));
				theDirInfo.dirInfo.ioVRefNum = localvRefNum;		// Now find the directory on the volume
				theDirInfo.dirInfo.ioDrDirID = localdirID;
				theDirInfo.dirInfo.ioNamePtr = (StringPtr)tempst;
				theDirInfo.dirInfo.ioFDirIndex = 0;
				if ((err = PBGetCatInfo(&theDirInfo, false)) != noErr) return(1);
				localdirID = theDirInfo.dirInfo.ioDrDirID;
				}		
			}
		}
	else
		{	// This is a relative pathname, start by taking care of any ".."s...
			Memory_WriteZero((Ptr)&theDirInfo, (long)sizeof(CInfoPBRec));
			while (NumberOfLevelsToGoUp && (localdirID != fsRtDirID)) {
				theDirInfo.dirInfo.ioVRefNum = localvRefNum;
				theDirInfo.dirInfo.ioDrDirID = localdirID;
				theDirInfo.dirInfo.ioNamePtr = 0;
				theDirInfo.dirInfo.ioFDirIndex = -1;
				if ((err = PBGetCatInfo(&theDirInfo, false)) != noErr) return(1);
				localdirID = theDirInfo.dirInfo.ioDrParID;
				NumberOfLevelsToGoUp--;
				}
		
			// Now use the relative pathname to find out the actual directory ID
			// Relative pathnames must begin with a colon, so put on on the beginning
			
			if(start[0]) {
				strcpy(tempst+1, start);
				tempst[0] = ':';
				StringUtilities_CToPInPlace(tempst);
				Memory_WriteZero((Ptr)&theDirInfo, (long)sizeof(CInfoPBRec));
				theDirInfo.dirInfo.ioVRefNum = localvRefNum;
				theDirInfo.dirInfo.ioDrDirID = localdirID;
				theDirInfo.dirInfo.ioNamePtr = (StringPtr)tempst;
				theDirInfo.dirInfo.ioFDirIndex = 0;
				if ((err = PBGetCatInfo(&theDirInfo, false)) != noErr) return(1);
				localdirID = theDirInfo.dirInfo.ioDrDirID;
				}
		}
	
	*inoutDirID = localdirID;		// Everything went ok, change the dirID and vRefNum
	*inoutVRefNum = localvRefNum;
	
	return(0);
}// ChangeDirectory


/*!
This convenient routine can be used to perform an
asynchronous search of a directory for which you
have a file specification, returning an array of
file specification records for each file in that
directory.

(3.0)
*/
OSErr
FileUtilities_GetAllFilesInDirectory	(const FSSpec*		inDirectorySpecPtr,
										 FSSpec*			inFileArrayStoragePtr,
										 UInt32*			inoutFileMaxCountFileActualCountPtr)
{
	return threadedGetFilesInDirectory(inDirectorySpecPtr, fsSBFlParID, '????', '????', inFileArrayStoragePtr,
										inoutFileMaxCountFileActualCountPtr, 20/* maximum blockage time, in ticks */);
}// GetAllFilesInDirectory


/*!
This routine can be used to determine the directory
ID of a directory described by a file system
specification record.

(3.0)
*/
long
FileUtilities_GetDirectoryIDFromFSSpec		(const FSSpec*		inFSSpecPtr)
{
	CInfoPBRec		thePB;
	long			result = 0L;
	
	
	thePB.dirInfo.ioCompletion = 0L;
	thePB.dirInfo.ioNamePtr = (StringPtr)&inFSSpecPtr->name;
	thePB.dirInfo.ioVRefNum = inFSSpecPtr->vRefNum;
	thePB.dirInfo.ioFDirIndex = 0;
	thePB.dirInfo.ioDrDirID = inFSSpecPtr->parID;
	
	if (PBGetCatInfoSync(&thePB) == noErr) result = thePB.dirInfo.ioDrDirID;
	
	return result;
}// GetDirectoryIDFromFSSpec


/*!
This routine can be used to determine the last
modified time of a directory described by a
file system specification record.  You can use
this routine to determine if the user has added
or removed files from a directory (by saving
an initial modification time and later comparing
it against another time).

(3.0)
*/
unsigned long
FileUtilities_GetDirectoryDateFromFSSpec	(const FSSpec*		inFSSpecPtr,
											 FileUtilitiesDate	inWhichDate)
{
	CInfoPBRec		thePB;
	unsigned long	result = 0L;
	
	
	thePB.dirInfo.ioCompletion = 0L;
	thePB.dirInfo.ioNamePtr = (StringPtr)&inFSSpecPtr->name;
	thePB.dirInfo.ioVRefNum = inFSSpecPtr->vRefNum;
	thePB.dirInfo.ioFDirIndex = 0;
	thePB.dirInfo.ioDrDirID = inFSSpecPtr->parID;
	
	if (PBGetCatInfoSync(&thePB) == noErr) switch (inWhichDate)
	{
		case kFileUtilitiesDateOfModification:
			result = thePB.dirInfo.ioDrMdDat;
			break;
		
		case kFileUtilitiesDateOfCreation:
			result = thePB.dirInfo.ioDrCrDat;
			break;
		
		case kFileUtilitiesDateOfBackup:
			result = thePB.dirInfo.ioDrBkDat;
			break;
		
		default:
			// ???
			result = 0L;
			break;
	}
	
	return result;
}// GetDirectoryModificationTimeFromFSSpec


/*!
To copy the name of the specified directory into
the given string, use this method.  To determine
the name of a volume, pass "fsRtDirID" for the
directory ID.

(2.6)
*/
void
FileUtilities_GetDirectoryName	(short		inVRefNum,
								 long		inDirID,
								 Str32		outName)
{
	CInfoPBRec		theInfo;
	
	
	Memory_WriteZero((Ptr)&theInfo, sizeof(CInfoPBRec));
	theInfo.dirInfo.ioVRefNum = inVRefNum;
	theInfo.dirInfo.ioDrDirID = inDirID;
	theInfo.dirInfo.ioNamePtr = outName;
	theInfo.dirInfo.ioFDirIndex = -1; // -1 == directory info only
	PBGetCatInfoSync(&theInfo);
}// GetDirectoryName


/*!
Determines the volume reference number of a
volume, given its index.  If any problems
occur, an error code is returned.

Updated to use modern File Manager data
structures and calls in 3.0.

(2.6)
*/
OSErr
FileUtilities_GetIndVolume		(short		inIndex,
								 short*		outVRefNum)
{
	HParamBlockRec	pb;
	OSErr			result = noErr;
	
	
	pb.volumeParam.ioCompletion = null;
	pb.volumeParam.ioNamePtr = null;
	pb.volumeParam.ioVolIndex = inIndex;
	
	result = PBHGetVInfoSync(&pb);
	
	*outVRefNum = pb.volumeParam.ioVRefNum;
	return result;
}// GetIndVolume


/*!
Determines the last-modified date and time of a file.
If any problems occur, an error code is returned.

(2.6)
*/
OSErr
FileUtilities_GetLastModDateTime	(FSSpec*			inFSSpecPtr,
									 unsigned long*		outLastModDateTime)
{
	CInfoPBRec	pBlock;
	OSErr		result = noErr;
	
	
	pBlock.hFileInfo.ioNamePtr = inFSSpecPtr->name;
	pBlock.hFileInfo.ioVRefNum = inFSSpecPtr->vRefNum;
	pBlock.hFileInfo.ioFDirIndex = 0;
	pBlock.hFileInfo.ioDirID = inFSSpecPtr->parID;
	result = PBGetCatInfoSync(&pBlock);
	if (result == noErr)
	{
		*outLastModDateTime = pBlock.hFileInfo.ioFlMdDat;
	}
	return result;
}// GetLastModDateTime


/*!
Finds the name in the given directory that
matches the wildcard specification.

(2.6)
*/
char*
FileUtilities_GetNameUsingWildcard	(char*			inWildcard,
									 CInfoPBRec*	inFInfoPtr)
{
	static char		gFileName[256];
	CInfoPBRec*		localfinfo;
	Boolean			done = false;
	
	
	localfinfo = (CInfoPBRec*)Memory_NewPtr(sizeof(CInfoPBRec));
	inFInfoPtr->hFileInfo.ioNamePtr = (StringPtr)gFileName;
	
	while (!done)
	{
		BlockMove(inFInfoPtr, localfinfo, sizeof(CInfoPBRec));
		if (PBGetCatInfo(localfinfo, false) != 0)
		{
			Memory_DisposePtr(&((Ptr)localfinfo));
			return null;
		}
		
		gFileName[gFileName[0] + 1] = 0;
		if ((!wccheck(&gFileName[1], inWildcard)) || // it doesnt match
			(((localfinfo->hFileInfo.ioFlAttrib & 0x10) && // its an invisible directory
			(localfinfo->dirInfo.ioDrUsrWds.frFlags & (fInvisible | 0x1000))) ||
			(!(localfinfo->hFileInfo.ioFlAttrib & 0x10) && // its an invisible file
			(localfinfo->hFileInfo.ioFlFndrInfo.fdFlags) & (fInvisible | 0x1000))))
		{
			inFInfoPtr->hFileInfo.ioFDirIndex++; // go to next one
		}
		else done = true;
	}

	if (localfinfo->hFileInfo.ioFlAttrib & 0x10) // its a directory
	{
		gFileName[++gFileName[0]] = '/';
		gFileName[gFileName[0] + 1] = 0;
	}
	
	Memory_DisposePtr(&((Ptr)localfinfo));
	return (&gFileName[1]);
}// GetNameUsingWildcard


/*!
Use this routine to obtain a Pascal string
representation of a directory pathname,
knowing only its directory ID and disk volume
reference number.

If the specified directory can't be found,
the OS error code "dirNFErr" is returned;
otherwise, "noErr" is returned.  If a call to
PBGetCatInfoSync() fails, this routine may
also return one of its error codes.

(2.6)
*/
OSErr
FileUtilities_GetPathnameFromDirectoryID	(const long		inDirectoryID,
											 const short	inVolumeReferenceNumber,
											 StringPtr		outFullPathNameString)
{
	CInfoPBRec	block;
	Str255		directoryName;
	OSErr		result = noErr;
	Boolean		isError = false;
	
	
	outFullPathNameString[0] = '\0';
	
	block.dirInfo.ioDrParID = inDirectoryID;
	block.dirInfo.ioNamePtr = directoryName;
	do
	{
		block.dirInfo.ioVRefNum = inVolumeReferenceNumber;
		block.dirInfo.ioFDirIndex = -1;
		block.dirInfo.ioDrDirID = block.dirInfo.ioDrParID;
		result = PBGetCatInfoSync(&block);
		if (result != noErr) isError = true;
		if (!isError)
		{
			PLstrcat(directoryName, STRING_PATHNAME_DELIMITER);
			StringUtilities_PInsert(outFullPathNameString, directoryName);
		}
	} while ((block.dirInfo.ioDrDirID != fsRtDirID) && (!isError));
	
	if (!isError) StringUtilities_PInsert(outFullPathNameString, STRING_PATHNAME_DELIMITER);
	
	return result;
}// GetPathnameFromDirectoryID


/*!
Use this routine to obtain a Pascal string
representation of a directory pathname,
knowing only its file system specification.
If either the file name of the given file
specification is the name of a directory,
or if the file name is blank, then the
resultant path will refer to either a
directory or a volume.

If the specified directory cant be found,
the OS error code "dirNFErr" is returned;
otherwise, "noErr" is returned.  If a call to
PBGetCatInfoSync() fails, this routine may
also return one of its error codes.

(3.0)
*/
OSErr
FileUtilities_GetPathnameFromFSSpec		(const FSSpec*		inDirectorySpecPtr,
										 StringPtr			outFullPathNameString,
										 const Boolean		inIsDirectory)
{
	OSErr		result = noErr;
	
	
	if ((inDirectorySpecPtr != null) && (outFullPathNameString != null))
	{
		CInfoPBRec	block;
		Str255		directoryName;
		Boolean		isError = false,
					catName = false;
		
		
		outFullPathNameString[0] = '\0';
		
		if ((inDirectorySpecPtr->name != null) && (StrLength(inDirectorySpecPtr->name))) catName = true;
		block.dirInfo.ioDrParID = inDirectorySpecPtr->parID;
		block.dirInfo.ioNamePtr = directoryName;
		
		do
		{
			block.dirInfo.ioVRefNum = inDirectorySpecPtr->vRefNum;
			block.dirInfo.ioFDirIndex = -1;
			block.dirInfo.ioDrDirID = block.dirInfo.ioDrParID;
			result = PBGetCatInfoSync(&block);
			if (result != noErr) isError = true;
			if (!isError)
			{
				PLstrcat(directoryName, STRING_PATHNAME_DELIMITER);
				StringUtilities_PInsert(outFullPathNameString, directoryName);
			}
		} while ((block.dirInfo.ioDrDirID != fsRtDirID) && (!isError));
		
		if (!isError) StringUtilities_PInsert(outFullPathNameString, STRING_PATHNAME_DELIMITER);
		
		if (catName)
		{
			// for file specifications that include a filename, append the name onto the end
			// of the parent directorys pathname specification
			PLstrcat(outFullPathNameString, inDirectorySpecPtr->name);
			if (inIsDirectory) PLstrcat(outFullPathNameString, STRING_PATHNAME_DELIMITER);
		}
	}
	else result = memPCErr;
	
	return result;
}// GetPathnameFromFSSpec


/*!
This convenient routine can be used to perform an
asynchronous search of a directory for which you
have a file specification, returning an array of
file specification records for each file in that
directory whose type and creator match those
given.

(3.0)
*/
OSErr
FileUtilities_GetTypedFilesInDirectory	(const FSSpec*		inDirectorySpecPtr,
										 const OSType		inDesiredTypeSignature,
										 const OSType		inDesiredCreatorSignature,
										 FSSpec*			inFileArrayStoragePtr,
										 UInt32*			inoutFileMaxCountFileActualCountPtr)
{
	return threadedGetFilesInDirectory(inDirectorySpecPtr, fsSBFlParID | fsSBFlFndrInfo,
										inDesiredTypeSignature, inDesiredCreatorSignature,
										inFileArrayStoragePtr, inoutFileMaxCountFileActualCountPtr,
										20/* maximum blockage time, in ticks */);
}// GetTypedFilesInDirectory


/*!
Converts the given volume name into a disk
volume reference number.  If the named
volume cannot be found, the default volume
reference number is returned.

(2.6)
*/
short
FileUtilities_GetVolumeReferenceNumberByName	(Str32		inVolumeName)
{
	short	result = -1;
	
	
#if TARGET_API_MAC_OS8
	if (HSetVol(inVolumeName, 0, fsRtDirID) == noErr) GetVol(null, &result);
#else
	HGetVol(inVolumeName, &result, null/* directory ID */);
#endif
	
	return result;
}// GetVolumeReferenceNumberByName


/*!
This routine launches an application with standard
launch flags and (under OS 8.5 or later) the
launch application Finder theme sound.  If any
problems occur, an error code is returned; if not,
"noErr" is returned.

(3.0)
*/
OSErr
FileUtilities_LaunchApplicationFromFSSpec		(FSSpec*		inFSSpecPtr)
{
	LaunchParamBlockRec		launchInfo;
	OSErr					result = noErr;
	
	
	launchInfo.reserved1 = 0L;
	launchInfo.reserved2 = 0;
	launchInfo.launchBlockID = extendedBlock;
	launchInfo.launchEPBLength = extendedBlockLen;
	launchInfo.launchFileFlags = 0;
	launchInfo.launchControlFlags = launchContinue | launchNoFileFlags;
	launchInfo.launchAppSpec = inFSSpecPtr;
	launchInfo.launchAppParameters = null;
	result = LaunchApplication(&launchInfo);
	if (result == noErr) if (FlagManager_Test(kFlagAppearance1_1API))
	{
		(OSStatus)PlayThemeSound(kThemeSoundLaunchApp);
	}
	
	return result;
}// LaunchApplicationFromFSSpec


/*!
Use this convenient routine to fire an Apple Event
back to MacTelnet (that is hence recordable) to open
the specified file via an open documents event.  If
you are opening a file for any reason, you should
generally handle it by calling this routine, and then
really open the file within the implementation of
the open documents Apple Event handler.

(3.0)
*/
OSErr
FileUtilities_OpenDocument		(FSSpec*		inFSSpecPtr)
{
	OSErr			result = noErr;
	
	
	if (inFSSpecPtr != null)
	{
		AppleEvent		openDocumentsEvent = { typeNull, null },
						reply = { typeNull, null };
		AEDescList		docList = { typeNull, null };
		OSErr			result = noErr;
		
		
		// Create a list of documents containing only the specified file, add the files
		// file specification and then send it back to Telnet as an open documents event.
		// A list is used because it is more versatile; in the future, another convenience
		// routine for opening multiple documents in a recordable fashion could easily be
		// generated from this code.
		if (result == noErr) result = AECreateList(null/* factoring */, 0/* size */,
													false/* is a record */, &docList);
		if (result == noErr) result = AEPutPtr(&docList, 1/* where to put it */,
												typeFSS, (Ptr)inFSSpecPtr, sizeof(FSSpec));
		if (result == noErr) result = InstallAE_CreateRecordableAppleEvent(kCoreEventClass, kAEOpenDocuments,
																			&openDocumentsEvent);
		if (result == noErr) result = AEPutParamDesc(&openDocumentsEvent, keyDirectObject, &docList);
		if (result == noErr) result = AESend(&openDocumentsEvent, &reply, kAENoReply | kAEAlwaysInteract,
											kAENormalPriority, kAEDefaultTimeout, null, null);
		AEDisposeDesc(&openDocumentsEvent);
		AEDisposeDesc(&docList);
	}
	else result = memPCErr;
	
	return result;
}// OpenDocument


/*!
To create a file in the specified location with the
specified name, or to create a file with a similar
name in the case of a duplicate, use this method.
Instead of failing with "dupFNErr" with FSpCreate(),
you can use this routine to be persistent about
using FSpCreate() repeatedly, until an error other
than "dupFNErr" occurs.  If any error occurs other
than "dupFNErr", it is returned.

If an alternate file name is used (because the given
one already exists), the resultant file specification
will be modified.

(3.0)
*/
OSErr
FileUtilities_PersistentCreate		(FSSpec*			inoutFSSpecPtr,
									 OSType				inCreator,
									 OSType				inType,
									 ScriptCode			inScriptTag)
{
	OSErr		result = noErr;
	
	
	if (inoutFSSpecPtr != null)
	{
		FSSpec		file;
		
		
		result = FSMakeFSSpec(inoutFSSpecPtr->vRefNum, inoutFSSpecPtr->parID, inoutFSSpecPtr->name, &file);
		if ((result == noErr) || (result == fnfErr))
		{
			SInt16		i = 1;
			Str31		numString;
			Str255		originalFileName;
			
			
			PLstrcpy(originalFileName, file.name);
			do
			{
				if (!i) result = FSpCreate(&file, inCreator, inType, inScriptTag); // first try creating the specified file
				else
				{
					// try to create a similar file (like File 1, File 2, etc.)
					PLstrcpy(file.name, "\p%a %b");
					NumToString(i, numString);
					StringUtilities_PMetaSetDefaults();
					StringUtilities_PBuild(file.name, originalFileName, numString, EMPTY_PSTRING, EMPTY_PSTRING);
					result = FSMakeFSSpec(file.vRefNum, file.parID, file.name, &file);
					if ((result == noErr) || (result == fnfErr)) result = FSpCreate(&file, inCreator, inType, inScriptTag);
				}
				i++;
			}
			while (result == dupFNErr);
		}
		
		*inoutFSSpecPtr = file;
	}
	else result = memPCErr;
	
	return result;
}// PersistentCreate


/*!
Reads a text file into the specified handle, resizing
it if necessary to fit the data.  The number of bytes
read from the file is returned.

This routine currently reads data one byte at a time.

If any problems occur, 0 is returned.

(3.0)
*/
SInt32
FileUtilities_TextToHandle		(SInt16 const		inFileRefNum,
								 Handle				inoutTextHandle)
{
	char		character = '\0';
	OSErr		error = noErr;
	SInt32		readHowMany = 1,
				count = 0L,
				result = 0L;
	
	
	// read file to determine its size
	count = 0L;
	while (character != EOF)
	{
		if ((error = FSRead(inFileRefNum, &readHowMany, &character)) == eofErr) character = EOF;
		else
		{
			if (readHowMany != 1) readHowMany = 1; // reset count (read failed)
			else count++;
		}
	}
	
	// attempt to size handle appropriately
	error = Memory_SetHandleSize(inoutTextHandle, count);
	if (error != noErr) result = 0L; // failure - return 0 to indicate no bytes were read
	else
	{
		// success
		SInt32		capacityLength = GetHandleSize(inoutTextHandle);
		char*		ptr = null;
		SInt8		hState = HGetState(inoutTextHandle);
		
		
		// read data into properly-sized handle
		readHowMany = 1;
		count = 0L;
		HLockHi(inoutTextHandle);
		for (ptr = *inoutTextHandle; (*ptr != EOF) && (count < capacityLength); ptr++)
		{
			if ((error = FSRead(inFileRefNum, &readHowMany, ptr)) == eofErr) *ptr = EOF;
			else
			{
				if (readHowMany != 1) readHowMany = 1; // reset count (read failed)
				else count++;
			}
		}
		result = count;
		HSetState(inoutTextHandle, hState);
	}
	
	return result;
}// TextToHandle


/*!
Determines if a volume supports the desktop database.
If any problems occur, an error code is returned.

(2.6)
*/
OSErr
FileUtilities_VolHasDesktopDB	(short			inVRefNum,
								 Boolean*		outHasDesktop)
{
	HParamBlockRec			pb;
	GetVolParmsInfoBuffer	info;
	OSErr					result = noErr;
	
	
	pb.ioParam.ioCompletion = null;
	pb.ioParam.ioNamePtr = null;
	pb.ioParam.ioVRefNum = inVRefNum;
	pb.ioParam.ioBuffer = (Ptr)&info;
	pb.ioParam.ioReqCount = sizeof(info);
	result = PBHGetVolParmsSync(&pb);
	*outHasDesktop = ((result == noErr) && ((info.vMAttrib & (1L << bHasDesktopMgr)) != 0));
	
	return result;
}// VolHasDesktopDB


/* firstname
*  find the first name in the given directory which matches the wildcard
*  specification	*/
char *firstname(char *spec, long dirID, short vRefNum, CInfoPBRec *finfo)
{
	CInfoPBRec *localfinfo;
	
	localfinfo = (CInfoPBRec *) NewPtrClear(sizeof(CInfoPBRec));	
	
	finfo->hFileInfo.ioNamePtr=(StringPtr) FileName;
	finfo->hFileInfo.ioFDirIndex=1;
	finfo->hFileInfo.ioVRefNum = vRefNum;
	finfo->hFileInfo.ioDirID = dirID;
	
	BlockMove(finfo, localfinfo, sizeof(CInfoPBRec));
	if (PBGetCatInfo(localfinfo,false)!=0) {
		DisposePtr((Ptr)localfinfo);
		return(0L);
		}

	FileName[FileName[0]+1]=0;
	
	while(!wccheck(&FileName[1],spec)) {
		finfo->hFileInfo.ioFDirIndex++;
		BlockMove(finfo, localfinfo, sizeof(CInfoPBRec));
		if (PBGetCatInfo(localfinfo,false)!=0) {
			DisposePtr((Ptr)localfinfo);
			return(0L);
			}
		FileName[FileName[0]+1]=0;
		}

	if (localfinfo->hFileInfo.ioFlAttrib & 16)  {
		FileName[++FileName[0]]='/';
		FileName[FileName[0]+1]=0;
		}

	DisposePtr((Ptr)localfinfo);
	return(&FileName[1]);
}// firstname


/**********************************************************************/
/* nextname
*  modify the path spec to contain the next file name in the
*  sequence as given by DOS
*
*  if at the end of the sequence, return null
*/
char *nextname(char *spec, CInfoPBRec *finfo)
{
	CInfoPBRec *localfinfo;
	
	localfinfo = (CInfoPBRec *) NewPtrClear(sizeof(CInfoPBRec));	
	finfo->hFileInfo.ioFDirIndex++;

	BlockMove(finfo, localfinfo, sizeof(CInfoPBRec));
	if (PBGetCatInfo(localfinfo,false)!=0) {
		DisposePtr((Ptr)localfinfo);
		return(0L);
		}

	FileName[FileName[0]+1]=0;
	while(!wccheck(&FileName[1], spec)) {
		//putln(&FileName[1]);
		finfo->hFileInfo.ioFDirIndex++;
		BlockMove(finfo, localfinfo, sizeof(CInfoPBRec));
		if (PBGetCatInfo(localfinfo,false)!=0) {
			DisposePtr((Ptr) localfinfo);
			return(0L);
			}
		FileName[FileName[0]+1]=0;
		}
	
	if (localfinfo->hFileInfo.ioFlAttrib & 16)  {
		FileName[++FileName[0]]='/';
		FileName[FileName[0]+1]=0;
		}

	DisposePtr((Ptr)localfinfo);
	return(&FileName[1]);
}// nextname


//
// internal methods
//

/*!
This convenient routine can be used to perform a
synchronous search of a directory for which you
have a file specification, returning an array of
file specification records for each file in that
directory.

(3.0)
*/
static OSErr
getFilesInDirectory		(const FSSpec*		inDirectorySpecPtr,
						 const SInt16		inSearchBits,
						 const OSType		inDesiredTypeSignature,
						 const OSType		inDesiredCreatorSignature,
						 FSSpec*			inFileArrayStoragePtr,
						 UInt32*			inoutFileMaxCountFileActualCountPtr)
{
	return threadedGetFilesInDirectory(inDirectorySpecPtr, inSearchBits, inDesiredTypeSignature,
										inDesiredCreatorSignature, inFileArrayStoragePtr, inoutFileMaxCountFileActualCountPtr,
										0/* maximum delay - 0 means never yield to threads */);
}// getFilesInDirectory


/*!
This convenient routine can be used to perform a
synchronous or asynchronous search of a directory
for which you have a file specification, returning
an array of file specification records for each
file in that directory (or a subset of them that
corresponds to certain criteria).

To yield to threads (highly recommended), pass a
nonzero value for "inMaximumDelayTime".  The
synchronous search will block for at most that
much time before yielding to a thread.  This
method will not return until the search finishes,
but many threads may be yielded to before then.

(3.0)
*/
static OSErr
threadedGetFilesInDirectory		(const FSSpec*		inDirectorySpecPtr,
								 const SInt16		inSearchBits,
								 const OSType		inDesiredTypeSignature,
								 const OSType		inDesiredCreatorSignature,
								 FSSpec*			inFileArrayStoragePtr,
								 UInt32*			inoutFileMaxCountFileActualCountPtr,
								 const SInt32		inMaximumDelayTime)
{
	OSErr			result = noErr;
	CSParam*		paramBlockPtr = (CSParam*)Memory_NewPtr(sizeof(CSParam));
	
	
	if (paramBlockPtr != null)
	{
		enum
		{
			kSearchPerformanceEnhancingBufferSizeInK = 16
		};
		CInfoPBPtr		lowerRangePtr = (CInfoPBPtr)Memory_NewPtr(sizeof(CInfoPBRec)),
						upperRangePtr = (CInfoPBPtr)Memory_NewPtr(sizeof(CInfoPBRec));
		
		
		if ((lowerRangePtr == null) || (upperRangePtr == null)) result = memFullErr;
		
		if (result == noErr)
		{
			Ptr		bufferPtr = Memory_NewPtr(INTEGER_KILOBYTES(kSearchPerformanceEnhancingBufferSizeInK));
			
			
			if (bufferPtr != null)
			{
				// Perform a basic search, for all files, allocating (if possible) a
				// fairly generous helper buffer to make this as quick as possible.
				paramBlockPtr->ioCompletion = null;
				paramBlockPtr->ioNamePtr = null;
				paramBlockPtr->ioVRefNum = inDirectorySpecPtr->vRefNum;
				paramBlockPtr->ioMatchPtr = inFileArrayStoragePtr;
				if (inoutFileMaxCountFileActualCountPtr != null)
				{
					paramBlockPtr->ioReqMatchCount = *inoutFileMaxCountFileActualCountPtr;
				}
				else result = memPCErr;
				
				paramBlockPtr->ioSearchBits = inSearchBits;
				{
					// set up the criteria for matching files
					lowerRangePtr->hFileInfo.ioNamePtr =
						upperRangePtr->hFileInfo.ioNamePtr = null;
					if (inSearchBits & fsSBFlFndrInfo)
					{
						lowerRangePtr->hFileInfo.ioFlFndrInfo.fdType = inDesiredTypeSignature;
						lowerRangePtr->hFileInfo.ioFlFndrInfo.fdCreator = inDesiredCreatorSignature;
					}
					else
					{
						lowerRangePtr->hFileInfo.ioFlFndrInfo.fdType =
							lowerRangePtr->hFileInfo.ioFlFndrInfo.fdCreator = (OSType)0xFFFFFFFF;
					}
					upperRangePtr->hFileInfo.ioFlFndrInfo.fdType =
						upperRangePtr->hFileInfo.ioFlFndrInfo.fdCreator = (OSType)0xFFFFFFFF;
					lowerRangePtr->hFileInfo.ioFlParID =
						upperRangePtr->hFileInfo.ioFlParID = FileUtilities_GetDirectoryIDFromFSSpec(inDirectorySpecPtr);
					upperRangePtr->hFileInfo.ioFlFndrInfo.fdFlags = 0;
					upperRangePtr->hFileInfo.ioFlFndrInfo.fdLocation.h = 0;
					upperRangePtr->hFileInfo.ioFlFndrInfo.fdLocation.v = 0;
					upperRangePtr->hFileInfo.ioFlFndrInfo.fdFldr = 0;
					
					// set up the search parameters to use the data just defined
					paramBlockPtr->ioSearchInfo1 = lowerRangePtr;
					paramBlockPtr->ioSearchInfo2 = upperRangePtr;
				}
				paramBlockPtr->ioSearchTime = inMaximumDelayTime;
				
				paramBlockPtr->ioCatPosition.initialize = 0L;
				
				paramBlockPtr->ioOptBuffer = bufferPtr;
				if (paramBlockPtr->ioOptBuffer != null)
				{
					paramBlockPtr->ioOptBufSize = GetPtrSize(bufferPtr);
				}
				
				if (result == noErr)
				{
					// PBCatSearchSync() returns "noErr" if the time expires or if
					// the search hits the maximum number of files
					while ((result == noErr) && (!paramBlockPtr->ioActMatchCount))
					{
						result = PBCatSearchSync(paramBlockPtr);
						
						// maximum number of files found?
						if ((result == noErr) &&
							(paramBlockPtr->ioActMatchCount >= *inoutFileMaxCountFileActualCountPtr))
						{
							break;
						}
						
						// if the volume changes in mid-search, restart the search, to be safe
						if (result == catChangedErr)
						{
							result = noErr;
							paramBlockPtr->ioCatPosition.initialize = 0L;
							continue;
						}
						
						// yield to another thread before trying to continue the search
						if (inMaximumDelayTime) Threads_YieldToAnother();
					}
					
					// now do something with the files found, if no errors occurred
					if ((result == eofErr) || ((result == noErr) && (paramBlockPtr->ioActMatchCount > 0)))
					{
						if (result == eofErr) result = noErr; // having to search the entire volume is not an error
						if (inoutFileMaxCountFileActualCountPtr != null)
						{
							*inoutFileMaxCountFileActualCountPtr = paramBlockPtr->ioActMatchCount;
						}
						else result = memPCErr;
					}
				}
				Memory_DisposePtr(&bufferPtr);
			}
			else result = memFullErr;
		}
		
		Memory_DisposePtr((Ptr*)&paramBlockPtr);
		Memory_DisposePtr((Ptr*)&lowerRangePtr);
		Memory_DisposePtr((Ptr*)&upperRangePtr);
	}
	else result = memFullErr;
	
	return result;
}// threadedGetFilesInDirectory


/*!
This routine now returns a short, for recursion.

(2.6)
*/
static short
wccheck		(char*		inFileName,
			 char*		inTemplate)
{
	Boolean		done = false;
	short		result = 0;
	
	
	while (*inTemplate)
	{
		if (*inTemplate == '*')
		{
			inTemplate++;
			if (*inTemplate)
			{
				while ((*inFileName) && !wccheck(inFileName, inTemplate))
				{
					inFileName++;
				}
				if ((*inFileName) == 0)
				{
					result = 0;
					done = true;
					break;
				}
			}
			else
			{
				result = 1;
				done = true;
				break;
			}
			continue;
		}
		else
		{
			if ((*inTemplate != '?') && (*inTemplate != *inFileName))
			{
				result = 0;
				done = true;
				break;
			}
		}
		inTemplate++;
		inFileName++;
	}
	
	if (!done) result = (*inFileName) ? 0 : 1;
	
	return result;
}// wccheck