/*###############################################################

	FormatDialog.h
	
	This module implements the Format dialog box, for editing
	multiple sets of font, size and color information from a
	terminal window.
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __FORMATDIALOG__
#define __FORMATDIALOG__

// Mac includes
#ifndef __MACTYPES__
#	include <MacTypes.h>
#endif



// constants
typedef UInt32 FormatDialogGeneralOptions;
enum
{
	// flags that affect the entire dialog box
	kFormatDialogGeneralOptionHideUpdateDefaultTerminalCheckBox = (1 << 0)
};
typedef UInt32 FormatDialogFormatOptions;
enum
{
	// flags that affect one particular format, not all formats
	kFormatDialogFormatOptionDisableFontItems = (1 << 0),
	kFormatDialogFormatOptionDisableFontSizeItems = (1 << 1),
	kFormatDialogFormatOptionDisableForeColorItems = (1 << 2),
	kFormatDialogFormatOptionDisableBackColorItems = (1 << 3),
	kFormatDialogFormatOptionMaskDisableAllItems = 0x0000000F
};
enum
{
	kFormatDialogNumberOfFormats = 3,	// the number of format indices, below
	kFormatDialogIndexNormalText = 0,
	kFormatDialogIndexBoldText = 1,
	kFormatDialogIndexBlinkingText = 2
};

// types
struct FormatDialogSetupData
{
	struct
	{
		FormatDialogFormatOptions		options;	// see the valid enumerated flags above
		
		struct
		{
			Str255			familyName;
			SInt16			size;			// provide an acceptable size value (4 or more)
		} font;
		
		struct
		{
			RGBColor		foreground;
			RGBColor		background;
		} colors;
	} format[kFormatDialogNumberOfFormats];
	
	FormatDialogGeneralOptions		generalOptions;	// see the valid enumerated flags above
};
#ifndef __cplusplus
typedef struct FormatDialogSetupData	FormatDialogSetupData;
#endif
typedef FormatDialogSetupData*			FormatDialogSetupDataPtr;



//
// public methods
//

void
	FormatDialog_Init					(FormatDialogSetupDataPtr		inSetupDataPtr);

void
	FormatDialog_Done					(void);

Boolean
	FormatDialog_Display				(void);

#endif