/*###############################################################

	InterfaceLib.h
	
	This file lists all public constants, functions, and other
	data from all modules of the Interface Library, which is
	useful for simplifying operations on the Mac OS.  This is a
	portable library, meaning it can be used with any program.
	You need to Rez the " InterfaceLibAppResources.r" file with
	your application if you link against this library.
	
	Interface Library 1.0
	 1998-2001 by Kevin Grant
	
	This library is free software; you can redistribute it or
	modify it under the terms of the GNU Lesser Public License
	as published by the Free Software Foundation; either version
	2.1 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU Lesser Public License for details.

    You should have received a copy of the GNU Lesser Public
	License along with this library; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

/*!
	@header Interface Utility Library
*/

#ifndef __INTERFACELIB__
#define __INTERFACELIB__

// InterfaceLib resource stuff
#define INTERFACELIB_ALERT_ICON_LEFT			20
#define INTERFACELIB_ALERT_DIALOG_WD			368			/* system standard is 368 */

// your application should have a 'SICN' resource with this ID for Notification Manager postings
#define kAlert_NotificationSICNResourceID		12000

// icon family resource ID for sort order reversal buttons (used if no 32-bit icon is available)
#define kInterfaceLibIconIDSortAscending		32000
#define kInterfaceLibIconIDSortDescending		32001

#define	kIDAppleGuideButton						12000
#define kInterfaceLibControlBaseID				32000
#define kInterfaceLibDialogBaseID				32000
#define kInterfaceLibAlertDialogID				kInterfaceLibDialogBaseID
#define kInterfaceLibNonmovableAlertDialogID	(kInterfaceLibDialogBaseID + 1)

// controls for alerts
#define kInterfaceLibControlButtonNormal		kInterfaceLibControlBaseID
#define kInterfaceLibControlAlertText			(kInterfaceLibControlBaseID + 1)
#define kInterfaceLibControlAlertIconStop		(kInterfaceLibControlBaseID + 2)
#define kInterfaceLibControlAlertIconNote		(kInterfaceLibControlBaseID + 3)
#define kInterfaceLibControlAlertIconCaution	(kInterfaceLibControlBaseID + 4)

// controls for Finder Lists
#define kInterfaceLibControlFinderListHeadingPlacard					(kInterfaceLibControlBaseID + 5)
#define kInterfaceLibControlFinderListHeadingBevelButton				(kInterfaceLibControlBaseID + 6)
#define kInterfaceLibControlFinderListHeadingSortOrderReversalButton	(kInterfaceLibControlBaseID + 7)
#define kInterfaceLibControlFinderListScrollBarV						(kInterfaceLibControlBaseID + 8)
#define kInterfaceLibControlFinderListContentUserPane					(kInterfaceLibControlBaseID + 9)
#define kInterfaceLibControlFinderListEncompassingUserPane				(kInterfaceLibControlBaseID + 10)



#ifndef REZ

#ifdef powerc

// Mac includes
#ifndef __AEDATAMODEL__
#	include <AEDataModel.h>
#endif
#ifndef __APPEARANCE__
#	include <Appearance.h>
#endif
#ifndef __COLORPICKER__
#	include <ColorPicker.h>
#endif
#ifndef __DIALOGS__
#	include <Dialogs.h>
#endif
#ifndef __FILES__
#	include <Files.h>
#endif
#ifndef __ICONS__
#	include <Icons.h>
#endif
#ifndef __LISTS__
#	include <Lists.h>
#endif
#ifndef __MACTCP__
#	include <MacTCP.h>
#endif
#ifndef __MACTYPES__
#	include <MacTypes.h>
#endif
#ifndef __NAVIGATION__
#	include <Navigation.h>
#endif
#ifndef __QUICKDRAW__
#	include <QuickDraw.h>
#endif

// Data Access Library includes
#ifndef __DATAACCESSLIB__
#	include "DataAccessLib.h"
#endif



#define topLeft(r)	(((Point *) &(r))[0])
#define botRight(r)	(((Point *) &(r))[1])

/**************************************************************************************************************************/
// Alert.c
//
// This module defines the powerful Interface Library Alert object, an abstract way to create the most powerful Mac OS
// alert windows youve ever seen!  By using the routines in this module, you can generate standard alert messages (which
// are Appearance-savvy, of course), or construct customized ones of your own.  You can make movable alerts (with or
// without title text) or non-movable alerts.  Alerts can contain a help button, up to three other buttons, any kind of
// alert icon, and both standard message text and smaller help text.  Whats more, the Alert module automatically sizes
// the alert boxs buttons so that they fit their titles, *and* supports a powerful event filter function that lets the
// user use the Tab key to change the default button.  To top it all off, there are a couple of utility functions available
// that help to auto-size buttons in any (not necessarily an alert) dialog box.  You can even set Notification Manager
// preferences, so the Alert module can automatically alert the user if your application is backgrounded.

typedef struct OpaqueInterfaceLibAlert { unsigned garbage; }*	InterfaceLibAlertRef;

enum
{
	// pass one of these to Alert_SetNotificationPreferences()
	kAlert_NotifyDoNothing = 0,
	kAlert_NotifyDisplayDiamondMark = 1,
	kAlert_NotifyDisplayIconAndDiamondMark = 2,
	kAlert_NotifyAlsoDisplayAlert = 3
};

enum
{
	kAlert_StyleOK,
	kAlert_StyleCancel,
	kAlert_StyleOKCancel,
	kAlert_StyleOKCancel_CancelIsDefault,
	kAlert_StyleYesNo,
	kAlert_StyleYesNo_NoIsDefault,
	kAlert_StyleYesNoCancel,
	kAlert_StyleYesNoCancel_NoIsDefault,
	kAlert_StyleYesNoCancel_CancelIsDefault,
	kAlert_StyleDontSaveCancelSave
};

// dialog item indices
enum
{
	kAlert_ItemButton1 = 1,
	kAlert_ItemButton2 = 2,
	kAlert_ItemDialogText = 3,
	kAlert_ItemHelpText = 4,
	kAlert_ItemButton3 = 5,
	kAlert_ItemHelpButton = 6,
	kAlert_ItemTitleText = 7,
	kAlert_ItemBalloons = 8
};



/*###############################################################
	INITIALIZING AND FINISHING WITH ALERTS
###############################################################*/

// CALL THIS ROUTINE ONCE, BEFORE ANY OTHER ALERT ROUTINE
void
	Alert_Init							(SInt16 const				inApplicationResourceFile);

// CALL THIS ROUTINE AFTER YOU ARE PERMANENTLY DONE WITH ALERTS
void
	Alert_Done							(void);

/*###############################################################
	CREATING, INITIALIZING AND DESTROYING ALERT WINDOWS
###############################################################*/

InterfaceLibAlertRef
	Alert_New							(void);

void
	Alert_Dispose						(InterfaceLibAlertRef*		inoutAlert);

/*###############################################################
	ALERT MODULE IDLE PROCEDURES
###############################################################*/

// CALL THIS METHOD WHEN YOUR APPLICATION DETECTS A "RESUME" EVENT
void
	Alert_ServiceNotification			(void);

void
	Alert_SetNotificationMessage		(StringPtr					inMessage);

void
	Alert_SetNotificationPreferences	(UInt16 const				inNotificationPreferences);

/*###############################################################
	ALERT GLOBAL SETTINGS
###############################################################*/

void
	Alert_SetUseSpeech					(Boolean					inUseSpeech);

/*###############################################################
	ROUTINES USEFUL FOR DEBUGGING
###############################################################*/

void
	Alert_DebugCheckpoint				(StringPtr					inDescription);

void
	Alert_DebugSendStringToSelf			(StringPtr					inSendWhat,
										 AEEventClass const			inEventClass,
										 AEEventID const			inEventID);

void
	Alert_DebugSystemProfile			(void);

void
	Alert_DebugWarning					(void);

void
	Alert_DebugWhatIsFile				(StringPtr					inDescription,
										 FSSpec const*				inFilePtr);

void
	Alert_DebugWhatIsNumber				(StringPtr					inDescription,
										 SInt32 const				inNumber);

void
	Alert_DebugWhatIsPoint				(StringPtr					inDescription,
										 Point const				inPoint);

void
	Alert_DebugWhatIsRectangle			(StringPtr					inDescription,
										 Rect const*				inRectPtr);

/*###############################################################
	UTILITY METHODS FOR DISPLAYING ALERTS WITHOUT CREATING THEM
###############################################################*/

void
	Alert_Message						(StringPtr					inDialogText,
										 StringPtr					inHelpText,
										 Boolean					inIsHelpButton);

Boolean
	Alert_ReportOSErr					(OSErr						inErrorCode);

Boolean
	Alert_ReportOSStatus				(OSStatus					inErrorCode);

/*###############################################################
	DISPLAYING CREATED ALERTS
###############################################################*/

OSErr
	Alert_Display						(InterfaceLibAlertRef		inAlert);

/*###############################################################
	GETTING FEEDBACK FROM DISPLAYED ALERTS
###############################################################*/

SInt16
	Alert_ItemHit						(InterfaceLibAlertRef		inAlert);

void
	Alert_SetFilterProc					(InterfaceLibAlertRef		inAlert,
										 ModalFilterUPP				inProc);

/*###############################################################
	HELPER METHODS TO SPECIFY ALERT WINDOW ADORNMENTS
###############################################################*/

// ALLOWS AN ALERT TO BE DISPLAYED AS A SHEET (BUT THIS REQUEST MAY BE IGNORED); A "null" WINDOW REMOVES SHEET BEHAVIOR
void
	Alert_AttachToWindow				(InterfaceLibAlertRef		inAlert,
										 WindowRef					inAttachToWhichWindowOrNull);

void
	Alert_SetButtonText					(InterfaceLibAlertRef		inAlert,
										 SInt16						inWhichButton,
										 StringPtr					inNewText);

void
	Alert_SetHelpButton					(InterfaceLibAlertRef		inAlert,
										 Boolean					inIsHelpButton);

void
	Alert_SetIsBackgrounded				(Boolean					inIsApplicationSuspended);

void
	Alert_SetMovable					(InterfaceLibAlertRef		inAlert,
										 Boolean					inIsMovable);

void
	Alert_SetParamsFor					(InterfaceLibAlertRef		inAlert,
										 SInt16						inAlertStyle);

void
	Alert_SetText						(InterfaceLibAlertRef		inAlert,
										 StringPtr					inDialogText,
										 StringPtr					inHelpText);

void
	Alert_SetTitle						(InterfaceLibAlertRef		inAlert,
										 StringPtr					inNewText);

void
	Alert_SetType						(InterfaceLibAlertRef		inAlert,
										 AlertType					inNewType);



/**************************************************************************************************************************/
// ColorUtilities.c
//
// Use this module to maximize the color capabilites of your Mac application.  You will find useful methods for accessing
// information required to use Mac OS Appearance theme-brush-based routines.  There is also a simple way to "completely"
// save the current graphics ports state, in a way sufficient for any "damage" that a theme brush can cause when drawing.
// Finally, theres a cool routine that lets you automatically display the most powerful Color Picker dialog available.
// A few useful methods have been thrown in that will allow you to use "standard" patterns without actually accessing
// QuickDraw global variables, very useful in preparation for the opaque QuickDraw structures of Mac OS X.
//
// Color Pen State portions of this file are from the OS Technologies group at Apple Computer Inc. (source: SDK for
// Appearance 1.0), and are written by Ed Voas with modifications by Kevin Grant.   1997-1999 by Apple Computer, Inc.,
// all rights reserved.

struct ColorPenState
{
	Boolean			isColorPort; // never "false" in Carbon
	RGBColor		foreColor;
	RGBColor		backColor;
	PenState		pen;
	SInt16			textMode;
	PixPatHandle	pnPixPat;
	PixPatHandle	bkPixPat;
	Pattern			bkPat; // not used in Carbon
	UInt32			fgColor;
	UInt32			bkColor;
};
#ifndef __cplusplus
typedef struct ColorPenState ColorPenState;
#endif

/*###############################################################
	APPEARANCE 1.0 HELPERS FOR USING THEME BRUSH SYSTEM CALLS
###############################################################*/

Boolean
	ColorUtilities_IsColorDevice				(GDHandle				inDevice);

Boolean
	ColorUtilities_IsColorGrafPort				(GrafPtr				inPort);

SInt16
	ColorUtilities_GetCurrentDepth				(CGrafPtr				inPort);

/*###############################################################
	MANIPULATING PEN STATES UNDER APPEARANCE 1.0
###############################################################*/

void
	ColorUtilities_GetColorAndPenState			(ColorPenState*			outStatePtr);

void
	ColorUtilities_NormalizeColorAndPen			(void);

void
	ColorUtilities_SetColorAndPenState			(ColorPenState*			inoutStatePtr);

/*###############################################################
	HIGHLIGHTING
###############################################################*/

void
	ColorUtilities_HiliteMode					(void);

/*###############################################################
	CHANGING THE PEN OR BACKGROUND TO A STANDARD PATTERN
###############################################################*/

void
	ColorUtilities_SetBlackBackgroundPattern	(void);

void
	ColorUtilities_SetGrayBackgroundPattern		(void);

void
	ColorUtilities_SetWhiteBackgroundPattern	(void);

void
	ColorUtilities_SetBlackPenPattern			(void);

void
	ColorUtilities_SetGrayPenPattern			(void);

void
	ColorUtilities_SetWhitePenPattern			(void);

/*###############################################################
	DISPLAYING STANDARD GET-COLOR DIALOG BOXES
###############################################################*/

Boolean
	ColorUtilities_ColorChooserDialogDisplay	(ConstStr255Param		inPrompt,
												 RGBColor const*		inColor,
												 RGBColor*				outColor,
												 Boolean				inIsModal,
												 UserEventUPP			inUserEventProc,
												 PickerMenuItemInfo*	inEditMenuInfo);



/**************************************************************************************************************************/
// Cursors.c
//
// Stop using your current cursor-changing method, and start using the Cursors module!  This powerful and efficient system
// lets you automatically take advantage of Mac OS 8.5s theme cursors (if available), or use its own versions if the
// runtime computer does not support theme cursors.  You can instantly set the cursor to one of a wide variety of standard
// and special cursors.  The Cursor module has a memory management scheme that ensures that cursors are only loaded as
// needed, and that the same cursor never gets loaded into memory twice.  The Cursors_Idle() method is the perfect routine
// to call in a multi-threaded application, since it handles cursor animations and deferred tasks.  What are deferred
// tasks?  Check out Cursors_DeferredUseWatch(), which guarantees that the current cursor will not change to a watch until
// at least the specified number of ticks has elapsed.  Setting a deferred watch cursor is great for applications that may
// run on computers with a wide range of computing power, because fast machines wont see a "flicker" as the cursor is set
// to a watch for a tenth of a second, and slow computers *will* see, appropriately, a watch cursor to indicate that an
// operation is taking some time.  So what are you waiting for, update your cursor code now!

enum
{
	kCursor_Arrow = 0, // dummy ID for reference purposes
	kCursor_IBeam = 1,
	kCursor_Crosshairs = 2,
	kCursor_Plus = 3,
	kCursor_Watch = 4,
	kCursor_ArrowHelp = 150,
	kCursor_ArrowCopy = 151,
	kCursor_ArrowWatch = 152,
	kCursor_ArrowContextualMenu = 153,
	kCursor_HandOpen = 154,
	kCursor_HandGrab = 155,
	kCursor_HandPoint = 156,
	kCursor_TerminalCursor = 256
};



/*###############################################################
	INITIALIZING AND FINISHING WITH CURSORS
###############################################################*/

// CALL THIS ROUTINE ONCE, BEFORE ANY OTHER CURSORS ROUTINE
void
	Cursors_Init						(void);

// CALL THIS ROUTINE AFTER YOU ARE PERMANENTLY DONE WITH CURSORS
void
	Cursors_Done						(void);

/*###############################################################
	DETERMINING IF APPEARANCE 1.1 CURSORS ARE AVAILABLE
###############################################################*/

Boolean
	Cursors_AppearanceCursorsAvail		(void);

/*###############################################################
	CHANGING THE CURSOR AND DETERMINING THE PREVIOUS CURSOR
###############################################################*/

SInt16
	Cursors_DeferredUseWatch			(UInt32 const		inForHowManyTicks);

SInt16
	Cursors_GetCurrent					(void);

void
	Cursors_Idle						(void);

SInt16
	Cursors_Use							(SInt16 const		inCursorID);

SInt16
	Cursors_UseArrow					(void);

SInt16
	Cursors_UseArrowContextualMenu		(void);

SInt16
	Cursors_UseArrowCopy				(void);

SInt16
	Cursors_UseArrowHelp				(void);

SInt16
	Cursors_UseArrowWatch				(void);

SInt16
	Cursors_UseCrosshairs				(void);

SInt16
	Cursors_UseHandGrab					(void);

SInt16
	Cursors_UseHandOpen					(void);

SInt16
	Cursors_UseHandPoint				(void);

SInt16
	Cursors_UseMagnifyingGlass			(void);

SInt16
	Cursors_UseIBeam					(void);

SInt16
	Cursors_UsePlus						(void);

SInt16
	Cursors_UseWatch					(void);



/**************************************************************************************************************************/
// DialogAdjust.c
//
// This extremely powerful module completely handles the dirty work when it comes to resizing an Appearance-savvy window
// or dialog box!  Just begin either a control or dialog item adjustment, assemble a list of adjustments to make to the
// various controls in a window or dialog box respectively, and end the adjustment to have your changes take effect!  The
// Dialog Adjust module assumes you are making adjustments *as if* the user were expanding the size of a window both
// horizontally and vertically; under this assumption, Dialog Adjust *automatically changes* the order of adjustment
// depending on what the actual delta-X and delta-Y values are!  In addition, it uses the Embedding module to cache the
// changes in memory (if possible), dumping everything to the screen in one motion.  But even if there isnt enough memory
// to cache the changes, the delta-sensitive order of operations makes for the smoothest possible window resizing!

typedef SInt32 DialogItemAdjustment;
enum
{
	// these are array indices - make them zero-based and consecutive, ensuring NUMBER_OF_DIALOG_ITEM_ADJUSTMENT_TYPES is right
	kDialogItemAdjustmentResizeH = 0,
	kDialogItemAdjustmentResizeV = 1,
	kDialogItemAdjustmentMoveH = 2,
	kDialogItemAdjustmentMoveV = 3,
	kControlAdjustmentResizeH = kDialogItemAdjustmentResizeH,
	kControlAdjustmentResizeV = kDialogItemAdjustmentResizeV,
	kControlAdjustmentMoveH = kDialogItemAdjustmentMoveH,
	kControlAdjustmentMoveV = kDialogItemAdjustmentMoveV
};



/*###############################################################
	WINDOW CONTROL OR DIALOG ITEM ADJUSTMENT MANAGEMENT
###############################################################*/

// WARNING - YOU CANNOT COMBINE DialogAdjust_AddDialogItem() CALLS WITHIN THE SAME BEGIN-END BRACKET AS THIS CALL
void
	DialogAdjust_AddControlAdjustment		(DialogItemAdjustment const		inAdjustment,
											 ControlRef						inControlToAdjust,
											 SInt32 const					inAmount);

// WARNING - YOU CANNOT COMBINE DialogAdjust_AddControlAdjustment() CALLS WITHIN THE SAME BEGIN-END BRACKET AS THIS CALL
void
	DialogAdjust_AddDialogItem				(DialogItemAdjustment const		inAdjustment,
											 DialogItemIndex const			inItemIndex,
											 SInt32 const					inAmount);

// CONTROL ADJUSTMENT OPERATIONS ARE NOT YET NESTABLE
void
	DialogAdjust_BeginControlAdjustment		(WindowRef const				inWindow);

// CONTROL ADJUSTMENT OPERATIONS ARE NOT YET NESTABLE
void
	DialogAdjust_BeginDialogItemAdjustment	(DialogRef const				inDialog);

void
	DialogAdjust_EndAdjustment				(SInt32 const					inDialogDeltaSizeX,
											 SInt32 const					inDialogDeltaSizeY);



/**************************************************************************************************************************/
// Embedding.c
//
// The Embedding module allows you to use offscreen graphics worlds easily, automatically taking into account multiple
// monitors and automatically using the best technology available to perform the graphics swapping.  You can perform
// offscreen operations on a *per-control* basis, which allows you to draw entire windows or only parts of them (if you
// use embedders, like user panes, to identify sub-regions of windows), from offscreen graphics worlds.

/*!
Offscreen Operation Method

An offscreen operation method should perform all
necessary changes to controls in a window.  The
Embedding_OffscreenControlOperation() routine
will always try to save the entire specified
control for a window in its buffer, so there is
no need for an offscreen operation method to draw
a control unless it is going to change its
appearance in some way.  You generally write
offscreen operation methods that perform a number
of collective tasks on controls, such as setting
the enabled state of more than one control.  Both
pieces of data provided to this method are
arbitrary, and specified when the method
Embedding_OffscreenControlOperation() is called.

Do not call DeviceLoop() in your routine; this
method is designed to provide you with all of
the information that DeviceLoop() would normally
provide.
*/
typedef pascal Boolean (*OffscreenOperationProcPtr)	(ControlRef const	inSpecificControlOrRoot,
													 SInt16 const		inColorDepth,
													 SInt16 const		inDeviceFlags,
													 GDHandle			inTargetDevice,
													 SInt32 const		inData1,
													 SInt32 const		inData2);
#define InvokeOffscreenOperationProc(userRoutine, inControl, inColorDepth, inDeviceFlags, inTargetDevice, inData1, inData2) \
	(*userRoutine)((inControl), (inColorDepth), (inDeviceFlags), (inTargetDevice), (inData1), (inData2))



/*###############################################################
	AUTOMATICALLY ACTIVATING AND DEACTIVATING WINDOWS
###############################################################*/

void
	Embedding_DeactivateFrontmostWindow			(void);

void
	Embedding_RestoreFrontmostWindow			(void);

/*###############################################################
	OFFSCREEN EMBEDDED-CONTROL DRAWING HELPER ROUTINES
###############################################################*/

OSErr
	Embedding_OffscreenControlOperation			(WindowRef						inForWhichWindow,
												 ControlRef						inForWhichControlOrNullForRoot,
												 OffscreenOperationProcPtr		inUPP,
												 SInt32							inData1,
												 SInt32							inData2);

OSErr
	Embedding_OffscreenControlOperationInMode	(WindowRef						inForWhichWindow,
												 ControlRef						inForWhichControlOrNullForRoot,
												 OffscreenOperationProcPtr		inUPP,
												 SInt32							inData1,
												 SInt32							inData2,
												 SInt32							inDrawingMode);

OSErr
	Embedding_OffscreenDrawOneControl			(ControlRef						inControl);

OSErr
	Embedding_OffscreenSwapOverlappingControls	(WindowRef						inForWhichWindow,
												 ControlRef						inControlToHide,
												 ControlRef						inControlToDisplay);

// EXTREMELY USEFUL ROUTINE!  USE EXCLUSIVELY OVER ActivateControl() and DeactivateControl() WHEN WORKING WITH EMBEDDERS
OSErr
	Embedding_OffscreenSetControlActive			(ControlRef						inControl,
												 Boolean						inIsActive);

OSErr
	Embedding_OffscreenSetRootControlActive		(WindowRef						inForWhichWindow,
												 Boolean						inIsActive);



/**************************************************************************************************************************/
// FinderList.c
//
// This module defines complex controls that look like Finder lists.  Each control consists of an encompassing user pane
// that handles all events and drawing, and bevel buttons or placards for column headings.  The list supports standard
// keyboard navigation, such as using the arrow keys to move the selection.  Currently, the content can only be scrolled
// vertically.  The list, however, supports drag-rectangle selection of multiple items, inline editing, and -dragging to
// scroll from anywhere.  For many lists, the viewing and editing advantages offered by a Finder List are much preferred!
// However, the Finder List does not support some features that List Box controls do: you do not have as much control over
// cell selection (you cant force range selection or disable disjoint selection), there is no way to scroll horizontally,
// and there is no current support for rendering or editing complex data.

typedef struct OpaqueFinderList { unsigned garbage; }*		FinderListRef;



typedef UInt32 FinderListFlags;
enum
{
	// initialization and attribute flags for Finder Lists
	kFinderListFlagsMultipleSelection			= (1 << 0),		// allow the user to select multiple items at once
	kFinderListFlagsInactiveItemsDimmed			= (1 << 1),		// give unclickable items a dimmed appearance
	kFinderListFlagsSizeBoxSpace				= (1 << 2),		// shorten the vertical scroll bar to make room for a size box
	kFinderListFlagsNoFocusRing					= (1 << 3),		// omit the focus ring, even if the list is focused
	kFinderListFlagsHeaderButtonsUseSmallFont	= (1 << 4)		// use small font (like Navigation Services), not the Views font
};

typedef UInt32 FinderListColumnFlags;
enum
{
	// initialization and attribute flags for Finder List columns
	kFinderListColumnFlagsNotSortable			= (1 << 0),		// is the column represented by a bevel button (or a placard)?
	kFinderListColumnFlagsActiveSort			= (1 << 1),		// will the lists rows be sorted based on this columns data?
	kFinderListColumnFlagsReadWrite				= (1 << 2),		// are cells in this column allowed to be edited?
	kFinderListColumnFlagsNoHighlighting		= (1 << 3),		// do cells in this column look normal even if the row is highlighted?
	kFinderListColumnFlagsNotResizeable			= (1 << 4)		// is the user allowed to resize this column?
};

typedef UInt32 FinderListRowFlags;
enum
{
	// initialization and attribute flags for Finder List rows
	kFinderListRowFlagsDisabled = (1 << 0),			// is the row unclickable (and therefore uneditable, if a column allows editing)?
	kFinderListRowFlagsSelected = (1 << 1)			// is the row highlighted?
};

typedef UInt16 FinderListCellEditEvent;
enum
{
	// possible event messages, and the kind of data that is provided for the event
	kFinderListCellEditEventBeginEdit = 0,			// data: none (reserved)
	kFinderListCellEditEventEndEdit = 1,			// data: none (reserved)
	kFinderListCellEditEventIdle = 2,				// data: none (reserved)
	kFinderListCellEditEventKeyDownOrAutoKey = 3,	// data1: UInt32 (high=keyCode, low=charCode); data2: UInt16; modifier key states
	kFinderListCellEditEventClickInContent = 4,		// data: none (reserved)
	kFinderListCellEditEventActivate = 5			// data1: Boolean; true=activate, false=deactivate; data2: none (reserved)
};

typedef FourCharCode FinderListColumnDataType;
enum
{
	// kinds of cell data that the Finder List can handle
	kFinderListColumnDataTypePlainText				= FOUR_CHAR_CODE('text')	// data: a series of 8-bit characters
};

typedef UInt16 FinderListComponentType;
enum
{
	// areas of interest within a Finder List
	kFinderListComponentTypeListContents = 0		// the rectangular, scrollable content region
};



/*!
Finder List Cell Editing Method

This type of routine handles messages from the Finder
List for a cell that is currently being edited.  The
kinds of events that this routine will receive include
edit-begin events, edit-end events, idle events,
key-down and auto-key events, click-in-cell-content
events, and activate events.

The function should return "true" only if the requested
event message is handled.
*/
typedef Boolean	 (*FinderListCellEditProcPtr)	(FinderListRef				inRef,
												 SInt16 const				inZeroBasedRowIndex,
												 SInt16 const				inZeroBasedColumnIndex,
												 FinderListCellEditEvent	inMessage,
												 UInt32						inData1,
												 UInt32						inData2);
#define InvokeFinderListCellEditProc(userRoutine, inRef, inZeroBasedRowIndex, inZeroBasedColumnIndex, inMessage, inData1, inData2) \
	(*userRoutine)((inRef), (inZeroBasedRowIndex), (inZeroBasedColumnIndex), (inMessage), (inData1), (inData2))



/*!
Finder List Cell Sorting Method

Routines of this type are used to compare two cells,
when sorting rows in a list.  Returns one of the
standard values -1 (cell 1 belongs before cell 2 in
an ascending order), 0 (both cells appear to be the
same), or 1 (cell 2 belongs before cell 1 in an
ascending order).

Obviously, your function must be designed to handle
data of a known type; for efficiency, this routine
does not pass in extra information about data size,
or even the Finder List that the data comes from.

The Finder List module currently does not support
external registration of comparison routines, so
this declaration is only used internally.  However,
the Finder List does support certain data types,
for which sorting is automatically handled correctly.
*/
typedef int	 (*FinderListCellCompareProcPtr)	(void const*		inCellData1,
												 void const*		inCellData2);
#define InvokeFinderListCellCompareProc(userRoutine, inCellData1, inCellData2) \
	(*userRoutine)((inCellData1), (inCellData2))



/*###############################################################
	CREATING, INITIALIZING AND DESTROYING FINDER LIST REFERENCES
###############################################################*/

FinderListRef
	FinderList_New						(WindowRef const							inOwningWindow,
										 FinderListFlags const						inFlags);

void
	FinderList_Dispose					(FinderListRef*								inoutRefPtr);

/*###############################################################
	USING FINDER LISTS IN A WINDOW
###############################################################*/

void
	FinderList_DeltaSizeControl			(FinderListRef								inRef,
										 SInt16 const								inChangeInWidth,
										 SInt16 const								inChangeInHeight);

ControlRef
	FinderList_GetContainerControl		(FinderListRef								inRef);

RgnHandle
	FinderList_GetNewComponentRegion	(FinderListRef								inRef,
										 FinderListComponentType const				inWhatComponent);

WindowRef
	FinderList_GetOwningWindow			(FinderListRef								inRef);

void
	FinderList_SizeControl				(FinderListRef								inRef,
										 SInt16 const								inWidth,
										 SInt16 const								inHeight);

/*###############################################################
	VIEW MANAGEMENT
###############################################################*/

SInt16
	FinderList_GetFirstVisibleRow		(FinderListRef								inRef);

SInt16
	FinderList_GetVisibleRowCount		(FinderListRef								inRef);

void
	FinderList_ScrollToPoint			(FinderListRef								inRef,
										 Point const								inZeroBasedViewTopLeftInPixels);

// MOVES THE LIST VIEW SO THE SPECIFIED ROWS TOP EDGE IS AT THE TOP OF THE VIEW (UNLESS VERTICAL-CENTERING IS SPECIFIED)
void
	FinderList_ScrollToRow				(FinderListRef								inRef,
										 SInt16 const								inZeroBasedRowIndex,
										 Boolean const								inVerticallyCenterInView);

/*###############################################################
	COLUMN OPERATIONS
###############################################################*/

void
	FinderList_DeleteColumn				(FinderListRef								inRef,
										 SInt16 const								inZeroBasedColumnIndex);

UInt16
	FinderList_GetColumnCount			(FinderListRef								inRef);

// IT IS MOST EFFICIENT TO ADD ALL COLUMNS BEFORE ADDING ANY ROWS; -1 MEANS ADD COLUMN TO FAR END OF LIST
UInt16
	FinderList_InsertColumn				(FinderListRef								inRef,
										 SInt16 const								inInsertBeforeWhichZeroBasedIndexOrMinus1,
										 FinderListColumnDataType const				inDataType,
										 FinderListColumnFlags const				inFlags,
										 StringPtr									inTitle);

void
	FinderList_SetColumnTitle			(FinderListRef								inRef,
										 SInt16 const								inZeroBasedColumnIndex,
										 StringPtr									inTitle);

void
	FinderList_SetColumnTitleOffset		(FinderListRef								inRef,
										 SInt16 const								inZeroBasedColumnIndex,
										 SInt16 const								inTitleOffsetInPixels);

// COLUMNS ARE ASSIGNED A DEFAULT WIDTH THAT FITS THE TITLE TEXT; IF YOU WANT TO SET DIFFERENT WIDTH, USE THIS METHOD
void
	FinderList_SetColumnWidthInPixels	(FinderListRef								inRef,
										 SInt16 const								inZeroBasedColumnIndex,
										 UInt16 const								inWidthInPixels);

void
	FinderList_SetSortColumn			(FinderListRef								inRef,
										 SInt16 const								inZeroBasedColumnIndex);

/*###############################################################
	ROW OPERATIONS
###############################################################*/

// DOES NOT ERASE ANY EXISTING SELECTION; TO DE-SELECT MULTIPLE ROWS, CALL THIS METHOD REPEATEDLY
void
	FinderList_ClearRowSelection		(FinderListRef								inRef,
										 SInt16 const								inZeroBasedRowIndex);

void
	FinderList_ClearSelection			(FinderListRef								inRef);

// EFFICIENTLY DELETES ALL ITEMS IN THE LIST, PRESERVING THE COLUMN STRUCTURE
void
	FinderList_DeleteAllRows			(FinderListRef								inRef);

void
	FinderList_DeleteRow				(FinderListRef								inRef,
										 SInt16 const								inZeroBasedRowIndex);

UInt16
	FinderList_GetRowCount				(FinderListRef								inRef);

void
	FinderList_GetSelectedRowCount		(FinderListRef								inRef,
										 SInt16 const*								outNumberOfSelectedRows);

// THIS FILLS IN AN ARRAY OF INTEGERS CONTAINING ZERO-BASED INDICES FOR ALL SELECTED ROWS
void
	FinderList_GetSelection				(FinderListRef								inRef,
										 UInt16 const								inMaximumArrayLength,
										 SInt16 const*								outIndexArray,
										 UInt16*									outActualArrayLengthOrNull);

// IT IS MOST EFFICIENT TO ADD ALL ROWS AFTER ALL COLUMNS; -1 MEANS ADD ROW TO END OF LIST
UInt16
	FinderList_InsertRow				(FinderListRef								inRef,
										 SInt16 const								inInsertBeforeWhichZeroBasedIndexOrMinus1,
										 FinderListRowFlags const					inFlags);

void
	FinderList_MoveRow					(FinderListRef								inRef,
										 SInt16 const								inZeroBasedRowIndex,
										 SInt16 const								inNewRowIndexOrMinusOne);

void
	FinderList_SelectAllRows			(FinderListRef								inRef);

// DOES NOT ERASE ANY EXISTING SELECTION; TO SELECT MULTIPLE ROWS, CALL THIS METHOD REPEATEDLY
void
	FinderList_SelectRow				(FinderListRef								inRef,
										 SInt16 const								inZeroBasedRowIndex);

/*###############################################################
	CELL OPERATIONS
###############################################################*/

void
	FinderList_GetCellData				(FinderListRef								inRef,
										 SInt16 const								inZeroBasedRowIndex,
										 SInt16 const								inZeroBasedColumnIndex,
										 void*										inoutDataPtr,
										 Size const									inBufferSpace,
										 Size*										outActualSizePtr);

Size
	FinderList_GetCellDataSize			(FinderListRef								inRef,
										 SInt16 const								inZeroBasedRowIndex,
										 SInt16 const								inZeroBasedColumnIndex);

// ALWAYS ADD AT LEAST ONE ROW AND COLUMN BEFORE ANY CELLS; A COPY OF THE SPECIFIED DATA IS USED FOR THE CELL
Boolean
	FinderList_SetCellData				(FinderListRef								inRef,
										 SInt16 const								inZeroBasedRowIndex,
										 SInt16 const								inZeroBasedColumnIndex,
										 void const*								inCellDataPtr,
										 Size const									inDataSize);



/**************************************************************************************************************************/
// IconManager.c
//
// By far the most powerful icon utility there is, the Icon Manager lets you seamlessly use either traditional icon suites
// or new, Icon Services icons, with one handy reference!  Whats more, you can create icons any way you like - if you
// use IconManager_MakeIconSuite(), you can specify an icons content in terms of icon resources, *but* the Icon Manager
// will *automatically* transform the icon reference into an Icon Services (32-bit) icon, if possible!  Conversely, you can
// specify an icons content in terms of Icon Services information, and if the operating system is Mac OS 8.0 or 8.1, the
// Icon Manager will *automatically* look for a suitable icon resource as a replacement for the nonexistent Icon Services
// version.  And if thats not powerful enough, there are useful utilities for drawing icons, *and* automatically setting
// the icon of a push button or bevel button or other kind of Appearance control.  In the case of the latter, the content
// type of Appearance controls is automatically set correctly, so you can actually specify an icon as an icon suite
// resource, and at runtime the Icon Manager may find a 32-bit Icon Services icon as a replacement!  Nothing is more
// powerful than polymorphic icon management.  You can specify icons any way you wish, and if the users OS is either
// more powerful or less powerful than the level required to directly implement the icon request, the Icon Manager will
// automatically and intelligently come up with a different icon to meet the request as best it can.

typedef struct OpaqueIconManagerIcon { unsigned garbage; }*	IconManagerIconRef;



/*###############################################################
	CREATING, INITIALIZING AND DESTROYING ICON REFERENCES
###############################################################*/

IconManagerIconRef
	IconManager_NewIcon			(void);

void
	IconManager_DisposeIcon		(IconManagerIconRef*		inoutRefPtr);

OSErr
	IconManager_MakeIconRef		(IconManagerIconRef			inRef,
								 SInt16 const				inIconServicesVolumeNumber,
								 OSType const				inIconServicesCreator,
								 OSType const				inIconServicesDescription);

OSErr
	IconManager_MakeIconSuite	(IconManagerIconRef			inRef,
								 SInt16 const				inSuiteResourceID,
								 IconSelectorValue			inWhichIcons);

/*###############################################################
	INQUIRING ABOUT ICONS
###############################################################*/

Boolean
	IconManager_IsIconServices	(IconManagerIconRef			inRef);

Boolean
	IconManager_IsIconSuite		(IconManagerIconRef			inRef);

/*###############################################################
	DRAWING ICONS
###############################################################*/

OSErr
	IconManager_PlotIcon		(IconManagerIconRef			inRef,
								 Rect const*				inBounds,
								 IconAlignmentType			inAlignment,
								 IconTransformType			inTransform);

/*###############################################################
	USING ICONS WITH APPEARANCE CONTROLS
###############################################################*/

OSErr
	IconManager_SetButtonIcon	(ControlRef					inControl,
								 IconManagerIconRef			inRef);



/**************************************************************************************************************************/
// LED.c
//
// You know those lights on your extended keyboard for Num Lock, Caps Lock and Scroll Lock?  Well, this spiffy module is
// here to let you muck with those lights!  You can do cool things like flash all three lights at once, set the on/off
// state of any individual light, or perform animation by rotating a single "on" light through the three lights (in either
// direction).  To top it off, theres even a function that can undo the "damage" you caused to the lights (did you turn
// off the Caps Lock light?!).  If you need a powerful way to communicate non-essential information to the user, the LED
// module is perfect.
//
// IMPORTANT:	These routines have no effect under Mac OS X.

typedef Byte LEDPattern;
enum
{
	kLEDNumLock = (1 << 0),
	kLEDCapsLock = (1 << 1),
	kLEDScrollLock = (1 << 2),
	kLEDAll = kLEDNumLock | kLEDCapsLock | kLEDScrollLock,
	kLEDNone = 0
};

/*###############################################################
	KEYBOARD LIGHT-EMITTING DIODE MANIPULATION ROUTINES
###############################################################*/

void
	LED_Flash			(void);

void
	LED_FlashQuick		(void);

void
	LED_Restore			(void);

void
	LED_RotateLeft		(void);

void
	LED_RotateRight		(void);

void
	LED_Set				(LEDPattern		inPattern);



/**************************************************************************************************************************/
// ListUtilities.c
//
// Just because Apple made it a pain to deal with lists doesnt mean you have to burden that pain.  Use this module to
// handle a number of common list-handling needs, such as extracting lists from list box controls, and counting items.

/*###############################################################
	OBTAINING LIST STRUCTURES FROM APPEARANCE CONTROLS
###############################################################*/

void
	ListUtilities_GetControlListHandle		(ControlRef					inListBoxControl,
											 ListHandle*				outListHandle);

void
	ListUtilities_GetDialogItemListHandle	(DialogRef const			inDialog,
											 DialogItemIndex const		inItemIndex,
											 ListHandle*				outListHandle);

/*###############################################################
	MANAGING ITEMS IN LISTS
###############################################################*/

UInt16
	ListUtilities_GetItemCount				(ListHandle					inListHandle);

UInt16
	ListUtilities_GetSelectedItemCount		(ListHandle					inListHandle);

/*###############################################################
	MANAGING LISTS WITH PASCAL STRING ITEM DATA
###############################################################*/

// ASSUMES FIRST COLUMN
void
	ListUtilities_GetListItemText			(ListHandle					inListHandle,
											 UInt16 const				inZeroBasedItemIndex,
											 Str255						outItemText);

void
	ListUtilities_GetUniqueItemText			(ListHandle					inListHandle,
											 Str255						inoutItemText);

Boolean
	ListUtilities_IsItemUnique				(ListHandle					inListHandle,
											 Str255						inItemText);

// ASSUMES FIRST COLUMN
void
	ListUtilities_SetListItemText			(ListHandle					inListHandle,
											 UInt16 const				inZeroBasedItemIndex,
											 StringPtr					inItemText);



/**************************************************************************************************************************/
// Localization.c
//
// When it comes to easing your localization efforts, theres nothing quite like the Interface Librarys Localization
// module!  Programming for cultures that read text left-to-right and those that read right-to-left is a snap, since all
// Localization module utility functions work automatically based on your specified text reading directions.  By designing
// your dialog boxes to follow the action button array model, such as in a Navigation Services file dialog box, you can
// use button arrangement facilities from the Localization module to resize and relocate your bottom-row buttons in a snap!
// There are also powerful utilities for flipping a pair of controls about an imaginary axis, which work automatically for
// the current text reading direction, allowing you to design *only* left-to-right dialog boxes and instantly have them
// reconstructed for right-to-left when appropriate!

enum
{
	kLocalization_FlagReadTextRightToLeft = (1 << 0),
	kLocalization_FlagReadTextBottomToTop = (1 << 1)
};

struct GrafPortFontState
{
	SInt16		fontID;
	SInt16		fontSize;
	Style		fontStyle;
	SInt16		textMode;
};
#ifndef __cplusplus
typedef struct GrafPortFontState GrafPortFontState;
#endif

/*###############################################################
	INITIALIZING THE LOCALIZATION UTILITIES
###############################################################*/

void
	Localization_Init							(UInt32 const				inFlags);

/*###############################################################
	METHODS FOR DETERMINING INITIALIZATION FLAGS LATER ON
###############################################################*/

Boolean
	Localization_IsLeftToRight					(void);

Boolean
	Localization_IsTopToBottom					(void);

/*###############################################################
	MISCELLANEOUS
###############################################################*/

OSStatus
	Localization_GetFontTextEncoding			(StringPtr					inFontName,
												 TextEncoding*				outTextEncoding);

/*###############################################################
	MANIPULATING FONT STATES UNDER APPEARANCE 1.0
###############################################################*/

void
	Localization_GetPortFontState				(GrafPortFontState*			outState);

void
	Localization_SetPortFontState				(GrafPortFontState const*	inState);

void
	Localization_UseThemeFont					(ThemeFontID				inThemeFontToUse,
												 StringPtr					outFontName,
												 SInt16*					outFontSizePtr,
												 Style*						outFontStylePtr);

/*###############################################################
	UTILITY METHODS FOR DIALOG ITEM AND CONTROL MANIPULATION
###############################################################*/

UInt16
	Localization_AdjustDialogButtonArray		(DialogRef const			inDialog,
												 DialogItemIndex const*		inDialogItemIndices,
												 UInt16 const				inDialogItemCount);

UInt16
	Localization_AdjustDialogButtons			(DialogRef const			inDialog,
												 Boolean const				inAdjustDefaultButton,
												 Boolean const				inAdjustCancelButton);

void
	Localization_AdjustHelpButtonControl		(ControlRef					inControl);

void
	Localization_AdjustHelpButtonItem			(DialogRef const			inDialog,
												 DialogItemIndex const		inHelpButtonIndex);

UInt16
	Localization_AutoSizeButtonControl			(ControlRef					inControl,
												 UInt16 const				inMinimumWidth);

UInt16
	Localization_AutoSizeButtonItem				(DialogRef const			inDialog,
												 DialogItemIndex const		inItemIndex,
												 UInt16 const				inMinimumWidth);

UInt16
	Localization_GetSingleLineTextHeight		(ThemeFontID				inThemeFontToUse);

UInt16
	Localization_GetSingleLineTextWidth			(StringPtr					inString,
												 ThemeFontID				inThemeFontToUse);

void
	Localization_HorizontallyPlaceControls		(ControlRef					inControl1,
												 ControlRef					inControl2);

void
	Localization_HorizontallyPlaceItems			(DialogRef const			inDialog,
												 DialogItemIndex const		inItemIndex1,
												 DialogItemIndex const		inItemIndex2);

OSStatus
	Localization_SetControlThemeFontInfo		(ControlRef					inControl,
												 ThemeFontID				inThemeFontToUse);

UInt16
	Localization_SetUpSingleLineTextControl		(ControlRef					inControl,
												 StringPtr					inTextContents,
												 ThemeFontID				inThemeFontToUse);

UInt16
	Localization_SetUpSingleLineTextControlMax	(ControlRef					inControl,
												 StringPtr					inTextContents,
												 ThemeFontID				inThemeFontToUse,
												 UInt16 const				inMaximumAllowedWidth,
												 StringUtilitiesTruncationMethod	inTruncationMethod);

UInt16
	Localization_SetUpMultiLineTextControl		(ControlRef					inControl,
												 StringPtr					inTextContents,
												 ThemeFontID				inThemeFontToUse);

void
	Localization_VerticallyPlaceControls		(ControlRef					inControl1,
												 ControlRef					inControl2);

void
	Localization_VerticallyPlaceItems			(DialogRef const			inDialog,
												 DialogItemIndex const		inItemIndex1,
												 DialogItemIndex const		inItemIndex2);



/**************************************************************************************************************************/
// Navigation.c
//
// Navigation Services is ugly.  But not if you have the Navigation module!  This baby is designed to be your soul mate
// for 99% of the things youll ever want to do with a Navigation Services dialog box.  There are standard routines that
// display dialogs for you, and utility methods to simplify accessing some of the garbage that Navigation Services returns.

/*###############################################################
	DISPLAYING NAVIGATION SERVICES DIALOG BOXES
###############################################################*/

OSErr
	Navigation_GetDirectory				(ConstStr255Param				inPromptMessage,
										 ConstStr255Param				inDialogTitle,
										 int							inPrefKey,
										 int							inFlags,
										 NavEventProcPtr				inEventProc,
										 FSSpec*						outFileSpecPtr);

OSStatus
	Navigation_GetFile					(ConstStr255Param				inPromptMessage,
										 ConstStr255Param				inDialogTitle,
										 OSType							inApplicationSignature,
										 int							inPrefKey,
										 int							inFlags,
										 short							inNumTypes,
										 OSType							inTypeList[],
										 NavEventProcPtr				inEventProc,
										 FSSpec*						inoutFileSpecPtr, // non-null for single-select
										 OSType*						inoutFileType, // non-null for single-select
										 FileInfo*						outFileInfo);  // non-null for single-select

OSStatus
	Navigation_PutFile					(ConstStr255Param				inPromptMessage,
										 ConstStr255Param				inDialogTitle,
										 OSType							inApplicationSignature,
										 OSType							inFileSignature,
										 int							inPrefKey,
										 int							inFlags,
										 NavEventProcPtr				inEventProc,
										 NavReplyRecord*				outReplyPtr,
										 FSSpec*						outFileSpecPtr);

/*###############################################################
	UTILITY ROUTINES FOR USE WITH NAVIGATION SERVICES
###############################################################*/

OSStatus
	Navigation_CompleteSave				(FSSpec const*,
										 NavReplyRecord*				inoutReplyPtr);

OSErr
	Navigation_EnsureOriginalFile		(FSSpec*						inoutFSSpecPtr);

OSErr
	Navigation_GetFSSpecArrayFromAEDesc	(AEDesc const*					inAEDescPtr,
										 void*							outFSSpecArray,
										 long*							inoutFSSpecArrayLengthPtr);

OSErr
	Navigation_GetFSSpecFromAEDesc		(AEDesc const*					inAEDescPtr,
										 FSSpec*						outFSSpecPtr);

pascal Boolean
	Navigation_NothingFilterProc		(AEDesc*						theItem,
										 void*							info,
										 NavCallBackUserData			callBackUD,
										 NavFilterModes					filterMode);



/**************************************************************************************************************************/
// Sound.c
//
// You cant beat this powerful sound management module!  In addition to providing straightforward support for playing
// sounds synchronously, this module can track a dynamic number of asynchronous sounds for you!  You simply register an
// asynchronous sound, call Sound_Idle() once in your event loop, and this module takes care of the rest.

typedef struct OpaqueSoundInfo { unsigned garbage; }*	SoundInfoRef;

/*!
Sound Completion Method

Use a method of this type to receive notification whenever
an Interface Library Sound finishes playing (if it was
started asynchronously).  You usually do not need a routine
like this, but if you have auxiliary data associated with a
sound that needs to be disposed of, this notification
routine is useful.

If you invoke a routine from this module that reads from a
resource, you are not responsible for disposing of those
resources (because you did not directly allocate them).
*/
typedef void (*SoundCompletionProcPtr)(SoundInfoRef		inSoundRef);
#define InvokeSoundCompletionProc(userRoutine, inSoundRef) \
	(*userRoutine)(inSoundRef)



/*###############################################################
	INITIALIZING AND FINISHING WITH SOUND
###############################################################*/

// CALL THIS ROUTINE ONCE, BEFORE ANY OTHER SOUND ROUTINE
void
	Sound_Init						(void);

// CALL THIS ROUTINE AFTER YOU ARE PERMANENTLY DONE WITH SOUND
void
	Sound_Done						(void);

/*###############################################################
	PLAYING SOUNDS
###############################################################*/

void
	Sound_Idle						(void);

void
	Sound_InstallCompletionProc		(SoundCompletionProcPtr	inProcPtr);

OSErr
	Sound_PlayAsyncStereo			(OSType const			inResourceType,
									 SInt16 const			inResourceID,
									 SoundInfoRef*			outRefOrNull);

OSErr
	Sound_PlaySyncStereo			(OSType const			inResourceType,
									 SInt16 const			inResourceID);



/**************************************************************************************************************************/
// WindowInfo.c
//
// This new, extremely powerful means of associating information with windows makes it possible to tag windows with a wide
// array of feature information and access it using a simple, Mac OS window pointer (WindowRef) or dialog pointer
// (DialogRef).  Use of attached procedure pointers allows standardization of the means of performing operations on all
// your windows, while each individual window can still be handled in a special way.  This class has very little deduction
// capability: that is, you generally cant call a "get" function until you call the corresponding "set" function at least
// once.  It is primarily a storage facility that allows you to associate a tremendous wealth of information with every
// window in your application, and, through this, allows you to greatly simplify regular tasks such as resizing windows.

typedef struct OpaqueWindowInfo { unsigned garbage; }*		WindowInfoRef;

typedef SInt32 WindowInfoDescriptor;
enum
{
	kInvalidWindowInfoDescriptor = -1722		// assign values to your application windows descriptors relative to this
};

/*!
Window Resize Notification Method

Whenever SizeWindow() is used to change the size of a window,
WindowInfo_CallWindowResizeResponderUPP() should be invoked on
the general Window Info reference for that window.  The function
is notified of the change in size of the window, and should respond
accordingly (usually by moving and/or resizing controls).  The
return value is user-defined.
*/
typedef pascal Boolean (*WindowResizeResponderProcPtr)	(WindowRef const	inWindow,
														 SInt32 const		inDeltaSizeX,
														 SInt32 const		inDeltaSizeY,
														 SInt32 const		inData);
#define InvokeWindowResizeResponderProc(userRoutine, inWindow, inDeltaSizeX, inDeltaSizeY, inData) \
	(*userRoutine)((inWindow), (inDeltaSizeX), (inDeltaSizeY), (inData))



//
// public methods
//

/*###############################################################
	CREATING, INITIALIZING AND DESTROYING WINDOW INFO DATA
###############################################################*/

WindowInfoRef
	WindowInfo_New								(void);

void
	WindowInfo_Dispose							(WindowInfoRef					inoutWindowFeaturesRef);

void
	WindowInfo_Init								(WindowInfoRef					inoutWindowFeaturesRef);

/*###############################################################
	POLYMORPHIC OPERATIONS USING GENERIC WINDOW INFO
###############################################################*/

SInt16
	WindowInfo_GrowWindow						(WindowRef						inWindow,
												 EventRecord*					inoutEventPtr);

/*###############################################################
	CALLING UNIVERSAL PROCEDURE POINTERS
###############################################################*/

SInt16
	WindowInfo_CallWindowResizeResponderUPP		(WindowRef const				inWindow,
												 SInt32 const					inDeltaSizeX, 
												 SInt32 const					inDeltaSizeY);

/*###############################################################
	ASSOCIATING WINDOW INFO WITH WINDOWS
###############################################################*/

WindowInfoRef
	WindowInfo_GetFromDialog					(DialogRef const				inDialog);

WindowInfoRef
	WindowInfo_GetFromWindow					(WindowRef const				inWindow);

void
	WindowInfo_SetForDialog						(DialogRef const				inDialog,
												 WindowInfoRef const			inWindowFeaturesRef);

void
	WindowInfo_SetForWindow						(WindowRef const				inWindow,
												 WindowInfoRef const			inWindowFeaturesRef);

/*###############################################################
	ACCESSING DATA IN WINDOW INFO STRUCTURES
###############################################################*/

void*
	WindowInfo_GetAuxiliaryDataPtr				(WindowInfoRef const				inWindowFeaturesRef);

WindowInfoDescriptor
	WindowInfo_GetWindowDescriptor				(WindowInfoRef const				inWindowFeaturesRef);

void
	WindowInfo_GetWindowMaximumDimensions		(WindowInfoRef const				inWindowFeaturesRef,
												 Point*								outMaximumSizePtr);

void
	WindowInfo_GetWindowMinimumDimensions		(WindowInfoRef const				inWindowFeaturesRef,
												 Point*								outMinimumSizePtr);

Rect*
	WindowInfo_GetWindowResizeLimits			(WindowInfoRef const				inWindowFeaturesRef);

Boolean
	WindowInfo_IsWindowFloating					(WindowInfoRef const				inWindowFeaturesRef);

void
	WindowInfo_SetAuxiliaryDataPtr				(WindowInfoRef						inWindowFeaturesRef,
												 void*								inAuxiliaryDataPtr);

void
	WindowInfo_SetDynamicResizing				(WindowInfoRef						inWindowFeaturesRefOrNullToSetGlobalFlag,
												 Boolean const						inLiveResizeEnabled);

void
	WindowInfo_SetWindowDescriptor				(WindowInfoRef						inWindowFeaturesRef,
												 WindowInfoDescriptor const			inNewWindowDescriptor);

void
	WindowInfo_SetWindowFloating				(WindowInfoRef const				inWindowFeaturesRef,
												 Boolean const						inIsFloating);

void
	WindowInfo_SetWindowResizeLimits			(WindowInfoRef						inWindowFeaturesRef,
												 short const						inMinimumHeight,
												 short const						inMinimumWidth,
												 short const						inMaximumHeight,
												 short const						inMaximumWidth);

void
	WindowInfo_SetWindowResizeResponderUPP		(WindowInfoRef						inWindowFeaturesRef,
												 WindowResizeResponderProcPtr const	inNewProc,
												 SInt32 const						inData);

#endif /* ifdef powerc */

#endif /* ifndef REZ */

#endif