/*###############################################################

	InternalEvents.h
	
	Lists constants and data types used for network events,
	which are implemented in "NetEvents.c".
	
	MacTelnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

#ifndef __INTERNALEVENTS__
#define __INTERNALEVENTS__

// Mac includes
#ifndef __MACTYPES__
#	include <MacTypes.h>
#endif



// constants
typedef SInt32 TelnetNetworkEventClass;
enum
{
	kTelnetNetworkEventClassUser = (1 << 0),
	kTelnetNetworkEventClassSession = (1 << 3),
	kTelnetNetworkEventClassConnection = (1 << 4)
};

// The following are for "kTelnetNetworkEventClassUser".
enum
{
	kUserEventDomainNameLookupBegin = 1,	// data1 = screen ID, data2 = DomainNameLookupDataPtr
	kUserEventDomainNameLookupEnd = 2,		// data1 = screen ID, data2 = DomainNameLookupDataPtr
	kUserEventFTPConnectionOpen = 20,		// data1 = -1 (unused), data2 is undefined
	kUserEventFTPConnectionClose = 21,		// data1 = -1 (unused), data2 is undefined
	kUserEventFTPTransferBegin = 22,		// data1 = -2 while transferring, -1 as soon as the
											//			last byte is transferred, 0 otherwise;
											//			data2 is undefined
	kUserEventFTPTransferEnd = 23,			// data1 = 0, data2 is undefined
	kUserEventRealGraphicsRedraw = 128,		// data1 = port number, data2 is undefined
	kUserEventPasteText = 130				// data1 = port number, data2 is undefined
};

// The following are for "kTelnetNetworkEventClassSession".
enum
{
	kSessionEventFTPTransferActive = 2,		// data1 = FTP data, data2 is undefined
	kSessionEventSendKeepAlivePackets = 3,	// data1 = undefined, data2 = undefined
	kSessionEventCloseFinished = 6,			// data1 = port number, data2 is undefined
	kSessionEventCloseFinishedWithError =	// data1 = port number, data2 is undefined
		kSessionEventCloseFinished + 1
};

// The following are for "kTelnetNetworkEventClassConnection".
enum
{
	// for all events in this class, data1 = port number, data2 is undefined
	kConnectionEventOpen = 1,
	kConnectionEventData = 2,
	kConnectionEventClose = 3,
	kConnectionEventFailure = 4,
	kConnectionEventKeepAlive = 5
};

enum
{
	HFTP = 21		// for FTP
};

#endif