/* IConfig.c */

#include "UniversalDefines.h"

// Mac includes
#include "ICAPI.h"
#include "ICKeys.h"
#include "ICTypes.h"

// resource includes
#include "GeneralResources.h"

// MacTelnet includes
#include "InternetConfig.h"



// The following constants MUST all have prefixes
// identical to the UniformResourceLocatorType
// defined in "InternetConfig.h", and suffixes as
// defined by the macro HELPER_BY_URL_KIND, below.
#define kNotURLHelper		null
#define kMailtoURLHelper	"\pHelpermailto"
#define kNewsURLHelper		"\pHelpernntp"
#define kNntpURLHelper		"\pHelpernntp"
#define kFtpURLHelper		"\pHelperftp"
#define kHttpURLHelper		"\pHelperhttp"
#define kGopherURLHelper	"\pHelpergopher"
#define kWaisURLHelper		"\pHelperwais"
#define kFingerURLHelper	"\pHelperfinger"
#define kWhoisURLHelper		"\pHelperwhois"
// the following are handled by Telnet, and hence are "null"
#define kTelnetURLHelper	null
#define kTn3270URLHelper	null

// macro to construct the above names out of a UniformResourceLocatorType
#define HELPER_BY_URL_KIND(kind)	(kind##Helper)



// variables
static ICInstance	gInternetConfigInstance;
static Boolean		gHaveIC = false;



/**
Call this method once before using any
other routines from this module.

(2.6)
*/
void
InternetConfig_Init	(void)
{
	ICError			ICerr;
	ICDirSpecArray	folderSpec;
	
	
	ICerr = ICStart(&gInternetConfigInstance, kApplicationCreatorSignature);
	
	folderSpec[0].vRefNum = -1; // -1 = search for system preferences
	folderSpec[0].dirID = 2;
	ICerr = ICFindConfigFile(gInternetConfigInstance, 1, (ICDirSpecArrayPtr)&folderSpec);
	gHaveIC = (ICerr == noErr);
}// Init


/**
Call this method when you are completely
done with this module.

(2.6)
*/
void
InternetConfig_Done		(void)
{
	if (gHaveIC)
	{
		(ICError)ICStop(gInternetConfigInstance);
	}
}// Done


/**
Returns the application signature of a helper
application capable of handling the specified
type of URL, or "kInternetConfigUnknownHelper"
if the specified URL is not handled.

(3.0)
*/
OSType
InternetConfig_HelperApplicationForURLType	(UniformResourceLocatorType		urlKind)
{
	OSType		result = kInternetConfigUnknownHelper;
	
	
	if (gHaveIC)
	{
		ConstStr255Param	key = null;
		
		
		// Use the macro HELPER_BY_URL_KIND to construct
		// a string on the fly; this saves the ludicrous
		// array of strings that were previously stored
		// as global variables in NCSA Telnet 2.6 and
		// not used for ANYTHING else.
		switch (urlKind)
		{
			case kNotURL:
				key = HELPER_BY_URL_KIND(kNotURL);
				break;
			
			case kMailtoURL:
				key = HELPER_BY_URL_KIND(kMailtoURL);
				break;
			
			case kNewsURL:
				key = HELPER_BY_URL_KIND(kNewsURL);
				break;
			
			case kNntpURL:
				key = HELPER_BY_URL_KIND(kNntpURL);
				break;
			
			case kFtpURL:
				key = HELPER_BY_URL_KIND(kFtpURL);
				break;
			
			case kHttpURL:
				key = HELPER_BY_URL_KIND(kHttpURL);
				break;
			
			case kGopherURL:
				key = HELPER_BY_URL_KIND(kGopherURL);
				break;
			
			case kWaisURL:
				key = HELPER_BY_URL_KIND(kWaisURL);
				break;
			
			case kTelnetURL:
				key = HELPER_BY_URL_KIND(kTelnetURL);
				break;
			
			case kTn3270URL:
				key = HELPER_BY_URL_KIND(kTn3270URL);
				break;
			
			case kFingerURL:
				key = HELPER_BY_URL_KIND(kFingerURL);
				break;
			
			case kWhoisURL:
				key = HELPER_BY_URL_KIND(kWhoisURL);
				break;
			
			case kQuickTimeURL:
				result = 'TVOD'; // QuickTime Player
				break;
			
			default:
				key = null;
				break;
		}
		if (key != null)
		{
			ICError				icErr;
			ICAttr				attr;
			long				size = 0L;
			ICAppSpec			icAppSpec;
			
			
			size = sizeof(icAppSpec);
			icErr = ICBegin(gInternetConfigInstance, icReadOnlyPerm); // icReadOnlyPerm = Telnet doesn't touch it
			icErr |= ICGetPref(gInternetConfigInstance, key, &attr, (Ptr)&icAppSpec, &size);
			icErr |= ICEnd(gInternetConfigInstance);
			
			if (icErr == noErr) result = icAppSpec.fCreator;
		}
	}
	
	return result;
}// HelperApplicationForURLType