/*###############################################################

	Keypads.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

// debugging - define the below to display the item number chosen in an alert box
//define KEYPADMENUS_SEE_CHOICES

#include "UniversalDefines.h"

// library includes
#include "DataAccessLib.h"
#ifdef KEYPADMENUS_SEE_CHOICES
#include "InterfaceLib.h"
#endif

// resource includes
#include "MenuResources.h"
#include "StringResources.h"

// MacTelnet includes
#include "ConnectionData.h"
#include "DialogUtilities.h"
#include "Keypads.h"
#include "Parse.h"
#include "PictureMDEFWindoid.h"
#include "Preferences.h"
#include "ResourceFileManager.h"
#include "ScreenManager.h"
#include "TelnetDescriptors.h"
#include "vsdata.h"
#include "Terminal.h"
#include "VSkeys.h"



// variables
static unsigned char gControlKeyMenuMap[] = // which menu items map to which key
{
/* 	no zero item ("no selection") */
	0,
/*	^@,		^A,		^B,		^C,		^D,		^E,		^F,		^G		*/
	0x00,	0x01,	0x02,	0x03,	0x04,	0x05,	0x06,	0x07,
/*	^H,		^I,		^J,		^K,		^L,		^M,		^N,		^O		*/
	0x08,	0x09,	0x0A,	0x0B,	0x0C,	0x0D,	0x0E,	0x0F,
/*	^P,		^Q,		^R,		^S,		^T,		^U,		^V,		^W,		*/
	0x10,	0x11,	0x12,	0x13,	0x14,	0x15,	0x16,	0x17,
/*	^X,		^Y,		^Z,		^[,		^\,		^],		^_,		^?		*/
	0x18,	0x19,	0x1A,	0x1B,	0x1C,	0x1D,	0x1E,	0x1F
};

static unsigned char gVT220FunctionKeyMenuMap[] = // which menu items map to which key
{
/* 	no zero item ("no selection") */
	0,
/*	f6,		f7,		f8,		f9,		f10,	*/
	VSF6,	VSF7,	VSF8,	VSF9,	VSF10,
/*	f11,	f12,	f13,	f14,	help	*/
	VSF11,	VSF12,	VSF13,	VSF14,	VSF15,
/*	do,	f17,	f18,	f19,	f20		*/
	VSF16,	VSF17,	VSF18,	VSF19,	VSF20
};

static unsigned char gVT220KeypadMenuMap[] = // which menu items map to which key
{
/* 	no zero item ("no selection") */
	0,
/*	find,	insert,	remove,	select,	prev,	next,	*/
	VSHELP,	VSHOME, VSPGUP,	VSDEL,	VSEND,	VSPGDN,
/*	up,		left,	down,	right,	*/
	VSUP,	VSLT,	VSDN,	VSRT,
/*	pf1,	pf2,	pf3,	pf4		*/
	VSF1,	VSF2,	VSF3,	VSF4,
/*	kp7,	kp8,	kp9,	minus,	*/
	VSK7,	VSK8,	VSK9,	VSKM,
/*	kp4,	kp5,	kp6,	comma,	*/
	VSK4,	VSK5,	VSK6,	VSKC,
/*	kp1,	kp2,	kp3,	enter,	*/
	VSK1,	VSK2,	VSK3,	VSKE,
/*	kp0,	period */
	VSK0,	VSKP
};

static PictureMDEFWindoidRef		gKeypadControlKeys = null,
									gKeypadFunctionKeys = null,
									gKeypadVT220Keys = null;

// internal methods
static ConnectionDataPtr	getCurrentScreen					(void);
#ifdef KEYPADMENUS_SEE_CHOICES
static void					seeChoice							(short		inMenuItemIndex);
#else
#define seeChoice(a)
#endif
static void					sendControlKeyFromMenuItemIndex		(short		inMenuItemIndex);
static void					sendFunctionKeyFromMenuItemIndex	(short		inMenuItemIndex);
static void					sendVT220KeyFromMenuItemIndex		(short		inMenuItemIndex);



//
// public methods
//

/*!
Creates a number of windoids that contain a set of
keys that can be clicked.   The windows are
automatically positioned according to any existing
user preferences (implicitly saved by Keypads_Done()),
and displayed (if the user left them visible when
last quitting).

(3.0)
*/
void
Keypads_Init	(void)
{
	Str255		windoidTitle;
	Boolean		windowIsVisible;
	Boolean		preferenceFound = false;
	Size		actualSize = 0L;
	
	
	// Control Keys windoid
	GetIndString(windoidTitle, rStringsMiscellaneous, siWindoidNameControlKeys);
	gKeypadControlKeys = PictureMDEFWindoid_New(windoidTitle,
												ResourceFileManager_GetResFile(kTelnetResourceFileApplication),
												kMDEFMenuIDControlKeys);
	WindowInfo_SetWindowDescriptor(PictureMDEFWindoid_GetWindowInfo(gKeypadControlKeys),
									kTelnetWindowDescriptorControlWindoid);
	preferenceFound = Preferences_ArrangeWindowNoResize(PictureMDEFWindoid_GetWindow(gKeypadControlKeys),
														kTelnetWindowPreferenceTag_ControlKeypad);
	unless (preferenceFound)
	{
		// no preference found, so pick a default location
		MoveWindow(PictureMDEFWindoid_GetWindow(gKeypadControlKeys), 200, 300, false/* activate */);
	}
	unless (Preferences_GetData(kTelnetPreferenceClassGeneral, kTelnetPreferenceTag_WasControlKeypadShowing,
									sizeof(windowIsVisible), &windowIsVisible,
									&actualSize) == kTelnetPrefsResultCodeSuccess)
	{
		windowIsVisible = false; // assume invisible if the preference cant be found
	}
	PictureMDEFWindoid_SetVisible(gKeypadControlKeys, windowIsVisible);
	
	// Function Keys windoid
	GetIndString(windoidTitle, rStringsMiscellaneous, siWindoidNameFunction);
	gKeypadFunctionKeys = PictureMDEFWindoid_New(windoidTitle,
													ResourceFileManager_GetResFile(kTelnetResourceFileApplication),
													kMDEFMenuIDFunction);
	WindowInfo_SetWindowDescriptor(PictureMDEFWindoid_GetWindowInfo(gKeypadFunctionKeys),
									kTelnetWindowDescriptorFunctionWindoid);
	preferenceFound = Preferences_ArrangeWindowNoResize(PictureMDEFWindoid_GetWindow(gKeypadFunctionKeys),
														kTelnetWindowPreferenceTag_FunctionKeypad);
	unless (preferenceFound)
	{
		// no preference found, so pick a default location
		MoveWindow(PictureMDEFWindoid_GetWindow(gKeypadFunctionKeys), 530, 70, false/* activate */);
	}
	unless (Preferences_GetData(kTelnetPreferenceClassGeneral, kTelnetPreferenceTag_WasFunctionKeypadShowing,
									sizeof(windowIsVisible), &windowIsVisible,
									&actualSize) == kTelnetPrefsResultCodeSuccess)
	{
		windowIsVisible = false; // assume invisible if the preference cant be found
	}
	PictureMDEFWindoid_SetVisible(gKeypadFunctionKeys, windowIsVisible);
	
	// VT220 Keys windoid
	GetIndString(windoidTitle, rStringsMiscellaneous, siWindoidNameKeypad);
	gKeypadVT220Keys = PictureMDEFWindoid_New(windoidTitle,
												ResourceFileManager_GetResFile(kTelnetResourceFileApplication),
												kMDEFMenuIDKeypad);
	WindowInfo_SetWindowDescriptor(PictureMDEFWindoid_GetWindowInfo(gKeypadVT220Keys),
									kTelnetWindowDescriptorKeypadWindoid);
	preferenceFound = Preferences_ArrangeWindowNoResize(PictureMDEFWindoid_GetWindow(gKeypadVT220Keys),
														kTelnetWindowPreferenceTag_VT220Keypad);
	unless (preferenceFound)
	{
		// no preference found, so pick a default location
		MoveWindow(PictureMDEFWindoid_GetWindow(gKeypadVT220Keys), 200, 70, false/* activate */);
	}
	unless (Preferences_GetData(kTelnetPreferenceClassGeneral, kTelnetPreferenceTag_WasVT220KeypadShowing,
									sizeof(windowIsVisible), &windowIsVisible,
									&actualSize) == kTelnetPrefsResultCodeSuccess)
	{
		windowIsVisible = false; // assume invisible if the preference cant be found
	}
	PictureMDEFWindoid_SetVisible(gKeypadVT220Keys, windowIsVisible);
}// Init


/*!
Invoke this routine when you are finished with
keypads.  The locations of all keypad windoids
are saved, and then the windoids are destroyed.

(3.0)
*/
void
Keypads_Done	(void)
{
	Boolean		windowIsVisible = false;
	
	
	Preferences_SaveWindowArrangement(Keypads_GetWindow(kKeypadTypeControlKeys), kTelnetWindowPreferenceTag_ControlKeypad);
	Preferences_SaveWindowArrangement(Keypads_GetWindow(kKeypadTypeFunctionKeys), kTelnetWindowPreferenceTag_FunctionKeypad);
	Preferences_SaveWindowArrangement(Keypads_GetWindow(kKeypadTypeVT220Keys), kTelnetWindowPreferenceTag_VT220Keypad);
	windowIsVisible = Keypads_IsVisible(kKeypadTypeControlKeys);
	(TelnetPrefsResultCode)Preferences_SetData(kTelnetPreferenceClassGeneral,
												kTelnetPreferenceTag_WasControlKeypadShowing,
												sizeof(Boolean), &windowIsVisible);
	windowIsVisible = Keypads_IsVisible(kKeypadTypeFunctionKeys);
	(TelnetPrefsResultCode)Preferences_SetData(kTelnetPreferenceClassGeneral,
												kTelnetPreferenceTag_WasFunctionKeypadShowing,
												sizeof(Boolean), &windowIsVisible);
	windowIsVisible = Keypads_IsVisible(kKeypadTypeVT220Keys);
	(TelnetPrefsResultCode)Preferences_SetData(kTelnetPreferenceClassGeneral,
												kTelnetPreferenceTag_WasVT220KeypadShowing,
												sizeof(Boolean), &windowIsVisible);
	(OSErr)Preferences_SaveData(kTelnetPreferenceClassGeneral); // write the implicit data to disk
	PictureMDEFWindoid_Dispose(&gKeypadControlKeys);
	PictureMDEFWindoid_Dispose(&gKeypadFunctionKeys);
	PictureMDEFWindoid_Dispose(&gKeypadVT220Keys);
}// Done


/*!
Returns a Mac OS window reference for the specified
keypads window.  If any problems occur, "null" is
returned.

(3.0)
*/
WindowRef
Keypads_GetWindow	(TelnetKeypadType const		inFromKeypad)
{
	WindowRef		result = null;
	
	
	switch (inFromKeypad)
	{
		case kKeypadTypeControlKeys:
			result = PictureMDEFWindoid_GetWindow(gKeypadControlKeys);
			break;
		
		case kKeypadTypeFunctionKeys:
			result = PictureMDEFWindoid_GetWindow(gKeypadFunctionKeys);
			break;
		
		case kKeypadTypeVT220Keys:
			result = PictureMDEFWindoid_GetWindow(gKeypadVT220Keys);
			break;
		
		default:
			break;
	}
	
	return result;
}// GetWindow


/*!
Sends the appropriate character to the session of
the frontmost window, based on what key was clicked
on by the user (if any).

(3.0)
*/
void
Keypads_HandleContentClick	(TelnetKeypadType const		inFromKeypad,
							 EventRecord*				inoutEventPtr)
{
	switch (inFromKeypad)
	{
		case kKeypadTypeControlKeys:
			sendControlKeyFromMenuItemIndex(PictureMDEFWindoid_HandleContentClick(gKeypadControlKeys, inoutEventPtr));
			break;
		
		case kKeypadTypeFunctionKeys:
			sendFunctionKeyFromMenuItemIndex(PictureMDEFWindoid_HandleContentClick(gKeypadFunctionKeys, inoutEventPtr));
			break;
		
		case kKeypadTypeVT220Keys:
			sendVT220KeyFromMenuItemIndex(PictureMDEFWindoid_HandleContentClick(gKeypadVT220Keys, inoutEventPtr));
			break;
		
		default:
			break;
	}
}// HandleContentClick


/*!
Returns "true" only if the specified keypad
is supposed to be visible.  This is not
necessarily the same as the result of
IsWindowVisible(Keypads_GetWindow()) - if
the application is backgrounded, for example,
the windoid may be set to visible (where this
routine would return "true"), but the actual
windoid is not visible.

(3.0)
*/
Boolean
Keypads_IsVisible	(TelnetKeypadType const		inFromKeypad)
{
	Boolean		result = null;
	
	
	switch (inFromKeypad)
	{
		case kKeypadTypeControlKeys:
			result = PictureMDEFWindoid_IsVisible(gKeypadControlKeys);
			break;
		
		case kKeypadTypeFunctionKeys:
			result = PictureMDEFWindoid_IsVisible(gKeypadFunctionKeys);
			break;
		
		case kKeypadTypeVT220Keys:
			result = PictureMDEFWindoid_IsVisible(gKeypadVT220Keys);
			break;
		
		default:
			break;
	}
	
	return result;
}// IsVisible


/*!
Changes the dimmed state of a windoid.

(3.0)
*/
void
Keypads_SetActive		(TelnetKeypadType const		inKeypad,
						 Boolean const				inIsActive)
{
	switch (inKeypad)
	{
		case kKeypadTypeControlKeys:
			PictureMDEFWindoid_SetActive(gKeypadControlKeys, inIsActive);
			break;
		
		case kKeypadTypeFunctionKeys:
			PictureMDEFWindoid_SetActive(gKeypadFunctionKeys, inIsActive);
			break;
		
		case kKeypadTypeVT220Keys:
			PictureMDEFWindoid_SetActive(gKeypadVT220Keys, inIsActive);
			break;
		
		default:
			break;
	}
}// SetActive


/*!
If "inIsBackgrounded" is true, the specified keypad
is hidden; conversely, if false, the keypad will be
displayed only if its visible state is true (that
is, it was showing before it was suspended).

(3.0)
*/
void
Keypads_SetSuspended	(TelnetKeypadType const		inKeypad,
						 Boolean const				inIsBackgrounded)
{
	switch (inKeypad)
	{
		case kKeypadTypeControlKeys:
			PictureMDEFWindoid_SetSuspended(gKeypadControlKeys, inIsBackgrounded);
			break;
		
		case kKeypadTypeFunctionKeys:
			PictureMDEFWindoid_SetSuspended(gKeypadFunctionKeys, inIsBackgrounded);
			break;
		
		case kKeypadTypeVT220Keys:
			PictureMDEFWindoid_SetSuspended(gKeypadVT220Keys, inIsBackgrounded);
			break;
		
		default:
			break;
	}
}// SetSuspended


/*!
Hides or shows the specified keypad.

(3.0)
*/
void
Keypads_SetVisibility	(TelnetKeypadType const		inKeypad,
						 Boolean const				inIsVisible)
{
	switch (inKeypad)
	{
		case kKeypadTypeControlKeys:
			PictureMDEFWindoid_SetVisible(gKeypadControlKeys, inIsVisible);
			break;
		
		case kKeypadTypeFunctionKeys:
			PictureMDEFWindoid_SetVisible(gKeypadFunctionKeys, inIsVisible);
			break;
		
		case kKeypadTypeVT220Keys:
			PictureMDEFWindoid_SetVisible(gKeypadVT220Keys, inIsVisible);
			break;
		
		default:
			break;
	}
}// SetVisibility


/*!
Hides the specified keypad if its visible, or
shows the keypad if its invisible.

(3.0)
*/
void
Keypads_ToggleVisibility	(TelnetKeypadType const		inKeypad)
{
	switch (inKeypad)
	{
		case kKeypadTypeControlKeys:
			PictureMDEFWindoid_ToggleVisible(gKeypadControlKeys);
			break;
		
		case kKeypadTypeFunctionKeys:
			PictureMDEFWindoid_ToggleVisible(gKeypadFunctionKeys);
			break;
		
		case kKeypadTypeVT220Keys:
			PictureMDEFWindoid_ToggleVisible(gKeypadVT220Keys);
			break;
		
		default:
			break;
	}
}// ToggleVisibility



//
// internal methods
//

/*!
This routine consistently determines the current
keyboard focus screen for all keypad menus.

(3.0)
*/
static ConnectionDataPtr
getCurrentScreen	(void)
{
	ConnectionDataPtr		result = null;
	
	
	if (ScreenManager_IsWindowAScreen(GetRealFrontWindow()))
	{
		result = ScreenManager_GetWindowConnectionData(GetRealFrontWindow());
	}
	return result;
}// getCurrentScreen


#ifdef KEYPADMENUS_SEE_CHOICES
/*!
This debugging routine displays an alert showing
you the item number of the selected keypad key.

(3.0)
*/
static void
seeChoice	(short		inMenuItemIndex)
{
	InterfaceLibAlertRef	box = null;
	Str255					dialogText,
							numText;
	
	
	NumToString((long)inMenuItemIndex, numText);
	PLstrcpy(dialogText, "\pI think item %a was picked.");
	StringUtilities_PMetaSetDefaults();
	StringUtilities_PBuild(dialogText, numText, EMPTY_PSTRING, EMPTY_PSTRING, EMPTY_PSTRING);
	
	box = Alert_New();
	Alert_SetHelpButton(box, false);
	Alert_SetParamsFor(box, kAlert_StyleOK);
	Alert_SetText(box, dialogText, "\pbut is it right?");
	Alert_SetType(box, kAlertNoteAlert);
	Alert_Display(box);
	Alert_Dispose(&box);
}// seeChoice
#endif


/*!
This routine maps the given menu item index from
the Control Keys menu into a key code, and then
sends that key code to the current session.

If the item index of 1 is passed, the floating
windoid for the Control Keys menu is displayed,
instead.

Passing 0 for "inMenuItemIndex" has no effect.

(2.6)
*/
static void
sendControlKeyFromMenuItemIndex		(short		inMenuItemIndex)
{
	seeChoice(inMenuItemIndex);
	if (inMenuItemIndex > 1)
	{
		ConnectionDataPtr		currentScreen = getCurrentScreen();
		
		
		if (currentScreen != null)
		{
			char	ck[1];
			
			
			ck[0] = gControlKeyMenuMap[inMenuItemIndex];
			Parse_TypeString(currentScreen, ck, 1, false/* record */);
		}
	}
	// zero means "do nothing"
}// sendControlKeyFromMenuItemIndex


/*!
This routine maps the given menu item index from
the Function menu into a key code, and then sends
that key code to the current session.

If the item index of 1 is passed, the floating
windoid for the Function menu is displayed, instead.

Passing 0 for "inMenuItemIndex" has no effect.

(2.6)
*/
static void
sendFunctionKeyFromMenuItemIndex	(short		inMenuItemIndex)
{
	seeChoice(inMenuItemIndex);
	if (inMenuItemIndex > 1)
	{
		ConnectionDataPtr		currentScreen = getCurrentScreen();
		
		
		if (currentScreen != null)
		{
			VSkbsend(currentScreen->vs, gVT220FunctionKeyMenuMap[inMenuItemIndex], currentScreen->echo);
		}
	}
	// zero means "do nothing"
}// sendFunctionKeyFromMenuItemIndex


/*!
This routine maps the given menu item index from
the Keypad menu into a key code, and then sends
that key code to the current session.

If the item index of 1 is passed, the floating
windoid for the Keypad menu is displayed, instead.

Passing 0 for "inMenuItemIndex" has no effect.

(2.6)
*/
static void
sendVT220KeyFromMenuItemIndex		(short		inMenuItemIndex)
{
	seeChoice(inMenuItemIndex);
	if (inMenuItemIndex > 1)
	{
		ConnectionDataPtr		currentScreen = getCurrentScreen();
		
		
		if (currentScreen != null)
		{
			VSkbsend(currentScreen->vs, gVT220KeypadMenuMap[inMenuItemIndex], currentScreen->echo);
		}
	}
	// zero means "do nothing"
}// sendVT220KeyFromMenuItemIndex