/*###############################################################

	ListenerModel.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#if 0

#include "UniversalDefines.h"

// MacTelnet includes
#include "ListenerModel.h"



// types
struct ListenerModel
{
	ListenerModelDescriptor		descriptor;	// user-defined identifier for this model
	ChainRef					eventList;	// list of "ListenerEvent" structures
};
typedef struct ListenerModel	ListenerModel;
typedef ListenerModel*			ListenerModelPtr;

struct ListenerEvent
{
	ListenerModelEventType		event;		// what event took place
	ChainRef					callbackList;
};
typedef struct ListenerEvent	ListenerEvent;
typedef ListenerEvent*			ListenerEventPtr;



//
// public methods
//

/*!
Creates a new listener model.  If any problems
occur, "null" is returned.

The model cannot be used for anything until you
add listeners to it.  Use methods such as
ListenerModel_AddListener() to manage the
listeners in the model, and respond to events
with ListenerModel_NotifyListeners().

(3.0)
*/
ListenerModelRef
ListenerModel_New	(ListenerModelDescriptor		inDescriptor)
{
	ListenerModelRef		result = (ListenerModelRef)Memory_NewPtr(sizeof(ListenerModel));
	
	
	if (result != null)
	{
		ListenerModelPtr	ptr = (ListenerModelPtr)result;
		
		
		ptr->descriptor = inDescriptor;
		ptr->eventList = Chain_New(kChainStyleSinglyLinked);
	}
	return result;
}// New


/*!
Destroys a listener model created with the
ListenerModel_New() routine.

(3.0)
*/
void
ListenerModel_Dispose	(ListenerModelRef*		inoutRefPtr)
{
	if (inoutRefPtr != null)
	{
		ListenerModelPtr		ptr = refAcquireLock(*inoutRefPtr);
		
		
		if (ptr != null)
		{
			ChainLinkRef		node = null;
			ListenerEventPtr	eventPtr = null;
			
			
			// destroy callback list
			node = Chain_GetFirstNode(ptr->eventList);
			do
			{
				eventPtr = Chain_GetNodeDataPtr(ptr->eventList, node);
				if (eventPtr != null)
				{
					Chain_Dispose(&eventPtr->callbackList);
					Memory_DisposePtr((Ptr*)&eventPtr);
				}
				node = Chain_GetNextNode(ptr->eventList, node);
			} while (node != null);
			Chain_Dispose(&ptr->eventList);
			Memory_DisposePtr((Ptr*)&ptr);
		}
		*inoutRefPtr = null;
	}
}// Dispose


//
// internal methods
//

/*!
To acquire a pointer to the internal structure,
given a reference to it, use this method.

(3.0)
*/
static ListenerModelPtr
refAcquireLock		(ListenerModelRef		inRef)
{
	return ((ListenerModelPtr)inRef);
}// refAcquireLock


/*!
To discontinue use of a pointer to the internal
structure, given a reference to it, use this
method.

(3.0)
*/
static void
refReleaseLock		(ListenerModelRef		inRef,
					 ListenerModelPtr*		inoutPtr)
{
	if (inoutPtr != null) *inoutPtr = null;
}// refReleaseLock

#endif