/*
**************************************************************************
	TelnetDictionaryText.h
	
	MacTelnet 3.0
	 1998-2000 by Kevin Grant
	
	This file contains text for Telnets scripting Dictionary, which is viewable by opening the dictionary of
	Telnet.app in the Mac OS Script Editor application.
	
	Please try to view this file in the font Geneva 9.
	
	IMPORTANT:		This file defines the scripting terminology that allows users to write scripts for Telnet.  The
							terms themselves should not only be translated, but made to the standard that Apple uses for
							localized versions of the Mac OS.  This makes Telnet scripting terms consistent with those
							terms used by scriptable programs in the Mac OS.
**************************************************************************
*/

#include "TelnetMenuItemNames.h"
#include "TelnetStrings.r"
#include "TelnetVersion.h"



#define PROPERTY_NAME_INHERITANCE												"<Inheritance>"
#define PROPERTY_DESCRIPTION_INHERITANCE_FROM_WINDOW				"also possesses properties of the window class"

/*********************************************
	REQUIRED SUITE
*********************************************/

#define SUITE_DESCRIPTION_REQUIRED					"Terms that every application should support."



/*********************************************
	SUBSET OF THE STANDARD SUITE
*********************************************/

#define SUITE_NAME_STANDARD							"Subset of the Standard Suite"
#define SUITE_DESCRIPTION_STANDARD				"Common terms used by this application."

// close
#define EVENT_NAME_CLOSE								"close"
#define EVENT_DESCRIPTION_CLOSE						"delete an object or discontinue an openable operation"
#define		EVENT_RESULT_DESCRIPTION_CLOSE								""
#define		EVENT_DIRECT_PARAM_DESCRIPTION_CLOSE					"the object to close"
#define		EVENT_PARAM_NAME_CLOSE_SAVING_IN							"saving in"
#define			EVENT_PARAM_DESCRIPTION_CLOSE_SAVING_IN \
	"[unimpl.] the file in which to save the object"

// count
#define EVENT_NAME_COUNT								"count"
#define EVENT_DESCRIPTION_COUNT						"return the number of elements of a particular class within an object"
#define		EVENT_RESULT_DESCRIPTION_COUNT							"the number of elements"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_COUNT					"the object whose elements are to be counted"
#define		EVENT_PARAM_NAME_COUNT_EACH							"each"
#define			EVENT_PARAM_DESCRIPTION_COUNT_EACH \
	"the class of the elements to be counted"

// data size
#define EVENT_NAME_DATASIZE							"data size"
#define EVENT_DESCRIPTION_DATASIZE				"return the size in bytes of an object"
#define		EVENT_RESULT_DESCRIPTION_DATASIZE						"the size of the object in bytes"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_DATASIZE					"the object whose data size is to be returned"
#define		EVENT_PARAM_NAME_DATASIZE_AS								"as"
#define			EVENT_PARAM_DESCRIPTION_DATASIZE_AS \
	"[unimpl.] the data type for which the size is calculated"

// delete
#define EVENT_NAME_DELETE								"delete"
#define EVENT_DESCRIPTION_DELETE					"delete an element from an object"
#define		EVENT_RESULT_DESCRIPTION_DELETE							""
#define		EVENT_DIRECT_PARAM_DESCRIPTION_DELETE					"the element to delete"

// duplicate
#define EVENT_NAME_DUPLICATE							"duplicate"
#define EVENT_DESCRIPTION_DUPLICATE				"clone object(s)"
#define		EVENT_RESULT_DESCRIPTION_DUPLICATE							"a clone of the specified object"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_DUPLICATE			"the object to create a clone of"

// exists
#define EVENT_NAME_EXISTS								"exists"
#define EVENT_DESCRIPTION_EXISTS					"test whether something exists within the context of "APPLICATION_SHORT
#define		EVENT_RESULT_DESCRIPTION_EXISTS								"true if it exists, false if not"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_EXISTS					"the object in question"

// get
#define EVENT_NAME_GET									"get"
#define EVENT_DESCRIPTION_GET							"return the data for an object"
#define		EVENT_RESULT_DESCRIPTION_GET									"the data from the object"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_GET						"the object whose data is to be returned"
#define		EVENT_PARAM_NAME_GET_AS											"as"
#define			EVENT_PARAM_DESCRIPTION_GET_AS \
	"[unimpl.] the desired types for the data, in order of preference"

// make
#define EVENT_NAME_MAKE									"make"
#define EVENT_DESCRIPTION_MAKE						"create a new element"
#define		EVENT_RESULT_DESCRIPTION_MAKE									"the new object"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_MAKE					""

#define		EVENT_PARAM_NAME_MAKE_NEW									"new"
#define		EVENT_PARAM_DESCRIPTION_MAKE_NEW \
	"[unimpl.] the class of the new element"

#define		EVENT_PARAM_NAME_MAKE_WITH_PROPERTIES				"with properties"
#define		EVENT_PARAM_DESCRIPTION_MAKE_WITH_PROPERTIES \
	"[unimpl.] the initial values for the properties of the element"

// print
#define EVENT_NAME_PRINT									"print"
#define EVENT_DESCRIPTION_PRINT						"send the specified object(s) to the default printer"
#define		EVENT_RESULT_DESCRIPTION_PRINT									""
#define		EVENT_DIRECT_PARAM_DESCRIPTION_PRINT					"the object(s) to print"

#define		EVENT_PARAM_NAME_PRINT_COPIES								"copies"
#define		EVENT_PARAM_DESCRIPTION_PRINT_COPIES \
	"[unimpl.] the number of copies "APPLICATION_SHORT" should print; default is 1"

// set
#define EVENT_NAME_SET									"set"
#define EVENT_DESCRIPTION_SET							"specify an objects data"
#define		EVENT_RESULT_DESCRIPTION_SET									""
#define		EVENT_DIRECT_PARAM_DESCRIPTION_SET						"the object to change"
#define		EVENT_PARAM_NAME_SET_TO											"to"
#define			EVENT_PARAM_DESCRIPTION_SET_TO \
	"[unimpl.] the new value"

// application class
#define CLASS_NAME_APPLICATION						"application"
#define CLASS_PLURAL_APPLICATION					"applications"
#define CLASS_DESCRIPTION_APPLICATION			APPLICATION_SHORT" itself, and all its global properties"
#define		PROPERTY_NAME_APPLICATION_ACTIVE_MACRO_SET							"active macro set"
#define		PROPERTY_DESCRIPTION_APPLICATION_ACTIVE_MACRO_SET					"[unimpl.] the current set of macros that "APPLICATION_SHORT" is using"
#define		PROPERTY_NAME_APPLICATION_CLIPBOARD										"clipboard"
#define		PROPERTY_DESCRIPTION_APPLICATION_CLIPBOARD									"[unimpl.] "APPLICATION_SHORT"s clipboard, the source of Paste operations and the destination of Cut and Copy operations"
#define		PROPERTY_NAME_APPLICATION_CURRENT_IP_ADDRESS						"current IP address"
#define		PROPERTY_DESCRIPTION_APPLICATION_CURRENT_IP_ADDRESS				"the 4-digit Internet Protocol address of the computer "APPLICATION_SHORT" is running on"
#define		PROPERTY_NAME_APPLICATION_FRONTMOST										"frontmost"
#define		PROPERTY_DESCRIPTION_APPLICATION_FRONTMOST								"whether "APPLICATION_SHORT" is the active application"
#define		PROPERTY_NAME_APPLICATION_PREFERENCES										"preferences"
#define		PROPERTY_DESCRIPTION_APPLICATION_PREFERENCES								"[unimpl.] all of the settings in "APPLICATION_SHORT" that can be customized"
#define		PROPERTY_NAME_APPLICATION_NAME													"name"
#define		PROPERTY_DESCRIPTION_APPLICATION_NAME											"the name of the "APPLICATION_SHORT" application program file"
#define		PROPERTY_NAME_APPLICATION_PRINTING_AVAILABLE						"printing available"
#define		PROPERTY_DESCRIPTION_APPLICATION_PRINTING_AVAILABLE					"whether UniversalPrintingLib is present (required to send selections to a printer)"
#define		PROPERTY_NAME_APPLICATION_SCRIPTS_MENU_ITEMS_FOLDER			"user Scripts Menu Items folder"
#define		PROPERTY_DESCRIPTION_APPLICATION_SCRIPTS_MENU_ITEMS_FOLDER	"the Scripts Menu Items folder in MacTelnets folder"
#define		PROPERTY_NAME_APPLICATION_STARTUP_ITEMS_FOLDER					"user Startup Items folder"
#define		PROPERTY_DESCRIPTION_APPLICATION_STARTUP_ITEMS_FOLDER			"the Startup Items folder in MacTelnets preferences folder"
#define		PROPERTY_NAME_APPLICATION_VERSION												"version"
#define		PROPERTY_DESCRIPTION_APPLICATION_VERSION										"the version of "APPLICATION_SHORT

// window class
#define CLASS_NAME_WINDOW								"window"
#define CLASS_PLURAL_WINDOW							"windows"
#define CLASS_DESCRIPTION_WINDOW					"a "APPLICATION_SHORT" window"
#define		PROPERTY_NAME_WINDOW_BOUNDS														"bounds"
#define		PROPERTY_DESCRIPTION_WINDOW_BOUNDS												"the area on the screen that the window occupies"
#define		PROPERTY_NAME_WINDOW_CLOSEABLE												"closeable"
#define		PROPERTY_DESCRIPTION_WINDOW_CLOSEABLE											"whether the window has a close box"
#define		PROPERTY_NAME_WINDOW_COLLAPSED												"collapsed"
#define		PROPERTY_DESCRIPTION_WINDOW_COLLAPSED											"whether the window is minimized into its title bar (MacOS8 and 9) or the dock (MacOSX)"
#define		PROPERTY_NAME_WINDOW_COLLAPSIBLE												"collapsible"
#define		PROPERTY_DESCRIPTION_WINDOW_COLLAPSIBLE										"whether the window can be minimized into its title bar (MacOS8 and 9) or the dock (MacOSX)"
#define		PROPERTY_NAME_WINDOW_FLOATING													"floating"
#define		PROPERTY_DESCRIPTION_WINDOW_FLOATING											"[unimpl.] whether the window always stays in front of all other windows"
#define		PROPERTY_NAME_WINDOW_INDEX															"index"
#define		PROPERTY_DESCRIPTION_WINDOW_INDEX													"[unimpl.] the number of the window in the list of all "APPLICATION_SHORT" windows, including floating windows"
#define		PROPERTY_NAME_WINDOW_MODAL														"modal"
#define		PROPERTY_DESCRIPTION_WINDOW_MODAL												"[unimpl.] whether the window blocks mouse clicks in other "APPLICATION_SHORT" windows"
#define		PROPERTY_NAME_WINDOW_NAME															"name"
#define		PROPERTY_DESCRIPTION_WINDOW_NAME													"the window title"
#define		PROPERTY_NAME_WINDOW_OBSCURED													"obscured"
#define		PROPERTY_DESCRIPTION_WINDOW_OBSCURED											"set to true if the window is behaving like a visible window, but is invisible"
#define		PROPERTY_NAME_WINDOW_POSITION													"position"
#define		PROPERTY_DESCRIPTION_WINDOW_POSITION											"the upper-left-hand corner of the window content area"
#define		PROPERTY_NAME_WINDOW_RESIZEABLE												"resizeable"
#define		PROPERTY_DESCRIPTION_WINDOW_RESIZEABLE											"[unimpl.] whether the window has a size box"
#define		PROPERTY_NAME_WINDOW_TITLE_BAR													"titled"
#define		PROPERTY_DESCRIPTION_WINDOW_TITLE_BAR											"whether the window can be moved"
#define		PROPERTY_NAME_WINDOW_VISIBLE														"visible"
#define		PROPERTY_DESCRIPTION_WINDOW_VISIBLE												"whether the window is showing"
#define		PROPERTY_NAME_WINDOW_ZOOMABLE													"zoomable"
#define		PROPERTY_DESCRIPTION_WINDOW_ZOOMABLE											"whether the window has a zoom box"
#define		PROPERTY_NAME_WINDOW_ZOOMED														"zoomed"
#define		PROPERTY_DESCRIPTION_WINDOW_ZOOMED												"[unimpl.] whether the window is occupying its application-defined boundaries, versus its user-defined boundaries"



/*********************************************
	UNIFORM RESOURCE LOCATOR SUITE
*********************************************/

#define SUITE_NAME_URL								"Subset of the Internet Suite"
#define SUITE_DESCRIPTION_URL						"Internet terms used by this application."

// handle URL
#define EVENT_NAME_HANDLE_URL						"handle URL"
#define EVENT_DESCRIPTION_HANDLE_URL			"open a connection to a domain specified in a uniform-resource-locator string"
#define		EVENT_RESULT_DESCRIPTION_HANDLE_URL						"result code - 0 for success, 1 for failure"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_HANDLE_URL			"a string or list of strings in uniform-resource-locator format; if "APPLICATION_SHORT" can handle a URL, it will; otherwise, it tries to launch an appropriate Internet helper application; if that fails, 1 is returned"



/*********************************************
	MACTELNET SUITE
*********************************************/

#define SUITE_NAME_APPLICATION						APPLICATION_SHORT" Suite"
#define SUITE_DESCRIPTION_APPLICATION			"Terms defined by "APPLICATION_FULL", incomplete!"

// connect
#define EVENT_NAME_NEW_CONNECTION				"connect"
#define EVENT_DESCRIPTION_NEW_CONNECTION	"open a connection to a single host; you can also use make"
#define		EVENT_RESULT_DESCRIPTION_NEW_CONNECTION				"the new connection"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_NEW_CONNECTION	""

#define		EVENT_PARAM_NAME_NEW_CONNECTION_TO					"to"
#define		EVENT_PARAM_DESCRIPTION_NEW_CONNECTION_TO \
	"a domain name, an IP address, or the name of a "SESSION_FAVORITES_SINGULAR" in "APPLICATION_SHORT

#define		EVENT_PARAM_NAME_NEW_CONNECTION_TITLE				"name"
#define		EVENT_PARAM_DESCRIPTION_NEW_CONNECTION_TITLE \
	"the title of the session window"

#define		EVENT_PARAM_NAME_NEW_CONNECTION_PORT				"port"
#define		EVENT_PARAM_DESCRIPTION_NEW_CONNECTION_PORT \
	"a port number to use; the default is the standard for the protocol"

#define		EVENT_PARAM_NAME_NEW_CONNECTION_PROTOCOL		"protocol"
#define		EVENT_PARAM_DESCRIPTION_NEW_CONNECTION_PROTOCOL \
	"[unimpl.] a supported protocol to use; the default is derived from the port number"

#define		EVENT_PARAM_NAME_NEW_CONNECTION_WAITING			"timeout"
#define		EVENT_PARAM_DESCRIPTION_NEW_CONNECTION_WAITING \
	"number of seconds to wait for opening before giving up"

#define		EVENT_PARAM_NAME_NEW_CONNECTION_PROXYSERVER	"via"
#define		EVENT_PARAM_DESCRIPTION_NEW_CONNECTION_PROXYSERVER \
	"[unimpl.] a configured proxy server to use"

// copy selected text
#define EVENT_NAME_COPY_SELECTED_TEXT				"copy selected text"
#define EVENT_DESCRIPTION_COPY_SELECTED_TEXT	"[unimpl.] place a copy of the selected text or graphics on the clipboard"
#define		EVENT_RESULT_DESCRIPTION_COPY_SELECTED_TEXT				""
#define		EVENT_DIRECT_PARAM_DESCRIPTION_COPY_SELECTED_TEXT	""

#define		EVENT_PARAM_NAME_COPY_SELECTED_TEXT_TABLE			"tabs in place of multiple spaces"
#define		EVENT_PARAM_DESCRIPTION_COPY_SELECTED_TEXT_TABLE \
	"whether to replace a certain number of consecutive space characters with a single tab, prior to copying; by default, no tabs are substituted"

// display alert
#define EVENT_NAME_DISPLAY_ALERT					"display alert"
#define EVENT_DESCRIPTION_DISPLAY_ALERT		"show an alert window"
#define		EVENT_RESULT_DESCRIPTION_DISPLAY_ALERT					"which button (if any) was chosen, and whether or not the alert timed out"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_DISPLAY_ALERT	"the text to display in normal print in the alert box"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_SOUND				"alert sound"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_SOUND \
	"whether an alert sound is played as the message window appears; by default, no sound is played"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_BUTTONS			"buttons"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_BUTTONS \
	"a list of up to three buttons, where the first button is the default one; pass an empty string in the second string to skip that button position; the default is a single OK button"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_BUTTON_KEYS	"button keys"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_BUTTON_KEYS \
	"[unimpl.] a list of up to three key equivalent characters, specified as single-character strings, that click their respective buttons; to not specify a key for a button in the list, pass an empty string for its key equivalent"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_GIVING_UP			"giving up after"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_GIVING_UP \
	"[unimpl.] number of seconds to wait before automatically dismissing the alert"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_HELP_BUTTON		"help button"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_HELP_BUTTON \
	"whether the alert has a help button; by default, it does not"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_HELP_TEXT			"help text"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_HELP_TEXT \
	"text to display in smaller print underneath the message text"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_WITH_TITLE		"name"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_WITH_TITLE \
	"a title for the alert window, displayed in huge text at the top of the window"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_DISRUPTION		"network disruption"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_DISRUPTION \
	"if false, network events continue to be serviced while the alert is displayed, which could cause other alerts to appear in front of this one"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_SPEECH				"speech"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_SPEECH \
	"whether to speak the alert text immediately when the alert is displayed"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_TITLE_BAR			"titled"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_TITLE_BAR \
	"whether the alert window can be moved"

#define		EVENT_PARAM_NAME_DISPLAY_ALERT_WITH_ICON			"with icon"
#define		EVENT_PARAM_DESCRIPTION_DISPLAY_ALERT_WITH_ICON \
	"a standard alert icon type; the default is no icon"

// launch Sherlock
#define EVENT_NAME_FIND_FILES							"launch Sherlock"
#define EVENT_DESCRIPTION_FIND_FILES					"Launches Sherlock (or Find File), prompting the user if the application cannot be found"
#define		EVENT_RESULT_DESCRIPTION_FIND_FILES							""
#define		EVENT_DIRECT_PARAM_DESCRIPTION_FIND_FILES			""

// send
#define EVENT_NAME_SEND_DATA						"send"
#define EVENT_DESCRIPTION_SEND_DATA				"Sends data to a connection"
#define		EVENT_RESULT_DESCRIPTION_SEND_DATA						""
#define		EVENT_DIRECT_PARAM_DESCRIPTION_SEND_DATA			"the string or list of strings to send (lists may also include special characters)"
#define		EVENT_DIRECT_PARAM_DESCRIPTION_SEND_DATA2			"the special character to send"

#define		EVENT_PARAM_NAME_SEND_LOCAL_ECHO						"echoing"
#define		EVENT_PARAM_DESCRIPTION_SEND_LOCAL_ECHO	 \
	"whether to echo the data that is sent (regardless of the current echo setting); default is the current session echo setting"

#define		EVENT_PARAM_NAME_SEND_NEWLINE								"newline"
#define		EVENT_PARAM_DESCRIPTION_SEND_NEWLINE	 \
	"whether to add an automatic carriage return (and possibly line feed) to the end of the specified string before sending; default is false"

#define		EVENT_PARAM_NAME_SEND_TIMEOUT								"timeout"
#define		EVENT_PARAM_DESCRIPTION_SEND_TIMEOUT	 \
	"[unimpl.] maximum number of seconds to wait before aborting and not finishing the data transmission"

#define		EVENT_PARAM_NAME_SEND_DATA_TO							"to"
#define		EVENT_PARAM_DESCRIPTION_SEND_DATA_TO \
	"[unimpl.] an open connection to send data to"

// wait
#define EVENT_NAME_WAIT									"wait"
#define EVENT_DESCRIPTION_WAIT							"Waits for specific data to arrive from a connection"
#define		EVENT_RESULT_DESCRIPTION_WAIT									""
#define		EVENT_DIRECT_PARAM_DESCRIPTION_WAIT						""

#define		EVENT_PARAM_NAME_WAIT_FROM									"from"
#define		EVENT_PARAM_DESCRIPTION_WAIT_FROM \
	"[unimpl.] an open connection to wait for data from"

#define		EVENT_PARAM_NAME_WAIT_FOR										"for"
#define		EVENT_PARAM_DESCRIPTION_WAIT_FOR \
	"[unimpl.] the data that must be received before continuing"

#define		EVENT_PARAM_NAME_WAIT_TIMEOUT								"timeout"
#define		EVENT_PARAM_DESCRIPTION_WAIT_TIMEOUT	 \
	"[unimpl.] maximum number of seconds to wait before aborting and not receiving the desired data"

// application preferences class
#define CLASS_NAME_APPLICATIONPREFERENCES						"application preferences"
#define CLASS_PLURAL_APPLICATIONPREFERENCES						"application preferences"
#define CLASS_DESCRIPTION_APPLICATIONPREFERENCES				"all user-configurable settings in "APPLICATION_SHORT", incomplete; you can only access one global instance of this class"
#define		PROPERTY_NAME_APPLICATIONPREFERENCES_CURSORSHAPE									"cursor shape"
#define		PROPERTY_DESCRIPTION_APPLICATIONPREFERENCES_CURSORSHAPE							"[unimpl.] what the text cursor looks like in a terminal window"
#define		PROPERTY_NAME_APPLICATIONPREFERENCES_SIMPLETELNET									"simplified user interface"
#define		PROPERTY_DESCRIPTION_APPLICATIONPREFERENCES_SIMPLETELNET							"[unimpl.] whether the user interface is scaled down"
#define		PROPERTY_NAME_APPLICATIONPREFERENCES_VISUALBELL										"visual bell"
#define		PROPERTY_DESCRIPTION_APPLICATIONPREFERENCES_VISUALBELL								"[unimpl.] whether terminal bell signals are inaudible"
#define		PROPERTY_NAME_APPLICATIONPREFERENCES_WINDOWSTACKINGORIGIN					"window stacking origin"
#define		PROPERTY_DESCRIPTION_APPLICATIONPREFERENCES_WINDOWSTACKINGORIGIN			"[unimpl.] the top-left corner of the first window arranged with "INAME_STACKWINDOWS

// clipboard window class
#define CLASS_NAME_CLIPBOARDWINDOW				"clipboard window"
#define CLASS_DESCRIPTION_CLIPBOARDWINDOW		APPLICATION_SHORT"s clipboard window, displaying scrap contents"
#define		PROPERTY_NAME_CLIPBOARDWINDOW_CONTENTS									"contents"
#define		PROPERTY_DESCRIPTION_CLIPBOARDWINDOW_CONTENTS							"[unimpl.] the data currently on the Clipboard"

// connection class
#define CLASS_NAME_CONNECTION							"connection"
#define CLASS_PLURAL_CONNECTION						"connections"
#define CLASS_DESCRIPTION_CONNECTION				"a connection, maintained by "APPLICATION_SHORT", to a host; you can close or make these"
#define		PROPERTY_NAME_CONNECTION_NAMEORIPADDRESS								"host"
#define		PROPERTY_DESCRIPTION_CONNECTION_NAMEORIPADDRESS						"[unimpl.] the name or IP address of the server"
#define		PROPERTY_NAME_CONNECTION_PORT													"port"
#define		PROPERTY_DESCRIPTION_CONNECTION_PORT											"[unimpl.] the port number used by the server"
#define		PROPERTY_NAME_CONNECTION_PROXY													"proxy"
#define		PROPERTY_DESCRIPTION_CONNECTION_PROXY											"[unimpl.] the proxy server to use for the connection"
#define		PROPERTY_NAME_CONNECTION_STATUS												"status"
#define		PROPERTY_DESCRIPTION_CONNECTION_STATUS										"[unimpl.] the current operation, reflected also by an icon in the "MENUNAME_WINDOW" menu"

// dialog reply class
#define CLASS_NAME_DIALOGREPLY							"dialog reply"
#define CLASS_PLURAL_DIALOGREPLY						"dialog replies"
#define CLASS_DESCRIPTION_DIALOGREPLY				"the result of displaying an alert or dialog box in "APPLICATION_SHORT
#define		PROPERTY_NAME_DIALOGREPLY_BUTTONRETURNED								"button returned"
#define		PROPERTY_DESCRIPTION_DIALOGREPLY_BUTTONRETURNED						"the name of the button selected"
#define		PROPERTY_NAME_DIALOGREPLY_GAVEUP												"gave up"
#define		PROPERTY_DESCRIPTION_DIALOGREPLY_GAVEUP										"whether the dialog timed out (closed itself)"

// file capture class
#define CLASS_NAME_FILECAPTURE							"file capture"
#define CLASS_PLURAL_FILECAPTURE						"file captures"
#define CLASS_DESCRIPTION_FILECAPTURE 				"a file capture in progress, which logs all incoming data into a text file; you can close or make these"
#define		PROPERTY_NAME_FILECAPTURE_CONNECTION										"connection"
#define		PROPERTY_DESCRIPTION_FILECAPTURE_CONNECTION								"[unimpl.] the connection from which incoming text is being captured"
#define		PROPERTY_NAME_FILECAPTURE_CREATOR											"creator"
#define		PROPERTY_DESCRIPTION_FILECAPTURE_CREATOR										"[unimpl.] the ID for the text editing application that will open the file (default is set by preferences)"
#define		PROPERTY_NAME_FILECAPTURE_FILE													"file"
#define		PROPERTY_DESCRIPTION_FILECAPTURE_FILE												"[unimpl.] the file where captured text is being saved"
#define		PROPERTY_NAME_FILECAPTURE_STATUS												"status"
#define		PROPERTY_DESCRIPTION_FILECAPTURE_STATUS										"[unimpl.] whether the capture is in progress, or complete"
#define		PROPERTY_NAME_FILECAPTURE_TEXT													"text"
#define		PROPERTY_DESCRIPTION_FILECAPTURE_TEXT											"[unimpl.] the captured text so far, if any"

// macro class
#define CLASS_NAME_MACRO									"macro"
#define CLASS_PLURAL_MACRO								"macros"
#define CLASS_DESCRIPTION_MACRO						"a string of text that can be typed using a key equivalent; you can duplicate or make these"
#define		PROPERTY_NAME_MACRO_KEY_EQUIVALENT											"key equivalent"
#define		PROPERTY_DESCRIPTION_MACRO_KEY_EQUIVALENT									"[unimpl.] the keystroke that will activate this macro"
#define		PROPERTY_NAME_MACRO_TEXT_STRING												"text"
#define		PROPERTY_DESCRIPTION_MACRO_TEXT_STRING										"[unimpl.] the text string that will be typed when the key equivalent is used"

// macro set class
#define CLASS_NAME_MACROSET							"macro set"
#define CLASS_PLURAL_MACROSET							"macro sets"
#define CLASS_DESCRIPTION_MACROSET					"a collection of ten macros, complete with key equivalents; you can delete, duplicate, or make these"
#define		PROPERTY_NAME_MACROSET_KEY_EQUIVALENTS									"key equivalents"
#define		PROPERTY_DESCRIPTION_MACROSET_KEY_EQUIVALENTS							"the keystrokes that will activate the respective macros in this set"

// proxy server class
#define CLASS_NAME_PROXYSERVER						"proxy"
#define CLASS_PLURAL_PROXYSERVER						"proxies"
#define CLASS_DESCRIPTION_PROXYSERVER				"a proxy server configuration; you can delete, duplicate, or make these"
#define		PROPERTY_NAME_PROXYSERVER_AUTHENTICATIONTYPE						"authentication"
#define		PROPERTY_DESCRIPTION_PROXYSERVER_AUTHENTICATIONTYPE				"[unimpl.] if supported by the method, the required authentication type"
#define		PROPERTY_NAME_PROXYSERVER_DIRECTDOMAINS								"direct domains"
#define		PROPERTY_DESCRIPTION_PROXYSERVER_DIRECTDOMAINS							"[unimpl.] a list of host names that should not be contacted via this proxy server"
#define		PROPERTY_NAME_PROXYSERVER_METHOD											"method"
#define		PROPERTY_DESCRIPTION_PROXYSERVER_METHOD										"[unimpl.] the best protocol supported by the proxy server"
#define		PROPERTY_NAME_PROXYSERVER_NAMEORIPADDRESS							"host"
#define		PROPERTY_DESCRIPTION_PROXYSERVER_NAMEORIPADDRESS					"[unimpl.] the name or IP address for the proxy server"
#define		PROPERTY_NAME_PROXYSERVER_PORT												"port"
#define		PROPERTY_DESCRIPTION_PROXYSERVER_PORT											"[unimpl.] a port number to use; the default is the standard for the protocol"
#define		PROPERTY_NAME_PROXYSERVER_USERNAME										"user name"
#define		PROPERTY_DESCRIPTION_PROXYSERVER_USERNAME									"[unimpl.] if supported by the method, the authenticating user name"

// session class
#define CLASS_NAME_SESSION							"session"
#define CLASS_PLURAL_SESSION							"sessions"
#define CLASS_DESCRIPTION_SESSION					"an open, interactive terminal window in "APPLICATION_SHORT"; you can close, duplicate, or make these"
#define		PROPERTY_NAME_SESSION_CONNECTION												"connection"
#define		PROPERTY_DESCRIPTION_SESSION_CONNECTION										"[unimpl.] the connection data (status, etc.)"
#define		PROPERTY_NAME_SESSION_WINDOW														"window"
#define		PROPERTY_DESCRIPTION_SESSION_WINDOW												"[unimpl.] the window housing the terminal for this session"

// terminal window class
#define CLASS_NAME_TERMINALWINDOW				"terminal window"
#define CLASS_PLURAL_TERMINALWINDOW			"terminal windows"
#define CLASS_DESCRIPTION_TERMINALWINDOW	"a special kind of window that displays a virtual terminal screen"
#define		PROPERTY_NAME_TERMINALWINDOW_COLUMNS												"columns"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_COLUMNS											"the horizontal display capacity, in terms of the number of characters"
#define		PROPERTY_NAME_TERMINALWINDOW_ROWS														"rows"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_ROWS												"the vertical display view size, in terms of the number of lines"
#define		PROPERTY_NAME_TERMINALWINDOW_SCROLLBACK											"scrollback"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_SCROLLBACK									"the number of lines saved, in addition to the rows"
#define		PROPERTY_NAME_TERMINALWINDOW_NORMAL_TEXT_COLOR							"normal text color"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_NORMAL_TEXT_COLOR						"the color to use for the foreground of screen cells that are not set to flash on and off"
#define		PROPERTY_NAME_TERMINALWINDOW_NORMAL_BACKGROUND_COLOR				"normal background color"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_NORMAL_BACKGROUND_COLOR		"the color to use for the background of screen cells that are not set to flash on and off"
#define		PROPERTY_NAME_TERMINALWINDOW_BLINKING_TEXT_COLOR							"blinking text color"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_BLINKING_TEXT_COLOR					"the color to use for the foreground of screen cells that are set to flash on and off"
#define		PROPERTY_NAME_TERMINALWINDOW_BLINKING_BACKGROUND_COLOR				"blinking background color"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_BLINKING_BACKGROUND_COLOR		"the color to use for the background of screen cells that are set to flash on and off"
#define		PROPERTY_NAME_TERMINALWINDOW_CONTENTS							"entire contents"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_CONTENTS						"the text in the window and the scrollback"
#define		PROPERTY_NAME_TERMINALWINDOW_SCREENCONTENTS							"screen contents"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_SCREENCONTENTS						"the text in the window"
#define		PROPERTY_NAME_TERMINALWINDOW_SCROLLBACKCONTENTS							"scrollback contents"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_SCROLLBACKCONTENTS						"the text in the scrollback buffer"
#define		PROPERTY_NAME_TERMINALWINDOW_FONT														"normal font"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_FONT												"[unimpl.] the font to use for drawing text in the window"
#define		PROPERTY_NAME_TERMINALWINDOW_FONT_SIZE												"normal font size"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_FONT_SIZE										"[unimpl.] the font point size to use for drawing text in the window"
#define		PROPERTY_NAME_TERMINALWINDOW_FONT_BOLD												"bold font"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_FONT_BOLD										"[unimpl.] the font to use for drawing boldface text in the window"
#define		PROPERTY_NAME_TERMINALWINDOW_FONT_SIZE_BOLD									"bold font size"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_FONT_SIZE_BOLD								"[unimpl.] the font point size to use for drawing boldface text in the window"
#define		PROPERTY_NAME_TERMINALWINDOW_BOLD_TEXT_COLOR									"bold text color"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_BOLD_TEXT_COLOR							"the color to use for the foreground of screen cells that are set to boldface"
#define		PROPERTY_NAME_TERMINALWINDOW_BOLD_BACKGROUND_COLOR					"bold background color"
#define		PROPERTY_DESCRIPTION_TERMINALWINDOW_BOLD_BACKGROUND_COLOR				"the color to use for the background of screen cells that are set to boldface"

// work space class
#define CLASS_NAME_WORKSPACE							"work space"
#define CLASS_PLURAL_WORKSPACE						"work spaces"
#define CLASS_DESCRIPTION_WORKSPACE				"definition of visible states of windows; you can delete or make these"
#define		PROPERTY_NAME_WORKSPACE_CONTENTS											"contents"
#define		PROPERTY_DESCRIPTION_WORKSPACE_CONTENTS									"[unimpl.] the windows belonging in this work space"
#define		PROPERTY_NAME_WORKSPACE_TITLE													"name"
#define		PROPERTY_DESCRIPTION_WORKSPACE_TITLE											"[unimpl.] the title of this work space"

// alert button type enumeration
#define ENUM_NAME_ALERT_BUTTON_TYPE_FIRST												"button 1"
#define ENUM_DESCRIPTION_ALERT_BUTTON_TYPE_FIRST										"indicates that the first button listed was chosen"
#define ENUM_NAME_ALERT_BUTTON_TYPE_SECOND											"button 2"
#define ENUM_DESCRIPTION_ALERT_BUTTON_TYPE_SECOND										"indicates that the second button listed was chosen"
#define ENUM_NAME_ALERT_BUTTON_TYPE_THIRD												"button 3"
#define ENUM_DESCRIPTION_ALERT_BUTTON_TYPE_THIRD										"indicates that the third button listed was chosen"
#define ENUM_NAME_ALERT_BUTTON_TYPE_HELP												"help button"
#define ENUM_DESCRIPTION_ALERT_BUTTON_TYPE_HELP											"indicates that the help button was chosen"
#define ENUM_NAME_ALERT_BUTTON_TYPE_TIMED_OUT										"timed out"
#define ENUM_DESCRIPTION_ALERT_BUTTON_TYPE_TIMED_OUT								"indicates that no button was chosen, the alert timed out (gave up)"

// alert icon enumeration
#define ENUM_NAME_ALERT_ICON_TYPE_NOTE														"note"
#define ENUM_DESCRIPTION_ALERT_ICON_TYPE_NOTE												"an icon of a talking face"
#define ENUM_NAME_ALERT_ICON_TYPE_STOP													"stop"
#define ENUM_DESCRIPTION_ALERT_ICON_TYPE_STOP												"an icon of a prohibiting hand"
#define ENUM_NAME_ALERT_ICON_TYPE_CAUTION												"caution"
#define ENUM_DESCRIPTION_ALERT_ICON_TYPE_CAUTION										"an exclamation point icon"

// capture status enumeration
#define ENUM_NAME_CAPTURE_STATUS_NOTBEGUN												"uninitiated"
#define ENUM_DESCRIPTION_CAPTURE_STATUS_NOTBEGUN										"specifies that a file capture has not yet begun"
#define ENUM_NAME_CAPTURE_STATUS_INPROGRESS											"capturing"
#define ENUM_DESCRIPTION_CAPTURE_STATUS_INPROGRESS									"specifies a file capture in progress"
#define ENUM_NAME_CAPTURE_STATUS_COMPLETE												"finished"
#define ENUM_DESCRIPTION_CAPTURE_STATUS_COMPLETE										"specifies a file capture that has stopped"

// connection method enumeration
#define ENUM_NAME_CONNECTION_METHOD_TELNET												"telnet"
#define ENUM_DESCRIPTION_CONNECTION_METHOD_TELNET										"specifies insecure telnet protocol for the new connection"
#define ENUM_NAME_CONNECTION_METHOD_FTP													"ftp"
#define ENUM_DESCRIPTION_CONNECTION_METHOD_FTP											"specifies file transfer protocol for the new connection"

// connection status enumeration
#define ENUM_NAME_CONNECTION_STATUS_WAITING_FOR_DNR							"unresolved"
#define ENUM_DESCRIPTION_CONNECTION_STATUS_WAITING_FOR_DNR					"specifies that a connection is currently looking up a host name"
#define ENUM_NAME_CONNECTION_STATUS_OPENING											"opening"
#define ENUM_DESCRIPTION_CONNECTION_STATUS_OPENING									"specifies that a connection is done looking up its host name, but is not yet active"
#define ENUM_NAME_CONNECTION_STATUS_ACTIVE												"active"
#define ENUM_DESCRIPTION_CONNECTION_STATUS_ACTIVE										"specifies that a connection is in use"
#define ENUM_NAME_CONNECTION_STATUS_CLOSED											"closed"
#define ENUM_DESCRIPTION_CONNECTION_STATUS_CLOSED										"specifies that a connection is closed, but its window is still open"
#define ENUM_NAME_CONNECTION_STATUS_NOT_IN_USE										"free"
#define ENUM_DESCRIPTION_CONNECTION_STATUS_NOT_IN_USE								"specifies that a connection is available for use with a new host"

// cursor shape enumeration
#define ENUM_NAME_CURSOR_SHAPE_BLOCK														"block"
#define ENUM_DESCRIPTION_CURSOR_SHAPE_BLOCK												"describes a square text cursor style"
#define ENUM_NAME_CURSOR_SHAPE_VERTICALBAR											"vertical bar"
#define ENUM_DESCRIPTION_CURSOR_SHAPE_VERTICALBAR										"describes a standard Mac OS insertion point text cursor style"
#define ENUM_NAME_CURSOR_SHAPE_VERTICALBARBOLD									"bold vertical bar"
#define ENUM_DESCRIPTION_CURSOR_SHAPE_VERTICALBARBOLD								"describes a thicker version of the normal vertical bar cursor"
#define ENUM_NAME_CURSOR_SHAPE_UNDERSCORE												"underline"
#define ENUM_DESCRIPTION_CURSOR_SHAPE_UNDERSCORE										"describes a text cursor that looks like an underscore"
#define ENUM_NAME_CURSOR_SHAPE_UNDERSCOREBOLD										"bold underline"
#define ENUM_DESCRIPTION_CURSOR_SHAPE_UNDERSCOREBOLD								"describes a thicker version of the normal underline cursor"

// macro key equivalent type enumeration
#define ENUM_NAME_MACRO_KEY_EQUIVALENT_TYPE_COMMAND_DIGIT				"command digit"
#define ENUM_DESCRIPTION_MACRO_KEY_EQUIVALENT_TYPE_COMMAND_DIGIT		"specifies that macro key equivalents are -0 through -9"
#define ENUM_NAME_MACRO_KEY_EQUIVALENT_TYPE_FUNCTION_KEYS				"function keys"
#define ENUM_DESCRIPTION_MACRO_KEY_EQUIVALENT_TYPE_FUNCTION_KEYS			"specifies that macro key equivalents are F1 through F10"

// proxy server authentication type enumeration
#define ENUM_NAME_AUTHENTICATIONTYPE_PASSWORD										"password"
#define ENUM_DESCRIPTION_CONNECTION_METHOD_PASSWORD								"specifies that a proxy server requires user name and password authentication"

// proxy server method enumeration
#define ENUM_NAME_PROXYSERVERMETHOD_SOCKS4											"SOCKS4"
#define ENUM_DESCRIPTION_PROXYSERVERMETHOD_SOCKS4									"specifies that a proxy server supports the SOCKS protocol, version 4"
#define ENUM_NAME_PROXYSERVERMETHOD_SOCKS4A											"SOCKS4A"
#define ENUM_DESCRIPTION_PROXYSERVERMETHOD_SOCKS4A									"specifies that a proxy server supports the protocol extensions to SOCKS 4"
#define ENUM_NAME_PROXYSERVERMETHOD_SOCKS5											"SOCKS5"
#define ENUM_DESCRIPTION_PROXYSERVERMETHOD_SOCKS5									"specifies that a proxy server supports the SOCKS protocol, version 5"

// special character enumeration
#define ENUM_NAME_SPECIAL_CHARACTER_AYT													"AYT"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_AYT											"are you there?"
#define ENUM_NAME_SPECIAL_CHARACTER_AO													"AO"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_AO											"abort output"
#define ENUM_NAME_SPECIAL_CHARACTER_BRK													"BRK"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_BRK											"break"
#define ENUM_NAME_SPECIAL_CHARACTER_CR													"CR"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_CR												"carriage return"
#define ENUM_NAME_SPECIAL_CHARACTER_IP														"IP"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_IP												"interrupt process"
#define ENUM_NAME_SPECIAL_CHARACTER_EC													"EC"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_EC												"erase character"
#define ENUM_NAME_SPECIAL_CHARACTER_EL													"EL"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_EL												"erase line"
#define ENUM_NAME_SPECIAL_CHARACTER_EOF													"EOF"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_EOF											"end of file"
#define ENUM_NAME_SPECIAL_CHARACTER_LF													"LF"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_LF												"line feed"
#define ENUM_NAME_SPECIAL_CHARACTER_SYNC												"SYNC"
#define ENUM_DESCRIPTION_SPECIAL_CHARACTER_SYNC											"sync"