/*
**************************************************************************
	TelnetMenuItemNames.h
	
	MacTelnet 3.0
	 1998-2003 by Kevin Grant
	
	This file contains strings for menu command names.
	
	Try to view this file in the font Monaco 10.
	
	Menu names are the words that appear in the menu bar at the top of
	the screen while the program is running, and are represented by the
	#defines beginning with MENUNAME_....
	
	Submenu names are used for menu items that have a triangle beside
	them (cascade menus or hierarchical menus), and are named using
	#defines beginning with SUBMENUNAME_....
**************************************************************************
*/

#include "TelnetVersion.h"

#define MENUNAME_FILE			"File"
#define MENUNAME_EDIT			"Edit"
#define MENUNAME_VIEW			"View"
#define MENUNAME_TERMINAL		"Terminal"
#define MENUNAME_KEYBOARD		"Keys"
#define MENUNAME_NETWORK		"Network"
#define MENUNAME_WINDOW			"Window"
#define MENUNAME_SCRIPTS		"Scripts"

#define SUBMENUNAME_FAV			"New Session From Favorites"
#define SUBMENUNAME_PREFS		"Preferences"
#define SUBMENUNAME_FONT		"Font (Normal Text)"
#define SUBMENUNAME_BOLDFONT	"Font (Bold Text)"
#define SUBMENUNAME_SIZE		"Size"
#define SUBMENUNAME_XLATION		"Translation Table"
#define SUBMENUNAME_TEK			"TEK"
#define SUBMENUNAME_WEB			"Internet Resources"

// attributes applicable to all in-menu titles (which are basically disabled items)
#define MARK_MENUTITLES			noMark
#define STYLE_MENUTITLES		bold | underline

// Notes on INAME_... Constants
//
// The suffixes of these constants is the capitalized
// version of everything following the corresponding
// "kCommand..." constant in "MenuResources.h".

/*
Some menu commands can appear in contextual menus.
These items have two versions: the menu item
version, and the contextual menu item version that
is more sensible for the context of the item.  Any
command that can appear in a contextual menu also
has a CINAME_... definition that should reflect
the item when it is applied to something in
particular.

Key names (in KEY_... defines) can be duplicated,
because they refer only to the character, not the
modifier keys.  For example, Command-W has the same
key as Command-Option-W, but these are different
menu item equivalents.

Try not to change MODIFIER_... constants.  They
are there to give flexibility in localization, but
are better left untouched.  Hopefully you can just
change the KEY_... and that will be sufficient.

Items whose text is dynamic is represented here as
an arbitrary description in angle brackets <>, and
NEED NOT be localized.  Following each such name
are an indented list of dynamic names, which should
be localized.

Items that display submenus are not listed below;
they are listed as SUBMENUNAME_... defines, above.
*/
// 
#define INAME_ABOUT						"About "APPLICATION_FULL

// File
#define INAME_OPENCONNECTION			"New Session"					// displays a dialog for opening one or more new connections
#define		KEY_OPENCONNECTION					"N"
#define		MODIFIERS_OPENCONNECTION			kMenuNoModifiers
#define INAME_LOADSET					"Open Saved Session"			// displays a file dialog box to load a session configuration set
#define		KEY_LOADSET							"O"
#define		MODIFIERS_LOADSET					kMenuNoModifiers
#define INAME_CLOSECONNECTION			"<close>"
#define		NORMAL_CLOSECONNECTION				"Close Window"			// destroys the frontmost window
#define		OPTION_CLOSECONNECTION				"Close All Windows"		// destroys all windows
#define		CINAME_CLOSECONNECTION				"Close This Window"		// Close Window command, in the context of a specific window
#define		KEY_CLOSECONNECTION					"W"
#define		MODIFIERS_CLOSECONNECTION			kMenuNoModifiers
#define		NORMAL_DISCONNECT					"Disconnect"			// disconnects the frontmost windows connection
#define		OPTION_DISCONNECT					"Disconnect All"		// closes all connections
#define		CINAME_DISCONNECT					"Disconnect"			// Disconnect command, in the context of a specific window
#define INAME_SAVESET					"Save Session"					// displays a file dialog box to save a session configuration set
#define		KEY_SAVESET							"S"
#define		MODIFIERS_SAVESET					kMenuNoModifiers
#define INAME_IMPORTMACROSET			"Import Macro Set"				// displays a file dialog box to import a macro set
#define		KEY_IMPORTMACROSET					"I"
#define		MODIFIERS_IMPORTMACROSET			kMenuOptionModifier
#define INAME_EXPORTCURRENTMACROSET		"Export Current Macro Set"		// displays a file dialog box to export the current set of macros
#define		KEY_EXPORTCURRENTMACROSET			"E"
#define		MODIFIERS_EXPORTCURRENTMACROSET		kMenuOptionModifier
#define INAME_PRINT						"Print Selection"				// displays a standard print dialog to print selected text or graphics
#define		KEY_PRINT							"P"
#define		MODIFIERS_PRINT						kMenuNoModifiers
#define INAME_PRINTONE					"Print Selection Now"			// prints one copy of selected text or graphics
#define		CINAME_PRINTONE						"Print Selection Now"
#define INAME_PRINTSCREEN				"Print Screen"
#define		KEY_PRINTSCREEN						"P"
#define		MODIFIERS_PRINTSCREEN				kMenuOptionModifier
#define INAME_PAGESETUP					"Page Setup"					// displays the standard Page Setup dialog box for printing selections
#define		KEY_PAGESETUP						"P"
#define		MODIFIERS_PAGESETUP					kMenuShiftModifier
#define INAME_QUIT						"Quit"							// standard text: item to quit the program
#define		KEY_QUIT							"Q"
#define		MODIFIERS_QUIT						kMenuNoModifiers

// Edit
#define INAME_UNDO						"Undo"							// standard text: item to undo an action
#define		INAME_UNDO_DIMENSIONCHANGES			"Undo Dimension Changes"
#define		INAME_UNDO_FORMATCHANGES			"Undo Format Changes"
#define		KEY_UNDO							"Z"
#define		MODIFIERS_UNDO						kMenuNoModifiers
#define INAME_REDO						"Redo"							// standard text: item to redo an action
#define		INAME_REDO_DIMENSIONCHANGES			"Redo Dimension Changes"
#define		INAME_REDO_FORMATCHANGES			"Redo Format Changes"
#define		KEY_REDO							"Z"
#define		MODIFIERS_REDO						kMenuShiftModifier
#define INAME_CUT						"Cut"							// standard text: item to cut a selection to the clipboard
#define		KEY_CUT								"X"
#define		MODIFIERS_CUT						kMenuNoModifiers
#define INAME_COPY						"Copy"							// standard text: item to copy a selection to the clipboard
#define		CINAME_COPY							"Copy Selected Text"
#define		KEY_COPY							"C"
#define		MODIFIERS_COPY						kMenuNoModifiers
#define INAME_COPYTABLE					"Copy Using Tabs For Spaces"	// copies text, but replaces spaces with tabs
#define		CINAME_COPYTABLE					"Copy Using Tabs For Spaces"
#define		KEY_COPYTABLE						"C"
#define		MODIFIERS_COPYTABLE					kMenuShiftModifier
#define INAME_TYPE						"Copy & Paste"					// types the selected text
#define		KEY_TYPE							"V"
#define		MODIFIERS_TYPE						kMenuShiftModifier
#define INAME_PASTE						"Paste"							// standard text: item to paste the clipboard contents into a selection
#define		CINAME_PASTE						"Paste Text"
#define		KEY_PASTE							"V"
#define		MODIFIERS_PASTE						kMenuNoModifiers
#define INAME_CLEAR						"Clear"							// standard text: item to delete a selection
#define INAME_SELECTALL					"Select All"					// standard text: item to select everything
#define		KEY_SELECTALL						"A"
#define		MODIFIERS_SELECTALL					kMenuNoModifiers
#define INAME_SHOWCLIPBOARD				"<clipboard>"
#define		SHOWNAME_SHOWCLIPBOARD				"Show Clipboard"		// standard text: item to display the clipboard window
#define		HIDENAME_SHOWCLIPBOARD				"Hide Clipboard"		// standard text: item to close the clipboard window
#define INAME_PREFERENCES				"Preferences"					// standard text: item to display consolidated Preferences dialog box
#define		KEY_PREFERENCES						","
#define		MODIFIERS_PREFERENCES				kMenuNoModifiers

// View
#define INAME_LARGESCREEN				"Wide Screen (132 x 24)"		// changes screen size to standard 132 x 24
#define INAME_SMALLSCREEN				"Standard Screen (80 x 24)"		// changes screen size to standard 80 x 24
#define INAME_SETSCREENSIZE				"Custom Screen Dimensions"
#define		CINAME_SETSCREENSIZE				"Screen Dimensions"
#define 	KEY_SETSCREENSIZE					"K"
#define		MODIFIERS_SETSCREENSIZE				kMenuNoModifiers
#define INAME_BIGGERTEXT				"Make Text Bigger"				// increases font size
#define 	KEY_BIGGERTEXT						"+"
#define		MODIFIERS_BIGGERTEXT				kMenuNoModifiers
#define INAME_SMALLERTEXT				"Make Text Smaller"				// decreases font size
#define 	KEY_SMALLERTEXT						"-"
#define		MODIFIERS_SMALLERTEXT				kMenuNoModifiers
#define INAME_FULLSCREEN				"Magnify and Maximize"			// set font size to fit the monitor as well as possible
#define INAME_FORMAT					"Format"
#define		CINAME_FORMAT						"Format"
#define		KEY_FORMAT							"T"
#define		MODIFIERS_FORMAT					kMenuNoModifiers

// Terminal
#define INAME_CAPTURETOFILE				"<capture to file>"
#define		NORMAL_CAPTURETOFILE				"Begin Capture To File"	// displayed when a capture is not in progress and can be started
#define		CHECKED_CAPTURETOFILE				"End Capture To File"		// displayed when a capture is already in progress for a window
#define		KEY_CAPTURETOFILE					"B"
#define		MODIFIERS_CAPTURETOFILE				kMenuNoModifiers
#define INAME_ECHO						"Local Echo"
#define INAME_WRAPMODE					"Line Wrap"
#define INAME_CLEARSCREENSAVESLINES		"Save Lines on Clear"
#define INAME_JUMPSCROLLING				"Jump Scrolling"
#define		KEY_JUMPSCROLLING					"J"
#define		MODIFIERS_JUMPSCROLLING				kMenuNoModifiers
#define INAME_SPEECH					"Speak Incoming Lines of Text"
#define		KEY_SPEECH							"V"
#define		MODIFIERS_SPEECH					kMenuOptionModifier
#define INAME_RESETTERMINAL				"Reset Terminal"
#define		KEY_RESETTERMINAL					"R"
#define		MODIFIERS_RESETTERMINAL				kMenuNoModifiers

// Keys
#define INAME_DELETEPRESSSENDSBACKSPACE	"Delete Sends Backspace"
#define		KEY_DELETEPRESSSENDSBACKSPACE		"["
#define		MODIFIERS_DELETEPRESSSENDSBACKSPACE	kMenuNoModifiers
#define INAME_DELETEPRESSSENDSDELETE	"Delete Sends Delete"
#define		KEY_DELETEPRESSSENDSDELETE			"]"
#define		MODIFIERS_DELETEPRESSSENDSDELETE	kMenuNoModifiers
#define INAME_EMACSARROWMAPPING			"Arrows Send EMACS Sequences"
#define INAME_LOCALPAGEUPDOWN			"Page Keys Affect Terminal Display Only"
#define INAME_SETKEYS					"Special Key Sequences"
#define		CINAME_SETKEYS						"Special Key Sequences"
#define INAME_TITLETRANSLATIONCHOICES	"Current Translation Table:"

// Network
#define INAME_SHOWNETWORKNUMBERS		"Get IP Address"
#define		KEY_SHOWNETWORKNUMBERS				"I"
#define		MODIFIERS_SHOWNETWORKNUMBERS		kMenuControlModifier
#define INAME_SENDINTERNETPROTOCOLNUMBER	"Send IP Address"
#define		KEY_SENDINTERNETPROTOCOLNUMBER		"I"
#define		MODIFIERS_SENDINTERNETPROTOCOLNUMBER	kMenuShiftModifier
#define INAME_SENDFTPCOMMAND			"Send FTP Command"
#define INAME_SENDSYNC					"Sync"
#define INAME_SENDBREAK					"Break"
#define INAME_SENDINTERRUPTPROCESS		"Interrupt Process"
#define INAME_SENDABORTOUTPUT			"Abort Output"
#define		KEY_SENDABORTOUTPUT					"."
#define		MODIFIERS_SENDABORTOUTPUT			kMenuNoModifiers
#define INAME_SENDAREYOUTHERE			"Are You There?"
#define		KEY_SENDAREYOUTHERE					"Y"
#define		MODIFIERS_SENDAREYOUTHERE			kMenuNoModifiers
#define INAME_SENDERASECHARACTER		"Erase Character"
#define		KEY_SENDERASECHARACTER				"E"
#define		MODIFIERS_SENDERASECHARACTER		kMenuNoModifiers
#define INAME_SENDERASELINE				"Erase Line"
#define		KEY_SENDERASELINE					"E"
#define		MODIFIERS_SENDERASELINE				kMenuShiftModifier
#define INAME_SENDENDOFFILE				"End of File"
#define INAME_WATCHFORACTIVITY			"Notify At Next Activity"
#define INAME_SUSPENDNETWORK			"Suspend (Scroll Lock)"

// Window
#define INAME_HIDEFRONTWINDOW			"Hide Front Window"
#define		CINAME_HIDEFRONTWINDOW				"Hide This Window"
#define INAME_HIDEOTHERWINDOWS			"Hide Others"
#define INAME_SHOWALLHIDDENWINDOWS		"Show All"
#define		KEY_SHOWALLHIDDENWINDOWS			"A"
#define		MODIFIERS_SHOWALLHIDDENWINDOWS		kMenuOptionModifier
#define INAME_STACKWINDOWS				"Arrange All in Front"
#define		CINAME_STACKWINDOWS					"Arrange All in Front"
#define INAME_NEXTWINDOW				"<next window>"				// <-- DONT LOCALIZE THIS
#define		NORMAL_NEXTWINDOW					"Bring Next to Front"
#define		SHIFT_NEXTWINDOW					"Bring Next to Front, Hide Previous"
#define		OPTION_NEXTWINDOW					"Bring Previous to Front"
#define		KEY_NEXTWINDOW						"`"
#define		MODIFIERS_NEXTWINDOW				kMenuNoModifiers
#define INAME_CHANGEWINDOWTITLE			"Rename"
#define		CINAME_CHANGEWINDOWTITLE			"Rename"
#define INAME_MINIMIZEFRONTWINDOW		"<minimize window>"
#define		SHOWNAME_MINIMIZEFRONTWINDOW		"Restore Window"
#define		HIDENAME_MINIMIZEFRONTWINDOW		"Minimize Window"
#define		KEY_MINIMIZEFRONTWINDOW				"M"
#define		MODIFIERS_MINIMIZEFRONTWINDOW		kMenuNoModifiers
#define INAME_SHOWSESSIONINSPECTOR		"<inspector>"					// <-- DONT LOCALIZE THIS
#define		SHOWNAME_SHOWSESSIONINSPECTOR		"Show Session Status"
#define		HIDENAME_SHOWSESSIONINSPECTOR		"Hide Session Status"
#define		KEY_SHOWSESSIONINSPECTOR			"I"
#define		MODIFIERS_SHOWSESSIONINSPECTOR		kMenuNoModifiers
#define INAME_SHOWFTPLOG				"<FTP log>"						// <-- DONT LOCALIZE THIS
#define		SHOWNAME_SHOWFTPLOG					"Show FTP Activity Log"
#define		HIDENAME_SHOWFTPLOG					"Hide FTP Activity Log"
#define INAME_SHOWCOMMANDLINE			"<command line>"				// <-- DONT LOCALIZE THIS
#define		SHOWNAME_SHOWCOMMANDLINE			"Show Floating Command Line"
#define		HIDENAME_SHOWCOMMANDLINE			"Hide Floating Command Line"
#define		KEY_SHOWCOMMANDLINE					"L"
#define		MODIFIERS_SHOWCOMMANDLINE			kMenuNoModifiers
#define INAME_SHOWCONTROLKEYS			"<control keys>"				// <-- DONT LOCALIZE THIS
#define		SHOWNAME_SHOWCONTROLKEYS			"Show Control Keys"
#define		HIDENAME_SHOWCONTROLKEYS			"Hide Control Keys"
#define INAME_SHOWFUNCTION				"<function>"					// <-- DONT LOCALIZE THIS
#define		SHOWNAME_SHOWFUNCTION				"Show Function Keys"
#define		HIDENAME_SHOWFUNCTION				"Hide Function Keys"
#define INAME_SHOWKEYPAD				"<keypad>"						// <-- DONT LOCALIZE THIS
#define		SHOWNAME_SHOWKEYPAD					"Show VT220 Keypad"
#define		HIDENAME_SHOWKEYPAD					"Hide VT220 Keypad"

// TEK (submenu)
#define INAME_TEKPAGECOMMAND			"PAGE Command"
#define		KEY_TEKPAGECOMMAND					"T"
#define		MODIFIERS_TEKPAGECOMMAND			kMenuOptionModifier
#define INAME_TEKPAGECLEARSSCREEN		"PAGE Clears Screen"

// Size (submenu)
#define INAME_SIZE4						"4"
#define INAME_SIZE9						"9"
#define INAME_SIZE10					"10"
#define INAME_SIZE11					"11"
#define INAME_SIZE12					"12"
#define INAME_SIZE14					"14"
#define INAME_SIZE18					"18"

// Internet Resources (submenu)
#define INAME_TELNETHOMEPAGE			"Visit the Official "APPLICATION_SHORT" Web Site"
#define INAME_TELNETAUTHORMAIL			"Send E-Mail to the Author"
#define INAME_TELNETSOURCELICENSE		"View Source Code License"
#define INAME_TELNETLIBRARYLICENSE		"View Runtime Libraries License"
#define INAME_TELNETPROJECTSTATUS		"Visit "APPLICATION_SHORT" Page on SourceForge"

// color box (pop-up menu)
#define INAME_COLORUSEPICKER			"Use Color Picker"		// item to display the standard Color Picker (choose-color) dialog box
#define INAME_COLORBLACK				"Black"
#define INAME_COLORDARKGRAY				"Dark Gray"
#define INAME_COLORSILVER				"Silver"
#define INAME_COLORWHITE				"White"
#define INAME_COLORBROWN				"Brown"
#define INAME_COLORDARKRED				"Dark Red"
#define INAME_COLORRED					"Red"
#define INAME_COLORDARKMAGENTA			"Dark Magenta"
#define INAME_COLORMAGENTA				"Magenta"
#define INAME_COLORDARKBLUE				"Purple"
#define INAME_COLORBLUE					"Blue"
#define INAME_COLORDARKGREEN			"Dark Green"
#define INAME_COLORGREEN				"Green"
#define INAME_COLORDARKCYAN				"Dark Cyan"
#define INAME_COLORCYAN					"Cyan"
#define INAME_COLORYELLOW				"Yellow"

// format type (pop-up menu)
#define INAME_FORMATNORMALTEXT			"Normal Text"
#define INAME_FORMATBOLDTEXT			"Bold Text"
#define INAME_FORMATBLINKINGTEXT		"Blinking Text"

// macro set (pop-up menu)
#define INAME_MACROSET1					"Macro Set 1"
#define INAME_MACROSET2					"Macro Set 2"
#define INAME_MACROSET3					"Macro Set 3"
#define INAME_MACROSET4					"Macro Set 4"
#define INAME_MACROSET5					"Macro Set 5"

// proxy server protocol (pop-up menu)
#define INAME_PROXYPROTOCOLSOCKS4		"SOCKS 4"
#define INAME_PROXYPROTOCOLSOCKS4A		"SOCKS 4A"
#define INAME_PROXYPROTOCOLSOCKS5		"SOCKS 5"

// name of Help items in contextual menus
#define CINAME_HELP						"Help"
#define CINAME_SHOWBALLOONS				"Show Balloons"
#define CINAME_HIDEBALLOONS				"Hide Balloons"
