/*###############################################################

	MacroSetupDialog.h
	
	Implements the Macro Setup dialog box.
	
	The entire dialog box for managing macros has now been
	re-written.  Not only is it Appearance-savvy, movable and
	*resizable*, but it contains tabs for switching between sets
	of macros (a new feature in Telnet 3.0)!
	
	This module assumes that the macro buttons are in
	consecutive order (the first item index is equal to
	iFirstMacroButton and the last item index is equal to
	iLastMacroButton).  Also, the macro text fields should be
	indexed consecutively (the first item index is equal to
	iFirstMacroTextField).
	
	The new code in this module, combined with the new code in
	"Macros.c", should make it about as easy as possible to
	make macros more flexible in the future.  That is, you
	might want to support more sets (Telnet 3.0 supports five),
	or you might want to make more macros in each set (Telnet
	has always supported ten macros in a set).
	
	MacTelnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __MACROSETUPDIALOG__
#define __MACROSETUPDIALOG__

// MacTelnet includes
#include "TelnetMacros.h"



//
// public methods
//

void
	MacroSetupDialog_Init				(void);

void
	MacroSetupDialog_Done				(void);

Boolean
	MacroSetupDialog_Display			(void);

MacroSet
	MacroSetupDialog_GetDisplayedSet	(void);

void
	MacroSetupDialog_ReceiveDrop		(DragReference		inDragRef,
										 unsigned char*		inData,
										 Size				inDataSize);

void
	MacroSetupDialog_SetDisplayedMode	(TelnetMacroMode	inMacroKeyEquivalents);

void
	MacroSetupDialog_UpdateTextField	(unsigned char*		inoutMacroText,
										 const int			inZeroBasedMacroNumber);

void
	MacroSetupDialog_UpdateTextFields	(void);

#endif