/*###############################################################

	MenuResources.h
	
	This important file contains descriptions of menu resources,
	including menu bars and menu resources, as well as menu item
	command IDs and indices of menus in the main menu array.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.

###############################################################*/

#ifndef __MENURESOURCES__
#define __MENURESOURCES__

// Base resource IDs - menus of the same type are offset from the
// base.  If you run out of space, change ONLY one of the offsets
// below to allow more room, and all should be well.
//
// IMPORTANT:	MacTelnet reserves a few IDs beyond 128 for submenus.
//
//				The window header base ID should be end-padded well,
//				because the terminal window header code uses auto-
//				incrementing to derive unique menu IDs.
#define POPUPMENU_BASE_ID			160
#define	MENU_BASE_ID				512
#define	MENU_SIMPLE_BASE_ID			256
#define	MENU_INTERNAL_BASE_ID		600

/*
*******************************************
	MENUS
*******************************************
*/

// 'MBAR' resource IDs
#define kMenuBarIDMain				256

// startup menus
#define kMenuIDAppleInit			(MENU_BASE_ID + 100)
#define kMenuIDFileInit				(MENU_BASE_ID + 101)
#define kMenuIDEditInit				(MENU_BASE_ID + 102)

// main menus
#define	kMenuIDApple				MENU_BASE_ID
#define	kMenuIDFile					(MENU_BASE_ID + 1)
#define	kMenuIDEdit					(MENU_BASE_ID + 2)
#define	kMenuIDAppearance			(MENU_BASE_ID + 3)
#define	kMenuIDSession				(MENU_BASE_ID + 4)
#define	kMenuIDKeyboard				(MENU_BASE_ID + 5)
#define	kMenuIDConnections			(MENU_BASE_ID + 6)
#define	kMenuIDNetwork				(MENU_BASE_ID + 7)
#define	kMenuIDScripts				(MENU_BASE_ID + 8)

// main menus, Simple Telnet mode
#define	kMenuIDSimpleFile			MENU_SIMPLE_BASE_ID
#define	kMenuIDSimpleEdit			(MENU_SIMPLE_BASE_ID + 1)

// Hierarchical menus - these cannot use Rez arithmetic due to the fact that Rez does not expect to find non-numbers for submenu IDs.
// The decimal values below should have corresponding hexadecimal character codes in the kChar... constants that follow them.
#define	kSubmenuIDOpen				128
#define	kSubmenuIDTranslation		129
#define	kSubmenuIDTektronix			130
#define	kSubmenuIDTelnetOnTheWeb	131

#ifdef REZ
// these are required so MacTelnet hierarchical menus work prior to Mac OS 8.5 (i.e. the old way)
#define	kCharSubmenuIDOpen				"\0x80"
#define	kCharSubmenuIDTranslation		"\0x81"
#define	kCharSubmenuIDTektronix			"\0x82"
#define	kCharSubmenuIDTelnetOnTheWeb	"\0x83"
#endif

// menus for pop-up buttons and menu bevel buttons
#define kPopUpMenuIDConnectRecent		(POPUPMENU_BASE_ID + 1)
#define kPopUpMenuIDFormatType			(POPUPMENU_BASE_ID + 2)
#define kPopUpMenuIDConfigurationType	(POPUPMENU_BASE_ID + 3)
#define kPopUpMenuIDEditMacroSet		(POPUPMENU_BASE_ID + 4)
#define kPopUpMenuIDFont				(POPUPMENU_BASE_ID + 5)
#define kPopUpMenuIDSize				(POPUPMENU_BASE_ID + 6)
#define kPopUpMenuIDProxyProtocol		(POPUPMENU_BASE_ID + 7)
#define kPopUpMenuIDColorBox			(POPUPMENU_BASE_ID + 8)

// menus with custom menu definition procedures
#define	kMDEFMenuIDKeypad			1000
#define	kMDEFMenuIDMacros			1001
#define	kMDEFMenuIDFunction			1002
#define	kMDEFMenuIDControlKeys		1003

// menus created internally by MacTelnet
#define kMenuIDNewSessionsProxies				MENU_INTERNAL_BASE_ID
#define kMenuIDNewSessionsTerminals				(MENU_INTERNAL_BASE_ID + 1)
#define kMenuIDPrefsConfigurationsListDummy		(MENU_INTERNAL_BASE_ID + 2)
#define kMenuIDSessionEditorTerminals			(MENU_INTERNAL_BASE_ID + 3)
#define kMenuIDSessionEditorTranslationTables	(MENU_INTERNAL_BASE_ID + 4)
#define kMenuIDTerminalEditorFonts				(MENU_INTERNAL_BASE_ID + 5)
#define kMenuIDFTPUserDummyMenu					(MENU_INTERNAL_BASE_ID + 6) /* temp */

// array indices - and, in the case of the first X number of menus, the exact order of menus in the menu bar (which is relied upon)
#define kMenuPositionApple				0
#define kMenuPositionFile				1
#define kMenuPositionEdit				2
#define kMenuPositionAppearance			3
#define kMenuPositionSession			4
#define kMenuPositionKeyboard			5
#define kMenuPositionNetwork			6
#define kMenuPositionConnections		7
#define kMenuPositionScripts			8
#define kMenuPositionFont				9
#define kMenuPositionOpen				10
#define kMenuPositionTranslation		11
#define kMenuPositionTektronix			12
#define kMenuPositionTelnetOnTheWeb		13
#define kMenuPositionSimpleFile			14
#define kMenuPositionSimpleEdit			15

// IMPORTANT -	The following is the number of spaces allocated in the global menu array (see Menus.c).
//				Dont include the three dummy menus (kMenuIDAppleInit, kMenuIDFileInit, kMenuIDEditInit),
//				however, because they dont stick around after startup time.  The number below should be
//				basically the count of the number of array indices constants above.
#define	kMenuResourceCount			16



//
// menu command IDs - must be unique for all items!
//

// Application (Apple) menu
#define kCommandAboutTelnet					20
#define kCommandTelnetOnTheWeb				21

// Session (File) menu
#define kCommandOpenConnection				100
#define kCommandOpenConnectionFavorite		101
#define kCommandLoadSet						102
#define kCommandCloseConnection				103
#define kCommandSaveSet						104
#define kCommandAddToFavorites				105
#define kCommandImportMacroSet				106
#define kCommandExportCurrentMacroSet		107
#define kCommandPrint						108
#define kCommandPrintOne					109
#define kCommandPrintScreen					110
#define kCommandPageSetup					111
#define kCommandQuit						112

#define kCommandFind						120

// Edit menu
#define kCommandUndo						200
#define kCommandRedo						201
#define kCommandCut							202
#define kCommandCopy						203
#define kCommandCopyTable					204
#define kCommandType						205
#define kCommandPaste						206
#define kCommandClear						207
#define kCommandSelectAll					208
#define kCommandShowClipboard				209
#define kCommandPreferences					210

// View menu
#define kCommandLargeScreen					300
#define kCommandSmallScreen					301
#define kCommandSetScreenSize				302
#define kCommandBiggerText					303
#define kCommandSmallerText					304
#define kCommandFullScreen					305
#define kCommandFormat						306

// Terminal menu
#define kCommandCaptureToFile				400
#define kCommandEcho						401
#define kCommandWrapMode					402
#define kCommandClearScreenSavesLines		403
#define kCommandJumpScrolling				404
#define kCommandTektronix					405
#define kCommandSpeechEnabled				406
#define kCommandResetTerminal				407

// Map menu
#define kCommandDeletePressSendsBackspace	500
#define kCommandDeletePressSendsDelete		501
#define kCommandEMACSArrowMapping			502
#define kCommandLocalPageUpDown				503
#define kCommandSetKeys						504
#define kCommandTitleTranslationChoices		505

// Network menu
#define kCommandSendFTPCommand				600
#define kCommandSendInternetProtocolNumber	601
#define kCommandShowNetworkNumbers			602
#define kCommandSendSync					603
#define kCommandSendBreak					604
#define kCommandSendInterruptProcess		605
#define kCommandSendAbortOutput				606
#define kCommandSendAreYouThere				607
#define kCommandSendEraseCharacter			608
#define kCommandSendEraseLine				609
#define kCommandSendEndOfFile				610
#define kCommandWatchForActivity			611
#define kCommandSuspendNetwork				612

// Window menu
#define kCommandHideFrontWindow				700
#define kCommandHideOtherWindows			701
#define kCommandShowAllHiddenWindows		702
#define kCommandStackWindows				703
#define kCommandNextWindow					704
#define kCommandChangeWindowTitle			705
#define kCommandMinimizeWindow				706
#define kCommandShowSessionInspector		707
#define kCommandShowFTPLog					708
#define kCommandShowCommandLine				709
#define kCommandShowMacros					710
#define kCommandShowKeypad					711
#define kCommandShowFunction				712
#define kCommandShowControlKeys				713

// TEK submenu
#define kCommandTEKPageCommand				6000
#define kCommandTEKPageClearsScreen			6001

// Internet Resources submenu
#define kCommandTelnetHomePage				6100
#define kCommandTelnetAuthorMail			6101
#define kCommandTelnetSourceLicense			6102
#define kCommandTelnetLibraryLicense		6103
#define kCommandTelnetProjectStatus			6104

#endif
