/*###############################################################

	PictureMDEFWindoid.c
	
	MacTelnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// standard-C includes
#include <limits.h>

// Mac includes
#include <Appearance.h>
#include <ControlDefinitions.h>
#include <Menus.h>
#include <Palettes.h>
#include <PLStringFuncs.h>
#include <Resources.h>
#include <QuickDraw.h>
#include <TextUtils.h>

// library includes
#include "InterfaceLib.h"

// resource includes
#include "ControlResources.h"
#include "DialogResources.h"
#include "StringResources.h"

// MacTelnet includes
#include "PictureMDEFWindoid.h"
#include "EventLoop.h"
#include "TelnetDescriptors.h"
#include "TelnetMenus.h"



// types
struct PictureMDEFWindoid
{
	SInt16					lockCount;		// how many locks are on the handle
	WindowRef				window;
	WindowInfoRef			windowInfo;
	MenuHandle				fakeMenuHandle;	// used to trick the MDEF into doing all the work
	ControlRef				root,
							picturePane;
	Boolean					isSuspended,
							isShowing;
};
typedef struct PictureMDEFWindoid	PictureMDEFWindoid;
typedef PictureMDEFWindoid*			PictureMDEFWindoidPtr;
typedef PictureMDEFWindoidPtr*		PictureMDEFWindoidHandle;

// internal methods
static pascal void				mainMDEF				(short					message,
														 MenuHandle				theMenu,
														 Rect*					theMenuRect,
														 Point					hitPt,
														 short*					whichItem);
static pascal void				pictureUserPaneDraw		(ControlRef				inControl,
														 SInt16					inPart);
static PictureMDEFWindoidPtr	refAcquireLock			(PictureMDEFWindoidRef	inRef);
static void						refReleaseLock			(PictureMDEFWindoidRef	inRef,
														 PictureMDEFWindoidPtr*	inoutPtr);



//
// public methods
//

/*!
Use this method to create a new Picture MDEF Windoid.
It is your responsibility to set a valid window
descriptor by using PictureMDEFWindoid_GetWindowInfo()
(if desired), and your responsibility to set the
window location properly.  The windoid size is set to
the size of the picture it is to contain.

(3.0)
*/
PictureMDEFWindoidRef
PictureMDEFWindoid_New	(ConstStr255Param		inWindowTitle,
						 UInt16					inResourceFile,
						 UInt16					inResourceID)
{
	PictureMDEFWindoidRef		result = (PictureMDEFWindoidRef)Memory_NewHandle(sizeof(PictureMDEFWindoid));
	
	
	if (result != null)
	{
		PictureMDEFWindoidPtr		ptr = refAcquireLock(result);
		Rect						bounds;
		short						oldResourceFile = CurResFile();
		
		
		// create the picture windoid
		UseResFile(inResourceFile);
		SetRect(&bounds, 0, 0, 0, 0);
		if (FlagManager_Test(kFlagCarbonWindowManager))
		{
			OSErr		error = noErr;
			
			
			error = CreateNewWindow(kFloatingWindowClass, kWindowStandardFloatingAttributes,
									&bounds, &ptr->window);
			if (error == noErr) SetWTitle(ptr->window, inWindowTitle);
		}
		else
		{
			ptr->window = NewCWindow(null, &bounds, inWindowTitle,
									false/* visible */,
									kWindowFloatProc,
									kInFront,
									true/* close  box */,
									0L/* reference constant - MUST be 0 for Window Info */);
		}
		
		// create the windows controls
		(OSErr)CreateRootControl(ptr->window, &ptr->root);
		ptr->picturePane = GetNewControl(kIDKeypadContentUserPane, ptr->window);
		//SetControlReference(ptr->picturePane, REINTERPRET_CAST(result, SInt32));
		//{
		//	ControlUserPaneDrawUPP		upp = NewControlUserPaneDrawUPP(pictureUserPaneDraw);
		//	
		//	
		//	(OSErr)SetControlData(ptr->picturePane, kControlNoPart, kControlUserPaneDrawProcTag,
		//							sizeof(ControlUserPaneDrawUPP), (Ptr)&upp);
		//}
		{
			PicHandle	picture = null;
			Rect		bounds;
			
			
			picture = GetPicture(inResourceID);
			if (picture != null)
			{
				bounds = (*picture)->picFrame;
				(OSErr)SetControlData(ptr->picturePane, kControlNoPart, kControlPictureHandleTag,
											sizeof(PicHandle), (Ptr)&picture);
				SizeControl(ptr->picturePane, bounds.right - bounds.left, bounds.bottom - bounds.top);
				SizeWindow(ptr->window, bounds.right - bounds.left, bounds.bottom - bounds.top, false/* update */);
			}
		}
		
		// set up the Window Info information
		ptr->windowInfo = WindowInfo_New();
		WindowInfo_SetWindowDescriptor(ptr->windowInfo, kInvalidWindowInfoDescriptor);
		WindowInfo_SetAuxiliaryDataPtr(ptr->windowInfo, (void*)result);
		WindowInfo_SetWindowFloating(ptr->windowInfo, true);
		WindowInfo_SetForWindow(ptr->window, ptr->windowInfo);
		
		// Create a fake MenuHandle so the Picture MDEF thinks its working with a real menu,
		// and then invoke the MDEF to determine the window size as if it were a menu.  This
		// windoid cannot be resized, so this is the only time "kMenuSizeMsg" is issued.
		ptr->fakeMenuHandle = NewMenu(inResourceID, EMPTY_PSTRING);
		
		UseResFile(oldResourceFile);
		refReleaseLock(result, &ptr);
	}
	
	return result;
}// New


/*!
This method cleans up a picture windoid window
by destroying data structures allocated by
PictureMDEFWindoid_New().

(3.0)
*/
void
PictureMDEFWindoid_Dispose	(PictureMDEFWindoidRef*		inoutRefPtr)
{
	if (inoutRefPtr != null)
	{
		PictureMDEFWindoidPtr		windoidPtr = refAcquireLock(*inoutRefPtr);
		
		
		if (windoidPtr != null)
		{
			DisposeWindow(windoidPtr->window); // disposes of the picture windoid window *and* all of its controls
			windoidPtr->window = null;
			WindowInfo_Dispose(windoidPtr->windowInfo);
			DisposeMenu(windoidPtr->fakeMenuHandle), windoidPtr->fakeMenuHandle = null;
			Memory_DisposeHandle((Handle*)inoutRefPtr);
		}
	}
}// Dispose


/*!
To obtain a reference to the Window Info
structure associated with a Picture MDEF
Windoid, use this method.  You may want
to set the window descriptor of the
windoid, which requires access to the
Window Info object.

The auxiliary data pointer is reserved,
and is set to be the corresponding
reference to a Picture MDEF Windoid
object.

(3.0)
*/
WindowInfoRef
PictureMDEFWindoid_GetWindowInfo	(PictureMDEFWindoidRef		inRef)
{
	PictureMDEFWindoidPtr		ptr = refAcquireLock(inRef);
	WindowInfoRef				result = null;
	
	
	if (ptr != null)
	{
		result = ptr->windowInfo;
		refReleaseLock(inRef, &ptr);
	}
	
	return result;
}// GetWindowInfo


/*!
To obtain the Mac OS window pointer for a
picture windoid, call this routine.

(3.0)
*/
WindowRef
PictureMDEFWindoid_GetWindow	(PictureMDEFWindoidRef		inRef)
{
	PictureMDEFWindoidPtr		ptr = refAcquireLock(inRef);
	WindowRef					result = null;
	
	
	if (ptr != null)
	{
		result = ptr->window;
		refReleaseLock(inRef, &ptr);
	}
	
	return result;
}// GetWindow


/*!
When a mouse-down event occurs in the content region
of a picture windoid, call this method.  The menu
item number of the click result is returned, as if
an item were chosen from the MDEF-based menu.  If
the menu is enabled, calling this routine has no
effect.

(3.0)
*/
SInt16
PictureMDEFWindoid_HandleContentClick		(PictureMDEFWindoidRef		inRef,
											 EventRecord*				inoutEventPtr)
{
	SInt16					result = 0;
	SInt16					windowPart = 0;
	WindowRef				window = PictureMDEFWindoid_GetWindow(inRef);
	PictureMDEFWindoidPtr	ptr = refAcquireLock(inRef);
	
	
	if ((window != null) && (ptr != null) && IsControlActive(ptr->picturePane))
	{
		windowPart = FindWindow(inoutEventPtr->where, &window);
		if (windowPart == inContent)
		{
			GrafPtr		oldPort = null;
			Rect		windowRect;
			Point		localMouse;
			OSErr		error = noErr;
			UInt16		serviceCounter = 0;
			SInt16		oldResult = 0;
			
			
			GetPortBounds(GetWindowPort(window), &windowRect);
			GetPort(&oldPort);
			SetPortWindowPort(window);
			
			// call MDEF continuously to track the mouse
			Cursors_UseHandPoint();
			if (FlagManager_Test(kFlagAppearance1_1API))
			{
				// play a button-press theme sound to start things off
				(OSStatus)PlayThemeSound(kThemeSoundButtonPress);
			}
			while (StillDown())
			{
				SetPortWindowPort(window); // in case another thread changed it
				GetMouse(&localMouse);
				mainMDEF(kMenuChooseMsg, ptr->fakeMenuHandle, &windowRect, localMouse, &result);
				if ((FlagManager_Test(kFlagAppearance1_1API)) && (result != oldResult))
				{
					// play a button-enter/button-exit theme sound if the item changes
					if (result == 0) // no item
					{
						(OSStatus)PlayThemeSound(kThemeSoundButtonExit);
					}
					else
					{
						(OSStatus)PlayThemeSound(kThemeSoundButtonEnter);
					}
				}
				oldResult = result;
				serviceCounter++;
				if (serviceCounter > 40) Threads_YieldToAnother(), serviceCounter = 0; // arbitrary maximum count
			}
			if ((FlagManager_Test(kFlagAppearance1_1API)) && (result != 0))
			{
				// play a button-release theme sound if a button was clicked
				(OSStatus)PlayThemeSound(kThemeSoundButtonRelease);
			}
			
			// unhighlight the selected item
			{
				SInt16		junk = result;
				
				
				SetPt(&localMouse, -1, -1);
				mainMDEF(kMenuChooseMsg, ptr->fakeMenuHandle, &windowRect, localMouse, &junk);
			}
			
			SetPort(oldPort);
		}
		refReleaseLock(inRef, &ptr);
	}
	
	return result;
}// HandleContentClick


/*!
To determine if a picture windoid is showing, call
this method.  This state is not exactly the window
state; specifically, if the windoid is suspended
but set to visible, the window will be invisible
(because the application is suspended) and yet
this method will return "true".

If you need to find the definite visible state of
the windoid, use PictureMDEFWindoid_GetWindow() to
obtain the Mac OS window pointer and then use the
IsWindowVisible() system call.

(3.0)
*/
Boolean
PictureMDEFWindoid_IsVisible	(PictureMDEFWindoidRef	inRef)
{
	PictureMDEFWindoidPtr		ptr = refAcquireLock(inRef);
	Boolean						result = false;
	
	
	if (ptr != null)
	{
		result = ptr->isShowing;
		refReleaseLock(inRef, &ptr);
	}
	
	return result;
}// IsVisible


/*!
To activate or deactivate the picture in a
Picture MDEF Windoid window, call this method.

(3.0)
*/
void
PictureMDEFWindoid_SetActive	(PictureMDEFWindoidRef	inRef,
								 Boolean const			inIsActive)
{
	PictureMDEFWindoidPtr		ptr = refAcquireLock(inRef);
	
	
	if (ptr != null)
	{
		HiliteWindow(ptr->window, inIsActive);
		if (inIsActive != IsControlActive(ptr->picturePane))
		{
			(OSErr)Embedding_OffscreenSetControlActive(ptr->picturePane, inIsActive);
		}
		refReleaseLock(inRef, &ptr);
	}
}// SetActive


/*!
To set the suspended state of a picture windoid,
call this method.  If the windoid is suspended, it
is invisible no matter what state it is in.  If
the windoid is not suspended, its previous state
is restored.  For example, if the window is
"visible" and suspended, it will be hidden.  If
the window is "invisible" and suspended, it will
be hidden as well.  But if the window is then set
to a non-suspended state, in the former case the
window will reappear, whereas in the latter case
the window remains invisible.

(3.0)
*/
void
PictureMDEFWindoid_SetSuspended		(PictureMDEFWindoidRef	inRef,
									 Boolean const			inIsSuspended)
{
	PictureMDEFWindoidPtr	ptr = refAcquireLock(inRef);
	WindowRef				window = PictureMDEFWindoid_GetWindow(inRef);
	
	
	if (ptr != null)
	{
		ptr->isSuspended = inIsSuspended;
		if (inIsSuspended) HideWindow(window);
		else
		{
			if (PictureMDEFWindoid_IsVisible(inRef)) ShowWindow(window);
			else HideWindow(window);
		}
		refReleaseLock(inRef, &ptr);
	}
}// SetSuspended


/*!
To show or hide a picture windoid, use this method.
You can also use PictureMDEFWindoid_ToggleVisible().

(3.0)
*/
void
PictureMDEFWindoid_SetVisible	(PictureMDEFWindoidRef	inRef,
								 Boolean const			inIsVisible)
{
	PictureMDEFWindoidPtr	ptr = refAcquireLock(inRef);
	
	
	if (ptr != null)
	{
		Boolean		oldVisibility = ptr->isShowing;
		
		
		ptr->isShowing = inIsVisible;
		if (oldVisibility != ptr->isShowing)
		{
			WindowRef		window = PictureMDEFWindoid_GetWindow(inRef);
			
			
			// show or hide the window
			if (inIsVisible) ShowWindow(window);
			else HideWindow(window);
		}
		refReleaseLock(inRef, &ptr);
	}
}// SetVisible


/*!
To show or hide a picture windoid based on whether
or not it is currently visible, call this method.

(3.0)
*/
void
PictureMDEFWindoid_ToggleVisible	(PictureMDEFWindoidRef	inRef)
{
	PictureMDEFWindoid_SetVisible(inRef, !PictureMDEFWindoid_IsVisible(inRef));
}// ToggleVisible


//
// internal methods
//

/*!
This method draws the user pane of a picture windoid
by assuming its control reference constant is a
reference to a Picture MDEF Windoid object.  The fake
menu handle of the object is used to trick the MDEF
into doing all of the drawing.

(3.0)
*/
static pascal void
pictureUserPaneDraw		(ControlRef		inControl,
						 SInt16			UNUSED_ARGUMENT(inPartCode))
{
	PictureMDEFWindoidRef		ref = REINTERPRET_CAST(GetControlReference(inControl), PictureMDEFWindoidRef);
	PictureMDEFWindoidPtr		ptr = null;
	
	
	ptr = refAcquireLock(ref);
	if (ptr != null)
	{
		Rect		bounds;
		Point		junk;
		short		unused = 0;
		
		
		GetControlBounds(inControl, &bounds);
		
		// use the fake menu handle to trick the MDEF into doing all the drawing
		if (IsControlActive(inControl))
		{
#if TARGET_API_MAC_OS8
			if (FlagManager_Test(kFlagCarbonMenuManager))
#endif
			{
				EnableMenuItem(ptr->fakeMenuHandle, 0);
			}
#if TARGET_API_MAC_OS8
			else
			{
				EnableItem(ptr->fakeMenuHandle, 0);
			}
#endif
		}
		else
		{
#if TARGET_API_MAC_OS8
			if (FlagManager_Test(kFlagCarbonMenuManager))
#endif
			{
				DisableMenuItem(ptr->fakeMenuHandle, 0);
			}
#if TARGET_API_MAC_OS8
			else
			{
				DisableItem(ptr->fakeMenuHandle, 0);
			}
#endif
		}
		SetPt(&junk, 0, 0);
		mainMDEF(kMenuDrawMsg, ptr->fakeMenuHandle, &bounds, junk, &unused);
		
		refReleaseLock(ref, &ptr);
	}
}// pictureUserPaneDraw


/*!
To obtain a pointer to the internal structure
given a reference to it, use this method.

(3.0)
*/
static PictureMDEFWindoidPtr
refAcquireLock	(PictureMDEFWindoidRef	inRef)
{
	PictureMDEFWindoidPtr		result = *((PictureMDEFWindoidHandle)inRef);
	
	
	HLock((Handle)inRef);
	result->lockCount++;
	return result;
}// refAcquireLock


/*!
To release a pointer to the internal structure
given a reference to it, use this method.

(3.0)
*/
static void
refReleaseLock	(PictureMDEFWindoidRef	inRef,
				 PictureMDEFWindoidPtr*	inoutPtr)
{
	PictureMDEFWindoidPtr		ptr = *((PictureMDEFWindoidHandle)inRef);
	
	
	if (--ptr->lockCount <= 0) HUnlock((Handle)inRef);
	if (inoutPtr != null) *inoutPtr = null;
}// refReleaseLock


//
// BEGIN PICTURE MDEF
//

/*
	Picture MDEF - by Carl W. Bell
	Copyright  1994-1997 Baylor University.

	If you have any questions, comments, suggestions, (constructive) criticism,
	or bug reports send e-mail to Carl_Bell@baylor.edu.

	This code is not public domain, but may be freely used for any purpose.
	You don't even have to give me (or Baylor University) credit but I won't
	complain if you do so anyway... :-)

	This MDEF allows one to include "arbitrary picture" menus in an app.  The
	picture that is displayed is stored as resources within the application.
	The orginal intent was to add function key and keypad menus to NCSA Telnet
	for those users with old keyboards w/out the function keys or who remap
	the function keys to some other purpose but still need that "Insert Here"
	key to work.

	The MDEF uses a 'PRCT' resource that sets up rectangular areas in the
	picture that correspond to menu items.  The picture and "picture rects"
	resources should have resource IDs the same as the menu's ID.  (That's the
	menu ID and NOT the menu's resource ID...)
	
	The MDEF does not support disabling of individual menu items.  If any of
	the items are disabled, then the entire menu is disabled.  This might be
	"fixed" in a future version.

	In case the ResEdit TMPL for 'PRCT' is missing, here is what it looks like:


			1) *****
			Label			*****
			Type			LSTB

			2) *****
			Label			A Rect
			Type			RECT

			3) *****
			Label			*****
			Type			LSTE

			4) *****


	There should be an entry for each rectangular area in the picture that has
	a corresponding menu item.  The rectangles are offset from the top-left
	corner of the picture's picFrame when drawn.  For example, the rect
	{ 0, 0, 10, 10 } will be drawn flush up against the top-left corner of
	the picture when the menu is drawn.  The MDEF doesn't frame anything - it
	just inverts.  So if you want frames around items in the menu, they
	will need to be included in the picture itself.

	Version 1.0
	
		Initial release with NCSA Telnet

	Version 1.1

		Fixed bug where sometimes arbitrary rectangles were inverted when no
			rectangle was current, i.e., cursor was between rectangles.
		No longer uses a separate "disabled" picture, but instead "grays out"
			the regular (enabled) image.

	To do:

		Individual enabling/disabling.
		More testing on older and B&W Macs.
		PPC/Fat version? Probably not.
*/

void DoSizeMessage(MenuHandle theMenu, Rect *theMenuRect);
void DoDrawMessage(MenuHandle theMenu, Rect *theMenuRect);
void DoChooseMessage(MenuHandle theMenu, Rect *theMenuRect, Point hitPt, short *whichItem);
pascal void DrawTheMenuProc(short depth, short deviceFlags, GDHandle targetDevice, long userData);

//	Entry point to the MDEF.  All we do is figure out what the calling program wants
//	and call the appropriate function.

static pascal void mainMDEF(short message, MenuHandle theMenu, Rect *theMenuRect, Point hitPt, short *whichItem)
{
	switch (message) {
	case kMenuDrawMsg:
		DoDrawMessage(theMenu, theMenuRect);
		break;
	case kMenuChooseMsg:
		DoChooseMessage(theMenu, theMenuRect, hitPt, whichItem);
		break;
	case kMenuSizeMsg:
		DoSizeMessage(theMenu, theMenuRect);
		break;
	default:
		// don't do anything
		break;
	}
}

//	This is where we calculate the height and width of the menu "box".  The height
//	is set to the height of the picture's picFrame.  The width is set to the
//	width of the picture's picFrame.  [Should we increase the height and width here
//	by some amount so that the picture is not flush up against the edge of the menu?
//	For now, we won't but instead let the picture include "white space" around the
//	edges if it needs a margin.]

void DoSizeMessage(MenuHandle theMenu, Rect* UNUSED_ARGUMENT(theMenuRect))
{
	short		theID;
	PicHandle	thePicture = null;
	Rect		theRect;
	
	// Get the resource ID of the picture (should be the same as the menuID)
	
	theID = GetMenuID(theMenu);
	thePicture = GetPicture(theID);

	// If there isn't a pict resource, then return.  

	if (thePicture == null) return;

	// Now calculate the height and width.

	theRect = (*thePicture)->picFrame;
	SetMenuWidth(theMenu, theRect.right - theRect.left);
	SetMenuHeight(theMenu, theRect.bottom - theRect.top);
}

//	Draw the picture in the menu's rect.  We assume that the menu manager sets
//	the current GrafPort to a valid port for us.

void DoDrawMessage(MenuHandle theMenu, Rect *theMenuRect)
{
	DeviceLoopDrawingUPP deviceLoopUPP = null;

	struct blah {
		PicHandle	thePicture;
		Rect		*theMenuRect;
		Boolean		disabled;
	} theData;

	// Get the resource ID of the picture (should be the same as the menuID)
	
	theData.thePicture = GetPicture(GetMenuID(theMenu));
	deviceLoopUPP = NewDeviceLoopDrawingUPP(DrawTheMenuProc);
	
	if ((theData.thePicture != null) && (deviceLoopUPP != null))
	{
		// Now draw it.  We use a device loop in case the menu crosses monitors.
		// Should this even happen?  Maybe if the menu is a popup menu or is way to
		// the side of the main monitor.  In any case, it doesn't hurt, and even
		// helps since we don't have to figure out what the screen depth is or
		// whether or not color qd is available.
		theData.theMenuRect = theMenuRect;
	#if TARGET_API_MAC_OS8
		theData.disabled = ((*theMenu)->enableFlags != 0xFFFFFFFF);
	#else
		theData.disabled = !MenuHasEnabledItems(theMenu);
	#endif
		
		// The original device loop code assumed qd->thePort was available which
		// isn't the case in a code resource.  Instead, we get thePort with
		// GetPort() and use it.  It seems to work.
		
		{
			GrafPtr			curPort = null;
			RgnHandle		visibleRegion = Memory_NewRegion();
			
			
			GetPort(&curPort);
			if (visibleRegion != null)
			{
				GetPortVisibleRegion(curPort, visibleRegion);
				DeviceLoop(visibleRegion, deviceLoopUPP, (long)&theData, 0);
				Memory_DisposeRegion(&visibleRegion);
			}
		}
	}
	DisposeDeviceLoopDrawingUPP(deviceLoopUPP), deviceLoopUPP = null;
}

// Our device loop procedure.

pascal void DrawTheMenuProc(short UNUSED_ARGUMENT(depth), short UNUSED_ARGUMENT(deviceFlags), GDHandle targetDevice, long userData)
{
	RGBColor		foreColor, backColor, grayColor;
	Pattern 		myGray;
	ColorPenState	state;
	struct blah {
		PicHandle	thePicture;
		Rect		*theMenuRect;
		Boolean		disabled;
	} *theData;
	
	
	ColorUtilities_GetColorAndPenState(&state);
	ColorUtilities_NormalizeColorAndPen();
	
	theData = (struct blah *)userData;

	// Just to be sure...

	if ((theData != null) && (theData->thePicture != null) && (theData->theMenuRect != null))
	{
		// temp - what the hell is going on here?
		{
			Rect	bounds = *(theData->theMenuRect);
			
			
			InsetRect(&bounds, 6, 6);
			FrameRect(&bounds);
		}
		
		// First draw the picture
	
		DrawPicture(theData->thePicture, theData->theMenuRect);	
	
		// Are we (completely) enabled?  Is so, we're done.
		
		if (theData->disabled)
		{
		
			// No, so disable everything.
		
			// targetDevice will be set to null if colorQD is not available, i.e., we are on an 68000
			// based Mac.  In that case we don't want to call colorQD.  Also, if we are B&W but have
			// colorQD, GetGray will return false and we will drop through to the "pattern" code.  
			
			if (targetDevice != null)
			{
				GetForeColor(&foreColor);
				GetBackColor(&backColor);
				grayColor = foreColor;
				if (GetGray(targetDevice, &backColor, &grayColor))
				{
					GrafPtr			curPort = null;
					
					
					GetPort(&curPort);
					
					// Just CopyBits our rect on top of itself to turn everything gray.
					RGBForeColor(&grayColor);
					CopyBits(GetPortBitMapForCopyBits(curPort), GetPortBitMapForCopyBits(curPort), theData->theMenuRect,
								theData->theMenuRect, patOr, null);
					RGBForeColor(&foreColor); 
					ColorUtilities_SetColorAndPenState(&state);
					return;
				}
			}
		
			// Either colorQD is not available, or we couldn't GetGray, e.g., we're B&W.  We just
			// paint over the rect with a gray pattern with a pen mode of notPatBic and everything
			// is grayed out.
		
			PenMode(notPatBic);
			GetIndPattern(&myGray, sysPatListID, 4); // We don't have qd globals so get gray this way
			PenPat(&myGray);
			PaintRect(theData->theMenuRect);
		}
	}
	ColorUtilities_SetColorAndPenState(&state);
}

//	This is the routine figures out which item is selected and (un)hilites
//	it as needed.  This routine is based on an example in Inside Mac.

void DoChooseMessage(MenuHandle theMenu, Rect *theMenuRect, Point hitPt, short *whichItem)
{
	short	theID;

	Handle	theRectsHandle = null;
	Rect	*theRects = null;
	Rect	theRectToInvert;
	short	numRects = 0;

	Point	thePoint = hitPt;
	short	selectedItem = 0;
	short	i;
	Boolean	menuIsEnabled = false;
	
	
#if TARGET_API_MAC_OS8
	menuIsEnabled = ((*theMenu)->enableFlags == 0xFFFFFFFF);
#else
	menuIsEnabled = MenuHasEnabledItems(theMenu);
#endif
	
	// If the menu is currently disabled, then return.
	if (menuIsEnabled)
	{
		// Get the resource ID of the PRCT "picture rects" resource.

		theID = GetMenuID(theMenu);

		// Get the "picture rects" resource.  If it doesn't exists, then return 0.

		theRectsHandle = GetResource('PRCT', theID);
		if (theRectsHandle == null) {
			*whichItem = 0;
			return;
		}

		// Lock the rectsHandle and 
		
		HLock(theRectsHandle);
		numRects = GetHandleSize(theRectsHandle) / sizeof(Rect);
		theRects = (Rect *)*theRectsHandle;

		if (PtInRect(thePoint, theMenuRect)) {

			// The cursor inside the menu's rect.

			// Offset the point.

			thePoint.v -= theMenuRect->top;
			thePoint.h -= theMenuRect->left;

			// Is the cursor currently in one of our rects?  If so, which one.

			for (i = 0; ((i < numRects) && (selectedItem == 0)); i++) {
				if (PtInRect(thePoint, &theRects[i])) {
					selectedItem = i + 1;
				}
			}

			// A previous item was hilighted, so unhilight it.
			
			if ((*whichItem > 0) && (*whichItem != selectedItem)) {
				theRectToInvert = theRects[*whichItem - 1];
				OffsetRect(&theRectToInvert, theMenuRect->left, theMenuRect->top);
				InvertRect(&theRectToInvert);
			}

			// Hilight the item.

			if (*whichItem != selectedItem) {
				*whichItem = selectedItem;

				// We need to check for 0 here because although regular menus don't
				// have gaps between menu items, ours may and if we invert our rect
				// at theRects[-1], then odd things might happen...

				if (*whichItem != 0) {
					theRectToInvert = theRects[*whichItem - 1];
					OffsetRect(&theRectToInvert, theMenuRect->left, theMenuRect->top);
					InvertRect(&theRectToInvert);
				}
			}
		}
		else if (*whichItem > 0) {

			// The cursor is outside the menu's rect.  Unhilight the current item.

			theRectToInvert = theRects[*whichItem - 1];
			OffsetRect(&theRectToInvert, theMenuRect->left, theMenuRect->top);
			InvertRect(&theRectToInvert);
			*whichItem = 0;
		}

		// Unlock the handle and return

		HUnlock(theRectsHandle);
	}
}
