/*###############################################################

	PictureMDEFWindoid.h
	
	This module allows windoids to be constructed whose content
	regions consist of pictures with hot rectangles controlled
	by Carl Bells Picture MDEF.  The code fools the MDEF into
	thinking it is operating on a real menu.
	
	NOTE:	This module is NOT the interface to handle keypads;
			the module Keypads.c should be used to manage them.
			Although the Keypads module internally uses this
			module for its implementation, that may change in
			the future.  So, you should not make direct calls to
			the PictureMDEFWindoid module except from within the
			Keypads module.
	
	When you create a windoid, you specify the resource file
	containing the 'PICT' and 'PRCT' resources required to make
	the windoid (both resources must be in the same file and
	both must have the same resource ID).  You can also give the
	windoid a title (specify a string containing a single space
	if you do not want the window to have a title).  The new
	windoid has Window Info associated with it, so you can
	obtain a reference to that object if needed (to set its
	window descriptor, for example).
	
	WARNING:	The fake menu created by this module will have
				the same ID as the 'PICT' and 'PRCT' resources.
				Ensure you do not have any conflicting menus.
	
	MacTelnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __PICTUREMDEFWINDOID__
#define __PICTUREMDEFWINDOID__

// Mac includes
#include <MacTypes.h>
#include <MacWindows.h>

// library includes
#include "InterfaceLib.h"



// types
typedef struct OpaquePictureMDEFWindoid { unsigned garbage; }*	PictureMDEFWindoidRef;



//
// public methods
//

PictureMDEFWindoidRef
	PictureMDEFWindoid_New					(ConstStr255Param		inWindowTitle,
											 UInt16					inResourceFile,
											 UInt16					inResourceID);

void
	PictureMDEFWindoid_Dispose				(PictureMDEFWindoidRef*	inoutRefPtr);

WindowInfoRef
	PictureMDEFWindoid_GetWindowInfo		(PictureMDEFWindoidRef	inRef);

WindowRef
	PictureMDEFWindoid_GetWindow			(PictureMDEFWindoidRef	inRef);

SInt16
	PictureMDEFWindoid_HandleContentClick	(PictureMDEFWindoidRef	inRef,
											 EventRecord*			inoutEventPtr);

Boolean
	PictureMDEFWindoid_IsVisible			(PictureMDEFWindoidRef	inRef);

void
	PictureMDEFWindoid_SetActive			(PictureMDEFWindoidRef	inRef,
											 Boolean const			inIsActive);

void
	PictureMDEFWindoid_SetSuspended			(PictureMDEFWindoidRef	inRef,
											 Boolean const			inIsSuspended);

void
	PictureMDEFWindoid_SetVisible			(PictureMDEFWindoidRef	inRef,
											 Boolean const			inIsVisible);

void
	PictureMDEFWindoid_ToggleVisible		(PictureMDEFWindoidRef	inRef);

#endif