/*###############################################################

	PrefPanelConfigurations.h
	
	Implements the Configurations panel of the new preferences
	dialog.  This sophisticated panel combines the traditional
	(NCSA Telnet 2.6) configuration list with tabs, so the user
	can display any set of configurations in the list.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

#ifndef __PREFPANELCONFIGURATIONS__
#define __PREFPANELCONFIGURATIONS__

// MacTelnet includes
#include "Panel.h"



// constants
typedef SInt16 TelnetConfigurationInstruction;
enum
{
	kTelnetConfigurationInstructionUserEdit = 0		// display a dialog to edit the configuration
};

// types
typedef struct { unsigned garbage; }*		TelnetConfigurationContextRef;

// universal procedure pointers
/*!
Configuration Editing Method

This routine is called when the user decides to
create or change a configuration.  Generally,
this routine would display a dialog box to allow
the user to edit the selected configuration.

You can use the configuration context to help
conduct your editing dialog box.  See the
"PrefPanelConfigurations_Context...()" methods,
below, to see how the context can be used.
*/
//typedef Boolean (*ConfigurationEditProcPtr)		(TelnetConfigurationContextRef		inContext,
//												 TelnetConfigurationInstruction		inDoWhat);
//#define InvokeConfigurationEditProc(userRoutine, inContext, inDoWhat) \
//			(*userRoutine)((inContext), (inDoWhat))
typedef Boolean (*ConfigurationEditProcPtr)		(StringPtr		inName);
#define InvokeConfigurationEditProc(userRoutine, inName) \
			(*userRoutine)((inName))



//
// public methods
//

PanelRef
	PrefPanelConfigurations_New					(void);

TelnetConfigurationContextRef
	PrefPanelConfigurations_NewContext			(PanelRef							inPanel);

void
	PrefPanelConfigurations_DisposeContext		(TelnetConfigurationContextRef*		inoutContextRefPtr);

void
	PrefPanelConfigurations_ContextGetName		(TelnetConfigurationContextRef		inContext,
												 Str255								outName);

Boolean
	PrefPanelConfigurations_ContextIsNew		(TelnetConfigurationContextRef		inContext);

#endif