/*###############################################################

	PreferencesData.h
	
	This file defines the layout of preferences structures, and
	is TEMPORARY.  MacTelnet 3.0 is migrating away from these
	structures, where all internal preferences information will
	be known only to "Preferences.c".  All setting and getting
	of preference information will be done through accessor
	functions that use class tags and preference tags.  No
	other part of the program should have access to the guts
	of the users preferences, because it is then very, very
	difficult to know (ahem) what the hell is going on, not to
	mention make any enhancements in the future.
	
	Be sure NOT TO DEFEAT THE PURPOSE OF THE NEW DESIGN.  The
	hope is that one day it will be very easy to migrate all of
	the preferences information to the Collection Manager, a new
	operating system service introduced with QuickDraw GX.  The
	Collection Manager makes it easy to store data on disk in
	terms of *what* it is, not *where* it is.  It also makes it
	easier to expand preferences in the future, or even redefine
	the on-disk representation for a preference.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __PREFERENCESDATA__
#define __PREFERENCESDATA__

// Mac includes
#include <QuickDraw.h>

// resource includes
#include "PreferenceResources.h"



// constants
enum
{
	kEMACSMetaKeyOff = 0,
	kEMACSMetaKeyControlCommand = 1,
	kEMACSMetaKeyOption = 2
};

enum
{
	kTerminalCursorTypeBlock = 0,
	kTerminalCursorTypeUnderscore = 1,
	kTerminalCursorTypeVerticalLine	= 2,
	kTerminalCursorTypeThickUnderscore = 3,
	kTerminalCursorTypeThickVerticalLine = 4
};

typedef SInt16 FTPServerMode;
enum
{
	kFTPServerModeOff = 0,
	kFTPServerModeOnNoProtection = 1,
	kFTPServerModeOnPasswordProtection = 2
};

// types
struct TerminalPrefs
{
	// This is the terminal preferences structure.  It specifies
	// what goes in a Terminal Configuration.  Note that, as a
	// preference structure that is dumped to disk as a whole,
	// you CANNOT MOVE and CANNOT CHANGE any parts of this
	// structure that are already defined without becoming
	// backwards-incompatible with older preferences files.  You
	// can only use the padding at the end of the structure (and
	// if you do, you must decrease the padding array to
	// accommodate your additions and keep the structure size
	// EXACTLY the same as it was before).
	SInt16			version;
	
	RGBColor		foregroundNormalColor,
					backgroundNormalColor,
					foregroundBlinkingColor,
					backgroundBlinkingColor,
					foregroundBoldColor,
					backgroundBoldColor;
	
	SInt16			emulation,				// A "kTerminalEmulationType..." constant
					columnCount,			// Width of the terminal screen in characters
					rowCount,				// Height of the terminal screen in characters
					fontSize,				// Size of DisplayFont to use to display text
					scrollbackBufferSize;	// Number of lines to save in scroll buffer
	
	UInt8			usesANSIColors,			// Recognize ANSI color sequences
					usesXTermSequences,		// Recognize Xterm sequences
					usesVTWrap,				// Use VT wrap mode
					metaKey,				// EMACS meta key - simulated with Macintosh key combinations
					usesEMACSArrows,		// Arrow keys and mouse position are EMACS flavor
					mapsPageJumpKeys,		// Map PageUp, PageDown, Home, End. (MAT == Mark Tamsky)
					usesEightBits,			// Dont strip the high bit
					savesOnClear;			// Save cleared lines
	
	Str63			normalFont;				// Font to use to display text
	
	Str32			answerBackMessage;		// Response to send when asked what terminal is being emulated
	UInt8	 		remapKeypad;			// remap keypad (2.7 CCP)
	
	SInt16			padding[98];
};
#ifndef __cplusplus
typedef struct TerminalPrefs	TerminalPrefs;
#endif
typedef TerminalPrefs*			TerminalPrefsPtr;
typedef TerminalPrefsPtr*		TerminalPrefsHandle;



#define OLD_CRAPPY_SESSIONPREFS_NAMES
struct SessionPrefs
{
	// This is the session preferences structure.  It specifies
	// what goes in a Session Favorite.  Note that, as a
	// preference structure that is dumped to disk as a whole,
	// you CANNOT MOVE and CANNOT CHANGE any parts of this
	// structure that are already defined without becoming
	// backwards-incompatible with older preferences files.  You
	// can only use the padding at the end of the structure (and
	// if you do, you must decrease the padding array to
	// accommodate your additions and keep the structure size
	// EXACTLY the same as it was before).
#ifdef OLD_CRAPPY_SESSIONPREFS_NAMES
	
	UInt16			version;				// Version of this structure
		
	UInt16			port;					// Port to connect to
	SInt16			tektype;				// A "kTektronixMode" constant (see tekdefs.h)
	SInt16			pastemethod;
	UInt16			pasteblocksize;			// Size of blocks to paste if using block mode
	
	UInt32			ip_address;				// IP address of the host

	UInt8			forcesave,				//
					crmap,					// Strange Berzerkley 4.3 CR mode needed
					linemode,				// Allow linemode to be used
					showlowlevelerrors,		// Not Used anymore
					tekclear,				// Clear TEK window vs. create new one
					halfdup;				// Half duplex required
		
	SInt8			bksp,					// 0 DELETE = BKSP, 1 DELETE = DELETE
					ckey,
					skey,
					qkey;

	Str32			TerminalEmulation,		// Name of terminal emulator to use
					TranslationTable;		// Name of translation table to use by default

	Str63			hostname;				// The DNS name of the host
		
	UInt8			authenticate,			// Kerberos authentication
					encrypt,				// Encrypted session
					localecho,				// Force local echo on?
					autoCaptureToFile;		// Automatically begin a file capture when this session opens?
	
	SInt16			NetBlockSize;			// Size of read buffer (2.7 CCP)
	
	SInt16			captureFileAliasID;		// ID of 'alis' resource identifying auto-capture destination file
	
	Str32			proxyServer;			// the name of the proxy server configuration to use
	char			pad;
	
	SInt16			padding[80];

#else

	UInt16			version;				// Version of this structure
		
	UInt16			port;					// Port to connect to
	SInt16			modeForTEK;				// A "kTektronixMode" constant (see tekdefs.h)
	SInt16			pasteMethod;			// A "kTelnetPasteMethod..." constant
	UInt16			pasteBlockSize;			// Size of blocks to paste if using block mode
	
	UInt32			ipAddress;				// IP address of the host
	
	UInt8			forceSave,				//
					mapCR,					// Strange Berzerkley 4.3 CR mode needed
					lineMode,				// Allow linemode to be used
					unused,
					tekPageClears,			// Clear TEK window vs. create new one
					halfDuplex;				// Half duplex required
	
	SInt8			deleteMapping,			// 0 DELETE = BKSP, 1 DELETE = DELETE
					cKey,
					sKey,
					qKey;
	
	Str32			terminalEmulationName,	// Name of terminal emulator to use
					translationTableName;	// Name of translation table to use by default

	Str63			hostName;				// The DNS name of the host
	
	UInt8			authenticate,			// Kerberos authentication
					encrypt,				// Encrypted session
					localEcho,				// Force local echo on?
					autoCaptureToFile;		// Automatically begin a file capture when this session opens?
	
	SInt16			netBlockSize;			// Size of read buffer (2.7 CCP)
	
	SInt16			captureFileAliasID;		// ID of 'alis' resource identifying auto-capture destination file
	
	Str32			proxyServer;			// the name of the proxy server configuration to use
	char			pad;
	
	SInt16			padding[80];

#endif
};
#ifndef __cplusplus
typedef struct SessionPrefs	SessionPrefs;
#endif
typedef SessionPrefs*		SessionPrefsPtr;
typedef SessionPrefsPtr*	SessionPrefsHandle;



struct FTPServerPrefs
{
	// This is the FTP preferences structure.  Note that, as a
	// preference structure that is dumped to disk as a whole,
	// you CANNOT MOVE and CANNOT CHANGE any parts of this
	// structure that are already defined without becoming
	// backwards-incompatible with older preferences files.  You
	// can only use the padding at the end of the structure (and
	// if you do, you must decrease the padding array to
	// accommodate your additions and keep the structure size
	// EXACTLY the same as it was before).
	SInt16			version;				// Version of this resource
	
	FTPServerMode	serverMode;				// A "kFTPServerMode..." constant
	
	OSType			binaryCreator,			// Default creator for files transferred in binary mode
					binaryType,				// Default type for files transferred in binary mode
					textCreator;			// Default creator for file transferred in ASCII mode
	
	UInt8			showLog,				// FTP log defaults to visible at startup time
					useDNSNamesForLog,		// Translate IP of connecting hosts to DNS names for log
					useMacBinaryII,			// Allow MacBinary II
					unused1,
					unused2,
					resetMacBinary,			// Reset MacBinary after each transfer
					useISOTranslation,		// Use ISO translation
					unused3;
	
	Str63			logFontName;			// The font to use for the FTP activity logs entries.
	SInt16			logFontSize;			// The font size to use for the FTP activity logs entries.
	
	SInt16			padding[67];
};
#ifndef __cplusplus
typedef struct FTPServerPrefs	FTPServerPrefs;
#endif
typedef FTPServerPrefs*			FTPServerPrefsPtr;
typedef FTPServerPrefsPtr*		FTPServerPrefsHandle;
// only one instance of the FTPServerPrefs structure (this extern is temporary)
extern	FTPServerPrefs*			gFTPServerPrefs;



struct FTPUser
{
	// This is the FTP user preferences structure.  It specifies
	// what goes in an FTP User Configuration.  Note that, as a
	// preference structure that is dumped to disk as a whole,
	// you CANNOT MOVE and CANNOT CHANGE any parts of this
	// structure that are already defined without becoming
	// backwards-incompatible with older preferences files.  You
	// can only use the padding at the end of the structure (and
	// if you do, you must decrease the padding array to
	// accommodate your additions and keep the structure size
	// EXACTLY the same as it was before).
	UInt16			version;				// Version of this resource
	
	Str32			password;				// Encrypted form of users password
	
	Str32			defaultDirVolName;		// obsolete - an alias resource is now used
	SInt32			defaultDirID;			// obsolete - an alias resource is now used
	
	UInt8			canChangeDirectories;	// User is allowed to change directories
	UInt8			unusedFlags[6];			// space for extra flags in the future
	UInt8			standardOptions;		// Are these preferences the same as the standard set?
	
	SInt16			defaultDirAliasID;		// ID of 'alis' resource identifying users default log-in directory
	
	SInt16			padding[19];
};
#ifndef __cplusplus
typedef struct FTPUser	FTPUser;
#endif
typedef FTPUser*		FTPUserPtr;
typedef FTPUserPtr*		FTPUserHandle;

#endif