/*###############################################################

	ProgressDialog.h
	
	This module handles all progress dialog boxes (modal or
	modeless) in MacTelnet.  Collecting all of these dialogs
	under one standard interface allows the user interface to
	work better, since each new dialog can be staggered over
	any existing one automatically (or for that matter, they
	could automatically be merged into a single dialog in the
	future, such as the progress window in Outlook Express 5).
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

#ifndef __PROGRESSDIALOG__
#define __PROGRESSDIALOG__

// Mac includes
#include <Appearance.h>
#include <Dialogs.h>
#include <MacTypes.h>



// types
typedef struct { unsigned garbage; }*	ProgressDialogRef;

typedef SInt16 TelnetProgressIndicator;
enum
{
	kTelnetProgressIndicatorThermometer = 0,	// a standard progress bar
	kTelnetProgressIndicatorIndeterminate = 1	// a barber-pole indeterminate progress bar
};



/*###############################################################
	CREATING AND DESTROYING PROGRESS DIALOGS
###############################################################*/

ProgressDialogRef
	ProgressDialog_New						(StringPtr					inPromptText,
											 const Boolean				inIsModal);

void
	ProgressDialog_Dispose					(ProgressDialogRef*			inoutRefPtr);

void
	ProgressDialog_Display					(ProgressDialogRef			inRef);

/*###############################################################
	MANIPULATING PROGRESS DIALOGS
###############################################################*/

WindowRef
	ProgressDialog_GetWindow				(ProgressDialogRef			inRef);

void
	ProgressDialog_SetProgressIndicator		(ProgressDialogRef			inRef,
											 TelnetProgressIndicator	inIndicatorType);

void
	ProgressDialog_SetProgressPercentFull	(ProgressDialogRef			inRef,
											 const SInt8				inPercentage);

void
	ProgressDialog_SetStatus				(ProgressDialogRef			inRef,
											 StringPtr					inStatusText);

void
	ProgressDialog_SetTitle					(ProgressDialogRef			inRef,
											 StringPtr					inTitleText);

#endif