/*###############################################################

	RealScreen.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/
/*
 *
 *      RSdraw(w,x,y,a,len,ptr)     - Draw @x,y in w string@ptr for length len
 *      RSdelchars(w,x,y,n)         - Delete n chars in w from x,y
 *      RSdellines(w,t,b,n)         - Delete n lines in region t->b in w
 *      RSinslines(w,t,b,n)         - Insert n lines in region t->b in w
 *      RSinsstring(w,x,y,a,len,ptr)- Insert len chars @x,y in w attrib a
 *
 *      Version Date    Notes
 *      ------- ------  ---------------------------------------------------
 *      0.01    861102  Initial coding -GBP
 *      0.25    861106  Added code from screen.c -GBP
 *      0.50    861113  First compiled edition -GBP
 *		2.1		871130	NCSA Telnet 2.1 -GBP
 *		2.2 	880715	NCSA Telnet 2.2 -GBP
 *		2.6		7/92	put globals into struct, cursors into array, and cleaned up 
 *						some of the font typedefs			Scott Bulmahn
 *		2.6b4	12/92	Cleaned up the code, and added double clicking -- Scott Bulmahn
 *
 */

#include "UniversalDefines.h"

#define __ALLNU__

// standard-C includes
#include <ctype.h>

// Mac includes
#include <Appearance.h>
#include <Controls.h>
#include <ControlDefinitions.h>
#include <Dialogs.h>
#include <Fonts.h>
#include <MacWindows.h>
#include <PLStringFuncs.h>
#include <Resources.h>
#include <TextServices.h>

// resource includes
#include "ControlResources.h"
#include "DialogResources.h"
#include "GeneralResources.h"
#include "MenuResources.h"
#include "StringResources.h"
#include "SpacingConstants.r"

// library includes
#include "DataAccessLib.h"
#include "InterfaceLib.h"

// MacTelnet includes
#include "Clipboard.h"
#include "Commands.h"
#include "Connections.h"
#include "Console.h"
#include "DialogItemIndices.h"
#include "DialogUtilities.h"
#include "EventLoop.h" //notify user proto
#include "MacLook.h"
#include "network.proto.h"
#include "Preferences.h"
#include "RealScreen.h"
#include "RealScreenData.h"
#include "ResourceFileManager.h"
#include "TelnetDescriptors.h"
#include "TelnetDrag.h"
#include "TelnetMenus.h"
#include "Terminal.h"
#include "URL.h"
#include "vskeys.h"
#include "vsdata.h"
#include "VirtualScreenInternal.h"



// constants
#define SHADOW_RGBCOLOR_INTENSITY_DIVISOR		4		/* number of pixels of shadow (and the gradient) */

// variables
static ControlUserPaneDrawUPP		gInvisibleScrollPaneDrawUPP = null,
									gRealScreenPaneDrawUPP = null,
									gRealScreenPanePictureDrawUPP = null;
static ControlUserPaneBackgroundUPP	gRealScreenPanePictureBackgroundUPP = null;
static ControlUserPaneIdleUPP		gRealScreenPaneIdleUPP = null;
static SInt16**						gTopLeftCorners = null;
static RgnHandle					gScrollingUpdateRegion = null;	// region to redraw; not persistent, but used often
static RGBColor						gShadowGrays[SHADOW_RGBCOLOR_INTENSITY_DIVISOR];

// internal methods
static pascal Boolean	adjustSetWindowTitleDialog		(WindowRef const				inWindow,
														 SInt32 const					inDeltaSizeX,
														 SInt32 const					inDeltaSizeY,
														 SInt32 const					inData);
static void				calculateWindowIndexPosition	(RealScreenDataPtr				inRealScreenPtr,
														 SInt16 const					inStaggerIndex,
														 Point*							outPositionPtr);
static void				calculateWindowPosition			(RealScreenDataPtr				inRealScreenPtr,
														 Rect*							outArrangement,
														 short							inRowsHigh,
														 short							inColumnsWide);
static void				createTextEncodingConverter		(RealScreenDataPtr				inRealScreenPtr);
static Boolean			cursorBlinks					(RealScreenDataPtr				inRealScreenPtr);
static void				drawShadow						(RealScreenDataPtr				inRealScreenPtr);
static void				drawText						(RealScreenDataPtr				inRealScreenPtr,
														 char const*					inTextBufferPtr,
														 SInt32 const					inCharacterCount);
static Point			findVirtualCellFromLocalPoint	(TelnetScreenID const			inScreenID,
														 Point							inLocalPoint,
														 Boolean						inAutomaticallyScrollToPoint);
static UInt16			getPlacardHeight				(RealScreenDataPtr				inRealScreenPtr);
static void				getRegionBounds					(RealScreenDataPtr				inRealScreenPtr,
														 RealScreenRegionCode const		inRegionCode,
														 Rect*							outBounds);
static void				getScreenColor					(RealScreenDataPtr				inRealScreenPtr,
														 RealScreenColorIndex const		inColorEntryNumber,
														 RGBColor*						outColorPtr);
static void				getScreenColorEntries			(RealScreenDataPtr				inRealScreenPtr,
														 SInt16 const					inAttributes,
														 SInt16*						outForeColorEntry,
														 SInt16*						outBackColorEntry);
static void				getScreenOrigin					(RealScreenDataPtr				inRealScreenPtr,
														 SInt16*						outScreenPositionX,
														 SInt16*						outScreenPositionY);
static void				getScreenSizeFromWindowSize		(RealScreenDataPtr				inRealScreenPtr,
														 SInt16 const					inWindowContentWidth,
														 SInt16 const					inWindowContentHeight,
														 SInt16*						outScreenInteriorWidth,
														 SInt16*						outScreenInteriorHeight);
static RgnHandle		getSelectedTextAsNewRegion		(RealScreenDataPtr				inRealScreenPtr);
static void				getVirtualVisibleRegion			(RealScreenDataPtr				inRealScreenPtr,
														 SInt16*						outLeftColumn,
														 SInt16*						outTopRow,
														 SInt16*						outRightColumn,
														 SInt16*						outBottomRow);
static void				getWindowSizeFromScreenSize		(RealScreenDataPtr				inRealScreenPtr,
														 SInt16 const					inScreenInteriorWidth,
														 SInt16 const					inScreenInteriorHeight,
														 SInt16*						outWindowContentWidth,
														 SInt16*						outWindowContentHeight);
static void				handleMultiClick				(TelnetScreenID const			inScreenID,
														 UInt16 const					inClickCount,
														 EventModifiers					modifiers);
static void				handlePlacardSizeChange			(TerminalWindowHeaderRef const	inHeader);
static void				handleScreenActivate			(RealScreenDataPtr				inRealScreenPtr);
static void				handleScreenDeactivate			(RealScreenDataPtr				inRealScreenPtr);
static void				highlightVirtualRange			(RealScreenDataPtr				inRealScreenPtr,
														 Point const					inZeroBasedAnchorRowAndColumnBeginning,
														 Point const					inZeroBasedAnchorRowAndColumnEnd,
														 Rect const*					inLocalConstraintRectPtr);
static void				invalidateScreenRectangle		(RealScreenDataPtr				inRealScreenPtr);
static pascal void		invisibleScrollBarUserPaneDraw	(ControlRef						inScrollBar,
														 SInt16							inPartCode);
static void				localToScreen					(RealScreenDataPtr				inRealScreenPtr,
														 SInt16*						inoutHorizontalPixelOffsetFromPortOrigin,
														 SInt16*						inoutVerticalPixelOffsetFromPortOrigin);
static void				localToScreenRect				(RealScreenDataPtr				inRealScreenPtr,
														 Rect*							inoutPortOriginBounds);
static pascal void		realScreenPictureDeviceLoop		(short							inColorDepth,
														 short							inDeviceFlags,
														 GDHandle						inTargetDevice,
														 long							inControl);
static pascal void		realScreenPictureDraw			(ControlRef						inControl,
														 SInt16							inPartCode);
static pascal void		realScreenPictureSetBackground	(ControlRef						inControl,
														 ControlBackgroundPtr			inBackgroundInfoPtr);
static pascal void		realScreenUserPaneDeviceLoop	(short							inColorDepth,
														 short							inDeviceFlags,
														 GDHandle						inTargetDevice,
														 long							inControl);
static pascal void		realScreenUserPaneDraw			(ControlRef						inControl,
														 SInt16							inPartCode);
static pascal void		realScreenUserPaneIdle			(ControlRef						inControl);
static void				screenToLocal					(RealScreenDataPtr				inRealScreenPtr,
														 SInt16*						inoutHorizontalPixelOffsetFromScreenOrigin,
														 SInt16*						inoutVerticalPixelOffsetFromScreenOrigin);
static void				screenToLocalRect				(RealScreenDataPtr				inRealScreenPtr,
														 Rect*							inoutScreenOriginBounds);
static pascal void		scrollProc						(ControlRef						inScrollBarClicked,
														 ControlPartCode				inPartCode);
static void				setCursorVisibility				(RealScreenDataPtr				inRealScreenPtr,
														 Boolean const					inIsVisible);
static void				setRealScreenAttr				(RealScreenDataPtr				inRealScreenPtr,
														 SInt16 const					inAttributes);
static void				setRealScreenColorAttr			(RealScreenDataPtr				inRealScreenPtr,
														 SInt16 const					inAttributes);
static void				setScreenColor					(RealScreenDataPtr				inRealScreenPtr,
														 RealScreenColorIndex const		inColorEntryNumber,
														 RGBColor const*				inColorPtr);
static void				setUpScreenFontMetrics			(RealScreenDataPtr				inRealScreenPtr);
static pascal Boolean	setWindowTitleDialogEventFilter	(DialogRef						inDialog,
														 EventRecord*					inoutEventPtr,
														 DialogItemIndex*				outItemIndex);
static void				updateTextSelection				(TelnetScreenID const			inScreenID,
														 Rect const*					inClipRectPtr);


/*!
Real Screen Attribute Bits

NOTE:	The first few bits must remain identical to the Mac OS
		"StyleParameter" values of "bold", "underline" and
		"outline".

ANSI bkgnd.   foreground   
15 14 13 12  11 10  9  8   7  6  5  4   3  2  1  0
-|--|--|--|---|--|--|--|---|--|--|--|---|--|--|--|-
 |  |  |  |   |  |  |  |   |  |  |  |   |  |  |  |
 |  |  |  |   |  |  |  |   |  |  |  |   |  |  |  +--- 0: bold?
 |  |  |  |   |  |  |  |   |  |  |  |   |  |  |
 |  |  |  |   |  |  |  |   |  |  |  |   |  |  +------ 1: UNDEFINED - set to 0
 |  |  |  |   |  |  |  |   |  |  |  |   |  |
 |  |  |  |   |  |  |  |   |  |  |  |   |  +--------- 2: UNDEFINED - set to 0
 |  |  |  |   |  |  |  |   |  |  |  |   |
 |  |  |  |   |  |  |  |   |  |  |  |   +------------ 3: underlined?
 |  |  |  |   |  |  |  |   |  |  |  |
 |  |  |  |   |  |  |  |   |  |  |  |
 |  |  |  |   |  |  |  |   |  |  |  +--- 4: blinking?
 |  |  |  |   |  |  |  |   |  |  |
 |  |  |  |   |  |  |  |   |  |  +------ 5: UNDEFINED - set to 0
 |  |  |  |   |  |  |  |   |  |
 |  |  |  |   |  |  |  |   |  +--------- 6: inverse video?
 |  |  |  |   |  |  |  |   |
 |  |  |  |   |  |  |  |   +------------ 7: use graphics character set?
 |  |  |  |   |  |  |  |
 |  |  |  |   |  |  |  |
 |  |  |  |   |  +--+--+-- 10-8: ANSI foreground color index (3 bits)
 |  |  |  |   |
 |  |  |  |   +----------- 11:   use ANSI foreground color?
 |  |  |  |
 |  |  |  |
 |  +--+--+-- 14-12: ANSI background color index (3 bits)
 |
 +----------- 15:    use ANSI background color?

The 3-bit ANSI color values can be one of the following:
	000 (0)		black			100 (4)		blue
	001 (1)		red				101 (5)		magenta
	010 (2)		green			110 (6)		cyan
	011 (3)		yellow			111 (7)		white
*/

#define STYLE_BOLD(x) 					((x) & 0x01)
#define STYLE_UNDERLINE(x) 				((x) & 0x08)
#define STYLE_BLINKING(x) 				((x) & 0x10)
#define STYLE_INVERSE_VIDEO(x)  		((x) & 0x40)
#define STYLE_USE_VT_FONT(x)	 		((x) & 0x80)

#define STYLE_USE_ANSI_FOREGROUND(x)	((x) & 0x0800)
#define STYLE_USE_ANSI_BACKGROUND(x) 	((x) & 0x8000)

// returns whether or not the specified ASCII code corresponds to an ANSI graphics character
// (i.e. a character that wont look correct unless rendered with the ANSI graphics font);
// MacTelnet uses David P. Alversons ANSI graphics font, from "ZTerm".
#define ASCII_IS_ANSI_GRAPHIC(ascii)		(((ascii) > 168) || ((ascii) < 32))
#define ASCII_IS_NOT_ANSI_GRAPHIC(ascii)	(((ascii) < 169) || ((ascii) > 31))



//
// public methods
//

/*!
Call this routine once, before any other method
from this module.

(2.6)
*/
void
RealScreen_Init		(void)
{
	gScrollingUpdateRegion = Memory_NewRegion();
	gInvisibleScrollPaneDrawUPP = NewControlUserPaneDrawUPP(invisibleScrollBarUserPaneDraw);
	gRealScreenPaneIdleUPP = NewControlUserPaneIdleUPP(realScreenUserPaneIdle);
	gRealScreenPaneDrawUPP = NewControlUserPaneDrawUPP(realScreenUserPaneDraw);
	gRealScreenPanePictureDrawUPP = NewControlUserPaneDrawUPP(realScreenPictureDraw);
	gRealScreenPanePictureBackgroundUPP = NewControlUserPaneBackgroundUPP(realScreenPictureSetBackground);
	gTopLeftCorners = (SInt16**)Memory_NewHandleInProperZone(kMaximumAllowedTelnetConnections * sizeof(SInt16),
															false/* needed for a short period of time */);
	
	// initialize shadow grays
	{
		register SInt16		i = 0;
		
		
		for (i = 0; i < SHADOW_RGBCOLOR_INTENSITY_DIVISOR; i++)
		{
			gShadowGrays[i].red = gShadowGrays[i].green = gShadowGrays[i].blue =
				(RGBCOLOR_INTENSITY_MAX / SHADOW_RGBCOLOR_INTENSITY_DIVISOR) * i;
		}
	}
}// Init


/*!
Call this routine when you are finished using
the Real Screen module.

(2.6)
*/
void
RealScreen_Done		(void)
{
	Memory_DisposeRegion(&gScrollingUpdateRegion);
	DisposeControlUserPaneDrawUPP(gInvisibleScrollPaneDrawUPP), gInvisibleScrollPaneDrawUPP = null;
	DisposeControlUserPaneIdleUPP(gRealScreenPaneIdleUPP), gRealScreenPaneIdleUPP = null;
	DisposeControlUserPaneDrawUPP(gRealScreenPaneDrawUPP), gRealScreenPaneDrawUPP = null;
	DisposeControlUserPaneDrawUPP(gRealScreenPanePictureDrawUPP), gRealScreenPanePictureDrawUPP = null;
	DisposeControlUserPaneBackgroundUPP(gRealScreenPanePictureBackgroundUPP), gRealScreenPanePictureBackgroundUPP = null;
	if (gTopLeftCorners != null) Memory_DisposeHandle(&((Handle)gTopLeftCorners));
}// Done


/*!
To create a new real screen, complete with virtual
screen storage, use this method.  The screen ID of
the new screen is returned, or a negative value if
any problems occur.

(3.0)
*/
TelnetScreenID
RealScreen_New	(TelnetScreenID const	inIDOfNewScreen,
				 SInt16 const			inVisibleColumnCount,
				 SInt16 const			inVisibleRowCount,
				 StringPtr				inFontName,
				 SInt16 const			inFontSize,
				 RealScreenFlags const	inFlags)
{
	Rect				windowRect;
	TelnetScreenID		result = -1;
	
	
	SetRect(&windowRect, 0, 0, 0, 0);
	
	// 3.0 - the virtual screen must now be created prior to invoking this routine, not here
	if (inIDOfNewScreen < 0) result = -1;
	else
	{
		RealScreenDataPtr	realScreenPtr = null;
		Rect				windowRect;
		Rect				pRect;
		UInt16				wheight = 0,
							wwidth = 0;
		
		
		realScreenPtr = ScreenManager_GetRealScreenDataOf(inIDOfNewScreen);
		
		PLstrcpy(realScreenPtr->fontName, inFontName);
		realScreenPtr->fsiz = inFontSize;
		createTextEncodingConverter(realScreenPtr);
		
		// create a new-style window with a Color QuickDraw graphics port
		SetRect(&windowRect, 0, 0, 0, 0);
		if (FlagManager_Test(kFlagCarbonWindowManager))
		{
			OSStatus		error = noErr;
			
			
			error = CreateNewWindow(kDocumentWindowClass, kWindowStandardDocumentAttributes, &windowRect,
									&realScreenPtr->window);
			if (error != noErr)
			{
				realScreenPtr->window = null;
			}
		}
		else
		{
			realScreenPtr->window = NewCWindow(null,
												&windowRect,
												EMPTY_PSTRING,
												false/* visible */,
												kWindowFullZoomGrowDocumentProc,
												(GetRealFrontWindow() != null) ? GetRealFrontWindow() : kInFront,
												true/* close box */,
												0L/* MUST be zero for Window Info! */);
		}
		
		if (realScreenPtr->window == null)
		{
			VSdestroy(inIDOfNewScreen);
			return(-2);
		}
		else
		{
			// initialize the window
			WindowInfoRef					windowInfo = WindowInfo_New();
			WindowResizeResponderProcPtr	resizeProc = RealScreen_HandleNewSize;
			
			
			WindowInfo_SetWindowDescriptor(windowInfo, kTelnetWindowDescriptorAnyTerminal);
			WindowInfo_SetWindowResizeLimits(windowInfo,
													200/* arbitrary minimum height */,
													250/* arbitrary minimum width */,
													SHRT_MAX/* maximum height */,
													SHRT_MAX/* maximum width */);
			WindowInfo_SetWindowResizeResponderUPP(windowInfo, resizeProc, 0L);
			WindowInfo_SetForWindow(realScreenPtr->window, windowInfo);
		}
		
		// 3.0 - using the *window* port, set up the font metrics
		{
			GrafPtr		oldPort = null,
						currentPort = null;
			
			
			GetPort(&oldPort);
			SetPortWindowPort(realScreenPtr->window);
			GetPort(&currentPort);
			RealScreen_TextFont(inFontName, inFontSize, normal);
			setUpScreenFontMetrics(realScreenPtr);
			SetPort(oldPort);
		}
		
		// 3.0 - MacTelnet will soon support the Text Services Manager!
		if (FlagManager_Test(kFlagTextServicesManager))
		{
			InterfaceTypeList		supportedTSMTypes;
			
			
			supportedTSMTypes[0] = kTextService;
			(OSErr)NewTSMDocument(1/* size of array */, supportedTSMTypes, &realScreenPtr->textServicesDocumentID,
									(long)realScreenPtr->window/* reference constant for routines */);
		}
		
		// Set up a control embedding hierarchy.  The root is a user pane,
		// so callbacks can be attached to it.  Any such callbacks take
		// precedence over callbacks installed in other user panes in the
		// control embedding hierarchy.  This means that you could create
		// a single content-click handler that automatically catches
		// clicks anywhere in a terminal window, an activate handler that
		// responds to activate events, etc.
		(OSErr)CreateRootControl(realScreenPtr->window, &realScreenPtr->rootControl);
		
		// create the color palette - 14 screen colors (8 ANSI, 6 others);
		// the Mac OS needs to know which colors are required by which
		// windows, so a palette must be created and attached to each
		// terminal window in order for it to display the correct colors
		{
			enum
			{
				kPaletteSize = (INTEGER_DOUBLED(kRealScreenColorCountNonANSIColors) +
								INTEGER_DOUBLED(kRealScreenColorCountNormalANSIColors))	// total of required screen colors
			};
			register SInt16		i = 0;
			CTabHandle			colorTableHandle = null;
			
			
			colorTableHandle = (CTabHandle)Memory_NewHandleInProperZone(sizeof(ColorTable) + kPaletteSize * sizeof(CSpecArray),
																		true/* needed only for short period of time */);
			if (colorTableHandle == null) 
			{
				DisposeWindow(realScreenPtr->window);
				VSdestroy(inIDOfNewScreen);
				return(-2);
			}
			
			(*colorTableHandle)->ctSize = kPaletteSize - 1;
			(*colorTableHandle)->ctFlags = 0;
			
			for (i = 0; i < INTEGER_DOUBLED(kRealScreenColorCountNonANSIColors); i++)
			{
				(*colorTableHandle)->ctTable[i].value = 0;
			}
			
			for (i = 0; i < INTEGER_DOUBLED(kRealScreenColorCountNormalANSIColors); i++) // get the ANSI colors from the palette
			{
				Preferences_GetANSIColor(i, &((*colorTableHandle)->ctTable[i + kRealScreenColorCountNonANSIColors].rgb));
				(*colorTableHandle)->ctTable[i + kRealScreenColorCountNonANSIColors].value = 0;
			}
			
			realScreenPtr->paletteTerminal = NewPalette(kPaletteSize, colorTableHandle, pmCourteous, 0);
			Memory_DisposeHandle(&((Handle)colorTableHandle));
			if (realScreenPtr->paletteTerminal == null)
			{
				DisposeWindow(realScreenPtr->window);
				VSdestroy(inIDOfNewScreen);
				return(-2);
			}
			NSetPalette(realScreenPtr->window, realScreenPtr->paletteTerminal, pmAllUpdates); // copy the palette to the window
		}
		
		// use terminal default preferences (stored in preferences file) to set up windows colors
		{
			TerminalPrefs**		terminalPreferencesHandle = null;
			SInt16				oldResourceFile = CurResFile(); // remember resource file currently in use by getting its ID
			
			
			ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
			terminalPreferencesHandle = (TerminalPrefs**)Get1Resource(TERMINALPREFS_RESTYPE, TERMINALPREFS_DEFAULTID);
			DetachResource((Handle)terminalPreferencesHandle);
			HLock((Handle)terminalPreferencesHandle);
			setScreenColor(realScreenPtr, kRealScreenColorIndexNormalText, &(*terminalPreferencesHandle)->foregroundNormalColor);
			setScreenColor(realScreenPtr, kRealScreenColorIndexNormalBackground, &(*terminalPreferencesHandle)->backgroundNormalColor);
			setScreenColor(realScreenPtr, kRealScreenColorIndexBlinkingText, &(*terminalPreferencesHandle)->foregroundBlinkingColor);
			setScreenColor(realScreenPtr, kRealScreenColorIndexBlinkingBackground, &(*terminalPreferencesHandle)->backgroundBlinkingColor);
			setScreenColor(realScreenPtr, kRealScreenColorIndexBoldText, &(*terminalPreferencesHandle)->foregroundBoldColor);
			setScreenColor(realScreenPtr, kRealScreenColorIndexBoldBackground, &(*terminalPreferencesHandle)->backgroundBoldColor);
			Memory_DisposeHandle(&((Handle)terminalPreferencesHandle));
			
			// restore to previous resource file using preserved ID
			UseResFile(oldResourceFile);
		}
		
		SetPortWindowPort(realScreenPtr->window);
		
		// size the window and stagger it
		{
			SInt16		newWidth = 0,
						newHeight = 0;
			
			
			getWindowSizeFromScreenSize(realScreenPtr, inVisibleColumnCount * realScreenPtr->fwidth,
										inVisibleRowCount * realScreenPtr->fheight,
										&newWidth, &newHeight);
			SizeWindow(realScreenPtr->window, newWidth, newHeight, false/* update */);
		}
		calculateWindowPosition(realScreenPtr, &windowRect, inVisibleRowCount, inVisibleColumnCount);
		wwidth = windowRect.right - windowRect.left;
		wheight = windowRect.bottom - windowRect.top;
		MoveWindow(realScreenPtr->window, windowRect.left, windowRect.top, false/* activate */);
		
		// initialize window states for zooming
		GetWindowUserState(realScreenPtr->window, &windowRect);
		pRect.top = windowRect.top;
		pRect.left = windowRect.left;
		pRect.right = pRect.left + (inVisibleColumnCount) * realScreenPtr->fwidth + RealScreen_GetScrollBarWidth();
		{
			Rect		screenRect;
			
			
			MacLook_GetWindowDeviceGrayRect(realScreenPtr->window, &screenRect);
			if (pRect.right > screenRect.right) pRect.right = screenRect.right;
		}
		{
			SInt16		height = 0;
			
			
			getWindowSizeFromScreenSize(realScreenPtr, 0/* ignore width */, (inVisibleRowCount) * realScreenPtr->fheight,
										null/* ignore width */, &height);
			pRect.bottom = pRect.top + height;
		}
		SetWindowStandardState(realScreenPtr->window, &pRect);
		
		// create a vertical scroll bar, and an invisible state user pane that draws its inactive scroll bar region
		SetRect(&pRect, 0, 0, 0, 0);
		realScreenPtr->scroll = NewControl(realScreenPtr->window, &pRect, EMPTY_PSTRING, false/* visible */,
											0/* value */,
											0/* minimum */,
											0/* maximum */,
											kControlScrollBarLiveProc,
											1L/* ref. con. */);
		if (realScreenPtr->scroll == null)
		{
			return(-3);
		}
		realScreenPtr->invisibleScrollBarV = NewControl(realScreenPtr->window, &pRect, EMPTY_PSTRING, false/* visible */,
														0/* value */,
														0/* minimum */,
														0/* maximum */,
														kControlUserPaneProc,
														0L/* ref. con. */);
		(OSErr)SetControlData(realScreenPtr->invisibleScrollBarV, kControlNoPart, kControlUserPaneDrawProcTag,
								sizeof(ControlUserPaneDrawUPP), (Ptr)&gInvisibleScrollPaneDrawUPP);
		
		// create a horizontal scroll bar, and an invisible state user pane that draws its inactive scroll bar region
		SetRect(&pRect, 0, 0, 0, 0);
		realScreenPtr->left = NewControl(realScreenPtr->window, &pRect, EMPTY_PSTRING, false/* visible */,
										0/* value */,
										0/* minimum */,
										0/* maximum */,
										kControlScrollBarLiveProc,
										1L/* ref. con. */);
		if (realScreenPtr->left == null)
		{
			return(-3);
		}
		realScreenPtr->invisibleScrollBarH = NewControl(realScreenPtr->window, &pRect, EMPTY_PSTRING, false/* visible */,
										0/* value */,
										0/* minimum */,
										0/* maximum */,
										kControlUserPaneProc,
										0L/* ref. con. */);
		(OSErr)SetControlData(realScreenPtr->invisibleScrollBarH, kControlNoPart, kControlUserPaneDrawProcTag,
								sizeof(ControlUserPaneDrawUPP), (Ptr)&gInvisibleScrollPaneDrawUPP);
		
		// create a user pane representing the terminal screen foreground
		SetRect(&pRect, 0, 0, 0, 0);
		realScreenPtr->contentUserPane = GetNewControl(kIDTerminalWindowScreenPane, realScreenPtr->window);
		{
			OSErr		error = noErr;
			
			
			// all callbacks require this data; for efficiency, attach it to the control directly
			SetControlReference(realScreenPtr->contentUserPane, REINTERPRET_CAST(realScreenPtr, SInt32));
			
			// 3.0 - the screen foreground is now rendered by a user pane, with an idle procedure
			// that does ALL periodic tasks such as cursor flashing, etc.
			error = SetControlData(realScreenPtr->contentUserPane, kControlNoPart, kControlUserPaneIdleProcTag,
									sizeof(ControlUserPaneIdleUPP), (Ptr)&gRealScreenPaneIdleUPP);
			
			// 3.0 - the screen foreground is now rendered by a user pane, with a drawing procedure
			// that renders only the foreground (text)
			error = SetControlData(realScreenPtr->contentUserPane, kControlNoPart, kControlUserPaneDrawProcTag,
									sizeof(ControlUserPaneDrawUPP), (Ptr)&gRealScreenPaneDrawUPP);
		}
		
		// create a user pane representing the terminal screen background
		SetRect(&pRect, 0, 0, 0, 0);
		realScreenPtr->backgroundUserPane = GetNewControl(kIDTerminalWindowBackgroundPicture, realScreenPtr->window);
		{
			OSErr		error = noErr;
			
			
			// all callbacks require this data; for efficiency, attach it to the control directly
			SetControlReference(realScreenPtr->backgroundUserPane, REINTERPRET_CAST(realScreenPtr, SInt32));
			
			// 3.0 - the screen background is now rendered by a user pane, with a background procedure
			// that renders only the backdrop (a solid color)
			error = SetControlData(realScreenPtr->backgroundUserPane, kControlNoPart,
									kControlUserPaneBackgroundProcTag, sizeof(ControlUserPaneBackgroundUPP),
									(Ptr)&gRealScreenPanePictureBackgroundUPP);
			
			// 3.0 - the screen background is now rendered by a user pane, with a drawing procedure
			// that renders only the backdrop (possibly a picture)
			error = SetControlData(realScreenPtr->backgroundUserPane, kControlNoPart,
									kControlUserPaneDrawProcTag, sizeof(ControlUserPaneDrawUPP),
									(Ptr)&gRealScreenPanePictureDrawUPP);
		}
		
		// the picture control embeds the content user pane; deactivating or activating the
		// picture control will therefore automatically change the state of the content pane
		(OSErr)EmbedControl(realScreenPtr->contentUserPane, realScreenPtr->backgroundUserPane);
		
		// once embedded, offset the content pane; this initializes the top-left inset, and
		// ensures that the offset will remain forever (if the parent control is ever moved,
		// the child is also offset by the same amount, so the relative position is the same)
		{
			Point		insetsTopLeft,
						insetsBottomRight;
			
			
			RealScreen_GetInsets(&insetsTopLeft, &insetsBottomRight);
			MoveControl(realScreenPtr->contentUserPane, insetsTopLeft.h, insetsTopLeft.v);
		}
		
		// if indicated, create the window header and its pluggable components
		if (inFlags & kRealScreenFlagWindowHeader)
		{
			SInt16		applicationResourceFile = ResourceFileManager_GetResFile(kTelnetResourceFileApplication);
			Boolean		headersInitiallyCollapsed = false;
			Size		actualSize = 0L;
			
			
			unless (Preferences_GetData(kTelnetPreferenceClassGeneral,
								kTelnetPreferenceTag_HeadersCollapsed,
								sizeof(headersInitiallyCollapsed), &headersInitiallyCollapsed,
								&actualSize) == kTelnetPrefsResultCodeSuccess)
			{
				headersInitiallyCollapsed = false; // assume window headers are initially visible, if preference cant be found
			}
			
			realScreenPtr->windowHeader = TerminalWindowHeader_New(realScreenPtr->window);
			MoveControl(TerminalWindowHeader_GetContainerControl(realScreenPtr->windowHeader), -1, -1);
			//TerminalWindowHeader_InstallPanel(realScreenPtr->windowHeader, ExtensionPanelDefault_New());
			//TerminalWindowHeader_InstallPanel(realScreenPtr->windowHeader, ExtensionPanelBufferedInput_New());
			{
				realScreenPtr->connectionStatusIcon = StatusIcon_New(realScreenPtr->window);
				StatusIcon_AddStage(realScreenPtr->connectionStatusIcon, applicationResourceFile,
									kIconSuiteConnectionActive, 0L/* ticks of delay in animation - N/A */);
				StatusIcon_AddStage(realScreenPtr->connectionStatusIcon, applicationResourceFile,
									kIconSuiteConnectionDead, 0L/* ticks of delay in animation - N/A */);
				StatusIcon_SetDoAnimate(realScreenPtr->connectionStatusIcon, false/* animate */, true/* reset current stage */);
				TerminalWindowHeader_InstallStatusIcon(realScreenPtr->windowHeader, realScreenPtr->connectionStatusIcon);
			}
			{
				realScreenPtr->encryptionStatusIcon = StatusIcon_New(realScreenPtr->window);
				// IMPORTANT:	There are constants in RealScreen.h that describe
				//				zero-based animation stage indices for encryption icons.  You
				//				must call StatusIcon_AddStage() for each icon in the same
				//				order as is listed by the constants, because the constants
				//				are used to give the status icon a particular appearance.  In
				//				short, if you screw up this simple relationship, the icon
				//				that gets displayed will be wrong.
				StatusIcon_AddStage(realScreenPtr->encryptionStatusIcon, applicationResourceFile,
									kIconSuiteEncryptionInsecure, 0L/* ticks of delay in animation - N/A */);
				StatusIcon_AddStage(realScreenPtr->encryptionStatusIcon, applicationResourceFile,
									kIconSuiteEncryptionEncrypting, 0L/* ticks of delay in animation - N/A */);
				StatusIcon_AddStage(realScreenPtr->encryptionStatusIcon, applicationResourceFile,
									kIconSuiteEncryptionSecure, 0L/* ticks of delay in animation - N/A */);
				StatusIcon_AddStage(realScreenPtr->encryptionStatusIcon, applicationResourceFile,
									kIconSuiteEncryptionDecrypting, 0L/* ticks of delay in animation - N/A */);
				StatusIcon_SetDoAnimate(realScreenPtr->encryptionStatusIcon, false/* animate */, true/* reset current stage */);
				TerminalWindowHeader_InstallStatusIcon(realScreenPtr->windowHeader, realScreenPtr->encryptionStatusIcon);
			}
			TerminalWindowHeader_SetStateChangeProc(realScreenPtr->windowHeader, handlePlacardSizeChange);
			TerminalWindowHeader_SetCollapsed(realScreenPtr->windowHeader, headersInitiallyCollapsed);
		}
		else realScreenPtr->windowHeader = null;
		
		// speech setup
		realScreenPtr->speech.enabled = false;
		realScreenPtr->speech.mode = kTelnetSpeechModeSpeakWhenActive;
		
		// miscellaneous settings
		realScreenPtr->isTextSelectionRectangular = false;
		realScreenPtr->customTextBackground = true;//false; // temp?
		realScreenPtr->picture = null;
		{
			Boolean		useBackgroundPicture = false;
			Size		actualSize = 0L;
			
			
			unless (Preferences_GetData(kTelnetPreferenceClassGeneral,
								kTelnetPreferenceTag_UseBackgroundPicture,
								sizeof(useBackgroundPicture), &useBackgroundPicture,
								&actualSize) == kTelnetPrefsResultCodeSuccess)
			{
				useBackgroundPicture = false; // assume no window pictures, if preference cant be found
			}
			RealScreen_SetSpecialBackground(inIDOfNewScreen, useBackgroundPicture);
		}
		realScreenPtr->obscured = false; // 3.0 - initially, the window will be shown
		realScreenPtr->drawingEnabled = true;
		realScreenPtr->max = 0; // scroll bar settings will be properly initialized by subsequent call to VSsetrgn()
		realScreenPtr->min = 0;
		realScreenPtr->current = 0;
		realScreenPtr->lmax = 0;
		realScreenPtr->lmin = 0;
		realScreenPtr->lcurrent = 0;
		realScreenPtr->isTextSelected = 0;
		realScreenPtr->cursorState = kTelnetCursorStateInvisible;
		realScreenPtr->isReverseVideo = 0;
		RSsetsize(inIDOfNewScreen);
		
		if (inFlags & kRealScreenFlagTextWraps) VSwrite(inIDOfNewScreen, "\033[?7h", 5); // turn on autowrap
		
		SetControlVisibility(realScreenPtr->rootControl, true/* visible */, false/* draw */);
		
		// 3.0 - evade a bug by doing this
		RSchangefont(inIDOfNewScreen, null, realScreenPtr->fsiz);
		
		result = inIDOfNewScreen;
	}
	
	return result;
}// New


/*!
Destroys a screen window created with RealScreen_New(),
freeing all resources allocated by that routine.

(2.6)
*/
void
RealScreen_Dispose	(TelnetScreenID*	inoutScreenIDPtr)
{
	if (inoutScreenIDPtr != null)
	{
		RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(*inoutScreenIDPtr);
		
		
		if (realScreenPtr != null)
		{
			Console_WriteLine("got real screen data");
			if (realScreenPtr->windowHeader != null) TerminalWindowHeader_Dispose(&realScreenPtr->windowHeader);
			StatusIcon_Dispose(&realScreenPtr->connectionStatusIcon);
			StatusIcon_Dispose(&realScreenPtr->encryptionStatusIcon);
			Console_WriteLine("destroyed header");
			
			--((*gTopLeftCorners)[realScreenPtr->positionIndex]); // one less window at this position
			
			// destroy any color palette in the window
			if (realScreenPtr->paletteTerminal != null)
			{
				DisposePalette(realScreenPtr->paletteTerminal), realScreenPtr->paletteTerminal = null;
			}
			Console_WriteLine("destroyed colors");
			
			// destroy the data that was displayed on this screen
			if (*inoutScreenIDPtr >= 0)
			{
				VSdestroy(*inoutScreenIDPtr);
				Console_WriteLine("destroyed virtual screen");
			}
			
			// 3.0 - destroy any picture
			//if (realScreenPtr->backgroundUserPane != null)
			//{
			//Console_WriteLine("destroyed picture");
			//}
			
			// 3.0 - destroy the Text Services Manager document for the window
			(OSErr)FixTSMDocument(realScreenPtr->textServicesDocumentID);
			(OSErr)DeactivateTSMDocument(realScreenPtr->textServicesDocumentID);
			(OSErr)DeleteTSMDocument(realScreenPtr->textServicesDocumentID);
			Console_WriteLine("destroyed TSM document");
			
			// if this window was hidden, decrement the hidden count
			if (realScreenPtr->obscured) ScreenManager_SetHiddenCount(ScreenManager_GetHiddenCount() - 1);
			
			// destroy the window
			if (realScreenPtr->window != null)
			{
				Console_WriteLine("preparing to delete window");
				if (IsWindowVisible(realScreenPtr->window))
				{
					// 3.0 - use a zoom effect to close windows
					HideWindow(realScreenPtr->window);
					if (FlagManager_Test(kFlagCarbonWindowManager))
					{
						Rect		lowerRight,
									screenRect;
						
						
						Console_WriteLine("executing Carbon window manager code");
						MacLook_GetWindowDeviceGrayRect(realScreenPtr->window, &screenRect);
						SetRect(&lowerRight, screenRect.right, screenRect.bottom, screenRect.right, screenRect.bottom);
						(OSStatus)TransitionWindow(realScreenPtr->window, kWindowZoomTransitionEffect,
													kWindowHideTransitionAction, &lowerRight);
					}
				}
				Console_WriteLine("destroying window");
				DisposeWindow(realScreenPtr->window); // close the window containing it, disposing of all controls automatically
				realScreenPtr->window = null;
			}
			
			// 3.0 - dispose of the text encoding converter
			(OSStatus)TECDisposeConverter(realScreenPtr->textEncodingConverter);
			realScreenPtr->textEncodingConverter = null;
			Console_WriteLine("destroyed text encoding converter");
			
			// invalidate the users copy of the screen ID
			*inoutScreenIDPtr = -1;
		}
	}
}// Dispose


/*!
This method handles bell signals in specific terminal
windows.  If the user has global bell sounding turned
off (visual bell), no bell is heard, but the window
is flashed.  If the specified window is not in front
at the time this method is invoked, the window will
flash regardless of the visual bell setting.  If
MacTelnet is suspended when the bell occurs, and the
user has notification preferences set, MacTelnet will
post a notification event.

(3.0)
*/
void
RealScreen_AudioEvent	(TelnetScreenID const		inScreenID)
{
	Boolean				visual = false,				// is visual used?
						visualPreference = false;	// is ONLY visual used?
	unsigned long		dummy = 0L;
	Rect				terminalScreenRect;
	GrafPtr				oldPort = null;
	Size				actualSize = 0L;
	
	
	GetPort(&oldPort);
	
	RealScreen_SetPortScreenPort(inScreenID);
	{
		RealScreenDataPtr		realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
		
		
		if (realScreenPtr != null) GetControlBounds(realScreenPtr->contentUserPane, &terminalScreenRect);
		else SetRect(&terminalScreenRect, 0, 0, 0, 0);
	}
	
	// If the user turned off audible bells, always use a visual;
	// otherwise, use a visual if the beep is in a background window.
	unless (Preferences_GetData(kTelnetPreferenceClassGeneral, kTelnetPreferenceTag_VisualBell,
								sizeof(visualPreference), &visualPreference,
								&actualSize) == kTelnetPrefsResultCodeSuccess)
	{
		visualPreference = false; // assume audible bell, if preference cant be found
	}
	visual = (visualPreference || (!IsWindowHilited(ScreenManager_GetWindowOf(inScreenID))));
	
	if (visual) InvertRect(&terminalScreenRect);
	unless (visualPreference)
	{
		SInt16		resID = ResourceFileManager_GetResFile(kTelnetResourceFileTerminalBellSound);
		Boolean		justBeep = false; // dont use a custom sound?
		
		
		if (resID >= 0)
		{
			// then the user has provided a Bell file; use its sound resource instead
			enum
			{
				kSoundResourceType = 'snd '
			};
			SInt16		oldResFile = CurResFile();
			OSErr		error = noErr;
			
			
			ResourceFileManager_UseResFile(kTelnetResourceFileTerminalBellSound);
			if (Count1Resources(kSoundResourceType) > 0)
			{
				Handle		resHandle = Get1IndResource(kSoundResourceType, 1); // get first available sound
				
				
				error = ResError();
				if (error == noErr)
				{
					SInt16		resID = 0;
					ResType		resType = kSoundResourceType;
					Str255		resName;
					
					
					GetResInfo(resHandle, &resID, &resType, resName);
					error = ResError();
					if (error == noErr) error = Sound_PlayAsyncStereo(resType, resID, null/* SoundInfoRef storage */);
				}
			}
			else error = resNotFound;
			UseResFile(oldResFile);
			
			justBeep = (error != noErr);
			if (justBeep) Console_WriteValue("Error while trying to play custom bell sound", error);
		}
		else justBeep = true;
		
		// no custom sound file, or an error occurred while trying to play the sound, so emit a standard beep
		if (justBeep) BEEP;
	}
	
	// Mac OS 8 asynchronous sounds mean that a sound generates
	// very little delay, therefore a standard visual delay of 8
	// ticks should be enforced even if a beep was emitted.
	Delay(8, &dummy);
	
	if (visual) InvertRect(&terminalScreenRect);
	
	SetPort(oldPort);
	NotifyUser();
}// AudioEvent


/*!
Displays a dialog box with a text field containing
the title of the specified window, and allows the
user to edit the title.  If the user decides to
change the title, the title of the window is updated
to reflect the new name; otherwise, the window is
unchanged.

This method has been updated in Telnet 3.0 to use a
movable, resizable and Appearance-savvy dialog box.
For Mac OS 8.5 users, this dialog is even cooler,
because it transitions to and from the title area of
the specified window.

(2.6)
*/
void
RealScreen_DisplaySetWTitleDialogFor	(WindowRef		inoutWhichWindow)
{
	if (inoutWhichWindow != null)
	{
		WindowInfoRef		windowInfo = null;
		DialogRef			dialog = null;
		WindowRef			parentWindow = null;
		DialogItemIndex		itemHit = 0;
		Str255				name;
		
		
		GetNewDialogWithWindowInfo(kDialogIDSetWindowTitle, &dialog, &windowInfo);
		
		// determine if the Help button should be disabled...
		SetUpHelpButtonDialogItem(dialog, iSetWindowTitleAppleGuideButton);
		
		SetDialogDefaultItem(dialog, iSetWindowTitleSetTitleButton);
		SetDialogCancelItem(dialog, iSetWindowTitleCancelButton);
		
		GetWTitle(inoutWhichWindow, name);
		SetDialogItemControlText(dialog, iSetWindowTitleTitleField, name);
		
		itemHit = 0;
		
		// adjust the Set Title and Cancel buttons so they are big enough for their titles
		(UInt16)Localization_AdjustDialogButtons(dialog, true/* OK */, true/* Cancel */);
		Localization_AdjustHelpButtonItem(dialog, iSetWindowTitleAppleGuideButton);
		Localization_HorizontallyPlaceItems(dialog, iSetWindowTitleFieldLabelText, iSetWindowTitleTitleField);
		
		parentWindow = GetRealFrontWindow();
		TransitionDialogDisplayFromWindowRegion(dialog, parentWindow, kWindowTitleTextRgn);
		
		Cursors_UseArrow();
		
		// set up the Window Info information
		{
			WindowResizeResponderProcPtr	resizeProc = adjustSetWindowTitleDialog;
			Rect							rect;
			
			
			MacLook_GetWindowRegionBounds(GetDialogWindow(dialog), kWindowContentRgn, &rect);
			windowInfo = WindowInfo_New();
			WindowInfo_SetWindowDescriptor(windowInfo, kTelnetDialogDescriptorSetWindowTitle);
			WindowInfo_SetWindowResizeLimits(windowInfo,
													rect.bottom - rect.top + 1 /* minimum height */,
													rect.right - rect.left + 1 /* minimum width */,
													rect.bottom - rect.top + 1 /* maximum height */,
													rect.right - rect.left + 200/* maximum width - arbitrary */);
			WindowInfo_SetWindowResizeResponderUPP(windowInfo, resizeProc, 0L);
		}
		WindowInfo_SetForDialog(dialog, windowInfo);
		
		{
			ControlRef		fieldControl = null;
			
			
			(OSErr)GetDialogItemAsControl(dialog, iSetWindowTitleTitleField, &fieldControl);
			(OSErr)SetKeyboardFocus(GetDialogWindow(dialog), fieldControl, kControlFocusNextPart);
		}
		
		Menus_EnableEditMenu(); // allows edit commands to show up as enabled
		
		{
			ModalFilterUPP		filterProc = NewModalFilterUPP(setWindowTitleDialogEventFilter);
			
			
			do
			{
				ModalDialog(filterProc, &itemHit);
				if (itemHit == iSetWindowTitleAppleGuideButton)
				{
					Str255		searchString;
					
					
					GetIndString(searchString, rStringsAppleGuideLookup, siAGIndexTermConnections);
					AppleGuideUtilities_OpenFirstMainGuideInSearch(searchString);
				}
			} while ((itemHit != iSetWindowTitleSetTitleButton) && (itemHit != iSetWindowTitleCancelButton));
			DisposeModalFilterUPP(filterProc), filterProc = null;
		}
		
		if (itemHit == iSetWindowTitleSetTitleButton)
		{
			// set title
			GetDialogItemControlText(dialog, iSetWindowTitleTitleField, name);
			if (StrLength(name) <= 0) PLstrcpy(name, "\p ");
			RealScreen_SetWTitle(ScreenManager_GetWindowScreenID(inoutWhichWindow), name);
			TransitionDialogCloseToWindowRegion(dialog, parentWindow, kWindowTitleTextRgn);
		}
		else
		{
			// cancelled
			TransitionDialogCloseCancel(dialog);
		}
		WindowInfo_Dispose(windowInfo);
		DisposeDialog(dialog), dialog = null;
	}
}// DisplaySetWTitleDialogFor


/*!
Erases a rectangular portion of the screen
display, preserving highlighted areas.

(2.6)
*/
void
RealScreen_EraseSection		(TelnetScreenID const		inScreenID,
							 SInt16 const				inLeftmostColumnToErase,
							 SInt16 const				inTopmostRowToErase,
							 SInt16 const				inRightmostColumnToErase,
							 SInt16 const				inBottommostRowToErase)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) if (realScreenPtr->drawingEnabled)
	{
		Rect		rect;
		
		
		//setRealScreenAttr(realScreenPtr, 0);
		SetRect(&rect, inLeftmostColumnToErase * realScreenPtr->fwidth,
						inTopmostRowToErase * realScreenPtr->fheight,
						(inRightmostColumnToErase + 1) * realScreenPtr->fwidth - 1,
						(inBottommostRowToErase + 1) * realScreenPtr->fheight + 1);
		if (rect.left < 0) rect.left = 0;
		if (rect.right >= realScreenPtr->width - 1) rect.right = realScreenPtr->rwidth - 2;
		if (rect.bottom >= realScreenPtr->height - 2) rect.bottom = realScreenPtr->rheight + 1;
		
		screenToLocalRect(realScreenPtr, &rect);
		unless (realScreenPtr->customTextBackground)
		{
			setRealScreenColorAttr(realScreenPtr, 0);
			EraseRect(&rect);
		}
		updateTextSelection(inScreenID, &rect);
	}
}// EraseSection


/*!
To get the color with the specified index from
the color table of the indicated screen, use
this method.  If the color could be acquired,
true is returned; otherwise, false is returned.

(3.0)
*/
Boolean
RealScreen_GetColor		(TelnetScreenID const			inScreenID,
						 RealScreenColorIndex const		inColorEntryNumber,
						 RGBColor*						outColorPtr)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	Boolean				result = false;
	
	
	if (outColorPtr != null)
	{
		outColorPtr->red = outColorPtr->green = outColorPtr->blue = 0;
		if (realScreenPtr != null)
		{
			getScreenColor(realScreenPtr, inColorEntryNumber, outColorPtr);
			result = true;
		}
	}
	return result;
}// GetColor


/*!
Returns the screen insets.  These are the pixel offsets
from the edges of the maximum viewable area that define
the padding between the absolute edge of the screen and
the outer rows and columns of text.  Without insets, the
text would be flush against the edges of the screen, which
can be very annoying and can make it harder to select text.

All of the insets are generally positive numbers.  If the
insets were negative, the screen text would get clipped at
its edges.

The top-left insets are measured intuitively relative to
the top-left corner of the screen area.

The bottom-right insets are expressed positively!!!  The
horizontal value refers to a number of pixels to the left
of the rightmost edge, and similarly the vertical value
refers to a number of pixels above the bottom edge.

(3.0)
*/
void
RealScreen_GetInsets		(Point*			outTopLeftInsetOrNull,
							 Point*			outBottomRightInsetOrNull)
{
	// NOTE: Insets shouldnt be TOO large (i.e. comparable to the size of a character),
	//       because then the user might mistake the padding for an empty row or column.
	//       Also, its generally more aesthetically pleasing if the width is larger
	//       than the height.
	if (outTopLeftInsetOrNull != null) SetPt(outTopLeftInsetOrNull, 3, SHADOW_RGBCOLOR_INTENSITY_DIVISOR);
	if (outBottomRightInsetOrNull != null) SetPt(outBottomRightInsetOrNull, 3, 2);
}// GetInsets


/*!
To determine the height of the placard control, use
this method.  The placard height is defined as the
number of pixels between the top edge of the window
and the top edge of the terminal screen.

(3.0)
*/
UInt16
RealScreen_GetPlacardHeight		(TelnetScreenID const	inScreenID)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	UInt16				result = 0;
	
	
	if (realScreenPtr != null)
	{
		if (realScreenPtr->windowHeader != null)
		{
			Point		size;
			
			
			if (TerminalWindowHeader_IsCollapsed(realScreenPtr->windowHeader)) size.h = size.v = 0;
			else TerminalWindowHeader_GetNormalSize(realScreenPtr->windowHeader, &size);
			result = size.v;
		}
	}
	return result;
}// GetPlacardHeight


/*!
Returns the boundaries of a particular region of
the indicated screen window.  If the indicated
region is not rectangular, this method returns
the tightest bounding rectangle of the region.

The possible region codes are as follows:

"kRealScreenRegionCodeScreenArea"
	This is the region of the window that can
	be scrolled with the scroll bars, the text
	area (the terminal screen itself).  It
	does not include the invisible screen border.

"kRealScreenRegionCodeScreenBackground"
	This is the region of the window that is
	filled with the screen background color;
	this includes the screen area (see the
	constant above), as well as a small border
	around its edges.

(3.0)
*/
void
RealScreen_GetRegionBounds		(TelnetScreenID const		inScreenID,
								 RealScreenRegionCode const	inRegionCode,
								 Rect*						outBounds)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) getRegionBounds(realScreenPtr, inRegionCode, outBounds);
}// GetRegionBounds


/*!
To determine the height of a scroll bar, use this method.
In Carbon, this routine automatically calls an Appearance
API to determine the possibly-dynamic scroll bar size.

(3.0)
*/
UInt16
RealScreen_GetScrollBarHeight	(void)
{
	UInt16		result = 0;
	SInt32		data = 0L;
	
	
	(OSStatus)GetThemeMetric(kThemeMetricScrollBarWidth, &data);
	result = data;
	return result;
}// GetScrollBarHeight


/*!
To determine the width of a scroll bar, use this method.
In Carbon, this routine automatically calls an Appearance
API to determine the possibly-dynamic scroll bar size.

(3.0)
*/
UInt16
RealScreen_GetScrollBarWidth	(void)
{
	UInt16		result = 0;
	SInt32		data = 0L;
	
	
	(OSStatus)GetThemeMetric(kThemeMetricScrollBarWidth, &data);
	result = data;
	return result;
}// GetScrollBarWidth


/*!
Returns the contents of the current selection for
the specified window, allocating a new handle.  If
there is no selection, an empty handle is returned.
If any problems occur, null is returned.

Use the parameters to this routine to affect how the
text is returned; for example, you can have the text
returned inline, or delimited by carriage returns.
The "inNumberOfSpacesToReplaceWithOneTabOrZero" value
can be 0 to perform no substitution, or a positive
integer to indicate how many consecutive spaces are
to be replaced with a single tab before returning the
text.

You must dispose of the returned handle yourself,
using Memory_DisposeHandle().

(3.0)
*/
Handle
RealScreen_GetSelectedTextAsNewHandle	(TelnetScreenID const		inScreenID,
										 UInt16 const				inNumberOfSpacesToReplaceWithOneTabOrZero,
										 RealScreenTextFlags const	inFlags)
{
    RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	Handle				result = null;
	
	
	if (realScreenPtr != null) if (RealScreen_IsTextSelected(inScreenID))
	{
		short		maxwid = 0;
		Size		size = 0L;
		Point		start,
					end;
		
		
		maxwid = VSmaxwidth(inScreenID);
		start = realScreenPtr->anchor;
		end = realScreenPtr->last;
		
		size = (start.v - end.v) * sizeof(char);
		if (size < 0) size = -size;
		size += sizeof(char); // lines 2, 3 selected can be 2 lines
		size *= ((maxwid + 2) * sizeof(char));
		result = Memory_NewHandle(size);
		if (result != null)
		{
			char*		characters = null;
			
			
			HLock(result);
			characters = *result;
			
			// this actually returns a character count, not a proper size, so multiply it by sizeof(char)
			size = sizeof(char) * VSgettext(inScreenID, start.h, start.v, end.h, end.v, characters, size,
											(inFlags & kRealScreenTextFlagInline) ? "" : "\015",
											inNumberOfSpacesToReplaceWithOneTabOrZero,
											realScreenPtr->isTextSelectionRectangular);
			HUnlock(result);
			Memory_SetHandleSize(result, size);
		}
	}
	
	return result;
}// GetSelectedTextAsNewHandle


/*!
Returns a region describing the highlighted
text in the specified window.  You must
destroy this region yourself!

(2.6)
*/
RgnHandle
RealScreen_GetSelectedTextAsNewRegion		(TelnetScreenID const		w)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
	RgnHandle			result = null;
	
	
	if (realScreenPtr != null) result = getSelectedTextAsNewRegion(realScreenPtr);
	
	return result;
}// GetSelectedTextAsNewRegion


/*!
Handles a click in a terminal window.  Possible
responses are to select text, drag text, display
a contextual menu, position the cursor with the
mouse, toggle a control in the window header, or
drag a scroll bar.

(2.6)
*/
SInt16
RealScreen_HandleContentClick		(WindowRef		inWindow,
									 EventRecord*	inoutEventPtr)
{
	SInt16		result = 0;
	
	
	if (ScreenManager_IsWindowAScreen(inWindow))
	{
		TelnetScreenID		screenID = 0;
		
		
		screenID = ScreenManager_GetWindowScreenID(inWindow);
		if (screenID < 0) result = -1;
		else
		{
			RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(screenID);
			
			
			if (realScreenPtr != null)
			{
				SInt16				x1 = 0,
									x2 = 0,
									y1 = 0,
									y2 = 0;
				Point				where;
				GrafPtr				oldPort = null;
				ControlRef			control = null;
				ControlPartCode		part = kControlNoPart;
				Boolean				handled = false;
				
				
				GetPort(&oldPort);
				SetPortWindowPort(inWindow);
				where = inoutEventPtr->where;
				GlobalToLocal((Point*)&where);
				control = FindControlUnderMouse(where, inWindow, &part);
				
				// content click?
				if (part != kControlNoPart) switch (part)
				{
					case kControlIndicatorPart:
					case kControlUpButtonPart:
					case kControlDownButtonPart:
					case kControlPageUpPart:
					case kControlPageDownPart:
						// 3.0
						{
							ControlActionUPP	scrollProcUPP = NewControlActionUPP(scrollProc);
							
							
							(SInt16)HandleControlClick(control, where, inoutEventPtr->modifiers, scrollProcUPP);
							DisposeControlActionUPP(scrollProcUPP), scrollProcUPP = null;
							handled = true;
						}
						break;
					
					default:
						break;
				}
				
				unless (handled)
				{
					Rect		rect;
					
					
					GetControlBounds(realScreenPtr->contentUserPane, &rect);
					if (PtInRect(where, &rect))
					{
						if ((inoutEventPtr->modifiers & optionKey) && (inoutEventPtr->modifiers & cmdKey))
						{
							// send host the appropriate sequences to move the cursor to the specified position
							RealScreen_MoveCursorWithArrowKeys(screenID, where);
						}
						else
						{
							if (ClickInContent(where, screenID))
							{
								Boolean		dragged = false;
								
								
								(OSErr)Drag_TrackScreenSelection(inoutEventPtr, where, screenID, &dragged);
								unless (dragged) RealScreen_TrackTextSelection(screenID, where, inoutEventPtr);
							}
						}
					}
					else if (realScreenPtr->windowHeader != null)
					{
						GetControlBounds(TerminalWindowHeader_GetContainerControl(realScreenPtr->windowHeader), &rect);
						if (PtInRect(where, &rect))
						{
							TerminalWindowHeader_HandleContentClick(realScreenPtr->windowHeader, inoutEventPtr);
						}
					}
				}
				SetPort(oldPort);
			}
		}
	}
	return result;
}// HandleContentClick


/*!
This method moves and resizes the contents of a
terminal window in response to a resize.  In
MacTelnet 3.0, the Window Info module makes it
possible for session windows to be resized
*dynamically*, which is incredibly cool!

If the window is resized successfully, "true"
is returned; otherwise, "false" is returned.

(3.0)
*/
static pascal Boolean
RealScreen_HandleNewSize	(WindowRef const		inWindow,
							 SInt32 const			UNUSED_ARGUMENT(inDeltaSizeX),
							 SInt32 const			UNUSED_ARGUMENT(inDeltaSizeY),
							 SInt32 const			UNUSED_ARGUMENT(inData))
{
	Boolean			result = pascal_true;
	
	
	if (ScreenManager_IsWindowAScreen(inWindow))
	{
		TelnetScreenID		screenID = ScreenManager_GetWindowScreenID(inWindow);
		RealScreenDataPtr	realScreenPtr = null;
		
		
		realScreenPtr = ScreenManager_GetRealScreenDataOf(screenID);
		
		if (realScreenPtr != null)
		{
			Rect		contentRegionBounds;
			SInt16		x1 = 0,
						x2 = 0,
						y1 = 0,
						y2 = 0;
			
			
			MacLook_GetWindowRegionBounds(inWindow, kWindowContentRgn, &contentRegionBounds);
			
			// determine the number of rows and columns that the users specified rectangle most closely matches
			VSgetrgn(screenID, &x1, &y1, &x2, &y2);
			{
				SInt16		width = 0,
							height = 0;
				
				
				getScreenSizeFromWindowSize(realScreenPtr, contentRegionBounds.right - contentRegionBounds.left,
											contentRegionBounds.bottom - contentRegionBounds.top, &width, &height);
				VSsetrgn(screenID, x1, y1,
						(short)((x1 + width / realScreenPtr->fwidth - 1)),
						(short)((y1 + height / realScreenPtr->fheight - 1)));
			}
			VSgetrgn(screenID, &x1, &y1, &x2, &y2); // get the new region
			
			// resize the screen (and adjust the window boundary slightly, if necessary)
			ScreenManager_ChangeDimensionsOf(screenID, x2 - x1 + 1, y2 - y1 + 1, false/* recordable */);
		}
		else result = pascal_false;
	}
	
	return result;
}// HandleNewSize


/*!
To determine whether screen updates are being serviced
for a particular terminal screen, use this method.

(2.6)
*/
Boolean
RealScreen_IsDrawingEnabled		(TelnetScreenID const		inScreenID)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	Boolean				result = false;
	
	
	if (realScreenPtr != null) result = realScreenPtr->drawingEnabled;
	return result;
}// IsDrawingEnabled


/*!
To determine whether there is currently any text
selected in the specified session window, use
this method.

(2.6)
*/
Boolean
RealScreen_IsTextSelected	(TelnetScreenID const		inScreenID)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	Boolean				result = false;
	
	
	if (realScreenPtr != null) result = realScreenPtr->isTextSelected;
	
	return result;
}// IsTextSelected


/*!
MacTelnet 3.0 supports two modes of text
selection: standard, like most Macintosh
applications, and rectangular, which allows
only rectangular regions, anywhere in a
terminal window, to be highlighted.  This
second mode is very useful for capturing
specific output from terminal applications
(for examples, columns of output from a
UNIX command line result, like "ls -l").

To determine if text selections in a
particular window use this new mode of
highlighting, use this method.

(3.0)
*/
Boolean
RealScreen_IsTextSelectionRectangular	(TelnetScreenID const		inScreenID)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	Boolean				result = 0;
	
	
	if (realScreenPtr != null) result = realScreenPtr->isTextSelectionRectangular;
	return result;
}// IsTextSelectionRectangular


/*!
To translate pixels in the coordinate system of a
window so that they are relative to the origin of
the screen, use this method.  In version 2.6, the
program always assumed that the screen origin was
the top-left corner of the window, which made it
extremely hard to change.  Now, this routine is
invoked everywhere to ensure that, before port
pixel offsets are used to refer to the screen,
they get translated correctly.

If you are not interested in translating a point,
but rather in shifting a dimension horizontally or
vertically, you can pass "null" for the dimension
that you do not use.

See also RealScreen_ScreenToLocal().

(3.0)
*/
void
RealScreen_LocalToScreen		(TelnetScreenID const	inScreenID,
								 SInt16*				inoutHorizontalPixelOffsetFromPortOrigin,
								 SInt16*				inoutVerticalPixelOffsetFromPortOrigin)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) localToScreen(realScreenPtr, inoutHorizontalPixelOffsetFromPortOrigin,
												inoutVerticalPixelOffsetFromPortOrigin);
}// LocalToScreen


/*!
To translate a boundary in the coordinate system of a
window so that it is relative to the origin of the
screen, use this method.  In version 2.6, the program
always assumed that the screen origin was the top-left
corner of the window, which made it extremely hard to
change.  Now, this routine is invoked everywhere to
ensure that, before port pixel offsets are used to
refer to the screen, they get translated correctly.

(3.0)
*/
void
RealScreen_LocalToScreenRect	(TelnetScreenID const	inScreenID,
								 Rect*					inoutPortOriginBounds)
{
	RealScreenDataPtr		realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) localToScreenRect(realScreenPtr, inoutPortOriginBounds);
}// LocalToScreenRect


/*!
MacTelnet 3.0 supports two modes of text
selection: standard, like most Macintosh
applications, and rectangular, which allows
only rectangular regions, anywhere in a
terminal window, to be highlighted.  This
second mode is very useful for capturing
specific output from terminal applications
(for examples, columns of output from a
UNIX command line result, like "ls -l").

To make text selections rectangular in a
particular window, use this method and pass
a value of "true" for the second parameter.
To reset text selections to normal, pass
"false".

IMPORTANT:	All of MacTelnets functions,
			including updating the terminal
			text selection of a window,
			rely on this flag.  Thus, you
			should ensure a text selection
			is *empty* before toggling this
			value (to prevent any selected
			text from appearing to cover a
			different region than it really
			does).

(3.0)
*/
void
RealScreen_MakeSelectionsRectangular	(TelnetScreenID const		inScreenID,
										 Boolean const				inAreSelectionsNotAttachedToScreenEdges)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) realScreenPtr->isTextSelectionRectangular = inAreSelectionsNotAttachedToScreenEdges;
}// MakeSelectionsRectangular


/*!
Positions the cursor at the screen row and column underneath
the specified point in coordinates local to the screen window.
This is done interactively, by sending the appropriate cursor
control key sequences to the server.  Invoking this function
to move the cursor ensures that remote applications see that
the cursor has moved, and where it has moved to.

(3.0)
*/
void
RealScreen_MoveCursorWithArrowKeys		(TelnetScreenID const	inScreenID,
										 Point const			inLocalMouse)
{
	Point		position;
	
	
	position = findVirtualCellFromLocalPoint(inScreenID, inLocalMouse, false/* scroll if necessary */);
	VSpossend(inScreenID, position.h, position.v, ScreenManager_GetConnectionDataOf(inScreenID)->echo);
}// MoveCursorWithArrowKeys


/*!
To change the width of a window (usually done to alternate
between 80 and 132 columns), use this method.  The window
is adjusted to just fit the specified number of columns.
The window height does not change.

(3.0)
*/
void
RealScreen_NewWidth		(TelnetScreenID const		inScreenID,
						 UInt16 const				inNewNumberOfColumns)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		SInt16		x1 = 0,
					x2 = 0,
					y1 = 0,
					y2 = 0;
		SInt16		lines = 0;
		SInt16		resizeWidth = 0,
					resizeHeight = 0;
		GrafPtr		oldPort = null;
		
		
		GetPort(&oldPort);
		RealScreen_SetPortScreenPort(inScreenID);
		setCursorVisibility(realScreenPtr, false/* visible */);
		VSsetcols(inScreenID, (short)(inNewNumberOfColumns - 1));
		VSgetrgn(inScreenID, &x1, &y1, &x2, &y2); // get current visible region
		x2 = inNewNumberOfColumns - 1;
		lines = VSgetlines(inScreenID);				/* NCSA: SB - trust me, you need this... */
		realScreenPtr->rwidth = realScreenPtr->width = (x2 - x1 + 1) * realScreenPtr->fwidth;
		realScreenPtr->rheight = realScreenPtr->height= (y2 - y1 + 1) * realScreenPtr->fheight; 
		
		if (realScreenPtr->rwidth > (inNewNumberOfColumns)*realScreenPtr->fwidth)
			 realScreenPtr->rwidth = (inNewNumberOfColumns)*realScreenPtr->fwidth;
		if (realScreenPtr->rheight > (lines)*realScreenPtr->fheight)
			 realScreenPtr->rheight = (lines)*realScreenPtr->fheight;
		
		// set the width and height
		getWindowSizeFromScreenSize(realScreenPtr, realScreenPtr->rwidth, realScreenPtr->rheight,
									&resizeWidth, &resizeHeight);
		{
			Rect		contentRegionBounds;
			
			
			// make sure the proposed width and height fit in the current monitors screen area;
			// if not, on output RScheckmaxwind() will change the size to acceptable values
			MacLook_GetWindowRegionBounds(realScreenPtr->window, kWindowContentRgn, &contentRegionBounds);
			RScheckmaxwind(inScreenID, &contentRegionBounds, resizeWidth, resizeHeight, &resizeWidth, &resizeHeight);
		}
		SizeWindow(realScreenPtr->window, resizeWidth, resizeHeight, false/* update */);
		getScreenSizeFromWindowSize(realScreenPtr, resizeWidth, resizeHeight,
									&realScreenPtr->rwidth, &realScreenPtr->rheight);
		RSsetsize(inScreenID);
		VSgetrgn(inScreenID, &x1, &y1, &x2, &y2);
		VSsetrgn(inScreenID, x1, y1,
			(short)(x1 + (realScreenPtr->rwidth) / realScreenPtr->fwidth - 1),
			(short)(y1 + (realScreenPtr->rheight) / realScreenPtr->fheight - 1));
		VSgetrgn(inScreenID, &x1, &y1, &x2, &y2);		/* Get new region */
		
		VSredraw(inScreenID, 0, 0, x2 - x1 + 1, y2 - y1 + 1); /* redraw newly-revealed area, if any */
		MacLook_ValidatePortRectangle(realScreenPtr->window);
		
		SetPort(oldPort);
		
		setCursorVisibility(realScreenPtr, false/* visible */);
	}
}// NewWidth


/*!
To activate the next screen window in the
window list, use this method.  If the option
key is down, the previous window is activated
instead.  If the shift key is down, the
current window is obscured before the next
window is activated.

(3.0)
*/
void
RealScreen_NextWindow		(void)
{
	register SInt16		currentWindowID = ScreenManager_GetWindowScreenID(GetRealFrontWindow()),
						nextWindowID = 0;
	WindowRef			nextWindow = null;
	
	
	if (EventLoop_IsShiftKeyDown())
	{
		// shift key means previous window, so find the previous window ID
		nextWindowID = currentWindowID - 1;
		if (nextWindowID < 1) nextWindowID = Connections_GetCount();
	}
	else
	{
		// no shift key means next window, so find the next window ID
		nextWindowID = currentWindowID + 1;
		if (nextWindowID > Connections_GetCount()) nextWindowID = 1;
	}
	nextWindow = ScreenManager_GetWindowOf(nextWindowID);
	
	// if the option key is down, hide the current window first
	if (EventLoop_IsOptionKeyDown()) Commands_ExecuteByID(kCommandHideFrontWindow, 0L/* modifiers */);
	
	// bring the next window to the front
	if (nextWindow != null) SelectOverRealFrontWindow(nextWindow);
}// NextWindow


/*!
If the user has the cursor flashing preference set, this
routine will hide the specified terminal screens cursor.
This helps to avoid drawing glitches prior to scrolling.

(3.0)
*/
void
RealScreen_ObscureCursor	(TelnetScreenID const		inScreenID)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		if (cursorBlinks(realScreenPtr)) setCursorVisibility(realScreenPtr, false/* visible */);
	}
}// ObscureCursor


/*!
Determines if the specified point, in global
coordinates, is within a particular region of
the specified screen window.

The region codes are identical to those used
with RealScreen_GetRegionBounds().

(3.0)
*/
Boolean
RealScreen_PtInRgn		(TelnetScreenID const			inScreenID,
						 Point const					inGlobalPoint,
						 RealScreenRegionCode const		inRegionCode)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	Boolean				result = false;
	
	
	if (realScreenPtr != null)
	{
		Point		localPoint = inGlobalPoint;
		Rect		rect;
		
		
		GlobalToLocal(&localPoint);
		getRegionBounds(realScreenPtr, inRegionCode, &rect);
		result = PtInRect(localPoint, &rect);
	}
	return result;
}// PtInRgn


/*!
Determines whether a point in the coordinate
system of a windows graphics port is in the
boundaries of the highlighted text in that
window (if any).

Updated in MacTelnet 3.0 to account for a
selection area that is rectangular.

(2.6)
*/
Boolean
RealScreen_PtInSelection	(TelnetScreenID const		inScreenID,
							 Point const				inLocalPoint)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	Boolean				result = false;
	
	
	if (realScreenPtr != null)
	{
		Point		normalizedPoint = findVirtualCellFromLocalPoint(inScreenID, inLocalPoint, false/* scroll if necessary */);
		
		
		if (realScreenPtr->isTextSelectionRectangular)
		{
			// purely rectangular selection range
			result = ((normalizedPoint.h >= realScreenPtr->anchor.h) &&
						(normalizedPoint.v >= realScreenPtr->anchor.v) &&
						(normalizedPoint.h < realScreenPtr->last.h) &&
						(normalizedPoint.v <= realScreenPtr->last.v));
		}
		else
		{
			// Macintosh-like selection range
			SInt16		columns = 0;
			SInt32		beg_offset = 0L,
						end_offset = 0L,
						current_offset = 0L;
			
			
			columns = VSgetcols(inScreenID) - 1;
			beg_offset = columns * realScreenPtr->anchor.v + realScreenPtr->anchor.h;
			end_offset = columns * realScreenPtr->last.v + realScreenPtr->last.h;
			if (beg_offset != end_offset)
			{
				current_offset = columns * normalizedPoint.v + normalizedPoint.h;
				result = ((current_offset >= beg_offset) && (current_offset < end_offset));
			}
		}
	}
	return result;
}// PtInSelection


/*!
To reverse the foreground and background colors
of a window, use this method.

(2.6)
*/
void
RealScreen_ReverseVideo		(TelnetScreenID const		inScreenID,
							 Boolean const				inReverseVideo)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		GrafPtr		oldPort = null;
		RGBColor	temp1,
					temp2;
		
		
		GetPort(&oldPort);
		RealScreen_SetPortScreenPort(inScreenID);
		
		if (inReverseVideo != realScreenPtr->isReverseVideo)
		{
			realScreenPtr->isReverseVideo = !realScreenPtr->isReverseVideo;
			
			// flip the background and foreground color positions
			getScreenColor(realScreenPtr, kRealScreenColorIndexNormalText, &temp1);
			getScreenColor(realScreenPtr, kRealScreenColorIndexNormalBackground, &temp2);
			setScreenColor(realScreenPtr, kRealScreenColorIndexNormalText, &temp2);
			setScreenColor(realScreenPtr, kRealScreenColorIndexNormalBackground, &temp1);
			
			// update the screen
			MacLook_InvalidatePortRectangle(realScreenPtr->window);
		}
		
		SetPort(oldPort);
	}
}// ReverseVideo


/*!
To translate pixels in the coordinate system of a
screen so that they are relative to the origin of
the window itself, use this method.  In version 2.6,
the program always assumed that the screen origin
was the top-left corner of the window, which made
it extremely hard to change.  Now, this routine is
invoked everywhere to ensure that, before pixel
offsets are used to refer to the screen, they get
translated correctly.

If you are not interested in translating a point,
but rather in shifting a dimension horizontally or
vertically, you can pass "null" for the dimension
that you do not use.

See also RealScreen_LocalToScreen().

(3.0)
*/
void
RealScreen_ScreenToLocal		(TelnetScreenID const	inScreenID,
								 SInt16*				inoutHorizontalPixelOffsetFromScreenOrigin,
								 SInt16*				inoutVerticalPixelOffsetFromScreenOrigin)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) screenToLocal(realScreenPtr, inoutHorizontalPixelOffsetFromScreenOrigin,
												inoutVerticalPixelOffsetFromScreenOrigin);
}// ScreenToLocal


/*!
To translate a boundary in the coordinate system of a
screen so that it is relative to the origin of the
window itself, use this method.  In version 2.6, the
program always assumed that the screen origin was the
top-left corner of the window, which made it extremely
hard to change.  Now, this routine is invoked
everywhere to ensure that, before pixel offsets are
used to refer to the screen, they get translated
correctly.

(3.0)
*/
void
RealScreen_ScreenToLocalRect	(TelnetScreenID const	inScreenID,
								 Rect*					inoutScreenOriginBounds)
{
	if (inoutScreenOriginBounds != null)
	{
		RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
		
		
		if (realScreenPtr != null) screenToLocalRect(realScreenPtr, inoutScreenOriginBounds);
	}
}// ScreenToLocalRect


/*!
Clears the text selection for the specified screen.

(2.6)
*/
void
RealScreen_SelectNothing	(TelnetScreenID const		inScreenID)
{
	RealScreenDataPtr		realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		Rect		screenArea;
		
		
		GetControlBounds(realScreenPtr->contentUserPane, &screenArea);
		highlightVirtualRange(realScreenPtr, realScreenPtr->anchor, realScreenPtr->last, &screenArea);
		realScreenPtr->isTextSelected = false;
	}
}// SelectNothing


/*!
Sets the active state of the specified screen window.

(2.6)
*/
void
RealScreen_SetActive		(WindowRef			inWindow,
							 Boolean const		inActivating)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetWindowRealScreenData(inWindow);
	
	
	if (realScreenPtr != null)
	{
		if (inActivating)
		{
			// its an activate event
			ConnectionDataPtr		connectionDataPtr = ScreenManager_GetWindowConnectionData(inWindow);
			
			
			if (connectionDataPtr != null) 
			{
				EventLoop_SetScriptFor(connectionDataPtr);
				if (connectionDataPtr->TEK.graphicsID > -1)
				{
					detachGraphics(connectionDataPtr->TEK.graphicsID);
				}
				handleScreenActivate(realScreenPtr);
			}
		}
		else
		{
			// its a deactivate event
			handleScreenDeactivate(realScreenPtr);
		}
	}
}// SetActive


/*!
Highlights the specified range of text, clearing any
other select outside the range.

In MacTelnet 3.0, the exact characters selected depends
on the current selection mode: in normal mode, the
specified anchors mark the first and last characters of
a Macintosh-style selection range.  However, if the new
rectangular text selection mode is being used, the
width of each line of the selected region is the same,
equal to the difference between the horizontal coordinates
of the given points.

(2.6)
*/
void
RealScreen_SelectVirtualRange	(TelnetScreenID const		inScreenID,
								 Point const				inRowColumnSelectionStart,
								 Point const				inRowColumnSelectionEnd)
{
	RealScreenDataPtr		realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		Rect		screenArea;
		
		
		if (realScreenPtr->isTextSelected) RealScreen_SelectNothing(inScreenID);
		
		realScreenPtr->anchor.v = inRowColumnSelectionStart.v;
		realScreenPtr->anchor.h = inRowColumnSelectionStart.h;
		realScreenPtr->last.v = inRowColumnSelectionEnd.v;
		realScreenPtr->last.h = inRowColumnSelectionEnd.h;
		
		GetControlBounds(realScreenPtr->contentUserPane, &screenArea);
		highlightVirtualRange(realScreenPtr, realScreenPtr->anchor, realScreenPtr->last, &screenArea);
		realScreenPtr->isTextSelected = true;
	}
}// SelectVirtualRange


/*!
To set whether ANSI-colored text and graphics
are allowed in the specified window, use this
method.

(2.6)
*/
void
RealScreen_SetANSIColorsEnabled		(TelnetScreenID const		inScreenID,
									 Boolean const				inUseANSIColorSequences)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) realScreenPtr->areANSIColorsEnabled = inUseANSIColorSequences;
}// SetANSIColorsEnabled


/*!
To set a new value for a specific color in a
terminal window, use this method.  If the color
could be set, true is returned; otherwise,
false is returned.

(3.0)
*/
Boolean
RealScreen_SetColor		(TelnetScreenID const			inScreenID,
						 RealScreenColorIndex const		inColorEntryNumber,
						 RGBColor const*				inColorPtr)
{
	Boolean		result = false;
	
	
    if ((inColorEntryNumber <= 5) && (inColorEntryNumber >= 0))
	{
		GrafPtr				oldPort = null;
		RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
		
		
		GetPort(&oldPort);
		setScreenColor(realScreenPtr, inColorEntryNumber, inColorPtr);
		SetPortWindowPort(realScreenPtr->window);
		MacLook_InvalidatePortRectangle(realScreenPtr->window);
		result = true;
		SetPort(oldPort);
	}
	
	return result;
}// SetColor


/*!
This method hides, shows or blinks the cursor
in the specified connection window.  It may be
invoked repeatedly, and in the case of cursor
blinking, the cursor only changes if the caret
time has elapsed since the last cursor state
change.

(3.0)
*/
void
RealScreen_SetCursorVisibility		(TelnetScreenID const		inScreenID,
									 Boolean const				inIsVisible)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) setCursorVisibility(realScreenPtr, inIsVisible);
}// SetCursorVisibility


/*!
To set whether screen updates should be serviced
for a particular terminal screen, use this method.

(2.6)
*/
void
RealScreen_SetDrawingEnabled	(TelnetScreenID const		inScreenID,
								 Boolean const				inIsDrawingEnabled)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) realScreenPtr->drawingEnabled = inIsDrawingEnabled;
}// SetDrawingEnabled


/*!
Sets the current port to the port of the
indicated terminal window.

Obsolete.

(2.6)
*/
SInt16
RealScreen_SetPortScreenPort	(TelnetScreenID const	inScreenID)
{
	static TelnetScreenID		oldScreenID = -1;
	SInt16						result = 0;
	RealScreenDataPtr			realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		WindowRef		window = realScreenPtr->window;
		
		
		if (window == null) result = -4;
		else
		{
			if (oldScreenID != inScreenID) // is last-used window different?
			{
				oldScreenID = inScreenID;
				realScreenPtr->attributes = -1; // attributes will need setting
				result = 1;
			}
			SetPortWindowPort(window);
		}
	}
	else result = -3;
	
	return result;
}// SetPortScreenPort


/*!
To specify whether a screen uses a special
background (a picture, defined elsewhere) or
a solid background color, use this method.

(3.0)
*/
void
RealScreen_SetSpecialBackground		(TelnetScreenID const		inScreenID,
									 Boolean const				inUsesSpecialBackground)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) realScreenPtr->customTextBackground = inUsesSpecialBackground;
}// SetSpecialBackground


/*!
Arranges the text cursor for the specified window, at
the specified position.  The position information is
in units of characters.  The exact cursor rectangle is
determined by its shape.  MacTelnet 3.0 now supports two
additional cursor shapes.

(3.0)
*/
void
RealScreen_SetUpCursor	(TelnetScreenID const	inScreenID,
						 SInt16 const			inX,
						 SInt16 const			inY)
{
	enum
	{
		// cursor dimensions, in pixels
		kTerminalCursorUnderscoreHeight = 1,
		kTerminalCursorThickUnderscoreHeight = 2,
		kTerminalCursorVerticalLineWidth = 1,
		kTerminalCursorThickVerticalLineWidth = 2
	};
	RealScreenDataPtr		realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		Point		characterSizeInPixels; // based on font metrics
		UInt16		thickness = 0; // used for non-block-shaped cursors
		UInt16		terminalCursorType = kTerminalCursorTypeBlock;
		Size		actualSize = 0L;
		
		
		unless (Preferences_GetData(kTelnetPreferenceClassGeneral, kTelnetPreferenceTag_TerminalCursorType,
										sizeof(terminalCursorType), &terminalCursorType,
										&actualSize) == kTelnetPrefsResultCodeSuccess)
		{
			terminalCursorType = kTerminalCursorTypeBlock; // assume a block-shaped cursor, if preference cant be found
		}
		
		SetPt(&characterSizeInPixels, realScreenPtr->fwidth, realScreenPtr->fheight);
		
		realScreenPtr->cursorRect.left = inX * characterSizeInPixels.h;
		realScreenPtr->cursorRect.top = inY * characterSizeInPixels.v;
		screenToLocal(realScreenPtr, &realScreenPtr->cursorRect.left, &realScreenPtr->cursorRect.top);
		
		switch (terminalCursorType)
		{
			case kTerminalCursorTypeUnderscore:
			case kTerminalCursorTypeThickUnderscore:
				thickness = (terminalCursorType == kTerminalCursorTypeUnderscore)
								? kTerminalCursorUnderscoreHeight
								: kTerminalCursorThickUnderscoreHeight;
				realScreenPtr->cursorRect.top += (characterSizeInPixels.v - thickness);
				realScreenPtr->cursorRect.right = realScreenPtr->cursorRect.left + characterSizeInPixels.h;	
				realScreenPtr->cursorRect.bottom = realScreenPtr->cursorRect.top + thickness;
				break;
			
			case kTerminalCursorTypeVerticalLine:
			case kTerminalCursorTypeThickVerticalLine:
				thickness = (terminalCursorType == kTerminalCursorTypeVerticalLine)
								? kTerminalCursorVerticalLineWidth
								: kTerminalCursorThickVerticalLineWidth;
				realScreenPtr->cursorRect.left += INTEGER_HALVED(characterSizeInPixels.h - thickness);
				realScreenPtr->cursorRect.right = realScreenPtr->cursorRect.left + thickness;
				realScreenPtr->cursorRect.bottom = realScreenPtr->cursorRect.top + characterSizeInPixels.v;
				break;
			
			case kTerminalCursorTypeBlock:
			default:
				realScreenPtr->cursorRect.right = realScreenPtr->cursorRect.left + characterSizeInPixels.h;
				realScreenPtr->cursorRect.bottom = realScreenPtr->cursorRect.top + characterSizeInPixels.v;
				break;
		}
	}
}// SetUpCursor


/*!
This method can be used to rename a session
window.  Its item name in the Window menu is
automatically updated to match.

(3.0)
*/
void
RealScreen_SetWTitle	(TelnetScreenID const		inScreenID,
						 Str255						inName)
{
    if (inName[0])
	{
		WindowRef		window = ScreenManager_GetWindowOf(inScreenID);
		
		
		if (inScreenID > 0) SetMenuTextFromScreen(inName, inScreenID); // the FTP log has the reserved ID of 0
		if (window != null) SetWTitle(window, inName);
	}
}// SetWTitle


/*!
Responds to a mouse-down event in the size box of
a terminal window by allowing the user to change
the dimensions of the window.  The window size is
restricted to a legal range.

Returns 0 if successful, nonzero on error.

(2.6)
*/
short
RealScreen_SizeWindow	(WindowRef			inWindow,
						 Point				inGlobalMousePosition,
						 EventModifiers		UNUSED_ARGUMENT(inModifiers))
{
	if (ScreenManager_IsWindowAScreen(inWindow))
	{
		long				size = 0L;
		TelnetScreenID		screenID = ScreenManager_GetWindowScreenID(inWindow);
		RealScreenDataPtr	realScreenPtr = null;
		
		
		realScreenPtr = ScreenManager_GetRealScreenDataOf(screenID);
		
		if (realScreenPtr != null)
		{
			Rect		constraints;
			
			
			SetRect(&constraints, 250/* arbitrary minimum width */, 200/* arbitrary minimum height */,
					SHRT_MAX/* maximum width */, SHRT_MAX/* maximum height */);
			size = GrowWindow(inWindow, inGlobalMousePosition, &constraints);
			
			if (size != 0L)
			{
				Rect		contentRegionBounds;
				SInt16		x1 = 0,
							x2 = 0,
							y1 = 0,
							y2 = 0;
				
				
				SizeWindow(inWindow, size & 0xFFFF, (size >> 16) & 0xFFFF, false);
				MacLook_GetWindowRegionBounds(inWindow, kWindowContentRgn, &contentRegionBounds);
				
				// determine the number of rows and columns that the users specified rectangle most closely matches
				VSgetrgn(screenID, &x1, &y1, &x2, &y2);
				{
					SInt16		width = 0,
								height = 0;
					
					
					getScreenSizeFromWindowSize(realScreenPtr, contentRegionBounds.right - contentRegionBounds.left,
												contentRegionBounds.bottom - contentRegionBounds.top, &width, &height);
					VSsetrgn(screenID, x1, y1,
							(short)((x1 + width / realScreenPtr->fwidth - 1)),
							(short)((y1 + height / realScreenPtr->fheight - 1)));
				}
				VSgetrgn(screenID, &x1, &y1, &x2, &y2); // get the new region
				
				// resize the screen (and adjust the window boundary slightly, if necessary)
				ScreenManager_ChangeDimensionsOf(screenID, x2 - x1 + 1, y2 - y1 + 1, true/* recordable */);
			}
		}
	}
	
	return (0);
}// SizeWindow


/*!
To rearrange all screen windows so that their
top-left corners form an invisible, diagonal
line, use this method.

(3.0)
*/
void
RealScreen_StackWindows	(void)
{
	RealScreenDataPtr	realScreenPtr = null;
	Point				windowLocation;
	register SInt16		i = 0;
	GrafPtr				oldPort = null;
	
	
	GetPort(&oldPort);
	for (i = 1; i <= Connections_GetCount(); i++)
	{
		realScreenPtr = ScreenManager_GetRealScreenDataOf(i);
		SetPortWindowPort(realScreenPtr->window);
		--((*gTopLeftCorners)[realScreenPtr->positionIndex]); // one less window at this position
		calculateWindowIndexPosition(realScreenPtr, realScreenPtr->positionIndex, &windowLocation);
		MoveWindow(realScreenPtr->window, windowLocation.h, windowLocation.v, false/* activate */);
	}
	SetPort(oldPort);
}// StackWindows


/*!
Tries to set the text font of the current graphics port
to match the requested values as well as possible.  In
particular, this routine may substitute the font used,
based on the size, if the style is bold.

The font size of the current port is set to the current
size, unless 0 is given.

NOTE:	This routine does not change the size or style
		information of the current port, even though
		these attributes are given.  Only the font itself
		is changed.

(2.6)
*/
void
RealScreen_TextFont		(StringPtr				inFontFamilyName,
						 SInt16 const			inFontSize,
						 StyleParameter const	inFontStyle)
{
	if ((inFontSize == 9) && (inFontStyle & bold)) TextFont(75); // special NCSA bold font
	else
	{
		TextFontByName(inFontFamilyName);
	}
	if (inFontSize > 0) TextSize(inFontSize);
}// TextFont


/*!
Responds to a mouse-down event in a session window by
changing the cursor to an I-beam and creating or
extending the text selection of the specified window
based on the users use of the mouse and modifier keys.
The window also scrolls if the user moves the mouse out
of the boundaries of the windows content area.  Make
sure that the is text selected flag has a value of 1
when using this routine -- otherwise, autoscrolling
screws up drawing of the text selection.

The given event record should not reflect a modifier
key state for the Option key; that is assumed to never
be the case, so Option-clicks are ignored by this
method.

In MacTelnet 3.0, this routine allows triple-clicks to
mean select line, and several bugs have been fixed.

(2.6)
*/
void
RealScreen_TrackTextSelection	(TelnetScreenID const	inScreenID,
								 Point const			inLocalMouse,
								 EventRecord*			inoutEventPtr)
{
	static long 		lastClick = 0L,
						lastLastClick = 0L;
	static Point	 	lastClickLoc = { 0, 0 },
						lastLastClickLoc = { 0 , 0 };
	Rect				screenArea;
	Point				curr,
						temp;
	long				clickTime = 0L;
	short				shift = (inoutEventPtr->modifiers & shiftKey);
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	GetControlBounds(realScreenPtr->contentUserPane, &screenArea);
	curr = findVirtualCellFromLocalPoint(inScreenID, inLocalMouse, true/* scroll if necessary */);
	clickTime = TickCount();
	
	if (EqualPt(realScreenPtr->anchor, curr) &&
			((lastClick - lastLastClick) <= GetDblTime()) && ((clickTime - lastClick) <= GetDblTime()) &&
			EqualPt(lastClickLoc, lastLastClickLoc) && EqualPt(curr, lastClickLoc))
	{
		// a triple click - select the entire line (3.0)
		realScreenPtr->anchor = realScreenPtr->last = curr;
		handleMultiClick(inScreenID, 3, inoutEventPtr->modifiers);
		lastLastClick = 0;
		lastLastClickLoc = lastClickLoc;
	}
	else if ((EqualPt(realScreenPtr->anchor, curr) || EqualPt(realScreenPtr->anchor, realScreenPtr->last)) &&
			((clickTime - lastClick) <= GetDblTime()) &&
			EqualPt(curr, lastClickLoc))
	{
		// a double click - select a word
		realScreenPtr->anchor = realScreenPtr->last = curr;
		handleMultiClick(inScreenID, 2, inoutEventPtr->modifiers);
		lastLastClick = lastClick;
		lastLastClickLoc = lastClickLoc;
		lastClick = clickTime;
		lastClickLoc = curr;
	}
	else if (inoutEventPtr->modifiers & cmdKey)
	{
		// a command-click means the selection is to be interpreted as a URL, or (if there
		// is no selection) the text nearest the cursor is to be searched for something
		// that looks like a URL, at which time the text is handled like a URL
		if ((realScreenPtr->isTextSelected) && (RealScreen_PtInSelection(inScreenID, inLocalMouse)))
		{
			URL_HandleForScreenID(inScreenID);
		}
		else
		{
			// find the URL around the click location, if possible
			if (URL_SetSelectionToProximalURL(curr, inScreenID)) URL_HandleForScreenID(inScreenID);
			else BEEP;
		}
	}
	else
	{
		GrafPtr		oldPort = null;
		Point		localMouse;
		Boolean		startNewSelection = true;	// usually, cancel the previous selection first
		
		
		GetPort(&oldPort);
		
		lastClick = clickTime;
		lastClickLoc = curr;
		if (realScreenPtr->isTextSelected)
		{
			unless (shift)
			{
				// unhighlight the current selection and start a new one
				highlightVirtualRange(realScreenPtr, realScreenPtr->anchor, realScreenPtr->last, &screenArea);
			}
			else
			{
				RSsortAnchors(inScreenID);
				if ((curr.v < realScreenPtr->anchor.v) ||
					((curr.v == realScreenPtr->anchor.v) && (curr.h < realScreenPtr->anchor.h)))
				{
					temp = realScreenPtr->anchor;
					realScreenPtr->anchor = realScreenPtr->last;
					realScreenPtr->last = temp;
				}
				startNewSelection = false;	// in this case, DONT cancel the selection; shift-drag means continue
			}
		}
		
		if (startNewSelection)
		{
			// start a new selection, using the modifier key states to decide on a selection mode
			RealScreen_MakeSelectionsRectangular
				(inScreenID, ((inoutEventPtr->modifiers & optionKey) != 0));
			if (inoutEventPtr->modifiers & optionKey)
			{
				inoutEventPtr->modifiers &= (~optionKey);
			}
			realScreenPtr->anchor = realScreenPtr->last = curr;
			realScreenPtr->isTextSelected = 1;
		}
		
		Cursors_UseIBeam();
		SetPortWindowPort(realScreenPtr->window);
		while (StillDown())
		{
			// wait for the mouse position to change
			do
			{
				GetMouse(&localMouse);
				curr = findVirtualCellFromLocalPoint(inScreenID, localMouse, true/* scroll if necessary */);
				Threads_YieldToAnother();
			} while (EqualPt(curr, realScreenPtr->last) && StillDown());
			
			// toggle the highlight state of text between the current and last mouse positions
			if (realScreenPtr->isTextSelectionRectangular)
			{
				highlightVirtualRange(realScreenPtr, realScreenPtr->anchor, realScreenPtr->last, &screenArea);
				realScreenPtr->last = curr;
				highlightVirtualRange(realScreenPtr, realScreenPtr->anchor, realScreenPtr->last, &screenArea);
			}
			else
			{
				highlightVirtualRange(realScreenPtr, curr, realScreenPtr->last, &screenArea);
				realScreenPtr->last = curr;
			}
		}
		
		RSsortAnchors(inScreenID); // 3.0 - the anchors might end up out of order
		SetPort(oldPort);
	}
	
	realScreenPtr->isTextSelected = (!EqualPt(realScreenPtr->anchor, realScreenPtr->last));
	if (realScreenPtr->isTextSelected)
	{
		Boolean		copySelectedText = false;
		Size		actualSize = 0L;
		
		
		unless (Preferences_GetData(kTelnetPreferenceClassGeneral, kTelnetPreferenceTag_CopySelectedText,
										sizeof(copySelectedText), &copySelectedText,
										&actualSize) == kTelnetPrefsResultCodeSuccess)
		{
			copySelectedText = false; // assume text isnt automatically copied, if preference cant be found
		}
		
		if (copySelectedText) Clipboard_TextToScrap(inScreenID, kTelnetCopyMethodTable | kTelnetCopyMethodInline);
	}
}// TrackTextSelection


/*!
Updates the horizontal scroll bar and associated variables
to reflect the current view of the virtual screen within the
specified window.

(2.6)
*/
void
RealScreen_UpdateHorizontalScrollBar	(TelnetScreenID const	inScreenID,
										 SInt16 const			inInvisibleCharacterPositionCount,
										 SInt16 const			inLeftmostVisibleColumn)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		realScreenPtr->leftmarg = inLeftmostVisibleColumn;
		
		if (realScreenPtr->lcurrent != inLeftmostVisibleColumn)
		{
			SetControlValue(realScreenPtr->left, (realScreenPtr->lcurrent = inLeftmostVisibleColumn));
		}
		if (realScreenPtr->lmax != inInvisibleCharacterPositionCount)
		{
			SetControlMaximum(realScreenPtr->left, (realScreenPtr->lmax = inInvisibleCharacterPositionCount));
		}
	}
}// UpdateHorizontalScrollBar


/*!
To make sure the scroll bars and internal variables
are up to date with the current virtual screen view
in the specified window, use this method.

(2.6)
*/
void
RealScreen_UpdateVerticalScrollBar	(TelnetScreenID const	inScreenID,
									 UInt16 const			inScrollbackBufferSize,
									 SInt16 const			inTopmostVisibleLineNumber,
									 SInt16 const			inBottommostVisibleLineNumber)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	SInt16				newMax = 0;
	
	
	newMax = (VSgetlines(inScreenID) - 1) - (inBottommostVisibleLineNumber - inTopmostVisibleLineNumber);

	realScreenPtr->topline = inTopmostVisibleLineNumber;
	if (realScreenPtr->min != -inScrollbackBufferSize)
	{
		SInt16		negativeSize = inScrollbackBufferSize;
		
		
		negativeSize = -negativeSize;
		SetControlMinimum(realScreenPtr->scroll, (realScreenPtr->min = negativeSize));
	}
	
	if (realScreenPtr->current != inTopmostVisibleLineNumber)
	{
		SetControlValue(realScreenPtr->scroll, (realScreenPtr->current = inTopmostVisibleLineNumber));
	}

	if (realScreenPtr->max != newMax) SetControlMaximum(realScreenPtr->scroll, (realScreenPtr->max = newMax));
}// UpdateVerticalScrollBar


/*!
To update the text in the text field of a Set Window Title
dialog box using text from an arbitrary buffer, call this
method.  The first line of text from the buffer is used,
or the whole buffer if no carriage return characters are
found, or the first 255 characters if the buffer is too long.

This is called by the drag-and-drop code to allow text
selections to affect the data in the text field of a
Set Window Title dialog box.

(3.0)
*/
void
RealScreen_UpdateSetWTitleDialogUsing	(unsigned char*		inSourcePtr,
										 Size				inSize,
										 DialogRef const	inDialog)
{
	Str255		firstLine;
	
	
	// update the dialog boxs text item
	StringUtilities_PExtractFirstLine(inSourcePtr, inSize, firstLine);
	if (StrLength(firstLine) <= 0) BEEP; // no text?
	else
	{
		ControlRef		control = null;
		
		
		SetDialogItemControlText(inDialog, iSetWindowTitleTitleField, firstLine);
		(OSErr)GetDialogItemAsControl(inDialog, iSetWindowTitleTitleField, &control);
		Draw1Control(control);
	}
}// UpdateSetWTitleDialogUsing


/*!
Zooms a terminal window, in response to a click
in its zoom box.  You previously call FindWindow()
to obtain the zoom code.

(2.6)
*/
void
RealScreen_ZoomWindow		(WindowRef					inWindow,
							 WindowPartCode const		inWindowPartFromCallToFindWindow,
							 Boolean const				inActivateAfterZooming)
{
  	if (ScreenManager_IsWindowAScreen(inWindow))
	{
		GrafPtr				oldPort = null;
		short				w = 0;
		short				h = 0,
							v = 0;
		short				width = 0,
							lines = 0;
		short				top = 0,
							left = 0;
		Rect				contentRegionBounds;
		RealScreenDataPtr	realScreenPtr = null;
		
		
		GetPort(&oldPort);
		SetPortWindowPort(inWindow);
		MacLook_GetWindowRegionBounds(inWindow, kWindowContentRgn, &contentRegionBounds);
		w = ScreenManager_GetWindowScreenID(inWindow);
		realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
		
		width = VSmaxwidth(w) + 1;
		lines = VSgetlines(w);
		{
			Rect		standardStateRect;
			SInt16		pixelWidth = 0,
						pixelHeight = 0;
			
			
			GetWindowUserState(inWindow, &standardStateRect);
			top = standardStateRect.top;
			left = standardStateRect.left;
			getWindowSizeFromScreenSize(realScreenPtr, (width) * realScreenPtr->fwidth, (lines) * realScreenPtr->fheight,
										&pixelWidth, &pixelHeight);
			SetRect(&standardStateRect, left, top, left + pixelWidth, top + pixelHeight);
			SetWindowStandardState(inWindow, &standardStateRect);
		}
		
		ZoomWindow(inWindow, inWindowPartFromCallToFindWindow, inActivateAfterZooming);
		EraseRect(&contentRegionBounds);
		
		// get the new window size, then resize the window and arrange the scroll bars
		MacLook_GetWindowRegionBounds(inWindow, kWindowContentRgn, &contentRegionBounds);
		h = contentRegionBounds.right - contentRegionBounds.left;
		v = contentRegionBounds.bottom - contentRegionBounds.top;
		RSsetsize(w);

		MacLook_InvalidateRectangle(inWindow, &contentRegionBounds);
		SetPort(oldPort);
	}
}// ZoomWindow



//
// BEGIN OBSOLETE METHODS
//

/*!
To draw a portion of text from a single line, use
this method.  Any text that falls within the current
text selection for the specified terminal screen is
automatically highlighted.  All of the text will
otherwise use the given attributes.

(2.6)
*/
void
RSdraw		(TelnetScreenID const		inScreenID,
			 SInt16 const				inStartingColumnNumber,
			 SInt16 const				inLineNumberToDraw,
			 SInt16 const				inAttributes,
			 SInt16 const				inCharacterCount,
			 char const*				inTextBufferPtr)
{
  	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
    Rect				rect;
	SInt16				ys = 0;
	
	
	if (realScreenPtr->drawingEnabled)
	{
		ys = inLineNumberToDraw * realScreenPtr->fheight;
		
		// set up the rectangle bounding the text being drawn
		SetRect(&rect,
				inStartingColumnNumber * realScreenPtr->fwidth,
				ys,
				(inStartingColumnNumber + inCharacterCount) * realScreenPtr->fwidth,
				ys + realScreenPtr->fheight);
		screenToLocalRect(realScreenPtr, &rect);
		
		setRealScreenAttr(realScreenPtr, inAttributes); // use attributes to set up current graphics port
		//unless (realScreenPtr->customTextBackground)
		//{
		//	EraseRect(&rect); // clear region to background color
		//}
		MoveTo(rect.left, rect.top + realScreenPtr->fascent);
		
		// draw text
		drawText(realScreenPtr, inTextBufferPtr, inCharacterCount);
		updateTextSelection(inScreenID, &rect);
		
		// text is the only content of this part of the window, so it is now valid here
		//MacLook_ValidateRectangle(realScreenPtr->window, &rect);
	}
}// RSdraw


/*!
To delete the specified number of columns of data
from the specified screen, use this method.  The
display scrolls to the left to fill the gap left
by the deleted columns.

(2.6)
*/
void
RSdelcols	(TelnetScreenID const		inScreenID,
			 SInt16 const				inDeleteHowManyColumns)
{
	RealScreenDataPtr		realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
    if (RealScreen_IsDrawingEnabled(inScreenID))
	{
		GrafPtr		oldPort = null;
		Rect		rect;
		
		
		GetPort(&oldPort);
		RealScreen_SetPortScreenPort(inScreenID);
		
		// bounds of entire text area, for scrolling
		SetRect(&rect, 0, 0, realScreenPtr->width, realScreenPtr->height);
		screenToLocalRect(realScreenPtr, &rect);
		
		if (realScreenPtr->customTextBackground)
		{
			// the entire screen has to be redrawn to keep the picture in place
			invalidateScreenRectangle(realScreenPtr);
		}
		else
		{
			ScrollRect(&rect, -inDeleteHowManyColumns * realScreenPtr->fwidth, 0, gScrollingUpdateRegion);
			
			// any necessary redrawing in newly-revealed area will be done by caller
			MacLook_ValidateRectangle(realScreenPtr->window, &rect);
			MacLook_InvalidateRegion(realScreenPtr->window, gScrollingUpdateRegion);
		}
		rect.left -= (inDeleteHowManyColumns * realScreenPtr->fwidth);
		updateTextSelection(inScreenID, &rect);
		
		SetPort(oldPort);
	}
}// RSdelcols


void
RSdelchars
  (
	TelnetScreenID const w, /* affected window */
	short x, /* column to delete from */
	short y, /* line on which to do deletion */
	short n /* number of characters to delete */
  )
  /* deletes the specified number of characters from the specified
	position to the right, moving the remainder of the line to the
	left. */
{
    Rect				rect;
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
	
	
    if (RealScreen_IsDrawingEnabled(w))
	{
		//setRealScreenAttr(realScreenPtr, 0); /* avoid funny pen modes */
		
		if (realScreenPtr->customTextBackground)
		{
			// with a background picture, the entire screen needs updating
			invalidateScreenRectangle(realScreenPtr);
		}
		else
		{
			/* bounds of area from starting column to end of line */
			SetRect(&rect,
					x * realScreenPtr->fwidth,
					y * realScreenPtr->fheight,
					realScreenPtr->width,
					(y + 1) * realScreenPtr->fheight);
			screenToLocalRect(realScreenPtr, &rect);
			
			if ((x + n) * realScreenPtr->fwidth > realScreenPtr->width)
			{
			  /* deleting to end of line */
				MacLook_InvalidateRectangle(realScreenPtr->window, &rect);
			}
			else
			  {
			  /* scroll remainder of line to the left */
				ScrollRect(&rect, - realScreenPtr->fwidth * n, 0, gScrollingUpdateRegion);
				MacLook_InvalidateRegion(realScreenPtr->window, gScrollingUpdateRegion);
				MacLook_ValidateRectangle(realScreenPtr->window, &rect); /* leave newly-revealed area blank */
				updateTextSelection(w, &rect);
			  } /* if */
		}
	}
}// RSdelchars


void RSdellines
  (
	TelnetScreenID const w, /* affected window */
	short t, /* top line of affected region */
	short b, /* bottom line of affected region */
	short n, /* number of lines to delete */
	short scrolled
	  /*
		-ve => cancel current selection, if any;
		+ve => selection has moved up one line;
		0 => don't touch selection
	  */
  )
  /* deletes lines at the top of the specified region of a window,
	inserting new blank lines at the bottom, and scrolling up the
	stuff in between.  In Telnet 3.0, the area is clipped to
	exclude window controls.*/
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
	
	
    if (RealScreen_IsDrawingEnabled(w))
	{
		GrafPtr		oldPort = null;
	    Rect		rect,
					screenArea;
		short		RSfheightTimesn, RSfheightTimesbplus1;
		SInt16		preservedAttributes = realScreenPtr->attributes;
		
		
		GetControlBounds(realScreenPtr->contentUserPane, &screenArea);
		//setRealScreenAttr(realScreenPtr, 0); /* avoid funny pen modes */
		
		if (scrolled)
		  {
			if (realScreenPtr->isTextSelected && (scrolled < 0))
			  {
			  /* unhighlight and cancel current selection */
				highlightVirtualRange(realScreenPtr, *(Point*)&realScreenPtr->anchor, *(Point *) &realScreenPtr->last,
										&screenArea);
				realScreenPtr->isTextSelected = 0;
			  }
			else
			  {
				realScreenPtr->last.v -= 1;		/* Subtract one from each of the */
				realScreenPtr->anchor.v -= 1;	/* vertical Selection components */
			  } /* if */
		  } /* if */
	
		rect.left = -1;							/* necessary */
		rect.right = realScreenPtr->width;
		rect.top = t * realScreenPtr->fheight;
		RSfheightTimesbplus1 = (b + 1) * realScreenPtr->fheight;
		rect.bottom = RSfheightTimesbplus1;
		screenToLocalRect(realScreenPtr, &rect);
	  /* adjust the update region to track the scrolled window contents */
		RSfheightTimesn = realScreenPtr->fheight * n;
		// 3.0 - why are we doing this?  CARBON doesn't support this manipulation anyway
		//OffsetRgn(((WindowPeek) realScreenPtr->window)->updateRgn,
		//		0, -RSfheightTimesn);
		if (realScreenPtr->customTextBackground)
		{
			// with a background picture, the entire screen needs updating
			invalidateScreenRectangle(realScreenPtr);
		}
		else
		{
			ScrollRect(&rect, 0, -RSfheightTimesn, gScrollingUpdateRegion);
			MacLook_InvalidateRegion(realScreenPtr->window, gScrollingUpdateRegion);
		  /* validate the area containing the newly-inserted blank lines. */
		  /* any necessary redrawing in newly-revealed area will be done by caller */
			SetRect(&rect, 0, (b - n + 1) * realScreenPtr->fheight - 1,
					realScreenPtr->width, RSfheightTimesbplus1 + 1);
			screenToLocalRect(realScreenPtr, &rect);
			MacLook_ValidateRectangle(realScreenPtr->window, &rect);
		}
		
		// restore mode for text drawing
		//setRealScreenAttr(realScreenPtr, preservedAttributes/*VSIw->attrib <- 3.0 - arent these totally different bits?!?!? */);
	}
  } /* RSdellines */


void RSinslines
  (
	TelnetScreenID const w, /* affected window */
	short t, /* where to insert blank lines */
	short b, /* bottom of area to scroll */
	short n, /* number of lines to insert */
	short scrolled /* -ve <=> cancel current selection, if any */
  )
  /* inserts blank lines at the top of the given area of the display,
	scrolling the rest of it down. */
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
	
	
    if (RealScreen_IsDrawingEnabled(w))
	{
		Rect		rect;
		
		
		//setRealScreenAttr(realScreenPtr, 0); /* avoid funny pen modes */
		if (realScreenPtr->isTextSelected && (scrolled < 0))
		{
			// unhighlight and cancel selection
			GetControlBounds(realScreenPtr->contentUserPane, &rect);
			highlightVirtualRange(realScreenPtr, *(Point*)&realScreenPtr->anchor, *(Point*)&realScreenPtr->last, &rect);
			realScreenPtr->isTextSelected = 0;
		}
		rect.left = -1;						/* necessary */
		rect.right = realScreenPtr->width;
		rect.top = t * realScreenPtr->fheight;
		rect.bottom = (b + 1) * realScreenPtr->fheight;
		screenToLocalRect(realScreenPtr, &rect);
	  /* adjust the update region to track the scrolled window contents */
	  	// 3.0 - why are we doing this?  CARBON doesn't support this manipulation anyway
		//OffsetRgn(((WindowPeek) realScreenPtr->window)->updateRgn,
		//	0, realScreenPtr->fheight * n);
		if (realScreenPtr->customTextBackground)
		{
			// with a background picture, the entire screen needs updating
			invalidateScreenRectangle(realScreenPtr);
		}
		else
		{
			ScrollRect(&rect, 0, realScreenPtr->fheight * n, gScrollingUpdateRegion);
			MacLook_InvalidateRegion(realScreenPtr->window, gScrollingUpdateRegion);
	  		/* newly-inserted area is already blank -- validate it to avoid redrawing. */
	  		/* any necessary redrawing will be done by caller */
			SetRect(&rect,
					0,
					t * realScreenPtr->fheight - 1,
					realScreenPtr->width,
					(t + n) * realScreenPtr->fheight + 1);
			screenToLocalRect(realScreenPtr, &rect);
			MacLook_ValidateRectangle(realScreenPtr->window, &rect);
		}
	}
}// RSinslines


void RSinscols
  (
	TelnetScreenID const w,
	short n /* number of columns to insert */
  )
  /* inserts blank columns at the left side of the text display in
	the specified window, scrolling its current contents to the right.
	Maintains the selection highlight, but doesn't move the selection.
	Doesn't even unhighlight text which moves out of the selection area. */
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
	
	
    if (RealScreen_IsDrawingEnabled(w))
	{
		GrafPtr		oldPort = null;
		Rect		rect;
		
		
		GetPort(&oldPort);
		RealScreen_SetPortScreenPort(w);
		SetRect(&rect, 0, 0, realScreenPtr->width, realScreenPtr->height);
		screenToLocalRect(realScreenPtr, &rect);
		
		if (realScreenPtr->customTextBackground)
		{
			invalidateScreenRectangle(realScreenPtr);
		}
		else
		{
			ScrollRect(&rect, n * realScreenPtr->fwidth, 0, gScrollingUpdateRegion);
			MacLook_InvalidateRegion(realScreenPtr->window, gScrollingUpdateRegion);
			MacLook_ValidateRectangle(realScreenPtr->window, &rect); // necessary redrawing will be done by the caller
		}
		
		// newly-inserted blank area
		SetRect(&rect, 0, 0, (n + 1) * realScreenPtr->fwidth - 1, realScreenPtr->height);
		screenToLocalRect(realScreenPtr, &rect);
		updateTextSelection(w, &rect);
		
		SetPort(oldPort);
	}
}// RSinscols


void RSinsstring
  (
	TelnetScreenID const w, /* affected window */
	short x, /* starting column at which to insert */
	short y, /* line on which to insert */
	short a, /* attributes for inserted text */
	short len, /* length of inserted text */
	char* ptr /* pointer to inserted text */
  )
  /* inserts a string of characters at the specified position, scrolling
	the rest of the line to the right. Highlights any part of the newly-
	inserted text lying within the current selection. */
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
	
	
	if (RealScreen_IsDrawingEnabled(w))
	{
		Rect		rect;
		
		
		// this boundary is from the specified position to the end of the same line
		SetRect(&rect,
				x * realScreenPtr->fwidth,
				y * realScreenPtr->fheight,
				realScreenPtr->width,
				(y + 1) * realScreenPtr->fheight);
		screenToLocalRect(realScreenPtr, &rect);
		
		if (realScreenPtr->customTextBackground)
		{
			// with a background picture, the entire screen needs updating
			invalidateScreenRectangle(realScreenPtr);
		}
		else
		{
			ScrollRect(&rect, len * realScreenPtr->fwidth, 0, gScrollingUpdateRegion); // scroll remainder of the line to the right
			MacLook_InvalidateRegion(realScreenPtr->window, gScrollingUpdateRegion);
			MacLook_ValidateRectangle(realScreenPtr->window, &rect); /* any necessary redrawing in newly-revealed area will be done by caller */
		}
		
		//if (realScreenPtr->attributes != a) setRealScreenAttr(realScreenPtr, a);
		SetRect /* bounds area to contain inserted string */
		(
			&rect,
			x * realScreenPtr->fwidth,
			y * realScreenPtr->fheight,
			(x + len) * realScreenPtr->fwidth,
			(y + 1) * realScreenPtr->fheight
		);
		screenToLocalRect(realScreenPtr, &rect);
		
		//unless (realScreenPtr->customTextBackground)
		//{
			MacLook_InvalidateRectangle(realScreenPtr->window, &rect);
		//	MoveTo(rect.left, rect.top + realScreenPtr->fascent);
		//	drawText(realScreenPtr, ptr, len);
		//}
		updateTextSelection(w, &rect);
	}
}// RSinsstring
	

/*!
To make sure that the start text selection range anchor
is positioned before the end anchor in a real screen,
use this method.  This does not change the selection, but
it ensures that the anchor names refer to the end of the
selection that they imply.

(2.6)
*/
void
RSsortAnchors	(TelnetScreenID const		w)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
	
	
	if (realScreenPtr != null)
	{
		Point		temp;
		
		
		if (realScreenPtr->anchor.v > realScreenPtr->last.v)
		{
			temp = realScreenPtr->anchor;
			realScreenPtr->anchor = realScreenPtr->last;
			realScreenPtr->last = temp;
		}
		
		if ((realScreenPtr->anchor.v == realScreenPtr->last.v) && (realScreenPtr->anchor.h > realScreenPtr->last.h))
		{
			temp = realScreenPtr->anchor;
			realScreenPtr->anchor = realScreenPtr->last;
			realScreenPtr->last = temp;
		}
	}
}// RSsortAnchors


/*!
Saves the new size settings for a window, and repositions
the scroll bars accordingly.  In MacTelnet 3.0, this method
calls SetControlViewSize() (if available) to make
proportional scrolling work, and handles sizing and
arranging of all the new controls associated with the new
connection window headers.

(2.6)
*/
void
RSsetsize	(TelnetScreenID const	inScreenID)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		Rect			backgroundArea,
						screenArea,
						windowArea;
		ControlRef		control = null;
		Point			insetsTopLeft,
						insetsBottomRight;
		
		
		// determine the port rectangle
		MacLook_GetWindowRegionBounds(realScreenPtr->window, kWindowContentRgn, &windowArea);
		RealScreen_GetInsets(&insetsTopLeft, &insetsBottomRight);
		
		// define the screen rectangle relative to a (0, 0) origin, and then translate it, if necessary
		SetRect(&screenArea, 0, 0, 0, 0);
		getScreenSizeFromWindowSize(realScreenPtr, windowArea.right - windowArea.left, windowArea.bottom - windowArea.top,
									&screenArea.right, &screenArea.bottom);
		screenToLocalRect(realScreenPtr, &screenArea);
		backgroundArea.left = screenArea.left - insetsTopLeft.h;
		backgroundArea.top = screenArea.top - insetsTopLeft.v;
		backgroundArea.right = screenArea.right + insetsBottomRight.h;
		backgroundArea.bottom = screenArea.bottom + insetsBottomRight.v;
		
		// move the scroll bars to their new positions and sizes, and redisplay them
		if ((control = realScreenPtr->scroll) != null)
		{
			UInt16		scrollBarTop = backgroundArea.top - 1;
			
			
			if (realScreenPtr->windowHeader != null)
			{
				if (TerminalWindowHeader_IsCollapsed(realScreenPtr->windowHeader))
				{
					Point		collapsedSize;
					
					
					SetPt(&collapsedSize, windowArea.right - windowArea.left, 0);
					TerminalWindowHeader_GetCollapsedSize(realScreenPtr->windowHeader, &collapsedSize);
					scrollBarTop += (collapsedSize.v - 1);
				}
			}
			
			SetControlVisibility(control, false/* visibility */, false/* draw */);
			
			SetControlValue(control, realScreenPtr->current); // because this isnt always set
			SizeControl(control,
						RealScreen_GetScrollBarWidth(),
						windowArea.bottom - windowArea.top + 2/* two-side frame overlap */ -
							RealScreen_GetScrollBarHeight() - scrollBarTop);
			MoveControl(control, windowArea.right - windowArea.left - RealScreen_GetScrollBarWidth() + 1, scrollBarTop);
			if (FlagManager_Test(kFlagCarbonControlManager))
			{
				// support proportional scroll thumbs
				SetControlViewSize(control, (screenArea.bottom - screenArea.top) / realScreenPtr->fheight);
			}
			
			SetControlVisibility(control, true/* visibility */, true/* draw */);
		}
		if ((control = realScreenPtr->left) != null)
		{
			SetControlVisibility(control, false/* visibility */, false/* draw */);
			
			SetControlValue(control, realScreenPtr->lcurrent); // because this isnt always set
			SizeControl(control,
						windowArea.right - windowArea.left + 2/* two-side frame overlap */ - RealScreen_GetScrollBarWidth() + 1,
						RealScreen_GetScrollBarHeight());
			MoveControl(control, backgroundArea.left - 1,
						windowArea.bottom - windowArea.top - RealScreen_GetScrollBarHeight() + 1);
			if (FlagManager_Test(kFlagCarbonControlManager))
			{
				// support proportional scroll thumbs
				SetControlViewSize(control, (screenArea.right - screenArea.left) / realScreenPtr->fwidth);
			}
			
			SetControlVisibility(control, true/* visibility */, true/* draw */);
		}
		
		// For Mac OS 8.1 and earlier, controls are used to display the proper inactive
		// (invisible) appearance of scroll bars.  Thus, their sizes and positions must
		// be the same as the scroll bars they represent.
		//unless (FlagManager_Test(kFlagCarbonControlManager))
		{
			Rect	scrollBarRect;
			
			
			GetControlBounds(realScreenPtr->left, &scrollBarRect);
			MoveControl(realScreenPtr->invisibleScrollBarH, scrollBarRect.left, scrollBarRect.top);
			SizeControl(realScreenPtr->invisibleScrollBarH, scrollBarRect.right - scrollBarRect.left,
						scrollBarRect.bottom - scrollBarRect.top);
			GetControlBounds(realScreenPtr->scroll, &scrollBarRect);
			MoveControl(realScreenPtr->invisibleScrollBarV, scrollBarRect.left, scrollBarRect.top);
			SizeControl(realScreenPtr->invisibleScrollBarV, scrollBarRect.right - scrollBarRect.left,
						scrollBarRect.bottom - scrollBarRect.top);
		}
		
		// resize the user pane encompassing the screen area
		if ((control = realScreenPtr->backgroundUserPane) != null)
		{
			SInt16		x = backgroundArea.left,
						y = backgroundArea.top,
						width = backgroundArea.right - backgroundArea.left,
						height = backgroundArea.bottom - backgroundArea.top;
			
			
			SetControlVisibility(control, false/* visibility */, false/* draw */);
			SizeControl(control, width, height);
			SizeControl(realScreenPtr->contentUserPane, width - insetsBottomRight.h - insetsTopLeft.h,
						height - insetsBottomRight.v - insetsTopLeft.v);
			MoveControl(control, x, y); // also moves the content pane, which is embedded
			SetControlVisibility(control, true/* visibility */, true/* draw */);
		}
		
		// new controls associated with the window header
		if (realScreenPtr->windowHeader != null)
		{
			TerminalWindowHeader_SetWidthInPixels(realScreenPtr->windowHeader,
													windowArea.right - windowArea.left + 2/* 2 == frames width sum */);
		}
	}
}// RSsetsize


void
RScheckmaxwind	(TelnetScreenID const	inScreenID,
				 Rect*					origRect,
				 short					origW, 
				 short					origH,
				 short*					endW,
				 short*					endH)
{
	Rect		grayRect;
	RgnHandle	grayRgn = GetGrayRgn();
	
	
	GetRegionBounds(grayRgn, &grayRect);
	*endW = origW;
	*endH = origH;
	if (origW > (grayRect.right - origRect->left - RealScreen_GetScrollBarWidth())) *endW = grayRect.right - origRect->left;
	if (origH > (grayRect.bottom - origRect->top - RealScreen_GetScrollBarHeight() - RealScreen_GetPlacardHeight(inScreenID) + 1)) *endH = grayRect.bottom - origRect->top;
}// RScheckmaxwind


/*!
To set the font and size for the specified screens text,
use this method.  Pass null if the font name should be
ignored (and therefore not changed).  Pass 0 if the font
size should not be changed.

(3.0)
*/
void
RSchangefont		(TelnetScreenID const		inScreenID,
					 StringPtr					inFontName,
					 long						fsiz)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null)
	{
		Rect			pRect;
		short			x1 = 0,
						x2 = 0,
						y1 = 0,
						y2 = 0,
						width = 0,
						lines = 0;
		short			srw = 0,
						srh = 0;
		short			resizeWidth = 0,
						resizeHeight = 0;
		GrafPtr			oldPort = null;
		
		
		GetPort(&oldPort);
		RealScreen_SetPortScreenPort(inScreenID);
		
		srw = realScreenPtr->rwidth;
		srh = realScreenPtr->rheight;
		
		if (inFontName != null)
		{
			RealScreen_TextFont(inFontName, fsiz, normal);
			PLstrcpy(realScreenPtr->fontName, inFontName);
			createTextEncodingConverter(realScreenPtr);
		}
		
		if (fsiz) realScreenPtr->fsiz = fsiz;
		
		// set the font metrics
		setUpScreenFontMetrics(realScreenPtr);
		
		width = VSmaxwidth(inScreenID) + 1;
		lines = VSgetlines(inScreenID);
		
	 	// resize window to preserve its dimensions in character cell units
		
		VSgetrgn(inScreenID, &x1, &y1, &x2, &y2);	/* get visible region */
		realScreenPtr->rwidth = realScreenPtr->width = (x2 - x1 + 1) * realScreenPtr->fwidth;
		realScreenPtr->rheight = realScreenPtr->height = (y2 - y1 + 1) * realScreenPtr->fheight;
		
		if (realScreenPtr->rwidth > (width * realScreenPtr->fwidth))
		{
			realScreenPtr->rwidth = (width * realScreenPtr->fwidth);
		}
		if (realScreenPtr->rheight > (lines * realScreenPtr->fheight))
		{
			realScreenPtr->rheight = (lines * realScreenPtr->fheight);
		}
		
		{
			Rect		contentRegionBounds;
			SInt16		width = 0,
						height = 0;
			
			
			MacLook_GetWindowRegionBounds(realScreenPtr->window, kWindowContentRgn, &contentRegionBounds);
			getWindowSizeFromScreenSize(realScreenPtr, realScreenPtr->rwidth, realScreenPtr->rheight, &width, &height);
			RScheckmaxwind(inScreenID, &contentRegionBounds, width, height, &resizeWidth, &resizeHeight);
		}
		
		SizeWindow(realScreenPtr->window, resizeWidth, resizeHeight, false/* update */);
		RSsetsize(inScreenID);
		
	#if 0
		// Mac OS 9.1 bug here?
		GetWindowStandardState(realScreenPtr->window, &pRect);
		pRect.right = pRect.left + (width) * realScreenPtr->fwidth + RealScreen_GetScrollBarWidth();
		{
			Rect		screenRect;
			
			
			MacLook_GetWindowDeviceGrayRect(realScreenPtr->window, &screenRect);
			if (pRect.right > screenRect.right) pRect.right = screenRect.right;
		}
		{
			SInt16		height = 0;
			
			
			getWindowSizeFromScreenSize(realScreenPtr, 0/* ignore width */, (lines) * realScreenPtr->fheight,
										null/* ignore width */, &height);
			pRect.bottom = pRect.top + height;
		}
		SetWindowStandardState(realScreenPtr->window, &pRect);
	#endif
		
		VSgetrgn(inScreenID, &x1, &y1, &x2, &y2);
		VSsetrgn(inScreenID, x1, y1,
				(short)(x1 + (realScreenPtr->rwidth) / realScreenPtr->fwidth - 1),
				(short)(y1 + (realScreenPtr->rheight) / realScreenPtr->fheight - 1));
		VSgetrgn(inScreenID, &x1, &y1, &x2, &y2); // get new region
		
		VSredraw(inScreenID, 0, 0, x2 - x1 + 1, y2 - y1 + 1); // redraw newly-revealed area, if any
		MacLook_ValidatePortRectangle(realScreenPtr->window);
		
		SetPort(oldPort);
	}
}// RSchangefont


/*!
To determine the current font ID and size setting
for the specified screen, use this method.  If
successful, 0 is returned; otherwise, -1 is
returned.

(2.6)
*/
SInt16
RSgetfont	(TelnetScreenID const		inScreenID,
			 short*						outFontID,
			 short*						outFontSize)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	short				result = -1;
	
	
	if (realScreenPtr != null)
	{
		GrafPtr				oldPort = null;
		
		
		GetPort(&oldPort);
    	if (RealScreen_SetPortScreenPort(inScreenID) >= 0)
		{
		#if TARGET_API_MAC_OS8
			GetFNum(realScreenPtr->fontName, outFontID);
		#else
			*outFontID = FMGetFontFamilyFromName(realScreenPtr->fontName);
		#endif
			*outFontSize = realScreenPtr->fsiz;
			result = 0;
		}
		SetPort(oldPort);
	}
	return result;
}// RSgetfont
  
  
/*!
This procedure is a quick check to see if a mouse click is in the
content region of a window.

(3.0)
*/
Boolean
ClickInContent		(Point const			inLocalMouseLocation,
					 TelnetScreenID const	inScreenID)
{
	Boolean		result = true;
	Point		normalizedPoint;
	
	
	normalizedPoint = findVirtualCellFromLocalPoint(inScreenID, inLocalMouseLocation, false/* scroll if necessary */);
	if (normalizedPoint.v >= VSgetlines(inScreenID)) result = false;
	
	return result;
}// ClickInContent
  
  
/*!
Flashes the selected text, to indicate that it
has been accepted for a 'GURL' event.

(2.6)
*/
void
FlashSelection	(TelnetScreenID const		inScreenID)
{
	register SInt16		i = 0;
	UInt32				finalTick = 0L;
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	Rect				screenArea;
	
	
	GetControlBounds(realScreenPtr->contentUserPane, &screenArea);
	for (i = 0; i < 2; i++)
	{
		Delay(5, &finalTick);
		highlightVirtualRange(realScreenPtr, realScreenPtr->anchor, realScreenPtr->last, &screenArea);
		Delay(5, &finalTick);
		highlightVirtualRange(realScreenPtr, realScreenPtr->anchor, realScreenPtr->last, &screenArea);
	}
}// FlashSelection


//
// END OBSOLETE METHODS
//


//
// internal methods
//

/*!
This method moves the buttons and resizes the text field
in response to a resize of the Set Window Title movable
modal dialog box.

(3.0)
*/
static pascal Boolean
adjustSetWindowTitleDialog		(WindowRef const		inWindow,
								 SInt32 const			inDeltaSizeX,
								 SInt32 const			inDeltaSizeY,
								 SInt32 const			UNUSED_ARGUMENT(inData))
{
	Boolean		result = pascal_true;
	
	
	DialogAdjust_BeginDialogItemAdjustment(GetDialogFromWindow(inWindow));
	{
		if (inDeltaSizeX)
		{
			SInt32		amountH = 0L;
			
			
			if (Localization_IsLeftToRight())
			{
				// controls which are moved horizontally
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iSetWindowTitleSetTitleButton, inDeltaSizeX);
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iSetWindowTitleCancelButton, inDeltaSizeX);
				amountH = INTEGER_QUARTERED(inDeltaSizeX);
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iSetWindowTitleTitleField, amountH);
				
				// controls which are resized horizontally
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iSetWindowTitleFieldLabelText, amountH);
				amountH *= 3;
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iSetWindowTitleTitleField, amountH);
			}
			else
			{
				// controls which are moved horizontally
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iSetWindowTitleAppleGuideButton, inDeltaSizeX);
				amountH = 3 * INTEGER_QUARTERED(inDeltaSizeX);
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iSetWindowTitleFieldLabelText, amountH);
				
				// controls which are resized horizontally
				amountH = INTEGER_QUARTERED(inDeltaSizeX);
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iSetWindowTitleFieldLabelText, amountH);
				amountH *= 3;
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iSetWindowTitleTitleField, amountH);
			}
		}
	}
	DialogAdjust_EndAdjustment(inDeltaSizeX, inDeltaSizeY); // moves and resizes controls properly
	
	return true;
}// adjustSetWindowTitleDialog


/*!
When a window is staggered, it is given
an onscreen index indicating what
stagger position (relative to the first
windows location) it occupies.  To
determine the global coordinates that
the top-left corner of the specified
window would occupy if it were in the
specified position, use this method.

This routine now supports multiple
monitors, although not as well as it
should; ideally, there would be a list
of stagger positions for each device,
instead of one list that crosses all
devices.  I believe the current
implementation will cause positions
to be skipped when adjacent windows
in the list occupy different devices.

(3.0)
*/
static void
calculateWindowIndexPosition	(RealScreenDataPtr		inRealScreenPtr,
								 SInt16 const			inStaggerIndex,
								 Point*					outPositionPtr)
{
	if ((inRealScreenPtr != null) && (outPositionPtr != null))
	{
		// calculate the stagger offset
		Point		stagger;
		Rect		screenRect;
		
		
		SetPt(&stagger, 20, 20); // arbitrary, only used if the proper calculation fails
		{
			Rect		structureRect,
						contentRect;
			
			
			MacLook_GetWindowRegionBounds(inRealScreenPtr->window, kWindowStructureRgn, &structureRect);
			MacLook_GetWindowRegionBounds(inRealScreenPtr->window, kWindowContentRgn, &contentRect);
			stagger.h = contentRect.left - structureRect.left + 5/* arbitrary */;
			stagger.v = contentRect.top - structureRect.top + 5/* arbitrary */;
		}
		
		MacLook_GetWindowDeviceGrayRect(inRealScreenPtr->window, &screenRect);
		outPositionPtr->v = screenRect.top + 40/* arbitrary */ + stagger.v * (1 + inStaggerIndex);
		outPositionPtr->h = screenRect.left + 10/* arbitrary */ + stagger.h * (1 + inStaggerIndex);
	}
}// calculateWindowIndexPosition


/*!
Calculates the position of windows.  In
Telnet 3.0, there is no user preference
for staggering windows: windows are
ALWAYS staggered upon creation, and the
amount to stagger by changes depending
on the current Appearance theme.

(2.6)
*/
static void
calculateWindowPosition	(RealScreenDataPtr		inRealScreenPtr,
						 Rect*					outArrangement,
						 short					inRowsHigh,
						 short					inColumnsWide)
{
	short			currentCount = 0,
					wideCount = 0;
	Boolean			done = false,
					tooFarRight = false,
					tooFarDown = false,
					tooBig = false;
	
	
	inRealScreenPtr->positionIndex = 0;
	while ((!done) && (!tooBig))
	{
		while (((*gTopLeftCorners)[inRealScreenPtr->positionIndex] > currentCount) && // find an empty spot
					(inRealScreenPtr->positionIndex < kMaximumAllowedTelnetConnections - 1))
		{
			inRealScreenPtr->positionIndex++;
		}
		
		{
			Point		topLeft;
			
			
			calculateWindowIndexPosition(inRealScreenPtr, inRealScreenPtr->positionIndex, &topLeft);
			outArrangement->top = topLeft.v;
			outArrangement->left = topLeft.h;
		}
		if (tooFarRight) wideCount += currentCount - 1;
		outArrangement->bottom = 30000/* arbitrary */ + outArrangement->top;
		outArrangement->right = 30000/* arbitrary */ + outArrangement->left;
		{
			Rect		screenRect;
			
			
			MacLook_GetWindowDeviceGrayRect(inRealScreenPtr->window, &screenRect);
			tooFarRight = ((outArrangement->left + (inColumnsWide + 1) * inRealScreenPtr->fwidth +
							RealScreen_GetScrollBarWidth()) > screenRect.right);
			tooFarDown = ((outArrangement->top + (inRowsHigh + 1) * inRealScreenPtr->fheight +
							RealScreen_GetScrollBarHeight()) > screenRect.bottom);
		}
		if (tooFarRight || tooFarDown)
		{
			// then the position is off the screen
			if (inRealScreenPtr->positionIndex == 0) tooBig = true; // the window is bigger than the screen size - quit now
			else
			{
				currentCount++; // go through again, pick spot with the least number already at it
				inRealScreenPtr->positionIndex = 0;
			}
		}
		else done = true;
	}	
	unless (tooBig) ((*gTopLeftCorners)[inRealScreenPtr->positionIndex])++; // add the window to the number at this spot
}// calculateWindowPosition


/*!
Creates a text encoding converter for translating text
between Roman and the current system encoding.  The
terminal screen font is used to determine an appropriate
display encoding.

(3.0)
*/
static void
createTextEncodingConverter		(RealScreenDataPtr		inRealScreenPtr)
{
	TextEncoding		displayEncoding = kTextEncodingMacRoman;
	OSStatus			error = noErr;
	
	
	// dispose of any old converter
	if (inRealScreenPtr->textEncodingConverter != null)
	{
		(OSStatus)TECDisposeConverter(inRealScreenPtr->textEncodingConverter);
		inRealScreenPtr->textEncodingConverter = null;
	}
	
	// create a new converter based on the screen font and system script
	error = Localization_GetFontTextEncoding(inRealScreenPtr->fontName, &displayEncoding);
	if (error == noErr)
	{
		Console_WriteValue("derived encoding (from terminal font)", displayEncoding);
		error = TECCreateConverter(&inRealScreenPtr->textEncodingConverter, kTextEncodingMacRoman, displayEncoding);
	}
	if (error != noErr)
	{
		// error, either while determining the encoding or while creating the converter
		inRealScreenPtr->textEncodingConverter = null;
		Console_WriteValue("could not make a text encoding converter, error", error);
	}
}// createTextEncodingConverter


/*!
Returns "true" only if the specified screens cursor
should be blinking.  Currently, this preference is
global, so either all screen cursors blink or all
dont; however, for future flexibility, this routine
takes a specific screen as a parameter.

(3.0)
*/
static Boolean
cursorBlinks	(RealScreenDataPtr		UNUSED_ARGUMENT(inRealScreenPtr))
{
	Size		actualSize = 0L;
	Boolean		result = false;
	
	
	unless (Preferences_GetData(kTelnetPreferenceClassGeneral,
								kTelnetPreferenceTag_CursorBlinks, sizeof(result), &result,
								&actualSize) == kTelnetPrefsResultCodeSuccess)
	{
		result = false; // assume default, if preference cant be found
	}
	return result;
}// cursorBlinks


/*!
Casts a shadow across the top of the terminal screen
area, giving the impression that the window header is
shadowing the terminal screen.

(3.0)
*/
static void
drawShadow		(RealScreenDataPtr		inRealScreenPtr)
{
	// SHADOW DRAWING SEEMS TO BE POTENTIALLY UNSTABLE - WHY THE HELL WOULD THAT BE?!?!?
#if 0
	register SInt16		i = 0;
	Boolean				darkOnLight = true; // dark foreground on light background?
	Rect				controlBounds;
	
	
	GetControlBounds(inRealScreenPtr->contentUserPane, &controlBounds);
	{
		RGBColor	foreColor,
					backColor;
		SInt32		sum = 0L;
		
		
		// do some kind of a weird test to see how the shadow should be drawn
		// (Im no expert, Im only guessing that this is a decent test to
		// see if the foreground color is brighter than the background color)
		GetForeColor(&foreColor);
		GetBackColor(&backColor);
		sum = (foreColor.red - backColor.red + foreColor.green - backColor.green + foreColor.blue - backColor.blue);
		if (sum < 0) darkOnLight = false;
	}
	PenMode(darkOnLight ? adMin : addMax);
	for (i = 0; i < SHADOW_RGBCOLOR_INTENSITY_DIVISOR; i++)
	{
		RGBForeColor(&gShadowGrays[i]);
		MoveTo(controlBounds.left, controlBounds.top + i);
		LineTo(controlBounds.right, controlBounds.top + i);
	}
#endif
}// drawShadow


/*!
Draws the specified text using the font, size, style
and color settings of the current graphics port.
If ANSI graphics are enabled for the specified screen,
then the text is automatically analyzed for candidate
ANSI graphics, and those characters are rendered as
graphics with no style in the ANSI graphics font, and
all other characters are rendered normally.

(3.0)
*/
static void
drawText	(RealScreenDataPtr		inRealScreenPtr,
			 char const*			inTextBufferPtr,
			 SInt32 const			inCharacterCount)
{
	if (inRealScreenPtr->areANSIColorsEnabled)
	{
		// draw all of the text, but scan for sub-sections that could be ANSI graphics
		char const*		textPtr = inTextBufferPtr;
		char const*		styleRunTextPtr = null;
		SInt32			characterCount = inCharacterCount,
						styleRunCharacterCount = 0;
		SInt16			terminalFontID = 0;
		Style			terminalFontStyle = normal;
		
		
		// get current font (used to draw non-ANSI-graphics text)
		{
			GrafPtr		port = null;
			
			
			GetPort(&port);
			terminalFontID = GetPortTextFont(port);
			terminalFontStyle = GetPortTextFace(port);
		}
		
		// iterate over characters, looking for style runs
		while (characterCount > 0)
		{
			styleRunTextPtr = textPtr;
			styleRunCharacterCount = 0;
			while (ASCII_IS_ANSI_GRAPHIC(*styleRunTextPtr) && (characterCount > 0))
			{
				styleRunTextPtr++;
				styleRunCharacterCount++, characterCount--;
			}
			if (styleRunCharacterCount)
			{
				TextFont(76); // use ANSI graphics font (and the text to be drawn is already known)
				TextFace(normal);
			}
			else
			{
				// use the terminal font - and find out what text needs to be drawn
				TextFont(terminalFontID);
				TextFace(terminalFontStyle);
				while (ASCII_IS_NOT_ANSI_GRAPHIC(*styleRunTextPtr) && (characterCount > 0))
				{
					styleRunTextPtr++;
					styleRunCharacterCount++, characterCount--;
				}
			}
			
			DrawText(textPtr, 0/* offset */, styleRunCharacterCount); // draw text using current font, size, color, etc.
			textPtr += styleRunCharacterCount; // skip characters that were just drawn
		}
	}
	else
	{
		// no ANSI graphics expected; save time and just draw the text
		DrawText(inTextBufferPtr, 0, inCharacterCount); // draw text using current font, size, color, etc.
	}
}// drawText


/*!
Finds the cell position in the virtual screen of the
indicated window that is closest to the given point.

If the given point is outside the screen, and if
"inAutomaticallyScrollToPoint" is "true", then the
screen will be automatically scrolled by one cell
in a direction away from the edge(s) of the screen
that the point is nearest to, effectively bringing
the users view slightly closer to the cell that
really is under the mouse position.

(2.6)
*/
static Point
findVirtualCellFromLocalPoint	(TelnetScreenID const		inScreenID,
								 Point						inLocalPixelPosition,
								 Boolean					inAutomaticallyScrollToPoint)
{
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	Point				result = inLocalPixelPosition;
	SInt32				time = TickCount();
	
	
	if (realScreenPtr != null)
	{
		// 3.0 - adjust point to fit in local screen area
		localToScreen(realScreenPtr, &result.h, &result.v);
		if (result.v < 0)
		{
			result.v = 0;
			if (inAutomaticallyScrollToPoint) Terminal_ScrollRowsTowardBottomEdge(inScreenID, 1);
		}
		if (result.v > realScreenPtr->height)
		{
			result.v = realScreenPtr->height;
			if (inAutomaticallyScrollToPoint) Terminal_ScrollRowsTowardTopEdge(inScreenID, 1);
		}
		
		result.v /= realScreenPtr->fheight;
		
		if (result.h < 0)
		{
			result.h = -1;
			if (inAutomaticallyScrollToPoint) Terminal_ScrollColumnsTowardRightEdge(inScreenID, 1);
		}
		
		if (result.h > realScreenPtr->width)
		{
			result.h = realScreenPtr->width;
			if (inAutomaticallyScrollToPoint) Terminal_ScrollColumnsTowardLeftEdge(inScreenID, 1);
		}
		
		result.h = (result.h + INTEGER_HALVED(realScreenPtr->fwidth)) / realScreenPtr->fwidth - 1;
		
		result.h += realScreenPtr->leftmarg;
		result.v += realScreenPtr->topline;
	}
	
	return result;
}// findVirtualCellFromLocalPoint


/*!
Returns the current height of the placard control
for the specified terminal window.  The placard height
is defined as the number of pixels between the top edge
of the window and the top edge of the terminal screen.

(3.0)
*/
UInt16
getPlacardHeight	(RealScreenDataPtr		inRealScreenPtr)
{
	UInt16		result = 0;
	
	
	if (inRealScreenPtr->windowHeader != null)
	{
		Point		size;
		
		
		if (TerminalWindowHeader_IsCollapsed(inRealScreenPtr->windowHeader)) size.h = size.v = 0;
		else TerminalWindowHeader_GetNormalSize(inRealScreenPtr->windowHeader, &size);
		result = size.v;
	}
	return result;
}// getPlacardHeight


/*!
Returns the boundaries of a particular region of
the indicated screen window.  If the indicated
region is not rectangular, this method returns
the tightest bounding rectangle of the region.

(3.0)
*/
void
getRegionBounds		(RealScreenDataPtr			inRealScreenPtr,
					 RealScreenRegionCode const	inRegionCode,
					 Rect*						outBounds)
{
	if (outBounds != null)
	{
		switch (inRegionCode)
		{
			case kRealScreenRegionCodeScreenArea:
				GetControlBounds(inRealScreenPtr->contentUserPane, outBounds);
				break;
			
			case kRealScreenRegionCodeScreenBackground:
				GetControlBounds(inRealScreenPtr->backgroundUserPane, outBounds);
				break;
			
			default:
				break;
		}
	}
}// getRegionBounds


/*!
Returns a color from a screen windows color table,
given an external color specifier.

(3.0)
*/
static void
getScreenColor		(RealScreenDataPtr				inRealScreenPtr,
					 RealScreenColorIndex const		inColorEntryNumber,
					 RGBColor*						outColorPtr)
{
	GetEntryColor(inRealScreenPtr->paletteTerminal, inColorEntryNumber, outColorPtr);
}// getScreenColor


/*!
Returns the entry indices, into the specified screens
window palette, of the correct foreground and background
colors for the specified text style attributes.

(3.0)
*/
static void
getScreenColorEntries		(RealScreenDataPtr		inRealScreenPtr,
							 SInt16 const			inAttributes,
							 SInt16*				outForeColorEntry,
							 SInt16*				outBackColorEntry)
{
	if (inRealScreenPtr != null)
	{
		short		fg = 0,
					bg = 0;
		
		
		// choose foreground color...
		if ((inRealScreenPtr->areANSIColorsEnabled) && STYLE_USE_ANSI_FOREGROUND(inAttributes))
		{
			fg = kRealScreenColorCountNonANSIColors + ((inAttributes >> 8) & 0x07); // ANSI color
			if (STYLE_BOLD(inAttributes)) fg += kRealScreenColorCountNormalANSIColors; // locate emphasized color in palette
		}
		else
		{
			if (STYLE_BOLD(inAttributes)) fg = 4;
			else if (STYLE_BLINKING(inAttributes)) fg = 2;
			else fg = 0; // normal
		}
		
		// choose background color...
		if ((inRealScreenPtr->areANSIColorsEnabled) && STYLE_USE_ANSI_BACKGROUND(inAttributes))
		{
			bg = kRealScreenColorCountNonANSIColors + ((inAttributes >> 12) & 0x07); // ANSI color
			if (STYLE_BOLD(inAttributes)) bg += kRealScreenColorCountNormalANSIColors; // locate emphasized color in palette
		}
		else
		{
			if (STYLE_BOLD(inAttributes)) bg = 5;
			else if (STYLE_BLINKING(inAttributes)) bg = 3;
			else bg = 1; // normal
		}
		
		if (outForeColorEntry != null) *outForeColorEntry = fg;
		if (outBackColorEntry != null) *outBackColorEntry = bg;
	}
}// getScreenColorEntries


/*!
Returns the origin, in local coordinates, of the
screen interior (i.e. taking into account any insets).
Since the user has control over collapsing the window
header, etc. you need to call this routine to get the
current origin.

(3.0)
*/
static void
getScreenOrigin		(RealScreenDataPtr		inRealScreenPtr,
					 SInt16*				outScreenPositionX,
					 SInt16*				outScreenPositionY)
{
	Point		insetsTopLeft,
				insetsBottomRight;
	
	
	RealScreen_GetInsets(&insetsTopLeft, &insetsBottomRight);
	if (outScreenPositionX != null) *outScreenPositionX = insetsTopLeft.h;
	if (outScreenPositionY != null) *outScreenPositionY = getPlacardHeight(inRealScreenPtr) + insetsTopLeft.v;
}// getScreenOrigin


/*!
Returns the width and height of the screen interior
(i.e. not including insets) of a terminal window
whose content region has the specified dimensions.
The resultant dimensions subtract out the size of any
window header (unless its collapsed), the scroll
bars, and the terminal screen insets (padding).

See also getWindowSizeFromScreenSize(), which does
the reverse.

(3.0)
*/
static void
getScreenSizeFromWindowSize		(RealScreenDataPtr				inRealScreenPtr,
								 SInt16 const					inWindowContentWidth,
								 SInt16 const					inWindowContentHeight,
								 SInt16*						outScreenInteriorWidth,
								 SInt16*						outScreenInteriorHeight)
{
	Point		insetsTopLeft,
				insetsBottomRight;
	
	
	RealScreen_GetInsets(&insetsTopLeft, &insetsBottomRight);
	if (outScreenInteriorWidth != null)
	{
		*outScreenInteriorWidth = inWindowContentWidth - insetsTopLeft.h - insetsBottomRight.h -
			RealScreen_GetScrollBarWidth() + 1;
	}
	if (outScreenInteriorHeight != null)
	{
		*outScreenInteriorHeight = inWindowContentHeight - insetsTopLeft.v - insetsBottomRight.v -
			getPlacardHeight(inRealScreenPtr) - RealScreen_GetScrollBarHeight() + 1;
	}
}// getScreenSizeFromWindowSize


/*!
Returns a region describing the highlighted
text in the specified window.  You must
destroy this region yourself!

(2.6)
*/
static RgnHandle
getSelectedTextAsNewRegion		(RealScreenDataPtr		inRealScreenPtr)
{
	RgnHandle		result = null;
	Boolean			errorOccurred = false;
	RgnHandle		tempRgn = null;
	
	
	result = Memory_NewRegion();
	tempRgn = Memory_NewRegion();
	
	if ((result != null) && (tempRgn != null))
	{
		GrafPtr		oldPort = null;
		Rect		temp,
					temp2,
					screenArea;
		Point		lb,
					ub;
		Point		curr;
		Point		last;
		
		
		GetPort(&oldPort);
		SetPortWindowPort(inRealScreenPtr->window);
		GetControlBounds(inRealScreenPtr->contentUserPane, &screenArea);
		
		curr = inRealScreenPtr->anchor;
		last = inRealScreenPtr->last;
		
		// normalize coordinates with respect to visible area of virtual screen
		curr.v -= inRealScreenPtr->topline;
		curr.h -= inRealScreenPtr->leftmarg;
		last.v -= inRealScreenPtr->topline;
		last.h -= inRealScreenPtr->leftmarg;
		
		if ((curr.v == last.v) || (inRealScreenPtr->isTextSelectionRectangular))
		{
			// single highlighted line - start by getting the end points the right way around
			if (curr.h < last.h) ub = curr, lb = last;
			else ub = last, lb = curr;
			
			// set up rectangle bounding area to be highlighted
			SetRect(&temp,
					(ub.h + 1) * inRealScreenPtr->fwidth,
					ub.v * inRealScreenPtr->fheight,
					(lb.h + 1) * inRealScreenPtr->fwidth,
					(lb.v + 1) * inRealScreenPtr->fheight);
			screenToLocalRect(inRealScreenPtr, &temp);
			SectRect(&temp, &screenArea, &temp2); // clip to constraint rectangle
			RectRgn(result, &temp2);
		}
		else
		{
			// highlighting across more than one line - start by getting the end points the right way around
			if (curr.v < last.v) ub = curr;
			else ub = last;
			if (curr.v > last.v) lb = curr;
			else lb = last;
			
			// bounds of first (possibly partial) line to be highlighted
			SetRect(&temp,
					(ub.h + 1) * inRealScreenPtr->fwidth,
					ub.v * inRealScreenPtr->fheight,
					inRealScreenPtr->width,
					(ub.v + 1) * inRealScreenPtr->fheight);
			screenToLocalRect(inRealScreenPtr, &temp);
			SectRect(&temp, &screenArea, &temp2); // clip to constraint rectangle
			RectRgn(result, &temp2);
			
			// bounds of last (possibly partial) line to be highlighted
			SetRect(&temp,
					0,
					lb.v * inRealScreenPtr->fheight,
					(lb.h + 1) * inRealScreenPtr->fwidth,
					(lb.v + 1) * inRealScreenPtr->fheight);
			screenToLocalRect(inRealScreenPtr, &temp);
			SectRect(&temp, &screenArea, &temp2); // clip to constraint rectangle
			RectRgn(tempRgn, &temp2);
			UnionRgn(tempRgn, result, result);
			
			if (lb.v - ub.v > 1)
			{
				// highlight extends across more than two lines - fill in the space in between
				SetRect(&temp,
						0,
						(ub.v + 1) * inRealScreenPtr->fheight,
						inRealScreenPtr->width,
						lb.v * inRealScreenPtr->fheight);
				screenToLocalRect(inRealScreenPtr, &temp);
				SectRect(&temp, &screenArea, &temp2); // clip to constraint rectangle
				RectRgn(tempRgn, &temp2);
				UnionRgn(tempRgn, result, result);
			}
		}
		
		SetPort(oldPort);
	}
	else errorOccurred = true;
	
	if (tempRgn != null) Memory_DisposeRegion(&tempRgn);
	if (errorOccurred) if (result != null) Memory_DisposeRegion(&result);
	
	return result;
}// getSelectedTextAsNewRegion


/*!
Returns the character positions of the smallest boundary
that completely encompasses the visible region of the
specified screen.

(3.0)
*/
static void
getVirtualVisibleRegion		(RealScreenDataPtr		inRealScreenPtr,
							 SInt16*				outLeftColumn,
							 SInt16*				outTopRow,
							 SInt16*				outRightColumn,
							 SInt16*				outBottomRow)
{
	RgnHandle		visibleRegion = Memory_NewRegion();
	
	
	if (visibleRegion != null)
	{
		GrafPtr		port = null;
		Rect		regionBounds;
		
		
		// find the boundaries of the text area needing updating, in terms of row-column anchors
		GetPort(&port);
		GetPortVisibleRegion(GetWindowPort(inRealScreenPtr->window), visibleRegion);
		GetRegionBounds(visibleRegion, &regionBounds);
		if (outLeftColumn != null)
		{
			*outLeftColumn = (regionBounds.left) / inRealScreenPtr->fwidth;
			if (*outLeftColumn < 0) *outLeftColumn = 0;
		}
		if (outTopRow != null)
		{
			*outTopRow = (regionBounds.top) / inRealScreenPtr->fheight;
			if (*outTopRow < 0) *outTopRow = 0;
		}
		if (outRightColumn != null)
		{
			*outRightColumn = ((regionBounds.right) + inRealScreenPtr->fwidth - 1) / inRealScreenPtr->fwidth;
			if (*outRightColumn < 0) *outRightColumn = 0;
		}
		if (outBottomRow != null)
		{
			*outBottomRow = ((regionBounds.bottom) + inRealScreenPtr->fheight - 1) / inRealScreenPtr->fheight;
			if (*outBottomRow < 0) *outBottomRow = 0;
		}
		Memory_DisposeRegion(&visibleRegion);
	}
}// getVirtualVisibleRegion


/*!
Returns the width and height of the content region
of a terminal window whose screen interior (i.e.
not including insets) has the specified dimensions.
The resultant dimensions add in the size of any
window header (unless its collapsed), the scroll
bars, and the terminal screen insets (padding).

See also getScreenSizeFromWindowSize(), which does
the reverse.

(3.0)
*/
static void
getWindowSizeFromScreenSize		(RealScreenDataPtr			inRealScreenPtr,
								 SInt16 const				inScreenInteriorWidth,
								 SInt16 const				inScreenInteriorHeight,
								 SInt16*					outWindowContentWidth,
								 SInt16*					outWindowContentHeight)
{
	Point		insetsTopLeft,
				insetsBottomRight;
	
	
	RealScreen_GetInsets(&insetsTopLeft, &insetsBottomRight);
	if (outWindowContentWidth != null)
	{
		*outWindowContentWidth = inScreenInteriorWidth + insetsTopLeft.h + insetsBottomRight.h +
			RealScreen_GetScrollBarWidth() - 1;
	}
	if (outWindowContentHeight != null)
	{
		*outWindowContentHeight = inScreenInteriorHeight + insetsTopLeft.v + insetsBottomRight.v +
			getPlacardHeight(inRealScreenPtr) + RealScreen_GetScrollBarHeight() - 1;
	}
}// getWindowSizeFromScreenSize


/*!
To create a selection appropriate for the
specified click count, use this method.
Only call this method if at least a double-
click ("inClickCount" == 2) has occurred.

(3.0)
*/
static void
handleMultiClick	(TelnetScreenID const	w,
					 UInt16 const			inClickCount,
					 EventModifiers			modifiers)													
{
	Rect				screenArea;
	Point				leftLoc,
						rightLoc,
						curr,
						oldcurr;													
	long				mySize = 0L;
	char				theChar[5];
	Boolean				foundEnd = false;
	RealScreenDataPtr	realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
	
	
	if (realScreenPtr != null)
	{
		GetControlBounds(realScreenPtr->contentUserPane, &screenArea);
		
		// this is a multi-click method, so the two locations below are probably the same
		leftLoc = realScreenPtr->anchor;
		rightLoc = realScreenPtr->last;
		
		// 3.0 - if a triple-click has occurred, a selection will already be highlighted, so unhighlight it first
		if (inClickCount == 3) highlightVirtualRange(realScreenPtr, realScreenPtr->anchor, realScreenPtr->last, &screenArea);
		
		if (inClickCount == 2)
		{
			// double-click - scan to the right and then to the left
			for (foundEnd = false; !foundEnd; )
			{
				mySize = VSgettext(w, rightLoc.h, rightLoc.v, rightLoc.h + 1, rightLoc.v, theChar, 1L, "\015", 0,
									realScreenPtr->isTextSelectionRectangular);
				foundEnd = ((mySize == 0) || isspace(*theChar));
				unless (foundEnd) rightLoc.h++;
			}
			
			for (foundEnd = false; !foundEnd; )
			{
				mySize = VSgettext(w, leftLoc.h - 1, leftLoc.v, leftLoc.h, leftLoc.v, theChar, 1L, "\015", 0,
									realScreenPtr->isTextSelectionRectangular);
				foundEnd = ((mySize == 0) || isspace(*theChar));
				unless (foundEnd) leftLoc.h--;
			}
		}
		else
		{
			// triple-click - select the whole line
			leftLoc.h = -1;
			rightLoc.h = realScreenPtr->width;
		}
																					
		if (leftLoc.h != rightLoc.h)
		{
			// selection is not empty
			realScreenPtr->anchor = leftLoc; // new left bound
			realScreenPtr->last = rightLoc; // new right bound
			realScreenPtr->isTextSelected = 1;
			highlightVirtualRange(realScreenPtr, realScreenPtr->anchor, realScreenPtr->last,
									&screenArea); // highlight new selection
			
			if (modifiers & cmdKey)
			{
				// -clicking a selection means try to open this as a URL
				URL_HandleForScreenID(w);
			}
			else
			{
				enum
				{
					kSelectionModeWord = -1,
					kSelectionModeUp = 1,
					kSelectionModeDown = 2
				};
				SInt16		mode = kSelectionModeWord,
							newMode = 0;
				GrafPtr		oldPort = null;
				Point		localMouse;
				
				
				GetPort(&oldPort);
				SetPortWindowPort(realScreenPtr->window);
				
				curr.h = curr.v = 0;
				
				while (StillDown())
				{
					// wait for the mouse position to change
					do
					{
						oldcurr = curr;
						GetMouse(&localMouse);
						curr = findVirtualCellFromLocalPoint(w, localMouse, true/* scroll if necessary */);
						Threads_YieldToAnother();
					} while (EqualPt(curr, oldcurr) && StillDown());
					
					if ((curr.v < leftLoc.v) || ((curr.v == leftLoc.v) && (curr.h < leftLoc.h))) newMode = kSelectionModeUp;
					else if ((curr.v > leftLoc.v) || ((curr.v == leftLoc.v) && (curr.h > rightLoc.h))) newMode = kSelectionModeDown;
					else newMode = kSelectionModeWord; // inside double-clicked word
						
					// toggle highlight state of text between current and last mouse positions
					if (mode == kSelectionModeWord)
					{
						if (newMode == kSelectionModeDown)
						{
							realScreenPtr->anchor = leftLoc;
							highlightVirtualRange(realScreenPtr, curr, rightLoc, &screenArea);
							realScreenPtr->last = curr;
						}
						if (newMode == kSelectionModeUp)
						{
							realScreenPtr->anchor = rightLoc;
							highlightVirtualRange(realScreenPtr, curr, leftLoc, &screenArea);
							realScreenPtr->last = curr;
						}
					}
					if (mode == kSelectionModeUp)
					{
						if (newMode == kSelectionModeDown)
						{
							realScreenPtr->anchor = leftLoc;
							highlightVirtualRange(realScreenPtr, oldcurr, leftLoc, &screenArea);
							highlightVirtualRange(realScreenPtr, rightLoc, curr, &screenArea);
							realScreenPtr->last = curr;
						}
						if (newMode == kSelectionModeWord)
						{
							realScreenPtr->anchor = leftLoc;
							highlightVirtualRange(realScreenPtr, oldcurr, leftLoc, &screenArea);
							realScreenPtr->last = rightLoc;
						}
						if (newMode == mode)
						{
							highlightVirtualRange(realScreenPtr, oldcurr, curr, &screenArea);
							realScreenPtr->last = curr;
						}
					}
					if (mode == kSelectionModeDown)
					{
						if (newMode == kSelectionModeUp)
						{
							realScreenPtr->anchor = rightLoc;
							highlightVirtualRange(realScreenPtr, oldcurr, rightLoc, &screenArea);
							highlightVirtualRange(realScreenPtr, leftLoc, curr, &screenArea);
							realScreenPtr->last = curr;
						}
						if (newMode == kSelectionModeWord)
						{
							realScreenPtr->anchor = leftLoc;
							highlightVirtualRange(realScreenPtr, oldcurr, rightLoc, &screenArea);
							realScreenPtr->last = rightLoc;
						}
						if (newMode == mode)
						{
							highlightVirtualRange(realScreenPtr, oldcurr, curr, &screenArea);
							realScreenPtr->last = curr;
						}
					}
					mode = newMode;
				}
				
				SetPort(oldPort);
			}
		}
	}
}// handleMultiClick


/*!
This routine is called whenever the user changes
the size of a terminal window header.

(3.0)
*/
static void
handlePlacardSizeChange		(TerminalWindowHeaderRef const		inHeader)
{
	if (inHeader != null)
	{
		WindowRef		window = TerminalWindowHeader_GetOwningWindow(inHeader);
		
		
		if (window != null)
		{
			RealScreenDataPtr		realScreenPtr = ScreenManager_GetWindowRealScreenData(window);
			
			
			if (realScreenPtr != null)
			{
				Point		uncollapsedSize;
				Rect		bounds;
				UInt16		deltaY = 0,
							width = 0,
							height = 0;
				
				
				MacLook_GetWindowRegionBounds(window, kWindowContentRgn, &bounds);
				
				// resize the window to match the new height
				TerminalWindowHeader_GetNormalSize(inHeader, &uncollapsedSize);
				deltaY = uncollapsedSize.v;
				if (TerminalWindowHeader_IsCollapsed(inHeader)) deltaY = -deltaY;
				width = bounds.right - bounds.left;
				height = bounds.bottom - bounds.top + deltaY;
				SizeWindow(window, width, height, true/* update */);
				
				// adjust the rest of the window
				RSsetsize(ScreenManager_GetWindowScreenID(window));
				MacLook_InvalidatePortRectangle(window);
			}
		}
	}
}// handlePlacardSizeChange


/*!
Enables the specified terminal screen control,
presumably in response to an activate event of
its window.

(2.6)
*/
static void
handleScreenActivate	(RealScreenDataPtr		inRealScreenPtr)
{
	if (inRealScreenPtr != null)
	{
		WindowRef		window = inRealScreenPtr->window;
		
		
		// 3.0 - support the Text Services Manager
		if (FlagManager_Test(kFlagTextServicesManager))
		{
			(OSErr)ActivateTSMDocument(inRealScreenPtr->textServicesDocumentID);
		}
		
		// 3.0 - make sure selected text gets redrawn, so it can become a solid, highlighted area once again
		if (inRealScreenPtr->isTextSelected)
		{	
			RgnHandle	textSelectionRegion = getSelectedTextAsNewRegion(inRealScreenPtr);
			
			
			if (textSelectionRegion != null)
			{
				MacLook_InvalidateRegion(window, textSelectionRegion);
				Memory_DisposeRegion(&textSelectionRegion);
			}
		}
		
		// activate the scroll bars
		unless (FlagManager_Test(kFlagCarbonControlManager))
		{
			// without the new APIs for putting scroll bars in a foregrounded state,
			// remove the auxiliary controls used for the backgrounded appearance
			(OSErr)SetControlVisibility(inRealScreenPtr->invisibleScrollBarH, false/* visibility */, false/* draw */);
			(OSErr)SetControlVisibility(inRealScreenPtr->invisibleScrollBarV, false/* visibility */, false/* draw */);
			(OSErr)SetControlVisibility(inRealScreenPtr->left, true/* visibility */, true/* draw */);
			(OSErr)SetControlVisibility(inRealScreenPtr->scroll, true/* visibility */, true/* draw */);
		}
	}
}// handleScreenActivate


/*!
Disables the specified terminal screen control,
presumably in response to a deactivate event of
its window.

(2.6)
*/
static void
handleScreenDeactivate	(RealScreenDataPtr		inRealScreenPtr)
{
	if (inRealScreenPtr != null)
	{
		// handle a deactivate event for the specified window
		WindowRef		window = inRealScreenPtr->window;
		
		
		// 3.0 - support the Text Services Manager
		if (FlagManager_Test(kFlagTextServicesManager))
		{
			(OSErr)DeactivateTSMDocument(inRealScreenPtr->textServicesDocumentID);
		}
		
		// 3.0 - make sure selected text gets redrawn, so it can become an outlined, highlighted area
		if (inRealScreenPtr->isTextSelected)
		{	
			RgnHandle	textSelectionRegion = getSelectedTextAsNewRegion(inRealScreenPtr);
			
			
			if (textSelectionRegion != null)
			{
				MacLook_InvalidateRegion(window, textSelectionRegion);
				Memory_DisposeRegion(&textSelectionRegion);
			}
		}
		
		// deactivate the scroll bars
		unless (FlagManager_Test(kFlagCarbonControlManager))
		{
			// without the new APIs for putting scroll bars in a backgrounded state,
			// use auxiliary controls to give them a backgrounded appearance
			(OSErr)SetControlVisibility(inRealScreenPtr->left, false/* visibility */, false/* draw */);
			(OSErr)SetControlVisibility(inRealScreenPtr->scroll, false/* visibility */, false/* draw */);
			(OSErr)SetControlVisibility(inRealScreenPtr->invisibleScrollBarH, true/* visibility */, true/* draw */);
			(OSErr)SetControlVisibility(inRealScreenPtr->invisibleScrollBarV, true/* visibility */, true/* draw */);
		}
	}
}// handleScreenDeactivate


/*!
This method highlights text in the specified terminal
window, from the starting point to the ending point,
inclusive.  The constraint area is used to restrict
the region that can be drawn in a highlighted color,
in coordinates local to the window, not to the screens
origin.  While the anchor points are in units of rows
and columns, the constraint is in units of pixels.

The two anchors do not need to represent the start and
end per se; this routine automatically draws the region
between them correctly no matter which anchor is which.

In MacTelnet 3.0, this method is implemented using
regions.  It also now checks to see if the graphics port
of the specified window is the same as that of the
frontmost window - if it is not, the highlighting is
done appropriately for an inactive window.

(2.6)
*/
static void
highlightVirtualRange	(RealScreenDataPtr		inRealScreenPtr,
						 Point const			inZeroBasedAnchorRowAndColumnBeginning,
						 Point const			inZeroBasedAnchorRowAndColumnEnd,
						 Rect const*			inLocalConstraintRectPtr)
{
	RgnHandle		region = Memory_NewRegion();
	
	
	if (region != null)
	{
		GrafPtr		oldPort = null;
		Rect		temp;
		Point		lb,
					ub,
					normalizedStart,
					normalizedEnd;
		Point		characterSizeInPixels; // based on font metrics
		Boolean		shouldOutline = false;
		
		
		SetPt(&characterSizeInPixels, inRealScreenPtr->fwidth, inRealScreenPtr->fheight);
		
		GetPort(&oldPort);
		SetPortWindowPort(inRealScreenPtr->window);
		
		// determine whether the highlighting should be drawn outlined or solid
		shouldOutline = ((inRealScreenPtr->window != GetRealFrontWindow()) ||
							FlagManager_Test(kFlagSuspended));
		
		// normalize coordinates with respect to visible area of the virtual screen
		SetPt(&normalizedStart, inZeroBasedAnchorRowAndColumnBeginning.h - inRealScreenPtr->leftmarg,
								inZeroBasedAnchorRowAndColumnBeginning.v - inRealScreenPtr->topline);
		SetPt(&normalizedEnd, inZeroBasedAnchorRowAndColumnEnd.h - inRealScreenPtr->leftmarg,
								inZeroBasedAnchorRowAndColumnEnd.v - inRealScreenPtr->topline);
		if ((normalizedStart.v > normalizedEnd.v) ||
			((normalizedStart.h > normalizedEnd.h) && (normalizedStart.v == normalizedEnd.v)))
		{
			Point		p;
			
			
			// swap anchors as necessary
			SetPt(&p, normalizedEnd.h, normalizedEnd.v);
			SetPt(&normalizedEnd, normalizedStart.h, normalizedStart.v);
			SetPt(&normalizedStart, p.h, p.v);
		}
		
		if ((normalizedStart.v == normalizedEnd.v) ||
			(inRealScreenPtr->isTextSelectionRectangular))
		{
			Rect		temp2;
			
			
			// handle the special (simpler) case of single-line text selections
			if (normalizedStart.h < normalizedEnd.h) // get bounds the right way round
			{
				ub = normalizedStart;
				lb = normalizedEnd;
			}
			else
			{
				ub = normalizedEnd;
				lb = normalizedStart;
			}
			
			// set up rectangle bounding area to be highlighted & clip to constraint rectangle
			SetRect(&temp, (ub.h + 1) * characterSizeInPixels.h,
					ub.v * characterSizeInPixels.v,
					(lb.h + 1) * characterSizeInPixels.h,
					(lb.v + 1) * characterSizeInPixels.v);
			screenToLocalRect(inRealScreenPtr, &temp);
			SectRect(&temp, inLocalConstraintRectPtr, &temp2);
			
			// 3.0
			OpenRgn();
			FrameRect(&temp2);
			CloseRgn(region);
			
			if (shouldOutline) OutlineRegion(region); // DialogUtilities.c
		}
		else
		{
			// handle the case of multiple-line selections
			Rect		topPart,
						middlePart,
						bottomPart;
			
			
			SetRect(&topPart, 0, 0, 0, 0);
			SetRect(&middlePart, 0, 0, 0, 0);
			SetRect(&bottomPart, 0, 0, 0, 0);
			
			if (normalizedStart.v < normalizedEnd.v) ub = normalizedStart;
			else ub = normalizedEnd;
			
			if (normalizedStart.v > normalizedEnd.v) lb = normalizedStart;
			else lb = normalizedEnd;
			
			// define & constrain bounds of first (possibly partial) line to be highlighted
			SetRect(&temp, (ub.h + 1) * characterSizeInPixels.h,
					ub.v * characterSizeInPixels.v,
					inRealScreenPtr->width,
					(ub.v + 1) * characterSizeInPixels.v);
			screenToLocalRect(inRealScreenPtr, &temp);
			SectRect(&temp, inLocalConstraintRectPtr, &topPart);
			
			// define & constrain bounds of last (possibly partial) line to be highlighted
			SetRect(&temp, 0, lb.v * characterSizeInPixels.v,
					(lb.h + 1) * characterSizeInPixels.h,
					(lb.v + 1) * characterSizeInPixels.v);
			screenToLocalRect(inRealScreenPtr, &temp);
			SectRect(&temp, inLocalConstraintRectPtr, &bottomPart);
			
			//if (lb.v - ub.v > 1) // highlight extends across more than two lines
			{
				// highlight all in-between lines
				SetRect(&temp,
							0,
							(ub.v + 1) * characterSizeInPixels.v,
							inRealScreenPtr->width,
							lb.v * characterSizeInPixels.v);
				screenToLocalRect(inRealScreenPtr, &temp);
				SectRect(&temp, inLocalConstraintRectPtr, &middlePart); // clip to constraint rectangle
			}
			
			// 3.0 - construct a region
			OpenRgn();
			
			if (shouldOutline)
			{
				// To outline the text selection, draw lines on the edges
				// of the rectangles.  The top line (if any) has its left,
				// top and right edges drawn, with a leading line along
				// its bottom edge.  The bottom line (if any) has its left,
				// bottom and right edges drawn, with a trailing line along
				// its top edge.  Any other line simply has its left and
				// right edges drawn.  When a series of lines are drawn in
				// sequence, through repeated invocations of this method,
				// the result is what appears to be the outline of a shape
				// containing the entire selected region.
				Point		oldPen;
				Rect		rect;
				
				
				// save pen location
				GetPen(&oldPen);
				
				// top rectangle outline
				//
				// Draw these lines:
				//							____________________
				//  _______________________|					|
				SetRect(&rect, middlePart.left, topPart.bottom - 1, topPart.left, topPart.bottom);
				FrameRect(&rect);
				SetRect(&rect, topPart.left, topPart.top, topPart.left + 1, topPart.bottom);
				FrameRect(&rect);
				SetRect(&rect, topPart.left + 1, topPart.top, topPart.right - 1, topPart.top + 1);
				FrameRect(&rect);
				SetRect(&rect, topPart.right - 1, topPart.top, topPart.right, topPart.bottom);
				FrameRect(&rect);
				
				// middle rectangle outline
				//
				// Draw these lines:
				//
				//  											|
				//  											|
				//  											|
				//  								____________|
				SetRect(&rect, middlePart.right - 1, middlePart.top, middlePart.right, middlePart.bottom);
				FrameRect(&rect);
				SetRect(&rect, bottomPart.right, bottomPart.top,
						bottomPart.right + 10000/*
												This should be "middlePart.right", but a WEIRD
												MPW compiler bug causes this to not work.  So,
												I just make sure the line is wide enough by
												making it infinity (it gets clipped when
												drawn anyway). -- KG
												*/,
						bottomPart.top + 1);
				FrameRect(&rect);
				if (rect.right < 0) BEEP;
				
				// bottom rectangle outline
				//
				// Draw these lines:
				//
				// |_______________________________|
				SetRect(&rect, bottomPart.right - 1, bottomPart.top, bottomPart.right, bottomPart.bottom);
				FrameRect(&rect);
				SetRect(&rect, bottomPart.left + 1, bottomPart.bottom - 1, bottomPart.right - 1, bottomPart.bottom);
				FrameRect(&rect);
				SetRect(&rect, bottomPart.left, bottomPart.top, bottomPart.left + 1, bottomPart.bottom);
				FrameRect(&rect);
				
				// middle rectangle outline
				//
				// Draw this line:
				//
				// |											
				// |											
				// |											
				// |											
				SetRect(&rect, middlePart.left, middlePart.top, middlePart.left + 1, middlePart.bottom);
				FrameRect(&rect);
				
				// restore pen location
				MoveTo(oldPen.h, oldPen.v);
			}
			else
			{
				// Basic rectangles are sufficient for non-outlined drawing.
				FrameRect(&topPart);
				FrameRect(&middlePart);
				FrameRect(&bottomPart);
			}
			CloseRgn(region);
		}
		
		// Highlight the region.  By setting highlight mode just prior to
		// calling a QuickDraw inversion routine, it is possible to use a
		// background highlighting color instead of doing simple inversion.
		// The highlighting mode automatically disengages when the inversion
		// finishes.  The user may choose not to use the highlighting color.
		{
			Boolean		invertOnly = false;
			Size		actualSize = 0L;
			
			
			unless (Preferences_GetData(kTelnetPreferenceClassGeneral, kTelnetPreferenceTag_PureInverse,
										sizeof(invertOnly), &invertOnly, &actualSize) == kTelnetPrefsResultCodeSuccess)
			{
				invertOnly = false; // assume text is highlighted according to Mac OS standards, if preference cant be found
			}
			if (shouldOutline) EraseRgn(region); // test
			unless (invertOnly) ColorUtilities_HiliteMode();
			InvertRgn(region);
		}
		
		Memory_DisposeRegion(&region);
		SetPort(oldPort);
	}
}// highlightVirtualRange


/*!
Marks the entire screen area for the specified
screen as part of its windows update region
(effectively causing the screen to be redrawn).

(3.0)
*/
static void
invalidateScreenRectangle		(RealScreenDataPtr		inRealScreenPtr)
{
	MacLook_InvalidateControlRectangle(inRealScreenPtr->backgroundUserPane);
}// invalidateScreenRectangle


/*!
A standard ControlUserPaneDrawUPP, this method
draws in the area of a scroll bar that is in an
inactive window.  MacTelnet hides the real
scroll bar control, replacing it with a user pane
that has this drawing routine.

With Appearance 1.1 and beyond, scroll bar
inactive states are drawn correctly, so this
method is only used on Appearance 1.0 systems.

(3.0)
*/
static pascal void
invisibleScrollBarUserPaneDraw	(ControlRef		inScrollBar,
								 SInt16			UNUSED_ARGUMENT(inPartCode))
{
	ColorPenState		state;
	RGBColor			solid;
	Rect				rect;
	
	
	ColorUtilities_GetColorAndPenState(&state);
	ColorUtilities_NormalizeColorAndPen();
	GetControlBounds(inScrollBar, &rect);
	solid.red = solid.blue = solid.green = RGBCOLOR_INTENSITY_MAX;
	RGBBackColor(&solid);
	EraseRect(&rect);
	solid.red = solid.blue = solid.green = 0;
	RGBForeColor(&solid);
	FrameRect(&rect);
	ColorUtilities_SetColorAndPenState(&state);
}// invisibleScrollBarUserPaneDraw


/*!
To translate pixels in the coordinate system of a
window so that they are relative to the origin of
the screen, use this method.  In version 2.6, the
program always assumed that the screen origin was
the top-left corner of the window, which made it
extremely hard to change.  Now, this routine is
invoked everywhere to ensure that, before port
pixel offsets are used to refer to the screen,
they get translated correctly.

If you are not interested in translating a point,
but rather in shifting a dimension horizontally or
vertically, you can pass "null" for the dimension
that you do not use.

See also screenToLocal().

(3.0)
*/
static void
localToScreen		(RealScreenDataPtr		inRealScreenPtr,
					 SInt16*				inoutHorizontalPixelOffsetFromPortOrigin,
					 SInt16*				inoutVerticalPixelOffsetFromPortOrigin)
{
	Point		origin;
	
	
	getScreenOrigin(inRealScreenPtr, &origin.h, &origin.v);
	if (inoutHorizontalPixelOffsetFromPortOrigin != null)
	{
		(*inoutHorizontalPixelOffsetFromPortOrigin) -= origin.h;
	}
	if (inoutVerticalPixelOffsetFromPortOrigin != null)
	{
		(*inoutVerticalPixelOffsetFromPortOrigin) -= origin.v;
	}
}// localToScreen


/*!
To translate a boundary in the coordinate system of a
window so that it is relative to the origin of the
screen, use this method.  In version 2.6, the program
always assumed that the screen origin was the top-left
corner of the window, which made it extremely hard to
change.  Now, this routine is invoked everywhere to
ensure that, before port pixel offsets are used to
refer to the screen, they get translated correctly.

(3.0)
*/
static void
localToScreenRect	(RealScreenDataPtr		inRealScreenPtr,
					 Rect*					inoutPortOriginBounds)
{
	if (inoutPortOriginBounds != null)
	{
		localToScreen(inRealScreenPtr, &inoutPortOriginBounds->left, &inoutPortOriginBounds->top);
		localToScreen(inRealScreenPtr, &inoutPortOriginBounds->right, &inoutPortOriginBounds->bottom);
	}
}// localToScreenRect


/*!
This method, of standard DeviceLoopDrawingUPP form,
handles drawing of terminal window backgrounds
correctly no matter what the device characteristics
are.  If the background is a picture, then the
picture is drawn; otherwise, the background is
erased to a specific color.

The controls reference constant is expected to be
of type "RealScreenDataPtr", a pointer to valid
real screen data (which defines the background
characteristics).

(3.0)
*/
static pascal void
realScreenPictureDeviceLoop		(short				UNUSED_ARGUMENT(inColorDepth),
								 short				UNUSED_ARGUMENT(inDeviceFlags),
								 GDHandle			UNUSED_ARGUMENT(inTargetDevice),
								 long				inControl)
{
	RealScreenDataPtr	realScreenPtr = (RealScreenDataPtr)GetControlReference(REINTERPRET_CAST(inControl, ControlRef));
	Rect				bounds;
	
	
	GetControlBounds(REINTERPRET_CAST(inControl, ControlRef), &bounds);
	
	// draw background picture, if any
	if (realScreenPtr != null)
	{
		// set up the control background color
		setRealScreenColorAttr(realScreenPtr, 0);
		
		// paint the background using the current background color
		EraseRect(&bounds);
		
		if ((realScreenPtr->customTextBackground) && (realScreenPtr->picture != null))
		{
			PicHandle		picture = realScreenPtr->picture;
			
			
			// reset "bounds" to be the pictures dimensions
			bounds.right = bounds.left + ((*picture)->picFrame.right - (*picture)->picFrame.left);
			bounds.bottom = bounds.top + ((*picture)->picFrame.bottom - (*picture)->picFrame.top);
			DrawPicture(picture, &bounds);
		}
		
		// draw an ultra-cool shadow that appears to be cast by the window header
		drawShadow(realScreenPtr);
	}
}// realScreenPictureDeviceLoop


/*!
A standard ControlUserPaneDrawUPP, this
method repaints the terminal background.

(3.0)
*/
static pascal void
realScreenPictureDraw	(ControlRef		inControl,
						 SInt16			UNUSED_ARGUMENT(inPartCode))
{
	DeviceLoopDrawingUPP	upp = NewDeviceLoopDrawingUPP(realScreenPictureDeviceLoop);
	ColorPenState			state;
	
	
	ColorUtilities_GetColorAndPenState(&state);
	ColorUtilities_NormalizeColorAndPen();
	DeviceLoopClipAndDrawControl(inControl, upp);
	DisposeDeviceLoopDrawingUPP(upp), upp = null;
	ColorUtilities_SetColorAndPenState(&state);
}// realScreenPictureDraw


/*!
Sets up the background color or pattern of a terminal
screen, so that any attempt by the embedded control
(the screen itself) to erase the background will cause
the correct color to appear.

(3.0)
*/
static pascal void
realScreenPictureSetBackground		(ControlRef					inControl,
									 ControlBackgroundPtr		UNUSED_ARGUMENT(inBackgroundInfoPtr))
{
	WindowRef		window = GetControlOwner(inControl);
	
	
	// this sanity check ensures that no crash occurs by attempting to access a non-screen window
	if (ScreenManager_IsWindowAScreen(window))
	{
		RealScreenDataPtr	realScreenPtr = ScreenManager_GetWindowRealScreenData(window);
		
		
		// for graphics debugging, you might want to choose a bright color always (like yellow)
		//{RGBColor bright = {RGBCOLOR_INTENSITY_MAX, RGBCOLOR_INTENSITY_MAX, 0}; RGBBackColor(&bright); } // TEMP
		if (realScreenPtr != null) setRealScreenColorAttr(realScreenPtr, 0);
	}
}// realScreenPictureSetBackground


/*!
This method, of standard DeviceLoopDrawingUPP form,
handles drawing of terminal windows correctly no
matter what the device characteristics are.

(3.0)
*/
static pascal void
realScreenUserPaneDeviceLoop	(short			inColorDepth,
								 short			UNUSED_ARGUMENT(inDeviceFlags),
								 GDHandle		inTargetDevice,
								 long			inControl)
{
	WindowRef			window = GetControlOwner(REINTERPRET_CAST(inControl, ControlRef));
	RealScreenDataPtr	realScreenPtr = (RealScreenDataPtr)GetControlReference(REINTERPRET_CAST(inControl, ControlRef));
	Rect				controlBounds;
	
	
	GetControlBounds(REINTERPRET_CAST(inControl, ControlRef), &controlBounds);
	
	// set up the control background and foreground colors
	(OSErr)SetUpControlBackground(REINTERPRET_CAST(inControl, ControlRef), inColorDepth,
									ColorUtilities_IsColorDevice(inTargetDevice));
	
	// this sanity check ensures that no crash occurs by attempting to access a non-screen window
	if (ScreenManager_IsWindowAScreen(window) && (realScreenPtr != null))
	{
		TelnetScreenID		screenID = ScreenManager_GetWindowScreenID(window);
		SInt16				x1 = 0,
							x2 = 0,
							y1 = 0,
							y2 = 0;
		
		
		getVirtualVisibleRegion(realScreenPtr, &x1, &y1, &x2, &y2);
		if (x2 > x1)
		{
			VSredraw(screenID, x1, y1, x2, y2); // draw the window text
			realScreenPtr->attributes = -1; // forces attributes to reset themselves properly
		}
	}
}// realScreenUserPaneDeviceLoop


/*!
A standard ControlUserPaneDrawUPP, this
method repaints the terminal screen.

(3.0)
*/
static pascal void
realScreenUserPaneDraw	(ControlRef		inControl,
						 SInt16			UNUSED_ARGUMENT(inPartCode))
{
	DeviceLoopDrawingUPP	upp = NewDeviceLoopDrawingUPP(realScreenUserPaneDeviceLoop);
	ColorPenState			state;
	
	
	ColorUtilities_GetColorAndPenState(&state);
	ColorUtilities_NormalizeColorAndPen();
	DeviceLoopClipAndDrawControl(inControl, upp);
	DisposeDeviceLoopDrawingUPP(upp), upp = null;
	ColorUtilities_SetColorAndPenState(&state);
}// realScreenUserPaneDraw


/*!
A standard ControlUserPaneIdleUPP, this
method handles cursor flashing for
terminal screens.

(3.0)
*/
static pascal void
realScreenUserPaneIdle		(ControlRef			inControl)
{
	WindowRef			window = GetControlOwner(inControl);
	RealScreenDataPtr	realScreenPtr = REINTERPRET_CAST(GetControlReference(inControl), RealScreenDataPtr);
	
	
	// do an insane amount of error-checking here, because this is run
	// in a separate thread and cant afford to screw up...
	if (ScreenManager_IsWindowAScreen(window))
	{
		if ((realScreenPtr != null) && cursorBlinks(realScreenPtr) && (!FlagManager_Test(kFlagSuspended)))
		{
			TelnetCursorState	newCursorState = realScreenPtr->cursorState;
			
			
			// yield to other threads until at least the caret time has passed
			Threads_DelayMinimumTicks(GetCaretTime());
			
			// now change the cursor state, redrawing the cursor if necessary
			switch (realScreenPtr->cursorState)
			{
				case kTelnetCursorStateVisible:
					setCursorVisibility(realScreenPtr, false/* visible */);
					break;
				
				case kTelnetCursorStateInvisible:
				default:
					setCursorVisibility(realScreenPtr, true/* visible */);
					break;
			}
		}
	}
}// realScreenUserPaneIdle


/*!
To translate pixels in the coordinate system of a
screen so that they are relative to the origin of
the window itself, use this method.  In version 2.6,
the program always assumed that the screen origin
was the top-left corner of the window, which made
it extremely hard to change.  Now, this routine is
invoked everywhere to ensure that, before pixel
offsets are used to refer to the screen, they get
translated correctly.

If you are not interested in translating a point,
but rather in shifting a dimension horizontally or
vertically, you can pass "null" for the dimension
that you do not use.

See also localToScreen().

(3.0)
*/
static void
screenToLocal		(RealScreenDataPtr		inRealScreenPtr,
					 SInt16*				inoutHorizontalPixelOffsetFromScreenOrigin,
					 SInt16*				inoutVerticalPixelOffsetFromScreenOrigin)
{
	Point		origin;
	
	
	getScreenOrigin(inRealScreenPtr, &origin.h, &origin.v);
	if (inoutHorizontalPixelOffsetFromScreenOrigin != null)
	{
		(*inoutHorizontalPixelOffsetFromScreenOrigin) += origin.h;
	}
	if (inoutVerticalPixelOffsetFromScreenOrigin != null)
	{
		(*inoutVerticalPixelOffsetFromScreenOrigin) += origin.v;
	}
}// screenToLocal


/*!
To translate a boundary in the coordinate system of a
screen so that it is relative to the origin of the
window itself, use this method.  In version 2.6, the
program always assumed that the screen origin was the
top-left corner of the window, which made it extremely
hard to change.  Now, this routine is invoked
everywhere to ensure that, before pixel offsets are
used to refer to the screen, they get translated
correctly.

(3.0)
*/
static void
screenToLocalRect	(RealScreenDataPtr		inRealScreenPtr,
					 Rect*					inoutScreenOriginBounds)
{
	if (inoutScreenOriginBounds != null)
	{
		screenToLocal(inRealScreenPtr, &inoutScreenOriginBounds->left, &inoutScreenOriginBounds->top);
		screenToLocal(inRealScreenPtr, &inoutScreenOriginBounds->right, &inoutScreenOriginBounds->bottom);
	}
}// screenToLocalRect


/*!
This is a standard control action procedure which,
as of MacTelnet 3.0, dynamically scrolls a terminal
window.

(2.6)
*/
static pascal void
scrollProc		(ControlRef			inScrollBarClicked,
				 ControlPartCode	inPartCode)
{
	enum
	{
		kPageScrollDelayTicks = 2
	};
	TelnetScreenID			w = 0;
	SInt16					kind = -1;
	SInt16					x1 = 0,
							y1 = 0,
							x2 = 0,
							y2 = 0,
							visibleWidth = 0,
							visibleHeight = 0;
	
	
	kind = ScreenManager_FindScrollBar(inScrollBarClicked, &w); // 3.0
	VSgetrgn(w, &x1, &y1, &x2, &y2);
	visibleWidth = x2 - x1;
	visibleHeight = y2 - y1;
	
	// 3.0 - lock down the cursor so the thread handling cursor flashing doesnt mess up the display
	// unimplemented
	
	Threads_YieldToAnother(); // 3.0 - allows MacTelnet processes to continue (although slowly) while dragging the scroll bar
	
	if (kind == 2)
	{
		switch (inPartCode)
		{
			case kControlUpButtonPart: // up arrow on a horizontal scroll bar means left arrow
				Terminal_ScrollColumnsTowardRightEdge(w, 1);
				break;
			
			case kControlDownButtonPart: // down arrow on a horizontal scroll bar means right arrow
				Terminal_ScrollColumnsTowardLeftEdge(w, 1);
				break;
			
			case kControlPageUpPart:
				// 3.0 - animate page scrolling a bit
				Terminal_ScrollColumnsTowardRightEdge(w, INTEGER_QUARTERED(visibleWidth));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollColumnsTowardRightEdge(w, INTEGER_QUARTERED(visibleWidth));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollColumnsTowardRightEdge(w, INTEGER_QUARTERED(visibleWidth));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollColumnsTowardRightEdge(w, visibleWidth - 3 * INTEGER_QUARTERED(visibleWidth));
				break;
			
			case kControlPageDownPart:
				// 3.0 - animate page scrolling a bit
				Terminal_ScrollColumnsTowardLeftEdge(w, INTEGER_QUARTERED(visibleWidth));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollColumnsTowardLeftEdge(w, INTEGER_QUARTERED(visibleWidth));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollColumnsTowardLeftEdge(w, INTEGER_QUARTERED(visibleWidth));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollColumnsTowardLeftEdge(w, visibleWidth - 3 * INTEGER_QUARTERED(visibleWidth));
				break;
			
			case kControlIndicatorPart:
				// 3.0 - live scrolling
				{
					SInt16		value = GetControlValue(inScrollBarClicked);
					
					
					VSsetrgn(w, value, y1, value + (visibleWidth), y2);
				}
				break;
			
			default:
				break;
		}
	}
	else if (kind == 1)
	{
		switch (inPartCode)
		{
			case kControlUpButtonPart:
				Terminal_ScrollRowsTowardBottomEdge(w, 1);
				break;
			
			case kControlDownButtonPart:
				Terminal_ScrollRowsTowardTopEdge(w, 1);
				break;
			
			case kControlPageUpPart:
				// 3.0 - animate page scrolling a bit (users can more easily see what is happening)
				Terminal_ScrollRowsTowardBottomEdge(w, INTEGER_QUARTERED(visibleHeight));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollRowsTowardBottomEdge(w, INTEGER_QUARTERED(visibleHeight));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollRowsTowardBottomEdge(w, INTEGER_QUARTERED(visibleHeight));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollRowsTowardBottomEdge(w, visibleHeight - 3 * INTEGER_QUARTERED(visibleHeight));
				break;
			
			case kControlPageDownPart:
				// 3.0 - animate page scrolling a bit (users can more easily see what is happening)
				Terminal_ScrollRowsTowardTopEdge(w, INTEGER_QUARTERED(visibleHeight));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollRowsTowardTopEdge(w, INTEGER_QUARTERED(visibleHeight));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollRowsTowardTopEdge(w, INTEGER_QUARTERED(visibleHeight));
				Threads_DelayMinimumTicks(kPageScrollDelayTicks);
				Terminal_ScrollRowsTowardTopEdge(w, visibleHeight - 3 * INTEGER_QUARTERED(visibleHeight));
				break;
			
			case kControlIndicatorPart:
				// 3.0 - live scrolling
				{
					SInt16		value = GetControlValue(inScrollBarClicked);
					
					
					VSsetrgn(w, x1, value, x2, value + (visibleHeight));
				}
				break;
			
			default:
				break;
		}
	}
	
	// handle update events so the background picture gets redrawn
	{
		RealScreenDataPtr		realScreenPtr = ScreenManager_GetRealScreenDataOf(w);
		
		
		if (realScreenPtr != null) if (realScreenPtr->customTextBackground)
		{
			EventRecord			updateEvent;
			
			
			WaitNextEvent(updateMask, &updateEvent, 1L/* sleep ticks */, null/* mouse-moved region */);
			(Boolean)EventLoop_HandleEvent(&updateEvent);
		}
	}
}// scrollProc


/*!
Hides or shows the cursor in the specified screen.
It may be invoked repeatedly.

(3.0)
*/
static void
setCursorVisibility		(RealScreenDataPtr		inRealScreenPtr,
						 Boolean const			inIsVisible)
{
	TelnetCursorState	newCursorState = (inIsVisible) ? kTelnetCursorStateVisible : kTelnetCursorStateInvisible;
	
	
	if (inRealScreenPtr->drawingEnabled)
	{
		if (inRealScreenPtr->cursorState != newCursorState)
		{
			GrafPtr		oldPort = null;
			
			
			GetPort(&oldPort);
			SetPortWindowPort(inRealScreenPtr->window);
			InvertRect(&inRealScreenPtr->cursorRect);
			inRealScreenPtr->cursorState = newCursorState;
			SetPort(oldPort);
		}
	}
}// setCursorVisibility


/*!
Sets the screen variable for the current text
attributes to be the specified attributes, and
sets the QuickDraw settings appropriately for
the requested attributes (text font, size,
style, color, inverse video, etc.).

In MacTelnet 3.0, session windows can now have
special backgrounds!  For this reason, this
method now checks custom flags in the data for
the specified screen before setting the
background color, and uses "transparent" text
mode if the background is not a solid color.

(2.6)
*/
static void
setRealScreenAttr		(RealScreenDataPtr		inRealScreenPtr,
						 SInt16 const			inAttributes)
{
	if (inRealScreenPtr != null)
	{
		short			fg = 0,
						bg = 0;
		static GrafPtr	lastPort = null;
		GrafPtr			currentPort = null;
		
		
		GetPort(&currentPort);
		if ((inRealScreenPtr->attributes == -1) ||
			(inRealScreenPtr->attributes != inAttributes) ||
			(currentPort != lastPort))
		{
			lastPort = currentPort;
			inRealScreenPtr->attributes = inAttributes;
			
			// For special glyphs, use the NCSA VT font.  Otherwise,
			// use the default text attributes.  Note that ANSI
			// graphics are recognized elsewhere.
			if (STYLE_USE_VT_FONT(inAttributes)) TextFont(74);
			else RealScreen_TextFont(inRealScreenPtr->fontName, inRealScreenPtr->fsiz, inAttributes);
			TextFace((inAttributes & outline) >> 1); // do outline as underline setting
			
			// 3.0 - for a sufficiently large font, allow underlining
			if (STYLE_UNDERLINE(inAttributes))
			{
				SInt16		fontSize = GetPortTextSize(currentPort);
				Style		fontFace = GetPortTextFace(currentPort);
				
				
				if (fontSize > 10) // arbitrary (should this be a user preference?)
				{
					TextFace(fontFace | underline);
				}
			}
			
			// find the correct colors in the color table
			getScreenColorEntries(inRealScreenPtr, inAttributes, &fg, &bg);
			
			// set up text modes: inverse or normal video?
			if (STYLE_INVERSE_VIDEO(inAttributes)) TextMode(notSrcCopy);
			else
			{
				TextMode((inRealScreenPtr->customTextBackground) ? transparent : srcCopy);
			}
			
			// set up colors
			{
				RGBColor	color;
				Boolean		dontDim = false;
				Size		actualSize = 0L;
				
				
				// determine if the user wants windows to be dimmed when theyre deactivated
				unless (Preferences_GetData(kTelnetPreferenceClassGeneral, kTelnetPreferenceTag_DontDimBackgroundScreens,
										sizeof(dontDim), &dontDim, &actualSize) == kTelnetPrefsResultCodeSuccess)
				{
					dontDim = false; // assume windows dim when deactivated, if preference cant be found
				}
				
				// set up foreground color
				GetEntryColor(inRealScreenPtr->paletteTerminal, fg, &color);
				RGBForeColor(&color);
				
				// set up background color
				GetEntryColor(inRealScreenPtr->paletteTerminal, bg, &color);
				RGBBackColor(&color);
				
				// dim the colors if the window is inactive
				unless ((IsControlActive(inRealScreenPtr->rootControl)) || (dontDim)) UseInactiveColors();
			}
		}
	}
}// setRealScreenAttr


/*!
Sets the QuickDraw background color settings
appropriately for the requested attributes
(only style and color generally affect color).

(3.0)
*/
static void
setRealScreenColorAttr		(RealScreenDataPtr		inRealScreenPtr,
							 SInt16 const			inAttributes)
{
	if (inRealScreenPtr != null)
	{
		short			fg = 0,
						bg = 0;
		static GrafPtr	lastPort = null;
		GrafPtr			currentPort = null;
		RGBColor		color,
						preservedForeColor;
		Boolean			dontDim = false;
		Size			actualSize = 0L;
		
		
		// find the correct colors in the color table
		getScreenColorEntries(inRealScreenPtr, inAttributes, &fg, &bg);
		
		// set up colors
		GetForeColor(&preservedForeColor);
		
		// determine if the user wants windows to be dimmed when theyre deactivated
		unless (Preferences_GetData(kTelnetPreferenceClassGeneral, kTelnetPreferenceTag_DontDimBackgroundScreens,
								sizeof(dontDim), &dontDim, &actualSize) == kTelnetPrefsResultCodeSuccess)
		{
			dontDim = false; // assume windows dim when deactivated, if preference cant be found
		}
		
		// set up foreground color (needed to calculate dimmed background color properly)
		GetEntryColor(inRealScreenPtr->paletteTerminal, fg, &color);
		RGBForeColor(&color);
		
		// set up background color
		GetEntryColor(inRealScreenPtr->paletteTerminal, bg, &color);
		RGBBackColor(&color);
		
		// dim the colors if the window is inactive
		unless ((IsControlActive(inRealScreenPtr->rootControl)) || (dontDim)) UseInactiveColors();
		RGBForeColor(&preservedForeColor);
	}
}// setRealScreenColorAttr


/*!
To copy a color into a screen windows color table,
given an external color specifier, use this method.

(3.0)
*/
static void
setScreenColor		(RealScreenDataPtr				inRealScreenPtr,
					 RealScreenColorIndex const		inColorEntryNumber,
					 RGBColor const*				inColorPtr)
{
	SetEntryColor(inRealScreenPtr->paletteTerminal, inColorEntryNumber, inColorPtr);
}// setScreenColor


/*!
To set the font ascent, height, width, and
monospaced attributes of the font information
for the specified screen, use this method.
The font information is determined by looking
at the current settings for the specified port.

This routine corrects a long-standing bug where
Telnet would incorrectly recalculate the font
width attribute based on the current font of
the specified port, not the default font of the
specified screen.  On a terminal screen, every
cell must be the same width, so all rendered
fonts must use the same width, regardless of
the width they need.  Thus, as of Telnet 3.0,
font metrics ignore the current font of the

(3.0)
*/
static void
setUpScreenFontMetrics	(RealScreenDataPtr	inScreenPtr)
{
	FontInfo	fontInfo;
	GrafPtr		port = null;
	SInt16		oldFont = 0;
	
	
	GetPort(&port);
	oldFont = GetPortTextFont(port);
	GetFontInfo(&fontInfo);
	inScreenPtr->fascent = fontInfo.ascent;
	inScreenPtr->fheight = fontInfo.ascent + fontInfo.descent + fontInfo.leading;
	inScreenPtr->monospaced = (CharWidth('W') == CharWidth('i'));
	TextFontByName(inScreenPtr->fontName);
	inScreenPtr->fwidth = CharWidth('W');
	TextFont(oldFont);
}// setUpScreenFontMetrics


/*!
Handles display of help balloons in the
Set Window Title dialog box.

(3.0)
*/
static pascal Boolean
setWindowTitleDialogEventFilter	(DialogRef			inDialog,
								 EventRecord*		inoutEventPtr,
								 DialogItemIndex*	outItemIndex)
{
	Boolean		result = pascal_false;
	
	
	// first, make sure help balloons show up properly for the cursor location, if Balloon Help is on
	if (IsPopUpHelpEnabled()) DisplayBalloonForDialogItemUnderMouse(inDialog, inoutEventPtr->where,
																		kDialogBalloonsIDSetWindowTitle);
	
	unless (result) result = (StandardDialogEventFilter(inDialog, inoutEventPtr, outItemIndex));
	
	return result;
}// setWindowTitleDialogEventFilter


/*!
To redraw the specified screens text selection,
clipped to the specified rectangle in the current
graphics port, use this method.

(3.0)
*/
static void
updateTextSelection		(TelnetScreenID const		inScreenID,
						 Rect const*				inClipRectPtr)
{
	RealScreenDataPtr		realScreenPtr = ScreenManager_GetRealScreenDataOf(inScreenID);
	
	
	if (realScreenPtr != null) if (RealScreen_IsTextSelected(inScreenID))
	{
		highlightVirtualRange(realScreenPtr, *(Point*)&realScreenPtr->anchor, *(Point*)&realScreenPtr->last, inClipRectPtr);
	}
}// updateTextSelection