/*###############################################################

	RealScreenData.h
	
	Contains weird stuff and the critical definition of the
	"RealScreenData" structure.
	
	MacTelnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

#ifndef __REALSCREENDATA__
#define __REALSCREENDATA__

// Mac includes
#include <Menus.h>
#include <Palettes.h>
#include <SpeechSynthesis.h>
#include <TextEncodingConverter.h>
#include <TextServices.h>

// MacTelnet includes
#include "StatusIcon.h"
#include "TelnetHeader.h"
#include "TerminalWindowHeader.h"



// constants
typedef UInt8 TelnetCursorState;
enum
{
	kTelnetCursorStateInvisible = 0,
	kTelnetCursorStateVisible = 1
};

typedef UInt8 TelnetSpeechMode;
enum
{
	kTelnetSpeechModeSpeakAlways = 0,
	kTelnetSpeechModeSpeakWhenActive = 1,
	kTelnetSpeechModeSpeakWhenInactive = 2
};



// types
struct RealScreenData
{
	short			positionIndex; // window stagger position
	
	SInt16			attributes;	// current text attribute flags
	
	Rect			unused;
	Rect			cursorRect;	// the exact local rectangle of the cursors visible region
	
	WindowRef		window;
	PaletteHandle	paletteTerminal; // text drawing colors (attached to the window)
	
	char			cursorState,
					isTextSelected;
	
	Point			last,
					anchor;
	short			topline,
					leftmarg;	/* leftmost visible column position */
	short			rheight,	/* entire window dimensions (visible or not) */
					rwidth;
	short			height,
					width;		/* visible window dimensions */
	
	Boolean			isReverseVideo,	/* Set when we've reversed normal fg and bg colors */
					areANSIColorsEnabled;	// whether ANSI-colored text is allowed in this window
	ControlRef		left,		/* The CH for the horizontal scroll bar */
					scroll,		/* The CH for the vertical scroll bar */
					invisibleScrollBarH,	// used to represent inactive-window scroll bars on Mac OS 8.1 and earlier
					invisibleScrollBarV,	// used to represent inactive-window scroll bars on Mac OS 8.1 and earlier
					rootControl,	// used with new Appearance controls and embedding
					contentUserPane,
					backgroundUserPane;
	
	TerminalWindowHeaderRef		windowHeader; // reference to module for handling window header
	StatusIconRef		connectionStatusIcon, // displays connection alive, dead, etc.
						encryptionStatusIcon; // displays encryption activity
	
	short			min,		/* Minimum vertical scrollbar value (number of lines in screen buffer and scrollback) */
					max,		/* Maximum vertical scrollbar value */
					current,	/* current vertical scrollbar value */
					lmin,		/* Minimum horizontal scrollbar value (always 0) */
					lmax,		/* Maximum horizontal scrollbar value (number of columns not visible) */
					lcurrent;	/* current horizontal scrollbar value (leftmost visible column) */
	
	short			fascent,	/* Font Ascent */
					fsiz,		/* Font Size */
					fheight,	/* Font Height/character */
					fwidth,		/* Font Width /character */
					monospaced;	/* Font is monospaced */
	
	Str255			fontName;	// the name of the screen rendering fonts family
	
	Boolean			obscured;	// is the window hidden, via a command in the Window menu?
	Boolean			drawingEnabled;		// is the window drawing pen visible?
	
	Boolean			customTextBackground;		// does the window have a background picture?
	Boolean			isTextSelectionRectangular;	// is the text selection unattached from the left and right screen edges?
	
	struct
	{
		Boolean				enabled;				// is window text spoken?
		TelnetSpeechMode	mode;					// when is speech activated and deactivated?
		Fixed				rate;					// speed of speech
		Fixed				pitch;					// frequency of speech
		SpeechChannel		channel;				// speech channel for the window
		VoiceSpec			currentVoice;			// voice used to speak text
	} speech;
	
	TSMDocumentID	textServicesDocumentID;		// this ID represents the TSM impression of a session windows text input
	
	PicHandle		picture;	// background picture (if any)
	
	TECObjectRef	textEncodingConverter;		// converter for text
};
#ifndef __cplusplus
typedef struct RealScreenData	RealScreenData;
#endif
typedef RealScreenData*			RealScreenDataPtr;


#endif