/*###############################################################

	Releases.c
	
	Utility library for managing system-defined versions.
	See "Releases.h" for macros which manage versions for
	shared libraries specific to one program.
	
	Data Access Library 1.0
	 1998-2000 by Kevin Grant
	
	This library is free software; you can redistribute it or
	modify it under the terms of the GNU Lesser Public License
	as published by the Free Software Foundation; either version
	2.1 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU Lesser Public License for details.

    You should have received a copy of the GNU Lesser Public
	License along with this library; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// Mac includes
#ifndef __MACTYPES_
#	include <MacTypes.h>
#endif

// library includes
#include "DataAccessLib.h"



//
// public methods
//

/**
Using the Gestalt() system call to obtain
version information returns a garbled,
long integer containing a ton of information.
To acquire the major revision number only
from a messy Gestalt version long integer,
call this method.

ex. Mac OS 8.5.1
           ^
		major revision number

(3.0)
*/
UInt8
Releases_GetMajorRevisionForVersion 	(long		inVersion)
{
	long		scrap = ((inVersion >> 8) & 0x000000FF);
	UInt8		result = (scrap);
	
	
	return result;
}// GetMajorRevisionForVersion


/**
Using the Gestalt() system call to obtain
version information returns a garbled,
long integer containing a ton of information.
To acquire the minor revision number only
from a messy Gestalt version long integer,
call this method.

ex. Mac OS 8.5.1
             ^
		minor revision number

(3.0)
*/
UInt8
Releases_GetMinorRevisionForVersion 	(long		inVersion)
{
	long		scrap = ((inVersion >> 4) & 0x0000000F);
	UInt8		result = (scrap);
	
	
	return result;
}// GetMinorRevisionForVersion


/**
Using the Gestalt() system call to obtain
version information returns a garbled,
long integer containing a ton of information.
To acquire the superminor revision number only
from a messy Gestalt version long integer,
call this method.

ex. Mac OS 8.5.1
               ^
		superminor revision number

(3.0)
*/
UInt8
Releases_GetSuperminorRevisionForVersion 	(long		inVersion)
{
	long		scrap = ((inVersion >> 0) & 0x0000000F);
	UInt8		result = (scrap);
	
	
	return result;
}// GetSuperminorRevisionForVersion