/*###############################################################

	ResourceFileManager.c
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// Mac includes
#ifndef __FOLDERS__
#	include <Folders.h>
#endif

// MacTelnet includes
#include "ResourceFileManager.h"



// variables
static SInt16		gResourceFileApplication = -1, // IMPORTANT - initialize to an INVALID file descriptor
					gResourceFileLocalizationData = -1,
					gResourceFilePreferences = -1,
					gResourceFileTerminalBellSound = -1;



//
// public methods
//

/*!
To determine a resource files reference number
given its resource file descriptor, use this
method.

See also ResourceFileManager_UseResFile(), a
convenient routine for changing the current
resource file.

(3.0)
*/
SInt16
ResourceFileManager_GetResFile		(const TelnetResourceFile		inResourceFileType)
{
	SInt16		result = -1; // IMPORTANT - initialize to an INVALID file descriptor
	
	
	switch (inResourceFileType)
	{
		case kTelnetResourceFileApplication:
			result = gResourceFileApplication;
			break;
		
		case kTelnetResourceFilePreferences:
			result = gResourceFilePreferences;
			break;
		
		case kTelnetResourceFileTerminalBellSound:
			result = gResourceFileTerminalBellSound;
			break;
		
		default:
			// ???
			break;
	}
	return result;
}// GetResFile


/*!
To assign a resource file reference number to a
particular resource file descriptor, use this
method.

WARNING:	This is only done once for each type
			of resource file, at the start of
			your program when youve determined
			the descriptors of important open
			resource files.  Unless you use one
			or more Resource Manager routines to
			explicitly close and re-open your
			resource files, do not call this
			routine again for the same resource
			file type.

(3.0)
*/
void
ResourceFileManager_SetResFile		(const TelnetResourceFile		inResourceFileType,
									 const SInt16					inResourceFileRefNum)
{
	switch (inResourceFileType)
	{
		case kTelnetResourceFileApplication:
			gResourceFileApplication = inResourceFileRefNum;
			break;
		
		case kTelnetResourceFilePreferences:
			gResourceFilePreferences = inResourceFileRefNum;
			break;
		
		case kTelnetResourceFileTerminalBellSound:
			gResourceFileTerminalBellSound = inResourceFileRefNum;
			break;
		
		default:
			// ???
			break;
	}
}// SetResFile


/*!
Use this convenient method to update the
specified resource file (equivalent to calling
the Resource Manager function UpdateResFile()
on the appropriate file reference number).

(3.0)
*/
void
ResourceFileManager_UpdateResFile		(const TelnetResourceFile		inResourceFileType)
{
	UpdateResFile(ResourceFileManager_GetResFile(inResourceFileType));
}// UpdateResFile


/*!
Use this convenient method to change the
current resource file (equivalent to calling
the Resource Manager function UseResFile()
on the appropriate file reference number).

(3.0)
*/
void
ResourceFileManager_UseResFile		(const TelnetResourceFile		inResourceFileType)
{
	UseResFile(ResourceFileManager_GetResFile(inResourceFileType));
}// UseResFile