/*###############################################################

	ResourceFileManager.h
	
	To reduce the number of global variables in this program,
	and to more intelligently manage multiple open resource
	files, the Resource File Manager module has been created
	in Telnet 3.0.  It contains initialization routines for
	setting resource file reference numbers, which are then
	stored internally in this module.  You subsequently access
	these reference numbers using Telnets own descriptors,
	which utilize more abstract concepts like the preferences
	resource file or the application resource file.
	
	For convenience, there are special routines for setting
	the current resource file and updating a resource file,
	the two most common Resource Manager operations.  To use
	other Resource Manager routines, you use the method
	ResourceFileManager_GetResFile(), which gives you the file
	reference number of the specified open resource file.
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __RESOURCEFILEMANAGER__
#define __RESOURCEFILEMANAGER__

// Mac includes
#ifndef __MACTYPES__
#	include <MacTypes.h>
#endif
#ifndef __MACWINDOWS__
#	include <MacWindows.h>
#endif



// constants
typedef FourCharCode TelnetResourceFile;
enum
{
	kTelnetResourceFileApplication = 'TRFA',
	kTelnetResourceFilePreferences = 'TRFP',
	kTelnetResourceFileTerminalBellSound = 'TRF'
};



//
// public methods
//

SInt16
	ResourceFileManager_GetResFile		(const TelnetResourceFile		inResourceFileType);

void
	ResourceFileManager_SetResFile		(const TelnetResourceFile		inResourceFileType,
										 const SInt16					inResourceFileRefNum);

void
	ResourceFileManager_UpdateResFile	(const TelnetResourceFile		inResourceFileType);

void
	ResourceFileManager_UseResFile		(const TelnetResourceFile		inResourceFileType);

#endif