/*
**************************************************************************
	SpacingConstants.r
	
	 1998-2000 by Kevin Grant
	
	This file contains constants for standard dimensions and spacings
	for controls.  They are largely based on the Apple Human Interface
	Guidelines and the Aqua Layout Guidelines.  They are used both in
	resource files to set up window and dialog box controls consistently,
	and in source files, in rare cases (such as the Preferences window)
	where controls are set up programmatically.
	
	All of the measurements are in pixels.  The following conventions
	should be noticeable:
	- constants containing "_WD" represent widths
	- constants containing "_HT" represent heights
	- constants prefixed by "HSP_" represent horizontal-only spacings
	- constants prefixed by "VSP_" represent vertical-only spacings
	- constants containing "CUSHION" or "SPACE_EXTRA" are 2D spacings
**************************************************************************
*/

// standard control dimensions
#define BUTTON_WD_MINIMUM				75					/* push button minimum acceptable width, and standard height (Aqua) */
#define BUTTON_HT						20
#define BUTTON_HT_SMALL					18
#define CHASING_ARROWS_WD				16					/* asynchronous arrow animations, which are always the same size */
#define CHASING_ARROWS_HT				16
#define CHECKBOX_HT						21
#define COLORBOX_WD						48
#define COLORBOX_HT						22
#define DISCLOSURETRIANGLE_WD			12
#define DISCLOSURETRIANGLE_HT			12
#define ICON_WD_SMALL					16					/* small icons */
#define ICON_HT_SMALL					16
#define ICON_WD_BIG						32					/* standard icons */
#define ICON_HT_BIG						32
#define ICON_WD_HUGE					48					/* huge icons */
#define ICON_HT_HUGE					48
#define LITTLEARROWS_WD					13					/* double-arrow-button controls, which are always the same size */
#define LITTLEARROWS_HT					23
#define POPUPMENUBUTTONONLY_WD			22
#define POPUPMENUBUTTON_HT				21
#define PROGRESSBAR_HT					12					/* thermometer controls, which are always the same height */
#define RADIOBUTTON_HT					CHECKBOX_HT
#define SCROLL_BAR_WD					16
#define SCROLL_BAR_HT					SCROLL_BAR_WD
#define SEPARATOR_WD					4					/* horizontal or vertical etched lines, which are always the same thickness */
#define SEPARATOR_HT					SEPARATOR_WD
#define SLIDER_WD						18
#define SLIDER_HT						SLIDER_WD
#define TAB_HT_BIG						22					/* necessary clearance to place controls below a tab */
#define TAB_HT_SMALL					16
#define TEXT_HT_BIG						16					/* static or editable text, system font */
#define TEXT_HT_HUGE					24					/* static or editable text, system font in large print */
#define TEXT_HT_SMALL					14					/* static or editable text, small font */

// cushions
#define SPACE_EXTRA_CONTAINER			3					/* difference between the perceived edges and the actual (outset) edges */
#define SPACE_EXTRA_TEXTEDIT			3					/* difference between the control edges and the visible (outset) edges */
#define SPACE_EXTRA_TEXT_TOP			2					/* necessary additional space for static text below some kind of top edge */
#define MINIMUM_BUTTON_TITLE_CUSHION	10					/* horizontal space between a buttons text title and its outer edges */
#define MINIMUM_PLACARD_CUSHION			2					/* space between a placards edges and its content, ignoring bevels */
#define MINIMUM_TICK_MARKS_CUSHION		6					/* space padded to a sliders size to make room for tick marks */

// spacings between controls and dialog box edges
#define HSP_BUTTON_AND_DIALOG			20					/* space between buttons edges and dialog box interior */
#define VSP_BUTTON_AND_DIALOG			14
#define HSP_CONTROL_AND_DIALOG			14					/* spacing between most other controls edges and dialog boxs interior */
#define VSP_CONTROL_AND_DIALOG			HSP_CONTROL_AND_DIALOG
#define HSP_GROUPBOX_AND_DIALOG			20					/* space between group boxs edge and dialog box interior */
#define VSP_GROUPBOX_AND_DIALOG			20
#define HSP_TAB_AND_DIALOG				8					/* space between top/left/right tab control edges and dialog box interior */
#define VSP_TAB_AND_DIALOG				13
#define HSP_TEXT_AND_DIALOG				18					/* space between static text control edges and dialog box interior */
#define VSP_TEXT_AND_DIALOG				HSP_TEXT_AND_DIALOG

// spacings between controls of the same type
#define HSP_BUTTONS						12					/* spacing between two push button controls */
#define VSP_BUTTONS						10
#define HSP_CHECKBOXES					HSP_BUTTONS			/* spacing between two checkboxes */
#define VSP_CHECKBOXES					6
#define HSP_CONTROLS					10					/* spacing between two typical controls */
#define VSP_CONTROLS					6
#define HSP_EDITABLETEXTAREAS			(2 * SPACE_EXTRA_TEXTEDIT + HSP_CONTROLS)
#define VSP_EDITABLETEXTAREAS			(2 * SPACE_EXTRA_TEXTEDIT + VSP_CONTROLS)
#define HSP_GROUPBOXES					10					/* spacing between two group boxes */
#define VSP_GROUPBOXES					6
#define HSP_POPUPMENUBUTTONS			4					/* spacing between two pop-up menu buttons */
#define VSP_POPUPMENUBUTTONS			6
#define HSP_RADIOS						HSP_CHECKBOXES		/* spacing between two radio buttons */
#define VSP_RADIOS						VSP_CHECKBOXES

// spacings between controls of different types
#define HSP_BUTTON_AND_CONTROL			12					/* spacing between a push button and some non-button control */
#define VSP_BUTTON_AND_CONTROL			12
#define VSP_BUTTON_AND_TEXT_LABEL		4					/* space between a bevel button and its small-font text label beneath */
#define VSP_GROUPCHECKBOX_AND_CONTROL	27					/* spacing between the top of a checkbox-groupbox and its interior */
#define VSP_GROUPCHECKBOX_AND_TEXT		(VSP_GROUPCHECKBOX_AND_CONTROL + SPACE_EXTRA_TEXT_TOP)
#define VSP_GROUPPOPUP_AND_CONTROL		27					/* spacing between the top of a pop-up-menu-groupbox and its interior */
#define VSP_GROUPPOPUP_AND_TEXT			(VSP_GROUPPOPUP_AND_CONTROL + SPACE_EXTRA_TEXT_TOP)
#define VSP_GROUPBOXTEXT_AND_CONTROL	24					/* spacing between the top of a text-title-groupbox and its interior */
#define VSP_GROUPBOXTEXT_AND_TEXT		(VSP_GROUPBOXTEXT_AND_CONTROL + SPACE_EXTRA_TEXT_TOP)
#define HSP_GROUPBOX_AND_CONTROL		15					/* spacing between other edges of a groupbox and its interior */
#define VSP_GROUPBOX_AND_CONTROL		HSP_GROUPBOX_AND_CONTROL
#define VSP_GROUPBOX_AND_TEXT			(VSP_GROUPBOX_AND_CONTROL + SPACE_EXTRA_TEXT_TOP)
#define HSP_GROUPBOX_AND_HELPTEXT		5					/* spacing between a secondary group box and interior help text */
#define VSP_GROUPBOX_AND_HELPTEXT		HSP_GROUPBOX_AND_HELPTEXT
#define HSP_ICON_AND_TEXT				4					/* spacing between a small icon and associated text */
#define HSP_PANE_AND_CONTROL			2					/* spacing between an invisible pane (like a panel) and its interior */
#define VSP_PANE_AND_CONTROL			2
#define HSP_TABS_AND_CONTROL			HSP_CONTROLS		/* spacing between a set of dialog box tabs and anything around the tabs */
#define VSP_TABS_AND_CONTROL			16
#define HSP_TABPANE_AND_TAB				12					/* spacing between a tab controls interior and its invisible pane */
#define VSP_TABPANE_AND_TAB				12
#define HSP_TRIANGLE_AND_TEXT_LABEL		4					/* space between a disclosure triangle and its small-font text label */

// standard help balloon offsets
#define BUTTON_BALLOON_TIP_H			60
#define BUTTON_BALLOON_TIP_V			(BUTTON_HT / 2)
#define SMALL_BUTTON_BALLOON_TIP_H		15
#define SMALL_BUTTON_BALLOON_TIP_V		(BUTTON_HT / 2)
#define POPUPMENUBUTTON_BALLOON_TIP_H	10
#define POPUPMENUBUTTON_BALLOON_TIP_V	(BUTTON_HT / 2)
#define CHECKBOX_BALLOON_TIP_H			40
#define CHECKBOX_BALLOON_TIP_V			(CHECKBOX_HT / 2)
#define RADIOBUTTON_BALLOON_TIP_H		CHECKBOX_BALLOON_TIP_H
#define RADIOBUTTON_BALLOON_TIP_V		(RADIOBUTTON_HT / 2)
#define HELPBUTTON_BALLOON_TIP_H		15
#define HELPBUTTON_BALLOON_TIP_V		5
#define LITTLEARROWS_BALLOON_TIP_H		(LITTLEARROWS_WD / 2)
#define LITTLEARROWS_BALLOON_TIP_V		(LITTLEARROWS_HT / 2)
#define LIST_BALLOON_TIP_H				50
#define LIST_BALLOON_TIP_V				50
#define TEXT_BIG_BALLOON_TIP_H			25
#define TEXT_BIG_BALLOON_TIP_V			(TEXT_HT_BIG / 2)
#define TEXT_SMALL_BALLOON_TIP_H		15
#define TEXT_SMALL_BALLOON_TIP_V		(TEXT_HT_SMALL / 2)