/*
**************************************************************************
	TelnetFormatDialog.r
	
	MacTelnet 3.0
	 1998-2000 by Kevin Grant
	
	This file defines the Format Terminal dialog box.
**************************************************************************
*/
#ifdef REZ

#include "Appearance.r"
#include "Balloons.r"
#include "Controls.r"
#include "ControlDefinitions.r"
#include "Dialogs.r"
#include "MacTypes.r"
#include "Windows.r"

#endif

#include "ControlResources.h"
#include "DialogResources.h"
#include "MenuResources.h"
#include "InterfaceLib.h"
#include "SpacingConstants.r"
#ifndef __TELNETBALLOONS_R__
#	include "TelnetBalloons.r"
#endif
#include "TelnetButtonNames.h"


/*
*******************************************
	FORMAT DIALOG BOX
*******************************************
*/

/* the entire dialog box area */
#define FORMAT_DIALOG_WD		280
#define	FORMAT_DIALOG_HT		250

#ifdef REZ

#define DIALOG_WD	FORMAT_DIALOG_WD
#define DIALOG_HT	FORMAT_DIALOG_HT

/* Edit group box dimensions */
#define POPUPEDITMENU_TITLE_WD			40
#define EDIT_MENU_GROUP_B				(DIALOG_HT - VSP_BUTTON_AND_DIALOG - BUTTON_HT - VSP_BUTTON_AND_CONTROL - TEXT_HT_SMALL - VSP_CONTROLS)
#define EDIT_MENU_GROUP_T				VSP_GROUPBOX_AND_DIALOG
#define EDIT_MENU_GROUP_L				HSP_GROUPBOX_AND_DIALOG
#define EDIT_MENU_GROUP_R				(DIALOG_WD - HSP_GROUPBOX_AND_DIALOG)

/* helpful values */
#define EDIT_MENU_GROUP_INTERIOR_L		(EDIT_MENU_GROUP_L + HSP_GROUPBOX_AND_CONTROL)
#define EDIT_MENU_GROUP_INTERIOR_WD		(EDIT_MENU_GROUP_R - EDIT_MENU_GROUP_L - 2 * HSP_GROUPBOX_AND_CONTROL)

/* other controls dimensions */
#define BOTTOM_CHECKBOX_T				(EDIT_MENU_GROUP_B + VSP_CONTROLS)
#define BOTTOM_CHECKBOX_L				HSP_CONTROL_AND_DIALOG
#define BOTTOM_CHECKBOX_B				(BOTTOM_CHECKBOX_T + CHECKBOX_HT)
#define BOTTOM_CHECKBOX_R				(DIALOG_WD - HSP_CONTROL_AND_DIALOG)

// font group
#define POPUPFONTMENU_T					(EDIT_MENU_GROUP_T + VSP_GROUPPOPUP_AND_CONTROL)
#define POPUPFONTMENU_L					(EDIT_MENU_GROUP_INTERIOR_L - 3)
#define POPUPFONTMENU_WD				EDIT_MENU_GROUP_INTERIOR_WD
#define POPUPFONTMENU_TITLE_WD			60
#define FONT_ITEMS_CONTAINER_T			(POPUPFONTMENU_T - 2)
#define FONT_ITEMS_CONTAINER_L			(POPUPFONTMENU_L - 2)
#define FONT_ITEMS_CONTAINER_B			(FONT_ITEMS_CONTAINER_T + BUTTON_HT + 4)
#define FONT_ITEMS_CONTAINER_R			(FONT_ITEMS_CONTAINER_L + EDIT_MENU_GROUP_INTERIOR_WD + 4)

// font size group
#define FONTSIZE_LABEL_T				(POPUPFONTMENU_T + BUTTON_HT + VSP_CONTROLS + SPACE_EXTRA_TEXTEDIT)
#define FONTSIZE_LABEL_L				EDIT_MENU_GROUP_INTERIOR_L
#define FONTSIZE_LABEL_B				(FONTSIZE_LABEL_T + TEXT_HT_BIG)
#define FONTSIZE_LABEL_R				(FONTSIZE_LABEL_L + POPUPFONTMENU_TITLE_WD - SPACE_EXTRA_TEXTEDIT - 2)
#define FONTSIZE_FIELD_T				FONTSIZE_LABEL_T
#define FONTSIZE_FIELD_L				(POPUPFONTMENU_L + POPUPFONTMENU_TITLE_WD + SPACE_EXTRA_TEXTEDIT)
#define FONTSIZE_FIELD_B				(FONTSIZE_FIELD_T + TEXT_HT_BIG)
#define FONTSIZE_FIELD_R				(FONTSIZE_FIELD_L + 50)
#define FONTSIZE_ARROWS_T				(FONTSIZE_LABEL_T + (TEXT_HT_BIG - LITTLEARROWS_HT) / 2)
#define FONTSIZE_ARROWS_L				(FONTSIZE_FIELD_R + SPACE_EXTRA_TEXTEDIT + HSP_CONTROLS)
#define FONTSIZE_ARROWS_B				(FONTSIZE_ARROWS_T + LITTLEARROWS_HT)
#define FONTSIZE_ARROWS_R				(FONTSIZE_ARROWS_L + LITTLEARROWS_WD)
#define FONTSIZE_POPUPMENU_T			(FONTSIZE_LABEL_T + (TEXT_HT_BIG - BUTTON_HT) / 2)
#define FONTSIZE_POPUPMENU_L			(FONTSIZE_ARROWS_R + HSP_CONTROLS)
#define FONTSIZE_POPUPMENU_B			(FONTSIZE_POPUPMENU_T + BUTTON_HT)
#define FONTSIZE_POPUPMENU_R			(FONTSIZE_POPUPMENU_L + POPUPMENUBUTTONONLY_WD)
#define FONTSIZE_ITEMS_CONTAINER_T		(FONTSIZE_ARROWS_T - 2)
#define FONTSIZE_ITEMS_CONTAINER_L		(FONTSIZE_LABEL_L - 2)
#define FONTSIZE_ITEMS_CONTAINER_B		(FONTSIZE_FIELD_T + TEXT_HT_BIG + SPACE_EXTRA_TEXTEDIT + 4)
#define FONTSIZE_ITEMS_CONTAINER_R		(FONTSIZE_ITEMS_CONTAINER_L + EDIT_MENU_GROUP_INTERIOR_WD + 4)

// foreground color group
#define COLORBOX_FOREGROUND_T			(FONTSIZE_LABEL_B + SPACE_EXTRA_TEXTEDIT + VSP_BUTTONS)
#define COLORBOX_FOREGROUND_L			EDIT_MENU_GROUP_INTERIOR_L
#define COLORBOX_FOREGROUND_LABEL_T		(COLORBOX_FOREGROUND_T + (COLORBOX_HT - TEXT_HT_BIG) / 2)
#define COLORBOX_FOREGROUND_LABEL_L		(EDIT_MENU_GROUP_INTERIOR_L + COLORBOX_WD + HSP_CONTROLS)
#define FONTFORECOLOR_ITEMS_CONTAINER_T	(COLORBOX_FOREGROUND_T - 2)
#define FONTFORECOLOR_ITEMS_CONTAINER_L	(COLORBOX_FOREGROUND_L - 2)
#define FONTFORECOLOR_ITEMS_CONTAINER_B	(FONTFORECOLOR_ITEMS_CONTAINER_T + BUTTON_HT + 4)
#define FONTFORECOLOR_ITEMS_CONTAINER_R	(FONTFORECOLOR_ITEMS_CONTAINER_L + EDIT_MENU_GROUP_INTERIOR_WD + 4)

// background color group
#define COLORBOX_BACKGROUND_T			(COLORBOX_FOREGROUND_T + COLORBOX_HT + VSP_BUTTONS)
#define COLORBOX_BACKGROUND_L			COLORBOX_FOREGROUND_L
#define COLORBOX_BACKGROUND_LABEL_T		(COLORBOX_BACKGROUND_T + (COLORBOX_HT - TEXT_HT_BIG) / 2)
#define COLORBOX_BACKGROUND_LABEL_L		COLORBOX_FOREGROUND_LABEL_L
#define FONTBACKCOLOR_ITEMS_CONTAINER_T	(COLORBOX_BACKGROUND_T - 2)
#define FONTBACKCOLOR_ITEMS_CONTAINER_L	(COLORBOX_BACKGROUND_L - 2)
#define FONTBACKCOLOR_ITEMS_CONTAINER_B	(FONTBACKCOLOR_ITEMS_CONTAINER_T + BUTTON_HT + 4)
#define FONTBACKCOLOR_ITEMS_CONTAINER_R	(FONTBACKCOLOR_ITEMS_CONTAINER_L + EDIT_MENU_GROUP_INTERIOR_WD + 4)



resource 'DLOG' (kDialogIDFormat, FORMAT_WINDOWNAME, purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowMovableModalGrowProc,
	invisible,
	0, // close box
	0,
	kDialogIDFormat,
	FORMAT_WINDOWNAME,
	centerParentWindow
};

resource 'dlgx' (kDialogIDFormat)
{
	versionZero
	{
		kDialogFlagsUseThemeBackground		|
		kDialogFlagsUseControlHierarchy		|
		kDialogFlagsHandleMovableModal		|
		kDialogFlagsUseThemeControls
	}
};

resource 'DITL' (kDialogIDFormat, FORMAT_WINDOWNAME, purgeable)
{
	{
		// 1
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDOKButton },
		
		// 2
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDCancelButton },
		
		// 3
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDAppleGuideButton },
		
		// 4
		{ BOTTOM_CHECKBOX_T, BOTTOM_CHECKBOX_L, -1, -1 },
		Control { enabled, kIDCheckBox },
		
		// 5
		{ EDIT_MENU_GROUP_T, EDIT_MENU_GROUP_L, -1, -1 },
		Control { enabled, kIDFormatEditMenuPrimaryGroup },
		
		// 6
		{ FONT_ITEMS_CONTAINER_T, FONT_ITEMS_CONTAINER_L, -1, -1 },
		Control { disabled, kIDFormatEditMenuFontItemsPane },
		
		// 7
		{ POPUPFONTMENU_T, POPUPFONTMENU_L, -1, -1 },
		Control { enabled, kIDFormatEditFontPopUpMenu },
		
		// 8
		{ FONTSIZE_ITEMS_CONTAINER_T, FONTSIZE_ITEMS_CONTAINER_L, -1, -1 },
		Control { disabled, kIDFormatEditMenuFontSizeItemsPane },
		
		// 9
		{ FONTSIZE_LABEL_T, FONTSIZE_LABEL_L, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 10
		{ FONTSIZE_FIELD_T, FONTSIZE_FIELD_L, -1, -1 },
		Control { disabled, kIDFormatEditSizeField },
		
		// 11
		{ FONTSIZE_ARROWS_T, FONTSIZE_ARROWS_L, -1, -1 },
		Control { enabled, kIDLittleArrows },
		
		// 12
		{ FONTSIZE_POPUPMENU_T, FONTSIZE_POPUPMENU_L, -1, -1 },
		Control { enabled, kIDFormatEditSizePopUpMenu },
		
		// 13
		{ FONTFORECOLOR_ITEMS_CONTAINER_T, FONTFORECOLOR_ITEMS_CONTAINER_L,
			FONTFORECOLOR_ITEMS_CONTAINER_B, FONTFORECOLOR_ITEMS_CONTAINER_R },
		Control { disabled, kIDFormatEditMenuForeColorItemsPane },
		
		// 14
		{ COLORBOX_FOREGROUND_T, COLORBOX_FOREGROUND_L, -1, -1 },
		Control { enabled, kIDColorBox },
		
		// 15
		{ COLORBOX_FOREGROUND_LABEL_T, COLORBOX_FOREGROUND_LABEL_L, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 16
		{ FONTBACKCOLOR_ITEMS_CONTAINER_T, FONTBACKCOLOR_ITEMS_CONTAINER_L,
			FONTBACKCOLOR_ITEMS_CONTAINER_B, FONTBACKCOLOR_ITEMS_CONTAINER_R },
		Control { disabled, kIDFormatEditMenuBackColorItemsPane },
		
		// 17
		{ COLORBOX_BACKGROUND_T, COLORBOX_BACKGROUND_L, -1, -1 },
		Control { enabled, kIDColorBox },
		
		// 18
		{ COLORBOX_BACKGROUND_LABEL_T, COLORBOX_BACKGROUND_LABEL_L, -1, -1 },
		Control { disabled, kIDStaticText }
	}
};

resource 'hdlg' (kDialogBalloonsIDFormat, purgeable)
{
	HelpMgrVersion,
	0, // zero-based dialog item number of first item that has a help balloon
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMSkipItem {/* no help */}, // 1
		HMStringResItem // 2
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_CANCEL,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 3
		{
			{ HELPBUTTON_BALLOON_TIP_V, HELPBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_HELP,
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_DISABLED_BUTTON_HELP,
			0, 0,
			0, 0
		},
		HMStringResItem // 4
		{
			{ CHECKBOX_BALLOON_TIP_V, CHECKBOX_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericSessions, BALLOON_GENERIC_CONNECTIONS_UNCHECKED_CHECKBOX_UPDATEDEFTERM,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 5
		{
			{ POPUPMENUBUTTON_BALLOON_TIP_V, POPUPMENUBUTTON_BALLOON_TIP_H + POPUPEDITMENU_TITLE_WD }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_ENABLED_POPUPMENU_EDIT_WHAT,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 6
		HMStringResItem // 7
		{
			{ POPUPMENUBUTTON_BALLOON_TIP_V, POPUPMENUBUTTON_BALLOON_TIP_H + POPUPFONTMENU_TITLE_WD }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_ENABLED_POPUPMENU_FONT,
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_DISABLED_POPUPMENU_FONT,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 8
		HMSkipItem {/* no help */}, // 9
		HMStringResItem // 10
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_ENABLED_FIELD_FONTSIZE,
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_DISABLED_FIELD_FONTSIZE,
			0, 0,
			0, 0
		},
		HMStringResItem // 11
		{
			{ LITTLEARROWS_BALLOON_TIP_V, LITTLEARROWS_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_ENABLED_LITTLEARROWS_FONTSIZE,
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_DISABLED_LITTLEARROWS_FONTSIZE,
			0, 0,
			0, 0
		},
		HMStringResItem // 12
		{
			{ POPUPMENUBUTTON_BALLOON_TIP_V, POPUPMENUBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_ENABLED_POPUPMENU_FONTSIZE,
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_DISABLED_POPUPMENU_FONTSIZE,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 13
		HMStringResItem // 14
		{
			{ SMALL_BUTTON_BALLOON_TIP_V, SMALL_BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_ENABLED_COLORBOX_FOREGROUND,
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_DISABLED_COLORBOX_FOREGROUND,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 15
		HMSkipItem {/* no help */}, // 16
		HMStringResItem // 17
		{
			{ SMALL_BUTTON_BALLOON_TIP_V, SMALL_BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_ENABLED_COLORBOX_BACKGROUND,
			rStringsBalloonTextFormatDialog, BALLOON_DIALOG_FORMAT_DISABLED_COLORBOX_BACKGROUND,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */} // 18
	}
};

resource 'dftb' (kDialogIDFormat, FORMAT_WINDOWNAME": Font Auxiliary Info")
{
	versionZero
	{
		{
			skipItem { }, // 1
			skipItem { }, // 2
			skipItem { }, // 3
			skipItem { }, // 4
			skipItem { }, // 5
			skipItem { }, // 6
			skipItem { }, // 7
			skipItem { }, // 8
			skipItem { }, // 9
			skipItem { }, // 10
			skipItem { }, // 11
			skipItem { }, // 12
			skipItem { }, // 13
			skipItem { }, // 14
			skipItem { }, // 15
			skipItem { }, // 16
			skipItem { }, // 17
			skipItem { } // 18
		}
	};
};

/* primary group boxes */

resource 'CNTL' (kIDFormatEditMenuPrimaryGroup, FORMAT_WINDOWNAME": Edit Pop-Up Menu Group Box", purgeable)
{
	{ 0, 0, EDIT_MENU_GROUP_B - EDIT_MENU_GROUP_T, EDIT_MENU_GROUP_R - EDIT_MENU_GROUP_L },
	popupTitleLeftJust, // see page 5-119 of IM:MTE for valid values
	visible,
	POPUPEDITMENU_TITLE_WD, // title width in pixels
	kPopUpMenuIDFormatType, // menu resource ID
	kControlGroupBoxPopupButtonProc | kControlPopupVariableWidthVariant,
	0,
	FORMAT_POPUPNAME_EDIT
};

/* Edit group box controls */

// this control is used to embed all font-related controls together, so one DeactivateControl() call can be used
resource 'CNTL' (kIDFormatEditMenuFontItemsPane, FORMAT_WINDOWNAME": Container For Font Controls", purgeable)
{
	{ 0, 0, FONT_ITEMS_CONTAINER_B - FONT_ITEMS_CONTAINER_T, FONT_ITEMS_CONTAINER_R - FONT_ITEMS_CONTAINER_L },
	kControlSupportsEmbedding, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

// this control is used to embed all font size-related controls together, so one DeactivateControl() call can be used
resource 'CNTL' (kIDFormatEditMenuFontSizeItemsPane, FORMAT_WINDOWNAME": Container For Font Size Controls", purgeable)
{
	{ 0, 0, FONTSIZE_ITEMS_CONTAINER_B - FONTSIZE_ITEMS_CONTAINER_T,
		FONTSIZE_ITEMS_CONTAINER_R - FONTSIZE_ITEMS_CONTAINER_L },
	kControlSupportsEmbedding, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

// this control is used to embed all foreground color-related controls together, so one DeactivateControl() call can be used
resource 'CNTL' (kIDFormatEditMenuForeColorItemsPane, FORMAT_WINDOWNAME": Container For Foreground Color Controls", purgeable)
{
	{ 0, 0, FONTFORECOLOR_ITEMS_CONTAINER_B - FONTFORECOLOR_ITEMS_CONTAINER_T,
		FONTFORECOLOR_ITEMS_CONTAINER_R - FONTFORECOLOR_ITEMS_CONTAINER_L },
	kControlSupportsEmbedding, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

// this control is used to embed all background color-related controls together, so one DeactivateControl() call can be used
resource 'CNTL' (kIDFormatEditMenuBackColorItemsPane, FORMAT_WINDOWNAME": Container For Background Color Controls", purgeable)
{
	{ 0, 0, FONTBACKCOLOR_ITEMS_CONTAINER_B - FONTBACKCOLOR_ITEMS_CONTAINER_T,
		FONTBACKCOLOR_ITEMS_CONTAINER_R - FONTBACKCOLOR_ITEMS_CONTAINER_L },
	kControlSupportsEmbedding, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

resource 'CNTL' (kIDFormatEditFontPopUpMenu, FORMAT_WINDOWNAME": Font Pop-Up Menu", purgeable)
{
	{ 0, 0, BUTTON_HT, POPUPFONTMENU_WD },
	popupTitleLeftJust, // see page 5-119 of IM:MTE for valid values
	visible,
	POPUPFONTMENU_TITLE_WD, // title width in pixels
	kPopUpMenuIDFont, // menu resource ID
	kControlPopupButtonProc | kControlPopupFixedWidthVariant,
	0,
	FORMAT_POPUPNAME_FONT
};

resource 'CNTL' (kIDFormatEditSizeLabel, FORMAT_WINDOWNAME": Size Label", purgeable)
{
	{ 0, 0, FONTSIZE_LABEL_B - FONTSIZE_LABEL_T, FONTSIZE_LABEL_R - FONTSIZE_LABEL_L },
	0,
	visible,
	0,
	0,
	kControlStaticTextProc,
	0,
	""
};

resource 'CNTL' (kIDFormatEditSizeField, FORMAT_WINDOWNAME": Size Field", purgeable)
{
	{ 0, 0, FONTSIZE_FIELD_B - FONTSIZE_FIELD_T, FONTSIZE_FIELD_R - FONTSIZE_FIELD_L },
	0,
	visible,
	0,
	0,
	kControlEditTextProc,
	0,
	""
};

resource 'CNTL' (kIDFormatEditSizePopUpMenu, FORMAT_WINDOWNAME": Size Pop-Up Menu", purgeable)
{
	{ 0, 0, FONTSIZE_POPUPMENU_B - FONTSIZE_POPUPMENU_T, FONTSIZE_POPUPMENU_R - FONTSIZE_POPUPMENU_L },
	popupTitleLeftJust, // see page 5-119 of IM:MTE for valid values
	visible,
	0, // title width in pixels
	kPopUpMenuIDSize, // menu resource ID
	kControlPopupButtonProc | kControlPopupFixedWidthVariant,
	0,
	""
};

resource 'CNTL' (kIDFormatEditForegroundColorLabel, FORMAT_WINDOWNAME": Foreground Color Label", purgeable)
{
	{ 0, 0, TEXT_HT_BIG, BOTTOM_CHECKBOX_R - BOTTOM_CHECKBOX_L },
	0,
	visible,
	0,
	0,
	kControlStaticTextProc,
	0,
	""
};

resource 'CNTL' (kIDFormatEditBackgroundColorLabel, FORMAT_WINDOWNAME": Background Color Label", purgeable)
{
	{ 0, 0, TEXT_HT_BIG, BOTTOM_CHECKBOX_R - BOTTOM_CHECKBOX_L },
	0,
	visible,
	0,
	0,
	kControlStaticTextProc,
	0,
	""
};

#endif
