/*
**************************************************************************
	TelnetHelpLookup.r
	
	MacTelnet 3.0
	 1998-2000 by Kevin Grant
	
	This file contains lookup keywords for MacTelnet Help, which are used
	to implement dialog box help buttons.  The name of each 'TEXT'
	resource is the keyword, and the content is the path, relative to the
	MacTelnet Help folder, of the HTML file to display.
	
	These resources will be built into the MacTelnet Help guide file.
**************************************************************************
*/

#include "MacTypes.r"

#include "TelnetHelpKeyNames.h"



// the resource ID of the first 'TEXT' resource
#define HELP_LOOKUP_KEYWORD_BASE_ID		2000

// the path base of "MacTelnet Help"
#define PATH_HELP_BASE					":MacTelnet Help:"

// other paths, which are relative to the help base, above
#define PATH_SCRIPTING					"dc:"
#define PATH_DEFINITIONS				"df:"
#define PATH_INTRO						"in:"
#define PATH_SHORTCUTS					"sh:"
#define PATH_TUTORIALS					"tu:"

// the name of any directory containing HTML help files
#define PATH_PAGES						"pgs:"

// convenience definitions for complete paths to the HTML help content directories of all categories
#define INTRO_CATEGORY					PATH_HELP_BASE""PATH_INTRO""PATH_PAGES
#define SHORTCUTS_CATEGORY				PATH_HELP_BASE""PATH_SHORTCUTS""PATH_PAGES
#define TUTORIALS_CATEGORY				PATH_HELP_BASE""PATH_TUTORIALS""PATH_PAGES
#define DEFINITIONS_CATEGORY			PATH_HELP_BASE""PATH_DEFINITIONS""PATH_PAGES
#define SCRIPTING_CATEGORY				PATH_HELP_BASE""PATH_SCRIPTING""PATH_PAGES



// define the 'TEXT' resource as a simple string; pathnames to help files will never be longer than a string anyway
#define HELP_KEY_VALUE_RESOURCE			'TEXT'
type HELP_KEY_VALUE_RESOURCE { string; };



/*
**************************************************************************
	APPLE HELP DEFINITIONS
	
	These resources allow the guide file to be recognized by Apple Help.
**************************************************************************
*/

type 'html' {};
resource 'html' (1000)
{
	// this is supposed to be an empty resource
};

resource 'TEXT' (1000, "MacTelnet Help"/* must match the Guide Maker source file specification for the help menu item */)
{
	":MacTelnet Help:MacTelnet Help" // must be the Help-folder-relative path of the help systems HTML file
};

#define MISSING_APPLICATION_MESSAGE "This file allows the item MacTelnet Help, which opens the MacOSHelpCenter, to appear in MacTelnets Help menu."

resource 'STR ' (-16397, purgeable)
{
	MISSING_APPLICATION_MESSAGE
};

type 'CCI' { string; };
resource 'CCI' (128, purgeable)
{
	MISSING_APPLICATION_MESSAGE
};

/*
**************************************************************************
	BEGIN KEY DEFINITIONS
	
	Every keys resource name must be the key itself, and the value
	is the content of the 'TEXT' resource.  The relative pathname of the
	HTML help file to open for a particular keyword must be specified as
	the value.  USE THE CONVENIENT CONSTANTS DEFINED EARLIER IN THIS
	FILE, DO NOT PLACE HARD-CODED PATHNAMES BEYOND THIS POINT!
**************************************************************************
*/

resource HELP_KEY_VALUE_RESOURCE (HELP_LOOKUP_KEYWORD_BASE_ID, HELPKEY_SESSIONS, purgeable)
{
	INTRO_CATEGORY"GetConnected.htm"
};

resource HELP_KEY_VALUE_RESOURCE (HELP_LOOKUP_KEYWORD_BASE_ID + 1, HELPKEY_MACROS, purgeable)
{
	""
};

resource HELP_KEY_VALUE_RESOURCE (HELP_LOOKUP_KEYWORD_BASE_ID + 2, HELPKEY_PREFERENCES, purgeable)
{
	INTRO_CATEGORY"subpgs:Preferences.htm"
};

resource HELP_KEY_VALUE_RESOURCE (HELP_LOOKUP_KEYWORD_BASE_ID + 3, HELPKEY_FORMATTING, purgeable)
{
	""
};

resource HELP_KEY_VALUE_RESOURCE (HELP_LOOKUP_KEYWORD_BASE_ID + 4, HELPKEY_COMMANDLINE, purgeable)
{
	""
};

resource HELP_KEY_VALUE_RESOURCE (HELP_LOOKUP_KEYWORD_BASE_ID + 5, HELPKEY_SCREENSIZE, purgeable)
{
	""
};

resource HELP_KEY_VALUE_RESOURCE (HELP_LOOKUP_KEYWORD_BASE_ID + 6, HELPKEY_FAVORITES, purgeable)
{
	""
};