/*
**************************************************************************
	TelnetMacrosDialog.r
	
	MacTelnet
		 1998-2003 by Kevin Grant.
		 2001-2003 by Ian Anderson.
		 1986-1994 University of Illinois Board of Trustees
		(see About box for full list of U of I contributors).
	
	This file defines the Macro Setup dialog box.
**************************************************************************
*/
#ifdef REZ

#include "Appearance.r"
#include "Balloons.r"
#include "Controls.r"
#include "ControlDefinitions.r"
#include "Dialogs.r"
#include "MacTypes.r"
#include "MacWindows.r"

#endif

#include "ControlResources.h"
#include "DialogResources.h"
#include "InterfaceLib.h"
#include "MenuResources.h"
#include "SpacingConstants.r"
#ifndef __TELNETBALLOONS_R__
#	include "TelnetBalloons.r"
#endif
#include "TelnetButtonNames.h"


/*
*******************************************
	MACROS DIALOG BOX
*******************************************
*/

/*
Relative positioning of macro buttons and text fields.
IMPORTANT:	See "DialogAdjust.c" and "MacroSetupDialog.c",
			which use these numbers.
*/
#define MACRO_LABEL_AND_FIELD_TOP_OFFSET	0
#define HSP_MACRO_LABEL_AND_FIELD			13
#define VSP_MACRO_LABELS					17
#define MACRO_LABEL_HT						TEXT_HT_BIG
#define MACRO_LABEL_WD						30
#define MACRO_FIELD_WD						195
#define ACTIVE_SET_CURRENT_BUTTON_WD		108
#define ACTIVE_SET_BEVELBUTTON_WD			28
#define ACTIVE_SET_BEVELBUTTON_HT			BUTTON_HT

/* the entire dialog box area */
#define MACROSETUP_DIALOG_WD		580
#define	MACROSETUP_DIALOG_HT		414

#ifdef REZ

#define DIALOG_WD	MACROSETUP_DIALOG_WD
#define DIALOG_HT	MACROSETUP_DIALOG_HT

/* Macro Keys group box dimensions */
#define MACROKEYS_GROUP_B	(DIALOG_HT - VSP_BUTTON_AND_DIALOG - BUTTON_HT - VSP_BUTTON_AND_CONTROL)
#define MACROKEYS_GROUP_T	(MACROKEYS_GROUP_B - VSP_GROUPBOX_AND_CONTROL - RADIOBUTTON_HT - VSP_GROUPBOXTEXT_AND_CONTROL)
#define MACROKEYS_GROUP_L	HSP_GROUPBOX_AND_DIALOG
#define MACROKEYS_GROUP_R	(MACROKEYS_GROUP_L + 220)

/* Active Set group box dimensions */
#define ACTIVESET_GROUP_B	MACROKEYS_GROUP_B
#define ACTIVESET_GROUP_T	MACROKEYS_GROUP_T
#define ACTIVESET_GROUP_L	(MACROKEYS_GROUP_R + HSP_GROUPBOXES)
#define ACTIVESET_GROUP_R	(DIALOG_WD - HSP_GROUPBOX_AND_DIALOG)

/* Macro Text Equivalents group box dimensions */
#define POPUPEDITMACROSMENU_TITLE_WD	55
#define MACROTEXTEQ_GROUP_T				VSP_GROUPBOX_AND_DIALOG
#define MACROTEXTEQ_GROUP_L				HSP_GROUPBOX_AND_DIALOG
#define MACROTEXTEQ_GROUP_B				(MACROKEYS_GROUP_T - VSP_CONTROLS)
#define MACROTEXTEQ_GROUP_R				(DIALOG_WD - HSP_GROUPBOX_AND_DIALOG)

/* data for specific controls */
#define EXPORT_BUTTON_WD					78
#define IMPORT_BUTTON_WD					78
#define MACROTEXTEQ_SEPARATOR_WIDTH			(MACROTEXTEQ_GROUP_R - HSP_GROUPBOX_AND_CONTROL - HSP_GROUPBOX_AND_CONTROL - MACROTEXTEQ_GROUP_L)

#define FIRST_BUTTON_COLUMN_LEFT			(MACROTEXTEQ_GROUP_L + HSP_GROUPBOX_AND_CONTROL)
#define SECOND_BUTTON_COLUMN_LEFT			(MACROTEXTEQ_GROUP_R - HSP_GROUPBOX_AND_CONTROL - MACRO_FIELD_WD - HSP_MACRO_LABEL_AND_FIELD - MACRO_LABEL_WD - SPACE_EXTRA_TEXTEDIT)
#define FIRST_BUTTON_ROW_TOP				(MACROTEXTEQ_GROUP_T + VSP_GROUPPOPUP_AND_CONTROL + BUTTON_HT + VSP_BUTTON_AND_CONTROL + SEPARATOR_HT + VSP_BUTTON_AND_CONTROL)
#define SECOND_BUTTON_ROW_TOP				(FIRST_BUTTON_ROW_TOP + MACRO_LABEL_HT + VSP_MACRO_LABELS)
#define THIRD_BUTTON_ROW_TOP				(SECOND_BUTTON_ROW_TOP + MACRO_LABEL_HT + VSP_MACRO_LABELS)
#define FOURTH_BUTTON_ROW_TOP				(THIRD_BUTTON_ROW_TOP + MACRO_LABEL_HT + VSP_MACRO_LABELS)
#define FIFTH_BUTTON_ROW_TOP				(FOURTH_BUTTON_ROW_TOP + MACRO_LABEL_HT + VSP_MACRO_LABELS)
#define SIXTH_BUTTON_ROW_TOP				(FIFTH_BUTTON_ROW_TOP + MACRO_LABEL_HT + VSP_MACRO_LABELS)

#define FIRST_FIELD_COLUMN_LEFT				(FIRST_BUTTON_COLUMN_LEFT + MACRO_LABEL_WD + HSP_MACRO_LABEL_AND_FIELD)
#define SECOND_FIELD_COLUMN_LEFT			(SECOND_BUTTON_COLUMN_LEFT + MACRO_LABEL_WD + HSP_MACRO_LABEL_AND_FIELD)
#define FIRST_FIELD_ROW_TOP					(FIRST_BUTTON_ROW_TOP + MACRO_LABEL_AND_FIELD_TOP_OFFSET)
#define SECOND_FIELD_ROW_TOP				(SECOND_BUTTON_ROW_TOP + MACRO_LABEL_AND_FIELD_TOP_OFFSET)
#define THIRD_FIELD_ROW_TOP					(THIRD_BUTTON_ROW_TOP + MACRO_LABEL_AND_FIELD_TOP_OFFSET)
#define FOURTH_FIELD_ROW_TOP				(FOURTH_BUTTON_ROW_TOP + MACRO_LABEL_AND_FIELD_TOP_OFFSET)
#define FIFTH_FIELD_ROW_TOP					(FIFTH_BUTTON_ROW_TOP + MACRO_LABEL_AND_FIELD_TOP_OFFSET)
#define SIXTH_FIELD_ROW_TOP					(SIXTH_BUTTON_ROW_TOP + MACRO_LABEL_AND_FIELD_TOP_OFFSET)

/* relative position of radio buttons */
#define MACRO_KEYS_RADIO_BUTTON_WD			90
#define MACRO_KEYS_RADIO_BUTTON_TOP			(MACROKEYS_GROUP_T + VSP_GROUPBOXTEXT_AND_CONTROL)
#define MACRO_KEYS_RADIO_BUTTON_FIRST_LEFT	(MACROKEYS_GROUP_L + HSP_GROUPBOX_AND_CONTROL)
#define MACRO_KEYS_RADIO_BUTTON_SECOND_LEFT	(MACRO_KEYS_RADIO_BUTTON_FIRST_LEFT + MACRO_KEYS_RADIO_BUTTON_WD + HSP_CONTROLS)

#define ACTIVE_SET_RADIO_BUTTON_TOP			(ACTIVESET_GROUP_T + VSP_GROUPBOXTEXT_AND_CONTROL)
#define ACTIVE_SET_RADIO_BUTTON_COL_DIFF	ACTIVE_SET_BEVELBUTTON_WD
#define ACTIVE_SET_RADIO_BUTTON_FIRST_LEFT	(ACTIVESET_GROUP_L + HSP_GROUPBOX_AND_CONTROL)
#define ACTIVE_SET_RADIO_BUTTON_SECOND_LEFT	(ACTIVE_SET_RADIO_BUTTON_FIRST_LEFT + ACTIVE_SET_RADIO_BUTTON_COL_DIFF)
#define ACTIVE_SET_RADIO_BUTTON_THIRD_LEFT	(ACTIVE_SET_RADIO_BUTTON_SECOND_LEFT + ACTIVE_SET_RADIO_BUTTON_COL_DIFF)
#define ACTIVE_SET_RADIO_BUTTON_FOURTH_LEFT	(ACTIVE_SET_RADIO_BUTTON_THIRD_LEFT + ACTIVE_SET_RADIO_BUTTON_COL_DIFF)
#define ACTIVE_SET_RADIO_BUTTON_FIFTH_LEFT	(ACTIVE_SET_RADIO_BUTTON_FOURTH_LEFT + ACTIVE_SET_RADIO_BUTTON_COL_DIFF)
#define ACTIVE_SET_RADIO_BUTTON_SIXTH_LEFT	(ACTIVE_SET_RADIO_BUTTON_FIFTH_LEFT + ACTIVE_SET_RADIO_BUTTON_COL_DIFF)
#define ACTIVE_SET_CURRENT_BUTTON_LEFT		(ACTIVE_SET_RADIO_BUTTON_SIXTH_LEFT + ACTIVE_SET_RADIO_BUTTON_COL_DIFF + HSP_BUTTONS)

resource 'DLOG' (kDialogIDMacroSetup, MACROSETUP_WINDOWNAME, purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowMovableModalGrowProc,
	invisible,
	0, // close box
	0,
	kDialogIDMacroSetup,
	MACROSETUP_WINDOWNAME,
	centerMainScreen
};

resource 'dlgx' (kDialogIDMacroSetup)
{
	versionZero
	{
		kDialogFlagsUseThemeBackground		|
		kDialogFlagsUseControlHierarchy		|
		kDialogFlagsHandleMovableModal		|
		kDialogFlagsUseThemeControls
	}
};

resource 'DITL' (kDialogIDMacroSetup, MACROSETUP_WINDOWNAME, purgeable)
{
	{
		// 1
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDOKButton },
		
		// 2
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDCancelButton },
		
		// 3
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDAppleGuideButton },
		
		// 4
		{ MACROTEXTEQ_GROUP_T, MACROTEXTEQ_GROUP_L, -1, -1 },
		Control { enabled, kIDMacroSetupMacroTextEquivalentsPrimaryGroup },
		
		// 5
		{	MACROTEXTEQ_GROUP_T + VSP_GROUPPOPUP_AND_CONTROL,
			FIRST_BUTTON_COLUMN_LEFT + EXPORT_BUTTON_WD + HSP_BUTTONS, -1, -1 },
		Control { enabled, kIDMacroSetupImportButton },
		
		// 6
		{ MACROTEXTEQ_GROUP_T + VSP_GROUPPOPUP_AND_CONTROL, FIRST_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupExportButton },
		
		// 7
		{	MACROTEXTEQ_GROUP_T + VSP_GROUPPOPUP_AND_CONTROL + BUTTON_HT + VSP_BUTTON_AND_CONTROL,
			MACROTEXTEQ_GROUP_L + HSP_GROUPBOX_AND_CONTROL, -1, -1 },
		Control { disabled, kIDMacroSetupMacroTextEquivalentsDivider },
		
		// 8
		{ FIRST_BUTTON_ROW_TOP, FIRST_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 9
		{ SECOND_BUTTON_ROW_TOP, FIRST_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 10
		{ THIRD_BUTTON_ROW_TOP, FIRST_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 11
		{ FOURTH_BUTTON_ROW_TOP, FIRST_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 12
		{ FIFTH_BUTTON_ROW_TOP, FIRST_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 13
		{ SIXTH_BUTTON_ROW_TOP, FIRST_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 14
		{ FIRST_BUTTON_ROW_TOP, SECOND_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 15
		{ SECOND_BUTTON_ROW_TOP, SECOND_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 16
		{ THIRD_BUTTON_ROW_TOP, SECOND_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 17
		{ FOURTH_BUTTON_ROW_TOP, SECOND_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 18
		{ FIFTH_BUTTON_ROW_TOP, SECOND_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 19
		{ SIXTH_BUTTON_ROW_TOP, SECOND_BUTTON_COLUMN_LEFT, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 20
		{ FIRST_FIELD_ROW_TOP, FIRST_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 21
		{ SECOND_FIELD_ROW_TOP, FIRST_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 22
		{ THIRD_FIELD_ROW_TOP, FIRST_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 23
		{ FOURTH_FIELD_ROW_TOP, FIRST_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 24
		{ FIFTH_FIELD_ROW_TOP, FIRST_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 25
		{ SIXTH_FIELD_ROW_TOP, FIRST_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 26
		{ FIRST_FIELD_ROW_TOP, SECOND_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 27
		{ SECOND_FIELD_ROW_TOP, SECOND_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 28
		{ THIRD_FIELD_ROW_TOP, SECOND_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 29
		{ FOURTH_FIELD_ROW_TOP, SECOND_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 30
		{ FIFTH_FIELD_ROW_TOP, SECOND_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 31
		{ SIXTH_FIELD_ROW_TOP, SECOND_FIELD_COLUMN_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupCommandXField },
		
		// 32
		{ ACTIVESET_GROUP_T, ACTIVESET_GROUP_L, -1, -1 },
		Control { disabled, kIDMacroSetupActiveSetPrimaryGroup },
		
		// 33
		{ ACTIVE_SET_RADIO_BUTTON_TOP, ACTIVE_SET_RADIO_BUTTON_FIRST_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupActiveSet1BevelButton },
		
		// 34
		{ ACTIVE_SET_RADIO_BUTTON_TOP, ACTIVE_SET_RADIO_BUTTON_SECOND_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupActiveSet2BevelButton },
		
		// 35
		{ ACTIVE_SET_RADIO_BUTTON_TOP, ACTIVE_SET_RADIO_BUTTON_THIRD_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupActiveSet3BevelButton },
		
		// 36
		{ ACTIVE_SET_RADIO_BUTTON_TOP, ACTIVE_SET_RADIO_BUTTON_FOURTH_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupActiveSet4BevelButton },
		
		// 37
		{ ACTIVE_SET_RADIO_BUTTON_TOP, ACTIVE_SET_RADIO_BUTTON_FIFTH_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupActiveSet5BevelButton },
		
		// 38
		{ ACTIVE_SET_RADIO_BUTTON_TOP, ACTIVE_SET_CURRENT_BUTTON_LEFT, -1, -1 },
		Control { enabled, kIDMacroSetupActiveSetCurrentButton },
		
		// 39
		{ MACROKEYS_GROUP_T, MACROKEYS_GROUP_L, -1, -1 },
		Control { disabled, kIDMacroSetupMacroKeysPrimaryGroup },
		
		// 40
		{	MACRO_KEYS_RADIO_BUTTON_TOP,
			MACRO_KEYS_RADIO_BUTTON_FIRST_LEFT,
			MACRO_KEYS_RADIO_BUTTON_TOP + RADIOBUTTON_HT,
			MACRO_KEYS_RADIO_BUTTON_FIRST_LEFT + MACRO_KEYS_RADIO_BUTTON_WD },
		RadioButton { enabled, MACROSETUP_RBNAME_MACROKEYSCOMMANDKEYS },
		
		// 41
		{	MACRO_KEYS_RADIO_BUTTON_TOP,
			MACRO_KEYS_RADIO_BUTTON_SECOND_LEFT,
			MACRO_KEYS_RADIO_BUTTON_TOP + RADIOBUTTON_HT,
			MACRO_KEYS_RADIO_BUTTON_SECOND_LEFT + MACRO_KEYS_RADIO_BUTTON_WD },
		RadioButton { enabled, MACROSETUP_RBNAME_MACROKEYSFUNCTIONKEYS }
	}
};

resource 'hdlg' (kDialogBalloonsIDMacroSetup, purgeable)
{
	HelpMgrVersion,
	0, // zero-based dialog item number of first item that has a help balloon
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMSkipItem {/* no help */}, // 1
		HMStringResItem // 2
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_CANCEL,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 3
		{
			{ HELPBUTTON_BALLOON_TIP_V, HELPBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_HELP,
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_DISABLED_BUTTON_HELP,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 4
		HMStringResItem // 5
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_ENABLED_BUTTON_IMPORT,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 6
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_ENABLED_BUTTON_EXPORT,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 7
		HMStringResItem // 8
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 9
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 10
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 11
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 12
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 13
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 14
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 15
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 16
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 17
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 18
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 19
		{
			{ (MACRO_LABEL_HT / 2), (MACRO_LABEL_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_LABEL_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 20
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 21
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 22
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 23
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 24
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 25
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 26
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 27
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 28
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 29
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 30
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 31
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_FIELD_GENERIC_MACRO,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 32
		HMStringResItem // 33
		{
			{ SMALL_BUTTON_BALLOON_TIP_V, SMALL_BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_UNCHECKED_RADIOBUTTON_SET,
			0, 0,
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_CHECKED_RADIOBUTTON_SET,
			0, 0
		},
		HMStringResItem // 34
		{
			{ SMALL_BUTTON_BALLOON_TIP_V, SMALL_BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_UNCHECKED_RADIOBUTTON_SET,
			0, 0,
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_CHECKED_RADIOBUTTON_SET,
			0, 0
		},
		HMStringResItem // 35
		{
			{ SMALL_BUTTON_BALLOON_TIP_V, SMALL_BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_UNCHECKED_RADIOBUTTON_SET,
			0, 0,
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_CHECKED_RADIOBUTTON_SET,
			0, 0
		},
		HMStringResItem // 36
		{
			{ SMALL_BUTTON_BALLOON_TIP_V, SMALL_BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_UNCHECKED_RADIOBUTTON_SET,
			0, 0,
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_CHECKED_RADIOBUTTON_SET,
			0, 0
		},
		HMStringResItem // 37
		{
			{ SMALL_BUTTON_BALLOON_TIP_V, SMALL_BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_UNCHECKED_RADIOBUTTON_SET,
			0, 0,
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_CHECKED_RADIOBUTTON_SET,
			0, 0
		},
		HMStringResItem // 38
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_ENABLED_BUTTON_MAKE_CURRENT_ACTIVE,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 39
		HMStringResItem // 40
		{
			{ RADIOBUTTON_BALLOON_TIP_V, RADIOBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_UNCHECKED_RADIOBUTTON_COMMANDKEYS,
			0, 0,
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_CHECKED_RADIOBUTTON_COMMANDKEYS,
			0, 0
		},
		HMStringResItem // 41
		{
			{ RADIOBUTTON_BALLOON_TIP_V, RADIOBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_UNCHECKED_RADIOBUTTON_FUNCTIONKEYS,
			0, 0,
			rStringsBalloonTextMacroSetupDialog, BALLOON_DIALOG_MACROSETUP_CHECKED_RADIOBUTTON_FUNCTIONKEYS,
			0, 0
		}
	}
};

resource 'dftb' (kDialogIDMacroSetup, "Macro Setup: Font Auxiliary Info")
{
	versionZero
	{
		{
			skipItem { }, // 1
			skipItem { }, // 2
			skipItem { }, // 3
			skipItem { }, // 4
			skipItem { }, // 5
			skipItem { }, // 6
			skipItem { }, // 7
			skipItem { }, // 8
			skipItem { }, // 9
			skipItem { }, // 10
			skipItem { }, // 11
			skipItem { }, // 12
			skipItem { }, // 13
			skipItem { }, // 14
			skipItem { }, // 15
			skipItem { }, // 16
			skipItem { }, // 17
			skipItem { }, // 18
			skipItem { }, // 19
			skipItem { }, // 20
			skipItem { }, // 21
			skipItem { }, // 22
			skipItem { }, // 23
			skipItem { }, // 24
			skipItem { }, // 25
			skipItem { }, // 26
			skipItem { }, // 27
			skipItem { }, // 28
			skipItem { }, // 29
			skipItem { }, // 30
			skipItem { }, // 31
			skipItem { }, // 32
			skipItem { }, // 33
			skipItem { }, // 34
			skipItem { }, // 35
			skipItem { }, // 36
			skipItem { }, // 37
			skipItem { }, // 38
			skipItem { }, // 39
			skipItem { }, // 40
			skipItem { } // 41
		}
	};
};

/* main macro setup dialog controls (group boxes & buttons) */

/* primary group boxes */

resource 'CNTL' (kIDMacroSetupMacroTextEquivalentsPrimaryGroup, "Macro Setup: "MACROSETUP_GROUPBOXNAME_EDITMACROSET" Group Box", purgeable)
{
	{ 0, 0, MACROTEXTEQ_GROUP_B - MACROTEXTEQ_GROUP_T, MACROTEXTEQ_GROUP_R - MACROTEXTEQ_GROUP_L },
	popupTitleLeftJust, // see page 5-119 of IM:MTE for valid values
	visible,
	POPUPEDITMACROSMENU_TITLE_WD, // title width in pixels
	kPopUpMenuIDEditMacroSet, // menu resource ID
	kControlGroupBoxPopupButtonProc | kControlPopupVariableWidthVariant,
	0,
	PREFERENCES_MACROS_POPUPNAME_EDITMACROSET
};

resource 'CNTL' (kIDMacroSetupActiveSetPrimaryGroup, "Macro Setup: Active Set Group Box", purgeable)
{
	{ 0, 0, ACTIVESET_GROUP_B - ACTIVESET_GROUP_T, ACTIVESET_GROUP_R - ACTIVESET_GROUP_L },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	MACROSETUP_GROUPBOXNAME_ACTIVESET
};

resource 'CNTL' (kIDMacroSetupMacroKeysPrimaryGroup, "Macro Setup: Macro Keys Group Box", purgeable)
{
	{ 0, 0, MACROKEYS_GROUP_B - MACROKEYS_GROUP_T, MACROKEYS_GROUP_R - MACROKEYS_GROUP_L },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	MACROSETUP_GROUPBOXNAME_MACROKEYS
};

/* Macro Text Equivalents primary box */

resource 'CNTL' (kIDMacroSetupImportButton, "Macro Setup: Macro Text Equivalents: Import... Button", purgeable)
{
	{ 0, 0, BUTTON_HT, IMPORT_BUTTON_WD },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	MACROSETUP_BTNNAME_IMPORT
};

resource 'CNTL' (kIDMacroSetupExportButton, "Macro Setup: Macro Text Equivalents: Export... Button", purgeable)
{
	{ 0, 0, BUTTON_HT, EXPORT_BUTTON_WD },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	MACROSETUP_BTNNAME_EXPORT
};

resource 'CNTL' (kIDMacroSetupMacroTextEquivalentsDivider, "Macro Setup: Macro Text Equivalents: Divider", purgeable)
{
	{ 0, 0, SEPARATOR_HT, MACROTEXTEQ_SEPARATOR_WIDTH },
	0,
	visible,
	0,
	0,
	kControlSeparatorLineProc,
	0,
	""
};

resource 'CNTL' (kIDMacroSetupCommandXField, "Macro Setup: Macro Text Equivalents: Command-X Field", purgeable)
{
	{ 0, 0, TEXT_HT_BIG, MACRO_FIELD_WD },
	0,
	visible,
	0,
	0,
	kControlEditTextInlineInputProc,//kControlEditTextProc,
	0,
	""
};

/* Active Set primary group box */

resource 'CNTL' (kIDMacroSetupActiveSetCurrentButton, "Macro Setup: Active Set: Use Set Above Button", purgeable)
{
	{ 0, 0, BUTTON_HT, ACTIVE_SET_CURRENT_BUTTON_WD },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	MACROSETUP_BTNNAME_USESETABOVE
};

resource 'CNTL' (kIDMacroSetupActiveSet1BevelButton, "Macro Setup: Active Set: Set 1 Button", purgeable)
{
	{ 0, 0, ACTIVE_SET_BEVELBUTTON_HT, ACTIVE_SET_BEVELBUTTON_WD },
	0, // menu to attach, or 0
	visible,
	0, // resource ID of content
	kControlBehaviorToggles, // behavior + content type
	kControlBevelButtonSmallBevelProc,
	0,
	MACROSETUP_RBNAME_ACTIVESET1
};

resource 'CNTL' (kIDMacroSetupActiveSet2BevelButton, "Macro Setup: Active Set: Set 2 Button", purgeable)
{
	{ 0, 0, ACTIVE_SET_BEVELBUTTON_HT, ACTIVE_SET_BEVELBUTTON_WD },
	0, // menu to attach, or 0
	visible,
	0, // resource ID of content
	kControlBehaviorToggles, // behavior + content type
	kControlBevelButtonSmallBevelProc,
	0,
	MACROSETUP_RBNAME_ACTIVESET2
};

resource 'CNTL' (kIDMacroSetupActiveSet3BevelButton, "Macro Setup: Active Set: Set 3 Button", purgeable)
{
	{ 0, 0, ACTIVE_SET_BEVELBUTTON_HT, ACTIVE_SET_BEVELBUTTON_WD },
	0, // menu to attach, or 0
	visible,
	0, // resource ID of content
	kControlBehaviorToggles, // behavior + content type
	kControlBevelButtonSmallBevelProc,
	0,
	MACROSETUP_RBNAME_ACTIVESET3
};

resource 'CNTL' (kIDMacroSetupActiveSet4BevelButton, "Macro Setup: Active Set: Set 4 Button", purgeable)
{
	{ 0, 0, ACTIVE_SET_BEVELBUTTON_HT, ACTIVE_SET_BEVELBUTTON_WD },
	0, // menu to attach, or 0
	visible,
	0, // resource ID of content
	kControlBehaviorToggles, // behavior + content type
	kControlBevelButtonSmallBevelProc,
	0,
	MACROSETUP_RBNAME_ACTIVESET4
};

resource 'CNTL' (kIDMacroSetupActiveSet5BevelButton, "Macro Setup: Active Set: Set 5 Button", purgeable)
{
	{ 0, 0, ACTIVE_SET_BEVELBUTTON_HT, ACTIVE_SET_BEVELBUTTON_WD },
	0, // menu to attach, or 0
	visible,
	0, // resource ID of content
	kControlBehaviorToggles, // behavior + content type
	kControlBevelButtonSmallBevelProc,
	0,
	MACROSETUP_RBNAME_ACTIVESET5
};

#endif
