/*
**************************************************************************
	TelnetMenus.r
	
	MacTelnet 3.0
	 1998-2001 by Kevin Grant
	
	This file contains all resource data for completely defining the
	menus.  For balloon items, the data listed for each object (such as
	an individual menu item) are in the order "enabled, disabled, checked,
	other".
**************************************************************************
*/

#ifndef __TELNETMENUS_R__
#define __TELNETMENUS_R__

#include "Balloons.r"
#include "MacTypes.r"
#include "Menus.r"
#include "MenuResources.h"
#ifndef __TELNETBALLOONS_R__
#	include "TelnetBalloons.r"
#endif
#include "TelnetMenuItemNames.h"

/* working menu bar layout */

/*!
WARNING:	The presence of this resource does NOT
			suggest that the menu order is easily
			changed.  There is code (in "Menus.c"
			alone) that dynamically manipulates
			menus, and this code will behave
			strangely (but will not crash) if the
			menu order is mucked with in the MBAR
			resource specification alone.
*/
resource 'MBAR' (kMenuBarIDMain, purgeable, preload)
{
	{
		kMenuIDApple,
		kMenuIDFile,
		kMenuIDEdit,
		kMenuIDAppearance,
		kMenuIDSession,
		kMenuIDKeyboard,
		kMenuIDNetwork,
		kMenuIDConnections
	}
};

/*  menu */

resource 'MENU' (kMenuIDApple, preload)
{
	kMenuIDApple, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	apple,
	{
		INAME_ABOUT, noIcon, noKey, noMark, plain;
		SUBMENUNAME_WEB, noIcon, hierarchicalMenu, kCharSubmenuIDTelnetOnTheWeb, plain
	}
};

resource 'xmnu' (kMenuIDApple, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandAboutTelnet,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandTelnetOnTheWeb,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				kSubmenuIDTelnetOnTheWeb,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDApple, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMSkipItem {/* the Mac OS provides this */}, // title
		HMStringResItem
		{
			rStringsBalloonTextAppleMenu, BALLOON_MENU_APPLE_ENABLED_ITEM_ABOUTTELNET,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* > */}
	}
};

/* File menu (normal) */

resource 'MENU' (kMenuIDFile, preload)
{
	kMenuIDFile, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_FILE,
	{
		INAME_OPENCONNECTION, noIcon, KEY_OPENCONNECTION, noMark, plain;
		SUBMENUNAME_FAV, noIcon, hierarchicalMenu, kCharSubmenuIDOpen, plain;
		INAME_LOADSET, noIcon, KEY_LOADSET, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_CLOSECONNECTION, noIcon, KEY_CLOSECONNECTION, noMark, plain;
		INAME_SAVESET, noIcon, KEY_SAVESET, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_IMPORTMACROSET, noIcon, KEY_IMPORTMACROSET, noMark, plain;
		INAME_EXPORTCURRENTMACROSET, noIcon, KEY_EXPORTCURRENTMACROSET, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_PRINT, noIcon, KEY_PRINT, noMark, plain;
		INAME_PRINTONE, noIcon, noKey, noMark, plain;
		INAME_PRINTSCREEN, noIcon, KEY_PRINTSCREEN, noMark, plain;
		INAME_PAGESETUP, noIcon, KEY_PAGESETUP, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_QUIT, noIcon, KEY_QUIT, noMark, plain
	}
};

resource 'xmnu' (kMenuIDFile, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandOpenConnection,
				MODIFIERS_OPENCONNECTION,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandOpenConnectionFavorite,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				kSubmenuIDOpen,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandLoadSet,
				MODIFIERS_LOADSET,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandCloseConnection,
				MODIFIERS_CLOSECONNECTION,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSaveSet,
				MODIFIERS_SAVESET,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandImportMacroSet,
				MODIFIERS_IMPORTMACROSET,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandExportCurrentMacroSet,
				MODIFIERS_EXPORTCURRENTMACROSET,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandPrint,
				MODIFIERS_PRINT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandPrintOne,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandPrintScreen,
				MODIFIERS_PRINTSCREEN,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandPageSetup,
				MODIFIERS_PAGESETUP,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandQuit,
				MODIFIERS_QUIT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDFile, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_ENABLED_MENU_FILE,
			0, 0,
			0, 0,
			0, 0
		}, // title
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_OPENCONNECTION,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* > */},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_LOADSET,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_CLOSECONNECTION,
			rStringsBalloonTextGenericSessions, BALLOON_GENERIC_CONNECTIONS_DISABLED_MENU_ITEM_NONE_OPEN,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_SAVESET,
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_DISABLED_ITEM_SAVESET,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_IMPORTMACROSET,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_EXPORTCURRENTMACROSET,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_PRINT,
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_DISABLED_ITEM_PRINT,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_PRINTONE,
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_DISABLED_ITEM_PRINT,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_PRINTSCREEN,
			rStringsBalloonTextGenericSessions, BALLOON_GENERIC_CONNECTIONS_DISABLED_MENU_ITEM_NONE_OPEN,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_PAGESETUP,
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_DISABLED_ITEM_PAGESETUP,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMSkipItem {/* NO HELP */}
	}
};

/* File menu (simple) */

resource 'MENU' (kMenuIDSimpleFile, preload)
{
	kMenuIDSimpleFile, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_FILE,
	{
		INAME_OPENCONNECTION, noIcon, noKey, noMark, plain;
		SUBMENUNAME_FAV, noIcon, hierarchicalMenu, kCharSubmenuIDOpen, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_CLOSECONNECTION, noIcon, KEY_CLOSECONNECTION, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_PRINT, noIcon, KEY_PRINT, noMark, plain;
		INAME_PAGESETUP, noIcon, noKey, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_QUIT, noIcon, KEY_QUIT, noMark, plain
	}
};

resource 'xmnu' (kMenuIDSimpleFile, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandOpenConnection,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandOpenConnectionFavorite,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				kSubmenuIDOpen,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandCloseConnection,
				MODIFIERS_CLOSECONNECTION,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandPrint,
				MODIFIERS_PRINT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandPageSetup,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandQuit,
				MODIFIERS_QUIT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDSimpleFile, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_ENABLED_MENU_FILE_SIMPLE,
			0, 0,
			0, 0,
			0, 0
		}, // title
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_OPENCONNECTION,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* > */},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_CLOSECONNECTION,
			rStringsBalloonTextGenericSessions, BALLOON_GENERIC_CONNECTIONS_DISABLED_MENU_ITEM_NONE_OPEN,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_PRINT,
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_DISABLED_ITEM_PRINT,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_ENABLED_ITEM_PAGESETUP,
			rStringsBalloonTextFileMenu, BALLOON_MENU_FILE_DISABLED_ITEM_PAGESETUP,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMSkipItem {/* NO HELP */}
	}
};

/* Edit menu (normal) */

resource 'MENU' (kMenuIDEdit, preload)
{
	kMenuIDEdit, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_EDIT,
	{
		INAME_UNDO, noIcon, KEY_UNDO, noMark, plain;
		INAME_REDO, noIcon, KEY_REDO, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_CUT, noIcon, KEY_CUT, noMark, plain;
		INAME_COPY, noIcon, KEY_COPY, noMark, plain;
		INAME_COPYTABLE, noIcon, KEY_COPYTABLE, noMark, plain;
		INAME_TYPE, noIcon, KEY_TYPE, noMark, plain;
		INAME_PASTE, noIcon, KEY_PASTE, noMark, plain;
		INAME_CLEAR, noIcon, noKey, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_SELECTALL, noIcon, KEY_SELECTALL, noMark, plain;
		INAME_SHOWCLIPBOARD, noIcon, noKey, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_PREFERENCES, noIcon, KEY_PREFERENCES, noMark, plain
	}
};

resource 'xmnu' (kMenuIDEdit, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandUndo,
				MODIFIERS_UNDO,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandRedo,
				MODIFIERS_REDO,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandCut,
				MODIFIERS_CUT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandCopy,
				MODIFIERS_COPY,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandCopyTable,
				MODIFIERS_COPYTABLE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandType,
				MODIFIERS_TYPE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandPaste,
				MODIFIERS_PASTE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandClear,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandSelectAll,
				MODIFIERS_SELECTALL,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandShowClipboard,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandPreferences,
				MODIFIERS_PREFERENCES,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDEdit, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_ENABLED_MENU_EDIT,
			0, 0,
			0, 0,
			0, 0
		}, // title
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_UNDO,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_UNDO,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_REDO,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_REDO,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_CUT,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_CUT,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_COPY,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_COPY,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_COPYTABLE,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_COPY,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_TYPE,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_TYPE,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_PASTE,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_PASTE,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_CLEAR,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_CLEAR,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_SELECTALL,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_SELECTALL,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_SHOWCLIPBOARD,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_PREFERENCES,
			0, 0,
			0, 0,
			0, 0
		}
	}
};

/* Edit menu (simple) */

resource 'MENU' (kMenuIDSimpleEdit, preload)
{
	kMenuIDSimpleEdit, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_EDIT,
	{
		INAME_CUT, noIcon, KEY_CUT, noMark, plain;
		INAME_COPY, noIcon, KEY_COPY, noMark, plain;
		INAME_PASTE, noIcon, KEY_PASTE, noMark, plain;
		INAME_CLEAR, noIcon, noKey, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_PREFERENCES, noIcon, KEY_PREFERENCES, noMark, plain
	}
};

resource 'xmnu' (kMenuIDSimpleEdit, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandCut,
				MODIFIERS_CUT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandCopy,
				MODIFIERS_COPY,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandPaste,
				MODIFIERS_PASTE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandClear,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandPreferences,
				MODIFIERS_PREFERENCES,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDSimpleEdit, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_ENABLED_MENU_EDIT_SIMPLE,
			0, 0,
			0, 0,
			0, 0
		}, // title
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_CUT,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_CUT,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_COPY,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_COPY,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_PASTE,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_PASTE,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_CLEAR,
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_DISABLED_ITEM_CLEAR,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextEditMenu, BALLOON_MENU_EDIT_ENABLED_ITEM_PREFERENCES,
			0, 0,
			0, 0,
			0, 0
		}
	}
};

/* View menu */

resource 'MENU' (kMenuIDAppearance, preload)
{
	kMenuIDAppearance, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_VIEW,
	{
		INAME_LARGESCREEN, noIcon, noKey, noMark, plain;
		INAME_SMALLSCREEN, noIcon, noKey, noMark, plain;
		INAME_SETSCREENSIZE, noIcon, KEY_SETSCREENSIZE, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_BIGGERTEXT, noIcon, KEY_BIGGERTEXT, noMark, plain;
		INAME_SMALLERTEXT, noIcon, KEY_SMALLERTEXT, noMark, plain;
		INAME_FULLSCREEN, noIcon, noKey, noMark, plain;
		INAME_FORMAT, noIcon, KEY_FORMAT, noMark, plain
	}
};

resource 'xmnu' (kMenuIDAppearance, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandLargeScreen,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSmallScreen,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSetScreenSize,
				MODIFIERS_SETSCREENSIZE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandBiggerText,
				MODIFIERS_BIGGERTEXT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSmallerText,
				MODIFIERS_SMALLERTEXT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandFullScreen,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandFormat,
				MODIFIERS_FORMAT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDAppearance, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMStringResItem
		{
			rStringsBalloonTextViewMenu, BALLOON_ENABLED_MENU_VIEW,
			rStringsBalloonTextViewMenu, BALLOON_DISABLED_MENU_VIEW,
			0, 0,
			0, 0
		}, // title
		HMStringResItem
		{
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_ENABLED_ITEM_LARGESCREEN,
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_DISABLED_ITEM_LARGESCREEN,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_ENABLED_ITEM_SMALLSCREEN,
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_DISABLED_ITEM_SMALLSCREEN,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_ENABLED_ITEM_SETSCREENSIZE,
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_DISABLED_ITEM_SETSCREENSIZE,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			0, 0/* NO HELP YET */,
			0, 0/* NO HELP YET */,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			0, 0/* NO HELP YET */,
			0, 0/* NO HELP YET */,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_ENABLED_ITEM_FULLSCREEN,
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_DISABLED_ITEM_FULLSCREEN,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_ENABLED_ITEM_FORMAT,
			rStringsBalloonTextViewMenu, BALLOON_MENU_VIEW_DISABLED_ITEM_FORMAT,
			0, 0,
			0, 0
		}
	}
};

/* Terminal menu */

resource 'MENU' (kMenuIDSession, preload)
{
	kMenuIDSession, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_TERMINAL,
	{
		INAME_CAPTURETOFILE, noIcon, KEY_CAPTURETOFILE, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_ECHO, noIcon, noKey, noMark, plain;
		INAME_WRAPMODE, noIcon, noKey, noMark, plain;
		INAME_CLEARSCREENSAVESLINES, noIcon, noKey, noMark, plain;
		INAME_JUMPSCROLLING, noIcon, KEY_JUMPSCROLLING, noMark, plain;
		SUBMENUNAME_TEK, noIcon, hierarchicalMenu, kCharSubmenuIDTektronix, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_SPEECH, noIcon, KEY_SPEECH, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_RESETTERMINAL, noIcon, KEY_RESETTERMINAL, noMark, plain
	}
};

resource 'xmnu' (kMenuIDSession, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandCaptureToFile,
				MODIFIERS_CAPTURETOFILE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandEcho,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandWrapMode,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandClearScreenSavesLines,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandJumpScrolling,
				MODIFIERS_JUMPSCROLLING,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandTektronix,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				kSubmenuIDTektronix,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandSpeechEnabled,
				MODIFIERS_SPEECH,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandResetTerminal,
				MODIFIERS_RESETTERMINAL,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDSession, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMStringResItem
		{
			rStringsBalloonTextTerminalMenu, BALLOON_ENABLED_MENU_TERMINAL,
			rStringsBalloonTextTerminalMenu, BALLOON_DISABLED_MENU_TERMINAL,
			0, 0,
			0, 0
		}, // title
		HMStringResItem
		{
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_ENABLED_ITEM_CAPTURETOFILE,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_DISABLED_ITEM_CAPTURETOFILE,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_CHECKED_ITEM_CAPTURETOFILE,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_ENABLED_ITEM_ECHO,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_DISABLED_ITEM_ECHO,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_CHECKED_ITEM_ECHO,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_ENABLED_ITEM_WRAPMODE,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_DISABLED_ITEM_WRAPMODE,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_CHECKED_ITEM_WRAPMODE,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_ENABLED_ITEM_CLEARSCREENSAVESLINES,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_DISABLED_ITEM_CLEARSCREENSAVESLINES,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_CHECKED_ITEM_CLEARSCREENSAVESLINES,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_ENABLED_ITEM_JUMPSCROLLING,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_DISABLED_ITEM_JUMPSCROLLING,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			0, 0,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_DISABLED_ITEM_TEKTRONIX,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_ENABLED_ITEM_SPEECHENABLED,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_DISABLED_ITEM_SPEECHENABLED,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_ENABLED_ITEM_RESETTERMINAL,
			rStringsBalloonTextTerminalMenu, BALLOON_MENU_TERMINAL_DISABLED_ITEM_RESETTERMINAL,
			0, 0,
			0, 0
		}
	}
};

/* Keyboard menu */

resource 'MENU' (kMenuIDKeyboard, preload)
{
	kMenuIDKeyboard, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_KEYBOARD,
	{
		INAME_DELETEPRESSSENDSBACKSPACE, noIcon, KEY_DELETEPRESSSENDSBACKSPACE, noMark, plain;
		INAME_DELETEPRESSSENDSDELETE, noIcon, KEY_DELETEPRESSSENDSDELETE, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_EMACSARROWMAPPING, noIcon, noKey, noMark, plain;
		INAME_LOCALPAGEUPDOWN, noIcon, noKey, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_SETKEYS, noIcon, noKey, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_TITLETRANSLATIONCHOICES, noIcon, noKey, MARK_MENUTITLES, STYLE_MENUTITLES
	}
};

resource 'xmnu' (kMenuIDKeyboard, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandDeletePressSendsBackspace,
				MODIFIERS_DELETEPRESSSENDSBACKSPACE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandDeletePressSendsDelete,
				MODIFIERS_DELETEPRESSSENDSDELETE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandEMACSArrowMapping,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandLocalPageUpDown,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandSetKeys,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandTitleTranslationChoices,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDKeyboard, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMStringResItem
	{
		rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_ENABLED_ITEM_TRANSLATIONCHOICE,
		rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_DISABLED_ITEM_TRANSLATIONCHOICE,
		0, 0,
		0, 0
	}, // missing item strings
	{
		HMStringResItem
		{
			rStringsBalloonTextKeysMenu, BALLOON_ENABLED_MENU_KEYBOARD,
			rStringsBalloonTextKeysMenu, BALLOON_DISABLED_MENU_KEYBOARD,
			0, 0,
			0, 0
		}, // title
		HMStringResItem
		{
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_ENABLED_ITEM_DELETEPRESSSENDSBACKSPACE,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_DISABLED_ITEM_DELETEPRESSSENDSBACKSPACE,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_CHECKED_ITEM_DELETEPRESSSENDSBACKSPACE,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_ENABLED_ITEM_DELETEPRESSSENDSDELETE,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_DISABLED_ITEM_DELETEPRESSSENDSDELETE,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_CHECKED_ITEM_DELETEPRESSSENDSDELETE,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_ENABLED_ITEM_EMACSARROWMAPPING,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_DISABLED_ITEM_EMACSARROWMAPPING,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_CHECKED_ITEM_EMACSARROWMAPPING,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_ENABLED_ITEM_LOCALPAGEUPDOWN,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_DISABLED_ITEM_LOCALPAGEUPDOWN,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_CHECKED_ITEM_LOCALPAGEUPDOWN,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_ENABLED_ITEM_SETKEYS,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_DISABLED_ITEM_SETKEYS,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			0, 0,
			rStringsBalloonTextKeysMenu, BALLOON_MENU_KEYBOARD_DISABLED_ITEM_TRANSLATIONCHOICETITLE,
			0, 0,
			0, 0
		}
	}
};

/* Network menu */

resource 'MENU' (kMenuIDNetwork, preload)
{
	kMenuIDNetwork, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_NETWORK,
	{
		INAME_SHOWNETWORKNUMBERS, noIcon, KEY_SHOWNETWORKNUMBERS, noMark, plain;
		INAME_SENDINTERNETPROTOCOLNUMBER, noIcon, KEY_SENDINTERNETPROTOCOLNUMBER, noMark, plain;
		INAME_SENDFTPCOMMAND, noIcon, noKey, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_SENDSYNC, noIcon, noKey, noMark, plain;
		INAME_SENDBREAK, noIcon, noKey, noMark, plain;
		INAME_SENDINTERRUPTPROCESS, noIcon, noKey, noMark, plain;
		INAME_SENDABORTOUTPUT, noIcon, KEY_SENDABORTOUTPUT, noMark, plain;
		INAME_SENDAREYOUTHERE, noIcon, KEY_SENDAREYOUTHERE, noMark, plain;
		INAME_SENDERASECHARACTER, noIcon, KEY_SENDERASECHARACTER, noMark, plain;
		INAME_SENDERASELINE, noIcon, KEY_SENDERASELINE, noMark, plain;
		INAME_SENDENDOFFILE, noIcon, noKey, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_WATCHFORACTIVITY, noIcon, noKey, noMark, plain;
		INAME_SUSPENDNETWORK, noIcon, noKey, noMark, plain
	}
};

resource 'xmnu' (kMenuIDNetwork, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandShowNetworkNumbers,
				MODIFIERS_SHOWNETWORKNUMBERS,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSendInternetProtocolNumber,
				MODIFIERS_SENDINTERNETPROTOCOLNUMBER,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSendFTPCommand,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandSendSync,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSendBreak,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSendInterruptProcess,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSendAbortOutput,
				MODIFIERS_SENDABORTOUTPUT,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSendAreYouThere,
				MODIFIERS_SENDAREYOUTHERE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSendEraseCharacter,
				MODIFIERS_SENDERASECHARACTER,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSendEraseLine,
				MODIFIERS_SENDERASELINE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSendEndOfFile,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandWatchForActivity,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandSuspendNetwork,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDNetwork, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_ENABLED_MENU_NETWORK,
			0, 0,
			0, 0,
			0, 0
		}, // title
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SHOWNETWORKNUMBERS,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDINTERNETPROTOCOLNUMBER,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDINTERNETPROTOCOLNUMBER,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDFTPCOMMAND,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDFTPCOMMAND,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDSYNC,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDSYNC,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDBREAK,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDBREAK,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDINTERRUPTPROCESS,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDINTERRUPTPROCESS,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDABORTOUTPUT,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDABORTOUTPUT,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDAREYOUTHERE,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDAREYOUTHERE,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDERASECHARACTER,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDERASECHARACTER,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDERASELINE,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDERASELINE,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SENDENDOFFILE,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SENDENDOFFILE,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_WATCHFORACTIVITY,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_WATCHFORACTIVITY,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_CHECKED_ITEM_WATCHFORACTIVITY,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_ENABLED_ITEM_SUSPENDNETWORK,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_DISABLED_ITEM_SUSPENDNETWORK,
			rStringsBalloonTextNetworkMenu, BALLOON_MENU_NETWORK_CHECKED_ITEM_SUSPENDNETWORK,
			0, 0
		}
	}
};

/* Window menu */

resource 'MENU' (kMenuIDConnections, preload)
{
	kMenuIDConnections, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_WINDOW,
	{
		INAME_HIDEFRONTWINDOW, noIcon, noKey, noMark, plain;
		INAME_HIDEOTHERWINDOWS, noIcon, noKey, noMark, plain;
		INAME_SHOWALLHIDDENWINDOWS, noIcon, KEY_SHOWALLHIDDENWINDOWS, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_STACKWINDOWS, noIcon, noKey, noMark, plain;
		INAME_NEXTWINDOW, noIcon, KEY_NEXTWINDOW, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_CHANGEWINDOWTITLE, noIcon, noKey, noMark, plain;
		INAME_MINIMIZEFRONTWINDOW, noIcon, KEY_MINIMIZEFRONTWINDOW, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_SHOWSESSIONINSPECTOR, noIcon, KEY_SHOWSESSIONINSPECTOR, noMark, plain;
		INAME_SHOWFTPLOG, noIcon, noKey, noMark, plain;
		INAME_SHOWCOMMANDLINE, noIcon, KEY_SHOWCOMMANDLINE, noMark, plain;
		INAME_SHOWCONTROLKEYS, noIcon, noKey, noMark, plain;
		INAME_SHOWFUNCTION, noIcon, noKey, noMark, plain;
		INAME_SHOWKEYPAD, noIcon, noKey, noMark, plain
	}
};

resource 'xmnu' (kMenuIDConnections, preload)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandHideFrontWindow,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandHideOtherWindows,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandShowAllHiddenWindows,
				MODIFIERS_SHOWALLHIDDENWINDOWS,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandStackWindows,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandNextWindow,
				MODIFIERS_NEXTWINDOW,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandChangeWindowTitle,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandMinimizeWindow,
				MODIFIERS_MINIMIZEFRONTWINDOW,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandShowSessionInspector,
				MODIFIERS_SHOWSESSIONINSPECTOR,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandShowFTPLog,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandShowCommandLine,
				MODIFIERS_SHOWCOMMANDLINE,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandShowControlKeys,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandShowFunction,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandShowKeypad,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kMenuIDConnections, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMStringResItem
	{
		// The Other balloon is required because the first 9 connection items are
		// marked with numbers, which are not checkmarks but are marks nonetheless,
		// so Balloon Help does not use the standard Enabled balloon for these items.
		// It is the same balloon as the standard Enabled balloon because, if more
		// than 9 connection items exist, they have no mark.
		rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_GENERIC_CONNECTION,
		rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_DISABLED_MENU_ITEM,
		rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_CHECKED_ITEM_GENERIC_CONNECTION,
		rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_GENERIC_CONNECTION
	}, // missing item
	{
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_ENABLED_MENU_WINDOW,
			rStringsBalloonTextWindowMenu, BALLOON_DISABLED_MENU_WINDOW,
			0, 0,
			0, 0
		}, // title
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_HIDEFRONTWINDOW,
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_DISABLED_ITEM_HIDEFRONTWINDOW,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_HIDEOTHERWINDOWS,
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_DISABLED_ITEM_HIDEOTHERWINDOWS,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_SHOWALLHIDDENWINDOWS,
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_DISABLED_ITEM_SHOWALLHIDDENWINDOWS,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_STACKWINDOWS,
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_DISABLED_ITEM_STACKWINDOWS,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_NEXTWINDOW,
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_DISABLED_ITEM_NEXTWINDOW,
			0, 0,
			0, 0
		},
		HMSkipItem {/****************************/},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_CHANGEWINDOWTITLE,
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_DISABLED_ITEM_CHANGEWINDOWTITLE,
			0, 0,
			0, 0
		},
		HMSkipItem {/* NO HELP YET */},
		HMSkipItem {/****************************/},
		HMSkipItem {/* NO HELP YET */},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_SHOWFTPLOG,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_SHOWCOMMANDLINE,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_SHOWCONTROLKEYS,
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_DISABLED_ITEM_SHOWCONTROLKEYS,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_SHOWFUNCTION,
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_DISABLED_ITEM_SHOWFUNCTION,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_ENABLED_ITEM_SHOWKEYPAD,
			rStringsBalloonTextWindowMenu, BALLOON_MENU_WINDOW_DISABLED_ITEM_SHOWKEYPAD,
			0, 0,
			0, 0
		}
	}
};

/* Scripts menu */

resource 'MENU' (kMenuIDScripts)
{
	kMenuIDScripts, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	MENUNAME_SCRIPTS,
	{
	}
};



resource 'hmnu' (kMenuIDScripts, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMStringResItem
	{
		// The Other balloon is required because the first 9 connection items are
		// marked with numbers, which are not checkmarks but are marks nonetheless,
		// so Balloon Help does not use the standard Enabled balloon for these items.
		// It is the same balloon as the standard Enabled balloon because, if more
		// than 9 connection items exist, they have no mark.
		rStringsBalloonTextScriptsMenu, BALLOON_MENU_SCRIPTS_ENABLED_ITEM_GENERIC_SCRIPT,
		0, 0,
		0, 0,
		0, 0
	}, // missing item
	{
		HMStringResItem
		{
			rStringsBalloonTextScriptsMenu, BALLOON_ENABLED_MENU_SCRIPTS,
			0, 0,
			0, 0,
			0, 0
		} // title
	}
};

/* New Session From Favorites submenu */

resource 'MENU' (kSubmenuIDOpen)
{
	kSubmenuIDOpen, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	"",
	{
	}
};

/* Translation submenu */

resource 'MENU' (kSubmenuIDTranslation)
{
	kSubmenuIDTranslation, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	SUBMENUNAME_XLATION,
	{
	}
};

/* Tektronix submenu */

resource 'MENU' (kSubmenuIDTektronix)
{
	kSubmenuIDTektronix, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	SUBMENUNAME_TEK,
	{
		INAME_TEKPAGECOMMAND, noIcon, KEY_TEKPAGECOMMAND, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_TEKPAGECLEARSSCREEN, noIcon, noKey, noMark, plain
	}
};

resource 'xmnu' (kSubmenuIDTektronix)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandTEKPageCommand,
				MODIFIERS_TEKPAGECOMMAND,
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			skipItem {/****************************/},
			dataItem
			{
				kCommandTEKPageClearsScreen,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kSubmenuIDTektronix, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMSkipItem {}, // title
		HMStringResItem
		{
			rStringsBalloonTextTektronixMenu, BALLOON_MENU_TEKTRONIX_ENABLED_ITEM_TEKPAGECOMMAND,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextTektronixMenu, BALLOON_MENU_TEKTRONIX_ENABLED_ITEM_TEKPAGECLEARSSCREEN,
			0, 0,
			rStringsBalloonTextTektronixMenu, BALLOON_MENU_TEKTRONIX_CHECKED_ITEM_TEKPAGECLEARSSCREEN,
			0, 0
		}
	}
};

/* Internet Resources submenu */

resource 'MENU' (kSubmenuIDTelnetOnTheWeb)
{
	kSubmenuIDTelnetOnTheWeb, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	SUBMENUNAME_WEB,
	{
		INAME_TELNETHOMEPAGE, noIcon, noKey, noMark, plain;
		INAME_TELNETAUTHORMAIL, noIcon, noKey, noMark, plain;
		INAME_TELNETSOURCELICENSE, noIcon, noKey, noMark, plain;
		INAME_TELNETLIBRARYLICENSE, noIcon, noKey, noMark, plain;
		INAME_TELNETPROJECTSTATUS, noIcon, noKey, noMark, plain
	}
};

resource 'xmnu' (kSubmenuIDTelnetOnTheWeb)
{
	versionZero
	{
		{
			dataItem
			{
				kCommandTelnetHomePage,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandTelnetAuthorMail,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandTelnetSourceLicense,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandTelnetLibraryLicense,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			},
			dataItem
			{
				kCommandTelnetProjectStatus,
				kMenuNoModifiers, // except 
				currScript,
				0,
				0,
				noHierID,
				sysFont,
				naturalGlyph
			}
		}
	}
};

resource 'hmnu' (kSubmenuIDTelnetOnTheWeb, purgeable)
{
	HelpMgrVersion,
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMSkipItem {}, // title
		HMStringResItem
		{
			rStringsBalloonTextInternetResourcesMenu, BALLOON_MENU_TELNETONTHEWEB_ENABLED_ITEM_TELNETHOMEPAGE,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextInternetResourcesMenu, BALLOON_MENU_TELNETONTHEWEB_ENABLED_ITEM_TELNETAUTHORMAIL,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextInternetResourcesMenu, BALLOON_MENU_TELNETONTHEWEB_ENABLED_ITEM_TELNETSOURCELICENSE,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextInternetResourcesMenu, BALLOON_MENU_TELNETONTHEWEB_ENABLED_ITEM_TELNETLIBRARYLICENSE,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem
		{
			rStringsBalloonTextInternetResourcesMenu, BALLOON_MENU_TELNETONTHEWEB_ENABLED_ITEM_TELNETPROJECTSTATUS,
			0, 0,
			0, 0,
			0, 0
		}
	}
};

/* Open Connections recent items pop-up menu */

resource 'MENU' (kPopUpMenuIDConnectRecent)
{
	kPopUpMenuIDConnectRecent, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	"",
	{
	}
};

/* Format display information pop-up menu */

resource 'MENU' (kPopUpMenuIDFormatType)
{
	kPopUpMenuIDFormatType, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	FORMAT_POPUPNAME_EDIT,
	{
		INAME_FORMATNORMALTEXT, noIcon, noKey, noMark, plain;
		INAME_FORMATBOLDTEXT, noIcon, noKey, noMark, plain;
		INAME_FORMATBLINKINGTEXT, noIcon, noKey, noMark, plain
	}
};

/* Configurations display pop-up menu */

resource 'MENU' (kPopUpMenuIDConfigurationType)
{
	kPopUpMenuIDConfigurationType, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	PREFERENCES_CONFIGURATIONS_POPUPNAME_SHOW,
	{
		SESSION_FAVORITES_AS_TITLE, noIcon, noKey, noMark, plain;
		TERMINAL_PLURAL_AS_TITLE, noIcon, noKey, noMark, plain;
		PROXY_PLURAL_AS_TITLE, noIcon, noKey, noMark, plain
	}
};

/* Configurations display pop-up menu */

resource 'MENU' (kPopUpMenuIDEditMacroSet)
{
	kPopUpMenuIDEditMacroSet, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	PREFERENCES_MACROS_POPUPNAME_EDITMACROSET,
	{
		INAME_MACROSET1, noIcon, noKey, noMark, plain;
		INAME_MACROSET2, noIcon, noKey, noMark, plain;
		INAME_MACROSET3, noIcon, noKey, noMark, plain;
		INAME_MACROSET4, noIcon, noKey, noMark, plain;
		INAME_MACROSET5, noIcon, noKey, noMark, plain
	}
};

/* Terminal pop-up menu */

resource 'MENU' (1004)
{
	1004, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	"",
	{
	}
};

/* Session pop-up menu */

resource 'MENU' (1005)
{
	1005, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	"",
	{
	}
};

/* Header Display pop-up menu */

resource 'MENU' (1006)
{
	1006, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	disabled, /* entire menu */
	"",
	{
	}
};

/* Font pop-up menu */

resource 'MENU' (kPopUpMenuIDFont)
{
	kPopUpMenuIDFont, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	SUBMENUNAME_FONT,
	{
	}
};

/* Size pop-up menu */

resource 'MENU' (kPopUpMenuIDSize)
{
	kPopUpMenuIDSize, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	SUBMENUNAME_SIZE,
	{
		INAME_SIZE4, noIcon, noKey, noMark, plain;
		INAME_SIZE9, noIcon, noKey, noMark, plain;
		INAME_SIZE10, noIcon, noKey, noMark, plain;
		INAME_SIZE11, noIcon, noKey, noMark, plain;
		INAME_SIZE12, noIcon, noKey, noMark, plain;
		INAME_SIZE14, noIcon, noKey, noMark, plain;
		INAME_SIZE18, noIcon, noKey, noMark, plain
	}
};

/* Proxy Server Protocol pop-up menu */

resource 'MENU' (kPopUpMenuIDProxyProtocol)
{
	kPopUpMenuIDProxyProtocol, kMenuStdMenuProc,
	0x00000003, /* item enabled flags */
	enabled, /* entire menu */
	"",
	{
		// WARNING: if you change the order of these, update "ProxyServerDialog.c"
		INAME_PROXYPROTOCOLSOCKS4, noIcon, noKey, noMark, plain;
		INAME_PROXYPROTOCOLSOCKS4A, noIcon, noKey, noMark, plain;
		INAME_PROXYPROTOCOLSOCKS5, noIcon, noKey, noMark, plain
	}
};

/* color box pop-up menu */

resource 'MENU' (kPopUpMenuIDColorBox)
{
	kPopUpMenuIDColorBox, kMenuStdMenuProc,
	allEnabled, /* item enabled flags */
	enabled, /* entire menu */
	"",
	{
		INAME_COLORUSEPICKER, noIcon, noKey, noMark, plain;
		"-", noIcon, noKey, noMark, plain;
		INAME_COLORBLACK, noIcon, noKey, noMark, plain;
		INAME_COLORDARKGRAY, noIcon, noKey, noMark, plain;
		INAME_COLORSILVER, noIcon, noKey, noMark, plain;
		INAME_COLORWHITE, noIcon, noKey, noMark, plain;
		INAME_COLORBROWN, noIcon, noKey, noMark, plain;
		INAME_COLORDARKRED, noIcon, noKey, noMark, plain;
		INAME_COLORRED, noIcon, noKey, noMark, plain;
		INAME_COLORDARKMAGENTA, noIcon, noKey, noMark, plain;
		INAME_COLORMAGENTA, noIcon, noKey, noMark, plain;
		INAME_COLORDARKBLUE, noIcon, noKey, noMark, plain;
		INAME_COLORBLUE, noIcon, noKey, noMark, plain;
		INAME_COLORDARKGREEN, noIcon, noKey, noMark, plain;
		INAME_COLORGREEN, noIcon, noKey, noMark, plain;
		INAME_COLORDARKCYAN, noIcon, noKey, noMark, plain;
		INAME_COLORCYAN, noIcon, noKey, noMark, plain;
		INAME_COLORYELLOW, noIcon, noKey, noMark, plain
	}
};

#endif