/*
**************************************************************************
	TelnetMiscDialogs.r
	
	MacTelnet 3.0
	 1998-2000 by Kevin Grant
	
	This file contains data for dialogs and their associated controls.
**************************************************************************
*/

#include "Appearance.r"
#include "Balloons.r"
#include "Controls.r"
#include "ControlDefinitions.r"
#include "Dialogs.r"
#include "MacTypes.r"
#include "Windows.r"

#include "ControlResources.h"
#include "DialogResources.h"
#include "InterfaceLib.h"
#include "SpacingConstants.r"
#ifndef __TELNETBALLOONS_R__
#	include "TelnetBalloons.r"
#endif
#include "TelnetButtonNames.h"



/*
*******************************************
	ABOUT BOX
*******************************************
*/

/* the entire dialog box area */
#define	DIALOG_WD		486
#define	DIALOG_HT		304

resource 'DLOG' (kDialogIDAboutBox, "About Box", purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowModalDialogProc,
	invisible,
	0, // close box
	0,
	kDialogIDAboutBox,
	"",
	centerMainScreen
};

resource 'dlgx' (kDialogIDAboutBox)
{
	versionZero
	{
		kDialogFlagsUseControlHierarchy		|
		kDialogFlagsUseThemeControls
	}
};

resource 'DITL' (kDialogIDAboutBox, "About Box", purgeable)
{
	{
		// 1
		{ 4, 0, -1, -1 },
		Control { enabled, kIDAboutPicture },
		
		// 2
		{ 106, 10, -1, -1 },
		Control { enabled, kIDAboutScrollingTextUserPane }
	}
};

/* main dialog controls (picture, user pane and text items) */

resource 'CNTL' (kIDAboutPicture, "About Box Picture", purgeable)
{
	{ 0, 0, 88, 472 }, // set to the size of the picture
	1025, // 'PICT' resource ID
	visible,
	0,
	0,
	kControlPictureNoTrackProc,
	0,
	""
};

resource 'CNTL' (kIDAboutScrollingTextUserPane, "About Box Scrolling Text User Pane", purgeable)
{
	{ 0, 0, 180, (DIALOG_WD - 20) }, // arbitrary, but set to fit inside large black area of background picture
	kControlSupportsEmbedding | kControlWantsIdle, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};


/*
*******************************************
	SMALL, GENERIC PROGRESS DIALOG BOX
*******************************************
*/

/* the entire dialog box area */
#define	DIALOG_WD		370
#define	DIALOG_HT		90

resource 'DLOG' (kDialogIDGenericProgress, "Generic Progress", purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowModalDialogProc,
	invisible,
	0, // close box
	0,
	kDialogIDGenericProgress,
	"",
	centerMainScreen
};

resource 'DLOG' (kDialogIDGenericProgressModeless, "Generic Progress (Modeless)", purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowDocumentProc,
	invisible,
	0, // close box
	0,
	kDialogIDGenericProgress, // use the same DITL for the modeless version
	"",
	centerMainScreen
};

resource 'dlgx' (kDialogIDGenericProgress)
{
	versionZero
	{
		kDialogFlagsUseThemeBackground		|
		kDialogFlagsUseControlHierarchy		|
		kDialogFlagsUseThemeControls
	}
};

resource 'dlgx' (kDialogIDGenericProgressModeless)
{
	versionZero
	{
		kDialogFlagsUseThemeBackground		|
		kDialogFlagsUseControlHierarchy		|
		kDialogFlagsUseThemeControls
	}
};

resource 'DITL' (kDialogIDGenericProgress, "Generic Progress", purgeable)
{
	{
		// 1
		{ VSP_TEXT_AND_DIALOG, HSP_TEXT_AND_DIALOG, -1, -1 },
		Control { enabled, kIDGenericProgressText },
		
		// 2
		{ (DIALOG_HT - VSP_BUTTON_AND_DIALOG - PROGRESSBAR_HT), HSP_BUTTON_AND_DIALOG, -1, -1 },
		Control { enabled, kIDGenericProgressBar }
	}
};

/* main dialog controls (text & progress bar) */

resource 'CNTL' (kIDGenericProgressBar, "Generic Progress: Progress Bar", purgeable)
{
	{ 0, 0, PROGRESSBAR_HT, (DIALOG_WD - HSP_BUTTON_AND_DIALOG - HSP_BUTTON_AND_DIALOG) },
	0,
	visible,
	0,
	0,
	kControlProgressBarProc,
	0,
	""
};

resource 'CNTL' (kIDGenericProgressText, "Generic Progress: Text Message", purgeable)
{
	{ 0, 0, TEXT_HT_BIG, (DIALOG_WD - HSP_TEXT_AND_DIALOG - HSP_TEXT_AND_DIALOG) },
	0,
	visible,
	0,
	0,
	kControlStaticTextProc,
	0,
	""
};


/*
*******************************************
	"SET WINDOW TITLE" DIALOG BOX
*******************************************
*/

/* the entire dialog box area */
#define	DIALOG_WD		350
#define	DIALOG_HT		110

#define SETTITLE_BUTTON_WD	78
#define SETTITLE_LABEL_WD	84

resource 'DLOG' (kDialogIDSetWindowTitle, SETWINDOWTITLE_WINDOWNAME, purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowMovableModalGrowProc,
	invisible,
	0, // close box
	0,
	kDialogIDSetWindowTitle,
	SETWINDOWTITLE_WINDOWNAME,
	alertPositionParentWindow
};

resource 'dlgx' (kDialogIDSetWindowTitle)
{
	versionZero
	{
		kDialogFlagsUseThemeBackground |
		kDialogFlagsUseControlHierarchy |
		kDialogFlagsHandleMovableModal |
		kDialogFlagsUseThemeControls
	}
};

resource 'DITL' (kDialogIDSetWindowTitle, SETWINDOWTITLE_WINDOWNAME, purgeable)
{
	{
		// 1
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDSetWindowTitleSetTitleButton },
		
		// 2
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDCancelButton },
		
		// 3
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDAppleGuideButton },
		
		// 4
		{ VSP_TEXT_AND_DIALOG, HSP_TEXT_AND_DIALOG,
			VSP_TEXT_AND_DIALOG + TEXT_HT_BIG, HSP_TEXT_AND_DIALOG + SETTITLE_LABEL_WD },
		StaticText { enabled, DIALOGTEXT_SETWINDOWTITLE_LABEL },
		
		// 5
		{ VSP_TEXT_AND_DIALOG, HSP_TEXT_AND_DIALOG + SETTITLE_LABEL_WD + HSP_CONTROLS, -1, -1 },
		Control { enabled, kIDSetWindowTitleWindowTitleField }
	}
};

resource 'hdlg' (kDialogBalloonsIDSetWindowTitle, purgeable)
{
	HelpMgrVersion,
	0, // zero-based dialog item number of first item that has a help balloon
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMStringResItem // 1
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextSetWindowTitleDialog, BALLOON_DIALOG_SETWINDOWTITLE_ENABLED_BUTTON_SETTITLE,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 2
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_CANCEL,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 3
		{
			{ HELPBUTTON_BALLOON_TIP_V, HELPBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_HELP,
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_DISABLED_BUTTON_HELP,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 4
		HMStringResItem // 5
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextSetWindowTitleDialog, BALLOON_DIALOG_SETWINDOWTITLE_FIELD_WINDOW_TITLE,
			0, 0,
			0, 0,
			0, 0
		}
	}
};

/* main dialog controls (buttons and text field) */

resource 'CNTL' (kIDSetWindowTitleSetTitleButton, SETWINDOWTITLE_WINDOWNAME": Set Title Button", purgeable)
{
	{ 0, 0, BUTTON_HT, SETTITLE_BUTTON_WD },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	SETWINDOWTITLE_BTNNAME_SETTITLE
};

resource 'CNTL' (kIDSetWindowTitleWindowTitleField, SETWINDOWTITLE_WINDOWNAME": Title Field", purgeable)
{
	{ 0, 0, TEXT_HT_BIG,
		DIALOG_WD - HSP_TEXT_AND_DIALOG - HSP_TEXT_AND_DIALOG - SETTITLE_LABEL_WD - HSP_CONTROLS },
	0,
	visible,
	0,
	0,
	kControlEditTextProc,
	0,
	""
};


/*
*******************************************
	"CONFIGURATION NAME" DIALOG BOX
*******************************************
*/

/* the entire dialog box area */
#define	DIALOG_WD		320
#define	DIALOG_HT		110

#define CREATE_BUTTON_WD	78
#define NAME_LABEL_WD		74

resource 'DLOG' (kDialogIDNewSessionFavoriteName, "Configuration Name", purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowMovableModalGrowProc,
	invisible,
	0, // close box
	0,
	kDialogIDNewSessionFavoriteName,
	"",
	centerParentWindow
};

resource 'dlgx' (kDialogIDNewSessionFavoriteName)
{
	versionZero
	{
		kDialogFlagsUseThemeBackground |
		kDialogFlagsUseControlHierarchy |
		kDialogFlagsHandleMovableModal |
		kDialogFlagsUseThemeControls
	}
};

resource 'DITL' (kDialogIDNewSessionFavoriteName, "Configuration Name", purgeable)
{
	{
		// 1
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDNewSessionFavoriteCreateButton },
		
		// 2
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDCancelButton },
		
		// 3
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDAppleGuideButton },
		
		// 4
		{ VSP_TEXT_AND_DIALOG, HSP_TEXT_AND_DIALOG,
			VSP_TEXT_AND_DIALOG + TEXT_HT_BIG, HSP_TEXT_AND_DIALOG + NAME_LABEL_WD },
		StaticText { enabled, "Name:" },
		
		// 5
		{ VSP_TEXT_AND_DIALOG, HSP_TEXT_AND_DIALOG + NAME_LABEL_WD + HSP_CONTROLS, -1, -1 },
		Control { enabled, kIDNewSessionFavoriteNameField }
	}
};

resource 'hdlg' (kDialogBalloonsIDNewSessionFavoriteName, purgeable)
{
	HelpMgrVersion,
	0, // zero-based dialog item number of first item that has a help balloon
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMSkipItem {/* INCOMPLETE HELP */}, // 1
		HMStringResItem // 2
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_CANCEL,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 3
		{
			{ HELPBUTTON_BALLOON_TIP_V, HELPBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_HELP,
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_DISABLED_BUTTON_HELP,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 4
		HMSkipItem {/* INCOMPLETE HELP */} // 5
	}
};

/* main dialog controls (buttons and text field) */

resource 'CNTL' (kIDNewSessionFavoriteCreateButton, "New "SESSION_FAVORITES_SINGULAR": Create Button", purgeable)
{
	{ 0, 0, BUTTON_HT, CREATE_BUTTON_WD },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	NEWSESSIONFAVORITE_BTNNAME_CREATE
};

resource 'CNTL' (kIDNewSessionFavoriteNameField, "New "SESSION_FAVORITES_SINGULAR": Title Field", purgeable)
{
	{ 0, 0, TEXT_HT_BIG,
		DIALOG_WD - HSP_TEXT_AND_DIALOG - HSP_TEXT_AND_DIALOG - NAME_LABEL_WD - HSP_CONTROLS },
	0,
	visible,
	0,
	0,
	kControlEditTextProc,
	0,
	""
};


/*
*******************************************
	CONFIGURATIONS LIST DIALOG BOX
*******************************************
*/

// set the list height to the item height multiplied by the desired number of items
#define CONFIGURATIONS_LIST_ITEM_HT		18
#define CONFIGURATIONS_LIST_WD			170
#define CONFIGURATIONS_LIST_HT			(CONFIGURATIONS_LIST_ITEM_HT * 12)

/* Configurations list */

resource kControlListDescResType (kIDConfigurationsList)
{
	versionZero
	{
		0, // rows
		1, // columns
		CONFIGURATIONS_LIST_ITEM_HT, // cell height
		0, // cell width
		hasVertScroll,
		noHorizScroll,
		0, // LDEF
		noGrowSpace
	}
};

resource 'CNTL' (kIDConfigurationsList, "Rez-DITL Configurations List", purgeable)
{
	{ 0, 0, CONFIGURATIONS_LIST_HT, CONFIGURATIONS_LIST_WD },
	kIDConfigurationsList, // resource ID of "kControlListDescResType" resource
	visible,
	0,
	0,
	kControlListBoxProc,
	0,
	""
};


/*
*******************************************
	"EDIT FTP LOG-IN MESSAGE" DIALOG BOX
*******************************************
*/

/* the entire dialog box area */
#define	DIALOG_WD		350
#define	DIALOG_HT		230

resource 'DLOG' (kDialogIDEditFTPLogInMessage, "Edit FTP Log-In Message", purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowMovableModalGrowProc,
	invisible,
	0, // close box
	0,
	kDialogIDEditFTPLogInMessage,
	EDITFTPLOGINMESSAGE_WINDOWNAME,
	alertPositionParentWindow
};

resource 'dlgx' (kDialogIDEditFTPLogInMessage)
{
	versionZero
	{
		kDialogFlagsUseThemeBackground |
		kDialogFlagsUseControlHierarchy |
		kDialogFlagsHandleMovableModal |
		kDialogFlagsUseThemeControls
	}
};

resource 'DITL' (kDialogIDEditFTPLogInMessage, "Edit FTP Log-In Message", purgeable)
{
	{
		// 1
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDOKButton },
		
		// 2
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDCancelButton },
		
		// 3
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDAppleGuideButton },
		
		// 4
		{ 0, 0/* auto-sized and auto-positioned control */, -1, -1 },
		Control { enabled, kIDEditableText },
		
		// 5
		{ 0, 0/* auto-sized and auto-positioned control */, -1, -1 },
		Control { enabled, kIDStaticText }
	}
};

resource 'hdlg' (kDialogBalloonsIDEditFTPLogInMessage, purgeable)
{
	HelpMgrVersion,
	0, // zero-based dialog item number of first item that has a help balloon
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMSkipItem {/* no help */}, // 1
		HMStringResItem // 2
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_CANCEL,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 3
		{
			{ HELPBUTTON_BALLOON_TIP_V, HELPBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_HELP,
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_DISABLED_BUTTON_HELP,
			0, 0,
			0, 0
		},
		HMStringResItem // 4
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextEditLogInMessageDialog, BALLOON_DIALOG_EDITFTPLOGINMESSAGE_MAIN_FIELD,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */} // 5
	}
};