/*
**************************************************************************
	TelnetNewSessionsDialog.r
	
	MacTelnet 3.0
	 1998-2000 by Kevin Grant
	
	This file defines the New Sessions dialog box.
**************************************************************************
*/
#ifdef REZ

#include "Appearance.r"
#include "Balloons.r"
#include "Controls.r"
#include "ControlDefinitions.r"
#include "Dialogs.r"
#include "MacTypes.r"
#include "Windows.r"
#include "ControlResources.h"
#include "DialogResources.h"
#include "MenuResources.h"
#include "InterfaceLib.h"

#endif

#include "SpacingConstants.r"
#ifndef __TELNETBALLOONS_R__
#	include "TelnetBalloons.r"
#endif
#include "TelnetButtonNames.h"


/*
*******************************************
	"NEW SESSIONS" DIALOG BOX
*******************************************
*/

/* the entire dialog box area */
#define	NEWSESSIONS_DIALOG_WD		520
#define	NEWSESSIONS_DIALOG_HT		340

#ifdef REZ

#define	DIALOG_WD		NEWSESSIONS_DIALOG_WD
#define	DIALOG_HT		NEWSESSIONS_DIALOG_HT

/* miscellaneous settings for buttons' width, height, and location */
#define ADDBUTTON_WD						70
#define ADDBUTTON_T							VSP_GROUPBOX_AND_DIALOG
#define ADDBUTTON_L							(DIALOG_WD - HSP_BUTTON_AND_DIALOG - ADDBUTTON_WD)
#define POPUPBEVELBUTTONS_T					(ADDBUTTON_T + BUTTON_HT + VSP_BUTTONS)
#define POPUPBEVELBUTTONS_L					ADDBUTTON_L
#define POPUPBEVELBUTTONS_WD				35
#define POPUPBEVELBUTTONS_HT				25

/* Session Info group box info */
#define GROUPBOX_SESSIONINFO_T				VSP_GROUPBOX_AND_DIALOG
#define GROUPBOX_SESSIONINFO_L				HSP_GROUPBOX_AND_DIALOG
#define GROUPBOX_SESSIONINFO_WD				(DIALOG_WD - 2 * HSP_BUTTON_AND_DIALOG - ADDBUTTON_WD - HSP_CONTROLS)
#define GROUPBOX_SESSIONINFO_HT				(VSP_GROUPBOXTEXT_AND_CONTROL + POPUPMENUBUTTON_HT + VSP_CONTROLS + \
												DISCLOSURETRIANGLE_HT + VSP_CONTROLS + SEPARATOR_HT + VSP_CONTROLS + \
												TEXT_HT_BIG + VSP_CONTROLS + LITTLEARROWS_HT + VSP_CONTROLS + \
												LITTLEARROWS_HT + VSP_GROUPBOX_AND_CONTROL)

// helpers
#define GROUPBOX_SESSIONINFO_CONTENT_T		(GROUPBOX_SESSIONINFO_T + VSP_GROUPBOXTEXT_AND_CONTROL)
#define GROUPBOX_SESSIONINFO_CONTENT_L		(GROUPBOX_SESSIONINFO_L + HSP_GROUPBOX_AND_CONTROL)
#define GROUPBOX_SESSIONINFO_CONTENT_B		(GROUPBOX_SESSIONINFO_T + GROUPBOX_SESSIONINFO_HT - VSP_GROUPBOX_AND_CONTROL)
#define GROUPBOX_SESSIONINFO_CONTENT_R		(GROUPBOX_SESSIONINFO_L + GROUPBOX_SESSIONINFO_WD - HSP_GROUPBOX_AND_CONTROL)

// controls in this group box
#define HOSTLABEL_SESSIONINFO_T				(GROUPBOX_SESSIONINFO_CONTENT_T + SPACE_EXTRA_TEXTEDIT)
#define HOSTLABEL_SESSIONINFO_L				GROUPBOX_SESSIONINFO_CONTENT_L
#define HOSTLABEL_SESSIONINFO_WD			65
#define HOSTLABEL_SESSIONINFO_HT			TEXT_HT_BIG
#define HOSTFIELD_SESSIONINFO_T				HOSTLABEL_SESSIONINFO_T
#define HOSTFIELD_SESSIONINFO_L				(HOSTLABEL_SESSIONINFO_L + HOSTLABEL_SESSIONINFO_WD + HSP_CONTROLS + SPACE_EXTRA_TEXTEDIT)
#define HOSTFIELD_SESSIONINFO_WD			190
#define HOSTFIELD_SESSIONINFO_HT			TEXT_HT_BIG
#define PORTLABEL_SESSIONINFO_T				HOSTLABEL_SESSIONINFO_T
#define PORTLABEL_SESSIONINFO_L				(HOSTFIELD_SESSIONINFO_L + HOSTFIELD_SESSIONINFO_WD + SPACE_EXTRA_TEXTEDIT + \
												HSP_CONTROLS)
#define PORTLABEL_SESSIONINFO_WD			40
#define PORTLABEL_SESSIONINFO_HT			TEXT_HT_BIG
#define PORTFIELD_SESSIONINFO_T				HOSTFIELD_SESSIONINFO_T
#define PORTFIELD_SESSIONINFO_L				(PORTLABEL_SESSIONINFO_L + PORTLABEL_SESSIONINFO_WD + HSP_CONTROLS + \
												SPACE_EXTRA_TEXTEDIT)
#define PORTFIELD_SESSIONINFO_WD			(GROUPBOX_SESSIONINFO_CONTENT_R - PORTFIELD_SESSIONINFO_L - SPACE_EXTRA_TEXTEDIT)
#define PORTFIELD_SESSIONINFO_HT			TEXT_HT_BIG
#define TRIANGLE_SESSIONINFO_T				(GROUPBOX_SESSIONINFO_CONTENT_T + BUTTON_HT + VSP_CONTROLS)
#define TRIANGLE_SESSIONINFO_L				GROUPBOX_SESSIONINFO_CONTENT_L
#define TRIANGLELABEL_SESSIONINFO_T			TRIANGLE_SESSIONINFO_T
#define TRIANGLELABEL_SESSIONINFO_L			(TRIANGLE_SESSIONINFO_L + DISCLOSURETRIANGLE_WD + HSP_TRIANGLE_AND_TEXT_LABEL)
#define TRIANGLELABEL_SESSIONINFO_WD		130
#define TRIANGLELABEL_SESSIONINFO_HT		TEXT_HT_SMALL
// note: the disclosure pane is outset by an arbitrary number of pixels, to ensure visual containment (and embedding)
#define DISCLOSED_SESSIONINFO_CONTENT_T		(TRIANGLE_SESSIONINFO_T + DISCLOSURETRIANGLE_HT + VSP_CONTROLS - 5)
#define DISCLOSED_SESSIONINFO_CONTENT_L		(GROUPBOX_SESSIONINFO_CONTENT_L - 5)
#define DISCLOSED_SESSIONINFO_CONTENT_WD	(GROUPBOX_SESSIONINFO_CONTENT_R - GROUPBOX_SESSIONINFO_CONTENT_L + 10)
#define DISCLOSED_SESSIONINFO_CONTENT_HT	(GROUPBOX_SESSIONINFO_CONTENT_B - LITTLEARROWS_HT - VSP_CONTROLS - \
												LITTLEARROWS_HT - VSP_CONTROLS - TEXT_HT_BIG - VSP_CONTROLS - SEPARATOR_HT \
												+ 10)
#define SEPARATOR_SESSIONINFO_T				DISCLOSED_SESSIONINFO_CONTENT_T
#define SEPARATOR_SESSIONINFO_L				DISCLOSED_SESSIONINFO_CONTENT_L
#define SEPARATOR_SESSIONINFO_WD			DISCLOSED_SESSIONINFO_CONTENT_WD
#define SEPARATOR_SESSIONINFO_HT			SEPARATOR_HT
#define WINDOWTITLELABEL_SESSIONINFO_T		(SEPARATOR_SESSIONINFO_T + SEPARATOR_SESSIONINFO_HT + VSP_CONTROLS + SPACE_EXTRA_TEXTEDIT)
#define WINDOWTITLELABEL_SESSIONINFO_L		HOSTLABEL_SESSIONINFO_L
#define WINDOWTITLELABEL_SESSIONINFO_WD		120
#define WINDOWTITLELABEL_SESSIONINFO_HT		TEXT_HT_BIG
#define WINDOWTITLEFIELD_SESSIONINFO_T		(SEPARATOR_SESSIONINFO_T + SEPARATOR_SESSIONINFO_HT + VSP_CONTROLS + SPACE_EXTRA_TEXTEDIT)
#define WINDOWTITLEFIELD_SESSIONINFO_L		(WINDOWTITLELABEL_SESSIONINFO_L + WINDOWTITLELABEL_SESSIONINFO_WD + \
												HSP_CONTROLS + SPACE_EXTRA_TEXTEDIT)
#define WINDOWTITLEFIELD_SESSIONINFO_WD		(GROUPBOX_SESSIONINFO_CONTENT_R - WINDOWTITLEFIELD_SESSIONINFO_L - \
												SPACE_EXTRA_TEXTEDIT)
#define WINDOWTITLEFIELD_SESSIONINFO_HT		TEXT_HT_BIG
#define TERMINALPOPUPMENU_SESSIONINFO_T		(WINDOWTITLELABEL_SESSIONINFO_T + WINDOWTITLELABEL_SESSIONINFO_HT + VSP_CONTROLS + \
												SPACE_EXTRA_TEXTEDIT + (LITTLEARROWS_HT - POPUPMENUBUTTON_HT) / 2)
#define TERMINALPOPUPMENU_SESSIONINFO_L		(WINDOWTITLELABEL_SESSIONINFO_L - 3)
#define TERMINALPOPUPMENU_TITLE_WD			(WINDOWTITLELABEL_SESSIONINFO_WD + HSP_CONTROLS + 3)
#define TERMINALPOPUPMENU_BUTTON_WD			140
#define TERMINALPOPUPMENU_SESSIONINFO_WD	(TERMINALPOPUPMENU_TITLE_WD + TERMINALPOPUPMENU_BUTTON_WD)
#define TERMINALPOPUPMENU_SESSIONINFO_HT	POPUPMENUBUTTON_HT
#define TERMINALARROWS_SESSIONINFO_T		(TERMINALPOPUPMENU_SESSIONINFO_T - (LITTLEARROWS_HT - POPUPMENUBUTTON_HT) / 2)
#define TERMINALARROWS_SESSIONINFO_L		(TERMINALPOPUPMENU_SESSIONINFO_L + TERMINALPOPUPMENU_SESSIONINFO_WD + HSP_CONTROLS)
#define TERMINALARROWS_SESSIONINFO_WD		LITTLEARROWS_WD
#define TERMINALARROWS_SESSIONINFO_HT		LITTLEARROWS_HT
#define PROXYLABEL_SESSIONINFO_T			(TERMINALARROWS_SESSIONINFO_T + TERMINALARROWS_SESSIONINFO_HT + VSP_CONTROLS + \
												(LITTLEARROWS_HT - CHECKBOX_HT) / 2 - 1)
#define PROXYLABEL_SESSIONINFO_L			WINDOWTITLELABEL_SESSIONINFO_L
#define PROXYLABEL_SESSIONINFO_WD			WINDOWTITLELABEL_SESSIONINFO_WD
#define PROXYLABEL_SESSIONINFO_HT			CHECKBOX_HT
#define PROXYPOPUPMENU_SESSIONINFO_T		(TERMINALARROWS_SESSIONINFO_T + TERMINALARROWS_SESSIONINFO_HT + VSP_CONTROLS + \
												(LITTLEARROWS_HT - POPUPMENUBUTTON_HT) / 2)
#define PROXYPOPUPMENU_SESSIONINFO_L		(WINDOWTITLEFIELD_SESSIONINFO_L - SPACE_EXTRA_TEXTEDIT)
#define PROXYPOPUPMENU_SESSIONINFO_WD		TERMINALPOPUPMENU_BUTTON_WD
#define PROXYPOPUPMENU_SESSIONINFO_HT		POPUPMENUBUTTON_HT
#define PROXYARROWS_SESSIONINFO_T			(PROXYPOPUPMENU_SESSIONINFO_T - (LITTLEARROWS_HT - POPUPMENUBUTTON_HT) / 2)
#define PROXYARROWS_SESSIONINFO_L			(PROXYPOPUPMENU_SESSIONINFO_L + PROXYPOPUPMENU_SESSIONINFO_WD + HSP_CONTROLS)
#define PROXYARROWS_SESSIONINFO_WD			LITTLEARROWS_WD
#define PROXYARROWS_SESSIONINFO_HT			LITTLEARROWS_HT

/* list info */
#define LIST_SESSIONS_T						(GROUPBOX_SESSIONINFO_T + GROUPBOX_SESSIONINFO_HT + VSP_CONTROLS)
#define LIST_SESSIONS_L						GROUPBOX_SESSIONINFO_L
#define LIST_SESSIONS_WD					GROUPBOX_SESSIONINFO_WD
#define LIST_SESSIONS_HT					(DIALOG_HT - VSP_GROUPBOX_AND_DIALOG - GROUPBOX_SESSIONINFO_HT - VSP_CONTROLS - \
												VSP_BUTTON_AND_DIALOG - BUTTON_HT - VSP_BUTTON_AND_CONTROL - SEPARATOR_HT - \
												VSP_CONTROLS)
#define REMOVEBUTTON_T						LIST_SESSIONS_T
#define REMOVEBUTTON_L						ADDBUTTON_L
#define REMOVEBUTTON_WD						ADDBUTTON_WD



resource 'DLOG' (kDialogIDNewSessions, NEWSESSIONS_WINDOWNAME, purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowMovableModalGrowProc,
	invisible,
	0, // close box
	0,
	kDialogIDNewSessions,
	NEWSESSIONS_WINDOWNAME,
	alertPositionMainScreen
};

resource 'dlgx' (kDialogIDNewSessions)
{
	versionZero
	{
		kDialogFlagsUseThemeBackground |
		kDialogFlagsUseControlHierarchy |
		kDialogFlagsHandleMovableModal |
		kDialogFlagsUseThemeControls
	}
};

resource 'DITL' (kDialogIDNewSessions, NEWSESSIONS_WINDOWNAME, purgeable)
{
	{
		// 1
		{ 0, 0/* autosized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDNewSessionsConnectButton },
		
		// 2
		{ 0, 0/* autosized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDCancelButton },
		
		// 3
		{ 0, 0/* autosized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDAppleGuideButton },
		
		// 4
		{	DIALOG_HT - VSP_BUTTON_AND_DIALOG - BUTTON_HT - VSP_CONTROLS - SEPARATOR_HT, HSP_CONTROL_AND_DIALOG, -1, -1 },
		Control { disabled, kIDNewSessionsBottomSeparator },
		
		// 5
		{ ADDBUTTON_T, ADDBUTTON_L, -1, -1 },
		Control { enabled, kIDNewSessionsAddButton },
		
		// 6
		{	POPUPBEVELBUTTONS_T, POPUPBEVELBUTTONS_L, -1, -1 },
		Control { enabled, kIDNewSessionsFavoritesPopUpMenuBevelButton },
		
		// 7
		{	POPUPBEVELBUTTONS_T, POPUPBEVELBUTTONS_L + POPUPBEVELBUTTONS_WD, -1, -1 },
		Control { enabled, kIDNewSessionsRecentPopUpMenuBevelButton },
		
		// 8
		{ REMOVEBUTTON_T, REMOVEBUTTON_L, -1, -1 },
		Control { enabled, kIDNewSessionsRemoveButton },
		
		// 9
		{ GROUPBOX_SESSIONINFO_T, GROUPBOX_SESSIONINFO_L, -1, -1 },
		Control { disabled, kIDNewSessionsNewConnectionPrimaryGroup },
		
		// 10
		{	HOSTLABEL_SESSIONINFO_T,
			HOSTLABEL_SESSIONINFO_L,
			HOSTLABEL_SESSIONINFO_T + HOSTLABEL_SESSIONINFO_HT,
			HOSTLABEL_SESSIONINFO_L + HOSTLABEL_SESSIONINFO_WD },
		StaticText { disabled, DIALOGTEXT_NEWSESSIONS_HOST_LABEL },
		
		// 11
		{	HOSTFIELD_SESSIONINFO_T, HOSTFIELD_SESSIONINFO_L, -1, -1 },
		Control { enabled, kIDNewSessionsHostNameField },
		
		// 12
		{	PORTLABEL_SESSIONINFO_T,
			PORTLABEL_SESSIONINFO_L,
			PORTLABEL_SESSIONINFO_T + PORTLABEL_SESSIONINFO_HT,
			PORTLABEL_SESSIONINFO_L + PORTLABEL_SESSIONINFO_WD },
		StaticText { disabled, DIALOGTEXT_NEWSESSIONS_PORT_LABEL },
		
		// 13
		{	PORTFIELD_SESSIONINFO_T, PORTFIELD_SESSIONINFO_L, -1, -1 },
		Control { enabled, kIDNewSessionsPortNumberField },
		
		// 14
		{ TRIANGLE_SESSIONINFO_T, TRIANGLE_SESSIONINFO_L, -1, -1 },
		Control { enabled, kIDDisclosureTriangle },
		
		// 15
		{	TRIANGLELABEL_SESSIONINFO_T,
			TRIANGLELABEL_SESSIONINFO_L,
			TRIANGLELABEL_SESSIONINFO_T + TRIANGLELABEL_SESSIONINFO_HT,
			TRIANGLELABEL_SESSIONINFO_L + TRIANGLELABEL_SESSIONINFO_WD },
		StaticText { disabled, DIALOGTEXT_NEWSESSIONS_SHOWDETAILS },
		
		// 16
		{	DISCLOSED_SESSIONINFO_CONTENT_T, DISCLOSED_SESSIONINFO_CONTENT_L, -1, -1 },
		Control { disabled, kIDNewSessionsSessionInfoDisclosurePane },
		
		// 17
		{	SEPARATOR_SESSIONINFO_T, SEPARATOR_SESSIONINFO_L, -1, -1 },
		Control { disabled, kIDNewSessionsSessionInfoSeparator },
		
		// 18
		{	WINDOWTITLELABEL_SESSIONINFO_T,
			WINDOWTITLELABEL_SESSIONINFO_L,
			WINDOWTITLELABEL_SESSIONINFO_T + WINDOWTITLELABEL_SESSIONINFO_HT,
			WINDOWTITLELABEL_SESSIONINFO_L + WINDOWTITLELABEL_SESSIONINFO_WD },
		StaticText { disabled, DIALOGTEXT_NEWSESSIONS_TITLE_LABEL },
		
		// 19
		{	WINDOWTITLEFIELD_SESSIONINFO_T, WINDOWTITLEFIELD_SESSIONINFO_L, -1, -1 },
		Control { enabled, kIDNewSessionsWindowTitleField },
		
		// 20
		{	TERMINALPOPUPMENU_SESSIONINFO_T, TERMINALPOPUPMENU_SESSIONINFO_L, -1, -1 },
		Control { enabled, kIDNewSessionsTerminalsPopUpMenu },
		
		// 21
		{	TERMINALARROWS_SESSIONINFO_T, TERMINALARROWS_SESSIONINFO_L, -1, -1 },
		Control { enabled, kIDLittleArrows },
		
		// 22
		{	PROXYLABEL_SESSIONINFO_T,
			PROXYLABEL_SESSIONINFO_L,
			PROXYLABEL_SESSIONINFO_T + PROXYLABEL_SESSIONINFO_HT,
			PROXYLABEL_SESSIONINFO_L + PROXYLABEL_SESSIONINFO_WD },
		CheckBox { enabled, NEWSESSIONS_CBNAME_PROXIES },
		
		// 23
		{	PROXYPOPUPMENU_SESSIONINFO_T, PROXYPOPUPMENU_SESSIONINFO_L, -1, -1 },
		Control { enabled, kIDNewSessionsProxiesPopUpMenu },
		
		// 24
		{	PROXYARROWS_SESSIONINFO_T, PROXYARROWS_SESSIONINFO_L, -1, -1 },
		Control { enabled, kIDLittleArrows },
		
		// 25
		{	LIST_SESSIONS_T, LIST_SESSIONS_L + 0, -1, -1 },
		Control { enabled, kIDNewSessionsConnectionsToOpenList }
	}
};

resource 'hdlg' (kDialogBalloonsIDNewSessions, purgeable)
{
	HelpMgrVersion,
	0, // zero-based dialog item number of first item that has a help balloon
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMStringResItem // 1
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_ENABLED_BUTTON_CONNECT,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 2
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_CANCEL,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 3
		{
			{ HELPBUTTON_BALLOON_TIP_V, HELPBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_HELP,
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_DISABLED_BUTTON_HELP,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 4
		HMStringResItem // 5
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_ENABLED_BUTTON_ADD,
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_DISABLED_BUTTON_ADD,
			0, 0,
			0, 0
		},
		HMStringResItem // 6
		{
			{ (POPUPBEVELBUTTONS_HT / 2), (POPUPBEVELBUTTONS_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_ENABLED_BUTTON_FAVORITES,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 7
		{
			{ (POPUPBEVELBUTTONS_HT / 2), (POPUPBEVELBUTTONS_WD / 2) }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_ENABLED_BUTTON_RECENT,
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_DISABLED_BUTTON_RECENT,
			0, 0,
			0, 0
		},
		HMStringResItem // 8
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_ENABLED_BUTTON_REMOVE,
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_DISABLED_BUTTON_REMOVE,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 9
		HMSkipItem {/* no help */}, // 10
		HMStringResItem // 11
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_FIELD_HOST_NAME,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 12
		HMSkipItem {/* INCOMPLETE */}, // 13
		HMSkipItem {/* INCOMPLETE */}, // 14
		HMSkipItem {/* no help */}, // 15
		HMSkipItem {/* no help */}, // 16
		HMSkipItem {/* no help */}, // 17
		HMSkipItem {/* no help */}, // 18
		HMStringResItem // 19
		{
			{ TEXT_BIG_BALLOON_TIP_V, TEXT_BIG_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_FIELD_WINDOW_TITLE,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 20
		{
			{ POPUPMENUBUTTON_BALLOON_TIP_V, POPUPMENUBUTTON_BALLOON_TIP_H + 0 }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_ENABLED_POPUPMENU_TERMINAL,
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_DISABLED_POPUPMENU_TERMINAL,
			0, 0,
			0, 0
		},
		HMStringResItem // 21
		{
			{ LITTLEARROWS_BALLOON_TIP_V, LITTLEARROWS_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_ENABLED_LITTLEARROWS_TERMINAL,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 22
		{
			{ CHECKBOX_BALLOON_TIP_V, CHECKBOX_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_UNCHECKED_CHECKBOX_VIA_PROXY,
			0, 0,
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_CHECKED_CHECKBOX_VIA_PROXY,
			0, 0
		},
		HMStringResItem // 23
		{
			{ POPUPMENUBUTTON_BALLOON_TIP_V, POPUPMENUBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_ENABLED_POPUPMENU_PROXIES,
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_DISABLED_POPUPMENU_PROXIES,
			0, 0,
			0, 0
		},
		HMSkipItem {/* INCOMPLETE */}, // 24
		HMStringResItem // 25
		{
			{ LIST_BALLOON_TIP_V, LIST_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextNewSessionsDialog, BALLOON_DIALOG_NEWSESSIONS_LIST_CONNECTIONS_TO_OPEN,
			0, 0,
			0, 0,
			0, 0
		}
	}
};

resource 'dftb' (kDialogIDNewSessions, "New Sessions: Font Auxiliary Info")
{
	versionZero
	{
		{
			skipItem { }, // 1
			skipItem { }, // 2
			skipItem { }, // 3
			skipItem { }, // 4
			skipItem { }, // 5
			skipItem { }, // 6
			skipItem { }, // 7
			skipItem { }, // 8
			skipItem { }, // 9
			skipItem { }, // 10
			skipItem { }, // 11
			skipItem { }, // 12
			skipItem { }, // 13
			skipItem { }, // 14
			dataItem // 15
			{
				kDialogFontUseFontMask | kDialogFontUseSizeMask | kDialogFontUseFaceMask, // flags determine what data isn't ignored
				kThemeSmallSystemFont, // font
				10, // font size
				0x01/* bold */, // font style
				0, // text mode
				0, // justification
				0, 0, 0, // RGB forecolor
				0, 0, 0, // RGB backcolor
				"" // font name
			},
			skipItem { }, // 16
			skipItem { }, // 17
			skipItem { }, // 18
			skipItem { }, // 19
			skipItem { }, // 20
			skipItem { }, // 21
			skipItem { }, // 22
			skipItem { }, // 23
			skipItem { }, // 24
			skipItem { }, // 25
			skipItem { }, // 26
			skipItem { }, // 27
			skipItem { }, // 28
			skipItem { }, // 29
			skipItem { }, // 30
			skipItem { }, // 31
			skipItem { }, // 32
			skipItem { }, // 33
			skipItem { }, // 34
			skipItem { }, // 35
			skipItem { }, // 36
			skipItem { } // 37
		}
	};
};

/* buttons */

resource 'CNTL' (kIDNewSessionsConnectButton, "New Sessions: "NEWSESSIONS_BTNNAME_CONNECT" Button", purgeable)
{
	{ 0, 0, 0/* auto-sized and auto-positioned button */, 0 },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	NEWSESSIONS_BTNNAME_CONNECT
};

resource 'CNTL' (kIDNewSessionsAddButton, "New Sessions: "NEWSESSIONS_BTNNAME_ADD" Button", purgeable)
{
	{ 0, 0, BUTTON_HT, ADDBUTTON_WD },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	NEWSESSIONS_BTNNAME_ADD
};

resource 'CNTL' (kIDNewSessionsRemoveButton, "New Sessions: "NEWSESSIONS_BTNNAME_REMOVE" Button", purgeable)
{
	{ 0, 0, BUTTON_HT, REMOVEBUTTON_WD },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	NEWSESSIONS_BTNNAME_REMOVE
};

/* pop-up menu controls */

resource 'CNTL' (kIDNewSessionsProxiesPopUpMenu, "New Sessions: Proxies Pop-up Menu", purgeable)
{
	{ 0, 0, PROXYPOPUPMENU_SESSIONINFO_HT, PROXYPOPUPMENU_SESSIONINFO_WD },
	popupTitleLeftJust, // see page 5-119 of IM:MTE for valid values
	visible,
	0, // title width in pixels
	-12345, // menu resource ID (-12345 = use a handle, not a resource)
	kControlPopupButtonProc | kControlPopupFixedWidthVariant,
	0,
	""
};

resource 'CNTL' (kIDNewSessionsTerminalsPopUpMenu, "New Sessions: "NEWSESSIONS_POPUPNAME_TERMINALS" Pop-up Menu", purgeable)
{
	{ 0, 0, TERMINALPOPUPMENU_SESSIONINFO_HT, TERMINALPOPUPMENU_SESSIONINFO_WD },
	popupTitleLeftJust, // see page 5-119 of IM:MTE for valid values
	visible,
	TERMINALPOPUPMENU_TITLE_WD, // title width in pixels
	-12345, // menu resource ID (-12345 = use a handle, not a resource)
	kControlPopupButtonProc | kControlPopupFixedWidthVariant,
	0,
	NEWSESSIONS_POPUPNAME_TERMINALS
};

resource 'CNTL' (kIDNewSessionsFavoritesPopUpMenuBevelButton, "New Sessions: Favorites Pop-up Menu Bevel Button", purgeable)
{
	{ 0, 0, POPUPBEVELBUTTONS_HT, POPUPBEVELBUTTONS_WD },
	kSubmenuIDOpen, // menu ID
	visible,
	kIconSuiteOpenConnectionFavorites, // resource ID of icon contents
	kControlBehaviorPushbutton | kControlBehaviorCommandMenu | kControlContentIconSuiteRes,
	kControlBevelButtonNormalBevelProc,
	0,
	""
};

resource 'CNTL' (kIDNewSessionsRecentPopUpMenuBevelButton, "New Sessions: Recent Pop-up Menu Bevel Button", purgeable)
{
	{ 0, 0, POPUPBEVELBUTTONS_HT, POPUPBEVELBUTTONS_WD },
	kPopUpMenuIDConnectRecent, // menu ID
	visible,
	kIconSuiteOpenConnectionRecent, // resource ID of icon contents
	kControlBehaviorPushbutton | kControlBehaviorCommandMenu | kControlContentIconSuiteRes,
	kControlBevelButtonNormalBevelProc,
	0,
	""
};

/* primary group boxes */

resource 'CNTL' (kIDNewSessionsNewConnectionPrimaryGroup, "New Sessions: "NEWSESSIONS_GROUPBOXNAME_NEWSESSION" Group Box", purgeable)
{
	{ 0, 0, GROUPBOX_SESSIONINFO_HT, GROUPBOX_SESSIONINFO_WD },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	"<session info>"
};

// this control is used to embed all controls hidden or displayed by the disclosure triangle
resource 'CNTL' (kIDNewSessionsSessionInfoDisclosurePane, "New Sessions: "NEWSESSIONS_GROUPBOXNAME_NEWSESSION": Container For Discloseable Controls", purgeable)
{
	{ 0, 0, DISCLOSED_SESSIONINFO_CONTENT_HT, DISCLOSED_SESSIONINFO_CONTENT_WD },
	kControlSupportsEmbedding, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

/* separators */

resource 'CNTL' (kIDNewSessionsSessionInfoSeparator, "New Sessions: "NEWSESSIONS_GROUPBOXNAME_NEWSESSION": Separator", purgeable)
{
	{ 0, 0, SEPARATOR_SESSIONINFO_HT, SEPARATOR_SESSIONINFO_WD },
	0,
	visible,
	0,
	0,
	kControlSeparatorLineProc,
	0,
	""
};

resource 'CNTL' (kIDNewSessionsBottomSeparator, "New Sessions: Bottom Separator", purgeable)
{
	{ 0, 0, SEPARATOR_HT, DIALOG_WD - 2 * HSP_CONTROL_AND_DIALOG },
	0,
	visible,
	0,
	0,
	kControlSeparatorLineProc,
	0,
	""
};

/* text fields */

resource 'CNTL' (kIDNewSessionsHostNameField, "New Sessions: "NEWSESSIONS_GROUPBOXNAME_NEWSESSION": Host Name Field", purgeable)
{
	{ 0, 0, HOSTFIELD_SESSIONINFO_HT, HOSTFIELD_SESSIONINFO_WD },
	0,
	visible,
	0,
	0,
	kControlEditTextProc,
	0,
	""
};

resource 'CNTL' (kIDNewSessionsPortNumberField, "New Sessions: "NEWSESSIONS_GROUPBOXNAME_NEWSESSION": Port Field", purgeable)
{
	{ 0, 0, PORTFIELD_SESSIONINFO_HT, PORTFIELD_SESSIONINFO_WD },
	0,
	visible,
	0,
	0,
	kControlEditTextInlineInputProc,//kControlEditTextProc,
	0,
	""
};

resource 'CNTL' (kIDNewSessionsWindowTitleField, "New Sessions: "NEWSESSIONS_GROUPBOXNAME_NEWSESSION": Window Title Field", purgeable)
{
	{ 0, 0, WINDOWTITLEFIELD_SESSIONINFO_HT, WINDOWTITLEFIELD_SESSIONINFO_WD },
	0,
	visible,
	0,
	0,
	kControlEditTextInlineInputProc,//kControlEditTextProc,
	0,
	""
};

/* Sessions to Open list - temp, this is becoming a Finder List object */

resource kControlListDescResType (kIDNewSessionsConnectionsToOpenList)
{
	versionZero
	{
		0, // rows
		1, // columns
		18, // cell height
		0, // cell width
		hasVertScroll,
		noHorizScroll,
		0, // LDEF
		noGrowSpace
	}
};

resource 'CNTL' (kIDNewSessionsConnectionsToOpenList, "New Sessions: Sessions List", purgeable)
{
	{ 0, 0, LIST_SESSIONS_HT, LIST_SESSIONS_WD },
	kIDNewSessionsConnectionsToOpenList, // resource ID of "kControlListDescResType" resource
	visible,
	0,
	0,
	kControlListBoxProc,
	0,
	""
};

#endif
