/*
**************************************************************************
	TelnetPrefsDialog.r
	
	MacTelnet 3.0
	 1998-2003 by Kevin Grant
	
	This file contains data for the new Preferences tabbed dialog,
	describing the controls in the dialog and in all five tab panes.
**************************************************************************
*/

#ifdef REZ

#include "ControlDefinitions.r"
#include "Controls.r"
#include "Dialogs.r"
#include "MacTypes.r"
#include "Windows.r"

#endif

#include "ControlResources.h"
#include "DialogResources.h"
#include "InterfaceLib.h"
#include "MenuResources.h"
#include "SpacingConstants.r"
#include "TelnetButtonNames.h"
#include "TelnetMiscellaneousText.h"


// the entire dialog box area
#define	PREFERENCES_DIALOG_WD	500
#define	PREFERENCES_DIALOG_HT	450

// main areas in the dialog
#define CATEGORYLISTCELL_HT		60

#define CATEGORYUSERPANE_L		-1
#define CATEGORYUSERPANE_T		-1
#define CATEGORYUSERPANE_WD		100
#define CATEGORYUSERPANE_HT		(PREFERENCES_DIALOG_HT + 2)

#define CATEGORYLISTBOX_L		(CATEGORYUSERPANE_L + HSP_CONTROL_AND_DIALOG)
#define CATEGORYLISTBOX_T		(CATEGORYUSERPANE_T + VSP_CONTROL_AND_DIALOG)
#define CATEGORYLISTBOX_WD		(CATEGORYUSERPANE_WD - 2 * HSP_CONTROL_AND_DIALOG)
	// the ListBox height has the % stuff so that its a multiple of the cell height
#define CATEGORYLISTBOX_HT		( (CATEGORYUSERPANE_HT - VSP_CONTROL_AND_DIALOG - \
									VSP_BUTTON_AND_DIALOG - BUTTON_HT - \
									VSP_BUTTON_AND_CONTROL) - \
									( (CATEGORYUSERPANE_HT - VSP_CONTROL_AND_DIALOG - \
									VSP_BUTTON_AND_DIALOG - BUTTON_HT - \
									VSP_BUTTON_AND_CONTROL) % CATEGORYLISTCELL_HT ) )

#define SEPARATORLINE_L			(CATEGORYUSERPANE_L + CATEGORYUSERPANE_WD + 1)
#define SEPARATORLINE_T			(VSP_CONTROL_AND_DIALOG + 1) // offset to line up background better
#define SEPARATORLINE_WD		SEPARATOR_WD
#define SEPARATORLINE_HT		(DIALOG_HT - 2 * VSP_CONTROL_AND_DIALOG)

#define PANELUSERPANE_L			(CATEGORYUSERPANE_L + CATEGORYUSERPANE_WD + SEPARATORLINE_WD + \
									12)
#define PANELUSERPANE_T			CATEGORYUSERPANE_T
#define PANELUSERPANE_WD		(PREFERENCES_DIALOG_WD - PANELUSERPANE_L)
#define PANELUSERPANE_HT		CATEGORYUSERPANE_HT

#define BEVELBUTTON_WD			56
#define BEVELBUTTON_HT			48

#ifdef REZ

#define DIALOG_WD	PREFERENCES_DIALOG_WD
#define DIALOG_HT	PREFERENCES_DIALOG_HT

resource 'WIND' (kWindowIDPreferences, "General Prefs", purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowGrowDocumentProc,
	invisible,
	goAway, // close box
	0,
	PREFERENCES_WINDOWNAME,
	centerMainScreen
};

resource kControlListDescResType (kIDPrefCategoryListBox, purgeable)
{
	versionZero
	{
		0,										// rows
		1,										// columns
		CATEGORYLISTCELL_HT,					// cell height (18 for text-only list)
		0,										// cell width (set programmatically)
		hasVertScroll,
		noHorizScroll,
		128, // LDEF ID compiled for IconListDef (make sure its built this way!)
		noGrowSpace
	}
};

resource 'CNTL' (kIDPrefCategoryListBox, "Preferences: Panel Choice List Box", purgeable)
{
	{	CATEGORYLISTBOX_T,
		CATEGORYLISTBOX_L,
		CATEGORYLISTBOX_T + CATEGORYLISTBOX_HT,
		CATEGORYLISTBOX_L + CATEGORYLISTBOX_WD },
	kIDPrefCategoryListBox,		// resource ID of "kControlListDescResType" resource
	visible,
	0,
	0,
	kControlListBoxProc,
	0,
	""
};

/* panel tab information */

resource kControlTabListResType (kIDPrefGeneralTabs, "Preferences: General: Tab Auxiliary Info")
{
	versionZero
	{
		{
			0, PREFERENCES_GENERAL_TABNAME_OPTIONS;
			0, PREFERENCES_GENERAL_TABNAME_SPECIAL;
			0, PREFERENCES_GENERAL_TABNAME_COLOR;
			0, PREFERENCES_GENERAL_TABNAME_NOTIFICATION
		}
	};
};

resource kControlTabListResType (kIDPrefFileTransfersTabs, "Preferences: File Transfers: Tab Auxiliary Info")
{
	versionZero
	{
		{
			0, PREFERENCES_FTP_TABNAME_OPTIONS;
			0, PREFERENCES_FTP_TABNAME_FILETYPES;
			0, PREFERENCES_FTP_TABNAME_SERVER
		}
	};
};

/* main preferences dialog controls */

resource 'CNTL' (kIDPrefBevelButtonPlacard, "Preferences: Bevel Button Placard", purgeable)
{
	{	CATEGORYUSERPANE_T,
		CATEGORYUSERPANE_L,
		CATEGORYUSERPANE_T + CATEGORYUSERPANE_HT,
		CATEGORYUSERPANE_L + CATEGORYUSERPANE_WD },
	0,
	visible,
	0,
	0,
	kControlPlacardProc/*kControlWindowHeaderProc*/,
	0,
	""
};

resource 'CNTL' (kIDPrefPanelPlacard, "Preferences: Panel User Pane", purgeable)
{
	{	PANELUSERPANE_T,
		PANELUSERPANE_L,
		PANELUSERPANE_T + PANELUSERPANE_HT,
		PANELUSERPANE_L + PANELUSERPANE_WD },
	kControlSupportsEmbedding | kControlWantsActivate, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

resource 'CNTL' (kIDPrefGenericBevelButton, "Preferences: Bevel Button", purgeable)
{
	{ 0, 0, BEVELBUTTON_HT, BEVELBUTTON_WD },
	0,
	visible,
	0,
	kControlBehaviorPushbutton | kControlContentIconSuiteHandle/* content type altered programmatically */,
	kControlBevelButtonSmallBevelProc,
	0,
	""
};

/* General panel controls */

resource 'CNTL' (kIDPrefGeneralTabs, "Preferences: General: Tab Control", purgeable)
{
	// relative to the parent controls origin
	{	VSP_TAB_AND_DIALOG,
		0,
		VSP_TAB_AND_DIALOG + (PANELUSERPANE_HT - VSP_TAB_AND_DIALOG),
		PANELUSERPANE_WD },
	kIDPrefGeneralTabs,
	visible,
	0,
	0,
	kControlTabLargeNorthProc,
	0,
	""
};

// Special pane

resource 'CNTL' (kIDPrefGeneralCursorGroupBox, "Preferences: "PREFERENCES_GENERAL_GROUPBOXNAME_CURSOR" Group Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	PREFERENCES_GENERAL_GROUPBOXNAME_CURSOR
};

resource 'CNTL' (kIDPrefGeneralCursorShapeBlockBevelButton, "Preferences: Block Cursor Shape Bevel Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	kIconSuiteCursorShapeBlock, // icon resource ID
	kControlBehaviorSticky | kControlContentIconSuiteRes,
	kControlBevelButtonNormalBevelProc,
	0,
	""
};

resource 'CNTL' (kIDPrefGeneralCursorShapeVerticalBarBevelButton, "Preferences: Vertical Bar Cursor Shape Bevel Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	kIconSuiteCursorShapeVerticalBar, // icon resource ID
	kControlBehaviorSticky | kControlContentIconSuiteRes,
	kControlBevelButtonNormalBevelProc,
	0,
	""
};

resource 'CNTL' (kIDPrefGeneralCursorShapeUnderlineBevelButton, "Preferences: Underline Cursor Shape Bevel Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	kIconSuiteCursorShapeUnderline, // icon resource ID
	kControlBehaviorSticky | kControlContentIconSuiteRes,
	kControlBevelButtonNormalBevelProc,
	0,
	""
};

resource 'CNTL' (kIDPrefGeneralCursorShapeThickVerticalBarBevelButton, "Preferences: Thick Vertical Bar Cursor Shape Bevel Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	kIconSuiteCursorShapeThickVerticalBar, // icon resource ID
	kControlBehaviorSticky | kControlContentIconSuiteRes,
	kControlBevelButtonNormalBevelProc,
	0,
	""
};

resource 'CNTL' (kIDPrefGeneralCursorShapeThickUnderlineBevelButton, "Preferences: Thick Underline Cursor Shape Bevel Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	kIconSuiteCursorShapeThickUnderline, // icon resource ID
	kControlBehaviorSticky | kControlContentIconSuiteRes,
	kControlBevelButtonNormalBevelProc,
	0,
	""
};

resource 'CNTL' (kIDPrefGeneralTimeoutsGroupBox, "Preferences: "PREFERENCES_GENERAL_GROUPBOXNAME_TIMEOUTS" Group Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	PREFERENCES_GENERAL_GROUPBOXNAME_TIMEOUTS
};

resource 'CNTL' (kIDPrefGeneralCopyTableGroupBox, "Preferences: "PREFERENCES_GENERAL_GROUPBOXNAME_COPYTABLE" Group Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	PREFERENCES_GENERAL_GROUPBOXNAME_COPYTABLE
};

resource 'CNTL' (kIDPrefGeneralCaptureFileGroupBox, "Preferences: "PREFERENCES_GENERAL_GROUPBOXNAME_CAPTUREFILE" Group Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	PREFERENCES_GENERAL_GROUPBOXNAME_CAPTUREFILE
};

resource 'CNTL' (kIDPrefGeneralAutoCalcANSIColorsButton, "Preferences: "PREFERENCES_GENERAL_BTNNAME_ANSICOLORSDEFAULT" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	PREFERENCES_GENERAL_BTNNAME_ANSICOLORSDEFAULT
};

// Notification pane

resource 'CNTL' (kIDPrefGeneralBackgroundNotificationGroupBox, "Preferences: "PREFERENCES_GENERAL_GROUPBOXNAME_NOTIFICATION" Group Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	PREFERENCES_GENERAL_GROUPBOXNAME_NOTIFICATION
};

resource 'CNTL' (kIDPrefGeneralBackgroundNotificationBeepsCheckBox, "Preferences: "PREFERENCES_GENERAL_CBNAME_NOTIFYOFBEEPS" Check Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	kControlCheckBoxUncheckedValue, // value
	visible,
	kControlCheckBoxMixedValue, // maximum
	kControlCheckBoxUncheckedValue, // minimum
	kControlCheckBoxProc,
	0,
	PREFERENCES_GENERAL_CBNAME_NOTIFYOFBEEPS
};

resource 'CNTL' (kIDPrefGeneralNotificationVisualBellCheckBox, "Preferences: "PREFERENCES_GENERAL_CBNAME_VISUALBELL" Check Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	kControlCheckBoxUncheckedValue, // value
	visible,
	kControlCheckBoxMixedValue, // maximum
	kControlCheckBoxUncheckedValue, // minimum
	kControlCheckBoxProc,
	0,
	PREFERENCES_GENERAL_CBNAME_VISUALBELL
};

/* Configurations panel controls */

resource 'CNTL' (kIDPrefConfigurationsTabs, "Preferences: Configurations Tabs", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	kIDPrefConfigurationsTabs,
	visible,
	0,
	0,
	kControlTabLargeNorthProc,
	0,
	""
};

resource kControlTabListResType (kIDPrefConfigurationsTabs, "Preferences: Configurations: Tab Auxiliary Info")
{
	versionZero
	{
		{
			0, SESSION_FAVORITES_AS_TITLE;
			0, TERMINAL_PLURAL_AS_TITLE;
			0, PROXY_PLURAL_AS_TITLE;
			0, FTP_USER_PLURAL_AS_TITLE
		}
	};
};

resource kControlListDescResType (kIDPrefConfigurationsListBox, purgeable)
{
	versionZero
	{
		0, // rows
		1, // columns
		18, // cell height
		0, // cell width
		hasVertScroll,
		noHorizScroll,
		0, // LDEF
		noGrowSpace
	}
};

resource 'CNTL' (kIDPrefConfigurationsListBox, "Preferences: Configurations List Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	kIDPrefConfigurationsListBox, // resource ID of "kControlListDescResType" resource
	visible,
	0,
	0,
	kControlListBoxProc,
	0,
	""
};

resource 'CNTL' (kIDPrefConfigurationsNewButton, "Preferences: "PREFERENCES_CONFIGURATIONS_BTNNAME_NEW" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	PREFERENCES_CONFIGURATIONS_BTNNAME_NEW
};

resource 'CNTL' (kIDPrefConfigurationsNewGroupButton, "Preferences: "PREFERENCES_CONFIGURATIONS_BTNNAME_NEWGROUP" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	PREFERENCES_CONFIGURATIONS_BTNNAME_NEWGROUP
};

resource 'CNTL' (kIDPrefConfigurationsRemoveButton, "Preferences: "PREFERENCES_CONFIGURATIONS_BTNNAME_REMOVE" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	PREFERENCES_CONFIGURATIONS_BTNNAME_REMOVE
};

resource 'CNTL' (kIDPrefConfigurationsDuplicateButton, "Preferences: "PREFERENCES_CONFIGURATIONS_BTNNAME_DUPLICATE" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	PREFERENCES_CONFIGURATIONS_BTNNAME_DUPLICATE
};

resource 'CNTL' (kIDPrefConfigurationsEditButton, "Preferences: "PREFERENCES_CONFIGURATIONS_BTNNAME_EDIT" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	PREFERENCES_CONFIGURATIONS_BTNNAME_EDIT
};

/* Macros panel controls */

resource 'CNTL' (kIDPrefMacrosTabs, "Preferences: Macro Set Tabs", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	kIDPrefMacrosTabs,
	visible,
	0,
	0,
	kControlTabLargeNorthProc,
	0,
	""
};

resource kControlTabListResType (kIDPrefMacrosTabs, "Preferences: Macros: Tab Auxiliary Info")
{
	versionZero
	{
		{
			0, PREFERENCES_MACROS_TABNAME_SET1;
			0, PREFERENCES_MACROS_TABNAME_SET2;
			0, PREFERENCES_MACROS_TABNAME_SET3;
			0, PREFERENCES_MACROS_TABNAME_SET4;
			0, PREFERENCES_MACROS_TABNAME_SET5
		}
	};
};

resource 'CNTL' (kIDPrefUseSetAboveButton, "Preferences: "MACROSETUP_BTNNAME_USESETABOVE" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	MACROSETUP_BTNNAME_USESETABOVE
};

/* File Transfers panel controls */

resource 'CNTL' (kIDPrefFileTransfersTabs, "Preferences: File Transfers: Tab Control", purgeable)
{
	// relative to the parent controls origin
	{	VSP_TAB_AND_DIALOG,
		0,
		VSP_TAB_AND_DIALOG + (PANELUSERPANE_HT - VSP_TAB_AND_DIALOG),
		PANELUSERPANE_WD },
	kIDPrefFileTransfersTabs,
	visible,
	0,
	0,
	kControlTabLargeNorthProc,
	0,
	""
};

// File Types pane

resource 'CNTL' (kIDPrefFileTransfersTextFilesGroupBox, "Preferences: "PREFERENCES_FTP_GROUPBOXNAME_TEXTFILES" Group Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	PREFERENCES_FTP_GROUPBOXNAME_TEXTFILES
};

resource 'CNTL' (kIDPrefFileTransfersBinaryFilesGroupBox, "Preferences: "PREFERENCES_FTP_GROUPBOXNAME_BINARYFILES" Group Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	0,
	visible,
	0,
	0,
	kControlGroupBoxTextTitleProc,
	0,
	PREFERENCES_FTP_GROUPBOXNAME_BINARYFILES
};

resource 'CNTL' (kIDPrefFileTransfersTextFilesFromAppButton, "Preferences: "PREFERENCES_FTP_BTNNAME_FROMAPP" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	PREFERENCES_FTP_BTNNAME_FROMAPP
};

resource 'CNTL' (kIDPrefFileTransfersBinaryFilesExampleButton, "Preferences: "PREFERENCES_FTP_BTNNAME_EXAMPLEFILE" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	PREFERENCES_FTP_BTNNAME_EXAMPLEFILE
};

// Server pane

resource 'CNTL' (kIDPrefFileTransfersServerUserSetupButton, "Preferences: "PREFERENCES_FTP_BTNNAME_REGISTEREDUSERSETUP" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	kColorIconPrefsFTPUsers, // 'cicn' resource ID
	0,
	kControlPushButLeftIconProc,
	0,
	PREFERENCES_FTP_BTNNAME_REGISTEREDUSERSETUP
};

resource 'CNTL' (kIDPrefFileTransfersServerEditLogInMessageButton, "Preferences: "PREFERENCES_FTP_BTNNAME_EDITLOGINMESSAGE" Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned button */ },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	PREFERENCES_FTP_BTNNAME_EDITLOGINMESSAGE
};

#endif
