/*
**************************************************************************
	TelnetReusableControls.r
	
	MacTelnet 3.0
	 1998-2000 by Kevin Grant
	
	This file contains data for generic controls.
**************************************************************************
*/

#include "Controls.r"
#include "ControlDefinitions.r"
#include "MacTypes.r"

#include "ControlResources.h"
#include "SpacingConstants.r"
#include "TelnetButtonNames.h"


/*
A user pane for no other purpose than to act as an anchor for
overlaid dialog item lists.
*/
resource 'CNTL' (kIDAnchorUserPane, "Anchor", purgeable)
{
	{ 0, 0, 1, 1 },
	kControlSupportsEmbedding | kControlWantsActivate | kControlWantsIdle, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};


/*
A pair of chasing arrows, which animate in a circular
fashion automatically.
*/
resource 'CNTL' (kIDAsynchronousArrows, "Asynchronous Arrows", purgeable)
{
	{ 0, 0, CHASING_ARROWS_HT, CHASING_ARROWS_WD },
	0,
	visible,
	0,
	0,
	kControlChasingArrowsProc,
	0,
	""
};


/*
This is a standard Cancel button that is 68 pixels wide.
*/
resource 'CNTL' (kIDCancelButton, "Cancel Button", purgeable)
{
	{ 0, 0, BUTTON_HT, 68 },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	REUSABLE_BTNNAME_CANCEL
};


/*
This is a check box control with no title.  It is important to
initialize this control to have the proper maximum and minimum
values, hence the definition as a reusable control.
*/
resource 'CNTL' (kIDCheckBox, "Generic Check Box", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	kControlCheckBoxUncheckedValue, // value
	visible,
	kControlCheckBoxMixedValue, // maximum
	kControlCheckBoxUncheckedValue, // minimum
	kControlCheckBoxProc,
	0,
	""
};


/*
A user pane for clickable color boxes, such as ANSI colors.
The drawing and tracking routines must be installed
programmatically, however.

WARNING: see Rez-DITL versions for old dialogs!
*/
resource 'CNTL' (kIDColorBox, "Color Box", purgeable)
{
	{ 0, 0, COLORBOX_HT, COLORBOX_WD },
	kControlWantsActivate |
	kControlHandlesTracking, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};


/*
A disclosure triangle that initially points to the right
(in left-to-right locales).
*/
resource 'CNTL' (kIDDisclosureTriangle, "Disclosure Triangle", purgeable)
{
	{ 0, 0, DISCLOSURETRIANGLE_HT, DISCLOSURETRIANGLE_WD },
	0,
	visible,
	0,
	0,
	kControlTriangleAutoToggleProc,
	0,
	""
};


/*
A disclosure triangle that initially points downwards,
and points to the right (in left-to-right locales) when
turned off.
*/
resource 'CNTL' (kIDDisclosureTriangleDown, "Disclosure Triangle, Down-Pointing Initially", purgeable)
{
	{ 0, 0, DISCLOSURETRIANGLE_HT, DISCLOSURETRIANGLE_WD },
	1,
	visible,
	0,
	0,
	kControlTriangleAutoToggleProc,
	0,
	""
};


/*
A disclosure triangle that initially points downwards,
and points to the left (in left-to-right locales) when
turned off.
*/
resource 'CNTL' (kIDDisclosureTriangleDownReversed, "Disclosure Triangle, Down-Pointing Initially, Reversed", purgeable)
{
	{ 0, 0, DISCLOSURETRIANGLE_HT, DISCLOSURETRIANGLE_WD },
	1,
	visible,
	0,
	0,
	kControlTriangleLeftFacingAutoToggleProc,
	0,
	""
};


/*
A disclosure triangle that initially points to the left
(in left-to-right locales).
*/
resource 'CNTL' (kIDDisclosureTriangleReversed, "Disclosure Triangle Reversed", purgeable)
{
	{ 0, 0, DISCLOSURETRIANGLE_HT, DISCLOSURETRIANGLE_WD },
	0,
	visible,
	0,
	0,
	kControlTriangleLeftFacingAutoToggleProc,
	0,
	""
};


/*
An editable text control that can be used for anything.  Before using
it, it must be sized properly and have its text contents specified.  If
it is used in a dialog box, its font information can be specified using
an entry in the 'dftb' resource.
*/
resource 'CNTL' (kIDEditableText, "Editable Text", purgeable)
{
	{ 0, 0, TEXT_HT_BIG, 0 },
	0,
	visible,
	0,
	0,
	kControlEditTextProc,
	0,
	""
};


/*
This is a secondary group box control with no title, commonly
used as a border for help text in a window.
*/
resource 'CNTL' (kIDHelpTextGroupBox, "Help Text Group Box", purgeable)
{
	{ 0, 0, 0/* auto-sized, auto-positioned control */, 0 },
	0,
	visible,
	0,
	0,
	kControlGroupBoxSecondaryTextTitleProc,
	0,
	""
};


/*
A little arrows control for adjusting things.  This control is
always exactly the same size and is used in only one way, so it
is obviously a standard, reusable control.
*/
resource 'CNTL' (kIDLittleArrows, "Little Arrows", purgeable)
{
	{ 0, 0, LITTLEARROWS_HT, LITTLEARROWS_WD },
	0,
	visible,
	0,
	0,
	kControlLittleArrowsProc,
	0,
	""
};


/*
This is a standard OK button that is 68 pixels wide.
*/
resource 'CNTL' (kIDOKButton, "OK Button", purgeable)
{
	{ 0, 0, BUTTON_HT, 68 },
	0,
	visible,
	0,
	0,
	kControlPushButtonProc,
	0,
	REUSABLE_BTNNAME_OK
};


/*
A user pane for no other purpose than to act as a generic panel.  It
is initially invisible because its contents are generated invisibly.
*/
resource 'CNTL' (kIDPanelUserPane, "Panel", purgeable)
{
	{ 0, 0, 1, 1 },
	kControlSupportsEmbedding, // control feature bits
	invisible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};


/*
This is a radio button control with no title.  It is important
to initialize this control to have the proper maximum and minimum
values, hence the definition as a reusable control.
*/
resource 'CNTL' (kIDRadioButton, "Generic Radio Button", purgeable)
{
	{ 0, 0, 0, 0 /* auto-sized and auto-positioned control */ },
	kControlRadioButtonUncheckedValue, // value
	visible,
	kControlRadioButtonMixedValue, // maximum
	kControlRadioButtonUncheckedValue, // minimum
	kControlRadioButtonProc,
	0,
	""
};


/*
A separator control that can be used for anything.  Before using it,
it must be sized properly.  The standard height or width of a line
is 4 pixels, depending on its orientation.
*/
resource 'CNTL' (kIDSeparator, "Separator", purgeable)
{
	{ 0, 0, 0, 0/* auto-sized and auto-positioned control */ },
	0,
	visible,
	0,
	0,
	kControlSeparatorLineProc,
	0,
	""
};


/*
A static text control that can be used for anything.  Before using it,
it must be sized properly and have its text contents specified.  If it
is used in a dialog box, its font information can be specified using
an entry in the 'dftb' resource.
*/
resource 'CNTL' (kIDStaticText, "Static Text", purgeable)
{
	{ 0, 0, TEXT_HT_BIG, 0 },
	0,
	visible,
	0,
	0,
	kControlStaticTextProc,
	0,
	""
};


/*
A user pane that draws a small status icon.
*/
resource 'CNTL' (kIDStatusIcon, "Status Icon", purgeable)
{
	{ 0, 0, ICON_HT_SMALL, ICON_WD_SMALL },
	kControlWantsIdle | kControlWantsActivate, // control feature bits
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};



/*
A user pane for no other purpose than to act as a tab container.  It
is initially invisible because its contents are generated invisibly.
*/
resource 'CNTL' (kIDTabUserPane, "Tab Pane", purgeable)
{
	{ 0, 0, 1, 1 },
	kControlSupportsEmbedding, // control feature bits
	invisible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};
