/*
**************************************************************************
	TelnetSizeDialog.r
	
	MacTelnet 3.0
	 1998-2000 by Kevin Grant
	
	This file defines the Format dialog box.
**************************************************************************
*/
#ifdef REZ

#include "Appearance.r"
#include "Balloons.r"
#include "Controls.r"
#include "ControlDefinitions.r"
#include "Dialogs.r"
#include "MacTypes.r"
#include "Windows.r"

#endif

#include "ControlResources.h"
#include "DialogResources.h"
#include "MenuResources.h"
#include "InterfaceLib.h"
#include "SpacingConstants.r"
#ifndef __TELNETBALLOONS_R__
#	include "TelnetBalloons.r"
#endif
#include "TelnetButtonNames.h"


/*
*******************************************
	TERMINAL SCREEN SIZE DIALOG BOX
*******************************************
*/

/* the entire dialog box area */
#define SIZE_DIALOG_WD		270
#define	SIZE_DIALOG_HT		170

#ifdef REZ

#define DIALOG_WD	SIZE_DIALOG_WD
#define DIALOG_HT	SIZE_DIALOG_HT

/* useful values */
#define WIDTHHEIGHT_LABEL_WD			55
#define WIDTHHEIGHT_LABEL_L				HSP_TEXT_AND_DIALOG
#define WIDTHHEIGHT_FIELD_WD			50
#define WIDTHHEIGHT_FIELD_L				(WIDTHHEIGHT_LABEL_L + WIDTHHEIGHT_LABEL_WD + HSP_CONTROLS + SPACE_EXTRA_TEXTEDIT)
#define WIDTHHEIGHT_ARROWS_L			(WIDTHHEIGHT_FIELD_L + WIDTHHEIGHT_FIELD_WD + SPACE_EXTRA_TEXTEDIT + HSP_CONTROLS)
#define WIDTHHEIGHT_UNITS_WD			55
#define WIDTHHEIGHT_UNITS_L				(DIALOG_WD - 2 * HSP_TEXT_AND_DIALOG - WIDTHHEIGHT_UNITS_WD)
#define WIDTHHEIGHT_CONTROLS_HT			(TEXT_HT_BIG + 2 * SPACE_EXTRA_TEXTEDIT)
#define WIDTH_CONTROLS_T				(VSP_TEXT_AND_DIALOG + SPACE_EXTRA_TEXTEDIT)
#define HEIGHT_CONTROLS_T				(WIDTH_CONTROLS_T + WIDTHHEIGHT_CONTROLS_HT + VSP_CONTROLS)

/* other controls dimensions */
#define BOTTOM_CHECKBOX_T				(HEIGHT_CONTROLS_T + WIDTHHEIGHT_CONTROLS_HT + VSP_CONTROLS)
#define BOTTOM_CHECKBOX_L				HSP_CONTROL_AND_DIALOG
#define BOTTOM_CHECKBOX_B				(BOTTOM_CHECKBOX_T + CHECKBOX_HT)
#define BOTTOM_CHECKBOX_R				(DIALOG_WD - HSP_CONTROL_AND_DIALOG)



resource 'DLOG' (kDialogIDScreenSize, SCREENSIZE_WINDOWNAME, purgeable)
{
	{ 0, 0, DIALOG_HT, DIALOG_WD },
	kWindowMovableModalGrowProc,
	invisible,
	0, // close box
	0,
	kDialogIDScreenSize,
	SCREENSIZE_WINDOWNAME,
	centerParentWindow
};

resource 'dlgx' (kDialogIDScreenSize)
{
	versionZero
	{
		kDialogFlagsUseThemeBackground		|
		kDialogFlagsUseControlHierarchy		|
		kDialogFlagsHandleMovableModal		|
		kDialogFlagsUseThemeControls
	}
};

resource 'DITL' (kDialogIDScreenSize, SCREENSIZE_WINDOWNAME, purgeable)
{
	{
		// 1
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDOKButton },
		
		// 2
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDCancelButton },
		
		// 3
		{ 0, 0/* auto-sized and auto-positioned button */, -1, -1 },
		Control { enabled, kIDAppleGuideButton },
		
		// 4
		{ BOTTOM_CHECKBOX_T, BOTTOM_CHECKBOX_L, BOTTOM_CHECKBOX_B, BOTTOM_CHECKBOX_R },
		CheckBox { enabled, "" }, // this dialog item is not used
		
		// 5
		{ WIDTH_CONTROLS_T, WIDTHHEIGHT_LABEL_L, -1, -1 },
		Control { enabled, kIDStaticText },
		
		// 6
		{ WIDTH_CONTROLS_T, WIDTHHEIGHT_FIELD_L, -1, -1 },
		Control { disabled, kIDScreenSizeEditDimensionField },
		
		// 7
		{ WIDTH_CONTROLS_T - (LITTLEARROWS_HT - TEXT_HT_BIG) / 2, WIDTHHEIGHT_ARROWS_L, -1, -1 },
		Control { enabled, kIDLittleArrows },
		
		// 8
		{ WIDTH_CONTROLS_T, WIDTHHEIGHT_UNITS_L, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 9
		{ HEIGHT_CONTROLS_T, WIDTHHEIGHT_LABEL_L, -1, -1 },
		Control { disabled, kIDStaticText },
		
		// 10
		{ HEIGHT_CONTROLS_T, WIDTHHEIGHT_FIELD_L, -1, -1 },
		Control { disabled, kIDScreenSizeEditDimensionField },
		
		// 11
		{ HEIGHT_CONTROLS_T - (LITTLEARROWS_HT - TEXT_HT_BIG) / 2, WIDTHHEIGHT_ARROWS_L, -1, -1 },
		Control { enabled, kIDLittleArrows },
		
		// 12
		{ HEIGHT_CONTROLS_T, WIDTHHEIGHT_UNITS_L, -1, -1 },
		Control { enabled, kIDStaticText }
	}
};

resource 'hdlg' (kDialogBalloonsIDScreenSize, purgeable)
{
	HelpMgrVersion,
	0, // zero-based dialog item number of first item that has a help balloon
	0,
	0,
	0,
	HMSkipItem {}, // no missing item string
	{
		HMSkipItem {/* no help */}, // 1
		HMStringResItem // 2
		{
			{ BUTTON_BALLOON_TIP_V, BUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_CANCEL,
			0, 0,
			0, 0,
			0, 0
		},
		HMStringResItem // 3
		{
			{ HELPBUTTON_BALLOON_TIP_V, HELPBUTTON_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_ENABLED_BUTTON_HELP,
			rStringsBalloonTextGenericDialogs, BALLOON_GENERIC_DIALOG_DISABLED_BUTTON_HELP,
			0, 0,
			0, 0
		},
		HMStringResItem // 4
		{
			{ CHECKBOX_BALLOON_TIP_V, CHECKBOX_BALLOON_TIP_H }, // tip point (v, h), relative to the location of the dialog item
			{ 0, 0, 0, 0 }, // hot rectangle, relative to the location of the dialog item - only needed in special cases
			rStringsBalloonTextGenericSessions, BALLOON_GENERIC_CONNECTIONS_UNCHECKED_CHECKBOX_UPDATEDEFTERM,
			0, 0,
			0, 0,
			0, 0
		},
		HMSkipItem {/* no help */}, // 5
		HMSkipItem {/* INCOMPLETE help */}, // 6
		HMSkipItem {/* INCOMPLETE help */}, // 7
		HMSkipItem {/* no help */}, // 8
		HMSkipItem {/* no help */}, // 9
		HMSkipItem {/* INCOMPLETE help */}, // 10
		HMSkipItem {/* INCOMPLETE help */}, // 11
		HMSkipItem {/* no help */} // 12
	}
};

resource 'dftb' (kDialogIDScreenSize, SCREENSIZE_WINDOWNAME": Font Auxiliary Info")
{
	versionZero
	{
		{
			skipItem { }, // 1
			skipItem { }, // 2
			skipItem { }, // 3
			skipItem { }, // 4
			skipItem { }, // 5
			skipItem { }, // 6
			skipItem { }, // 7
			skipItem { }, // 8
			skipItem { }, // 9
			skipItem { }, // 10
			skipItem { }, // 11
			skipItem { } // 12
		}
	};
};

resource 'CNTL' (kIDScreenSizeEditDimensionField, SCREENSIZE_WINDOWNAME": Dimension Field", purgeable)
{
	{ 0, 0, TEXT_HT_BIG, WIDTHHEIGHT_FIELD_WD },
	0,
	visible,
	0,
	0,
	kControlEditTextProc,
	0,
	""
};

#endif
