/*
**************************************************************************
	TelnetWindows.r
	
	MacTelnet 3.0
	 1998-2001 by Kevin Grant
	
	This file contains all resource data for special windows.
**************************************************************************
*/

#ifndef __TELNETWINDOWS_R__
#define __TELNETWINDOWS_R__

#include "Appearance.r"
#include "Controls.r"
#include "ControlDefinitions.r"
#include "Dialogs.r"
#include "Finder.r"
#include "MacTypes.r"
#include "MacWindows.r"
#include "Palettes.r"
#include "ControlResources.h"
#include "DialogResources.h"
#include "PreferenceResources.h"
#include "SpacingConstants.r"
#include "TelnetButtonNames.h"

/*
message window - dimensions are basically arbitrary
*/

resource 'WIND' (kWindowIDCommandLine, "Message window", purgeable)
{
	{ 40, 40, 50, 400 }, kWindowFloatHorizZoomGrowProc, invisible, goAway, 0, "", noAutoCenter
};

/*
clipboard window - dimensions are basically arbitrary; "Clipboard.c" takes care of it
*/

resource 'WIND' (kWindowIDClipboard, "Clipboard window", purgeable)
{
	{ 100, 100, 215, 400 }, kWindowFullZoomGrowDocumentProc, invisible, goAway, 0, CLIPBOARD_WINDOWNAME, centerMainScreen
};

/*
TEK graphics windows
*/

resource 'WIND' (256, "TEK window", purgeable)
{
	{ 100, 100, 400, 500 }, kWindowFloatGrowProc, invisible, goAway, 0, "", staggerParentWindow
};

/*
terminal windows - palette of colors
*/

resource 'pltt' (ANSICOLORS_DEFAULTID, "Standard ANSI Colors (indices are important)")
{
	{
		// black
		0, 0, 0,					// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// red
		45219, 0, 0,				// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// green
		0, 25558, 4587,				// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// brown
		37354, 28835, 15073,		// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// purple
		18349, 0, 42597,			// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// magenta
		45219, 0, 45219,			// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// cyan
		0, 45219, 45219,			// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// silver
		49151, 49151, 49151,		// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// black emphasized (gray)
		16384, 16384, 16384,		// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// red emphasized
		57015, 0, 0,				// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// green emphasized
		2, 41942, 1,				// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// brown emphasized (yellow)
		65535, 65532, 17696,		// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// purple emphasized (blue)
		1, 5513, 55049,				// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// magenta emphasized
		62258, 1024, 34078,			// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// cyan emphasized
		34734, 61945, 65535,		// RGB
		pmCourteous,				// usage
		0,							// tolerance
		// silver emphasized (white)
		65535, 65535, 65535,		// RGB
		pmCourteous,				// usage
		0							// tolerance
	}
};

/*
terminal windows - window header controls
*/

resource 'CNTL' (kIDTerminalWindowHeaderPlacard, "Terminal Window Header Placard", purgeable)
{
	{ -1, -1, 0, 0 }, // only top-left is important
	0,
	visible,
	0,
	0,
	kControlPlacardProc/*kControlWindowHeaderProc*/,
	0,
	""
};

resource 'CNTL' (kIDTerminalWindowHeaderDisplayGroupBox, "Terminal Window Header Display Group Box", purgeable)
{
	{ 0, 0, 0, 0/* auto-sized */ },
	0,
	visible,
	0,
	0,
	kControlGroupBoxSecondaryTextTitleProc,
	0,
	""
};

/*
terminal windows - window content controls
*/

resource 'CNTL' (kIDTerminalWindowBackgroundPicture, "Terminal Window Background Picture", purgeable)
{
	{ 0, 0, 0, 0 }, // auto-sized and auto-positioned control
	kControlSupportsEmbedding | kControlHasSpecialBackground | kControlWantsActivate,
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

resource 'CNTL' (kIDTerminalWindowScreenPane, "Terminal Window Screen Pane", purgeable)
{
	{ 0, 0, 0, 0 }, // auto-sized and auto-positioned control
	kControlGetsFocusOnClick | kControlSupportsFocus | kControlWantsActivate | kControlWantsIdle,
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

/*
session Inspector window - content controls
*/

resource 'CNTL' (kIDInspectorPlacard, "Inspector Background Placard", purgeable)
{
	{ 0, 0, 0, 0 }, // auto-sized and auto-positioned control
	0,
	visible,
	0,
	0,
	kControlPlacardProc,
	0,
	""
};

/*
clipboard window - content controls
*/

resource 'CNTL' (kIDClipboardWindowContentUserPane, "Clipboard Contents Renderer", purgeable)
{
	{ 0, 0, 0, 0 }, // auto-sized and auto-positioned control
	kControlSupportsEmbedding | kControlWantsActivate,
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

/*
setup assistant window
*/

resource 'WIND' (kWindowIDAssistant, "Assistant", purgeable)
{
	{ 0, 0, 316, 400 },
	kWindowDocumentProc,
	invisible,
	goAway, // close box
	0,
	"",
	centerMainScreen
};

resource 'CNTL' (kIDSetupAssistantBackBevelButton, "Assistant Window Back Button", purgeable)
{
	{ 0, 0, 16, 16 }, // auto-positioned control
	0, // menu ID, or 0
	visible,
	kIconSuiteAssistantArrowBack, // resource ID of icon contents
	kControlBehaviorPushbutton | kControlContentIconSuiteRes,
	kControlBevelButtonSmallBevelProc,
	0,
	""
};

resource 'CNTL' (kIDSetupAssistantForwardBevelButton, "Assistant Window Forward Button", purgeable)
{
	{ 0, 0, 16, 16 }, // auto-positioned control
	0, // menu ID, or 0
	visible,
	kIconSuiteAssistantArrowForward, // resource ID of icon contents
	kControlBehaviorPushbutton | kControlContentIconSuiteRes,
	kControlBevelButtonSmallBevelProc,
	0,
	""
};

resource 'CNTL' (kIDSetupAssistantHeaderContentUserPane, "Assistant Window Header User Pane", purgeable)
{
	{ 0, 0, 0, 0 }, // auto-sized and auto-positioned control
	kControlHasSpecialBackground | kControlSupportsEmbedding | kControlWantsActivate,
	visible,
	0,
	0,
	kControlUserPaneProc,
	0,
	""
};

resource 'CNTL' (kIDSetupAssistantFooterPlacard, "Assistant Window Footer Placard", purgeable)
{
	{ 0, 0, 0, 0 }, // auto-sized and auto-positioned control
	0,
	visible,
	0,
	0,
	kControlPlacardProc,
	0,
	""
};

/*
keypad controls
*/

resource 'CNTL' (kIDKeypadContentUserPane, "Keypad Content User Pane", purgeable)
{
	{ 0, 0, 0, 0 }, // auto-sized and auto-positioned control
	0, // PICT ID // control feature bits
	visible,
	0,
	0,
	kControlPictureNoTrackProc/*kControlUserPaneProc*/,
	0,
	""
};

#endif