/*###############################################################

	SessionEditorDialog.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

// standard-C includes
#ifdef __cplusplus
#	include <climits>
#else
#	include <limits.h>
#endif
#include <ctype.h>

// Mac includes
#include <ControlDefinitions.h>
#include <MacErrors.h>
#include <PLStringFuncs.h>

// library includes
#include "DataAccessLib.h"
#include "InterfaceLib.h"

// resource includes
#include "GeneralResources.h"
#include "MenuResources.h"
#include "TelnetSessionEditorDialog.r"

// MacTelnet includes
#include "Clipboard.h"
#include "configure.proto.h"
#include "Console.h"
#include "DialogItemIndices.h"
#include "DialogUtilities.h"
#include "DialogResources.h"
#include "EventLoop.h"
#include "FileUtilities.h"
#include "FolderManager.h"
#include "Preferences.h"
#include "ResourceFileManager.h"
#include "ScreenManager.h"
#include "SessionEditorDialog.h"
#include "StringResources.h"
#include "TelnetDescriptors.h"
#include "TelnetMenus.h"
#include "Terminal.h"



// variables
extern MenuHandle			gListItemsDummyMenu;
static DialogPtr			gSessionEditorDialog = null; // the Mac OS dialog pointer
static WindowInfoRef		gSessionEditorDialogWindowInfo = null;
static ControlKeyFilterUPP	gNumericalLimiterUPP = null;

// internal methods
static void				done							(void);
static pascal Boolean	handleNewSize					(const WindowRef				inWindowRef,
														 const SInt32					inDeltaSizeX,
														 const SInt32					inDeltaSizeY,
														 const SInt32					inData);
static void				init							(void);
static pascal Boolean	sessionEditorDialogEventFilter	(DialogPtr						inDialogPtr,
														 EventRecord*					inoutEventPtr,
														 short*							outItemIndex);
static void				setItemStates					(void);



//
// public methods
//

/**
This method displays and handles events in the
Session Favorite Editor dialog box.  If the
user clicks OK, "true" is returned; otherwise,
"false" is returned.

(3.0)
*/
Boolean
SessionEditorDialog_Display		(StringPtr		inoutPreferencesResourceName)
{
	short				ditem = 0,
						scratchshort = 0,
						resourceID = 0;
	long				scratchlong = 0L;
	ResType				scratchResType = '????';
	Boolean				isNewDataHandle = false,
						wasInAliasText = false;
	FSSpec				pendingCaptureFile;
	TelnetPrefsAliasID	pendingCaptureFileAliasID = kInvalidTelnetPrefsAliasID;
	SessionPrefs**		SessPrefsHdl = null;
	SessionPrefs*		SessPrefsPtr = null;
	Str255				scratchPstring,
						scratchPstring2;
	MenuHandle			terminalsMenu = null,
						translationMenu = null;
	ModalFilterUPP		filterProc = NewModalFilterUPP(sessionEditorDialogEventFilter);
	
	
	init();
	
	ditem = 3;
	
	{
		ControlRef		popUpMenuControl = null;
		SInt16			count = 0;
		
		
		// set up Terminals pop-up menu
		(OSErr)GetDialogItemAsControl(gSessionEditorDialog, SessTermPopup, &popUpMenuControl);
		ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
		terminalsMenu = Menus_NewMenuThemeSavvy(kMenuIDSessionEditorTerminals, EMPTY_PSTRING);
		count = Count1Resources(TERMINALPREFS_RESTYPE);
		InsertResMenu(terminalsMenu, TERMINALPREFS_RESTYPE, 999/* insert after item */);
		#if TARGET_API_MAC_OS8
		EnableItem(terminalsMenu, 0);
		#else
		EnableMenuItem(terminalsMenu, 0);
		#endif
		SetControlMinimum(popUpMenuControl, 1);
		SetControlMaximum(popUpMenuControl, count);
		(OSErr)SetControlData(popUpMenuControl, kControlMenuPart, kControlPopupButtonMenuHandleTag,
								sizeof(terminalsMenu), (Ptr)&terminalsMenu);
		
		// set up Translation Tables pop-up menu
		(OSErr)GetDialogItemAsControl(gSessionEditorDialog, SessTransTablePopup, &popUpMenuControl);
		ResourceFileManager_UseResFile(kTelnetResourceFileApplication);
		translationMenu = Menus_NewMenuThemeSavvy(kMenuIDSessionEditorTranslationTables, EMPTY_PSTRING);
		count = Count1Resources(USER_TRSL);
		GetIndString(scratchPstring, rStringsMiscellaneous, siNone); // "None" string
		AppendMenu(translationMenu, scratchPstring);
		InsertResMenu(translationMenu, USER_TRSL, 999/* insert after item */);
		#if TARGET_API_MAC_OS8
		EnableItem(translationMenu, 0);
		#else
		EnableMenuItem(translationMenu, 0);
		#endif
		SetControlMinimum(popUpMenuControl, 1);
		SetControlMaximum(popUpMenuControl, count);
		(OSErr)SetControlData(popUpMenuControl, kControlMenuPart, kControlPopupButtonMenuHandleTag,
								sizeof(translationMenu), (Ptr)&translationMenu);
	}
	
	ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
	
	if (inoutPreferencesResourceName[0] > 0)
	{
		isNewDataHandle = false;
		ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
		SessPrefsHdl = (SessionPrefs**)Get1NamedResource(SESSIONPREFS_RESTYPE, inoutPreferencesResourceName);
		{
			Str255		defaultName;
			
			
			StringUtilities_CToP(SESSIONPREFS_DEFAULTNAME, defaultName);
			if (EqualString(inoutPreferencesResourceName, defaultName, false/* case */, false/* diacritics */))
			{
				HideDialogItem(gSessionEditorDialog, SessAlias);
				SetDialogItemControlText(gSessionEditorDialog, SessAliasStatText, defaultName);
			}
		}
	}
	else
	{
		SessPrefsHdl = Preferences_GetDefaultSession();
		isNewDataHandle = true;
		GetIndString(inoutPreferencesResourceName, rStringsMiscellaneous, siNewSessionDefaultName);
	}
	HLock((Handle)SessPrefsHdl);
	SessPrefsPtr = *SessPrefsHdl;
	SetDialogItemValue(gSessionEditorDialog, SessTEKinhib, (SessPrefsPtr->tektype == kTektronixModeNotAllowed));
	SetDialogItemValue(gSessionEditorDialog, SessTEK4014, (SessPrefsPtr->tektype == kTektronixMode4014));
	SetDialogItemValue(gSessionEditorDialog, SessTEK4105, (SessPrefsPtr->tektype == kTektronixMode4105));
	SetDialogItemValue(gSessionEditorDialog, SessPasteQuick, (SessPrefsPtr->pastemethod == kTelnetPasteMethodStandard));
	SetDialogItemValue(gSessionEditorDialog, SessPasteBlock, (SessPrefsPtr->pastemethod == kTelnetPasteMethodBlock));
	SetDialogItemValue(gSessionEditorDialog, SessDeleteDel, (SessPrefsPtr->bksp == 1));
	SetDialogItemValue(gSessionEditorDialog, SessDeleteBS, (SessPrefsPtr->bksp == 0));
	SetDialogItemValue(gSessionEditorDialog, SessForceSave, SessPrefsPtr->forcesave);
	SetDialogItemValue(gSessionEditorDialog, SessBezerkeley, SessPrefsPtr->crmap);
	SetDialogItemValue(gSessionEditorDialog, SessLinemode, SessPrefsPtr->linemode);
	SetDialogItemValue(gSessionEditorDialog, SessTEKclear, SessPrefsPtr->tekclear);
	SetDialogItemValue(gSessionEditorDialog, SessHalfDuplex, SessPrefsPtr->halfdup);
	SetDialogItemValue(gSessionEditorDialog, SessAuthenticate, SessPrefsPtr->authenticate);
	SetDialogItemValue(gSessionEditorDialog, SessEncrypt, SessPrefsPtr->encrypt);
	SetDialogItemValue(gSessionEditorDialog, SessLocalEcho, SessPrefsPtr->localecho);
	SetDialogItemControlText(gSessionEditorDialog, SessHostName, SessPrefsPtr->hostname);
	SetDialogItemControlText(gSessionEditorDialog, SessAlias, inoutPreferencesResourceName);
	NumToString(SessPrefsPtr->port, scratchPstring);
	SetDialogItemControlText(gSessionEditorDialog, SessPort, scratchPstring);
	NumToString(SessPrefsPtr->pasteblocksize, scratchPstring);
	SetDialogItemControlText(gSessionEditorDialog, SessBlockSize, scratchPstring);
	
	HideDialogItem(gSessionEditorDialog, SessAuthenticate);
	HideDialogItem(gSessionEditorDialog, SessEncrypt);
	
	SetDialogItemValue(gSessionEditorDialog, SessAutoCaptureToFile, SessPrefsPtr->autoCaptureToFile);
	if (GetDialogItemValue(gSessionEditorDialog, SessAutoCaptureToFile) == kControlCheckBoxCheckedValue)
	{
		ActivateDialogItem(gSessionEditorDialog, SessAutoCaptureSetFile);
		ActivateDialogItem(gSessionEditorDialog, SessAutoCaptureFilePath);
	}
	else
	{
		DeactivateDialogItem(gSessionEditorDialog, SessAutoCaptureSetFile);
		DeactivateDialogItem(gSessionEditorDialog, SessAutoCaptureFilePath);
	}
	
	if (SessPrefsPtr->NetBlockSize <= 512) SessPrefsPtr->NetBlockSize = 512;
	NumToString(SessPrefsPtr->NetBlockSize, scratchPstring);
	SetDialogItemControlText(gSessionEditorDialog, SessBlockSizeTE, scratchPstring);
	
	scratchPstring[0] = 2;
	scratchPstring[1] = '^';
	if (SessPrefsPtr->ckey != -1) {
		scratchPstring[2] = SessPrefsPtr->ckey ^ 64;
		SetDialogItemControlText(gSessionEditorDialog, SessInterrupt, scratchPstring);
		}
	if (SessPrefsPtr->skey != -1) {
		scratchPstring[2] = SessPrefsPtr->skey ^ 64;
		SetDialogItemControlText(gSessionEditorDialog, SessSuspend, scratchPstring);
		}
	if (SessPrefsPtr->qkey != -1) {
		scratchPstring[2] = SessPrefsPtr->qkey ^ 64;
		SetDialogItemControlText(gSessionEditorDialog, SessResume, scratchPstring);
		}
	for(scratchshort = CountMenuItems(translationMenu); scratchshort; scratchshort--) {
		GetMenuItemText(translationMenu, scratchshort, scratchPstring);
		if (EqualString(scratchPstring, (SessPrefsPtr->TerminalEmulation), true, false))
			SetDialogItemValue(gSessionEditorDialog, SessTermPopup, scratchshort);
		}
	for(scratchshort = CountMenuItems(translationMenu); scratchshort; scratchshort--) {
		GetMenuItemText(translationMenu, scratchshort, scratchPstring);
		if (EqualString(scratchPstring, (SessPrefsPtr->TranslationTable), true, false))
			SetDialogItemValue(gSessionEditorDialog, SessTransTablePopup, scratchshort);
		}
	
	// the capture file alias record is stored as a separate resource - find it or create it
	if (SessPrefsPtr->autoCaptureToFile)
	{
		Str255		string,
					defaultName;
		OSErr		error = noErr;
		
		
		// get the default capture file name
		GetIndString(defaultName, rStringsImportantFileNames, siImportantFileNameCaptureTextDefault);
		
		// initialize this structure, in case FSMakeFSSpec() fails
		PLstrcpy(pendingCaptureFile.name, defaultName);
		pendingCaptureFile.vRefNum = -1;
		pendingCaptureFile.parID = fsRtDirID;
		
		// 3.0 - read an alias record for the default directory
		if (Preferences_IsAliasStored(SessPrefsPtr->captureFileAliasID))
		{
			// a capture file preference exists - read it and resolve it (re-saving changes if necessary)
			pendingCaptureFileAliasID = Preferences_NewSavedAlias(SessPrefsPtr->captureFileAliasID);
			(Boolean)Preferences_ParseAlias(pendingCaptureFileAliasID, &pendingCaptureFile);
		}
		else
		{
			// no capture file preference exists - create a new one with defaults (on the
			// desktop, if possible; otherwise, create it at the root of the startup disk)
			error = FolderManager_GetFSSpec(kTelnetFolderMacDesktop, &pendingCaptureFile);
			if (error == noErr)
			{
				error = FSMakeFSSpec(pendingCaptureFile.vRefNum, pendingCaptureFile.parID, defaultName,
										&pendingCaptureFile);
			}
			if ((error != noErr) && (error != fnfErr))
			{
				error = FSMakeFSSpec(-1, fsRtDirID, defaultName, &pendingCaptureFile);
			}
			
			if (error == fnfErr)
			{
				OSType		captureFileCreator;
				Size		actualSize = 0L;
				
				
				// get the users Capture File Creator preference, if possible
				unless (Preferences_GetData(kTelnetPreferenceClassGeneral,
											kTelnetPreferenceTag_CaptureFileCreator,
											sizeof(captureFileCreator), &captureFileCreator,
											&actualSize) == kTelnetPrefsResultCodeSuccess)
				{
					captureFileCreator = 'ttxt'; // default to SimpleText if a preference cant be found
				}
				error = FSpCreate(&pendingCaptureFile, captureFileCreator, 'TEXT',
									GetScriptManagerVariable(smSysScript));
			}
			if (error == noErr)
			{
				// create an 'alis' resource in the preferences file, and remember its ID
				pendingCaptureFileAliasID = Preferences_NewAlias(&pendingCaptureFile);
			}
			else
			{
				pendingCaptureFileAliasID = kInvalidTelnetPrefsAliasID;
				Console_WriteValue("capture file preference creation error", error);
			}
		}
		if (pendingCaptureFileAliasID == kInvalidTelnetPrefsAliasID)
		{
			BEEP;
			Console_WriteLine("capture file alias could not be created");
		}
		FileUtilities_GetPathnameFromFSSpec(&pendingCaptureFile, string, false/* is directory */);
		
		// update the text area to show the capture file pathname
		SetDialogItemCenterTruncatedText(gSessionEditorDialog, SessAutoCaptureFilePath, string,
										kThemeSmallSystemFont);
	}
	
	setItemStates(); // 3.0
	
	SelectDialogItemText(gSessionEditorDialog, SessAlias, 0, 32767);
	
	wasInAliasText = false;
	
	TransitionDialogDisplay(gSessionEditorDialog);
	
	do {
		ModalDialog(filterProc, &ditem);
		if ((wasInAliasText)&&(ditem != SessAlias))
		{
			wasInAliasText = false;
			GetTEText(gSessionEditorDialog, SessAlias,scratchPstring);
			if (EqualString(scratchPstring,inoutPreferencesResourceName,true,false))
				goto okaySessName;
			if (!Menus_IsItemUnique(gListItemsDummyMenu, scratchPstring)) // we already have that name
			{
				InterfaceLibAlertRef	box = null;
				Str255					dialogText,
										helpText,
										configurationText;
				
				
				GetIndString(configurationText, rStringsConfigurationData, siConfigurationDataASession);
				GetIndString(dialogText, rStringsGeneralMessages, siDuplicateConfiguration);
				StringUtilities_PMetaSetDefaults();
				StringUtilities_PBuild(dialogText, configurationText, EMPTY_PSTRING, EMPTY_PSTRING, EMPTY_PSTRING);
				GetIndString(configurationText, rStringsConfigurationData, siConfigurationDataSessions);
				GetIndString(helpText, rStringsGeneralMessages, siDuplicateConfigurationHelpText);
				StringUtilities_PMetaSetDefaults();
				StringUtilities_PBuild(helpText, configurationText, EMPTY_PSTRING, EMPTY_PSTRING, EMPTY_PSTRING);
				
				box = Alert_New();
				Alert_SetHelpButton(box, false);
				Alert_SetParamsFor(box, kAlert_StyleOK);
				Alert_SetText(box, dialogText, helpText);
				Alert_SetType(box, kAlertStopAlert);
				Alert_Display(box);
				Alert_Dispose(&box);
				
				SetPortWindowPort(GetDialogWindow(gSessionEditorDialog)); // back to old dialog
				SetDialogItemControlText(gSessionEditorDialog,SessAlias,inoutPreferencesResourceName);//set the string back to the original
				SelectDialogItemText(gSessionEditorDialog,SessAlias,0,32767);
				ditem = SessAlias; //so that if they hit ok or cancel, it doesn't kill the window
			}
			else
				goto okaySessName;
		}
		else
		{
			okaySessName:
			switch (ditem) {
				case SessAutoCaptureToFile:
				case SessAutoCaptureSetFile:
					{
						Boolean		choose = false;
						
						
						if (ditem == SessAutoCaptureToFile)
						{
							// then the checkbox was selected - change its state
							FlipCheckBox(gSessionEditorDialog, ditem);
							setItemStates();
							
							// if the user turns on the auto-capture-file option,
							// automatically click the Set button
							choose = (GetDialogItemValue(gSessionEditorDialog, ditem) == kControlCheckBoxCheckedValue);
							if (choose) FlashButton(gSessionEditorDialog, SessAutoCaptureSetFile);
						}
						else choose = true;
						
						if (choose) if (Terminal_FileCaptureSaveDialog(&pendingCaptureFile))
						{
							// show path to file in the dialog box
							Str255		string;
							
							
							FileUtilities_GetPathnameFromFSSpec(&pendingCaptureFile, string, false/* is directory */);
							SetDialogItemCenterTruncatedText(gSessionEditorDialog, SessAutoCaptureFilePath, string,
																kThemeSmallSystemFont);
						}
					}
					break;
				
				case	SessForceSave:
				case	SessBezerkeley:
				case	SessLinemode:
				case	SessTEKclear:
				case	SessHalfDuplex:
				case	SessAuthenticate:
				case	SessEncrypt:
				case	SessLocalEcho:
					FlipCheckBox(gSessionEditorDialog, ditem);
					setItemStates();
					break;
				
				
				case	SessTEKinhib:
					SetDialogItemValue(gSessionEditorDialog, SessTEKinhib, 1);
					SetDialogItemValue(gSessionEditorDialog, SessTEK4014, 0);
					SetDialogItemValue(gSessionEditorDialog, SessTEK4105, 0);
					break;
				case	SessTEK4014:
					SetDialogItemValue(gSessionEditorDialog, SessTEKinhib, 0);
					SetDialogItemValue(gSessionEditorDialog, SessTEK4014, 1);
					SetDialogItemValue(gSessionEditorDialog, SessTEK4105, 0);
					break;
				case	SessTEK4105:
					SetDialogItemValue(gSessionEditorDialog, SessTEKinhib, 0);
					SetDialogItemValue(gSessionEditorDialog, SessTEK4014, 0);
					SetDialogItemValue(gSessionEditorDialog, SessTEK4105, 1);
					break;
					
				case	SessPasteQuick:
					SetDialogItemValue(gSessionEditorDialog, SessPasteQuick, 1);
					SetDialogItemValue(gSessionEditorDialog, SessPasteBlock, 0);
					break;
		
				case	SessPasteBlock:
					SetDialogItemValue(gSessionEditorDialog, SessPasteQuick, 0);
					SetDialogItemValue(gSessionEditorDialog, SessPasteBlock, 1);
					break;
		
				case	SessDeleteDel:
					SetDialogItemValue(gSessionEditorDialog, SessDeleteDel, 1);
					SetDialogItemValue(gSessionEditorDialog, SessDeleteBS, 0);
					break;
		
				case	SessDeleteBS:
					SetDialogItemValue(gSessionEditorDialog, SessDeleteDel, 0);
					SetDialogItemValue(gSessionEditorDialog, SessDeleteBS, 1);
					break;
		
				case	SessInterrupt:
				case	SessSuspend:
				case	SessResume:
					GetTEText(gSessionEditorDialog, ditem, scratchPstring);
					if ((scratchPstring[1] < 32) && (scratchPstring[1] > 0)) {
						scratchPstring[0] = 2;
						scratchPstring[2] = scratchPstring[1] ^ 64;
						scratchPstring[1] = '^';
						SetDialogItemControlText(gSessionEditorDialog, ditem, scratchPstring);
						}
					break;
				case	SessAlias:
					wasInAliasText = true;
					break;
				default:
					break;
				} // switch
			} //else not bad alias
	} while ((ditem != SessOK) && (ditem != SessCancel));
	
	GetMenuItemText(terminalsMenu, GetDialogItemValue(gSessionEditorDialog, SessTermPopup), scratchPstring);
	GetMenuItemText(translationMenu, GetDialogItemValue(gSessionEditorDialog, SessTransTablePopup), scratchPstring2);
	
	if (ditem == SessCancel)
	{
		if (isNewDataHandle) Memory_DisposeHandle(&((Handle)SessPrefsHdl));
		else ReleaseResource((Handle)SessPrefsHdl);
		TransitionDialogCloseCancel(gSessionEditorDialog);
		done();
		return(false);			// No changes should be made.
	}
	
	if (StrLength(scratchPstring) > 63) scratchPstring[0] = 63;
	BlockMove(scratchPstring, (SessPrefsPtr->TerminalEmulation), scratchPstring[0]+1);
	
	if (StrLength(scratchPstring2) > 63) scratchPstring[0] = 63;
	BlockMove(scratchPstring2, (SessPrefsPtr->TranslationTable), scratchPstring2[0]+1);
	
	if (GetDialogItemValue(gSessionEditorDialog, SessTEK4105) == kControlCheckBoxCheckedValue) SessPrefsPtr->tektype = kTektronixMode4105;
	else if (GetDialogItemValue(gSessionEditorDialog, SessTEK4014) == kControlCheckBoxCheckedValue) SessPrefsPtr->tektype = kTektronixMode4014;
	else SessPrefsPtr->tektype = kTektronixModeNotAllowed;
	
	{
		short		value = GetDialogItemValue(gSessionEditorDialog, SessPasteQuick);
		
		
		if (value) SessPrefsPtr->pastemethod = kTelnetPasteMethodStandard;
		else SessPrefsPtr->pastemethod = kTelnetPasteMethodBlock;
	}
	
	// 3.0 - create an alias record for the capture file
	{
		OSErr		error = noErr;
		
		
		if (error == noErr)
		{
			Preferences_ChangeAlias(pendingCaptureFileAliasID, &pendingCaptureFile);
			Preferences_SaveAlias(pendingCaptureFileAliasID, inoutPreferencesResourceName);
			SessPrefsPtr->captureFileAliasID = pendingCaptureFileAliasID;
		}
		Alert_ReportOSErr(error);
	}
	
	SessPrefsPtr->autoCaptureToFile = GetDialogItemValue(gSessionEditorDialog, SessAutoCaptureToFile);
	SessPrefsPtr->bksp = GetDialogItemValue(gSessionEditorDialog, SessDeleteDel);
	SessPrefsPtr->forcesave = GetDialogItemValue(gSessionEditorDialog, SessForceSave);
	SessPrefsPtr->crmap = GetDialogItemValue(gSessionEditorDialog, SessBezerkeley);
	SessPrefsPtr->linemode = GetDialogItemValue(gSessionEditorDialog, SessLinemode);
	SessPrefsPtr->tekclear = GetDialogItemValue(gSessionEditorDialog, SessTEKclear);
	SessPrefsPtr->halfdup = GetDialogItemValue(gSessionEditorDialog, SessHalfDuplex);
	SessPrefsPtr->authenticate = GetDialogItemValue(gSessionEditorDialog, SessAuthenticate);
	SessPrefsPtr->encrypt = GetDialogItemValue(gSessionEditorDialog, SessEncrypt);
	SessPrefsPtr->localecho = GetDialogItemValue(gSessionEditorDialog, SessLocalEcho);
	
	GetTEText(gSessionEditorDialog, SessPort, scratchPstring);
	StringToNum(scratchPstring, &scratchlong);
	if (scratchlong > 65530) scratchlong = 65530;
	if (scratchlong < 1) scratchlong = 1;
	SessPrefsPtr->port = (short) scratchlong;

	GetTEText(gSessionEditorDialog, SessBlockSize, scratchPstring);
	StringToNum(scratchPstring, &scratchlong);
	if (scratchlong > 4097) scratchlong = 4097;
	if (scratchlong < 10) scratchlong = 10;
	SessPrefsPtr->pasteblocksize = (short) scratchlong;

	GetTEText(gSessionEditorDialog, SessHostName, scratchPstring);
	if (StrLength(scratchPstring) > 63) scratchPstring[0] = 63;
	BlockMove(scratchPstring, SessPrefsPtr->hostname, scratchPstring[0]+1);
	
	GetTEText(gSessionEditorDialog, SessHostName, scratchPstring);
	if (StrLength(scratchPstring) > 63) scratchPstring[0] = 63;
	BlockMove(scratchPstring, SessPrefsPtr->hostname, scratchPstring[0]+1);

	GetTEText(gSessionEditorDialog, SessAlias, inoutPreferencesResourceName);

	GetTEText(gSessionEditorDialog, SessInterrupt, scratchPstring);
	if (scratchPstring[0]) SessPrefsPtr->ckey = toupper(scratchPstring[2]) ^ 64;
	else SessPrefsPtr->ckey = -1;
	
	GetTEText(gSessionEditorDialog, SessSuspend, scratchPstring);
	if (scratchPstring[0]) SessPrefsPtr->skey = toupper(scratchPstring[2]) ^ 64;
	else SessPrefsPtr->skey = -1;
	
	GetTEText(gSessionEditorDialog, SessResume, scratchPstring);
	if (scratchPstring[0]) SessPrefsPtr->qkey = toupper(scratchPstring[2]) ^ 64;
	else SessPrefsPtr->qkey = -1;
	
	GetTEText(gSessionEditorDialog,SessBlockSizeTE,scratchPstring);
	StringToNum(scratchPstring,&scratchlong);
	if (scratchlong > 4096) scratchlong = 4096;
	if (scratchlong < 512) scratchlong = 512;
	SessPrefsPtr->NetBlockSize = (short) scratchlong;

	if (isNewDataHandle)
	{
		ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
		resourceID = Unique1ID(SESSIONPREFS_RESTYPE);
		AddResource((Handle)SessPrefsHdl, SESSIONPREFS_RESTYPE, resourceID, inoutPreferencesResourceName);
		ResourceFileManager_UpdateResFile(kTelnetResourceFilePreferences);
		ReleaseResource((Handle)SessPrefsHdl);
	}
	else
	{
		ResourceFileManager_UseResFile(kTelnetResourceFilePreferences);
		GetResInfo((Handle)SessPrefsHdl, &resourceID, &scratchResType,(StringPtr) &scratchPstring);
		SetResInfo((Handle)SessPrefsHdl, resourceID, inoutPreferencesResourceName);
		ChangedResource((Handle)SessPrefsHdl);
		if (ResError() == noErr) WriteResource((Handle)SessPrefsHdl);
		ReleaseResource((Handle)SessPrefsHdl);
	}
	
	TransitionDialogClose(gSessionEditorDialog);
	
	done();
	
	Preferences_DisposeAlias(pendingCaptureFileAliasID);
	
	return(true);			// A resource has changed or been added.	
}// Display


//
// internal methods
//

/**
Call this method to destroy the dialog box
and its associated data structures.

(3.0)
*/
static void
done	(void)
{
	WindowInfo_Dispose(gSessionEditorDialogWindowInfo);
	DisposeControlKeyFilterUPP(gNumericalLimiterUPP), gNumericalLimiterUPP = null;
	DisposeDialog(gSessionEditorDialog), gSessionEditorDialog = null;
}// done


/**
This method moves and resizes controls in response to a
resize of the session editor dialog box.

(3.0)
*/
static pascal Boolean
handleNewSize		(const WindowRef		inWindowRef,
					 const SInt32			inDeltaSizeX,
					 const SInt32			inDeltaSizeY,
					 const SInt32			UNUSED_ARGUMENT(inData))
{
	Boolean		result = pascal_true;
	
	
#if 0
	DialogAdjust_BeginDialogItemAdjustment(GetDialogFromWindow(inWindowRef));
	{
		SInt32		amountH = 0L,
					amountV = 0L;
		
		
		if (inDeltaSizeX)
		{
			// controls which are moved horizontally
			if (Localization_IsLeftToRight())
			{
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iScreenSizeOKButton, inDeltaSizeX);
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iScreenSizeCancelButton, inDeltaSizeX);
			}
			else
			{
				DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iScreenSizeAppleGuideButton, inDeltaSizeX);
			}
			amountH = (inDeltaSizeX / 3);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iScreenSizeWidthField, amountH);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iScreenSizeHeightField, amountH);
			amountH = (INTEGER_DOUBLED(inDeltaSizeX) / 3);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iScreenSizeWidthArrows, amountH);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iScreenSizeHeightArrows, amountH);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iScreenSizeWidthUnits, amountH);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentMoveH, iScreenSizeHeightUnits, amountH);
			
			// controls which are resized horizontally
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iScreenSizeUpdateDefaultTerminalCheckBox, inDeltaSizeX);
			amountH = (inDeltaSizeX / 3);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iScreenSizeWidthLabel, amountH);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iScreenSizeWidthField, amountH);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iScreenSizeWidthUnits, amountH);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iScreenSizeHeightLabel, amountH);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iScreenSizeHeightField, amountH);
			DialogAdjust_AddDialogItem(kDialogItemAdjustmentResizeH, iScreenSizeHeightUnits, amountH);
		}
	}
	DialogAdjust_EndAdjustment(inDeltaSizeX, inDeltaSizeY); // moves and resizes controls properly
#endif
	
	return result;
}// handleNewSize


/**
This method is used to initialize the Session Favorite
Editor dialog box.  It creates the dialog box invisibly,
and sets up the controls in the dialog box.

(3.0)
*/
static void
init	(void)
{
	GetNewDialogWithWindowInfo(kDialogIDSessionFavoriteEditor, &gSessionEditorDialog,
								&gSessionEditorDialogWindowInfo);
	
	if (gSessionEditorDialog != null)
	{
		// determine if the Help button should be disabled...
		//SetUpHelpButtonDialogItem(gSessionEditorDialog, iScreenSizeAppleGuideButton);
		
		// adjust the OK and Cancel buttons so they are big enough for their titles
		SetDialogDefaultItem(gSessionEditorDialog, SessOK);
		SetDialogCancelItem(gSessionEditorDialog, SessCancel);
		(UInt16)Localization_AdjustDialogButtons(gSessionEditorDialog, true/* OK */, true/* Cancel */);
		//Localization_AdjustHelpButtonItem(gSessionEditorDialog, iScreenSizeAppleGuideButton);
		
		// prevent the user from typing non-digits into certain text fields
		{
			ControlRef				field = null;
			
			
			gNumericalLimiterUPP = NewControlKeyFilterUPP(NumericalLimiter);
			
			GetDialogItemAsControl(gSessionEditorDialog, SessPort, &field);
			(OSErr)SetControlData(field, kControlNoPart, kControlKeyFilterTag, sizeof(gNumericalLimiterUPP),
									(Ptr)&gNumericalLimiterUPP);
			GetDialogItemAsControl(gSessionEditorDialog, SessBlockSize, &field);
			(OSErr)SetControlData(field, kControlNoPart, kControlKeyFilterTag, sizeof(gNumericalLimiterUPP),
									(Ptr)&gNumericalLimiterUPP);
			GetDialogItemAsControl(gSessionEditorDialog, SessBlockSizeTE, &field);
			(OSErr)SetControlData(field, kControlNoPart, kControlKeyFilterTag, sizeof(gNumericalLimiterUPP),
									(Ptr)&gNumericalLimiterUPP);
		}
		
		// set up the Window Info information
		{
			//WindowResizeResponderProcPtr	resizeProc = handleNewSize;
			
			
			WindowInfo_SetWindowDescriptor(gSessionEditorDialogWindowInfo, kTelnetDialogDescriptorSessionEditor);
			#if 0
			WindowInfo_SetWindowResizeLimits(gSessionEditorDialogWindowInfo,
													?/* minimum height */,
													?/* minimum width */,
													? + 128/* maximum height */,
													? + 50/* maximum width */);
			WindowInfo_SetWindowResizeResponderUPP(gSessionEditorDialogWindowInfo, resizeProc, 0L);
			#endif
		}
		
		WindowInfo_SetForDialog(gSessionEditorDialog, gSessionEditorDialogWindowInfo);
	}
}// init


/**
Handles special key events in the
Session Favorite Editor dialog box.

(3.0)
*/
static pascal Boolean
sessionEditorDialogEventFilter	(DialogPtr		inDialogPtr,
								 EventRecord*	inoutEventPtr,
								 short*			outItemIndex)
{
	Boolean		result = pascal_false,
				isButtonAndShouldFlash = false; // simplifies handling of multiple key-to-button maps
	
	
	// first, make sure help balloons show up properly for the cursor location, if Balloon Help is on
	if (IsPopUpHelpEnabled()) DisplayBalloonForDialogItemUnderMouse(inDialogPtr, inoutEventPtr->where,
																		kDialogBalloonsIDSessionFavoriteEditor);
	
	if (!result) result = (StandardDialogEventFilter(inDialogPtr, inoutEventPtr, outItemIndex));
	else if (isButtonAndShouldFlash) FlashButton(inDialogPtr, *outItemIndex);
	
	return result;
}// sessionEditorDialogEventFilter


/**
To change the active states of controls
based on the values of certain other
controls, use this method.

(3.0)
*/
static void
setItemStates	(void)
{
	// disable Half Duplex unless Local Echo is enabled
	if (GetDialogItemValue(gSessionEditorDialog, SessLocalEcho) == kControlCheckBoxCheckedValue)
	{
		ActivateDialogItem(gSessionEditorDialog, SessHalfDuplex);
	}
	else
	{
		DeactivateDialogItem(gSessionEditorDialog, SessHalfDuplex);
	}
	
	// set correct states for Capture File controls
	if (GetDialogItemValue(gSessionEditorDialog, SessAutoCaptureToFile) == kControlCheckBoxCheckedValue)
	{
		ActivateDialogItem(gSessionEditorDialog, SessAutoCaptureSetFile);
		ActivateDialogItem(gSessionEditorDialog, SessAutoCaptureFilePath);
	}
	else
	{
		DeactivateDialogItem(gSessionEditorDialog, SessAutoCaptureSetFile);
		DeactivateDialogItem(gSessionEditorDialog, SessAutoCaptureFilePath);
	}
}// setItemStates