/*###############################################################

	StatusIcon.h
	
	This module allows status icons to be defined, displayed
	and updated in a simple and abstract manner that uses an
	Appearance user pane control.  The kind of refined control
	over icon controls necessary to implement this module is
	only available in Mac OS 8.5 and beyond, so this module
	does its icon drawing manually, using a generic user pane
	and the Mac OS Icon Utilities, rather than using a standard
	icon control.  Status icons use the small icon information
	from one or more icon suite resources to draw icons either
	statically or in an animating sequence.
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __STATUSICON__
#define __STATUSICON__

// Mac includes
#ifndef __MACTYPES__
#	include <MacTypes.h>
#endif
#ifndef __CONTROLS__
#	include <Controls.h>
#endif

// library includes
#ifndef __INTERFACELIB__
#	include "InterfaceLib.h"
#endif



// types
typedef struct { unsigned garbage; }*		StatusIconRef;



//
// public methods
//

StatusIconRef
	StatusIcon_New					(const WindowRef		inOwningWindow);

void
	StatusIcon_Dispose				(StatusIconRef*			inoutRefPtr);

void
	StatusIcon_AddStage				(StatusIconRef			inRef,
									 const SInt16			inResourceFile,
									 const SInt16			inIconSuiteResourceID,
									 const UInt32			inAnimationDelayInTicks);

ControlRef
	StatusIcon_GetContainerControl	(StatusIconRef			inRef);

Boolean
	StatusIcon_IsAnimating			(StatusIconRef			inRef);

void
	StatusIcon_SetDoAnimate			(StatusIconRef			inRef,
									 const Boolean			inIconShouldAnimate,
									 const Boolean			inResetStages);

void
	StatusIcon_SetNextCell			(StatusIconRef			inRef,
									 const UInt16			inZeroBasedNextCellIndex);

void
	StatusIcon_SetTransformType		(StatusIconRef			inRef,
									 IconTransformType		inTransform);

#endif