/*###############################################################

	StringResources.h
	
	Constants describing string resources, both their resource
	IDs and their indices (for use with the GetIndString()
	system call).  There are additional string list resource IDs
	and indices defined for help balloons, in other files.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.

###############################################################*/

#ifndef __STRINGRESOURCES__
#define __STRINGRESOURCES__

#ifndef REZ

// strings located inside MacTelnets application program file
enum
{
	rStringsSaveSetKeywords = 23239,
	rStringsAffirmativeWords = 23240,
	rStringsFTPCommands = 23228, // see FTPInterpreter.h for string indices
	rStringsFTPMessages = 23227,
	rStringCountSaveSetKeywords = 46,
	rStringCountAffirmativeWords = 13,
	rStringCountFTPCommands = 51 // see FTPInterpreter.h to confirm this!
};

enum
{
	rStringsGeneralMessages = 2000,
	siTooManyOpenConnections = 1,
	siTooManyOpenConnectionsHelpText = 2,
	siDomainNamesWontWorkHelpText = 3,
	siDomainNameResolverOpenFailedBecause = 4,
	siWillNotBeAbleToOpenThatManyConnections = 5,
	siPrintingFailed = 6,
	siPrintingFailedHelpText = 7,
	siConnectionOpeningFailed = 8,
	siOSErrAndInternalErrHelpText = 9,
	siCantRemoveDefaultListItem = 10,
	siDuplicateConfiguration = 11,
	siDuplicateConfigurationHelpText = 12,
	siCommandCannotContinue = 13,
	siIPAddressNotAttainable = 14
};

enum
{
	rStringsOperationFailed = 2001,
	siOpFailedCantCreateFile = 1,
	siOpFailedCantOpenFile = 2,
	siOpFailedOutOfMemory = 3,
	siOpFailedBadSet = 4,
	siOpFailedNoPrinter = 5
};

enum
{
	rStringsMiscellaneous = 2002,
	siInspectorHostName = 1,
	siNewSessionDefaultName = 2,
	siNewTerminalDefaultName = 3,
	siUserPrompt = 4,
	siPasswordPrompt = 5,
	siMacOS8_5AndBeyondFindFileName = 6,
	siPreMacOS8_5FindFileName = 7,
	siPickNewColor = 8,
	siNone = 9,
	siFTPLogName = 10,
	siNewSessionsGroupBoxNameNewSessions = 11,
	siNewSessionsGroupBoxNameSelectedSession = 12,
	siNewSessionsShowDetails = 13,
	siAwaitingDismissal = 14,
	siBeingOpened = 15,
	siBeingLookedUp = 16,
	siSessionName = 17,
	siNewSessionsHideDetails = 18,
	siWindoidNameKeypad = 19,
	siWindoidNameFunction = 20,
	siWindoidNameSessionInspector = 21,
	siWindoidNameControlKeys = 22,
	siAboutBoxAuthorByText = 23,
	siInterruptProcessTerminalFeedback = 24
};

enum
{
	rStringsStatus = 2003,
	siStatusTheConnection = 1,
	siStatusHelpText = 2,
	siStatusPrintingInProgress = 3,
	siStatusBuildingFontList = 4,
	siStatusFTPServerStarting = 5,
	siStatusFTPServerShuttingDown = 6,
	siStatusBuildingScriptsMenu = 7,
	siStatusGeneratingKeys = 8,
	siStatusNotConnected = 9,
	siStatusConnectedToHostSinceTime = 10,
	siStatusTimeToday = 11,
	siStatusTimeYesterday = 12
};

enum
{
	rStringsDNRMessages = 2004,
	siDNRGeneralError = 1,
	siDNRLookupCancelled = 2,
	siDNROutOfMemory = 3,
	siDNRNetworkUnreachable = 4,
	siDNRNetworkTimeout = 5,
	siDNRNetworkConfigurationError = 6,
	siDNRNameDoesNotExist = 7
};

enum
{
	rStringsAlertTitles = 2005,
	siAlertTitleExportNotPossible = 1,
	siAlertTitleConnectionOpeningFailed = 2,
	siAlertTitleStartupError = 3,
	siAlertTitleFatalErrorOSProtectionMechanism = 4,
	siAlertTitleClose = 5,
	siAlertTitleScriptError = 6,
	siAlertTitleHostKeyUnknown = 7,
	siAlertTitleHostKeyChanged = 8
};

enum
{
	rStringsNavigationServices = 2006,
	siNavDialogTitleExampleBinaryApplication = 1,
	siNavDialogTitleExampleTextApplication = 2,
	siNavDialogTitleDefaultDirectory = 3,
	siNavDialogTitleExportMacros = 4,
	siNavDialogTitleImportMacros = 5,
	siNavDialogTitleSaveSet = 6,
	siNavDialogTitleLoadSet = 7,
	siNavDialogTitleSaveCapturedText = 8,
	siNavDialogTitleLocateApplication = 9,
	siNavPromptSelectBinaryFile = 10,
	siNavPromptSelectTextFile = 11,
	siNavPromptSelectDefaultDirectory = 12,
	siNavPromptExportMacrosToFile = 13,
	siNavPromptImportMacrosFromFile = 14,
	siNavPromptSaveConfigurationSet = 15,
	siNavPromptLoadConfigurationSet = 16,
	siNavPromptSaveCapturedText = 17,
	siNavPromptLocateApplication = 18
};

enum
{
	rStringsAppleGuideLookup = 2007,
	siAGIndexTermConnections = 1,
	siAGIndexTermMacros = 2,
	siAGIndexTermPreferences = 3,
	siAGIndexTermFormatting = 4,
	siAGIndexTermCommandLine = 5,
	siAGIndexTermScreenSize = 6,
	siAGIndexTermFavorites = 7
};

enum
{
	rStringsHelpText = 2008,
	siHelpTextPrefsFTPServerAnonymousModeWarning = 1,
	siHelpTextPrefsGeneralNotification = 2,
	siHelpTextCommandLine = 3,
	siHelpTextMapKeyboard = 4,
	siHelpTextFormatBottomTextFrontmostWindowOnly = 5,
	siHelpTextEditFTPLogInMessage = 6,
	siHelpTextPrefsFTPServerHowToDefineUsers = 7
};

enum
{
	rStringsClipboardData = 2009,
	siClipboardContents = 1,
	siClipboardTypeText = 2,
	siClipboardTypePicture = 3,
	siClipboardTypeUnrecognized = 4,
	siClipboardEmpty = 5,
	siClipboardScalingFactor = 6,
	siClipboardAbout = 7,
	siClipboardBytes = 8,
	siClipboardByte = 9,
	siClipboardK = 10,
	siClipboardMB = 11
};

enum
{
	rStringsShowHide = 2010,
	siShowClipboard = 1,
	siHideClipboard = 2,
	siShowFTPLog = 3,
	siHideFTPLog = 4,
	siShowCommandLine = 5,
	siHideCommandLine = 6,
	siShowMacros = 7,
	siHideMacros = 8,
	siShowKeypad = 9,
	siHideKeypad = 10,
	siShowFunction = 11,
	siHideFunction = 12,
	siShowControlKeys = 13,
	siHideControlKeys = 14,
	siRestoreWindow = 15,
	siMinimizeWindow = 16,
	siShowSessionInspector = 17,
	siHideSessionInspector = 18
};

enum
{
	rStringsRandomStartupMessages = 2011,
	siRandomStartupMessagesFontName = 1,
	siRandomStartupMessagesCountOfRemainingStrings = 2
	// the remaining strings in this resource are the startup messages themselves
};

enum
{
	rStringsMenuItemTextSubstitutions = 2012,
	siMenuItemTextClose = 1,
	siMenuItemTextCloseAll = 2,
	siMenuItemTextDisconnect = 3,
	siMenuItemTextDisconnectAll = 4,
	siMenuItemTextNextWindow = 5,
	siMenuItemTextNextWindowHidingPreviousOne = 6,
	siMenuItemTextPreviousWindow = 7,
	siMenuItemTextCaptureToFileBegin = 8,
	siMenuItemTextCaptureToFileEnd = 9,
	siMenuItemTextUndoDefault = 10,
	siMenuItemTextRedoDefault = 11,
	siMenuItemTextUndoDimensionChanges = 12,
	siMenuItemTextRedoDimensionChanges = 13,
	siMenuItemTextUndoFormatChanges = 14,
	siMenuItemTextRedoFormatChanges = 15,
	siMenuItemTextAboutNormal = 16,
	siMenuItemTextAboutEgg = 17,
	siMenuItemTextFileMenuTitleUnderAqua = 18,
	siMenuItemTextSetupAssistant = 19
};

enum
{
	rStringsNoteAlerts = 2013,
	siActivityWatchBegun = 1,
	siActivityWatchBegunHelpText = 2,
	siExportNotPossible = 3,
	siExportNotPossibleHelpText = 4,
	siPrereleaseWarning = 5,
	siPrereleaseWarningHelpText = 6,
	siMacroImportSuccessful = 7,
	siShowIPAddress = 8,
	siActivityDetected = 9,
	siNotificationAlert = 10,
	siScriptErrorHelpText = 11
};

enum
{
	rStringsCautionAlerts = 2014,
	siConnectionsAreOpenQuitAnyway = 1,
	siConnectionsAreOpenCloseAllAnyway = 2,
	siConfirmConnectionClose = 3,
	siConfirmConnectionCloseHelpText = 4,
	siThisWillRequireConnection = 5,
	siThisWillRequireConnectionHelpText = 6,
	siLocatingMentionedCommandHelpText = 7,
	siImportOverwriteWarning = 8,
	siProcedureNotUndoableHelpText = 9,
	siPreferencesSavedAreNewer = 10,
	siConfirmRemoveListItem = 11,
	siConfirmRemoveManyListItems = 12,
	siConfirmAcceptHostKey = 13,
	siConfirmAcceptHostKeyHelpText = 14,
	siInconsistentHostKey = 15,
	siInconsistentHostKeyHelpText = 16
};

enum
{
	rStringsStartupErrors = 2015,
	siPreferencesCorruptedAttemptFix = 1,
	siPreferencesRepairFailed = 2,
	siVitalResourceMissing = 3,
	siUnableToOpenMacTCP = 4,
	siUnableToCreatePreferencesFile = 5,
	siWrongSysEnvironsVersion = 6,
	siMacOSVersionTooOld = 7,
	siMacROMTooOld = 8,
	siUnableToInstallAppleEventHandlers = 9,
	siUnableToInitOpenTransport = 10
};

enum
{
	rStringsConfigurationData = 2016,
	siConfigurationDataASession = 1,
	siConfigurationDataSessions = 2,
	siConfigurationDataATerminal = 3,
	siConfigurationDataTerminals = 4,
	// 5 is unused
	siConfigurationDataPreferencePanelNameScripts = 6,
	siConfigurationDataPreferencePanelNameGeneral = 7,
	siConfigurationDataPreferencePanelNameMacros = 8,
	siConfigurationDataPreferencePanelNameConfigurations = 9,
	siConfigurationDataPreferencePanelNameFileTransfers = 10,
	siConfigurationDataPreferencePanelConfigurationsSessionsLabel = 11,
	siConfigurationDataPreferencePanelConfigurationsSessionsDuplicateTitle = 12,
	siConfigurationDataPreferencePanelConfigurationsSessionsDescription = 13,
	siConfigurationDataPreferencePanelConfigurationsTerminalsLabel = 14,
	siConfigurationDataPreferencePanelConfigurationsTerminalsDuplicateTitle = 15,
	siConfigurationDataPreferencePanelConfigurationsTerminalsDescription = 16,
	siConfigurationDataPreferencePanelConfigurationsFirewallsLabel = 17,
	siConfigurationDataPreferencePanelConfigurationsFirewallsDuplicateTitle = 18,
	siConfigurationDataPreferencePanelConfigurationsFirewallsDescription = 19,
	siConfigurationDataPreferencePanelConfigurationsFTPUsersLabel = 20,
	siConfigurationDataPreferencePanelConfigurationsFTPUsersDuplicateTitle = 21,
	siConfigurationDataPreferencePanelConfigurationsFTPUsersDescription = 22
};

enum
{
	rStringsFTPClientMessages = 2017,
	siFTPCommandInvalid = 1,
	siFTPCommandAmbiguous = 2,
	siFTPHelpCommandInvalid = 3,
	siFTPHelpCommandAmbiguous = 4,
	siFTPCommandsMayBeAbbreviated = 5,
	siFTPInteractiveModeOn = 6,
	siFTPUnableToChangeDirectory = 7,
	siFTPNoFilesFound = 8,
	siFTPCouldNotOpenFile = 9,
	siFTPCannotOpenFileToReceive = 10,
	siFTPEnterFileNameGetBinaryFile = 11,
	siFTPEnterFileNamePutBinaryFile = 12,
	siFTPEnterFileNameToDelete = 13,
	siFTPEnterFileNameGetFile = 14,
	siFTPEnterFileNamePutFile = 15,
	siFTPEnterFileNameOldNameRename = 16,
	siFTPEnterFileNameNewNameRename = 17,
	siFTPEnterFileNamesGet = 18,
	siFTPEnterFileNamesPut = 19,
	siFTPEnterFileNamesToDelete = 20,
	siFTPEnterDirectoryNameToChangeTo = 21,
	siFTPEnterDirectoryNameForNewDirectory = 22,
	siFTPEnterDirectoryNameToDelete = 23,
	siFTPEnterDirectoryNamesToList = 24,
	siFTPEnterCommandNameForQuote = 25,
	siFTPEnterUserName = 26,
	siFTPEnterPassword = 27,
	siFTPEnterAccount = 28,
	siPathDelimiter = 29
};

enum
{
	rStringsContextualMenuCommands = 2018,
	siContextHelp = 1,
	siContextShowBalloons = 2,
	siContextHideBalloons = 3,
	siContextCloseConnection = 4,
	siContextDisconnect = 5,
	siContextPrintOne = 6,
	siContextCopy = 7,
	siContextCopyTable = 8,
	siContextPaste = 9,
	siContextSetScreenSize = 10,
	siContextFormat = 11,
	siContextSetKeys = 12,
	siContextHideFrontWindow = 13,
	siContextStackWindows = 14,
	siContextChangeWindowTitle = 15
};

enum
{
	rStringsNetworkErrors = 2019,
	siNetErrorUnableToGetTCPStatus = 1,
	siNetErrorUnableToGetIPAddress = 2,
	siNetErrorCantOpenSocketOpenFailed = 3,
	siNetErrorCantAbortConnection = 4,
	siNetErrorHostOrGatewayHasStillNotResponded = 5,
	siNetErrorHostOrGatewayHasStillNotRespondedHelpText = 6,
	siNetErrorFatalCannotAllocateTCPBuffer = 7
};

enum
{
	rStringsDiskErrors = 2020,
	siResErrorDiskIsFull = 1
};

enum
{
	rStringsMemoryErrors = 2021,
	siMemErrorCannotLoadSet = 1,
	siMemErrorCannotCreateRasterGraphicsWindow = 2,
	siMemErrorFatalOutOfMemory = 3
};

enum
{
	rStringsCommandLineUNIXCommands = 2022,
	siUNIX_help = 1,
	siUNIX_lynx = 2,
	siUNIX_telnet = 3,
	siUNIX_ftp = 4,
	siUNIX_rlogin = 5,
	siUNIX_open = 6
};

enum
{
	rStringsCommandLineCommandDescriptions = 2023,
	siCommandDescriptionTemplate = 1,
	siCommandDescriptionNotAvailable = 2,
	siCommandDescriptionForBadArgument = 3,
	siCommandDescription_help = 4,
	siCommandDescription_lynx = 5,
	siCommandDescription_telnet = 6,
	siCommandDescription_ftp = 7,
	siCommandDescription_rlogin = 8,
	siCommandDescription_open = 9
};

enum
{
	rStringsCommandLineUNIXHelpArguments = 2024,
	rStringCountCommandLineUNIXHelpArguments = 4
};

enum
{
	rStringsOSProtectionMechanismMessages = 2025,
	siOSProtectionGenericMessageText = 1,
	siOSProtectionHelpText = 2,
	siOSProtectionErrorNameIllegalArrayAccess = 3,
	siOSProtectionErrorDescriptionIllegalArrayAccess = 4
};

enum
{
	// The base of each set of control titles is equal to the
	// value of the previous base, plus the number of control
	// titles in that previous set.  By doing the calculations
	// this way, all control titles are listed together in the
	// string resource *and* the indices remain correct when
	// you add a new string as long as you simply increment the
	// base of the following set.  Why did I do it this way?
	// Well, I think I was drunk at the time...
	rStringsMiscellaneousControlTitles = 2026,
	siControlTitlePrefsGeneralOptionsBase = 1, // the first index of a set of checkbox names
	siControlTitlePrefsGeneralNotificationMethodsBase
		= siControlTitlePrefsGeneralOptionsBase + 10/* no. options in General Options tab */,
	siControlTitlePrefsFTPOptionsBase
		= siControlTitlePrefsGeneralNotificationMethodsBase + 4/* no. notification methods */,
	siControlTitlePrefsFTPModesBase
		= siControlTitlePrefsFTPOptionsBase + 4/* no. options in FTP Options tab */,
	siControlTitlePrefsCursorFlashing
		= siControlTitlePrefsFTPModesBase + 3/* no. FTP server modes */,
	siControlTitleUpdateDefaultTerminal
		= siControlTitlePrefsCursorFlashing + 1
};

enum
{
	rStringsMiscellaneousStaticText = 2027,
	siStaticTextWhenPostingNotifications = 1,
	siStaticTextCursorShape = 2,
	siStaticTextTimeoutOpen = 3,
	siStaticTextTimeoutSend = 4,
	siStaticTextCopyTableThreshold = 5,
	siStaticTextCopyTableThresholdUnits = 6,
	siStaticTextFormatSizeLabel = 7,
	siStaticTextFormatForegroundColor = 8,
	siStaticTextFormatBackgroundColor = 9,
	siStaticTextScreenSizeWidth = 10,
	siStaticTextScreenSizeHeight = 11,
	siStaticTextScreenSizeColumns = 12,
	siStaticTextScreenSizeRows = 13,
	siStaticTextANSIColorLabelBlack = 14,
	siStaticTextANSIColorLabelRed = 15,
	siStaticTextANSIColorLabelGreen = 16,
	siStaticTextANSIColorLabelYellow = 17,
	siStaticTextANSIColorLabelBlue = 18,
	siStaticTextANSIColorLabelMagenta = 19,
	siStaticTextANSIColorLabelCyan = 20,
	siStaticTextANSIColorLabelWhite = 21,
	siStaticTextANSIColorsHeader = 22,
	siStaticTextANSIColorsNormalHeader = 23,
	siStaticTextANSIColorsEmphasizedHeader = 24
};

enum
{
	rStringsInternetResources = 2028,
	siInternetResourceOfficialHomePage = 1,
	siInternetResourceMailToTheAuthor = 2,
	siInternetResourceSourceLicense = 3,
	siInternetResourceLibraryLicense = 4,
	siInternetResourceProjectStatus = 5
};

enum
{
	rStringsImportantFileNames = 2029,
	siImportantFileNamePreferences = 1,
	siImportantFileNamePreferencesFontList = 2,
	siImportantFileNamePreferencesFTPLogInMessage = 3,
	siImportantFileNameAboutBoxMovie = 4,
	siImportantFileNameAboutBoxTitleMovie = 5,
	siImportantFileNameTerminalBellSound = 6,
	siImportantFileNameCaptureTextDefault = 7,
	siImportantFileNameSessionDescriptionDefault = 8,
	siImportantFileNameMacroSetDefault = 9,
	siImportantFileNameMacroSet1 = 10, // all of the macro set file names must be consecutive!
	siImportantFileNameMacroSet2 = siImportantFileNameMacroSet1 + 1,
	siImportantFileNameMacroSet3 = siImportantFileNameMacroSet1 + 2,
	siImportantFileNameMacroSet4 = siImportantFileNameMacroSet1 + 3,
	siImportantFileNameMacroSet5 = siImportantFileNameMacroSet1 + 4
};

enum
{
	rStringsImportantFolderNames = 2030,
	siImportantFolderNameApplicationFavorites = 1,
	siImportantFolderNameApplicationHelpSystem = 2,
	siImportantFolderNameApplicationPreferences = 3,
	siImportantFolderNameApplicationScriptsMenuItems = 4,
	siImportantFolderNameApplicationStartupItems = 5,
	siImportantFolderNameApplicationFavoritesFTPUsers = 6,
	siImportantFolderNameApplicationFavoritesProxies = 7,
	siImportantFolderNameApplicationFavoritesSessions = 8,
	siImportantFolderNameApplicationFavoritesTerminals = 9,
	siImportantFolderNameApplicationRecentSessions = 10
};

enum
{
	rStringsGoodFonts = 4000,	// obsolete, used to delete old preferences
	rStringsBadFonts = 5000,	// obsolete, used to delete old preferences
	rStringsAllFonts = 4500
};

enum
{
	rStringsSpecialButtons = 6000,
	siButtonAbort = 1,
	siButtonCopy = 2,
	siButtonContinue = 3,
	siButtonRepair = 4,
	siButtonClose = 5,
	siButtonQuit = 6
};

#endif /* ifndef REZ */

#endif