/*###############################################################

	TektronixRealGraphics.c
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/
/*
by Gaige B. Paulsen
spawned from rgp.c by Aaron Contorer for NCSA
Copyright 1987, board of trustees, University of Illinois

Routines for Macintosh Window output.  
*/

#include "UniversalDefines.h"

// standard-C includes
#ifdef __cplusplus
#	include <cstdio>
#	include <cstring>
#else
#	include <stdio.h>
#	include <string.h>
#endif

// MacTelnet includes
#include "TelnetHeader.h"
#include "TelnetDescriptors.h"

#include "TektronixRealGraphics.h"
#include "MacLook.h"
#include "TektronixVirtualGraphics.h"
#include "RealScreen.h"
#include "NetEvents.h"
#include "rg0.proto.h"
#include "rgmp.proto.h"
#include "InternalEvents.h"
#include "tekdefs.h"		/* NCSA: sb - all the TEK defines are now here */


// variables
long	RGMlastclick = 0L;
short	RGMcolor[] =
		{
			30,			// black
			33,			// white
			205,		// red
			341,		// green
			409,		// blue
			273,		// cyan
			137,		// magenta
			69			// yellow
		};
struct RGMwindows*		RGMwind[MAXWIND];


//
// public methods
//

/**
To determine if the specified window is a
Tektronix Real Graphics window, use this
method.  If true is returned, the output
"outRG" will be the number of the graphics
window.  If false is returned, the output
is negative in "outRG".

(3.0)
*/
Boolean
TektronixRealGraphics_IsRealGraphicsWindow	(const WindowRef		inWindow,
											 SInt16*				outRG)
{
	Boolean		result = false;
	
	
	*outRG = RGfindbywind(inWindow);
	result = (*outRG > -1);
	
	return result;
}// IsRealGraphicsWindow


char*
RGMdevname	(void)
{
	return("Macintosh Windows");
}// RGMwindows


short
RGMnewwin	(void)
{
	short	i = 0;

	while ((i < MAXWIND) && (RGMwind[i] != null)) i++;

	if (i >= MAXWIND) return(-1);

	RGMwind[i] = (struct RGMwindows *) Memory_NewPtr(sizeof(struct RGMwindows));
	if (RGMwind[i] == null) {
		return -1;
		}

	RGMwind[i]->id = 'RGMW';

	RGMwind[i]->wind = GetNewWindow(256, null, kInFront);
	if (RGMwind[i]->wind == 0L) {
		Memory_DisposePtr(&((Ptr)RGMwind[i]));
		RGMwind[i] = null;
		return -1;
		}
	
	MoveWindow(RGMwind[i]->wind, 70, 100, true/* activate */); // 3.0 - move to arbitrary position
	SetWindowKind(RGMwind[i]->wind, WIN_TEK);
	ShowWindow(RGMwind[i]->wind);
	BringToFront(RGMwind[i]->wind);
	
	RGMwind[i]->vg = -1;
	RGMwind[i]->vs = -1;
	RGMwind[i]->xorigin = 0;
	RGMwind[i]->yorigin = 0;
	RGMwind[i]->xscale  = WINXMAX;
	RGMwind[i]->yscale  = WINYMAX;
	RGMwind[i]->width   = 400;
	RGMwind[i]->height  = 300;
	RGMwind[i]->ingin   = 0;
	
	MacLook_ValidatePortRectangle(RGMwind[i]->wind);
	
	return(i);
}// RGMnewwin


void
RGMinit		(void)
{
	register short	i = 0;
	
	
	for (i = 0; i < MAXWIND; i++) RGMwind[i] = null;
}// RGMinit


short
RGMgin	(short		w)
{
	short	result = 0;
	
	
	if (RGsetwind(w)) result = -1;
	else
	{
		setgraphcurs();
		RGMwind[w]->ingin = 1;
	}
	return result;
}// RGMgin


short
RGMpencolor		(short		w,
				 short		color)
{
	short	result = 0;
	
	
	if (RGsetwind(w)) result = -1;
	else ForeColor((long)RGMcolor[color]);
	
	return result;
}// RGMpencolor


short
RGMclrscr	(short		w)
{
	short	result = 0;
	
	
	if (RGsetwind(w)) result = -1;
	else
	{
		Rect		bounds;
		
		
		GetPortBounds(GetWindowPort(RGMwind[w]->wind), &bounds);
		PaintRect(&bounds);
	}
	
	return result;
}// RGMclrscr


short
RGMclose	(short		w)
{
	short	result = 0;
	
	
	if (RGsetwind(w)) result = -1;
	else
	{
		DisposeWindow(RGMwind[w]->wind);
		Memory_DisposePtr(&((Ptr)RGMwind[w]->name));
		Memory_DisposePtr(&((Ptr)RGMwind[w]));
		RGMwind[w] = null;
	}
	return result;
}// RGMclose


short
RGMpoint	(short		w,
			 short		x,
			 short		y)
{
	short	result = 0;
	
	
	if (RGsetwind(w)) result = -1;
	else
	{
		MoveTo(x, y);
		LineTo(x, y);
	}
	return result;
}// RGMpoint


short
RGMdrawline		(short		w,
				 short		x0,
				 short		y0,
				 short		x1,
				 short		y1)
{
	long	xl0 = 0L,
			yl0 = 0L,
			xl1 = 0L,
			yl1 = 0L;
	short	result = 0;
	
	
	if (RGsetwind(w)) result = -1;
	else
	{
		xl0 = ((long)x0 * RGMwind[w]->width) / INXMAX;
		yl0 = (long)RGMwind[w]->height - (((long)y0 * RGMwind[w]->height) / INYMAX);
		xl1 = ((long)x1 * RGMwind[w]->width) / INXMAX;
		yl1 = (long)RGMwind[w]->height - (((long)y1 * RGMwind[w]->height) / INYMAX);
	
		MoveTo((short)xl0, (short)yl0);
		LineTo((short)xl1, (short)yl1);
	}
	return result;
}// RGMdrawline


void
RGMinfo		(short		w,
			 short		v,
			 short		UNUSED_ARGUMENT(a),
			 short		UNUSED_ARGUMENT(b),
			 short		UNUSED_ARGUMENT(c),
			 short		UNUSED_ARGUMENT(d))
{
	RGMwind[w]->vg=v;
}

void
RGMpagedone	(short		UNUSED_ARGUMENT(w))
{
}

void
RGMdataline	(short		UNUSED_ARGUMENT(w),
			 short		UNUSED_ARGUMENT(data),
			 short		UNUSED_ARGUMENT(count))
{
}


void
RGMcharmode	(short		UNUSED_ARGUMENT(w),
			 short		UNUSED_ARGUMENT(rotation),
			 short		UNUSED_ARGUMENT(size))
{
}

void	RGMgmode(void)
{}

void	RGMtmode(void)
{}

void	RGMshowcur(void)
{
}

void	RGMlockcur(void)
{
}

void	RGMhidecur(void)
{
}

void
RGMbell	(short	UNUSED_ARGUMENT(w))
{
}

void
RGMuncover	(short	UNUSED_ARGUMENT(w))
{
}

short
RGMoutfunc	(short	(*UNUSED_ARGUMENT(f))())
{
	return(0);
}

short	RGsetwind(short dnum)
{
	if (dnum<0 || dnum>=MAXWIND) return(-1);

	if (RGMwind[dnum] == null) return -1;

	SetPortWindowPort(RGMwind[dnum]->wind);
	return(0);
}

short	RGfindbyVG(short vg)
{
	short	i = 0;
	
	while (i < MAXWIND) {
		if (RGMwind[i] != null) {
			if (RGMwind[i]->vg == vg)
				break;
			}
		i++;
		}

	if (i >= MAXWIND) return(-1);
	return(i);
}


short
RGattach	(short			vg,
			 short			virt,
			 char*			name,
			 TektronixMode	TEKtype)
{
	short			dnum = 0;
	unsigned long	time = 0L;
	char			myname[256],
					ts[256];
	short			result = -1;
	
	
	if ((dnum = RGfindbyVG(vg)) >= 0)
	{
		RGMwind[dnum]->vs = virt;
		RGMwind[dnum]->name = (unsigned char*)Memory_NewPtr((long)256);
		
		if (RGMwind[dnum]->name == 0L)
			result = -2;
		else
		{
			myname[0] = '';
			if (TEKtype == kTektronixMode4105)
				strcpy(&myname[1], "[4105] ");
			else if (TEKtype == kTektronixMode4014)
				strcpy(&myname[1], "[4014] ");
			else
				strcpy(&myname[1], "[?] ");
				
			strncpy(&myname[strlen(myname)], name, 120);
			
			GetDateTime(&time);
			{
				Str255		string;
				
				
				TimeString(time, false, string, null); // put the time in the temp string
				StringUtilities_PToC(string, ts);
			}
			
			strncat(myname, "  ", 2);
			strncat(myname, ts, 64);
			strcpy((char*)RGMwind[dnum]->name, myname);
			
			if (RGMwind[dnum]->wind != null)
			{
				StringUtilities_CToPInPlace(myname);
				SetWTitle(RGMwind[dnum]->wind, (StringPtr)myname);
			}
			result = 0;
		}
	}
	
	return result;
}// RGattach


short	RGdetach( short vg)
{
		short	dnum = 0;
		char	myname[256];

	if ((dnum = RGfindbyVG(vg)) < 0) return(-1);
	if (dnum >= MAXWIND)  return(-1);

	if (RGMwind[dnum]->vs != -1) {
		if (RGMwind[dnum]->wind != null) {
#if 1
			strncpy((char *) RGMwind[dnum]->name,
			        (char *) &RGMwind[dnum]->name[1],255);
			strncpy(myname, (char *) RGMwind[dnum]->name,256);
			StringUtilities_CToPInPlace(myname);
			SetWTitle(RGMwind[dnum]->wind, (StringPtr)myname);
#else
			strncpy((char *) RGMwind[dnum]->name,
			        (char *) &RGMwind[dnum]->name[1],255);
			SetWTitle(RGMwind[dnum]->wind, RGMwind[dnum]->name);
#endif
		}
	}
	return(0);
}

short	RGfindbywind(WindowRef wind)
{
	short	i = 0;

	while (i < MAXWIND) {
		if (RGMwind[i] != null) {
			if (RGMwind[i]->wind == wind)
				break;
			}
		i++;
		}

	if (i >= MAXWIND) return(-1);
	return(i);
}

short	RGupdate( WindowRef wind)
{
	short	i = 0,
			done;

	if ((i = RGfindbywind(wind)) < 0)
		return(-1);

	SetPortWindowPort(wind);
	BeginUpdate(wind);

	VGstopred(RGMwind[i]->vg);
	VGpage(RGMwind[i]->vg);
	done = VGpred(RGMwind[i]->vg,RGMwind[i]->vg);

	EndUpdate(wind);
	if (!done)
		NetEvents_Post(kTelnetNetworkEventClassUser,kUserEventRealGraphicsRedraw,RGMwind[i]->vg,0);
	return(done);
}

short	RGsupdate( short i)
{
	short	rg;

	rg = RGfindbyVG(i);

	if (rg < 0) return(0);
	SetPortWindowPort(RGMwind[rg]->wind);
	if (!VGpred(RGMwind[rg]->vg,RGMwind[rg]->vg))
	{
		NetEvents_Post(kTelnetNetworkEventClassUser,kUserEventRealGraphicsRedraw,i,0);
	}
	else
		return(1);
	return(0);
}

short	RGgetVG(WindowRef wind)
{
	short	i;

	i = RGfindbywind(wind);

	return(RGMwind[i]->vg);
}

short	RGgetVS(short dnum)
{
	return(RGMwind[dnum]->vs);
}

short	inSplash(Point *p1, Point *p2)
{
	if ((p1->h - p2->h > 3) || (p2->h - p1->h > 3))
		return(0);
	if ((p1->v - p2->v > 3) || (p2->v - p1->v > 3))
		return(0);

	return(1);
}

void VidSync( void)
{
	long	a;
	a = TickCount();
	while (a == TickCount()) {}
}

void RGmousedown
  (
	WindowRef wind,
	Point *wherein
  )
{
	unsigned long	lx = 0L,
					ly = 0L;
	char			thispaceforent[6];
	short			i = 0;
	Point			where;
	char			dum[32];

	where = *wherein;
	if ((i = RGfindbywind(wind)) < 0)
		return;

	if (!RGMwind[i]->ingin)
	{
	Point	anchor,current,last;
	long	tc;
	short	x0,y0,x1,y1;
	Rect	rect;
	
		SetPortWindowPort(wind);
	
		last  = where;
		current = where;
		anchor = where;
		
		ColorUtilities_SetGrayPenPattern();
		PenMode(patXor);
	
		SetRect(&rect,0,0,0,0);
	
		while (StillDown())
		{
			GetMouse(&current);
			if (inSplash(&current,&anchor)) continue;
			tc = TickCount();
			while(TickCount() == tc)
				/* loop */;
			VidSync();
			FrameRect(&rect);
	
			if (anchor.v < current.v)
			{
				rect.top = anchor.v;
				rect.bottom = current.v;
			}
			else
			{
				rect.top = current.v;
				rect.bottom = anchor.v;
			}
	
			if (anchor.h < current.h)
			{
				rect.left = anchor.h;
				rect.right = current.h;
			}
			else
			{
				rect.right = anchor.h;
				rect.left = current.h;
			}
	
			FrameRect(&rect);
			last = current;
		}
		
		FrameRect(&rect);
		
		ColorUtilities_SetBlackPenPattern();
		PenMode(patCopy);
		
		if (!inSplash(&anchor,&current))
		{
			x0 = (short) ((long) rect.left * RGMwind[i]->xscale / RGMwind[i]->width );
			y0 = (short) (RGMwind[i]->yscale -
					(long) rect.top * RGMwind[i]->yscale / RGMwind[i]->height);
			x1 = (short) ((long) rect.right * RGMwind[i]->xscale / RGMwind[i]->width);
			y1 = (short) (RGMwind[i]->yscale -
					(long) rect.bottom * RGMwind[i]->yscale / RGMwind[i]->height);
			x1 = (x1 < x0+2) ? x0 + 4 : x1;
			y0 = (y0 < y1+2) ? y1 + 4 : y0;

			VGzoom( i,
					x0 + RGMwind[i]->xorigin,
					y1 + RGMwind[i]->yorigin,
					x1 + RGMwind[i]->xorigin,
					y0 + RGMwind[i]->yorigin);

			VGpage(RGMwind[i]->vg);

			RGMwind[i]->xscale  = x1 - x0;
			RGMwind[i]->yscale  = y0 - y1;
			RGMwind[i]->xorigin = x0 + RGMwind[i]->xorigin;
			RGMwind[i]->yorigin = y1 + RGMwind[i]->yorigin;

			while(!VGpred(RGMwind[i]->vg,RGMwind[i]->vg)) NetEvents_Service();
			RGMlastclick = 0L;
		}
		else
		{
			if (RGMlastclick && ((RGMlastclick + GetDblTime()) > TickCount()))
			{
				RGMwind[i]->xscale  = WINXMAX;
				RGMwind[i]->yscale  = WINYMAX;
				RGMwind[i]->xorigin = 0;
				RGMwind[i]->yorigin = 0;

				VGzoom(i,0,0,WINXMAX-1,WINYMAX-1);
				VGpage( RGMwind[i]->vg);
				while(!VGpred(RGMwind[i]->vg,RGMwind[i]->vg)) NetEvents_Service();
				RGMlastclick = 0L;
			}
			else RGMlastclick = TickCount();
		}
		return;
	
	}
	
	/* NCSA: SB */
	/* NCSA: SB - These computations are being truncated and turned into signed ints. */
	/* NCSA: SB - Just make sure everything is cast correctly, and were fine */
	
	lx = ((unsigned long)RGMwind[i]->xscale * (unsigned long)where.h) 	/* NCSA: SB */
			/ (unsigned long)RGMwind[i]->width;						 	/* NCSA: SB */
	ly = (unsigned long)RGMwind[i]->yscale - 							/* NCSA: SB */
		((unsigned long)RGMwind[i]->yscale * (unsigned long)where.v) / (unsigned long)RGMwind[i]->height; /* NCSA: SB */

	VGgindata(RGMwind[i]->vg,(unsigned short) lx,(unsigned short)ly,' ',thispaceforent);	/* NCSA: SB */
	
	ScreenManager_SendData(RGMwind[i]->vs,thispaceforent,5);
	sprintf(dum," \r\n");
	ScreenManager_SendData(RGMwind[i]->vs,dum,3);
	

    /*	RGMwind[i]->ingin = 0; */
	unsetgraphcurs();
	RGMlastclick = TickCount();
}


/*--------------------------------------------------------------------------*/
/* NCSA: SB - RGMgrowme														*/
/* This allows the TEK window to be grown like any other real window.		*/
/* The grow box is not shown on the screen, but it acts accordingly	 - SMB	*/
/*--------------------------------------------------------------------------*/
void
RGMgrowme		(short		myRGMnum,
				 WindowRef	window,
				 long*		where,
				 short		UNUSED_ARGUMENT(modifiers))
{
	long size;
	Rect SizRect;

	SetRect(&SizRect, 50, 50, 1500, 1500);			/* NCSA: SB - max size - arbitrary */

	if (where)
	{		
		size = GrowWindow(window, *(Point *) where, &SizRect);	
		if (size != 0L)
		{
			Rect		windowPortRect;
			GrafPtr		oldPort = null;
			
			
			GetPort(&oldPort);
			SizeWindow(window, size & 0xffff, (size >> 16) & 0xffff, false);
			GetPortBounds(GetWindowPort(window), &windowPortRect);
			RGMwind[myRGMnum]->width   = windowPortRect.right - windowPortRect.left;		
			RGMwind[myRGMnum]->height  = windowPortRect.bottom - windowPortRect.top;		 
			VGpage(RGMwind[myRGMnum]->vg);
			MacLook_InvalidateRectangle(window, &windowPortRect);
			SetPort(oldPort);
			RGupdate(window);
		}
		else return;							/* user skipped growing */
	}
}

