/*###############################################################

	TelnetErrors.h
	
	This module handles all of the error reporting in one
	centralized place.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __TELNETERRORS__
#define __TELNETERRORS__

// Mac includes
#include <MacTypes.h>

// MacTelnet includes
#include "ResourceFileManager.h"
#include "TelnetDescriptors.h"



// types
struct TelnetAlertParameterBlock
{
	struct
	{
		StringPtr	title1,
					title2,
					title3;	// OK, Cancel and Other buttons; invalid strings invoke defaults
		StringPtr	key1,
					key2,
					key3;	// only the first character of these strings is considered
	} buttons;
	
	struct
	{
		UInt32		timeout;	// number of SECONDS before the alert disappears automatically
		AlertType	type;
	} dialog;
	
	struct
	{
		Boolean		alertSound,
					helpButton,
					networkDisruption,
					notTitled,
					speech;
	} flags;
	
	struct
	{
		StringPtr	large,		// REQUIRED parameter
					small,
					title;
	} text;
};
typedef struct TelnetAlertParameterBlock	TelnetAlertParameterBlock;
typedef struct TelnetAlertParameterBlock*	TelnetAlertParameterBlockPtr;



//
// public methods
//

void
	Errors_Init							(void);

void
	Errors_Done							(void);

void
	Errors_DisplayNoteMessage			(SInt16 const							inStringListResourceID,
										 UInt16 const							inStringIndex,
										 SInt16 const							inTitleStringIndex);

void
	Errors_DisplayOSProtectionMessage	(StringPtr								inErrorName,
										 StringPtr								inDescriptionSentence);

SInt16
	Errors_DisplayParameterizedAlert	(TelnetAlertParameterBlockPtr const		inParameters);

void
	Errors_DisplayStopMessage			(SInt16 const							inStringListResourceID,
										 UInt16 const							inStringIndex,
										 SInt16 const							inTitleStringIndex);

void
	Errors_DisplayStopQuitMessage		(SInt16 const							inStringListResourceID,
										 UInt16 const							inStringIndex,
										 SInt16 const							inTitleStringIndex);

void
	OperationFailedAlert				(short									inMessageID,
										 short									inInternalID,
										 short									inOSID);

void
	Errors_OutOfMemory					(InternalTelnetMemoryError const		inInternalID);

#endif