/*###############################################################

	TelnetMacros.h
	
	Lists methods to access data for -<digit> equivalents.
	
	The entire macro implementation has been re-written in
	MacTelnet 3.0 to be abstract, efficient and very powerful.
	The macro code has now been separated into two levels: high
	level ("MacroSetupDialog.c") and low level (this file).
	
	MacTelnet 3.0 allows up to five sets of macros in memory at
	once, and allows the user to use any one set at once.  This
	code has now been written in such a way that it should be
	possible to change the number of macro sets supported, as
	well as the number of macros in each set, without too much
	trouble.  Note, however, that if you change the number of
	macros in each set to anything but 10, you will have to
	re-write the Macro Setup Dialog code so that the user can
	enter the macros.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __TELNETMACROS__
#define __TELNETMACROS__

// Mac includes
#include <Files.h>

// MacTelnet includes
#include "ScreenManager.h"



// constants
typedef SInt32 TelnetMacroMode;
enum
{
	kTelnetMacroModeCommandDigit = 0,
	kTelnetMacroModeFunctionKeys = 1
};

enum
{
	MACRO_COUNT = 12,		// number of macros supported in a given set
	MACRO_SET_COUNT = 5		// number of sets (of MACRO_COUNT macros each) to choose from
};

// types
typedef UInt16		MacroIndex;
typedef Handle*		MacroSet;



/*###############################################################
	INITIALIZING AND FINISHING WITH MACRO SETS
###############################################################*/

// CALL THIS ROUTINE ONCE, BEFORE ANY OTHER MACRO SET ROUTINE
void
	Macros_Init					(void);

// CALL THIS ROUTINE AFTER YOU ARE PERMANENTLY DONE WITH MACRO SETS
void
	Macros_Done					(void);

/*###############################################################
	CREATING AND DESTROYING MACRO SETS
###############################################################*/

MacroSet
	Macros_NewSet				(void);

void
	Macros_DisposeSet			(MacroSet*					inoutSetToDestroy);

/*###############################################################
	GLOBAL MACRO ROUTINES
###############################################################*/

MacroSet
	Macros_GetActiveSet			(void);

UInt16
	Macros_GetActiveSetNumber	(void);

TelnetMacroMode
	Macros_GetMode				(void);

SInt16
	Macros_Send					(ConnectionDataPtr			inConnectionDataPtr,
								 MacroIndex					inZeroBasedMacroNumber);

// WARNING - THE MAXIMUM VALUE (ONE-BASED) YOU CAN SPECIFY IS MACRO_SET_COUNT
void
	Macros_SetActiveSetNumber	(UInt16						inNewActiveSetNumber);

void
	Macros_SetMode				(TelnetMacroMode			inNewMode);

/*###############################################################
	UTILITIES FOR GETTING AND SETTING MACROS
###############################################################*/

// IF YOU PASS A NULL FILE, THE USER IS QUERIED (VIA NAVIGATION SERVICES) TO SPECIFY A FILE
void
	Macros_ExportToText			(MacroSet					inSet,
								 FSSpec*					inFileSpecPtrOrNull,
								 TelnetMacroMode			inMacroModeOfExportedSet);

void
	Macros_Get					(MacroSet					inFromWhichSet,
								 MacroIndex					inZeroBasedMacroNumber,
								 char*						s,
								 SInt16						room);

// IF YOU PASS A NULL FILE, THE USER IS QUERIED (VIA NAVIGATION SERVICES) TO SPECIFY A FILE
Boolean
	Macros_ImportFromText		(MacroSet					inSet,
								 const FSSpec*				inFileSpecPtrOrNull,
								 TelnetMacroMode*			outMacroModeOfImportedSet);

void
	Macros_ParseTextBuffer		(MacroSet					inSet,
								 unsigned char*				buffer,
								 Size						size,
								 TelnetMacroMode*			outMacroModeOfImportedSet);

void
	Macros_Set					(MacroSet					inFromWhichSet,
								 MacroIndex					inZeroBasedMacroNumber,
								 char*						s);

/*###############################################################
	OTHER MACRO UTILITIES
###############################################################*/

Boolean
	Macros_AllEmpty				(MacroSet					inSet);

void
	Macros_Copy					(MacroSet					inSource,
								 MacroSet					inDestination);

#endif