/*###############################################################

	TelnetMain.h
	
	This module contains, interestingly enough, main().  It also
	contains the two forced-shutdown commands that MacTelnet 3.0
	allows: forced-quit (kill it and kill it now) and plain quit
	(try to clean up and then let the event loop terminate).
	Theoretically, one way or the other MacTelnet will never
	exit without using one of these.  (In a perfect world...)
	
	The Main module also supports the concept of registering
	initialization and completion functions.  When a module
	registers itself, its initialization function is invoked
	automatically in the proper order, and at quitting time the
	completion routines are automatically called in reverse
	order.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __TELNETMAIN__
#define __TELNETMAIN__



/*!
Module Initialization Routine

This type of routine is invoked when MacTelnet starts
up, and is meant to initialize a module.  Your module
should do whatever is appropriate to initialize itself;
the rest of the code is not expected to use your module
until it has been properly initialized.
*/
typedef void	 (*ModuleInitProcPtr)	(void);
#define InvokeModuleInitProc(userRoutine) (*userRoutine)()

/*!
Module Clean-Up Routine

This type of routine is invoked when MacTelnet is about
to quit normally, and is meant to clean up a module.
Your module should do whatever is appropriate to clean
itself up; the rest of the code is not expected to use
your module after it has been cleaned up.
*/
typedef void	 (*ModuleCleanUpProcPtr)	(void);
#define InvokeModuleCleanUpProc(userRoutine) (*userRoutine)()



/*###############################################################
	MACTELNET EXIT POINTS
###############################################################*/

void
	Main_QuitImmediately	(void);

void
	Main_QuitSignal			(void);

/*###############################################################
	REGISTERING MODULES
###############################################################*/

void
	Main_RegisterModule		(StringPtr				inModuleName,
							 ModuleInitProcPtr		inInitializationFunctionOrNull,
							 ModuleCleanUpProcPtr	inCleanUpFunctionOrNull);

#endif