/*###############################################################

	TelnetPrinting.h
	
	This file lists Telnet-specific print calls, which are now
	implemented using the opaque UniversalPrintingLib!  This
	basically means that Telnet does not directly refer to any
	Mac OS functions or data structures for printing; instead,
	it asks UniversalPrintingLib whenever it needs to perform
	an operation, and UniversalPrintingLib automatically uses
	the best available runtime API to do it!
	
	The new abstract printing library allows code to remain
	completely unchanged when transitioning from the traditional
	printing mechanism to the new, Carbon printing mechanism.
	The abstract library automatically takes advantage of all
	issues, including backwards compatibility with Mac OS 8
	printing data structures.  There are two runtime versions
	of the same printing library symbols, and the end users
	operating system (Mac OS 8 or Mac OS X) will determine the
	library that he or she is allowed to use.  Mac OS 8 users
	have no choice; Mac OS 9 users may use either, and Mac OS X
	users really should use the Mac OS X version.
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#include "UniversalDefines.h"

#ifndef __TELNETPRINTING__
#define __TELNETPRINTING__

// Mac includes
#ifndef __MACTYPES__
#	include <MacTypes.h>
#endif

// library includes
#ifndef __PRINTINGLIB__
#	include "PrintingLib.h"
#endif



//
// public methods
//

UniversalPrintRecordRef
	TelnetPrinting_GetNewPrintRecord			(void);

void
	TelnetPrinting_PrintSelection				(void);

void
	TelnetPrinting_PageSetup					(const SInt16				inResourceFileRefNum);

void
	printPages									(UniversalPrintRecordRef	inPrintRecordRef,
												 StringPtr					inTitle,
												 SInt16						inNumberOfColumns,
												 char**						inDataOrNull,
												 SInt16						inDataFileRefNumIfNullData);

#endif