/*###############################################################

	Terminal.h
	
	This file contains the control and interface calls for the
	NCSA virtual screen kernel.  By Gaige B. Paulsen.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __TERMINAL__
#define __TERMINAL__

// MacTelnet includes
#include "RealScreen.h"
#include "ScreenManager.h"
#include "vsdata.h"



// universal procedure pointers
/*!
Screen Line Operation Routine

This defines a function that can be used as an iterator
over all of the rows of a virtual screen.  The specified
text buffer (which is read/write) includes the contents
of the current row.

If your operation modifies the buffer (which is allowed),
return "true" to notify the virtual screen module that
you changed the text.  Otherwise, return "false" if you
leave the text alone.
*/
typedef Boolean (*ScreenLineOperationProcPtr)	(TelnetScreenID const		inScreenID,
												 char*						inoutLineTextBuffer,
												 UInt16 const				inLineTextBufferLength,
												 UInt16 const				inOneBasedLineNumber,
												 void*						inContextPtr);
#define InvokeScreenLineOperationProc(userRoutine, inScreenID, inoutLineTextBuffer, inLineTextBufferLength, inOneBasedLineNumber, inContextPtr) \
			(*userRoutine)((inScreenID), (inoutLineTextBuffer), (inLineTextBufferLength), (inOneBasedLineNumber), (inContextPtr))



//
// public methods
//

SInt16
	Terminal_Init							(SInt16 const				inMaximumNumberOfTerminals);

TelnetScreenID
	Terminal_NewWindow						(StringPtr					inFont,
											 SInt16 const				inFontSize,
											 RealScreenFlags			inFlags,
											 SInt16 const				inLineCountScrollBackBuffer,
											 SInt16 const				inLineCountVisibleRows,
											 SInt16 const				inMaximumColumnCount,
											 Boolean const				inForceLineSaving);

Boolean
	Terminal_FileCaptureBegin				(TelnetScreenID const		inScreenID,
											 FSSpec const*				inFileToCaptureToOrNull);

void
	Terminal_FileCaptureEnd					(TelnetScreenID const		inScreenID);

Boolean
	Terminal_FileCaptureInProgress			(TelnetScreenID const		inScreenID);

Boolean
	Terminal_FileCaptureSaveDialog			(FSSpec*					outFSSpecPtr);

void
	Terminal_FileCaptureWriteData			(unsigned char*				inoutBuffer,
											 SInt32 const				inLength);

SInt16
	Terminal_ForEachLineDo					(TelnetScreenID const		inScreenID,
											 UInt16 const				inOneBasedRangeFirstLine,
											 UInt16 const				inOneBasedRangeLastLine,
											 ScreenLineOperationProcPtr	inDoWhat,
											 void*						inContextPtr);

SInt16
	Terminal_ForEveryLineDo					(TelnetScreenID const		inScreenID,
											 ScreenLineOperationProcPtr	inDoWhat,
											 void*						inContextPtr);

UInt16
	Terminal_GetAllocatedColumnCount		(void);

void
	Terminal_PrintingEnd					(TelnetScreenID const		inScreenID);

Boolean
	Terminal_PrintingInProgress				(TelnetScreenID const		inScreenID);

void
	Terminal_Reset							(TelnetScreenID const		inScreenID);

void
	Terminal_SavesLinesOnClear				(TelnetScreenID const		inScreenID,
											 Boolean const				inClearScreenSavesLines);

void
	Terminal_ScrollColumnsTowardLeftEdge	(TelnetScreenID const		inScreenID,
											 UInt16 const				inNumberOfColumnsToScroll);

void
	Terminal_ScrollColumnsTowardRightEdge	(TelnetScreenID const		inScreenID,
											 UInt16 const				inNumberOfColumnsToScroll);

short
	Terminal_ScrollRowsTowardBottomEdge		(TelnetScreenID const		inScreenID,
											 UInt16 const				inNumberOfRowsToScroll);

short
	Terminal_ScrollRowsTowardTopEdge		(TelnetScreenID const		inScreenID,
											 UInt16 const				inNumberOfRowsToScroll);

void
	Terminal_ScrollToBeginning				(TelnetScreenID const		inScreenID);

void
	Terminal_ScrollToEnd					(TelnetScreenID const		inScreenID);

VSscrn *VSwhereis(short i);
void VSIclrbuf(void);
short VSdestroy(short w);
short VSredraw(short w, short x1, short y1, short x2, short y2);
short VSwrite(short w, char *ptr, short len);
void VSpossendEM(short w, short x, short y, short echo);
void VSpossend(short w, short x, short y, short echo);
char VSkbsend(short w, unsigned char k, short echo);
char *VSgetline(short w, short y);
void VSredrawLine(short w);
short VSsetrgn(short w, short x1, short y1, short x2, short y2);
short VSgetrgn(short w, short *x1, short *y1, short *x2, short *y2);
short VSmaxwidth(short w);
VSlinePtr VSIGetLineStart(short w, short y1);
char *VSIstrcopy(char *src, short len, char *dest, short table);
long VSgettext(short w, short x1, short y1, short x2, short y2, char *charp, long max, char *EOLS, short table, Boolean inIsRectangular);
short VSgetlines(short w);
short VSsetlines(short w, short lines);
short VSsetcols(short w, short myWidth);
short VSgetcols(TelnetScreenID const inScreenID);
short VSIgetNextTabDistance(void);

#endif