/*###############################################################

	TerminalWindowHeader.h
	
	This module defines terminal window headers in an abstract
	way.  All information displayed by this header is generic
	enough that the interface is both extensible and independent
	of the real screen implementation.
	
	Telnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1992 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __TERMINALWINDOWHEADER__
#define __TERMINALWINDOWHEADER__

// Mac includes
#ifndef __MACTYPES__
#	include <MacTypes.h>
#endif
#ifndef __CONTROLS__
#	include <Controls.h>
#endif

// library includes
#ifndef __INTERFACELIB__
#	include "InterfaceLib.h"
#endif

// Telnet includes
#include "Panel.h"
#include "StatusIcon.h"



// constants
typedef SInt16		TerminalWindowHeaderComponentType;
enum
{
	kTerminalWindowHeaderComponentTypePanel = -3,
	kTerminalWindowHeaderComponentTypeCollapsingTriangle = -2,
	kInvalidTerminalWindowHeaderComponentType = 0
	// positive IDs 1-16 are RESERVED for status icons, and are considered valid references to the Nth installed status icon
};

// types
typedef struct { unsigned garbage; }*		TerminalWindowHeaderRef;

// universal procedure pointers
/**
Terminal Window Header Size Change Notification Method

This routine is called whenever a terminal window header
changes size, such as when the user clicks a disclosure
triangle.  Your code for handling a window containing a
terminal window header must have a method like this so
that you can resize the window to accommodate changes to
the header size.

You can use the reference provided to determine the new
size of the window header.
*/
typedef void (*TerminalWindowHeaderStateChangeProcPtr)(const TerminalWindowHeaderRef		inHeader);
#define InvokeTerminalWindowHeaderStateChangeProc(userRoutine, inHeader) \
			(*userRoutine)(inHeader)



//
// public methods
//

TerminalWindowHeaderRef
	TerminalWindowHeader_New						(const WindowRef							inOwningWindow);

void
	TerminalWindowHeader_Dispose					(TerminalWindowHeaderRef*					inoutRef);

void
	TerminalWindowHeader_GetCollapsedSize			(TerminalWindowHeaderRef					inRef,
													 Point*										outSize);

ControlRef
	TerminalWindowHeader_GetContainerControl		(TerminalWindowHeaderRef					inRef);

RgnHandle
	TerminalWindowHeader_GetNewComponentRegion		(TerminalWindowHeaderRef					inRef,
													 const TerminalWindowHeaderComponentType	inWhatComponent);

RgnHandle
	TerminalWindowHeader_GetNewPointRegion			(TerminalWindowHeaderRef					inRef,
													 const Point								inLocalPoint,
													 TerminalWindowHeaderComponentType*			outWhatComponentPtr);

void
	TerminalWindowHeader_GetNormalSize				(TerminalWindowHeaderRef					inRef,
													 Point*										outSize);

WindowRef
	TerminalWindowHeader_GetOwningWindow			(TerminalWindowHeaderRef					inRef);

void
	TerminalWindowHeader_HandleContentClick			(TerminalWindowHeaderRef					inRef,
													 EventRecord*								inoutEventPtr);

void
	TerminalWindowHeader_InstallStatusIcon			(TerminalWindowHeaderRef					inRef,
													 StatusIconRef								inIcon);

Boolean
	TerminalWindowHeader_IsCollapsed				(TerminalWindowHeaderRef					inRef);

void
	TerminalWindowHeader_SetChasingArrowsVisible	(TerminalWindowHeaderRef					inRef,
													 const Boolean								inShowArrows);

void
	TerminalWindowHeader_SetCollapsed				(TerminalWindowHeaderRef					inRef,
													 const Boolean								inIsCollapsed);

void
	TerminalWindowHeader_SetStateChangeProc			(TerminalWindowHeaderRef					inRef,
													 TerminalWindowHeaderStateChangeProcPtr		inProc);

void
	TerminalWindowHeader_SetStatusText				(TerminalWindowHeaderRef					inRef,
													 StringPtr									inText);

void
	TerminalWindowHeader_SetWidthInPixels			(TerminalWindowHeaderRef					inRef,
													 const UInt16								inWidth);

#endif