/*###############################################################

	TextTranslation.h
	
	Lists methods for character translation (i.e. between what
	the user sees and what is transmitted on the network).
	Written by Roland Mnsson (Lund University Computing Center,
	Sweden, roland_m@ldc.lu.se).  Modified by Pascal Maes (UCL/
	ELEC, Place du Levant, 3, B-1348 Louvain-la-Neuve).  Updated
	for Text Encoding Converter support by Kevin Grant.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __TEXTTRANSLATION__
#define __TEXTTRANSLATION__

// Mac includes
#include <Menus.h>
#include <TextCommon.h>



//
// public methods
//

/*###############################################################
	INITIALIZING TEXT TRANSLATION
###############################################################*/

void
	TextTranslation_Init						(MenuHandle				inTranslationMenu);

/*###############################################################
	TRANSLATING TELNET/SSH CHARACTERS
###############################################################*/

OSStatus
	TextTranslation_ConvertBufferToNewHandle	(unsigned char*			inText,
												 ByteCount const		inLength,
												 TECObjectRef			inConverter,
												 Handle*				outNewHandleWithTranslatedText);

unsigned char
	TextTranslation_ConvertCharacterFromMac		(unsigned char*			inoutCharacterPtr,
												 SInt16 const			inWhichTable);

void
	TextTranslation_ConvertTextFromMac			(unsigned char*			inBuffer,
												 SInt16 const			inLength,
												 SInt16 const			inWhichTable);

void
	TextTranslation_ConvertTextToMac			(unsigned char*			inBuffer,
												 SInt16 const			inLength,
												 SInt16 const			inWhichTable);

/*###############################################################
	TRANSLATING FTP CHARACTERS
###############################################################*/

unsigned char
	TextTranslation_ConvertISOCharacterToMac	(unsigned char*			inoutCharacterPtr);

void
	TextTranslation_ConvertISOTextToMac			(unsigned char*			inBuffer,
												 SInt16 const			inLength);

unsigned char
	TextTranslation_ConvertMacCharacterToISO	(unsigned char*			inoutCharacterPtr);

void
	TextTranslation_ConvertMacTextToISO			(unsigned char*			inBuffer,
												 SInt16 const			inLength);

#endif