/*###############################################################

	VirtualDevices.h
	
	This module handles virtual device allocation for the
	interactive color raster graphics kernel.  Windows that
	mimic real video devices are used for ICR graphics.
	
	This module has been completely rewritten in Telnet 3.0, in
	part because ICR windows were crashing under the PowerPC
	architecture, and in part to add the benefit of a new,
	abstract data type.
	
	MacTelnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __VIRTUALDEVICES__
#define __VIRTUALDEVICES__

// Mac includes
#include <Palettes.h>
#include <QDOffscreen.h>



// constants
typedef SInt16 VDeviceError;
enum
{
	kVDeviceErrorNoError = 0,					// success
	kVDeviceErrorInsufficientMemory = -1,		// not enough memory to do everything required
	kVDeviceErrorPointerCheck = -2,				// a null reference or pointer was provided
	kVDeviceErrorEmptyBoundaries = -3			// empty rectangle provided; default bounds are used, otherwise valid result
};

// types
typedef struct { unsigned garbage; }*		VirtualDeviceRef;



//
// public methods
//

/*###############################################################
	CREATING AND DESTROYING VIRTUAL DEVICES
###############################################################*/

VDeviceError
	VirtualDevices_New						(VirtualDeviceRef*		inoutVirtualDeviceRefPtr,
											 const Rect*			inInitialDeviceBounds,
											 PaletteHandle			inInitialPaletteHandle);

void
	VirtualDevices_Dispose					(VirtualDeviceRef*		inoutVirtualDeviceRefPtr);

/*###############################################################
	MANIPULATING VIRTUAL DEVICES WITH QUICKDRAW
###############################################################*/

VDeviceError
	VirtualDevices_GetBitMapForCopyBits		(VirtualDeviceRef		inVirtualDeviceRef,
											 BitMap const**			outBitMapPtrPtr);

VDeviceError
	VirtualDevices_GetBounds				(VirtualDeviceRef		inVirtualDeviceRef,
											 Rect*					outRectPtr);

VDeviceError
	VirtualDevices_GetGraphicsWorld			(VirtualDeviceRef		inVirtualDeviceRef,
											 GWorldPtr*				outGWorldPtrPtr);

VDeviceError
	VirtualDevices_GetPixelMap				(VirtualDeviceRef		inVirtualDeviceRef,
											 PixMapHandle*			outPixMapHandle);

VDeviceError
	VirtualDevices_SetColorTable			(VirtualDeviceRef		inVirtualDeviceRef,
											 PaletteHandle			inPaletteToUseForColorTable);



#endif