/*###############################################################

	VirtualScreenEmulation.h
	
	Lists routines for accessing terminal emulators, such as
	VT100 and VT220, and constants that describe them.
	
	MacTelnet 3.0,  1998-2001 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef __VIRTUALSCREENEMULATION__
#define __VIRTUALSCREENEMULATION__



// constants
enum
{
	// These masks chop up the 16-bit emulation type into two parts,
	// the emulator and the variant of it; this allows up to 256
	// terminal types, and 256 variants (for example, VT is a type,
	// and VT100 and VT220 are variants of the VT terminal type).
	//
	// Standardizing on this approach will make it *much* easier to
	// implement future terminal types - for example, many variants
	// of terminals share identical features, so you can check if
	// ANY variant of a particular terminal is in use just by
	// isolating the upper byte.  For convenience, two macros below
	// are included to isolate the upper or lower byte for you.
	// Use them!!!
	kTelnetTerminalEmulatorByteShift = 8,
	kTelnetTerminalEmulatorMask = (0x0000000F << kTelnetTerminalEmulatorByteShift),
	kTelnetTerminalVariantByteShift = 0,
	kTelnetTerminalVariantMask = (0x0000000F << kTelnetTerminalVariantByteShift)
};
#define GetTelnetTerminalEmulator(a)	(((a) & kTelnetTerminalEmulatorMask) >> kTelnetTerminalEmulatorByteShift)
#define GetTelnetTerminalVariant(a)		(((a) & kTelnetTerminalVariantMask) >> kTelnetTerminalVariantByteShift)
enum
{
	// use these constants only when you need to determine the terminal emulator family
	// (and if you add support for new terminal types, add constants to this list in
	// the same way as shown below)
	kTerminalEmulatorTypeVT = ((0 << kTelnetTerminalEmulatorByteShift) & kTelnetTerminalEmulatorMask),
		kTerminalEmulatorVariantVT100 = (0x00 & kTelnetTerminalVariantMask),
		kTerminalEmulatorVariantVT220 = (0x01 & kTelnetTerminalVariantMask),
		kTerminalEmulatorVariantVT320 = (0x02 & kTelnetTerminalVariantMask),
		kTerminalEmulatorVariantVT420 = (0x03 & kTelnetTerminalVariantMask)
};
enum
{
	// in many cases, it is easier to refer to a terminal type using these simpler constants
	kTerminalEmulationTypeVT100 = kTerminalEmulatorTypeVT | kTerminalEmulatorVariantVT100,
	kTerminalEmulationTypeVT220 = kTerminalEmulatorTypeVT | kTerminalEmulatorVariantVT220,
	kTerminalEmulationTypeVT320	= kTerminalEmulatorTypeVT | kTerminalEmulatorVariantVT320,
	kTerminalEmulationTypeVT420 = kTerminalEmulatorTypeVT | kTerminalEmulatorVariantVT420
};



//
// public methods
//

void
	VirtualScreenEmulation_Parse		(unsigned char*			inoutString,
										 SInt16 const			inLength);

#endif