/*###############################################################

	vsdata.h
	
	This file contains the control and interface calls for the
	NCSA virtual screen kernel.  By Gaige B. Paulsen.
	
	MacTelnet 3.0,  1998-2000 by Kevin Grant.
	Based on "NCSA Telnet",  1986-1994 UofI Board of Trustees.
	
	This program is free software; you can redistribute it or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version
	2 of the License, or (at your option) any later version.
	
    This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more
	details.

    You should have received a copy of the GNU General Public
	License along with this program; if not, write to:
	
		Free Software Foundation, Inc.
		59 Temple Place, Suite 330
		Boston, MA  02111-1307
		USA

###############################################################*/

#ifndef	__VSDATA__
#define __VSDATA__

#include <MacTypes.h>

typedef struct VSline
  {
	struct VSline
	  /* doubly-linked list of lines */
		*next,          /* Pointer to next line */
		*prev;          /* Pointer to previous line */
	char  	// Text for the line -- may be part of a block 
						//note this is now a short for ansi color (CP)
		*text;          /* of memory containing more than one line */
	short                /* flag for memory allocation coordination */
		mem;            /* nonzero for first line in memory block */
  } VSline,*VSlinePtr,**VSlineArray;

typedef struct VSattrline	//CCP 2.7 ANSI needs shorts
  {
	struct VSattrline
	  /* doubly-linked list of lines */
		*next,          /* Pointer to next line */
		*prev;          /* Pointer to previous line */
	unsigned short  	// macblue, now use 16 bit attribute; 8 for normal, 8 for ansi color
		*text;          /* of memory containing more than one line */
	short                /* flag for memory allocation coordination */
		mem;            /* nonzero for first line in memory block */
  } VSattrline, *VSattrlinePtr, **VSattrlineArray;


#define maxparms 16

struct VSscrn {
	OSType	id;				// VSCR
    VSlinePtr
         scrntop,           /* topmost line of the current screen (= linest[0]) */
         buftop,            /* top (i e oldest line) of scrollback buffer 	*/
         vistop;            /* topmost line within visible region (may be in screen or scrollback area) */
	VSlineArray
         linest;             /* pointer to array of screen text lines        */
 VSattrlineArray
       attrst;             /* pointer to array of screen attribute lines   */
    short lines,				/* How many lines are in the screen arrays      */
		 maxlines,          /* maximum number of lines to save off top      */
         numlines,          /* number of lines currently saved off top      */
         allwidth,          /* allocated width of screen lines              */
         maxwidth,          /* current screen width setting (<= allwidth)   */
         savelines,         /* save lines off top? 0=no                     */
 		 forcesave,			/* NCSA 2.5: always force lines to be saved off top */
         possibleForce,		//we have receive a "move to 0,0"; if next is EEOl, forcesave
         ESscroll,			/* Scroll screen when ES received				*/
         attrib,            /* current character writing attributes         */
         x,y,               /* current cursor positon                       */
         Px,Py,Pattrib,     /* saved cursor position and writing attributes */
         DECAWM,            /* Auto Wrap Mode 0=off 						*/
         DECCKM,            /* Cursor Key Mode      						*/
         DECPAM,            /* keyPad Application Mode						*/
		 DECORG,			/* origin mode                                  */
		 G0,G1,				/* Character set identifiers 					*/
		 charset,			/* Character set mode 							*/
         IRM,               /* Insert/Replace Mode  						*/
         escflg,            /* Current Escape level							*/
         top, bottom,       /* Vertical bounds of scrolling region 			*/
         Rtop,Rbottom,		/* Vertical bounds of visible region 			*/
         Rleft,Rright,		/* Horizontal bounds of visible region 			*/
         parmptr,           /* LU - index of current parm 					*/
		 prredirect;		/* LU - printer redirection or not				*/
	long prbuf;				/* LU - last four chars							*/
   short refNum;			/* LU - temporary file for printer redirection	*/
	char fname[40];			/* LU - file name for temporary file			*/
    short parms[maxparms];   /* Ansi Parameters 							*/
    char *tabs;             /* pointer to array for tab settings 			*/
                            /* contains 'x' at each tab position, blanks elsewhere */
    };

typedef struct VSscrn VSscrn;

struct VSscrndata {
    VSscrn *loc;            /* Location of the Screen record for this scrn */
    short
    	captureRN,	/* capture file's RefNum                       */
    	stat;		/* status of this screen (0=Uninitialized,     */
					/*                        1=In Use             */
					/*                        2=Initialized, but not IU */
    };

typedef struct VSscrndata VSscrndata;

#ifdef VSMASTER
VSscrn  *VSIw;
short      VSIwn;
#else
extern VSscrn   *VSIw;
extern short       VSIwn;
#endif

#define	VSPBOTTOM	(VSIw->lines)

#endif	// __VSDATA__